// Version 5.25c  06.07.2004 Markus Greither
// Copyright (c) 1991-2004 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file efdlgbas.h
    \brief Header for the common base class for dialogs
*/

#ifndef __EFDLGBAS__
#define __EFDLGBAS__

// ----------------------------------------------------------------------------
// TEFComboBox
// ----------------------------------------------------------------------------

//! Combo box with history
/*!
    This combo box can be used like a text control, but it holds
    up to 9 of the last entries in the list element.
*/
class TEFComboBox :public wxComboBox
{
 public:
  //! Ctor
  TEFComboBox(wxWindow* parent, wxWindowID ResourceId)
   : wxComboBox(parent,ResourceId) {}
  //! Get combo box entries
  void GetComboData(EFComboData &CD);
  //! Set combo box entries
  void SetComboData(EFComboData &CD);
};

// ----------------------------------------------------------------------------
// TEFHilfeDlg
// ----------------------------------------------------------------------------

//! Base class for dialogs providing help button function.
/*!
    This class provides event handlers for wxID_HELP, wxID_CANCEL und
    wxID_OK.
    It has also a patch that tries to reset the default button if changed.
*/
class TEFHilfeDlg :public wxDialog
{
 public:
  //! Ctor
  TEFHilfeDlg() : wxDialog() {defbutton = 0;};
  //! Dtor
  virtual ~TEFHilfeDlg() {};
  //! Responds to help button press, opens help topic for this dialogs id
  virtual void OnHilfe(wxCommandEvent &event);
  //! Responds to cancel button press
  virtual void OnCancel(wxCommandEvent &)
  {
    EndModal(wxID_CANCEL);
  }
  //! Responds to OK button press
  virtual void OnOK(wxCommandEvent &)
  {
    EndModal(wxID_OK);
  }
  //! Idle processing (reset default button)
  virtual void OnIdle(wxIdleEvent &event);
  //! Find item and convert to wxSpinCtrl
  wxSpinCtrl *FindSC(long x)
  {
    return wxDynamicCast(FindWindow(x),wxSpinCtrl);
  }
  //! Find item and convert to wxTextCtrl
  wxTextCtrl *FindTC(long x)
  {
    return wxDynamicCast(FindWindow(x),wxTextCtrl);
  }
  //! Find item and convert to wxRadioButton
  wxRadioButton *FindRB(long x)
  {
    return wxDynamicCast(FindWindow(x),wxRadioButton);
  }
  //! Find item and convert to wxCheckBox
  wxCheckBox *FindCBX(long x)
  {
    return wxDynamicCast(FindWindow(x),wxCheckBox);
  }
  //! Find item and convert to wxListBox
  wxListBox *FindLB(long x)
  {
    return wxDynamicCast(FindWindow(x),wxListBox);
  }
  //! Find item and convert to wxComboBox
  wxComboBox *FindCoB(long x)
  {
    return wxDynamicCast(FindWindow(x),wxComboBox);
  }
  //! Find item and convert to wxStaticText
  wxStaticText *FindST(long x)
  {
    return wxDynamicCast(FindWindow(x),wxStaticText);
  }
  //! Find item and convert to wxButton
  wxButton *FindB(long x)
  {
    return wxDynamicCast(FindWindow(x),wxButton);
  }
  //! Find item and convert to wxButton
  wxChoice *FindCH(long x)
  {
    return wxDynamicCast(FindWindow(x),wxChoice);
  }
  //! Interface for selecting a font using common dialogs
  static void GetFont(wxWindow *Wd,FontType &Font,const wxChar *Titel);
  //! Interface for selecting a colour using common dialogs
  static void GetColour(wxWindow *Wd,unsigned long &Farbe,const wxChar *Titel);

 protected:
  //! Original default button
  wxWindow *defbutton;
  DECLARE_EVENT_TABLE()
};

// ----------------------------------------------------------------------------
// TEFEditDlg
// ----------------------------------------------------------------------------

//! Dialog with TEFEdit control
/*!
    This dialog creates an TEFEdit dialog child and provides buttons
    for clipboard and printing.
*/
class TEFEditDlg :public TEFHilfeDlg
{
  //! Pointers to buttons
  wxWindow *OK,*Cancel,*Kopie,*Einfg,*Druck;
  //! Flag for buttons enabled
  bool ButtonsEnabled;
  //! Button distances and count
  enum ButtonDim
  {
    ButtonCount = 6,
    ButtonAbst = 10
  };

 public:
  //! Pointer to TEFEdit control
  TEFEdit *Editor;

  //! Ctor
  TEFEditDlg();
  //! Dtor
  virtual ~TEFEditDlg();
  //! Create dialog
  void Create(wxWindow *parent,wxWindowID Id,const wxString &title);
  //! Resize Editor and reposition buttons
  void OnSize(wxSizeEvent &event);
  //! Copy Editor selection to clipboard
  void OnNo(wxCommandEvent &event);
  //! Paste clipboard content into Editor
  void OnYes(wxCommandEvent &event);
  //! Print Editor
  void OnIgnore(wxCommandEvent &event);
  //! Copy Editor selection to clipboard (wxID_COPY)
  void OnEditCopy(wxCommandEvent &event)
  {
    if (Editor)
      Editor->OnEditCopy(event);
  }
  //! Cut Editor selection to clipboard (wxID_CUT)
  void OnEditCut(wxCommandEvent &event)
  {
    if (Editor)
      Editor->OnEditCut(event);
  }
  //! Paste clipboard content into Editor (wxID_PASTE)
  void OnEditPaste(wxCommandEvent &event)
  {
    if (Editor)
      Editor->OnEditPaste(event);
  }
  //! Set selection in Editor
  void OnMarkAll(wxCommandEvent &)
  {
    if (Editor)
      Editor->SetSelection(0,Editor->GetTextLen());
  }

 protected:
  //! Set button enable in idle time
  void OnIdle(wxIdleEvent &event);
  //! Enable / disable buttons
  void EnableButtons(bool A);
  DECLARE_EVENT_TABLE()
};

#endif

