// Version 5.25  06.03.2004 Markus Greither
// Copyright (c) 1991-2004 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file efdocum.h
    \brief Header for the document class of the Doc/View framework
*/

#ifndef __EFDOCUM__
#define __EFDOCUM__

#include "efmcxio.h"

// ----------------------------------------------------------------------------
// TEFDocument
// ----------------------------------------------------------------------------

//! Easy Funktion document class
/*!
    Has methods StdFunk() / WinFunk() for conversion from/to
    the OS number format.
*/
class TEFDocument : public EFDaten,public wxDocument
{
 public:
  //! Set modified flag
  inline void SetIsDirty(bool D) {Modify(D);}
  //! Save data to MCX file
  bool SaveFile(const wxString &file);
  //! Load data from MCX file
  bool LoadFile(const wxString &file);
  //! Returns modified state
  bool IsModified() const;
  //! Respond to file save event
  bool OnSaveDocument(const wxString& file);
  //! Respond to file load event
  bool OnOpenDocument(const wxString& file);
  //! Sets or clears modified state
  void Modify(bool mod);
  //! Returns the parser member
  ParserObj &GetParser() {return *m_Parser;}
  //! Return empty file flag
  bool GetEmptyFile() const {return m_EmptyFile;}
  //! Ctor
  TEFDocument();
  //! Dtor
  ~TEFDocument();

 protected:
  //! Function parser
  ParserObj *m_Parser;
  //! Flag for opened empty file
  bool m_EmptyFile;

  //! AkStr in lines and convert win->std or std->win
  void ConvertLines(MGPStr &AkStr,ZahlFormatObj &ZF,bool ToWin);
  //! Convert functions into english format
  void StdFunk();
  //! Convert functions into current user format
  void WinFunk();
  //! Convert equations into english format
  void StdEquations();
  //! Convert equations into current user format
  void WinEquations();
  //! Convert post processor function into english format
  void StdPostTerm(DiagDataPtArrObj &ar);
  //! Convert post processor function into current user format (for SaveFile)
  void WinPostTerm(DiagDataPtArrObj &ar);
  //! Convert post processor function into current user format (for LoadFile)
  void WinPostTerm(DiagDataArrObj &ar);

  DECLARE_DYNAMIC_CLASS(TEFDocument)
};

#endif

