// Version 5.23  27.11.2003 Markus Greither
// Copyright (c) 1991-2003 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file efeddnd.h
    \brief Header for the editor drag and drop class
*/

#include <wx/dnd.h>

// ----------------------------------------------------------------------------
// EFTextDropTarget
// ----------------------------------------------------------------------------

//! Class for DnD of text in TEFEdit
class EFTextDropTarget : public wxTextDropTarget
{
  //! Pointer to drag source
  TEFEdit *Editor;
  //! Original text selection
  unsigned int Anf,End;

 public:
  //! Ctor
  EFTextDropTarget(TEFEdit *e) : Editor(e) {}
  //! Get original text selection
  wxDragResult OnEnter(wxCoord x, wxCoord y,wxDragResult def)
  {
    Editor->GetSelection(Anf,End);
    return OnDragOver(x,y,def);
  }
  //! Respond to drop event
  bool OnDropText(wxCoord x, wxCoord y, const wxString &txt)
  {
    unsigned int Pos = Editor->OnDragText(x,y,true);
    Editor->SetSelection(Pos,Pos);
    if ((Pos >= Anf) && (Pos < End))
      return false; // Position innerhalb der Markierung

    Editor->Insert(txt.c_str());
    unsigned int tmp,End;
    Editor->GetSelection(tmp,End);
    Editor->SetSelection(Pos,End);
    return true;
  }
  //! Determine current text position while dragging
  wxDragResult OnDragOver(wxCoord x, wxCoord y, wxDragResult def)
  {
    Editor->OnDragText(x,y,true);
    return def;
  }
};


