// Version 5.29  06.08.2005 Markus Greither
// Copyright (c) 1991-2005 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file effnkclc.h
    \brief Header for the function calculator class
*/

#ifndef __EFFNKCLC__
#define __EFFNKCLC__

// ----------------------------------------------------------------------------
// TEFFunktionCalcData
// ----------------------------------------------------------------------------

//! Data structure for TEFFunktionCalc
struct TEFFunktionCalcData
{
  //! Ctor
  TEFFunktionCalcData(ParserObj::FehlerRec &F) : Fehler(F)
  {
    TestModified = false;
    BerFus = 0;
    AkFeld = 0;
    DiagData = 0;
  }
  //! Assignment operator
  void operator = (const TEFFunktionCalcData &) {}
  //! Pointer to diagram data
  DiagDataObj *DiagData;
  //! Function name
  MGPStr FuName;
  //! Graph data for current graph
  const GraphDataObj *AkFeld;
  //! Test flag for finding not updated functions
  bool TestModified;
  //! Array with modified flag for each function
  vector<bool> *BerFus;
  //! Error information
  ParserObj::FehlerRec &Fehler;
};

// ----------------------------------------------------------------------------
// TEFFunktionCalc
// ----------------------------------------------------------------------------

//! Class for function/graph calculation.
class TEFFunktionCalc
{
  enum
  {
#ifdef __WXGTK__
    MaxGaugeSteps = 5
#else
    MaxGaugeSteps = 10
#endif
  };

 protected:
  //! Assignment operator
  void operator = (const TEFFunktionCalc &) {}
  //! Calculate next value
  bool WerteBerechnung();
  //! Calculate next value using inverted fourier transformation
  bool FourierBerechnung();
  //! Initialize d() function so that 1. point is calculated correctly
  void InitDifferentiation();

 public:
  //! Ctor
  TEFFunktionCalc(ParserObj *psr,TEFFunktionCalcData &data);
  //! Dtor
  virtual ~TEFFunktionCalc();
  //! Return current gauge (progress information) value
  int GetGaugeValue();
  //! Return gauge (progress information) range
  static int GetGaugeRange();
  //! Returns current HI parameter
  double GetAkHi()
  {
    return HiVar;
  }
  //! Returns index of current parameter
  int GetAkPar()
  {
    return AkPar;
  }
  //! Returns fourier mode state
  bool GetFourMode()
  {
    return FourierMode;
  }
  //! Return true if finished
  bool GetFinished();
  //! Initialize graph with name HiString and default options
  void SetDefaultGraphOptions(const _TCHAR *HiString,
                              int GrSymb,int GrOpt,int GrDarst);
  //! Copies function name with parameters into str
  void GetFunktionName(MGPStr &str,double *Par = 0);
  //! Returns the number of data points of current graph
  unsigned int GetPunkte()
  {
    return AkPt;
  }
  //! Update current graph with AkFeld
  void UpdateGraph();
  //! Append new graph with name HiString to YDaten
  void AppendNewGraph(const _TCHAR *HiString);
  //! Returns Hi variable mode
  int GetCalcMode()
  {
    return Ma.Mode;
  }
  //! Calculate next value
  bool NextStep();

 protected:
  //! Locker for InitDifferentiation()
  bool InitDiffLock;
  //! Number of data points to be calculated for graph
  unsigned int Punkte;
  //! Helper variables
  Cpx z,y;
  //! Current X value (necessary because of recursive calculation for log step)
  double AkX;
  //! Step between points
  double Step;
  //! Index to current data point
  unsigned int AkPt;
  //! Index to current coefficient during inverted fourier transformation
  int AkKoeff;
  //! Inverted fourier transformation frequency step and number harmonics
  int FourierStep,FourierObw;
  //! Flag four inv. fourier transformation
  bool FourierMode;
  //! Pointer to coefficient array for inverted fourier transformation
  Cpx *KoeffVar;
  //! Function name
  MGPStr HiString;
  //! Current Hi value
  double HiVar;
  //! Function parser
  ParserObj *Parser;
  //! X mode settings
  int XLog;
  //! Current and previous (Hi)-Parameter numbers
  int AkPar,OldPar;
  //! Hi variable settings
  MaTyp Ma;
  //! Current x value range
  AxisData XAxis;
  //! Debug flag
  int Debug;
  //! Stepping for gauge
  int GaugeStep,FGaugeStep;
  //! Data of current graph
  GraphDataObj *AkFeld;
  //! Data object
  TEFFunktionCalcData &Data;
};

#endif

