// Version 6.04  27.04.2008 Markus Greither
// Copyright (c) 1991-2008 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file efgrdlgs.h
    \brief Header for dialogs related to the diagram window class
*/

#ifndef __EFGRDLGS__
#define __EFGRDLGS__

#include "efdlgbas.h"

// ----------------------------------------------------------------------------
// FourierData
// ----------------------------------------------------------------------------

//! Data structure for TEFFourierDlg
struct FourierData
{
  //! Hi variable settings
  MaTyp Ma;
  //! Dialog state
  int State;
};

// ----------------------------------------------------------------------------
// TEFFourierDlg
// ----------------------------------------------------------------------------

//! Dialog for options of Inverted fourier transformation
class TEFFourierDlg :public TEFHilfeDlg
{
  //! Dialog settings
  int FourMode,ResMode,Obw;
  //! Fundamental frequency
  double GFreq;
  //! Data for dialog
  FourierData &Data;

  //! Limit the value to valid numbers
  double GetAkWert(int Mode,int FourMode,double Wert);
 public:
   enum
   {
     //! Flag for succesful return from TEFFourierDlg
     FourOK = 0xF0
   };

  //! Ctor
  TEFFourierDlg(FourierData &D);
  //! Create dialog
  void Create(wxWindow *parent,wxWindowID Id,const wxString &title);
  //! Respond setting change
  void BnResMode();
  //! Respond setting change
  void OnBnResMode(wxCommandEvent &);
  //! Respond to button press
  void BnClicked();
  //! Respond to button press
  void OnBnClicked(wxCommandEvent &);
  //! Respond to OK click
  void OnOk(wxCommandEvent &event);

 protected:
  DECLARE_EVENT_TABLE()
};

// ----------------------------------------------------------------------------
// TEFLabelDlg
// ----------------------------------------------------------------------------

//! Dialog for modifying diagram labels
class TEFLabelDlg :public TEFHilfeDlg
{
  //! Dialog data
  LabelObj &Data;

 public:
  //! Ctor
  TEFLabelDlg(LabelObj &D);
  //! Create dialog
  void Create(wxWindow *parent,wxWindowID Id,const wxString &title);
  //! Respond to OK click
  void OnOk(wxCommandEvent &event);
  //! Respond to font change click
  void OnFont(wxCommandEvent &event);
  //! Respond to button press
  void BnClicked();
  //! Respond to button press
  void OnBnClicked(wxCommandEvent &);

 protected:
  DECLARE_EVENT_TABLE()
};

// ----------------------------------------------------------------------------
// TEFWerteDlg
// ----------------------------------------------------------------------------

//! Dialog for modifying graph data
class TEFWerteDlg :public TEFEditDlg
{
  //! Dialog data
  MGPStr &Data;

 public:
  //! Ctor
  TEFWerteDlg(MGPStr &MS);
  //! Create dialog
  void Create(wxWindow *parent,wxWindowID Id,const wxString &title);
  //! Respond to OK click
  void OnOk(wxCommandEvent &event);
  //! Show online help
  virtual void OnHilfe(wxCommandEvent &event);

 protected:
  DECLARE_EVENT_TABLE()
};

// ----------------------------------------------------------------------------
// DarstellungsData
// ----------------------------------------------------------------------------

//! Data structure for TEFDarstellungDlg
struct DarstellungsData
{
  //! Dialog options
  int Mode;
  //! Flag for post processor active
  bool PostProc;
  //! Post processor term
  MGPStr PostTerm;
  //! Conversion
  unsigned char Conversion;
  //! Colour
  unsigned long Colour;
  //! X and Y axis value ranges
  AxisDataArr XAxis,YAxis;
  //! Diagram title
  MGPStr DiagText;
  //! Fonts for axis, title and axis numbers
  FontType Achse,Titel,AchsLabels;
  //! Calculation setup data
  EinstData EData;
  //! Number of digits for diagram numbers
  int NumDigits;
  //! Initially shown page of dialog
  int DiagPage;
  //! Initially shown y axis of dialog
  int YAxisNr;
  //! Same scaling XY flag
  bool SameXAndYScaling;
};

// ----------------------------------------------------------------------------
// TEFDarstellungDlg
// ----------------------------------------------------------------------------

//! Dialog for diagram settings
class TEFDarstellungDlg :public TEFHilfeDlg
{
  //! Dialog data
  DarstellungsData &Data;
  //! Previous axis number
  int OldAxisNumber;

 public:
  //! Ctor
  TEFDarstellungDlg(DarstellungsData &D);
  //! Create dialog
  void Create(wxWindow *parent,wxWindowID Id,const wxString &title);
  //! Respond to button press
  void BnClicked();
  //! Respond to button press
  void OnBnClicked(wxCommandEvent &);
  //! Respond to OK click
  void OnOk(wxCommandEvent &event);
  //! Respond to axis font event
  void OnAchseFont(wxCommandEvent &event);
  //! Respond to title font event
  void OnTitelFont(wxCommandEvent &event);
  //! Respond to axis numbers font event
  void OnAchsLabelsFont(wxCommandEvent &event);
  //! Respond to y axis selection
  void OnAxisNumber(wxCommandEvent &event);
  //! Get current y axis settings, store in y axis axisno
  void GetYAxisSettings(int axisno);
  //! Respond to colour button press
  void OnColour(wxCommandEvent &event);

  DECLARE_EVENT_TABLE()
};

// ----------------------------------------------------------------------------
// GraphData
// ----------------------------------------------------------------------------

//! Data structure for TEFGraphDlg
struct GraphData
{
  //! Index to used axis
  unsigned char XAxisI[2],YAxisI[2];
  //! Graph settings
  unsigned short Opt,Darst,Symb;
  //! Graph name and file name
  MGPStr Txt,FileS;
  //! Graph color
  unsigned long Farbe;
  //! Legend settings
  LegendeType Legende;
};

// ----------------------------------------------------------------------------
// TEFGraphDlg
// ----------------------------------------------------------------------------

//! Dialog for modifying graph settings
class TEFGraphDlg :public TEFHilfeDlg
{
  //! Dialog data
  GraphData &Data;

 public:
  //! Ctor
  TEFGraphDlg(GraphData &D);
  //! Create dialog
  void Create(wxWindow *parent,wxWindowID Id,const wxString &title);
  //! Respond to Yes click
  void OnYes(wxCommandEvent &event);
  //! Respond to No click
  void OnNo(wxCommandEvent &event);
  //! Respond to graph color event
  void OnFarbe(wxCommandEvent &event);
  //! Respond to legend font event
  void OnLegendeFont(wxCommandEvent &event);
  //! Respond to button press
  void BnClicked();
  //! Respond to button press
  void OnBnClicked(wxCommandEvent &);

 protected:
  //! Get dialog data and close dialog
  void EndModal(int ID);

  DECLARE_EVENT_TABLE()
};

// ----------------------------------------------------------------------------
// TEFGraphladenDlg
// ----------------------------------------------------------------------------

//! Dialog for loading graphs from text files
class TEFGraphladenDlg :public TEFHilfeDlg
{
  //! Dialog data
  GraphladenData &Data;

 public:
  //! Ctor
  TEFGraphladenDlg(GraphladenData &D);
  //! Create dialog
  void Create(wxWindow *parent,wxWindowID Id,const wxString &title);
  //! Respond to browse event
  void OnYes(wxCommandEvent &event);
  //! Respond to OK event
  void OnOk(wxCommandEvent &event);
  //! Respond to settings change
  void OnBnClicked(wxCommandEvent &);

 protected:

  DECLARE_EVENT_TABLE()
};

// ----------------------------------------------------------------------------
// GraphspeichernData
// ----------------------------------------------------------------------------

//! Data structure for TEFGraphspeichernDlg
struct GraphspeichernData
{
  //! Graph save options
  enum GraphspeichernModes
  {
    GraphIngForm = 1,
    GraphGRD     = 2,
    GraphTab     = 4,
    GraphKomma   = 8
  };
  //! Graph save option
  unsigned short Mode;
  //! File name
  MGPStr FileName;
};

// ----------------------------------------------------------------------------
// TEFGraphspeichernDlg
// ----------------------------------------------------------------------------

//! Dialog for writing graphs to text files
class TEFGraphspeichernDlg :public TEFHilfeDlg
{
  //! Dialog data
  GraphspeichernData &Data;

 public:
  //! Ctor
  TEFGraphspeichernDlg(GraphspeichernData &D);
  //! Create dialog
  void Create(wxWindow *parent,wxWindowID Id,const wxString &title);
  //! Respond to browse event
  void OnYes(wxCommandEvent &event);
  //! Respond to OK event
  void OnOk(wxCommandEvent &event);

 protected:

  DECLARE_EVENT_TABLE()
};

#endif

