// Version 5.33  31.03.2007 Markus Greither
// Copyright (c) 1991-2007 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file efgrelem.h
    \brief Header for support classes for the diagram window class
*/

#ifndef __EFGRELEM__
#define __EFGRELEM__

// ----------------------------------------------------------------------------
// Forward Declarations
// ----------------------------------------------------------------------------

class EFDiagObj;

// ----------------------------------------------------------------------------
// MenueItemObj
// ----------------------------------------------------------------------------

//! Class for legend items
class MenueItemObj :public GraphObj
{
  //! Pointer to diagram
  EFDiagObj *Diag;
  //! Legend font color
  unsigned int Farbe;
  //! Options
  unsigned short Optionen;
  //! Index to symbol
  unsigned short Symbol;
  //! Item text
  MGPStr Eintrag;
  //! Item text character sizes
  ShortArrObj CharPos;
  //! Symbol position
  int SymbX,SymbY;

 public:
  //! Index
  unsigned int Nummer;
  //! Next menu entry
  MenueItemObj *Next;
  //! legend item flags
  enum ItemStates
  {
    ItemEnabled = 1,
  };
  //! Initialize menu entry
  void Init(const _TCHAR *Zeile,int &PosX,int &PosY,unsigned long FarbVar,
            unsigned short Opt,unsigned short Symb,int Nr,
            EFDiagObj *DO,wxDC &DC);
  //! Draw menu entry
  void Draw(int Mode,int x,int y,unsigned long Colour,wxDC &DC);
  //! Dtor
  virtual ~MenueItemObj();

 protected:
  //! Draw symbol
  void ZeichneSymbol(int x,int y,wxDC &DC);
};

// ----------------------------------------------------------------------------
// FunkMenueObj
// ----------------------------------------------------------------------------

//! Class for legend
class FunkMenueObj :public GraphObj
{
  //! Pointer to diagram
  EFDiagObj *Diag;
  //! Current position and x extension
  int AkX,AkY,XExt;
  //! Flag for frame around legend
  bool Frame;
  //! Legende items
  vector<MenueItemObj> LegendItems;
  //! Mousemove Marker size
  int MarkerSize;

 public:
  //! Ctor
  FunkMenueObj() 
  {
    MarkerSize = 0;
  }
  //! Dtor
  virtual ~FunkMenueObj();
  //! Search menu entry at X, Y
  int HoleBefehl(int X,int Y);
  //! Initialize menu
  void Init(int XE,bool F,EFDiagObj *DO,wxDC &DC);
  //! Determine
  //! Set menu position
  void SetPos(int X,int Y)
  {
    BildBer.XMin = X;
    BildBer.YMin = Y;
    BildBer.XMax += X;
    BildBer.YMax += Y;
  }
  //! Add new menu entry
  void NeuEintrag(int Nr,const _TCHAR *Zeile,unsigned long FarbVar,
                  unsigned short Opt,unsigned short Symb,bool Ft,wxDC &DC);
  //! Activate menu entry Nr
  void Selekt(int Nr,unsigned long Colour,wxDC &DC);
  //! Draw menu
  void DrawMenue(unsigned long Colour,unsigned long BColour,
                 bool Printing,wxDC &DC);

 protected:
  //! Delete menu entries
  void LoescheMenue();
  //! Draw frame around menu
  void DrawFrame(wxDC &DC);
};

// ----------------------------------------------------------------------------
// EFDiagObj
// ----------------------------------------------------------------------------

//! Easy Funktion diagram class based on DiagObj
/*!
    It holds a diagram of type DiagObj and holds graphs.
*/
class EFDiagObj :public DiagObj
{
  //! Flag for diagram markers shown
  bool Markiert;
  //! diagram marker position
  double MarkX,MarkY;
  //! Data object
  DarstDataObj *DarstData;
  //! Graph data
  GraphManagerObj *YDaten;
  //! Legend
  FunkMenueObj *FunkMenue;
  //! Strings for marker position
  MGPStr AkPosText,AkDelText;
  //! Function parser
  ParserObj *Parser;
  //! Flag for exponential / engineering number format
  int ZahlForm;
  //! Index for graph data for current marker position
  unsigned int Nummer;
  //! Diagram number
  int DiagNr;

 public:
  //! Line width and symbol radius
  int DuennGraphLinie,SymbolRad;

  //! Mode for graph selection
  enum GraphSelectModes
  {
    //! Do not change active function
    NotMoveCursor = -1
  };

  //! Move modes for diagram marker
  enum MoveModes
  {
    VorWaerts = 0,
    RueckWaerts,
    Links,
    Rechts,
    Min,
    Max,
    SchnellVorWaerts,
    SchnellRueckWaerts,
    AkXPos,
    NeuY,
    Peak
  };

  //! Diagram markers
  MarkObj Mark1,Mark2;
  //! Ctor
  EFDiagObj(ParserObj *Psr, int ZF)
  {
    SetFormats(Psr,ZF);
    YDaten = NULL;
    FunkMenue = NULL;
    DarstData = NULL;
    SetZoom(1);
  }
  //! Set the parser
  void SetFormats(ParserObj* Psr,int ZF)
  {
    Parser = Psr;
    ZahlForm = ZF;
  }
  //! Return marker data index
  unsigned int GetNummer()
  {
    return Nummer;
  }
  //! Initialize
  void Init(BildType &BildBerVar,GraphManagerObj *YD,FunkMenueObj *FM,
            DarstDataObj *DD,const FontType &A,const FontType &T,
            const FontType &AL,const unsigned long &col,
            const NumListFormat &LDV,
            wxDC &DC,const _TCHAR *DT,int DNr);
  //! Return diagram dimensions
  inline const BildType &GetBildBer()
  {
    return BildBer;
  }
  //! Set X screen range
  inline void SetXRange(int XMn,int XMx)
  {
    BildBer.XMin = XMn;
    BildBer.XMax = XMx;
    UpdateBildBer(BildBer);
    Mark1.UpdateBildBer(BildBer);
    Mark2.UpdateBildBer(BildBer);
  }
  //! Return data object
  inline DarstDataObj *GetDarstData()
  {
    return(DarstData);
  }
  //! Select diagram
  void Markieren(wxDC &DC,bool mark);
  //! Determine marker text strings
  void Text(bool Clear);
  //! Return diagram mode
  int GetDiagMode()
  {
    return DarstData->Mode;
  }
  //! Set diagram mode
  void SetDiagMode(int NeuMode)
  {
    DarstData->Mode = NeuMode;
    SetMode(NeuMode);
  }
  //! Returns marker position text
  const _TCHAR *GetAkPosText()
  {
    return AkPosText.c_str();
  }
  //! Returns marker delta text
  const _TCHAR *GetAkDelText()
  {
    return AkDelText.c_str();
  }
  //! Move marker
  void Move(int Ort,int DeltaX,wxDC &DC);
  //! Change zoom
  void SetZoom(double Z)
  {
    DiagObj::SetZoom(Z);
    DuennGraphLinie = MinLinie(1,20.0*Zoom);
    SymbolRad = MinLinie(1,70.0*Zoom);
    Mark1.SetZoom(Z);
    Mark2.SetZoom(Z);
  }
  //! Delete current graph
  bool DelGraph();
  //! Select next function
  bool NaechsteFunktion(int Mode,unsigned long Colour,wxDC &DC);
  //! Draw graph
  void ZeichneGraph(unsigned int Farbe,wxDC &DC);
};

#endif

