// Version 5.12  06.07.2002 Markus Greither
// Copyright (c) 1991-2003 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file efprint.h
    \brief Header for the printout class
*/

#ifndef __EFPRINT__
#define __EFPRINT__

// ----------------------------------------------------------------------------
// EFPrintOut
// ----------------------------------------------------------------------------

//! Printout class
/*!
    Does the printing operation depending on the passed window type
    (Diagram or editor).
*/
class EFPrintOut : public wxPrintout
{
 public:
  //! Ctor, sets window to print
  EFPrintOut (wxWindow* window,const wxString& title);
  //! Gets page count
  void GetPageInfo (int *minPage, int *maxPage, int *selFromPage, int *selToPage);
  //! Set up for printing
  void OnBeginPrinting();
  //! Set up for printing, calculate page numbers
  void BeginPrinting(wxDC *DC);
  //! Set up for screen display (undo changes)
  void OnEndPrinting();
  //! Set up page with correct side ratio etc
  void BeginPage (wxDC *DC);
  //! Print header line with title and date
  void TitelZeile (wxDC *DC,int w, int h,int page);
  //! Print page
  bool OnPrintPage (int page);
  //! Returns true if given page number is available
  bool HasPage (int pageNumber);

 protected:
  //! Window to print
  wxWindow *Window;
  //! Available pages
  int NumPages;
  //! Available pages
  vector<bool> Pages;
};

#endif

