// Version 5.32  23.08.2006 Markus Greither
// Copyright (c) 1991-2006 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file eftabcon.h
    \brief Header for the tab-key support class
*/

// ----------------------------------------------------------------------------
// TEFTabControl
// ----------------------------------------------------------------------------

//! Class for wxResizeableControl with tab-key support
template<class T> class TEFTabControlT : public T
{
  public:
    //! Ctor
    TEFTabControlT() : T() {}
    //! Ctor
    TEFTabControlT(wxWindow *AParent, int AnId,const wxPoint &pos,
                  const wxSize &size,long style = 0,
                  const wxString &name = wxPanelNameStr)
     : T(AParent,AnId,pos,size,style,name) {}
    //! Respond to tab key presses by selecting other child or parent
    void OnKeyDown(wxKeyEvent &event)
    {
      TEFTabControlT::EvaluateKeys(this,event);
    }
    //! Evaluate next child window focus after tab key press, and delete key
    static void EvaluateKeys(wxWindow *wnd,wxKeyEvent &event);
    //! Focus event handling for macro recording
    void OnSetFocus(wxFocusEvent &event);
    //! General event handling for macro recording
    bool ProcessEvent(wxEvent& event);
#ifdef __WXMSW__
    //! Force WS_CLIPSIBLINGS, as wxWidgets doesn't create the flag
    WXDWORD MSWGetStyle(long flags, WXDWORD *exstyle = NULL) const
    {
      return wxWindow::MSWGetStyle(flags,exstyle) | WS_CLIPSIBLINGS;
    }
#endif
};

class TEFTabControl: public TEFTabControlT<wxResizeableControl>
{
  public:
    //! Ctor
    TEFTabControl() : TEFTabControlT<wxResizeableControl>() {}
    //! Ctor
    TEFTabControl(wxWindow *AParent, int AnId,const wxPoint &pos,
                  const wxSize &size,long style = 0,
                  const wxString &name = wxPanelNameStr)
     : TEFTabControlT<wxResizeableControl>(AParent,AnId,pos,size,style,name) {}
    DECLARE_EVENT_TABLE()
    DECLARE_DYNAMIC_CLASS(TEFTabControl)
};

class TEFTabParentControl: public TEFTabControlT<wxResizeableParentControl>
{
  public:
    //! Ctor
    TEFTabParentControl() : TEFTabControlT<wxResizeableParentControl>() {}
    //! Ctor
    TEFTabParentControl(wxWindow *AParent, int AnId,const wxPoint &pos,
                  const wxSize &size,long style = 0,
                  const wxString &name = wxPanelNameStr)
     : TEFTabControlT<wxResizeableParentControl>(AParent,AnId,pos,size,style,name) {}
    DECLARE_EVENT_TABLE()
    DECLARE_DYNAMIC_CLASS(TEFTabParentControl)
};

// ----------------------------------------------------------------------------
// TEFBitmapControl
// ----------------------------------------------------------------------------

//! Class for wxBitmapControl with tab-key support
class TEFBitmapControl : public wxBitmapControl
{
  public:
     //! Ctor
    TEFBitmapControl() : wxBitmapControl() {};
    //! Ctor
    TEFBitmapControl(wxWindow *AParent, int AnId,const wxBitmap &Data,
                     const wxPoint &pos,
                     const wxSize &size,long style = 0,
                     const wxString &name = wxPanelNameStr)
     : wxBitmapControl(AParent,AnId,Data,pos,size,style,name) {}
#ifdef __WXMSW__
    //! Ctor with raw picture data in DIB format
    TEFBitmapControl(wxWindow *AParent, int AnId,char *Data,int Size,
                     const wxPoint &pos,const wxSize &size,long style = 0,
                     const wxString &name = wxPanelNameStr)
     : wxBitmapControl(AParent,AnId,Data,Size,pos,size,style,name) {}
#endif
    //! Respond to tab key presses by selecting other child or parent
    void OnKeyDown(wxKeyEvent &event)
    {
      TEFTabControl::EvaluateKeys(this,event);
    }
    //! Focus event handling for macro recording
    void OnSetFocus(wxFocusEvent &event);
    //! General event handling for macro recording
    bool ProcessEvent(wxEvent& event);
#ifdef __WXMSW__
    //! Force WS_CLIPSIBLINGS, as wxWidgets doesn't create the flag
    WXDWORD MSWGetStyle(long flags, WXDWORD *exstyle = NULL) const
    {
      return wxWindow::MSWGetStyle(flags,exstyle) | WS_CLIPSIBLINGS;
    }
#endif
    DECLARE_EVENT_TABLE()
    DECLARE_DYNAMIC_CLASS(TEFBitmapControl)
};

#ifdef __WXMSW__
// ----------------------------------------------------------------------------
// TEFMetafileControl
// ----------------------------------------------------------------------------

//! Class for wxBitmapControl with tab-key support
class TEFMetafileControl : public wxMetafileControl
{
  public:
     //! Ctor
    TEFMetafileControl() : wxMetafileControl() {};
    //! Ctor
    TEFMetafileControl(wxWindow *AParent, int AnId,WXHANDLE Meta,
                       const wxPoint &pos,
                       const wxSize &size,long style = 0,
                       const wxString &name = wxPanelNameStr)
     : wxMetafileControl(AParent,AnId,Meta,pos,size,style,name) {}
    //! Ctor with raw picture data in EMF format
    TEFMetafileControl(wxWindow *AParent, int AnId,char *Data,int Size,
                     const wxPoint &pos,const wxSize &size,long style = 0,
                     const wxString &name = wxPanelNameStr)
     : wxMetafileControl(AParent,AnId,Data,Size,pos,size,style,name) {}
    //! Respond to tab key presses by selecting other child or parent
    void OnKeyDown(wxKeyEvent &event)
    {
      TEFTabControl::EvaluateKeys(this,event);
    }
    //! Focus event handling for macro recording
    void OnSetFocus(wxFocusEvent &event);
    //! General event handling for macro recording
    bool ProcessEvent(wxEvent& event);
#ifdef __WXMSW__
    //! Force WS_CLIPSIBLINGS, as wxWidgets doesn't create the flag
    WXDWORD MSWGetStyle(long flags, WXDWORD *exstyle = NULL) const
    {
      return wxWindow::MSWGetStyle(flags,exstyle) | WS_CLIPSIBLINGS;
    }
#endif
    DECLARE_EVENT_TABLE()
    DECLARE_DYNAMIC_CLASS(TEFMetafileControl)
};
#endif

