// Version 5.33  31.03.2007 Markus Greither
// Copyright (c) 1991-2007 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file gauss.h
    \brief Header for the gaussian polynom solver and other approximation functions
*/

#ifndef __EFGAUSS__
#define __EFGAUSS__

// ----------------------------------------------------------------------------
// PktRec
// ----------------------------------------------------------------------------

//! Dynamic array holding X/Y value pairs
class PktRec
{
 protected:
  //! Current arrays size
  int ArrSz;
  //! Number of arrays elements
  int Anzahl;
  //! Resize the arrays
  void SetArr(int NewSize)
  {
    X = (double *)realloc(X,sizeof(double)*(NewSize+10));
    Y = (double *)realloc(Y,sizeof(double)*(NewSize+10));
    ArrSz = NewSize+9;
  }

 public:
  //! Pointer to arrays
  double *X,*Y;
  //! Ctor
  PktRec()
  {
    Anzahl = 0;
    ArrSz = 0;
    X = 0;
    Y = 0;
    SetArr(0);
  }
  //! Dtor
  virtual ~PktRec()
  {
    free(X);
    free(Y);
  }
  //! Change arrays size
  void SetAnzahl(int NewSize)
  {
    Anzahl = NewSize;
    if (Anzahl > ArrSz)
      SetArr(NewSize);
  }
  //! Return array size
  int GetAnzahl()
  {
    return Anzahl;
  }
};

// ----------------------------------------------------------------------------
// FuGenObj
// ----------------------------------------------------------------------------

//! Class for generating approximation functions
class FuGenObj :public ZahlFormatObj
{
 public:
  //! Available approximation/smallest error functions
  enum Mode
  {
    Polynom =       0,
    ExpApprox =     1,
    Linear =        2,
    ExpSmallError = 3,
    Potenz =        4,
    Square =        5
  };
 protected: 
  enum
  {
    //! Maximum number of koefficients for Gauss algorithm
    MaxKoeff = 12,
    //! Maximum number of data points for Gauss algorithm
    MaxPkts = 10
  };
  //! Data points
  PktRec PktFeld;

  //! Debug output of MatType
  void WriteMatrix(double **&Matrix,unsigned char &MaxUnb);
  //! Apply log to Arr, use Offset if necessary
  double LogArr(double *Arr,int Anzahl);
 public:
  //! Ctor
  FuGenObj(const NumListFormat &L) : ZahlFormatObj(L)
  {
  }
  //! Gaussian elimination
  bool Elim(double **&Matrix,int AnzUnb);
  //! Read data points from ParamStr
  bool ReadPkts(const _TCHAR *ParamStr);
  //! Determine approximation function
  const _TCHAR *Interpol();
  //! Determine the average function
  const _TCHAR *Ausgleich(int Mode);
  //! Calculate coefficients for average function
  bool Koeffs(double &A,double &B,double &C,bool quadrat);
};

#endif

