// Version 2.35  09.05.2007 Markus Greither
// Copyright (c) 1991-2007 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file mg.h
    \brief Header for general self defined data types
*/

#ifndef __MG__
#define __MG__

#if defined(__BORLANDC__) || defined(__MINGW32__) || defined(_MSC_VER)// || defined(__DMC__)

#include <tchar.h>

#if defined(__MINGW32__)
#if !defined(_TCHAR)
#if defined(UNICODE)
#define _TCHAR wchar_t
#else
#define _TCHAR char
#endif // UNICODE
#endif // _TCHAR
#endif // __MINGW32__

#define MGswprintf swprintf

#else

#if defined(UNICODE)
#define _TCHAR wchar_t
#define _tcschr wcschr
#define _tcsstr wcsstr
#define _tcscpy wcscpy
#define _tcscat wcscat
#define _tcsncpy wcsncpy
#define _tcslen wcslen
#define _tcscmp wcscmp
#define _tcsncmp wcsncmp
#define _stprintf MGswprintf

// Helper function for missing windows-compatible swprintf
int MGswprintf(wchar_t *s,const wchar_t *t,...);

#define _ftprintf fwprintf
#define _totupper(x) (x < 256 ? toupper(x) : x)
#define _totlower(x) (x < 256 ? tolower(x) : x)
#define _tfopen MGwfopen

// Helper for missing _fwopen under UNIX
FILE *MGwfopen(const wchar_t *n,const wchar_t *m);

#define _ttoi (int)_ttol
#define _tcstoul wcstoul
#define _istdigit(x) (x < 256 ? isdigit(x) : 0)
#define _istalnum(x) (x < 256 ? isalnum(x) : 0)
#define _ttol(x) wcstol(x,(_TCHAR **)NULL,10)
#define _mgT(L,x)  L ## x
#define _TEXT(x) _mgT(L,x)
#else
#define _TCHAR char
#define _tcsstr strstr
#define _tcscpy strcpy
#define _tcscat strcat
#define _tcsncpy strncpy
#define _tcslen strlen
#define _tcscmp strcmp
#define _tcsncmp strncmp
#define _stprintf sprintf
#define _ftprintf fprintf
#define _totupper toupper
#define _totlower tolower
#define _tfopen fopen
#define _ttoi atoi
#define _tcstoul strtoul
#define _istdigit isdigit
#define _istalnum isalnum
#define _ttol atol
#define _TEXT(x) x
#endif

#endif

//! Macro for checking character for digits
#define IST_ZAHL(x) (((x) >= '0') && ((x) <= '9'))
//! Macro for checking character for letters
#define IST_BUCHS(x) ((((x) >= 'A') && ((x) <= 'Z')) || (((x) >= 'a') && ((x) <= 'z')))

// ----------------------------------------------------------------------------
// Forward Declarations
// ----------------------------------------------------------------------------

template<class T> class FastString;

//! Read line znr from AkStr, like istream::GetLine()
template<class T> bool GetLine(const _TCHAR *AkStr,FastString<T> &Zeile,int znr);
//! Return number of lines within AkStr
int GetNumLines(const _TCHAR *AkStr);

#include "faststr.h"

// ----------------------------------------------------------------------------
// ZahlArrObj
// ----------------------------------------------------------------------------

//! Enumeration for return values by comparisons
enum CompRes
{
  IsDifferent = 0,
  IsIdentical = 1,
  IsKomplement = 2
};

#include <vector>

using namespace std;

//! Template class for dynamic number arrays
template <class T> class ZahlArrObj : protected vector<T>
{
  //! Embedded iterator
  typename vector<T>::iterator it;
 public:
  //! Ctor
  ZahlArrObj() {it = vector<T>::begin();}
  //! Copy Ctor
  ZahlArrObj(const ZahlArrObj<T> &other) : vector<T>()
  {
    *this = other;
  }
  //! Copy operator
  ZahlArrObj<T> &operator =(const ZahlArrObj<T> &other)
  {
    assign(other.begin(),other.end());
    it = vector<T>::begin();
    return *this;
  }
  //! Set item Int to value T
  void SetValue(unsigned int Int,T Value)
  {
    if (vector<T>::size() <= Int)
      vector<T>::resize(Int+1);
    it = vector<T>::begin();
    it[Int] = Value;
  }
  //! Increment value of item Int
  void IncValue(unsigned int Int)
  {
    if (vector<T>::size() <= Int)
      vector<T>::resize(Int+1);
    it = vector<T>::begin();
    it[Int] += 1;
  }
  //! Decrement value of item Int
  void DecValue(unsigned int Int)
  {
    if (vector<T>::size() <= Int)
      vector<T>::resize(Int+1);
    it = vector<T>::begin();
    it[Int] -= 1;
  }
  //! Return item Int
  T operator [](unsigned int Int)
  {
    it = vector<T>::begin();
    if (Int < vector<T>::size())
      return it[Int];
    else
      return 0;
  }
  //! Determine number of non-zero data items
  unsigned int AnzInts();
  //! Reinitialize
  void Clear()
  {
    vector<T>::clear();
  }
  //! Determine index of smallest value in array
  unsigned int MinNr()
  {
    return Extrema(false);
  }
  //! Determine index of extrema
  unsigned int Extrema(bool Max);
  //! Return index of max non zero data item
  unsigned int MaxInt();
};

//! Determine number of non-zero data items
template <class T> unsigned int ZahlArrObj<T>::AnzInts()
{
  size_t count = 0;
  it = vector<T>::begin();
  for (size_t i = 0;i < vector<T>::size();i++)
    if (it[i] != 0)
      count++;
  return count;
}

template <class T> unsigned int ZahlArrObj<T>::MaxInt()
{
  if (!vector<T>::size())
    return 0;
  it = vector<T>::begin();
  for (size_t i = vector<T>::size()-1;i > 0;i--)
    if (it[i] != 0)
      return i;
  return 0;
}

template <class T> unsigned int ZahlArrObj<T>::Extrema(bool Max)
{
  unsigned int i,Nr;
  T Wert;

  if (!vector<T>::size())
    return 0;

  it = vector<T>::begin();
  Wert = it[0]; // start value
  Nr = 0; // first value as default result
  i = 0;
  while (i < vector<T>::size())
  {
    if ((Max && (it[i] > Wert)) || (!Max && (it[i] < Wert)))
    {
      Wert = it[i];
      Nr = i;
    }
    i++;
  }
  return Nr;
}

// ----------------------------------------------------------------------------
// ShortArrObj
// ----------------------------------------------------------------------------

//! Dynamic number array for shorts
class ShortArrObj : public ZahlArrObj<short>
{
};

// ----------------------------------------------------------------------------
// IntArrObj
// ----------------------------------------------------------------------------

//! Dynamic number array for ints
class IntArrObj : public ZahlArrObj<int>
{
};

// ----------------------------------------------------------------------------
// LongArrObj
// ----------------------------------------------------------------------------

//! Dynamic number array for longs
class LongArrObj : public ZahlArrObj<long>
{
};

// ----------------------------------------------------------------------------
// SetString
// ----------------------------------------------------------------------------

//! Simple String type
/*!
    Emulates MGPStr::SetStr() so it is interchangable
    Better for clipboard copy (static data)
*/
class SetString
{
  //! Data
  _TCHAR str[100];
 public:
  //! Ctor
  SetString()
  {
    str[0] = 0;
  }
  //! Assignment Ctor
  SetString(const SetString &s)
  {
    SetStr(s.c_str());
  }
  //! Assignment
  SetString(const _TCHAR *s)
  {
    SetStr(s);
  }
  //! Bracket operator
  const _TCHAR &operator [](size_t pos) const
  {
    return str[pos];
  }
  //! Accessor
  const _TCHAR *c_str() const {return str;};
  //! Set string for compatibility with MGPStr
  void SetStr(const _TCHAR *other)
  {
    _tcsncpy(str,other,100);
    str[99] = 0;
  }
  //! Set string for compatibility with MGPStr
  void SetStr(const SetString &other)
  {
    _tcsncpy(str,other.c_str(),100);
    str[99] = 0;
  }
  //! Assignment operator
  void operator = (const SetString &Other)
  {
    SetStr(Other.c_str());
  }
  //! Return length for compatibility with MGPStr
  size_t Length() const
  {
    return _tcslen(str);
  }
};

#include "kettobj.h"

// ----------------------------------------------------------------------------
// MGStrListObj
// ----------------------------------------------------------------------------

//! Dynamic array of MGPStr objects
class MGStrListObj :public ListObj<MGPStr>
{
 public:
  //! Returns current entry
  inline MGPStr &GetAkStr()
  {
    return GetAkEint();
  }
  //! Set new value for current entry
  inline void SetAkStr(const _TCHAR *Zeile)
  {
    GetAkEint().SetStr(Zeile);
  }
};


// ----------------------------------------------------------------------------
// BruchZahl
// ----------------------------------------------------------------------------

//! Fraction number with counter and nominal
class BruchZahl
{
 public:
  //! Counter and denominator values
  double Zaehler, Nenner;
  //! Flag for mixed fraction
  bool GemischterBruch;
  //! Ctor
  BruchZahl(double G,double Z,double N)
  {
    if (G < 0)
      Zaehler = G*N-Z;
    else
     Zaehler = G*N+Z;
    if (N)
      Nenner = N;
    else
      Nenner = 1;
    GemischterBruch = (G != 0);
  }
  //! Ctor
  BruchZahl() {Zaehler = 0;Nenner = 1; GemischterBruch = false;};
  //! Multiplication
  BruchZahl operator *(BruchZahl N) const
  {
    BruchZahl Erg = *this;
    Erg.Zaehler *= N.Zaehler;
    Erg.Nenner *= fabs(N.Nenner);
    Erg.Kuerzen();
    return Erg;
  }
  //! Division
  BruchZahl operator /(BruchZahl N) const
  {
    BruchZahl Erg = *this;
    Erg.Zaehler *= N.Nenner;
    Erg.Nenner *= fabs(N.Zaehler);
    Erg.Kuerzen();
    return Erg;
  }
  //! Addition
  BruchZahl operator +(BruchZahl N) const
  {
    BruchZahl Erg = *this;
    Erg.Zaehler = Erg.Zaehler*N.Nenner+N.Zaehler*Erg.Nenner;
    Erg.Nenner = fabs(Erg.Nenner*N.Nenner);
    Erg.Kuerzen();
    return Erg;
  }
  //! Subtraction
  BruchZahl operator -(BruchZahl N) const
  {
    BruchZahl Erg = *this;
    Erg.Zaehler = Erg.Zaehler*N.Nenner-N.Zaehler*Erg.Nenner;
    Erg.Nenner = fabs(Erg.Nenner*N.Nenner);
    Erg.Kuerzen();
    return Erg;
  }
  //! Assignment
  void operator =(const double &dbl)
  {
    Zaehler = dbl;
    Nenner = 1;
  }
  //! Optimisation
  void Kuerzen();
  //! Conversion to double
  operator double () const
  {
    return GetDoubleValue();
  }
  //! Conversion to double
  double GetDoubleValue() const
  {
    double Erg;
    if (Nenner != 1)
      Erg = Zaehler/Nenner;
    else
      Erg = Zaehler;
    return Erg;
  }
  //! Check for fraction
  bool IsBruch()
  {
    return (fmod(Zaehler,1) == 0) && (Nenner != 1);
  }
};

// ----------------------------------------------------------------------------
// NumListFormat
// ----------------------------------------------------------------------------

//! Structure for List and Decimal separators, Leading Zero
struct NumListFormat
{
  //! Ctor
  NumListFormat() {}
  //! Ctor
  NumListFormat(const _TCHAR *L,const _TCHAR *D,_TCHAR V)
   : ListSeparator(L),DecSeparator(D),LeadZero(V)
  {
  }
  //! List separator
  MGPStr ListSeparator;
  //! Decimal separator
  MGPStr DecSeparator;
  //! Sign info, 0 for no leading zeros,1 for must have, 2 for don't care
  _TCHAR LeadZero;
};

// ----------------------------------------------------------------------------
// ZahlFormatObj
// ----------------------------------------------------------------------------

//! Class for formatting numbers.
/*!
    Uses internally defined number formats.
*/
class ZahlFormatObj
{
 public:
 protected:
  //! Current locale decimal separator
  static MGPStr LocDecSeparator;
  //! Current format definition
  NumListFormat Format;
  //! Marker used for translation of strings from one locale to another
  static const _TCHAR StdToWinChar;
  //! Compatibility for Easy Funktion Version 4 (Blank before multiplicator)
  bool KompatFormat;
  //! Read number of (up to 9) digits and converts to unsigned long
  unsigned long ZiffernFolge(const _TCHAR *Wert,unsigned int &p,int &Fehler);

 public:
  //! Default value for numeric and list separators
  static const NumListFormat DefaultFormat;
  //! Number formats
  enum ZahlFormate
  {
    IngForm = 0,
    ExpForm,
    ScalForm,
    FitForm,
    FitForm2,
    LongForm,
    LongExpForm,
    ScalNForm
  };
  //! Error flags
  enum ZahlFehler
  {
    NoError = 0,
    Overflow = 101,
    WrongValue,
    MaxZahlFehler
  };
  //! Ctor
  ZahlFormatObj()
  {
    Format = DefaultFormat;
    KompatFormat = false;
  }
  //! Copy Ctor
  ZahlFormatObj(const NumListFormat &iFormat)
  {
    Format = iFormat;
    KompatFormat = false;
  }
  //! Set number format
  void SetLDV(const NumListFormat &iFormat)
  {
    Format = iFormat;
  }
  //! Set current locale decimal separator
  static void SetLocDecSeparator(const _TCHAR *newsep)
  {
    LocDecSeparator = newsep;
  }
  //! Get current locale decimal separator
  static const _TCHAR *GetLocDecSeparator()
  {
    return LocDecSeparator.c_str();
  }
  //! Return number format
  const NumListFormat &GetLDV() const
  {
    return Format;
  }
  //! Parse Wert from position p and return number including mixed fractions
  double Zahl(const _TCHAR *Wert,unsigned int &p,int &Fehler);
  //! Read number broken up in its components
  bool ParseZahl(const _TCHAR *Wert,unsigned int &p,int &Fehler,BruchZahl &Z);
  //! Multiplication of Wert1 and Wert2 using mixed fractions
  bool BruchMulDiv(BruchZahl Wert1,BruchZahl Wert2,BruchZahl &Erg,
                   const _TCHAR RechZei);
  //! Parse Wert from position p and return number (No fractions)
  double ZahlOhneBruch(const _TCHAR *Wert,unsigned int &p,int &Fehler);
  //! Parse Wert and return number
  double ZahlWert(const _TCHAR *Wert);
  //! Format w into string AusStr with format Form and NumDigits digits
  void Ausgabe(MGPStr &AusStr,double w,int Form,int NumDigits = 0);
  //! Convert numbers in Erg from english into internal format
  void StdToWin(MGPStr &Erg);
  //! Convert numbers in Erg from internal into english format
  void WinToStd(MGPStr &Erg);
  //! Set compatibility for EF Versions before 4.11 (blank before multiplicator)
  inline void SetKompatMode(bool Kompatibel)
  {
    KompatFormat = Kompatibel;
  }
  //! Convertion to String
  bool BruchOut(BruchZahl Z,MGPStr &Erg);
};

// ----------------------------------------------------------------------------
// Functions
// ----------------------------------------------------------------------------

//! Convert str1 to lower case
void MGTolower(_TCHAR *str1);
//! Convert str1 to upper case
void MGToupper(_TCHAR *str1);
//! Compare str1 and str2 without case
int MGStricmp(const _TCHAR *str1,const _TCHAR *str2);
//! Compare n characters from str1 and str2 without case
int MGStrnicmp(const _TCHAR *str1,const _TCHAR *str2,int n);
//! Finds position of character c in str and returns pointer to it
const _TCHAR *MGStrnchr(const _TCHAR *str,int len,_TCHAR c);
//! Conversion from char to UNICODE
MGPStr ConvertStr(const char *astr);
//! Assign char string to UNICODE string
template<class T> void AssignStr(T &str,const char *astr)
{
  str.SetStr(ConvertStr(astr).c_str());
}
//! Conversion of string from UNICODE to char
void ConvertStr(const _TCHAR *AStr,char *res);
//! Conversion of string from UNICODE to char
CharString UnicodeToChar(const _TCHAR *AStr);
//! Stream output
ostream &operator <<(ostream &stream,const MGPStr &AStr);
//! Stream output
ostream &operator <<(ostream &stream,const SetString &AStr);
#ifdef UNICODE
//! Stream output
ostream &operator <<(ostream &stream,const _TCHAR *AStr);
#endif

template<class T>bool GetLine(const _TCHAR *AkStr,FastString<T> &Zeile,int znr)
{
  int i = 0,MxLen = 0;
  const _TCHAR *tst = AkStr, *AnfStr = 0, *EndStr;

  Zeile.SetStr(_TEXT(""));
  
  do
  { // Until end of string
    if (i == znr)
      AnfStr = tst; // Remember beginning of current line
    while (*tst && (*tst != '\r') && (*tst != '\n'))
      tst++; // find line break
    EndStr = tst; // Remember end of current line
    if ((*tst == '\r') && (*(tst+1) == '\n'))
      tst++; // DOS linebreak
    if (*tst) // If line break, skip it
      tst += 1;
    i++; // increase line count
    if (i > znr)
      MxLen = int(EndStr-AnfStr); // Calculate line length
  }
  while (*tst && (i <= znr));
  if (AnfStr)
  { // If line available
    Zeile.AddNChar(AnfStr,MxLen); // copy line
    return true;
  }
  else
    return false;
}


#endif


