// Version 3.41  29.11.2009 Markus Greither
// Copyright (c) 1991-2009 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file diagramm.cpp
    \brief Source code for the diagram class and related items
*/


#include "efprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#ifdef __DMC__
#include <typeinfo.h>
#endif
#include <wx/wx.h>

#include <cfloat>
#include <cmath>
#include <iostream>
#include <set>
#include <functional>
#if defined(__MINGW32__)
#include <wx/msw/winundef.h>
#endif

using namespace std;

#include "efdefs.h"
#include "mg.h"
#include "diagramm.h"

#endif

// ----------------------------------------------------------------------------
// TextAlignObj
// ----------------------------------------------------------------------------

wxSize TextAlignObj::CalcText(wxDC &dc)
{
  GesLen = 0;
  for (unsigned int i = 0;i < text.Length();i++)
  { // Abmessungen der Y-Einheit Buchstabe fuer Buchstabe ermitteln
    wxCoord x,y;
    wxString str(&text[i]);
    str.Truncate(1);
    dc.GetTextExtent(str,&x,&y);
    Height = y;
    CharLen.SetValue((int)i,x);
    GesLen += CharLen[i];
  }
  return wxSize(GesLen,Height);
}

wxPoint TextAlignObj::GetTopLeftPos(int outx,int outy)
{
  wxPoint pos;

  switch (AlX)
  {
    case taRIGHT:
      pos.x = outx-(Vertikal ? Height : GesLen);
      break;
    case taCENTER:
      pos.x = outx-(Vertikal ? Height : GesLen)/2;
      break;
    default:
      pos.x = outx;
      break;
  }
  switch (AlY)
  {
    case taBOTTOM:
      pos.y = outy-(Vertikal ? GesLen : Height);
      break;
    case taCENTER:
      pos.y = outy-(Vertikal ? GesLen : Height)/2;
      break;
    default:
      pos.y = outy;
      break;
  }
  return pos;
}

void TextAlignObj::TextOut(wxDC &dc,int outx,int outy)
{
  int x,y;

  if (!text[0]) return;

  switch (AlX)
  {
    case taRIGHT:
      x = outx-(Vertikal ? Height : GesLen);
      break;
    case taCENTER:
      x = outx-(Vertikal ? Height : GesLen)/2;
      break;
    default:
      x = outx;
      break;
  }
  switch (AlY)
  {
    case taBOTTOM:
      y = outy+(Vertikal ? 0 : -Height);
      break;
    case taCENTER:
      y = outy+(Vertikal ? GesLen : -Height)/2;
      break;
    case taTOP:
      y = outy+(Vertikal ? GesLen : 0);
      break;
    default:
      y = outy;
      break;
  }
  for (unsigned int i = 0;i < text.Length();i++)
  { // Text Buchstabe fr Buchstabe ausgeben
    if (Vertikal)
    {
      wxString str(&text[i]);
      str.Truncate(1);
      dc.DrawRotatedText(str,x,y,90);
      y -= CharLen[i];
    }
    else
    {
      wxString str(&text[i]);
      str.Truncate(1);
      dc.DrawText(str,x,y);
      x += CharLen[i];
    }
  }
}

// ----------------------------------------------------------------------------
// GraphObj
// ----------------------------------------------------------------------------

bool GraphObj::PunktInObject(int X,int Y)
{
  return((X >= BildBer.XMin) && (X <= BildBer.XMax) &&
         (Y >= BildBer.YMin) && (Y <= BildBer.YMax));
}

// ----------------------------------------------------------------------------
// SkalObj
// ----------------------------------------------------------------------------

void SkalObj::GrInit(BildType BildB,AxisDataArr &XA,AxisDataArr &YA)
{
  XAxis = &XA;
  YAxis = &YA;
  UpdateBildBer(BildB);
}

void SkalObj::UpdateBildBer(BildType BB)
{
  BildBer = BB;
  XAxis->Begin();
  do
  {
    AxisData &ad = XAxis->GetAkEint();
    if (ad.GetLog())
    {
      ad.Fak = (BildBer.XMax-BildBer.XMin)/(log(ad.Max)-log(ad.Min));
      ad.Sum = ad.Fak*log(ad.Min);
      if (fabs(ad.Sum) < 1e-30)
        ad.Sum = 0;
    }
    else
      ad.Fak = (BildBer.XMax-BildBer.XMin)/(ad.Max-ad.Min);
  }
  while (XAxis->NextEint());
  YAxis->Begin();
  do
  {
    AxisData &ad = YAxis->GetAkEint();
    if (ad.GetLog())
    {
      ad.Fak = (BildBer.YMax-BildBer.YMin)/(log(ad.Max)-log(ad.Min));
      ad.Sum = ad.Fak*log(ad.Min);
      if (fabs(ad.Sum) < 1e-30)
        ad.Sum = 0;
    }
    else
      ad.Fak = (BildBer.YMax-BildBer.YMin)/(ad.Max-ad.Min);
  }
  while (YAxis->NextEint());
}

long SkalObj::XPos(double Wert)
{
  if ((*XAxis)->GetLog())
  {
    if (Wert <= 1e-150)
      return BildBer.XMin-10;
    return (BildBer.XMin+long((*XAxis)->Fak*log(Wert)-(*XAxis)->Sum+.5));
  }
  else
  {
    if (Wert > (*XAxis)->Max+((*XAxis)->Max-(*XAxis)->Min)*3)
      return BildBer.XMax+(BildBer.XMax-BildBer.XMin)*3;
    if (Wert < (*XAxis)->Min-((*XAxis)->Max-(*XAxis)->Min)*3)
      return BildBer.YMin-(BildBer.XMax-BildBer.XMin)*3;
    return (BildBer.XMin+long((*XAxis)->Fak*(Wert-(*XAxis)->Min)+.5));
  }
}

long SkalObj::YPos(double Wert)
{
  if ((*YAxis)->GetLog())
  {
    if (Wert <= 1e-150)
      return BildBer.YMin-10;
    return(BildBer.YMax-long((*YAxis)->Fak*log(Wert)-(*YAxis)->Sum+.5));
  }
  else
  {
    if (Wert > (*YAxis)->Max+((*YAxis)->Max-(*YAxis)->Min)*3)
      return BildBer.YMin-(BildBer.YMax-BildBer.YMin)*3;
    if (Wert < (*YAxis)->Min-((*YAxis)->Max-(*YAxis)->Min)*3)
      return BildBer.YMax+(BildBer.YMax-BildBer.YMin)*3;
    return (BildBer.YMax-long((*YAxis)->Fak*(Wert-(*YAxis)->Min)+.5));
  }
}

double SkalObj::XWert(long Pos)
{
  if ((*XAxis)->GetLog())
    return(exp((Pos-BildBer.XMin+(*XAxis)->Sum)/(*XAxis)->Fak));
  else
    return((Pos-BildBer.XMin)/(*XAxis)->Fak+(*XAxis)->Min);
}


double SkalObj::YWert(long Pos)
{
  if ((*YAxis)->GetLog())
    return(exp((BildBer.YMax-Pos+(*YAxis)->Sum)/(*YAxis)->Fak));
  else
    return((BildBer.YMax-Pos)/(*YAxis)->Fak+(*YAxis)->Min);
}


SkalObj::~SkalObj()
{
}

// ----------------------------------------------------------------------------
// DiagObj
// ----------------------------------------------------------------------------

void DiagObj::SetTextSize(wxDC &DC,const MGPStr &Zeile,bool YLinien)
{
  wxSize size;

   DC.GetTextExtent(wxString(Zeile.c_str()),&size.x,&size.y);
   if (YLinien)
   { // Y-Achsen Labels
     size.x += TickLen;
     if (size.x > (*YAxis)->TextL)
       (*YAxis)->TextL = size.x;
     if (size.y > (*YAxis)->TextH)
       (*YAxis)->TextH = size.y;
   }
   else
   { // X-Achsen Labels
     if (Mode & DiaXVertikal)
     { // Vertikale X-Labels
       wxSize size1;
       DC.GetTextExtent(wxString(wxT("  ")),&size1.x,&size1.y);
       size.x += TickLen+size1.x;
       if (size.y > (*XAxis)->TextL)
         (*XAxis)->TextL = size.y;
       if (size.x > (*XAxis)->TextH)
         (*XAxis)->TextH = size.x;
     }
     else
     {
       size.y = size.y*3/2+TickLen;
       if (size.x > (*XAxis)->TextL)
         (*XAxis)->TextL = size.x;
       if (size.y > (*XAxis)->TextH)
         (*XAxis)->TextH = size.y;
     }
   }
}

void DiagObj::LabelOut(bool YLinien,int AkPos,const _TCHAR *Zeile,
                       TextAlignObj &TextAlign,int pos,bool OppositeSide,wxDC &DC)
{
  if (YLinien)
  {
    TextAlign.SetText(Zeile);
    TextAlign.CalcText(DC);
    TextAlign.TextOut(DC,!OppositeSide ? BildBer.XMin-TickLen-pos : 
                                         BildBer.XMax+TickLen+pos,
                      AkPos+(*YAxis)->TextH/2);
  }
  else
  {
    TextAlign.SetText(Zeile);
    TextAlign.CalcText(DC);
    TextAlign.TextOut(DC,AkPos,BildBer.YMax+TickLen+
                               (Mode & DiaXVertikal ? (*XAxis)->TextL/4 : (*XAxis)->TextH/4));
  }
}

void DiagObj::FormatWert(double Wert,MGPStr &Zeile,bool YLinien,int Format,bool OppositeSide)
{
  Ausgabe(Zeile,Wert,Format,NumDigits);
  if (YLinien)
  {
    if (!OppositeSide && (Zeile[Zeile.Length()-1] != ' '))
      Zeile.AddStr(wxT(" "));
    if (OppositeSide && (Zeile[0] != ' '))
      Zeile.InsStr(0,wxT(" "));
  }
  else
  if (!YLinien)
  {
    while (Zeile[Zeile.Length()-1] == ' ')
      Zeile.ShortStr(Zeile.Length()-1);
    while (Zeile[0] == ' ')
      Zeile.DelNChar(0,1);
  }
}

void DiagObj::LinLinien(AxisData &ad,bool YLinien,wxDC &DC,bool Darst,int textpos)
{
  int i,f;
  int AkPos;
  int Step = 0,AkZ;
  int UnDiLin;
  double AkWert;
  MGPStr Zeile;
  wxPen SolidLin,DickLin,DashedLin;
  TextAlignObj TextAlign;

  double DiLin = ad.Max-ad.Min;
  // Bereich relativ zur Position ermitteln
  int Ausschnitt = int(fabs(ad.Max+ad.Min)/2/DiLin);
  int MaxDig = 10;
  for (i = 1;i < NumDigits;i++)
    MaxDig *= 10;
  while (Ausschnitt > MaxDig)
    Ausschnitt /= 10; // Mindestens 1 Beschriftung zulassen
  if (Ausschnitt < 1)
    Ausschnitt = 1;
  // Maximale Beschriftungen, die darstellbar sind mit 3 Stellen
  int MaxLabels = MaxDig/Ausschnitt;
  int z = 0;
  while (DiLin*pow(10.0,double(z)) < 10)
    z++; // Anzahl Hauptlinien zw. 1 und 10 normieren
  while (DiLin*pow(10.0,double(z)) >= 100)
    z--;
  int Hi = int(DiLin*pow(10.0,double(z)));
  if (MaxLabels < 10)
    Hi = (Hi*10)/MaxLabels; // Anzahl der Beschriftungen auf max. setzen
  if (Hi <= 20)
  { // 2 und weniger Hauptlinien
    Hi = 2; // Hauptlinien in 1/5-Schritten
    UnDiLin = 2; // 1-fache Teilung zw. Hauptlinien

  }
  else
  if (Hi <= 40)
  { // wenn zw 2 und 4 Hauptlinien
    Hi = 5; // Hauptlinien in 1/2-Schritten
    UnDiLin = 2; // 1-fache Teilung zw. Hauptlinien
  }
  else
  if (Hi <= 80)
  { // wenn zw 4 und 8 Hauptlinien
    Hi = 10; // Hauptlinien in ganzen Schritten
    UnDiLin = 2; // 1-fache Teilung zw. Hauptlinien
  }
  else
  { // wenn mehr als 8 Hauptlinien
    Hi = 20; // Hauptlinien in 2-Schritten
    UnDiLin = 4; // 1-fache Teilung zw. Hauptlinien
  }
  DiLin = Hi*pow(10.0,double(-z));
  if (Darst)
  {
    SolidLin = wxPen(wxColour(0,0,0),0,wxSOLID);
    DashedLin = wxPen(wxColour(100,100,100),0,wxDOT);
    DickLin = wxPen(wxColour(0,0,0),FettLinie,wxSOLID);
    if (YLinien)
    {
      if (!ad.GetOppositeSide())
        TextAlign.SetTextAlign(TextAlignObj::taRIGHT,TextAlignObj::taBOTTOM);
      else
        TextAlign.SetTextAlign(TextAlignObj::taLEFT,TextAlignObj::taBOTTOM);
    }
    else
    {
      TextAlign.SetTextAlign(TextAlignObj::taCENTER,TextAlignObj::taTOP);
      TextAlign.SetVertikal(Mode & DiaXVertikal);
    }
    if (YLinien) // Pruefen ob Text zw. Dekaden passt
      Step = int(1.1*ad.TextH/(ad.Fak*DiLin)+1);
    else // Pruefen ob Text zw. Dekaden passt
      Step = int(1.1*ad.TextL/(ad.Fak*DiLin)+1);
  }
  i = int(ad.Min / DiLin);
  if (i*DiLin >= ad.Min)
    i--;
  if (!Darst || (i >= 0))
    AkZ = Step;
  else // Sicherstellen, dass 0-Achse immer ausgegeben wird !
    AkZ = Step - abs(i+1) % Step;
  int MinPos = YLinien ? (int)BildBer.YMax : (int)BildBer.XMin;
  int MaxPos = YLinien ? (int)BildBer.YMin : (int)BildBer.XMax;
  if (Mode & DiaEndLabels)
  {
    FormatWert(ad.Min,Zeile,YLinien,FitForm2,ad.GetOppositeSide());
    if (!Darst)
      SetTextSize(DC,Zeile,YLinien);
    else
      LabelOut(YLinien,MinPos,Zeile.c_str(),TextAlign,textpos,ad.GetOppositeSide(),DC);
    FormatWert(ad.Max,Zeile,YLinien,FitForm2,ad.GetOppositeSide());
    if (!Darst)
      SetTextSize(DC,Zeile,YLinien);
    else
      LabelOut(YLinien,MaxPos,Zeile.c_str(),TextAlign,textpos,ad.GetOppositeSide(),DC);
  }
  f = 0;
  while (i*DiLin+f*DiLin/UnDiLin < ad.Min)
    f++;
  if (f == UnDiLin)
    f = 0;
  while (i*DiLin < ad.Max)
  {
    if (f > 0)
    {
      while ((f < UnDiLin) && (i*DiLin+f*DiLin/UnDiLin <= ad.Max))
      {
        if (Darst)
        {
           AkWert = (i*DiLin+f*DiLin/UnDiLin);
           if (YLinien)
           {
             AkPos = (int)YPos(AkWert);
             DC.SetPen(DashedLin);
             YLinieZeichnen(DC,AkPos,false,textpos,ad.GetOppositeSide());
             DC.SetPen(wxNullPen);
           }
           else
           {
             AkPos = (int)XPos(AkWert);
             DC.SetPen(DashedLin);
             XLinieZeichnen(DC,AkPos,false);
             DC.SetPen(wxNullPen);
           }
        }
        f++;
      }
    }
    i++;
    if (i*DiLin <= ad.Max)
    {
      FormatWert(i*DiLin,Zeile,YLinien,ScalNForm,ad.GetOppositeSide());
      if (!Darst)
        SetTextSize(DC,Zeile,YLinien);
      AkZ++;
      if (AkZ > Step)
        AkZ = 1;
      if (Darst)
      {
        AkWert = (i*DiLin);
        if (YLinien)
        {
          AkPos = (int)YPos(AkWert);
          if (AkZ == 1)
          {
            if (!(Mode & DiaEndLabels) ||
                (AkPos+1.1*ad.TextH <= MinPos) && (AkPos-1.1*ad.TextH >= MaxPos))
              LabelOut(YLinien,AkPos,Zeile.c_str(),TextAlign,textpos,ad.GetOppositeSide(),DC);
            if (i == 0)
              DC.SetPen(DickLin);
            else
              DC.SetPen(SolidLin);
          }
          else
            DC.SetPen(DashedLin);
          YLinieZeichnen(DC,AkPos,AkZ == 1,textpos,ad.GetOppositeSide());
          DC.SetPen(wxNullPen);
        }
        else
        {
          AkPos = (int)XPos(AkWert);
          if (AkZ == 1)
          {
            if (!(Mode & DiaEndLabels) ||
                (AkPos-1.1*ad.TextL >= MinPos) && (AkPos+1.1*ad.TextL <= MaxPos))
              LabelOut(YLinien,AkPos,Zeile.c_str(),TextAlign,textpos,ad.GetOppositeSide(),DC);
            if (i == 0)
              DC.SetPen(DickLin);
            else
              DC.SetPen(SolidLin);
          }
          else
            DC.SetPen(DashedLin);
          XLinieZeichnen(DC,AkPos,AkZ == 1);
          DC.SetPen(wxNullPen);
        }
      }
      f = 1;
    }
  }
  if (Darst && YLinien && (textpos || ad.GetOppositeSide()))
  {
    DC.SetPen(SolidLin);
    if (ad.GetOppositeSide())
      DC.DrawLine(BildBer.XMax+textpos,BildBer.YMin,BildBer.XMax+textpos,BildBer.YMax);
    else
      DC.DrawLine(BildBer.XMin-textpos,BildBer.YMin,BildBer.XMin-textpos,BildBer.YMax);
    DC.SetPen(wxNullPen);
  }
}

void DiagObj::YLinieZeichnen(wxDC &DC,int y,bool Solid,int textpos,bool OppositeSide)
{
  if (!OppositeSide)
    DC.DrawLine(BildBer.XMin-(Solid ? TickLen : 0)-textpos,y,
                textpos ? BildBer.XMin-textpos : BildBer.XMax,y);
  else
    DC.DrawLine(BildBer.XMax+textpos,y,
                BildBer.XMax+(Solid ? TickLen : 0)+textpos,y);
}

void DiagObj::XLinieZeichnen(wxDC &DC,int x,bool Solid)
{
  if (Solid)
    DC.DrawLine(x,BildBer.YMax+TickLen,x,BildBer.YMin);
  else
    DC.DrawLine(x,BildBer.YMax,x,BildBer.YMin);
}

void DiagObj::LogLinien(AxisData &ad,bool YLinien,wxDC &DC,bool Darst,int textpos)
{
  int AkPos = 0;
  int Step = 0;
  int f,z,AkZ;
  MGPStr Zeile;
  wxPen SolidLin,DashedLin;
  TextAlignObj TextAlign;

  if (Darst)
  {
    SolidLin = wxPen(wxColour(0,0,0),0,wxSOLID);
    DashedLin = wxPen(wxColour(100,100,100),0,wxDOT);
    if (YLinien)
      TextAlign.SetTextAlign(TextAlignObj::taRIGHT,TextAlignObj::taBOTTOM);
    else
    {
      TextAlign.SetTextAlign(TextAlignObj::taCENTER,TextAlignObj::taTOP);
      TextAlign.SetVertikal(Mode & DiaXVertikal);
    }
    if (YLinien) // Pruefen ob Text zw. Dekaden passt
      Step = int(1.1*ad.TextH/(ad.Fak*log(10.0))+1);
    else // Pruefen ob Text zw. Dekaden passt
      Step = int(1.1*ad.TextL/(ad.Fak*log(10.0))+1);
  }
  int MinPos = YLinien ? (int)BildBer.YMax : (int)BildBer.XMin;
  int MaxPos = YLinien ? (int)BildBer.YMin : (int)BildBer.XMax;
  if (Mode & DiaEndLabels)
  {
    FormatWert(ad.Min,Zeile,YLinien,FitForm2,ad.GetOppositeSide());
    if (!Darst)
      SetTextSize(DC,Zeile,YLinien);
    else
      LabelOut(YLinien,MinPos,Zeile.c_str(),TextAlign,textpos,ad.GetOppositeSide(),DC);
    FormatWert(ad.Max,Zeile,YLinien,FitForm2,ad.GetOppositeSide());
    if (!Darst)
      SetTextSize(DC,Zeile,YLinien);
    else
      LabelOut(YLinien,MaxPos,Zeile.c_str(),TextAlign,textpos,ad.GetOppositeSide(),DC);
  }
  f = 1;
  z = 1;
  AkZ = Step;
  while (ad.Min > f*pow(10.0,double(z)))
    z += 1;
  while (ad.Min < f*pow(10.0,double(z)))
    z -= 1;
  int aktextpos = 0;
  do
  {
    if (Darst)
    {
      if (YLinien)
        AkPos = (int)YPos(f*pow(10.0,double(z)));
      else
        AkPos = (int)XPos(f*pow(10.0,double(z)));
    }
    if ((f*pow(10.0,double(z))) >= ad.Min)
    { // wenn Linie im sichtbaren Bereich
      FormatWert(f*pow(10.0,double(z)),Zeile,YLinien,ScalNForm,ad.GetOppositeSide());
      if (!Darst)
        SetTextSize(DC,Zeile,YLinien);
      else
      {
        if ((f == 1) || (double(f)/2 != int(f/2)) || (ad.Max <= ad.Min*1e5) ||
            ((Mode & DiaAlleLinienX) && !YLinien) ||
            ((Mode & DiaAlleLinienY) && YLinien))
        {// wenn Linie gezeichnet werden soll
          if (f != 1)
          { // Hilfslinie
            DC.SetPen(DashedLin);
            if (YLinien)
              YLinieZeichnen(DC,AkPos,false,textpos,ad.GetOppositeSide());
            else
              XLinieZeichnen(DC,AkPos,false);
            DC.SetPen(wxNullPen);
          }
          else
          { // Hauptlinie in jedem Fall durchziehen
            DC.SetPen(SolidLin);
            if (YLinien)
              YLinieZeichnen(DC,AkPos,true,textpos,ad.GetOppositeSide());
            else
              XLinieZeichnen(DC,AkPos,true);
            DC.SetPen(wxNullPen);
          }
          if ((f == 1) || (f == 3) && (ad.Max < ad.Min*300) || (ad.Max < ad.Min*10))
          { // wenn Linie beschriftet werden soll
            AkZ++;
            if (AkZ > Step)
              AkZ = 1;
            if (AkZ == 1)
            {
              if (YLinien)
              {
                if ((!(Mode & DiaEndLabels) ||
                     (AkPos+1.1*ad.TextH <= MinPos) &&
                     (AkPos-1.1*ad.TextH >= MaxPos)) &&
                    (abs(AkPos-aktextpos) >= 1.1*ad.TextH))
                {
                  LabelOut(YLinien,AkPos,Zeile.c_str(),TextAlign,textpos,ad.GetOppositeSide(),DC);
                  aktextpos = AkPos;
                }
              }
              else
              {
                if ((!(Mode & DiaEndLabels) ||
                     (AkPos-1.1*ad.TextL >= MinPos) &&
                     (AkPos+1.1*ad.TextL <= MaxPos)) &&
                    (abs(AkPos-aktextpos) >= 1.1*ad.TextL))
                {
                  LabelOut(YLinien,AkPos,Zeile.c_str(),TextAlign,textpos,ad.GetOppositeSide(),DC);
                  aktextpos = AkPos;
                }
              }
            }
          }
        }
      }
    }
    f = f+1;
    if (f == 10)
    { // Ende der Dekade
      z += 1; // Potenz erhoehen
      f = 1; // Zaehler zuruecksetzen
    }
  }
  while (f*pow(10.0,double(z)) <= ad.Max);
  if (Darst && YLinien && textpos)
  {
    DC.SetPen(SolidLin);
    if (!ad.GetOppositeSide())
      DC.DrawLine(BildBer.XMin-textpos,BildBer.YMin,BildBer.XMin-textpos,BildBer.YMax);
    else
      DC.DrawLine(BildBer.XMax+textpos,BildBer.YMin,BildBer.XMax-textpos,BildBer.YMax);
    DC.SetPen(wxNullPen);
  }
  if (Darst && YLinien && (textpos || ad.GetOppositeSide()))
  {
    DC.SetPen(SolidLin);
    if (ad.GetOppositeSide())
      DC.DrawLine(BildBer.XMax+textpos,BildBer.YMin,BildBer.XMax+textpos,BildBer.YMax);
    else
      DC.DrawLine(BildBer.XMin-textpos,BildBer.YMin,BildBer.XMin-textpos,BildBer.YMax);
    DC.SetPen(wxNullPen);
  }
}

void DiagObj::Linien(wxDC &DC,bool Darst)
{
  wxFont *HAchse = GetFont(DC,Achse); // Titel-Schrift ermitteln
  // Achsen-Beschriftungs-Schrift ermitteln
  wxFont *HAchsLabels = GetFont(DC, AchsLabels);
  // Vorherige Schrift ermitteln, Titel-Schrift aktivieren
  wxFont *HTitel = GetFont(DC, Titel);
  wxColor Farbe = DC.GetTextForeground();
  if (!Darst)
  { // Nicht Darstellen, sondern Abmessungen berechnen
    DC.SetTextForeground(Titel.Color);
    DC.SetFont(*HTitel); // Titel-Schrift aktivieren
    TText.SetText(DiagText.c_str());
    // Abmessungen des Titels Buchstabe fuer Buchstabe ermitteln
    TText.CalcText(DC);
    TText.SetTextAlign(TextAlignObj::taCENTER,TextAlignObj::taBOTTOM);
    DC.SetFont(*HAchsLabels);
    // Abmessungen der X-Einheit Buchstabe fuer Buchstabe ermitteln
    XText.ResetKette();
    XAxis->Begin();
    do
    {
      XText.CreateAkEint(XAxis->AkEintNummer());
      XText->SetText((*XAxis)->Label.c_str());
      XText->CalcText(DC);
      XText->SetTextAlign(TextAlignObj::taCENTER,TextAlignObj::taBOTTOM);
    }
    while (XAxis->NextEint());
    // Abmessungen der Y-Einheit Buchstabe fuer Buchstabe ermitteln
    YText.ResetKette();
    size_t count = 1;
    YAxis->Begin();
    YAxis->NextEint();
    do
      if ((*YAxis)->GetVisible())
        count++;
    while (YAxis->NextEint());
    YAxis->Begin();
    do
    {
      YText.CreateAkEint(YAxis->AkEintNummer());
      MGPStr tmp;
      if (count > 1)
      {
        tmp += _TEXT("[");
        tmp << YAxis->AkEintNummer();
        tmp += _TEXT("] ");
      }
      tmp += (*YAxis)->Label.c_str();
      YText->SetText(tmp.c_str());
      YText->CalcText(DC);
      YText->SetTextAlign(TextAlignObj::taLEFT,TextAlignObj::taCENTER);
      YText->SetVertikal(true);
    }
    while (YAxis->NextEint());
  }
  else
  {
    // Paint background of diagram
    wxPen pen(m_Colour,1,wxSOLID);
    wxBrush brush(m_Colour,wxSOLID);
    DC.SetPen(pen);
    const wxBrush &OldBr = DC.GetBrush();
    DC.SetBrush(brush);
    DC.DrawRectangle(BildBer.XMin,BildBer.YMin,
                     BildBer.XMax-BildBer.XMin,BildBer.YMax-BildBer.YMin);
    DC.SetPen(wxNullPen);
    DC.SetBrush(OldBr);
  }  
  // Y axis
  // Schriftfarbe fuer Achsen-Nummerierung einstellen
  DC.SetTextForeground(Achse.Color);
  DC.SetFont(*HAchse); // Schrift fuer Achsen-Nummerierung einstellen
  YAxis->Begin();
  YText.Begin();
  int xposleft = 0,xposright = 0;
  do
  {
    if ((*YAxis)->GetVisible())
      if ((*YAxis)->GetLog())
        LogLinien(YAxis->GetAkEint(),true,DC,Darst,(*YAxis)->GetOppositeSide() ? xposright : xposleft); // Y-Achse berechnen/zeichnen
      else
        LinLinien(YAxis->GetAkEint(),true,DC,Darst,(*YAxis)->GetOppositeSide() ? xposright : xposleft);
    if ((*YAxis)->GetOppositeSide())
      xposright += GetXTextLen();
    else
      xposleft += GetXTextLen();
  }
  while (YAxis->NextEint() && YText.NextEint());
  // X-Ma�tab
  XAxis->Begin();
//  do
  {
    if ((*XAxis)->GetLog())
      LogLinien(XAxis->GetAkEint(),false,DC,Darst,0); // X-Achse berechnen/zeichnen
    else
      LinLinien(XAxis->GetAkEint(),false,DC,Darst,0);
  }
//  while (XAxis->NextEint());
  if (Darst)
  { // wenn Diagramm gezeichnet werden soll
    if (Mode & DiaRahmen)
    { // wenn DiaRahmen, Rahmen um Diagramm ziehen
      wxPen SolidLin = wxPen(wxColour(0,0,0),RahmenLinie,wxSOLID);
      DC.SetPen(SolidLin);
      DC.DrawLine(BildBer.XMin,BildBer.YMin,BildBer.XMax,BildBer.YMin);
      DC.DrawLine(BildBer.XMax,BildBer.YMin,BildBer.XMax,BildBer.YMax);
      DC.DrawLine(BildBer.XMax,BildBer.YMax,BildBer.XMin,BildBer.YMax);
      DC.DrawLine(BildBer.XMin,BildBer.YMax,BildBer.XMin,BildBer.YMin);
      DC.SetPen(wxNullPen);
    }
    DC.SetTextForeground(Titel.Color);
    // Titel mit Titel-Schrift zentriert ueber Diagramm ausgeben
    DC.SetFont(*HTitel);
    TText.TextOut(DC,DiagTextPos());
    DC.SetTextForeground(AchsLabels.Color);
    // X-Achsen-Beschriftung zentriert unter Diagramm ausgeben
    DC.SetFont(*HAchsLabels);
    XText.Begin();
//    do
    {
      if (XText->GetText().Length() > 0)
      { // wenn X-Achsen-Beschriftung vorhanden
        wxPoint pt = XTextPos();
        XText->TextOut(DC,pt);
      }
    }
//    while (XText.NextEint());
    YText.Begin();
    YAxis->Begin();
    do
    {
      if (((*YAxis)->GetVisible()) && (YText->GetText().Length() > 0))
      { // wenn Y-Achsen-Beschriftung vorhanden
        wxPoint pt = YTextPos();
        size_t oldpos = YAxis->AkEintNummer();
        bool OppositeSide = (*YAxis)->GetOppositeSide();
        if (!OppositeSide)
        {
          YAxis->NextEint();
          do
            if (((*YAxis)->GetOppositeSide() == OppositeSide) && (*YAxis)->GetVisible())
              pt.x += xposarr[YAxis->AkEintNummer()];
          while (YAxis->NextEint());
          YText->SetTextAlign(TextAlignObj::taLEFT,TextAlignObj::taCENTER);
        }
        else
        {
          YAxis->Begin();
          do
            if (((*YAxis)->GetOppositeSide() == OppositeSide) && (*YAxis)->GetVisible())
              pt.x -= xposarr[YAxis->AkEintNummer()];
          while (YAxis->NextEint() && (YAxis->AkEintNummer() <= oldpos));
          pt.x = BildBer.XMax-pt.x;
          YText->SetTextAlign(TextAlignObj::taRIGHT,TextAlignObj::taCENTER);
        }
        YAxis->SetAkEint(oldpos);
        YText->TextOut(DC,pt);
      }
    }
    while (YAxis->NextEint() && YText.NextEint());
  }
  DC.SetFont(wxNullFont);
  delete HTitel;
  delete HAchsLabels;
  delete HAchse;
  DC.SetTextForeground(Farbe);
}

void DiagObj::GrInit(BildType &BildB,AxisDataArr &XA,AxisDataArr &YA,
                     const FontType &A,const FontType &T,const FontType &AL,
                     const unsigned long &col,
                     const NumListFormat &iLDV,
                     int M,int ND,wxDC &DC)
{
  SetLDV(iLDV);
  m_Colour = col;
  Achse = A;
  Titel = T;
  AchsLabels = AL;
  XAxis = &XA;
  YAxis = &YA;
  Mode = M;
  XAxis->Begin();
  do
  {
    (*XAxis)->TextL = 0;
    (*XAxis)->TextH = 0;
  }
  while (XAxis->NextEint());
  bool OppositeAvailable = false;
  YAxis->Begin();
  do
  {
    (*YAxis)->TextL = 0;
    (*YAxis)->TextH = 0;
    if ((*YAxis)->GetOppositeSide() && (*YAxis)->GetVisible())
      OppositeAvailable = true;
  }
  while (YAxis->NextEint());
  Linien(DC,false); // Schrift-Abmessungen berechnen
  DiagBer = BildB;
  // Raender fuer Beschriftung vorsehen
  XAxis->Begin();
  XText.Begin();
  BildB.YMax -= int((*XAxis)->TextH*1.2);
  if (XText->GetText().Length() > 0)
    BildB.YMax -= int(XText->GetHeight()*1.2);
  BildB.YMin += int(TText.GetHeight()*1.5);
  if (!OppositeAvailable)
    BildB.XMax -= int((*XAxis)->TextL*1.2/2)+5;
  YAxis->Begin();
  YText.Begin();
  BildB.XMin = 0;
  do
  {
    xposarr[YAxis->AkEintNummer()] = GetXTextLen();
    if ((*YAxis)->GetVisible())
    {
      if ((*YAxis)->GetOppositeSide())
        BildB.XMax -= GetXTextLen();
      else
        BildB.XMin += GetXTextLen();
    }
  }
  while (YAxis->NextEint() && YText.NextEint());
  UpdateBildBer(BildB);
  NumDigits = ND;
}

int DiagObj::GetXTextLen()
{
  int len = 0;
  if ((*YAxis)->GetVisible())
  {
    len += int((*YAxis)->TextL*1.2);
    if (YText->GetText().Length() > 0)
      len += int(YText->GetHeight()*1.2);
  }
  return len;
}

DiagObj::DiagObj()
{
  Mode = 0;
  NumDigits = 3;
  Points = 0;
}

DiagObj::~DiagObj()
{
  DeletePoints();
}

void DiagObj::ResetCLineTo(double X,double Y,bool spl)
{
  AkX = XPos(X);
  AkY = YPos(Y);
  InBer = false;
  DeletePoints();
  splines = spl;
  if (splines)
#if wxCHECK_VERSION(2,9,0)
    Points = new wxPointList;
#else
    Points = new wxList;
#endif
}

void DiagObj::DeletePoints()
{    
  if (Points)
  {
#if wxCHECK_VERSION(2,9,0)
    wxwxPointListNode *first = Points->GetFirst();
#else
    wxNode *first = Points->GetFirst();
#endif
    while (first)
    {
      delete (wxPoint *)first->GetData();
      delete first;
      first = Points->GetFirst();
    }
    delete Points;
  }
  Points = 0;
}

void DiagObj::FlushCLineTo(wxDC &DC)
{
  if (splines)
    DrawLines(DC);
}

void DiagObj::DrawLines(wxDC &DC)
{
  if (!splines || !Points)
    return;
  if (Points->GetCount() > 2)
    DC.DrawSpline(Points);
  else
    DC.DrawLines(Points);
  DeletePoints();
}

void DiagObj::CLineTo(wxDC &DC,double X,double Y)
{
  double BildX,BildY;

  BildX = XPos(X);
  BildY = YPos(Y);
  if (InBer && (AkX == BildX) && (AkY == BildY))
    return;
  bool AkInBer = ((BildX <= BildBer.XMax) && (BildX >= BildBer.XMin) &&
                  (BildY <= BildBer.YMax) && (BildY >= BildBer.YMin));
  if (splines && (AkInBer && !InBer))
  {
    DrawLines(DC);
#if wxCHECK_VERSION(2,9,0)
    Points = new wxPointList;
#else
    Points = new wxList;
#endif
  }
  if (AkInBer && InBer)
  { // Voriger Punkt und aktueller Punkt im Diagramm
    if (splines)
#if wxCHECK_VERSION(2,9,0)
      Points->Append(new wxPoint(int(BildX),int(BildY)));
#else
      Points->Append((wxObject *)new wxPoint(int(BildX),int(BildY)));
#endif
    else
      DC.DrawLine(int(AkX),int(AkY),int(BildX),int(BildY));
    AkX = BildX;
    AkY = BildY;
    InBer = true;
  }
  else
  if (!AkInBer && !InBer)
  { // weder voriger noch aktueller Punkt im Diagramm
    AkX = BildX;
    AkY = BildY;
    InBer = false;
    return;
  }
  else
  { // 1 Punkt ausserhalb, 1 Punkt innerhalb
    double NewX,NewY;
    ClipPoint(NewX,NewY,BildX,BildY,AkX,AkY,BildBer,InBer);
    if (!InBer)
    {
      if (splines)
      {
#if wxCHECK_VERSION(2,9,0)
        Points->Append(new wxPoint(int(NewX),int(NewY)));
        Points->Append(new wxPoint(int(BildX),int(BildY)));
#else
        Points->Append((wxObject *)new wxPoint(int(NewX),int(NewY)));
        Points->Append((wxObject *)new wxPoint(int(BildX),int(BildY)));
#endif
      }
      else
        DC.DrawLine(int(NewX),int(NewY),int(BildX),int(BildY));
      AkX = BildX;
      AkY = BildY;
      InBer = true;
    }
    else
    {
      if (splines)
#if wxCHECK_VERSION(2,9,0)
        Points->Append(new wxPoint(int(NewX),int(NewY)));
#else
        Points->Append((wxObject *)new wxPoint(int(NewX),int(NewY)));
#endif
      else
        DC.DrawLine(int(AkX),int(AkY),int(NewX),int(NewY));
      AkX = XPos(X);
      AkY = YPos(Y);
      InBer = false;
    }
  }
  if (splines && (Points->GetCount() > 1000))
  {
    DrawLines(DC);
#if wxCHECK_VERSION(2,9,0)
    Points = new wxPointList;
#else
    Points = new wxList;
#endif
  }
}

wxFont *DiagObj::GetFont(wxDC &/*dc*/,FontType Font)
{
  return new wxFont(int(FontFakt*Font.Size > 1 ? FontFakt*Font.Size : 1),
                    wxDEFAULT,
                    Font.Style & FontType::Italic ? wxITALIC : wxNORMAL,
                    Font.Style & FontType::Bold ? wxBOLD : wxNORMAL,
                    Font.Style & FontType::Underline,Font.Font.c_str());
}

//! Get y axis number under given point
int DiagObj::GetYAxisNumber(int X,int Y)
{
  int akposleft = 0,akposright = 0;
  YText.Begin();
  YAxis->Begin();
  do
  {
    if ((*YAxis)->GetVisible())
    {
      wxRect Rect;
      if ((*YAxis)->GetOppositeSide())
      {
        Rect = wxRect(wxPoint(BildBer.XMax+akposright,BildBer.YMin),
                      wxSize(xposarr[YText.AkEintNummer()],BildBer.YMax-BildBer.YMin));
        akposright += xposarr[YText.AkEintNummer()];
      }
      else
      {
        akposleft += xposarr[YText.AkEintNummer()];
        Rect = wxRect(wxPoint(BildBer.XMin-akposleft,BildBer.YMin),
                      wxSize(xposarr[YText.AkEintNummer()],BildBer.YMax-BildBer.YMin));
      }
      if ((X >= Rect.GetLeft()) && (X <= Rect.GetRight()) &&
          (Y >= Rect.GetTop()) && (Y <= Rect.GetBottom()))
        return YText.AkEintNummer();
    }    
  }
  while (YText.NextEint() && YAxis->NextEint());
  return 0;
}

/*int DiagObj::GetYDeltaForSameXYScaling()
{
  YAxis->Begin();
  XAxis->Begin();
  if (!(*YAxis)->GetLog() && !(*XAxis)->GetLog())
  {
    double yber = (*YAxis)->Max-(*YAxis)->Min;
    double xber = (*XAxis)->Max-(*XAxis)->Min;
    if ((0.1*yber > xber) || (yber < 0.1*xber))
      return 0;
    else
    {
      int ydelta = int(yber/xber*(BildBer.XMax-BildBer.XMin))-(BildBer.YMax-BildBer.YMin);
      return ydelta;
    }
  }
  else
    return 0;
}*/

int DiagObj::GetXDeltaForSameXYScaling()
{
  YAxis->Begin();
  XAxis->Begin();
  if (!(*YAxis)->GetLog() && !(*XAxis)->GetLog())
  {
    double yber = (*YAxis)->Max-(*YAxis)->Min;
    double xber = (*XAxis)->Max-(*XAxis)->Min;
    if ((0.1*yber > xber) || (yber < 0.1*xber))
      return 0;
    else
    {
      int xdelta = int(xber/yber*(BildBer.YMax-BildBer.YMin))-(BildBer.XMax-BildBer.XMin);
      return xdelta;
    }
  }
  else
    return 0;
}

// ----------------------------------------------------------------------------
// MarkObj
// ----------------------------------------------------------------------------

void MarkObj::Init(BildType BildB,AxisDataArr &XA,AxisDataArr &YA,long Frb)
{
  GrInit(BildB,XA,YA);
  Farbe = Frb;
  XMarke = false;
  YMarke = false;
  visible = true;
}

void MarkObj::SetVisible(bool vis,wxDC &DC)
{
  if (vis != visible)
  {
    Draw(DC,true);
    visible = vis;
  }
}

void MarkObj::UpdateBildBer(BildType BB)
{
  SkalObj::UpdateBildBer(BB);
  XMarke = false;
  YMarke = false;
}

void MarkObj::MarkLinieX(wxDC &DC,long MarkX,int Min,int Max)
{
  int f = MarkLaenge;

  for (int i = Max;i > Min;i -= MarkAbst)
  {
    if (i-f < Min)
      f = i-Min;
    DC.DrawLine((int)MarkX,i,(int)MarkX,i-f);
  }
}

void MarkObj::MarkLinieY(wxDC &DC,long MarkY,int Min,int Max)
{
  int f = MarkLaenge;

  for (int i = Min;i < Max;i += MarkAbst)
  {
    if (i+f > Max)
      f = Max-i;
    DC.DrawLine(i,(int)MarkY,i+f,(int)MarkY);
  }
}

void MarkObj::Draw(wxDC &DC,bool force)
{
  if (visible || force)
  {
    wxPen Pen(Farbe,MarkLinie,wxSOLID);
    DC.SetPen(Pen);
    DC.SetLogicalFunction(wxEQUIV);
    if (XMarke)
      MarkLinieX(DC,MarkPunkt.X,BildBer.YMin,BildBer.YMax+2*TickLen);
    if (YMarke)
      MarkLinieY(DC,MarkPunkt.Y,BildBer.XMin-2*TickLen,BildBer.XMax);
    DC.SetLogicalFunction(wxCOPY);
    DC.SetPen(wxNullPen);
  }
}

void MarkObj::UnMark(wxDC &DC)
{
  Draw(DC);
  XMarke = false;
  YMarke = false;
}

void MarkObj::Mark(long X, long Y,wxDC &DC)
{
  Draw(DC);
  MarkPunkt.X = X;
  MarkPunkt.Y = Y;
  XMarke = ((X >= BildBer.XMin) && (X <= BildBer.XMax));
  YMarke = ((Y >= BildBer.YMin) && (Y <= BildBer.YMax));
  Draw(DC);
}

bool MarkObj::Markiert()
{
  return(XMarke || YMarke);
}

MarkObj::~MarkObj()
{
}

// ----------------------------------------------------------------------------
// GraphBaseObj
// ----------------------------------------------------------------------------

bool GraphBaseObj::ClipPoint(double &NewX,double &NewY,
                             const double BildX,const double BildY,
                             const double AkX,const double AkY,
                             const GraphBaseObj::BildType BildBer,
                             const bool InBer)
{
  if (!InBer)
  {
    NewX = AkX;
    NewY = AkY;
  }
  else
  {
    NewX = BildX;
    NewY = BildY;
  }
  bool NextInBer = (BildX >= BildBer.XMin) && (BildX <= BildBer.XMax) &&
                   (BildY >= BildBer.YMin) && (BildY <= BildBer.YMax);
  if (InBer != NextInBer)
  {
    double BerX,BerY;
    double A,B;
    double C[4];
    bool Schnitt[4];
    BerX = BildX-AkX;
    BerY = BildY-AkY;
    if (BerY == 0)
      A = 0;
    else
    if (fabs(BerX)*1e10 < fabs(BerY))
    { // Steigung bestimmen
      if (((BerX > 0) && (BerY < 0)) || ((BerX < 0) && (BerY > 0)))
        A = -1e10;
      else
        A = 1e10;
    }
    else
      A = BerY/BerX; // Steigung der Geraden durch AkX,AkY und BildX,BildY
    B = BildY - A*BildX; // Y-Wert fuer Schnittpunkt bei 0
    if ((fabs(A) < 1e-10) || (fabs(A)*1e6 < fabs(BildBer.YMax-B)))
      Schnitt[0] = false;
    else
    { // Schnitt unten
      // Bestimmung des X-Schnittpunkts mit unterer Diagrammgrenze
      C[0] = (BildBer.YMax-B)/A;
      if ((C[0] <= BildBer.XMax) && (C[0] >= BildBer.XMin))
        Schnitt[0] = true;
      else
        Schnitt[0] = false;
    }
    if ((fabs(A) < 1e-10) || (fabs(A)*1e6 < fabs(BildBer.YMin-B)))
      Schnitt[1] = false;
    else
    { // Schnitt oben
      // Bestimmung des X-Schnittpunkts mit oberer Diagrammgrenze
      C[1] = (BildBer.YMin-B)/A;
      if ((C[1] <= BildBer.XMax) && (C[1] >= BildBer.XMin))
        Schnitt[1] = true;
      else Schnitt[1] = false;
    }
    // Bestimmung des Y-Schnittpunkts mit linker Diagrammgrenze
    C[2] = A * BildBer.XMin + B;
    if ((C[2] <= BildBer.YMax) && (C[2] >= BildBer.YMin))
      Schnitt[2] = true; // Schnitt links
    else
      Schnitt[2] = false;
    // Bestimmung des Y-Schnittpunkts mit rechter Diagrammgrenze
    C[3] = A * BildBer.XMax + B;
    if ((C[3] <= BildBer.YMax) && (C[3] >= BildBer.YMin))
      Schnitt[3] = true; // Schnitt rechts
    else
      Schnitt[3] = false;
    if (((AkY > BildY) && !InBer || (AkY < BildY) && InBer) && Schnitt[0])
    {
      NewY = BildBer.YMax;
      NewX = C[0];
    }
    else
    if (((AkY < BildY) && !InBer || (AkY > BildY) && InBer) && Schnitt[1])
    {
      NewY = BildBer.YMin;
      NewX = C[1];
    }
    else
    if (((AkX < BildX) && !InBer || (AkX > BildX) && InBer) && Schnitt[2])
    {
      NewX = BildBer.XMin;
      NewY = C[2];
    }
    else
    if (((AkX > BildX) && !InBer || (AkX < BildX) && InBer) && Schnitt[3])
    {
      NewX = BildBer.XMax;
      NewY = C[3];
    }
  }
  return NextInBer;
}

