// Version 6.07  21.01.2011 Markus Greither
// Copyright (c) 1991-2011 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file ef.cpp
    \brief Source code for the application main class and top level windows
*/

#include "efprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#include <wx/xrc/xmlres.h> // must be before any templates for DMC (?!)

#ifndef WX_PRECOMP
#ifdef __DMC__
#include <typeinfo.h>
#endif
#include <wx/wx.h>

#include <wx/docview.h>
#include <wx/docmdi.h>
#include <wx/clipbrd.h>
#include <wx/config.h>
#include <wx/utils.h>
#include <wx/image.h>
#include <wx/spinctrl.h>
#include <wx/resizec.h>

#include <complex>
#include <limits>
#include <climits>
#include <clocale>
#include <set>
#include <functional>
#if defined(__MINGW32__)
#include <wx/msw/winundef.h>
#endif

using namespace std;

#include "efdefs.h"
#include "mg.h"
#include "cpxpsr.h"
#include "efresid.h"
#include "diagramm.h"
#include "gauss.h"
#include "efgemein.h"
#include "efsolver.h"
#include "efedit.h"
#include "efdocum.h"
#include "ef.h"
#include "eftabcon.h"
#include "efgraphc.h"
#include "efview.h"
#include "effnkclc.h"
#include "efdlgs.h"
#include "efedplvw.h"

#endif

#include <signal.h>
#include <float.h>

#include <wx/xrc/xh_toolb.h>
#include <wx/xrc/xh_combo.h>

#include "efreg.h"

#ifdef __WXGTK__
#include "xpm/app_icon_ef.xpm"
#include "xpm/doc_icon_ef.xpm"
#endif

#ifdef __WXMSW__
#include <windows.h>
#include <wx/msw/winundef.h>
#endif

#include "xpm/ef_splash.xpm"

#include <wx/dnd.h>

// ----------------------------------------------------------------------------
// EFFileDropTarget
// ----------------------------------------------------------------------------

//! DnD support for MCX-Files
class EFFileDropTarget : public wxFileDropTarget
{
 public:
  //! Ctor
  EFFileDropTarget() : wxFileDropTarget() {};
  //! Respond to dropped files and open them
  bool OnDropFiles(wxCoord x,wxCoord y,const wxArrayString &files);
};

bool EFFileDropTarget::OnDropFiles(wxCoord /*x*/,wxCoord /*y*/,const wxArrayString &files)
{
  for (unsigned int i = 0;i < files.GetCount();i++)
  {
    if (wxGetApp().m_docManager->FindTemplateForPath(files[i]))
      wxGetApp().m_docManager->CreateDocument(files[i],wxDOC_SILENT);
  }
  return true;
}

// ----------------------------------------------------------------------------
// EFZoomBox
// ----------------------------------------------------------------------------

wxString EFZoomBox::Zooms[EFZoomBox::AnzZooms] =
{
  wxT("200%"),
  wxT("150%"),
  wxT("120%"),
  wxT("100%"),
  wxT("75%"),
  wxT("50%"),
  wxT("25%")
};

void EFZoomBox::OnKeyDown(wxKeyEvent &event)
{
  if (event.GetKeyCode() == WXK_RETURN)
  {
    wxCommandEvent pevent(wxEVT_COMMAND_ENTER,GetId());
    m_parent->GetEventHandler()->AddPendingEvent(pevent);
  }
  else
    event.Skip();
}

void EFZoomBox::OnChar(wxKeyEvent &event)
{
  if (event.GetKeyCode() != WXK_RETURN)
    event.Skip();
}

BEGIN_EVENT_TABLE(EFZoomBox,wxComboBox)
  EVT_KEY_DOWN(EFZoomBox::OnKeyDown)
  EVT_CHAR(EFZoomBox::OnChar)
END_EVENT_TABLE()

//! Initialize XML resources (XRC GUI Resources)
void InitXmlResource();

IMPLEMENT_APP(MyApp)

// ----------------------------------------------------------------------------
// MyApp
// ----------------------------------------------------------------------------

MyApp::MyApp() : wxApp()
{
  Locale = 0;
  OpenWindow = 0;
  SplashBitmap = 0;
  m_docManager = 0;
  sb = 0;
  zoombox = 0;
  tb = 0;
  tb2 = 0;
}

bool MyApp::GetZoomIn()
{
  if (wxGetApp().tb2)
    return tb2->GetToolState(XRCID("CMD_ZOOMIN"));
  else
    return false;
}

bool MyApp::OnInit()
{
  if (argc > 1)
  {
    if (wxStrcmp(argv[argc > 0 ? argc-1 : 0],wxT("/reg")) == 0)
    { // Register program
#ifdef __WXMSW__
      wxString str(argv[0]);
      EFRegister::SetRegValues(str.c_str(),false);
#endif
      return TRUE;
    }
    else
    if (wxStrcmp(argv[argc > 0 ? argc-1 : 0],wxT("/regd")) == 0)
    { // Register program, debug version
#ifdef __WXMSW__
      wxString str(argv[0]);
      EFRegister::SetRegValues(str.c_str(),true);
#endif
      return TRUE;
    }
    else
    if (wxStrcmp(argv[argc > 0 ? argc-1 : 0],wxT("/unreg")) == 0)
    { // Remove registration info
#ifdef __WXMSW__
      EFRegister::DelRegValues();
#endif
//      Debug Code for cleaning the program settings
//      wxConfig *config = new wxConfig(wxT("Easy Funktion 6"));
//      config->DeleteAll();
//      delete config;
      return TRUE;
    }
  }
#ifndef __WXDEBUG__
  SplashBitmap = new wxBitmap(ef_splash);
  wxMemoryDC *mem_dc = new wxMemoryDC;
  mem_dc->SelectObject(*SplashBitmap);
  mem_dc->SetFont(*wxSMALL_FONT);
//  mem_dc->SetFont(wxSystemSettings::GetFont(wxSYS_ANSI_VAR_FONT));
  mem_dc->DrawText(wxT(COPYRIGHTSTRING),10,160);
  mem_dc->SelectObject(wxNullBitmap);
  delete mem_dc;

  OpenWindow = new wxSplashScreen(*SplashBitmap,wxSPLASH_CENTRE_ON_SCREEN,0,
                                  0,-1,wxDefaultPosition,wxDefaultSize,
                                  wxSIMPLE_BORDER | wxSTAY_ON_TOP);
#endif
  diagformat.SetId(TEFGraphControl::DiagClipFormat);
  graphformat.SetId(TEFGraphControl::GraphClipFormat);
  editformat.SetId(TEFEdit::EditClipFormat);

  wxImage::AddHandler(new wxGIFHandler);
  wxImage::AddHandler(new wxXPMHandler);
#ifdef __WXGTK__
  wxImage::AddHandler(new wxPNGHandler); // required for GTK clipboard
#endif
  EFXmlResource::Get()->InitAllHandlers();

#ifdef __WXMSW__
  wxFileName name(argv[0]);
  name.MakeAbsolute();
  m_datadir = name.GetPath();
#else
  m_datadir = wxT("/usr/local/share/easyfunktion");
#endif

  InitXmlResource();

  wxString orgpath = m_datadir +wxT("/de");

  if (wxDirExists(orgpath))
  { // Locale-path found
    orgpath = wxGetCwd();
    wxSetWorkingDirectory(m_datadir); // Path for locale

    int lang = wxLocale::GetSystemLanguage();

#if defined(__BORLANDC__)
    // Problems of unknown nature with BCC and locale != "C"
    // localeconv() liest falsches Dezimaltrennzeichen ?!
    if (lang == wxLANGUAGE_GERMAN)
      Locale = new wxLocale(wxT("Deutsch"),wxT("de_DE"),wxT("C"));
    else
      Locale = new wxLocale(wxT("English"),wxT("en_EN"),wxT("C"));
    // Works with build without DLL (?!)
//    _tsetlocale(LC_ALL,wxT("German_Germany"));
#else
    // Define default locale on LINUX, as some sytems have problems with
    // using the same decimal separators for sprintf and localeconv->decimal_separator (!?)
    Locale = new wxLocale(wxLANGUAGE_DEFAULT);
#endif
    Locale->AddCatalogLookupPathPrefix(m_datadir);
    Locale->AddCatalog(wxT("de"));
    Locale->AddCatalog(wxT("efde"));
    wxSetWorkingDirectory(orgpath); // Reset path

    if (lang == wxLANGUAGE_GERMAN)
      m_datadir += wxT("/de");
    else
      m_datadir += wxT("/en");
  }
  else
#if defined(__WXMSW__)
    Locale = new wxLocale(wxT("English"),wxT("en_EN"),wxT("C"));
#else
    // Define default locale on LINUX, as some sytems have problems with
    // using the same decimal separators for sprintf and localeconv->decimal_separator (!?)
    Locale = new wxLocale(wxLANGUAGE_DEFAULT);
#endif

  GetLocaleLDV();
  //// Create a document manager
  m_docManager = new EFDocManager(wxDOC_MDI);

  //// Create a template relating drawing documents to their views
  new wxDocTemplate(m_docManager, _("Easy Funktion File"), wxT("*.mcx"),
                    wxT(""), wxT("mcx"), wxT(""), wxT(""),
                    CLASSINFO(TEFDocument), CLASSINFO(TEFView));

  // INI-Values erst nach Locale laden, damit Pfad richtig gesetzt wird
  GetIniValues();

  //// Create the main frame window
  MyFrame *frame = new MyFrame(m_docManager, 0, -1, wxT("Easy Funktion"),
                               IniRect.GetPosition(),IniRect.GetSize(),
                               wxDEFAULT_FRAME_STYLE | wxHSCROLL | wxVSCROLL |
                               wxCLIP_CHILDREN);

  //// Make a menubar
  frame->SetMenuBar(CreateFrameMenu());

  sb = frame->CreateStatusBar(3);

  // Give it an icon
  frame->SetIcon(wxICON(app_icon_ef));

  Zoom = 100;

#if !wxCHECK_VERSION(2,8,0)
  frame->SetToolBar(tb);
#endif
  zoombox = ((wxComboBox *)tb->FindWindow(XRCID("ID_ZOOM")));
  frame->SetWindowMenu(0);

#ifdef __WXMSW__
  if (Options & MyApp::keyMaximized)
    frame->Maximize();
#endif
  SetTopWindow(frame);

  frame->Show(true);

  if ((argc > 1) && wxStrcmp(argv[1],wxT("/t")))
  {
    wxString Pfad(argv[1]);
    if (!wxStrstr(Pfad,wxT(".")))
      Pfad += wxT(".mcx");
    if (wxGetApp().m_docManager->FindTemplateForPath(Pfad))
      wxGetApp().m_docManager->CreateDocument(Pfad,wxDOC_SILENT);
  }
  else
  {
    wxGetApp().m_docManager->CreateDocument(wxT(""),wxDOC_NEW);
    if ((argc > 1) && !wxStrcmp(argv[1],wxT("/t")))
    {
      wxView *view = wxGetApp().m_docManager->GetCurrentView();
      if (view)
      {
        wxCommandEvent event(wxEVT_COMMAND_MENU_SELECTED,XRCID("CMD_EINZWERT"));
        view->AddPendingEvent(event);
      }
    }
  }
  delete OpenWindow;
  delete SplashBitmap;

  return true;
}

int MyApp::OnExit()
{
  if (m_docManager)
  {
    // nur eintragen wenn Programm auch lief (nicht bei /reg, /unreg)
    SetIniValues();
    delete m_docManager;
  }
  delete Locale;
  return wxApp::OnExit();
}

int MyApp::OnRun()
{
  int ret = 0;
  if (tb) // only run program if not /reg, /unreg parameters given
  {
#if defined(__WXDEBUG__) && defined(__BORLANDC__)
    try
    {
#endif
      ret = wxApp::OnRun();
#if defined(__WXDEBUG__) && defined(__BORLANDC__)
    }
    catch (FastString<_TCHAR> error)
    {
      wxMessageBox(error.c_str());
    }
#endif
  }
  return ret;
}

EFMDIChildFrame *MyApp::CreateChildFrame(wxDocument *doc, wxView *view)
{
    //// Make a child frame
    int style = wxDEFAULT_FRAME_STYLE | wxCLIP_CHILDREN;

#if !wxCHECK_VERSION(2,6,0) && defined(__WXMSW__)
    style |= ((Options & MyApp::keyMaximChild) ? wxMAXIMIZE : 0);
#endif

    wxString tmp = doc->GetUserReadableName();
    EFMDIChildFrame *subframe =
      new EFMDIChildFrame(doc, view,(MyFrame *)wxGetApp().GetTopWindow(), -1,
                          tmp,wxDefaultPosition,wxDefaultSize,style);
    //// Make a menubar

    subframe->SetIcon(wxICON(doc_icon_ef));

    //// Associate the menu bar with the frame
    subframe->SetMenuBar(CreateViewMenu());

#if wxCHECK_VERSION(2,6,0)
    if (Options & MyApp::keyMaximChild)
      subframe->Maximize();
#endif

    subframe->Show(true);

    return subframe;
}

wxMenu *MyApp::DateiMenu(bool /*MainMenu*/)
{
  wxMenu *file_menu = EFXmlResource::Get()->LoadMenu(wxT("menuFile"));

  // A nice touch: a history of files visited. Use this menu.
  m_docManager->FileHistoryUseMenu(file_menu);
  m_docManager->FileHistoryAddFilesToMenu(file_menu);
  return file_menu;
}

wxMenu *MyApp::HilfeMenu()
{
  wxMenu *help_menu = EFXmlResource::Get()->LoadMenu(wxT("menuHelp"));

  return help_menu;
}

wxMenuBar *MyApp::CreateViewMenu()
{
  wxMenu *edit_menu = EFXmlResource::Get()->LoadMenu(wxT("menuEdit"));
  wxMenu *view_menu = EFXmlResource::Get()->LoadMenu(wxT("menuView"));
  wxMenu *insert_menu = EFXmlResource::Get()->LoadMenu(wxT("menuInsert"));
  wxMenu *extras_menu = EFXmlResource::Get()->LoadMenu(wxT("menuExtras"));
  wxMenu *calc_menu = CreateCalcMenu(false);
  wxMenu *diag_menu = CreateDiagramMenu(false);
  wxMenuBar *menu_bar = new wxMenuBar;

  menu_bar->Append(DateiMenu(false), _("&File"));
  menu_bar->Append(edit_menu, _("&Edit"));
  menu_bar->Append(view_menu, _("&View"));
  menu_bar->Append(insert_menu, _("&Insert"));
  menu_bar->Append(calc_menu, _("&Calculate"));
  menu_bar->Append(diag_menu, _("Dia&gram"));
  menu_bar->Append(extras_menu, _("E&xtras"));
  menu_bar->Append(HilfeMenu(), _("&Help"));

  return menu_bar;
}

void MyApp::InsertClipboardCommands(wxMenu *menu,bool selectall)
{
    wxMenu *edit_menu = EFXmlResource::Get()->LoadMenu(wxT("menuEdit"));
    wxMenuItemList &lst = edit_menu->GetMenuItems();
    menu->InsertSeparator(0);
    for (int i = (selectall ? 7 : 6);i > 2;i--)
    {
#ifdef __WXMSW__
      wxString label = lst[i]->GetItemLabel();
#else
      wxString label = lst[i]->GetLabel();
#endif
      wxMenuItem *item;
      item = new wxMenuItem(menu,lst[i]->GetId(),
                            label,
                            lst[i]->GetHelp());
//    item->SetBitmap(lst[i]->GetBitmap()); // Creates problems under UNICODE (?!)
      menu->Insert(0,item);
    }

    delete edit_menu;
}

wxMenu *MyApp::CreateCalcMenu(bool popup)
{
  wxMenu *calc_menu = EFXmlResource::Get()->LoadMenu(wxT("menuCalc"));

  if (popup)
    InsertClipboardCommands(calc_menu,true);

  return calc_menu;
}

wxMenu *MyApp::CreateDiagramMenu(bool popup)
{
  wxMenu *diag_menu = EFXmlResource::Get()->LoadMenu(wxT("menuDiagram"));

  if (popup)
    InsertClipboardCommands(diag_menu);

  return diag_menu;
}

wxMenuBar *MyApp::CreateFrameMenu()
{
  wxMenuBar *menu_bar = new wxMenuBar;

  menu_bar->Append(DateiMenu(true), _("&File"));
  wxMenu *extras_menu = EFXmlResource::Get()->LoadMenu(wxT("menuExtras"));
  menu_bar->Append(extras_menu, _("E&xtras"));
  menu_bar->Append(HilfeMenu(), _("&Help"));

  return menu_bar;
}

void MyApp::GetLocaleLDV()
{
  lconv *conv = localeconv();
#ifdef __WXMSW__
  wxChar buff[51];
  ::GetLocaleInfo(LOCALE_USER_DEFAULT,LOCALE_SLIST,buff,50);
  LDV.ListSeparator = buff;
  ::GetLocaleInfo(LOCALE_USER_DEFAULT,LOCALE_SDECIMAL,buff,50);
  LDV.DecSeparator = buff;
  ::GetLocaleInfo(LOCALE_USER_DEFAULT,LOCALE_ILZERO,buff,50);
  LDV.LeadZero = buff[0];
#else
  if (conv->decimal_point[0] != ',')
    LDV.ListSeparator = _TEXT(",");
  else
    LDV.ListSeparator = _TEXT(";");
  MGPStr tmp;
  tmp.AddChar(conv->decimal_point[0]);
  LDV.DecSeparator = tmp;
  LDV.LeadZero = 2;
#endif
  ZahlFormatObj::SetLocDecSeparator(ConvertStr(conv->decimal_point).c_str());
}

void MyApp::GetIniValues(bool getdefaults)
{
  wxConfig *config;
  if (getdefaults)
    // use a file name that doesn't exist, so we get the defaults
    config = new wxConfig(wxT("Easy Funktion 6 defaults"));
  else
    config = new wxConfig(wxT("Easy Funktion 6"));
  config->Read(wxT("/Formats/Numberformat"),&ZahlForm,0);
  config->Read(wxT("/Formats/Winseparators"),&WinZahlFormat,1);
  config->Read(wxT("/Formats/Options"),&Options,MyApp::keyLowerCase+
                                                MyApp::keyMaximChild+
                                                MyApp::keyMaximized+
                                                MyApp::keyWarnSolve+
                                                MyApp::keyWarnDelete+
                                                MyApp::keyFuncSelVisible);
  wxString tp;
  config->Read(wxT("/Formats/FilePath"),&tp,wxT(""));
  SetStdFilePath(tp);
  config->Read(wxT("/Formats/LabelFontStyle"),&tp,wxT("2"));
  LFontStyle = tp[0u];
  config->Read(wxT("/Formats/CommentFontStyle"),&tp,wxT("0"));
  KFontStyle = tp[0u];
  long lg;
  config->Read(wxT("/Formats/LabelFontColour"),&lg,0xC00000);
  LFontColour = lg;
  config->Read(wxT("/Formats/CommentFontColour"),&lg,0x007F00);
  KFontColour = lg;
  config->Read(wxT("/Formats/DiagColour"),&lg,0xFFFFFF);
  DiagColour = lg;
  config->Read(wxT("/Formats/GraphDarst"),&lg,GraphDataObj::gPunkteVerbinden);
  GraphDarst = lg;
  config->Read(wxT("/Formats/GraphOpt"),&lg,GraphDataObj::GraphFett);
  GraphOpt = lg;
  config->Read(wxT("/Formats/GraphSymb"),&lg,GraphDataObj::GraphMitSymbol);
  GraphSymb = lg;
  config->Read(wxT("/Formats/DiagFlags"),&lg,DiagInNewWorksheet |
                                             DiagHasFrame |
                                             DiagEndLabels);
  DiagFlags = lg;
  config->Read(wxT("/Formats/EditFlags"),&lg,EditScrollTrack);
  EditFlags = lg;
  config->Read(wxT("/Graph-File/Write-Format"),&GraphWriteForm,0);
  config->Read(wxT("/Window/XMin"),&lg,0);
  IniRect.x = lg;
  config->Read(wxT("/Window/YMin"),&lg,0);
  IniRect.y = lg;
  config->Read(wxT("/Window/XMax"),&lg,639);
  IniRect.width = lg-IniRect.x;
  config->Read(wxT("/Window/YMax"),&lg,479);
  IniRect.height = lg-IniRect.y;
  config->Read(wxT("/Diagram/UndoCount"),&m_UndoCount,20);
  config->Read(wxT("/Solver/DbgOut"),&SolverDbgOut,0);

  // Graph loading format defaults
  config->Read(wxT("/Graph-File-Load/Mode"),&lg,0);
  m_graphdata.Mode = lg;
  config->Read(wxT("/Graph-File-Load/DecSeparator"),&tp,GetLDV().DecSeparator.c_str());
  m_graphdata.DecSeparator.SetStr(tp.c_str());
  config->Read(wxT("/Graph-File-Load/ListSeparator"),&tp,GetLDV().ListSeparator.c_str());
  m_graphdata.ListSeparator.SetStr(tp.c_str());
  config->Read(wxT("/Graph-File-Load/TabSeparator"),&m_graphdata.TabSeparator,true);
  config->Read(wxT("/Graph-File-Load/SpaceSeparator"),&m_graphdata.SpaceSeparator,true);
  config->Read(wxT("/Graph-File-Load/ImaginaryDataIncluded"),&m_graphdata.ImaginaryDataIncluded,true);
  config->Read(wxT("/Graph-File-Load/HeaderIncluded"),&m_graphdata.HeaderIncluded,false);
  config->Read(wxT("/Graph-File-Load/XValuesIncluded"),&m_graphdata.XValuesIncluded,true);
  config->Read(wxT("/Graph-File-Load/XValuesColumn"),&m_graphdata.XValuesColumn,1);

  m_docManager->GetFileHistory()->Load(*config);
  delete config;
}

void MyApp::SetIniValues()
{
  wxConfig *config = new wxConfig(wxT("Easy Funktion 6"));
  config->Write(wxT("/Formats/Numberformat"),ZahlForm);
  config->Write(wxT("/Formats/Winseparators"),WinZahlFormat);
  Options |= MyApp::keyIniVorhanden; // Indikator f. Ini-Datei vorhanden
  config->Write(wxT("/Formats/Options"),Options);
  wxString tmp(StdFilePath);
  config->Write(wxT("/Formats/FilePath"),tmp);
  config->Write(wxT("/Formats/LabelFontColour"),(long)LFontColour);
  config->Write(wxT("/Formats/CommentFontColour"),(long)KFontColour);
  config->Write(wxT("/Formats/DiagColour"),(long)DiagColour);
  tmp = wxChar(LFontStyle);
  config->Write(wxT("/Formats/LabelFontStyle"),tmp);
  tmp = wxChar(KFontStyle);
  config->Write(wxT("/Formats/CommentFontStyle"),tmp);
  config->Write(wxT("/Formats/GraphDarst"),(long)GraphDarst);
  config->Write(wxT("/Formats/GraphOpt"),(long)GraphOpt);
  config->Write(wxT("/Formats/GraphSymb"),(long)GraphSymb);
  config->Write(wxT("/Formats/DiagFlags"),(long)DiagFlags);
  config->Write(wxT("/Formats/EditFlags"),(long)EditFlags);
  config->Write(wxT("/Graph-File/Write-Format"),GraphWriteForm);
  config->Write(wxT("/Window/XMin"),long(IniRect.x));
  config->Write(wxT("/Window/YMin"),long(IniRect.y));
  config->Write(wxT("/Window/XMax"),long(IniRect.x+IniRect.width));
  config->Write(wxT("/Window/YMax"),long(IniRect.y+IniRect.height));
  config->Write(wxT("/Diagram/UndoCount"),m_UndoCount);

  // Graph loading format defaults
  config->Write(wxT("/Graph-File-Load/Mode"),m_graphdata.Mode);
  config->Write(wxT("/Graph-File-Load/DecSeparator"),m_graphdata.DecSeparator.c_str());
  config->Write(wxT("/Graph-File-Load/ListSeparator"),m_graphdata.ListSeparator.c_str());
  config->Write(wxT("/Graph-File-Load/TabSeparator"),m_graphdata.TabSeparator);
  config->Write(wxT("/Graph-File-Load/SpaceSeparator"),m_graphdata.SpaceSeparator);
  config->Write(wxT("/Graph-File-Load/ImaginaryDataIncluded"),m_graphdata.ImaginaryDataIncluded);
  config->Write(wxT("/Graph-File-Load/HeaderIncluded"),m_graphdata.HeaderIncluded);
  config->Write(wxT("/Graph-File-Load/XValuesIncluded"),m_graphdata.XValuesIncluded);
  config->Write(wxT("/Graph-File-Load/XValuesColumn"),m_graphdata.XValuesColumn);
  m_docManager->GetFileHistory()->Save(*config);
  delete config;
}

// ----------------------------------------------------------------------------
// EFDocManager
// ----------------------------------------------------------------------------

wxDocument *EFDocManager::CreateDocument(const wxString &path,long flags)
{
  wxDocument *doc = wxDocManager::CreateDocument(path,flags);
  TEFView *efview = wxDynamicCast(GetCurrentView(),TEFView);
  TEFDocument *efdoc = wxDynamicCast(doc,TEFDocument);
  if (efdoc && efview)
    efview->LoadData(((flags & wxDOC_NEW) || efdoc->GetEmptyFile()) &&
                     (wxGetApp().DiagFlags & MyApp::DiagInNewWorksheet));
  return doc;
}

// ----------------------------------------------------------------------------
// EFMDIChildFrame
// ----------------------------------------------------------------------------

EFMDIChildFrame::EFMDIChildFrame(wxDocument* doc, wxView* view, wxMDIParentFrame* parent,
                                  wxWindowID id, const wxString& title,
                                  const wxPoint& pos,const wxSize& size,long style,
                                  const wxString& name)
   : wxDocMDIChildFrame(doc,view,parent,id,title,pos,size,style,name)
{
  edplvw = 0;
  edit = 0;
  m_childcount = 0;
  SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_BTNFACE));
}

void EFMDIChildFrame::OnSize(wxSizeEvent &event)
{ // Update keyMaximChild Flag
  wxGetApp().Options = (wxGetApp().Options & ~MyApp::keyMaximChild) |
                       (IsMaximized() ? MyApp::keyMaximChild : 0);
  if (edplvw && edit)
  {
#if defined(__WXGTK__)
    // Raise child menu, otherwise it may fall behind parent one on Linux (?!)
    if (GetMenuBar())
      GetMenuBar()->Raise();
#endif
    LayoutChildren();
  }
  else
    event.Skip();
}

void EFMDIChildFrame::LayoutChildren(wxPanel *pnlo)
{
  // if panel passed in, make it the current panel
  if (pnlo)
    m_curpnl = pnlo;
  // Go through children list
  wxSize psize(0,edplvw->GetSize().y);

  // Maximale Breite der Panels ermitteln
  wxWindowListNode *Child = GetChildren().GetFirst();
  while (Child)
  {
    wxPanel *pnlt = wxDynamicCast(Child->GetData(),wxPanel);
    // if we have a panel
    if (pnlt && (pnlt->IsShown() || (pnlt == pnlo)))
      psize.x = (pnlt->GetSize().x > psize.x ? pnlt->GetSize().x : psize.x);
    Child = Child->GetNext();
  }

  // Panels anordnen
  Child = GetChildren().GetFirst();
  while (Child)
  {
    wxPanel *pnlt = wxDynamicCast(Child->GetData(),wxPanel);
    // if we have a panel
    if (pnlt && (pnlt->IsShown() || (pnlt == pnlo)))
    {
      pnlt->SetSize(0,psize.y,psize.x,wxDefaultCoord,wxSIZE_USE_EXISTING);
      pnlt->Show(true);
      pnlt->Refresh();
      psize.y += pnlt->GetSize().y;
    }
    Child = Child->GetNext();
  }
  wxSize size = GetClientSize();
  edplvw->SetSize(size.x,-1);
  // plain view is always on top left position
  edplvw->Move(-1, 0);
  // edit is on the right of the panel, and below the plain view
  if (edit)
    edit->SetSize(psize.x, edplvw->GetSize().y, size.x-psize.x, size.y-edplvw->GetSize().y);
}

//! Set editor child and create plain view
void EFMDIChildFrame::SetEditor(wxWindow *ed)
{
  edit = ed;
  edplvw = new TEFEditPlainView(this,wxID_ANY,wxDefaultPosition,wxDefaultSize);
  LayoutChildren();
}

// Extend event processing to search the view's event table
bool EFMDIChildFrame::ProcessEvent(wxEvent& event)
{
  bool ret;
#if wxCHECK_VERSION(2,9,1)
  ret = wxMDIChildFrame::ProcessEvent(event);
#else
  static wxEvent *ActiveEvent = NULL;

  // Break recursion loops
  if (ActiveEvent == &event)
    return false;

  // Remember this event
  ActiveEvent = &event;

  if ( !m_childView || !m_childView->ProcessEvent(event) )
  {
    // Only hand up to the parent if it's a menu command
    if (!event.IsKindOf(CLASSINFO(wxCommandEvent)) ||
        !GetParent() || !GetParent()->GetEventHandler()->ProcessEvent(event))
      ret = wxEvtHandler::ProcessEvent(event);
    else
      ret = true;
  }
  else
    ret = true;

  ActiveEvent = NULL;
#endif
  return ret;
}

void EFMDIChildFrame::OnIdle(wxIdleEvent &event)
{
  event.Skip();
  if (GetChildren().GetCount() != m_childcount)
  {
    m_childcount = GetChildren().GetCount();
    LayoutChildren();
  }
}

EFMDIChildFrame::~EFMDIChildFrame()
{
  if (GetMenuBar())
    wxGetApp().m_docManager->FileHistoryRemoveMenu(GetMenuBar()->GetMenu(0));
}

BEGIN_EVENT_TABLE(EFMDIChildFrame, wxDocMDIChildFrame)
  EVT_SIZE(EFMDIChildFrame::OnSize)
  EVT_IDLE(EFMDIChildFrame::OnIdle)
END_EVENT_TABLE()

// ----------------------------------------------------------------------------
// MyFrame
// ----------------------------------------------------------------------------

MyFrame::MyFrame(wxDocManager *manager, wxFrame *frame, wxWindowID id,
                 const wxString& title,
                 const wxPoint& pos, const wxSize& size, const long type)
 : wxDocMDIParentFrame(manager, frame, id, title, pos, size, type)
{
  wxString dir;
  wxGetApp().GetAppDataDir(dir);
  wxString orgpath = dir;

#if wxCHECK_VERSION(2,8,0)
  m_frmmgr.SetManagedWindow(this);
#endif

  wxGetApp().tb = EFXmlResource::Get()->LoadToolBar(this,wxT("efToolBar"));
  wxGetApp().tb2 = EFXmlResource::Get()->LoadToolBar(this,wxT("diagToolBar"));

  wxGetApp().tb->InsertControl(23,new EFZoomBox(wxGetApp().tb,XRCID("ID_ZOOM")));

  wxGetApp().tb->Realize();
  wxGetApp().tb2->Realize();

#if wxCHECK_VERSION(2,8,0)
  m_frmmgr.AddPane(wxGetApp().tb,wxAuiPaneInfo().Name(wxT("tb1")).Caption(wxT("Toolbar1")).
                   ToolbarPane().Top().Row(1).LeftDockable(false).RightDockable(false));
  m_frmmgr.AddPane(wxGetApp().tb2,wxAuiPaneInfo().Name(wxT("tb2")).Caption(wxT("Toolbar2")).
                   ToolbarPane().Top().Row(2).LeftDockable(false).RightDockable(false));
  m_frmmgr.Update();
#endif

  if (wxDirExists(orgpath))
  {
#ifdef EFHTMLHELP
    orgpath = dir + wxT("/ef.hhp");
#else
    orgpath = dir + wxT("/") + HELPFILESTRING;
#endif
  }
  else
#ifdef EFHTMLHELP
    orgpath = dir + wxT("/ef.hhp");
#else
    orgpath = dir + '/' + HELPFILESTRING;
#endif
#if !defined(__WXDEBUG__) || !defined(__MINGW32__)
  wxGetApp().HelpCtrl.Initialize(orgpath);
#endif
  SetDropTarget(new EFFileDropTarget);
  SetAutoLayout(true);
}

MyFrame::~MyFrame()
{
#if !defined(__WXDEBUG__) || !defined(__MINGW32__)
#if !defined(_MSC_VER) // doesn't work with VC, causes crash (?!)
  wxGetApp().HelpCtrl.Quit(); // prevent crash if help still open here
#endif
#endif
#if wxCHECK_VERSION(2,8,0)
  m_frmmgr.UnInit();
#endif
}

void MyFrame::OnZoomChanged(wxCommandEvent &)
{
  int i = wxGetApp().zoombox->GetSelection();
  long Zoom;
  wxString CurZoom;
  CurZoom = wxGetApp().zoombox->GetValue();
  if (i >= 0)
    CurZoom = wxGetApp().zoombox->GetStringSelection();
  // Remove last char if no digit, otherwise ToLong may fail
  if ((CurZoom.Last() < '0') || (CurZoom.Last() > '9'))
    CurZoom.RemoveLast();
  CurZoom.ToLong(&Zoom);
  if (Zoom < 10 || Zoom > 1000)
  {
	Zoom = 100;
    wxGetApp().zoombox->SetValue(EFZoomBox::Zooms[2]);
    wxBell();
  }
  wxGetApp().Zoom = Zoom;
  wxCommandEvent cevent(wxEVT_COMMAND_MENU_SELECTED,XRCID("ICM_ZOOM"));
  AddPendingEvent(cevent);
}

void MyFrame::SetWindowMenu(wxMenu *menu)
{ // patch for non-translated window menu (translate manually)
#ifdef __WXMSW__
  menu = new wxMenu;
  if (menu)
  {
    menu->Append(4002,_("&Cascade"));
    menu->Append(4001,_("Tile &Horizontally"));
    menu->Append(4005,_("Tile &Vertically"));
    menu->AppendSeparator();
    menu->Append(4003,_("&Arrange Icons"));
    menu->Append(4004,_("&Next"));
  }
  wxDocMDIParentFrame::SetWindowMenu(menu);
#endif
}

void MyFrame::OnSize(wxSizeEvent &event)
{
  wxSize size = GetClientSize();
#if !wxCHECK_VERSION(2,8,0)
  // do the toolbar positioning manually
  if (wxGetApp().tb2)
  {
    int offset;
    wxGetApp().tb2->SetSize(0,0,size.x,-1,wxSIZE_USE_EXISTING);
    offset = wxGetApp().tb2->GetSize().y;
    if (GetClientWindow())
      GetClientWindow()->SetSize(0, offset, size.x, size.y-offset);
  }
  else
#endif
    event.Skip();
}

wxPoint MyFrame::GetClientAreaOrigin() const
{
  wxPoint Pt = wxMDIParentFrame::GetClientAreaOrigin();
  return Pt;
}

void MyFrame::OnCloseWindow(wxCloseEvent& event)
{
  wxGetApp().Options = (wxGetApp().Options & ~MyApp::keyMaximized) |
                       (IsMaximized() ? MyApp::keyMaximized : 0);
  if (!(wxGetApp().Options & MyApp::keyMaximized))
    wxGetApp().IniRect = GetRect();
  wxMDIChildFrame *tmp = GetActiveChild();
  if (tmp)
    wxGetApp().Options = (wxGetApp().Options & ~MyApp::keyMaximChild) |
                         (tmp->IsMaximized() ? MyApp::keyMaximChild : 0);
  event.Skip();
}

void MyFrame::OnAbout(wxCommandEvent &)
{
  TEFAboutDlg *dlg = new TEFAboutDlg;
  if (dlg)
  {
    dlg->Create(this,dlgAbout,_("About"));
    dlg->ShowModal();
    dlg->Destroy();
  }
}

void MyFrame::OnOptionen(wxCommandEvent &)
{
  if (wxGetApp().m_docManager->GetDocuments().GetCount() > 0)
  {
    if ((wxMessageBox(_("Options dialog can only be opened when all documents are closed !\nClose Documents now ?"),
                      wxT("Easy Funktion"),
                      wxYES+wxNO+wxICON_INFORMATION,this) == wxNO) ||
                      !wxGetApp().m_docManager->CloseDocuments(false))
      return;
  }
  SetFocus(); // Avoid program crash due to buggy last wxWidgets focus implementation ...
  TEFOptionenDlg *TD = new TEFOptionenDlg();
  if (TD)
  {
    TD->Create(this,dlgOptionen,_("Options"));
    TD->ShowModal();
    TD->Destroy();
  }
}

void MyFrame::OnHelp(wxCommandEvent &)
{
  wxGetApp().HelpCtrl.DisplayContents();
}

BEGIN_EVENT_TABLE(MyFrame, wxDocMDIParentFrame)
  EVT_SIZE(MyFrame::OnSize)
  EVT_COMBOBOX(XRCID("ID_ZOOM"), MyFrame::OnZoomChanged)
  EVT_COMMAND_ENTER(XRCID("ID_ZOOM"),MyFrame::OnZoomChanged)
  EVT_MENU(wxID_ABOUT, MyFrame::OnAbout)
  EVT_MENU(XRCID("CMD_OPTIONEN"), MyFrame::OnOptionen)
  EVT_CLOSE(MyFrame::OnCloseWindow)
  EVT_MENU(wxID_HELP_CONTENTS,MyFrame::OnHelp)
END_EVENT_TABLE()

