// Version 6.05 20.07.2008 Markus Greither
// Copyright (c) 1991-2008 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file efbrcalc.cpp
    \brief Source code for the fraction format calculation object of the function editor
*/


#include "efprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#ifdef __DMC__
#include <typeinfo.h>
#endif
#include <wx/wx.h>

#include <complex>
#include <limits>
#include <climits>
#include <set>
#include <functional>
#if defined(__MINGW32__)
#include <wx/msw/winundef.h>
#endif

using namespace std;

#include "efdefs.h"
#include "mg.h"
#include "cpxpsr.h"
#include "efresid.h"
#include "efbrcalc.h"

#endif

#include <wx/caret.h>

// ----------------------------------------------------------------------------
// BruchType
// ----------------------------------------------------------------------------

BruchType::BruchType()
{
  ObZ.SetValue(1,0);
  UntZ.SetValue(1,0);
  FehlerPos = 0;
  FormatError = true;
  Commentline = false;
  Modified = false;
};

bool BruchArrObj::IsModified(int Nr)
{
  SetAkEint(Nr);
  return AkZ().Modified;
}

void BruchArrObj::ClearModified(int Nr)
{
  SetAkEint(Nr);
  AkZ().Modified = false;
}

void BruchArrObj::SetModified(bool set,vector<bool> &SetFus)
{
  Begin();
  while (NextEint())
  {
    if ((AkEintNummer() < (int)SetFus.size()) && SetFus[AkEintNummer()])
      AkZ().Modified = set;
  }
}

// ----------------------------------------------------------------------------
// FormatStrObj
// ----------------------------------------------------------------------------

void FormatStrObj::InsFontMarker(unsigned int Pos,_TCHAR C,unsigned char H)
{ // Fuegt eine Schriftformat-Markierung ein
  int i = DelFontMarker(Pos);
  _TCHAR A[4];
  if ((C != _TCHAR(TEFTextCalc::PageMarker)) && (C & TEFTextCalc::FontHeight))
  {
    A[3] = 0;
    A[2] = H;
  }
  else
    A[2] = 0;
  A[1] = C;
  A[0] = TEFTextCalc::FontMarker;
  MGPStr::InsStr(i,A);
}

unsigned int FormatStrObj::DelFontMarker(unsigned int Pos)
{ // Loescht die Schriftformat-Markierung an Stelle Pos
  unsigned int i = ModMarkPos(Pos);
  do
  {
    if (AkStr[i] == _TCHAR(TEFTextCalc::FontMarker))
    {
      if ((AkStr[i+1] != _TCHAR(TEFTextCalc::PageMarker)) &&
          (AkStr[i+1] & TEFTextCalc::FontHeight))
        MGPStr::DelNChar(i,3);
      else
        MGPStr::DelNChar(i,2);
    }
    i = ModMarkPos(Pos);
  }
  while (AkStr[i] == _TCHAR(TEFTextCalc::FontMarker));
  return i;
}

unsigned int FormatStrObj::FLength()
{ // ermittelt die unformatierte Lnge des Strings
  unsigned int FLen = 0;
  if (AkStr[0] != 0)
  {
    int p = 0;
    while (AkStr[p] != 0)
    {
      if (AkStr[p] == _TCHAR(TEFTextCalc::FontMarker))
      {
        p++;
        if ((AkStr[p] != _TCHAR(TEFTextCalc::PageMarker)) &&
            (AkStr[p] & TEFTextCalc::FontHeight))
          p++;
      }
      else
        FLen++;
      p++;
    }
  }
  return FLen;
}

// Ermittelt die um die Schriftformat-Markierung korrigierte
// String-Position VOR der naechsten Markierung
unsigned int FormatStrObj::ModMarkPos(unsigned int Pos) const
{
  unsigned int p = 0,i = 0;
  while ((AkStr[p] != 0) && (i < Pos))
  {
    if (AkStr[p] == _TCHAR(TEFTextCalc::FontMarker))
    {
      p++;
      if ((AkStr[p] != _TCHAR(TEFTextCalc::PageMarker)) &&
          (AkStr[p] & TEFTextCalc::FontHeight))
        p++;
    }
    else
      i++;
    p++;
  }
  return p;
}

unsigned int FormatStrObj::ModCharPos(unsigned int Pos) const
{ // wie ModMarkPos, aber Position NACH der naechsten Markierung
  unsigned int p = ModMarkPos(Pos);
  while (AkStr[p] == _TCHAR(TEFTextCalc::FontMarker))
  {
    if ((AkStr[p+1] != _TCHAR(TEFTextCalc::PageMarker)) &&
        (AkStr[p+1] & TEFTextCalc::FontHeight))
      p += 3;
    else
      p += 2;
  }
  return p;
}

unsigned int FormatStrObj::ModLen(unsigned int Pos,unsigned int L) const
{ // korrigierte Laenge des String-Abschnitts zw. Pos und L
  unsigned int p = Pos,i = 0;
  while ((AkStr[p] != 0) && (i < L))
  {
    if (AkStr[p] == _TCHAR(TEFTextCalc::FontMarker))
    {
      p++;
      if ((AkStr[p] != _TCHAR(TEFTextCalc::PageMarker)) &&
          (AkStr[p] & TEFTextCalc::FontHeight))
        p++;
    }
    else
      i++;
    p++;
  }
  return p-Pos;
}

unsigned short FormatStrObj::GetTextStr(MGPStr &AStr) const
{ // ermittelt den unformatierten String
  AStr.SetStr(wxT(""));
  unsigned int p = 0;
  unsigned short Format = 0;
  do
  {
    unsigned int p1 = p;
    while (AkStr[p] && (AkStr[p] != _TCHAR(TEFTextCalc::FontMarker)))
      p++;
    if (p1 != p)
      AStr.AddNChar(&AkStr[p1],p-p1);
    if (AkStr[p] == _TCHAR(TEFTextCalc::FontMarker))
    {
      p++;
      Format = AkStr[p];
      if ((AkStr[p] != _TCHAR(TEFTextCalc::PageMarker)) &&
          (AkStr[p] & TEFTextCalc::FontHeight))
      {
        p++;
        Format += AkStr[p] << 8;
      }
      p++;
    }
  }
  while (AkStr[p] != 0);
  return Format;
};

// ----------------------------------------------------------------------------
// TEFTextCalc
// ----------------------------------------------------------------------------

IMPLEMENT_DYNAMIC_CLASS(TEFTextCalc,wxWindow)

TEFTextCalc::TEFTextCalc(wxWindow *parent,wxWindowID id,const wxPoint &pos,
                         const wxSize &size,long style,const wxString &title)
 : wxWindow(parent,id,pos,size,style,title),Printing(false)
{
  LabelFontStyle = Italic;
  KommentFontStyle = Normal;
  LabelFontColour = 0xC00000;
  KommentFontColour = 0x007F00;
  FontMetric.FontName = wxT("");
  WFont = 0;
  IFont = 0;
  CFont = 0;
  AveChrW = 5; // Mittlere Zeichenbreite
  FHeight = 5; // Schrifthoehe (um FSIZEFAK vergroessert gegenueber FontMetrix)
  SetCaret(new wxCaret(this,1,1));
  GetCaret()->Show();
  FontMetric.FPoints = 12;
  KlSize = wxSize(5,5); // Klammer-Breite und Hoehe
  DC = 0;
  CDC = 0;
  Zoom = 1;
  OrgZoom = 1;
  wxClientDC dc(this);
  wxSize sz = dc.GetPPI();
  PixPerInchX = sz.x;
  PixPerInchY = sz.y;
  FSizeFak = float(23.0/20.0);
  wxColour clr = wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW);
  SetBackgroundColour(clr);
#if wxCHECK_VERSION(2,5,0)
  ClearBackground();
#else
  Clear();
#endif
}

void TEFTextCalc::HideCaret()
{
  if (GetCaret())
    GetCaret()->Hide();
}

void TEFTextCalc::ShowCaret()
{
  if (GetCaret())
    GetCaret()->Show();
}

void TEFTextCalc::GetDC()
{
  DC = new wxClientDC(this);
  SetFormatFont(Normal,0,true);
  DC->SetBackgroundMode(wxSOLID);
}

void TEFTextCalc::DelDC()
{
  DC->SetFont(wxNullFont);
  delete DC;
  DC = 0;
  delete WFont;
  WFont = 0;
  delete IFont;
  IFont = 0;
  delete CFont;
  CFont = 0;
}

wxFont *TEFTextCalc::CreateFormatFont(char Format,unsigned char H) const
{
  int Weight,Style;
  bool Underlined;
  Weight = (Format & Bold) == Bold ? wxBOLD : wxNORMAL;
  Underlined = (Format & Underline) == Underline;
  Style = (Format & Italic) == Italic ? wxITALIC : wxNORMAL;
  int Height = ((Format & FontHeight) && H ? H : FontMetric.FPoints);
  return new wxFont(int(OrgZoom*Height > 1 ? OrgZoom*Height : 1),
                    wxDEFAULT,Style,Weight,Underlined,FontMetric.FontName);
}

void TEFTextCalc::SetFormatFont(char Format,unsigned char H,bool force)
{ // Ermittelt die aktuelle Schrift mit Format
  static int LastHeight = -1;
  static char LastFormat = 0;
  if (force || !WFont || (FontMetric.FPoints != LastHeight) ||
      (Format != LastFormat) || H)
  {
    LastHeight = FontMetric.FPoints;
    LastFormat = Format;
    wxFont *AFont = CreateFormatFont(Format,H);
    wxFont *AIFont = CreateFormatFont(LabelFontStyle);
    wxFont *ACFont = CreateFormatFont(KommentFontStyle);
    delete WFont;
    WFont = AFont;
    delete IFont;
    IFont = AIFont;
    delete CFont;
    CFont = ACFont;
  }
  if (WFont)
  {
    if (DC)
    {
      DC->SetFont(*WFont);
      wxCoord x,y;
      EFTextExtent(*DC,wxString(wxT("(")),&x,&y);
      KlSize = wxSize(x,y);
    }
    if (CDC)
      CDC->SetFont(*WFont);
  }
}

void TEFTextCalc::ChangeFontSize(bool up)
{ // Schriftgroesse veraendern
  if (up)
  { // Schrift vergroessern
    if (FontMetric.FPoints < 36)
    {
      FontMetric.FPoints += (FontMetric.FPoints/12+2);
      if (FontMetric.FPoints > 36)
        FontMetric.FPoints = 36;
      wxFont AFont(FontMetric.FPoints,wxDEFAULT,FontMetric.Style,
                   FontMetric.Weight,FontMetric.Underlined,FontMetric.FontName);
      SetFont(AFont);
    }
  }
  else
  { // Schrift verkleinern
    if (FontMetric.FPoints > 6)
    {
      FontMetric.FPoints -= (FontMetric.FPoints/12+2);
      if (FontMetric.FPoints < 6) FontMetric.FPoints = 6;
      wxFont AFont(FontMetric.FPoints,wxDEFAULT,FontMetric.Style,
                   FontMetric.Weight,FontMetric.Underlined,FontMetric.FontName);
      SetFont(AFont);
    }
  }
}

void TEFTextCalc::GetFontMetrics(FontMetricStruct &FM,const wxFont &font)
{
  FM.FontName = font.GetFaceName();
  FM.FPoints = font.GetPointSize(); // Groesse der Original-Schrift
  wxClientDC dc(this);
  dc.SetFont(font);
  FM.Weight = wxNORMAL;
  FM.Style = wxNORMAL;
  FM.Underlined = false;
  FM.OrgHeight = dc.GetCharHeight();
  FM.OrgAveCharW = dc.GetCharWidth();
  wxCoord w,h;
  EFTextExtent(dc,wxString(wxT("Mg")),&w,&h,&FM.Descent,&FM.ExternalLeading);
  dc.SetFont(wxNullFont);
}

wxSize TEFTextCalc::ZeileExtent(const _TCHAR* string,int stringLen) const
{
  if (!CDC)
  {
    wxSize sz;
    wxString str(string);
    str.Truncate(stringLen);
    EFTextExtent(*DC,str,&sz.x,&sz.y);
    return sz;
  }
  else
  {
    wxSize size(0,0);
    _TCHAR St[2];
    St[1] = 0;
    wxCoord x,y = 0;
    for (int i = 0;i < stringLen;i++)
    { // Stringlaengen-Berechnung fuer Zwischenablage mit EINZELNEN Zeichen
      St[0] = string[i];
      wxString str(&string[i]);
      str.Truncate(1);
      EFTextExtent(*DC,str,&x,&y);
      size.x += x;
    }
    size.y = y;
    return size;
  }
}

wxSize TEFTextCalc::EFTabbedTextExtent(const _TCHAR* string,
                                       int count,bool kursiv) const
{
  int altpos = 0;
  wxSize size(0,0),size1;

  while (altpos < count)
  {
    int anf = altpos;
    if (!kursiv)
      anf = count; // bei Fehler keine kursive Ausgabe der Labels
    else
    while ((anf < count) && (string[anf] != '[') && (string[anf] != '"'))
      anf++; // Anfangsposition des naechsten Labels ermitteln
    if (anf > altpos)
    { // String vor 1. Label vorhanden
      size1 = ZeileExtent(&string[altpos],anf-altpos);
      size.x += size1.x;
      size.y = size1.y;
      altpos += anf-altpos;
    }
    if (anf < count)
    { // Label oder Kommentar vorhanden
      _TCHAR EndChar = string[anf];
      if (EndChar == '[')
        EndChar = ']';
      int end = anf+1; // Anfangs-Zeichen ueberlesen (wg. ")
      while ((end < count) && (string[end] != EndChar))
        end++;
      int endpos = (end == count ? end-1 : end);
      MGPStr tmpstr;
      tmpstr.AddNChar(&string[altpos],endpos-anf+1);
      tmpstr.ReplChar(0,' ');
      if (end < count) // Ende eines Labels, ] durch ' ' ersetzen
        tmpstr.ReplChar(endpos-anf,' ');
      if (EndChar == '"') // Kommentar-Schrift einstellen
      {
        DC->SetFont(*CFont);
        if (CDC)
          CDC->SetFont(*CFont);
      }
      else
      {
        DC->SetFont(*IFont);
        if (CDC)
          CDC->SetFont(*IFont);
      }
      size1 = ZeileExtent(tmpstr.c_str(),endpos-anf+1);
      DC->SetFont(*WFont);
      if (CDC)
        CDC->SetFont(*WFont);
      size.x += size1.x;
      size.y = size1.y;
      altpos += endpos-anf+1;
    }
  }
  return size;
}

void TEFTextCalc::FindFirstChar(const _TCHAR *Zeile,unsigned int &f) const
{ // ermittelt die erste Position in String Zeile, die kein PosMarker ist
  while ((Zeile[f] == _TCHAR(PosMarker)) || (Zeile[f] == ' '))
  {
    if (Zeile[f] == _TCHAR(PosMarker))
      f += 3;
    else
      f++;
  }
}

// Klammern entfernen, auch wenn nach der letzten Klammer Leerzeichen folgen
void TEFTextCalc::KlammernEntfernen(MGPStr &AkStr,unsigned int f) const
{
  unsigned int KlZaehler = 0,endp = f;
  while (AkStr[endp] && ((AkStr[endp] != ')') || (KlZaehler != 1)))
  {
    if ((AkStr[endp] == '\"') || (AkStr[endp] == '['))
    { // Labels/Kommentare ueberlesen
      _TCHAR endchar = (AkStr[endp] == '\"' ? '\"' : ']');
      endp++;
      while (AkStr[endp] && (AkStr[endp] != endchar))
        endp++;
    }
    else
    if (AkStr[endp] == '(') // Klammern registrieren
      KlZaehler++;
    else
    if (AkStr[endp] == ')')
      KlZaehler--;
    endp++;
    FindFirstChar(AkStr.c_str(),endp);
  }
  if ((AkStr[f] == '(') && (AkStr[endp] == ')'))
  {
    AkStr.ReplChar(f,' ');
    AkStr.ReplChar(endp,' ');
  }
}

// ----------------------------------------------------------------------------
// BruchCalcObj
// ----------------------------------------------------------------------------

void BruchCalcObj::ZeilenVerschieben(bool BruchstrichOut,const int ThisInZeile,
                                     const int ThisOutZeile,
                                     const int AkMidZeile)
{ // verschiebt die anderen Zeilen um f, um Platz fuer neuen Bruch zu machen
  if ((BrAkZ.UntZ[AkMidZeile] || BrAkZ.ObZ[AkMidZeile]) && BruchstrichOut)
  { // Zeilen muessen eingefuegt werden ?
    // Sicherheitsabfrage (notwendig ?)
    if ((ThisOutZeile > 0) && (ThisInZeile > 0))
    {
      // Hoehe des Bruchs ermitteln
      int Anz = BrAkZ.UntZ[AkMidZeile] + BrAkZ.ObZ[AkMidZeile]+1;
      // Abstand zw. Outzeile und Inzeile messen
      int Dist = abs(BrAkZ.MidZ[OutZeile[ThisOutZeile]])-
                 abs(BrAkZ.MidZ[InZeile[ThisInZeile]])-1;
      Anz -= Dist; // notwendige zus. Zeilen berechnen
      if (Anz < 0)
        Anz = 0;
      // Zeilen verschieben
      for (int i = InZeile[ThisInZeile]+1;i <= OutZeile[ThisOutZeile];i++)
        BrAkZ.MidZ.SetValue(i,short(BrAkZ.MidZ[i]+
                                    (BrAkZ.MidZ[i] > 0 ? Anz : -Anz)));
    }
    // pruefen, ob aktuelle obere Zeilen unter aeusseren Bruchstrich passen
    int Abst = GetOutZeilen(AkMidZeile) -
              (abs(BrAkZ.MidZ[OutZeile[ThisOutZeile]]-
               BrAkZ.MidZ[AkMidZeile])-1);
    if (Abst > 0)
    {
      for (int i = AkMidZeile;i <= AkZeile;i++) // Zeilen verschieben
        BrAkZ.MidZ.SetValue(i,short(BrAkZ.MidZ[i]+
                                    (BrAkZ.MidZ[i] > 0 ? -Abst : Abst)));
    }
  }
}

bool BruchCalcObj::InsertPos(int &AkZeile,int AkMidZeile,int AkAbsch,
                             unsigned int Pos)
{
  if (AkZeile > 16382)
    return false;
  AkZeile++;
  BrAkZ.BZeile.AddChar(TEFTextCalc::PosMarker); // Anfang der Zeile markieren
  BrAkZ.BZeile.AddChar(GetBIndexLow(AkZeile)); // Index eintragen
  BrAkZ.BZeile.AddChar(GetBIndexHigh(AkZeile));
  // Referenz auf aktuellen Abschnittsanfang
  BrAkZ.RefMidZ.SetValue(AkZeile,short(AkAbsch));
  BrAkZ.X.SetValue(AkZeile,Pos); // Aktuelle X-Position eintragen
  if (AkZeile > AkMidZeile) // Mittelzeil-Position kopieren
    BrAkZ.MidZ.SetValue(AkZeile,BrAkZ.MidZ[AkMidZeile]);
  return true;
};

void BruchCalcObj::ZeileWithoutAsterix(MGPStr &tmp)
{
  int KlZaehler = 0;
  unsigned int p = 0;
  while (tmp[p])
  {
    switch (tmp[p])
    {
      case '*':
        if (!KlZaehler)
          tmp.ReplChar(p,(_TCHAR)183);
        break;
      case '[':
        KlZaehler++;
        break;
      case ']':
        KlZaehler--;
    }
    p++;
  }
}

int BruchCalcObj::UnderscoreTextExtent(const _TCHAR *Zeile,unsigned int SLen,
                                       int &AkZeile,const int AkMidZeile,
                                       int &AkAbsch,
                                       ShortArrObj &Abschs,int GesLen)
{
  int Len = 0;
  unsigned int p = 0;
  bool Komment = false;
  while (Zeile[p] && (p < SLen))
  {
    unsigned int p1 = p;
    int Anf = -1;
    while (Zeile[p1] && (p1 < SLen) && ((Zeile[p1] != ']') || Komment))
    {
      if (Zeile[p1] == '[')
        Anf = p1;
      else
      if (Zeile[p1] == '"')
        Komment = !Komment;
      p1++;
    }
    bool MitUnderscore = false;
    if ((p1 < SLen) && (Zeile[p1] == ']') && (Anf != -1))
    {
      const _TCHAR *tmp = MGStrnchr(&Zeile[Anf],p1+1-Anf,'_');
      if (tmp && (tmp-&Zeile[Anf] <= int(p1+1-Anf)))
      {
        p1++;
        // gesamte Label-Laenge
        int geslen = TxtCalc->EFTabbedTextExtent(&Zeile[Anf],
                                                 p1-Anf,MarkLabels).x;
        // Label-Lange inklusive _
        int undslen = TxtCalc->EFTabbedTextExtent(&Zeile[Anf],
                                                  tmp-&Zeile[Anf]+1,
                                                  MarkLabels).x;
        MitUnderscore = true;
        BrAkZ.BZeile.AddNChar(&Zeile[p],tmp-&Zeile[p]); // String vor _
        Len += TxtCalc->EFTabbedTextExtent(&Zeile[p],
                                           tmp-&Zeile[p],MarkLabels).x;

        // zus ' ' nach String wg. Kursiv (ist breiter als TextExtent)
        if ((TxtCalc->GetLabelFontStyle() & TEFTextCalc::Italic) ==
            TEFTextCalc::Italic)
          Len += TxtCalc->EFTabbedTextExtent(wxT(" "),1,MarkLabels).x;
        BrAkZ.BZeile.AddNChar(wxT(" "),1); // Leerzeichen statt _ ausgeben
        if (!InsertPos(AkZeile,AkMidZeile,Abschs[AkAbsch],GesLen+Len))
          return 0;

        BrAkZ.MidZ.IncValue(AkZeile); // tieferstellen des Strings nach _
        BrAkZ.BZeile.AddNChar(tmp+1,p1-Anf-(tmp-&Zeile[Anf])-1);
        // Lange nach _ ,so muss nicht Laenge "von der Mitte an" erm. werden
        Len += geslen-undslen;
        if (!InsertPos(AkZeile,AkMidZeile,Abschs[AkAbsch],GesLen+Len))
          return 0;
        // UntZ-Array ggf. updaten
        if (BrAkZ.UntZ[AkMidZeile] < 1)
          BrAkZ.UntZ.SetValue(AkMidZeile,1);
        if (BrAkZ.UntZ[Abschs[AkAbsch]] < 1)
          BrAkZ.UntZ.SetValue(Abschs[AkAbsch],1);
      }
    }
    if (!MitUnderscore)
    {
      if (Zeile[p1] && (p1 < SLen))
        p1++;
      BrAkZ.BZeile.AddNChar(&Zeile[p],p1-p);
      Len += TxtCalc->EFTabbedTextExtent(&Zeile[p],p1-p,MarkLabels).x;
    }
    p = p1;
  }
  return Len;
}

bool BruchCalcObj::ZeileInAbschnitten(const _TCHAR *Zeile,unsigned int SLen,
                                      int &AkZeile,
                                      int AkMidZeile,int &AkAbsch,
                                      ShortArrObj &Abschs,unsigned int AkPos,
                                      int &Len,bool InsPos)
{ // zerlegt Zeile in Abschnitte mit Klammerebenen und traegt sie ein
  unsigned int i = 0;

  if (InsPos && !InsertPos(AkZeile,AkMidZeile,Abschs[AkAbsch],AkPos+Len))
    return false;
  unsigned int akpos = 0;
  bool Kommentar = false;
  do
  {
    while ((i < SLen) && (Kommentar ||
           (Zeile[i] != '(') && (Zeile[i] != ')')) && (Zeile[i] != '"'))
      i++;
    bool Komm = (Zeile[i] == '"'); // Anfang eines Kommentars speichern
    bool KlZu = ((i < SLen) && (Zeile[i] == ')'));
    if (KlZu)
      i++;
    if (i-akpos > 0)
    {
      MGPStr tmp;
      tmp.AddNChar(&Zeile[akpos],i-akpos);
      ZeileWithoutAsterix(tmp);
      if (MarkLabels)
        Len += UnderscoreTextExtent(tmp.c_str(),tmp.Length(),AkZeile,AkMidZeile,
                                    AkAbsch,Abschs,Len+AkPos);
      else
      {
        Len += TxtCalc->EFTabbedTextExtent(tmp.c_str(),tmp.Length(),MarkLabels).x;
        BrAkZ.BZeile.AddStr(tmp);
      }
    }
    if (KlZu)
    { // Abschnittsende
      if (AkAbsch > 1)
      {
        if (BrAkZ.UntZ[Abschs[AkAbsch]] > BrAkZ.UntZ[Abschs[AkAbsch-1]])
          BrAkZ.UntZ.SetValue(Abschs[AkAbsch-1],BrAkZ.UntZ[Abschs[AkAbsch]]);
        if (BrAkZ.ObZ[Abschs[AkAbsch]] > BrAkZ.ObZ[Abschs[AkAbsch-1]])
          BrAkZ.ObZ.SetValue(Abschs[AkAbsch-1],BrAkZ.ObZ[Abschs[AkAbsch]]);
        AkAbsch--;
        if (i < SLen) // Am Ende der Zeile keine Position mehr speichern
          if (!InsertPos(AkZeile,AkMidZeile,Abschs[AkAbsch],AkPos+Len))
            return false;
      }
      akpos = i;
    }
    else
    if ((i < SLen) && (Zeile[i] == '('))
    { // Neuer Klammer-Abschnittsanfang
      if (!InsertPos(AkZeile,AkMidZeile,AkZeile+1,AkPos+Len))
        return false;
      AkAbsch++;
      Abschs.SetValue(AkAbsch,short(AkZeile));
      akpos = i; // sicherstellen, dass diese Klammer mit ausgegeben wird
      i++;
    }
    else
    { // fuer Kommentare, Kommentar-Anfang speichern und " ueberlesen
      akpos = i;
      i++;
    }
    Kommentar = Komm; // naechster Abschnitt ist ein Kommentar ?
  }
  while (akpos < SLen);
  return true;
}

inline void BruchCalcObj::AddInZeile(const int AkMidZeile)
{ // innere Zeile hinzufuegen
  if (BrAkZ.MidZ[AkMidZeile] > 0)
    BrAkZ.ObZ.IncValue(AkMidZeile);
  else
    BrAkZ.UntZ.IncValue(AkMidZeile);
}

inline void BruchCalcObj::AddOutZeile(const int AkMidZeile)
{ // aeussere Zeile hinzufuegen
  if (BrAkZ.MidZ[AkMidZeile] > 0)
    BrAkZ.UntZ.IncValue(AkMidZeile);
  else
    BrAkZ.ObZ.IncValue(AkMidZeile);
}

inline int BruchCalcObj::GetOutZeilen(int AkMidZeile)
{
  if (BrAkZ.MidZ[AkMidZeile] > 0)
    return BrAkZ.UntZ[AkMidZeile];
  else
    return BrAkZ.ObZ[AkMidZeile];
}

inline int BruchCalcObj::GetInZeilen(int AkMidZeile)
{
  if (BrAkZ.MidZ[AkMidZeile] > 0)
    return BrAkZ.ObZ[AkMidZeile];
  else
    return BrAkZ.UntZ[AkMidZeile];
}

void BruchCalcObj::NewMidZeile(bool Inc,const int AkMidZeile)
{ // alle Mittelzeilen der aktuellen Zeile verschieben
  for (int i = AkMidZeile;i < AkZeile;i++)
  {
    if ((BrAkZ.MidZ[AkMidZeile] > 0) && Inc ||
        (BrAkZ.MidZ[AkMidZeile] <= 0) && !Inc)
      BrAkZ.MidZ.IncValue(i);
    else
      BrAkZ.MidZ.DecValue(i);
  }
}

int BruchCalcObj::GetInAbst(int AkMidZeile)
{ // den geringsten Abstand dieser Zeile vom (aeueren) Bruchstrich ermitteln
  int Abst = -1;
  for (int i = AkMidZeile;i < AkZeile;i++)
  { // alle Positionen durchgehen
    int Ak = abs(BrAkZ.MidZ[OutZeile[AkOutZeile]]-BrAkZ.MidZ[i])-
             GetOutZeilen(i);
    if ((Abst == -1) || (Ak < Abst))
      Abst = Ak;
  }
  return Abst;
}

int BruchCalcObj::GetOutAbst(int AkMidZeile)
{ // den geringsten Abstand dieser Zeile vom (inneren) Bruchstrich ermitteln
  int Abst = -1;
  for (int i = AkMidZeile;i < AkZeile;i++)
  { // alle Positionen durchgehen
    int Ak = abs(BrAkZ.MidZ[i]-BrAkZ.MidZ[InZeile[AkInZeile]])-GetInZeilen(i);
    if ((Abst == -1) || (Ak < Abst))
      Abst = Ak;
  }
  return Abst;
}

void BruchCalcObj::ZeilenEinfuegen(int AkMidZeile,
                                   bool PGef,bool AkPGef,
                                   bool UGef,bool AkUGef)
{
  AkZeile++;
  if (AkPGef || AkUGef)
  { // Bruch/Potenz gefunden
    if (BrAkZ.MidZ[AkMidZeile] != 0)
    { // erster Bruch in dieser Zeile, nicht die Mittelzeile
      int AnsZ;
      // Abstand zum vorherigen Bruchstrich bestimmen
      if (BruchStrichOut(AkMidZeile))
        AnsZ = GetOutAbst(AkMidZeile);
      else
        AnsZ = GetInAbst(AkMidZeile);
      int f = 0; // Zeilen einfuegen in Abhaengigkeit vom vorhandenen Platz
      if (AddIn(AkMidZeile,PGef,AkPGef,UGef,AkUGef))
      {
        AddInZeile(AkMidZeile);
        f++;
      }
      if (AddOut(AkMidZeile,PGef,AkPGef,UGef,AkUGef))
      {
        AddOutZeile(AkMidZeile);
        f++;
      }
      AnsZ = f+1-AnsZ; // Mehr Zeilen notwendig als vorhanden ?
      if (BruchStrichOut(AkMidZeile) &&
          AddOut(AkMidZeile,PGef,AkPGef,UGef,AkUGef))
      { // wenn neue OutZeile, Mittelzeilposition in Abh. der neuen Zeilen
        if (AnsZ == 2)
          NewMidZeile(true,AkMidZeile); // Mittelzeile erhoehen
        else
        if (AnsZ <= 0)
          NewMidZeile(false,AkMidZeile); // Mittelzeile erniedrigen
      }
      else
      if (AddIn(AkMidZeile,PGef,AkPGef,UGef,AkUGef))
      {
        // Mittelzeile erhoehen wenn neue InZeile
        if (!BruchStrichOut(AkMidZeile) || (AnsZ > 0))
          NewMidZeile(true,AkMidZeile);
      }
    }
  }
  AkZeile--;
}

bool BruchCalcObj::EvalChars(const _TCHAR *Zeile,unsigned int &Pos,int &Pos1,ShortArrObj &Anfs,int &KlZaehler,
                             unsigned int &AnfLastKomment,unsigned int &Anf,int &End,int &BruchKl,
                             bool &FirstChar,int &AnfKlE,bool &AkBPGef,bool &VorZeichen,bool &RechZ,
                             unsigned int &AkUPos,const int AkMidZeile,bool &GanzeZeile)
{
  do
  { // ganzen String bearbeiten
    bool AltRechZ = RechZ;
    RechZ = false;
    bool AltVorZeichen = VorZeichen;
    VorZeichen = false;
    // Leerzeichen und Kommentare ignorieren
    AnfLastKomment = Psr.LeerZNKomment(Zeile,Pos);
    if (!_tcsncmp(&Zeile[Pos],Psr.GetLDV().ListSeparator.c_str(),
                 Psr.GetLDV().ListSeparator.Length()))
      Pos++; // Trennzeichen beruecksichtigen
    else
    if (Zeile[Pos] && ((Zeile[Pos] == '[') || (Zeile[Pos] == '\'') ||
        (IST_BUCHS(Zeile[Pos])) || (IST_ZAHL(Zeile[Pos])) ||
        !_tcsncmp(&Zeile[Pos],Psr.GetLDV().DecSeparator.c_str(),
                 Psr.GetLDV().DecSeparator.Length())))
    { // Funktionen,Labels und Zahlen lesen
      int i = Pos;
      if (!AkBPGef)
      {
        Anf = Pos;
        AnfKlE = KlZaehler;
      }
      while (IST_BUCHS(Zeile[Pos]) ||
             (Zeile[Pos] == '[') || (Zeile[Pos] == '\''))
      { // Funktionen lesen
        unsigned int f = Pos;
        if (!Psr.NextAkWert(Zeile,Pos))
        {
          SetFehlerPos(Zeile,Pos);
          return false;
        }
        if ((Zeile[f] == '[') || (Zeile[f] == '\''))
        {
          if ((Zeile[f] == '[') && (Zeile[Pos-1] != ']') ||
              (Zeile[f] == '\'') && (Zeile[Pos-1] != '\''))
          {
            SetFehlerPos(Zeile,Pos);
            return false;
          }
          else
          { // nach _ im Label suchen
            const _TCHAR *tmp = MGStrnchr(&Zeile[f],Pos-f,'_');
            if (MarkLabels && tmp && (tmp-&Zeile[f] <= int(Pos-f)))
              AkUPos = tmp-Zeile;
          }
        }
        // Leerzeichen und Kommentare ignorieren
        AnfLastKomment = Psr.LeerZNKomment(Zeile,Pos);
      }
      if (IST_ZAHL(Zeile[Pos]) ||
          !_tcsncmp(&Zeile[Pos],Psr.GetLDV().DecSeparator.c_str(),
                   Psr.GetLDV().DecSeparator.Length()))
      { // Parameter lesen
        int PrsFehler = ZahlFormatObj::NoError;
        int AltPos = Pos;
        BruchZahl tmp;
        Psr.ParseZahl(Zeile,Pos,PrsFehler,tmp);
        if (!tmp.GemischterBruch || (PrsFehler != ZahlFormatObj::NoError))
        {
          PrsFehler = ZahlFormatObj::NoError;
          Pos = AltPos;
          Psr.ZahlOhneBruch(Zeile,Pos,PrsFehler);
        }
        if (PrsFehler != ZahlFormatObj::NoError)
        {
          SetFehlerPos(Zeile,Pos);
          return false;
        }
      }
      else
      if (Zeile[Pos] == '(')
      { // Parameter in Klammern beruecksichtigen
        Anfs.SetValue(KlZaehler,(short)i); // Anfang ist Funktions-Anfang
        KlZaehler++;
        Pos++;
      }
      if (Zeile[Pos]) // Leerzeichen und Kommentare ignorieren
        AnfLastKomment = Psr.LeerZNKomment(Zeile,Pos);
      if ((BruchKl == KlZaehler) && (End == -1))
        End = Pos-1;
    }
    else
    if (Zeile[Pos])
    {
      switch (Zeile[Pos])
      { // restliche Zeichen auswerten
        case '(' : // Anf. neuer Klammerebenen speichern
           Anfs.SetValue(KlZaehler,(short)Pos);
           KlZaehler++;
           Pos++;
           break;
        case ')' : // obere Klammerebenen 'entwerten'
           Anfs.SetValue(KlZaehler,-1);
           if (KlZaehler > 0)
             KlZaehler--; // falsche Klammersetzung auswerten
           else
           {
             SetFehlerPos(Zeile,Pos);
             return false;
           }
           Pos++;
           // Leerzeichen und Kommentare ignorieren
           AnfLastKomment = Psr.LeerZNKomment(Zeile,Pos);
           if ((BruchKl == KlZaehler) && (End == -1))
             End = Pos-1;
           break;
        case ']' : // falsche Klammersetzung auswerten
           SetFehlerPos(Zeile,Pos);
           return false;
        case '\\':
        case '^':
           if ((BruchKl == -1) || (BruchKl > KlZaehler) &&
                                  (Anfs[KlZaehler] != -1) &&
                                  (Anfs[KlZaehler] < int(Anf)))
           { // Bruch gefunden
             if (!FirstChar)
             { // Bruchstrich nach Bruch nicht als Bruch darstellbar
               if (AnfKlE > KlZaehler)
                 Anf = Anfs[KlZaehler]; // Wenn Bruch-Term in Klammern
               Pos1 = Pos; // Position des Bruchstrichs
               AkBPGef = true;
               BruchKl = KlZaehler; // aktuelle Schachteltiefe speichern
               End = -1; // Flag fuer Ende des Bruchausdruckes ermitteln
             }
             if (AltRechZ)
             {  // Keine Potenz ohne zu potenzierenden Ausdruck
               SetFehlerPos(Zeile,Pos);
               return 0;
             }
             Pos++;
             // Leerzeichen und Kommentare ignorieren
             AnfLastKomment = Psr.LeerZNKomment(Zeile,Pos);
             if ((Zeile[Pos] == '*') || (Zeile[Pos] == '/') ||
                 (Zeile[Pos] == '\\'))
             {
               SetFehlerPos(Zeile,Pos);
               return 0;
             }
           }
           else
             Pos++;
           break;
        case '+' :
        case '-' : VorZeichen = true;
        case '/' :
        case '*' :
        case '<' :
        case '>' :
        case '=' :
        case '|' :
           if (((Zeile[Pos] == '>') && (Zeile[Pos+1] == '=')) ||
               ((Zeile[Pos] == '<') && (Zeile[Pos+1] == '=')) ||
               ((Zeile[Pos] == '<') && (Zeile[Pos+1] == '>')) ||
               ((Zeile[Pos] == '|') && (Zeile[Pos+1] == '|')))
             Pos++; // Mehrfache Operatoren zulassen
           if (AltRechZ && (AltVorZeichen || !VorZeichen))
           { // keine mehrfachen Vorzeichen/Rechenzeichen
             SetFehlerPos(Zeile,Pos);
             return false;
           }
           else
             RechZ = true;
           Anfs.SetValue(KlZaehler,-1); // obere Klammerebenen 'entwerten'
           Pos++;
           break;
        default:
           SetFehlerPos(Zeile,Pos);
           return false;
      }
    }
    if (FirstChar)
      FirstChar = false;
    if (Zeile[Pos]) // Leerzeichen und Kommentare ignorieren
      AnfLastKomment = Psr.LeerZNKomment(Zeile,Pos);
  }
  while ((Pos < _tcslen(Zeile)) &&
         (VorZeichen || (KlZaehler >= 1) || (AkMidZeile == 1) && GanzeZeile));
  return true;
}

bool BruchCalcObj::InsertFraction(const _TCHAR *Zeile,int &Pos1,unsigned int &Anf,int &RekCount,const int AkMidZeile,
                                  int &PotMidPos,ShortArrObj &Abschs,int &AkAbsch,int &Len,unsigned int &AkPos,
                                  unsigned int &Pos)
{
  int Lob = 0,Lunt = 0; // Laenge des oberen/unteren Bruch-Ausdrucks
  // Anfang und Ende der Positionen fuer oberen/unteren Bruch-Ausdruck
  int AnfUnt = 0,EndUnt = 0,AnfOb = 0,EndOb = 0;
  int BruchZeile = 0; // Marker fuer Bruchstrich-Ende
  // Marker fuer Anfang Oben, Anfang Bruchstrich und
  // Anfang unteren Bruch-Ausdruck
  int AnfO = 0,AnfB = 0;

  int LastIndex = AkInZeile;
  AkInZeile = RekCount;
  OutZeile.SetValue(RekCount,-1);
  InZeile.SetValue(AkInZeile,short(AkZeile));
  if (BrAkZ.MidZ[AkMidZeile] <= 0)
  { // wir sind oberhalb der Mittellinie
    if (Zeile[Pos1] == '^')
    { // wenn Potenz
      // immer oben anfangen
      BrAkZ.MidZ.SetValue(AkZeile+1,short(BrAkZ.MidZ[AkZeile]-PotMidPos));
      unsigned int f = BrAkZ.BZeile.Length();
      Len += FindObBruch(&Zeile[Pos1+1],AkPos+Len,AkMidZeile,
                         Abschs[AkAbsch],PotMidPos);
      TxtCalc->FindFirstChar(BrAkZ.BZeile.c_str(),f);
      if (!KlammerDarst)
        TxtCalc->KlammernEntfernen(BrAkZ.BZeile,f);
    }
    else
    { // Wenn Bruch
      // immer oben anfangen
      BrAkZ.MidZ.SetValue(AkZeile+1,short(BrAkZ.MidZ[AkZeile]-1));
      AnfOb = AkZeile+1; // markiere Anfang
      unsigned int f = BrAkZ.BZeile.Length();
      Lob = FindObBruch(&Zeile[Anf],AkPos+Len,AkMidZeile,Abschs[AkAbsch],1);
      TxtCalc->FindFirstChar(BrAkZ.BZeile.c_str(),f);
      if (!KlammerDarst)
        TxtCalc->KlammernEntfernen(BrAkZ.BZeile,f);
      EndOb = AkZeile; // markiere Ende
    }
  }
  else
  { // wir sind unterhalb der Mittellinie
    if (Zeile[Pos1] == '\\')
    { // Wenn Bruch
      // immer unten anf.
      BrAkZ.MidZ.SetValue(AkZeile+1,short(BrAkZ.MidZ[AkZeile]+1));
      AnfUnt = AkZeile+1; // markiere Anfang
      AnfO = BrAkZ.BZeile.Length(); // markiere oberen Anfang
      unsigned int f = BrAkZ.BZeile.Length();
      Lunt = FindUntBruch(&Zeile[Pos1+1],AkPos+Len,AkMidZeile,
                          Abschs[AkAbsch]);
      TxtCalc->FindFirstChar(BrAkZ.BZeile.c_str(),f);
      if (!KlammerDarst)
        TxtCalc->KlammernEntfernen(BrAkZ.BZeile,f);
      AnfB = BrAkZ.BZeile.Length(); // markiere Bruchstrich-Anfang
      EndUnt = AkZeile; // markiere Ende
    }
  }
  AkInZeile = LastIndex;
  InZeile.SetValue(RekCount,-1);
  if (Zeile[Pos1] == '\\')
  { // Wenn Bruch
    if (!InsertPos(AkZeile,AkMidZeile,Abschs[AkAbsch],AkPos+Len))
    { // Bruchstrich-Anfangsposition
      SetFehlerPos(Zeile,Pos);
      return false;
    }
    BrAkZ.BZeile.AddChar(TEFTextCalc::BruchMarker); // Bruchstrich einfuegen
    if (!InsertPos(AkZeile,AkMidZeile,Abschs[AkAbsch],AkPos+Len))
    { // Bruchstrich-Endposition
      SetFehlerPos(Zeile,Pos);
      return false;
    }
    BruchZeile = AkZeile;
  }
  if (BrAkZ.MidZ[AkMidZeile] != 0)
  { // wenn nicht in der Mittelzeile
    LastIndex = AkOutZeile;
    AkOutZeile = RekCount;
    OutZeile.SetValue(AkOutZeile,short(AkZeile));
  }
  else
  { // ansonsten InZeile erhoehen, Mittelzeile ist Spiegelachse
    LastIndex = AkInZeile;
    AkInZeile = RekCount;
    InZeile.SetValue(AkInZeile,short(AkZeile));
  }
  if (BrAkZ.MidZ[AkMidZeile] <= 0)
  { // wir sind oberhalb der Mittellinie
    if (Zeile[Pos1] == '\\')
    { // Wenn Bruch
      // unten weitermachen
      BrAkZ.MidZ.SetValue(AkZeile+1,short(BrAkZ.MidZ[AkZeile]+1));
      AnfUnt = AkZeile+1; // markiere Anfang
      unsigned int f = BrAkZ.BZeile.Length();
      Lunt = FindUntBruch(&Zeile[Pos1+1],AkPos+Len,AkMidZeile,
                          Abschs[AkAbsch]);
      TxtCalc->FindFirstChar(BrAkZ.BZeile.c_str(),f);
      if (!KlammerDarst)
        TxtCalc->KlammernEntfernen(BrAkZ.BZeile,f);
      EndUnt = AkZeile; // markiere Ende
    }
  }
  else
  { // wir sind unterhalb der Mittellinie
    if (Zeile[Pos1] == '^')
    { // wenn Potenz
      // oben weitermachen
      BrAkZ.MidZ.SetValue(AkZeile+1,short(BrAkZ.MidZ[AkZeile]-PotMidPos));
      unsigned int f = BrAkZ.BZeile.Length();
      Len += FindObBruch(&Zeile[Pos1+1],AkPos+Len,AkMidZeile,
                         Abschs[AkAbsch],PotMidPos);
      TxtCalc->FindFirstChar(BrAkZ.BZeile.c_str(),f);
      if (!KlammerDarst)
        TxtCalc->KlammernEntfernen(BrAkZ.BZeile,f);
    }
    else
    { // wenn Bruch
      // oben weitermachen
      BrAkZ.MidZ.SetValue(AkZeile+1,short(BrAkZ.MidZ[AkZeile]-1));
      AnfOb = AkZeile+1; // markiere Anfang
      int AnfU = BrAkZ.BZeile.Length(); // markiere Anfangs-Pos
      unsigned int f = BrAkZ.BZeile.Length();
      Lob = FindObBruch(&Zeile[Anf],AkPos+Len,AkMidZeile,Abschs[AkAbsch],1);
      TxtCalc->FindFirstChar(BrAkZ.BZeile.c_str(),f);
      if (!KlammerDarst)
        TxtCalc->KlammernEntfernen(BrAkZ.BZeile,f);
      EndOb = AkZeile; // markiere Ende
      MGPStr tmp;
      tmp.AddStr(&BrAkZ.BZeile[AnfU]); // unteren Ausdruck anhaengen
      BrAkZ.BZeile.ShortStr(AnfU);
      tmp.AddStr(&BrAkZ.BZeile[AnfB]); // Bruchstrich anhaengen
      BrAkZ.BZeile.ShortStr(AnfB);
      tmp.AddStr(&BrAkZ.BZeile[AnfO]); // oberen Ausdruck anhaengen
      BrAkZ.BZeile.ShortStr(AnfO); // BZEILE verkuerzen
      BrAkZ.BZeile.AddStr(tmp); // "umgedrehten" Bruch anhaengen
    }
  }
  if (BrAkZ.MidZ[AkMidZeile] != 0)
    AkOutZeile = LastIndex;
  else
    AkInZeile = LastIndex;
  if (Zeile[Pos1] == '\\')
  { // Wenn Bruch
    if (Lunt < Lob)
    { // oberer Term laenger als unterer
      Len += Lob;
      for (int i = AnfUnt;i <= EndUnt;i++)
        BrAkZ.X.SetValue(i,BrAkZ.X[i]+(Lob-Lunt)/2);
    }
    else
    { // unterer Term laenger als obener
      Len += Lunt;
      for (int i = AnfOb;i <= EndOb;i++)
        BrAkZ.X.SetValue(i,BrAkZ.X[i]+(Lunt-Lob)/2);
    }
    // Laenge des Bruchstrichs berichtigen
    BrAkZ.X.SetValue(BruchZeile,AkPos+Len);
  }
  return true;
}

int BruchCalcObj::FindBruch(const _TCHAR *Zeile,unsigned int AkPos,
                            bool &PGef,bool &UGef,
                            const int AkMidZeile,bool GanzeZeile)
{
  unsigned int Pos = 0; // Aktuelle Position
  unsigned int Anf = 0;
  int End; // Anfang, Ende fuer aktuellen Bruch
  int AnfKlE = -1; // Schachteltiefe fuer Anf
  int KlZaehler = 0; // aktuelle Klammer-Schachteltiefe
  unsigned int AkBruchAnf = 0;
  int Len = 0; // aktuelle Bruchlaenge in Pixel
  bool AkBPGef; // im aktuellen Lauf Bruch oder Potenz gefunden
  bool FirstChar; // Erstes Zeichen nach/vor Bruchstrich
  bool VorZeichen = false;  // Fuer Auswertung letztes Zeichen in [+,-]
  bool RechZ = false; // Zur Ueberpruefung mehrfacher Rechenzeichen
  ShortArrObj Anfs; // Anfaenge der Klammerebenen
  // Anfang des letzten, nicht mit " abgeschl. Kommentars
  unsigned int AnfLastKomment;
  // Index in Abschs des aktuellen Klammerebenen-Abschnitts dieser Zeile
  int AkAbsch = 1;
  // Array mit den Indexen der (Zeilenhoehen)-Abschnitte dieser Zeile
  ShortArrObj Abschs;
  int PotMidPos = 0; // Position einer Potenz ueber der aktuellen Mittelzeile
  unsigned int AkUPos = 0; // Aktuelle _ Position
  // aktuelle InZeile/OutZeile/Bruchstrich merken
  bool BruchstrichOut = BruchStrichOut(AkMidZeile);
  int ThisOutZeile = AkOutZeile;
  int ThisInZeile = AkInZeile;

  if (GanzeZeile)
    RekCount++;
  if (RekCount > MAXREK)
  {
    SetFehlerPos(Zeile,Pos);
    return 0;
  }
  if (!InsertPos(AkZeile,AkMidZeile,AkMidZeile,AkPos+Len))
  { // Mittelzeil-Position eintragen
    SetFehlerPos(Zeile,Pos);
    return 0;
  }
  Abschs.SetValue(1,short(AkZeile+1)); // Anfangsposition des 1. Abschnitts
  // Evaluate chars
  do
  { // alle Brueche dieser Zeile
    int Pos1 = 0; // Bruchstrich-Position fuer aktuellen Bruch
    int BruchKl = -1;
    AkBPGef = false;
    FirstChar = true;
    End = 0;
    if (!EvalChars(Zeile,Pos,Pos1,Anfs,KlZaehler,AnfLastKomment,Anf,End,BruchKl,
                   FirstChar,AnfKlE,AkBPGef,VorZeichen,RechZ,AkUPos,AkMidZeile,GanzeZeile))
      return 0;
    if (AnfLastKomment)
      Pos = AnfLastKomment; // Letzten Kommentar nicht mit ausgeben
    if (AkBPGef)
    {
      if (AkUPos > Anf)
        AkUPos = 0;
      ZeilenEinfuegen(AkMidZeile,PGef,true,
                                 UGef,(Zeile[Pos1] != '^'));
      if (!InsertPos(AkZeile,AkMidZeile,Abschs[AkAbsch],AkPos+Len))
      { // Position eintragen
        SetFehlerPos(Zeile,Pos);
        return 0;
      }
      if (AkBruchAnf < Anf)
      { // Eintrag VOR Bruch/Potenz eintragen
        if (!ZeileInAbschnitten(&Zeile[AkBruchAnf],Anf-AkBruchAnf,AkZeile,
                                AkMidZeile,AkAbsch,Abschs,AkPos,Len,false))
        {
          SetFehlerPos(Zeile,Pos);
          return 0;
        }
      }

      if (Zeile[Pos1] == '^')
      { // zu potenzierender Ausdruck eintragen
        InsertBase(Zeile,Pos1,Anf,PGef,UGef,Abschs,AkAbsch,
                   PotMidPos,Len,AkMidZeile,AkPos);
      }

      if (!InsertFraction(Zeile,Pos1,Anf,RekCount,AkMidZeile,
                          PotMidPos,Abschs,AkAbsch,Len,AkPos,Pos))
        return 0;

      if (Zeile[Pos1] != '^')
        UGef = true; // Unterlaenge nur bei Bruch
      PGef = true; // Oberlaenge bei Bruch UND Potenz
      Anf = 0;
      if (End == -1) // wenn kein Nenner vorhanden
      {
        SetFehlerPos(Zeile,Pos);
        return 0;
      }
      else
        Pos = End+1; // naechsten Bruch ermitteln
      KlZaehler = BruchKl;
      AkBruchAnf = Pos;
    }
  }
  while (!BrAkZ.FehlerPos && AkBPGef && (Pos < _tcslen(Zeile)));
  if (AkUPos && !UGef)
  {
    ZeilenEinfuegen(AkMidZeile,PGef,false,UGef,(AkUPos > 0));
    if (1 > BrAkZ.UntZ[AkMidZeile])
      BrAkZ.UntZ.SetValue(AkMidZeile,1);
    for (int i = 1;i <= AkAbsch;i++)
      if (1 > BrAkZ.UntZ[Abschs[i]])
        BrAkZ.UntZ.SetValue(Abschs[i],1);
    UGef = true;
  }
  if (!InsertPos(AkZeile,AkMidZeile,Abschs[AkAbsch],AkPos+Len))
  { // Ende der Zeile markieren
    SetFehlerPos(Zeile,Pos);
    return 0;
  }
  if (AkBruchAnf < Pos)
  { // Eintrag NACH Bruch eintragen
    if (!ZeileInAbschnitten(&Zeile[AkBruchAnf],Pos-AkBruchAnf,AkZeile,
                            AkMidZeile,AkAbsch,Abschs,AkPos,Len,false))
    {
      SetFehlerPos(Zeile,Pos);
      return 0;
    }
  }
  if ((BrAkZ.MidZ[AkMidZeile] == 0) && AnfLastKomment)
  { // letzten, nicht mit " abgeschl. Kommentar in die Mittelzeile eintragen
    if (!ZeileInAbschnitten(&Zeile[AnfLastKomment],
                            _tcslen(Zeile)-AnfLastKomment,AkZeile,AkMidZeile,
                            AkAbsch,Abschs,AkPos,Len))
    {
      SetFehlerPos(Zeile,Pos);
      return 0;
    }
  }
  if (GanzeZeile)
    RekCount--;
  ZeilenVerschieben(BruchstrichOut,ThisInZeile,ThisOutZeile,AkMidZeile);
  if (KlZaehler != 0)
  {
    BrAkZ.FormatError = true;
    return 0;
  }
  else
    return Len;
}

