// Version 6.06  29.11.2009 Markus Greither
// Copyright (c) 1991-2009 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file efedcmds.cpp
    \brief Source code for the function editor public commands
*/


#include "efprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#include <wx/xrc/xmlres.h>

#ifndef WX_PRECOMP
#ifdef __DMC__
#include <typeinfo.h>
#endif
#include <wx/wx.h>
#include <wx/resizec.h>

#include <wx/clipbrd.h>

#include <complex>
#include <limits>
#include <climits>
#include <set>
#include <functional>
#if defined(__MINGW32__)
#include <wx/msw/winundef.h>
#endif

using namespace std;

#include "efdefs.h"
#include "mg.h"
#include "cpxpsr.h"
#include "diagramm.h"
#include "efgemein.h"
#include "ef.h"
#include "efresid.h"
#include "efedit.h"

#endif

#include <wx/metafile.h>
#include <wx/caret.h>
#include "efmacro.h"

// ----------------------------------------------------------------------------
// EditTimer
// ----------------------------------------------------------------------------

void EditTimer::Notify()
{
  Editor->OnTimer();
}

// ----------------------------------------------------------------------------
// TEFEdit
// ----------------------------------------------------------------------------

void TEFEdit::SendModified()
{
  wxCommandEvent event(wxEVT_COMMAND_MENU_SELECTED,XRCID("ICM_SETMODIFIED"));
  event.SetEventObject(this);
  GetParent()->GetEventHandler()->ProcessEvent(event);
}

void TEFEdit::ClearLineModified()
{
  int tmp = BrZ.AkEintNummer();
  BrZ.Begin();
  while (BrZ.NextEint())
    BrZ.AkZ().Modified = false;
  BrZ.SetAkEint(tmp);
}

void TEFEdit::ShowValueWindow(const MGPStr &value)
{
  if (!ValueWin)
  {
#ifdef __WXMSW__
    wxWindow *frm = wxGetApp().GetTopWindow();
#else
    wxWindow *frm;
    if (m_Mode == EquationStyle)
      frm = GetGrandParent();
    else
    // not GetTopWindow(),
    // because this causes nasty redraw each time ValueWin disappears
      frm = this;
#endif
    if (!frm)
      frm = this;
    ValueWin = new ToolWindow(frm);
    wxPoint pt(ValueWinPos.x+10,ValueWinPos.y);
    pt = ClientToScreen(pt);
    pt = frm->ScreenToClient(pt);
    ValueWin->Move(pt);
    ValueWin->SetText(value);
  }
}

void TEFEdit::AddChild(wxWindowBase *child)
{
  wxWindow::AddChild(child);
  SetDocSize(true);
}

void TEFEdit::RemoveChild(wxWindowBase *child)
{
  wxWindow::RemoveChild(child);
  SetDocSize(true);
}

void TEFEdit::UndoRedo(bool redo)
{
  TextPos NeuAnf = m_Markiert.GetAnf(),NeuEnd = m_Markiert.GetEnd();

  if (redo  && !Rueck.RedoAvailable() ||
      !redo && !Rueck.UndoAvailable())
    return;
  if (redo)
    Rueck++;
  UndoItem &AkItem = Rueck.GetAkItem();
  MGPStr tmp;
  if ((AkItem.End.Pos != AkItem.Anf.Pos) ||
      (AkItem.End.Zeile != AkItem.Anf.Zeile))
  { // if marking available delete it
    NeuAnf = AkItem.Anf;
    NeuEnd = AkItem.End;
    m_Markiert.Set(NeuAnf,NeuEnd,false);
    GetSelText(tmp,true,false);
    NeuAnf = m_Markiert.GetAnf();
    NeuEnd = m_Markiert.GetEnd();
  }
  NeuEnd = AkItem.Ins;
  NeuAnf = NeuEnd;
  // Call Insert in any case as it updates the scrollbars
  m_Markiert.Set(NeuAnf,NeuEnd,false);
  Insert(AkItem.Str.c_str(),true,false); // insert at AkItem.Ins
  NeuEnd = m_Markiert.GetEnd();
  NeuAnf = AkItem.Ins;
  Rueck.InvertAkItem(NeuEnd,tmp);
  if (!redo)
    Rueck--;
  m_Markiert.Set(NeuAnf,NeuEnd);
  ScrollToCaret();
  Refresh();
}

void TEFEdit::SetFocus()
{
  TEFTextCalc::SetFocus();
}

void TEFEdit::GetNewSelection(unsigned int &Anf,unsigned int &End)
{
  int KlZ = 0;
  while ((End < BrZ.AkZ().FZeile.FLength()) &&
         !((BrZ.AkZ().FZeile[End] == ']') && !KlZ) &&
         !((BrZ.AkZ().FZeile[End] == ')') && !KlZ) &&
         !((BrZ.AkZ().FZeile[End] == '=') && !KlZ))
  {
    if ((BrZ.AkZ().FZeile[End] == '(') ||
        (BrZ.AkZ().FZeile[End] == '['))
      KlZ++;
    if ((BrZ.AkZ().FZeile[End] == ')') ||
        (BrZ.AkZ().FZeile[End] == ']'))
      KlZ--;
    End++;
  }
  if (Anf > 0)
    Anf--;
  KlZ = 0;
  while ((Anf > 0) &&
         !((BrZ.AkZ().FZeile[End] == ']') &&
           (BrZ.AkZ().FZeile[Anf] == '[') && !KlZ) &&
         !((BrZ.AkZ().FZeile[End] == ')') &&
           (BrZ.AkZ().FZeile[Anf] == '(') && !KlZ) &&
         !((BrZ.AkZ().FZeile[Anf] == '=') && !KlZ))
  {
    if ((BrZ.AkZ().FZeile[Anf] == '(') ||
        (BrZ.AkZ().FZeile[Anf] == '['))
      KlZ--;
    if ((BrZ.AkZ().FZeile[Anf] == ')') ||
        (BrZ.AkZ().FZeile[Anf] == ']'))
      KlZ++;
    Anf--;
  }
  if (End && (BrZ.AkZ().FZeile[End] == '='))
  {
    End--;
    while (End && BrZ.AkZ().FZeile[End] == ' ')
      End--;
  }

  if ((BrZ.AkZ().FZeile[End]) &&
      ((BrZ.AkZ().FZeile[End] == ']') || (BrZ.AkZ().FZeile[End] == ')')))
    End++;

  if (Anf && (BrZ.AkZ().FZeile[Anf] == '='))
  {
    Anf++;
    while (BrZ.AkZ().FZeile[Anf] && BrZ.AkZ().FZeile[Anf] == ' ')
      Anf++;
  }
}

bool TEFEdit::ScrollWithCursor(wxPoint Pt)
{
  bool TimeOut = false;
  if (Pt.y < 0)
  {
    OnVScroll(wxEVT_SCROLLWIN_LINEUP,0);
    m_sw.Start();
    Update();
    TimeOut = true;
  }
  else
  if (Pt.y > GetClientSize().y)
  {
    OnVScroll(wxEVT_SCROLLWIN_LINEDOWN,0);
    m_sw.Start();
    Update();
    TimeOut = true;
  }
  if (Pt.x < X0)
  {
    OnHScroll(wxEVT_SCROLLWIN_LINEUP,0);
    m_sw.Start();
    Update();
    TimeOut = true;
  }
  else
  if (Pt.x > GetClientSize().x)
  {
    OnHScroll(wxEVT_SCROLLWIN_LINEDOWN,0);
    m_sw.Start();
    Update();
    TimeOut = true;
  }
  return TimeOut;
}


// Initialize the editor
void TEFEdit::ClearEditor()
{
  LastMidZ = -1; // no bracket size information available
  m_Markiert.Set(TextPos(0,1),TextPos(0,1));
  CaretXPos = X0-1; // Caret position
  CaretYPos = 0;
  UpdateAll = true; // redraw everything
  FirstVisZeile = 1;
  MaxZeilen = 1;
  XAnf = 0;
  FirstVisPos = 0;
  AkLen = 0;
  modified = false;
  Capt = false;
  timerlin = 0;
  BrZ.ResetKette();
  BrZ.NeuAkEint();
  BrZ.NeuAkEint();
  Buffer.SetStr(wxT(""));
  Pt = wxPoint(0,0);
  Rueck.Clear();
  VisYMax = 1;
  VisXMax = 0;
  Zoom = 1; // Calculated zoom factor, from font size difference
  OrgZoom = 1; // Originally set zoom factor
  Printing = false;
  ColorPrint = false;
}

bool TEFEdit::SetFont(const wxFont &font)
{
  wxWindow::SetFont(font);
  GetFontMetrics(FontMetric,font);
  GetDC();
  DelDC();
  SetZoom(OrgZoom); // Recalculate zoom for new font
  if (!FirstFont)
    UpdateScrollPos(XAnf);
  FirstFont = false;
  if ((FindFocus() == this))
    InitCaret();
  return true;
}

bool TEFEdit::DeleteSelection()
{
  MGPStr Zeile;
  return GetSelText(Zeile,true);
}

bool TEFEdit::SetSelection(unsigned int A,unsigned int E,bool Scroll)
{
  unsigned int len = 0,len1;
  TextPos NeuAnf = m_Markiert.GetAnf(),NeuEnd = m_Markiert.GetEnd();

  BrZ.Begin();
  while (BrZ.NextEint())
  {
    len1 = len;
    len += BrZ.AkZ().FZeile.FLength()+2;
    if ((len1 <= A) && (len > A))
    {
      NeuAnf.Zeile = BrZ.AkEintNummer();
      NeuAnf.Pos = A-len1;
    }
    if ((len1 <= E) && (len > E))
    {
      NeuEnd.Zeile = BrZ.AkEintNummer();
      NeuEnd.Pos = E-len1;
    }
  }
  m_Markiert.Set(NeuAnf,NeuEnd);
  if (Scroll && IsShown())
    ScrollToCaret();
  // true, to erase background (for CaretPosMarker in X-Pos 0 !)
  EFInvalidate(true);
  ShowCaretPos();
  return true;
}

void TEFEdit::GetSelection(unsigned int &A,unsigned int &E)
{
  int len = 0,len1;

  BrZ.Begin();
  while (BrZ.NextEint())
  {
    len1 = len;
    len += BrZ.AkZ().FZeile.FLength()+2;
    if (m_Markiert.GetAnf().Zeile == BrZ.AkEintNummer())
      A = len1+m_Markiert.GetAnf().Pos;
    if (m_Markiert.GetEnd().Zeile == BrZ.AkEintNummer())
      E = len1+m_Markiert.GetEnd().Pos;
  }
  if (A > E)
  { // sort increasing
    unsigned int tmp = A;
    A = E;
    E = tmp;
  }
}

unsigned int TEFEdit::GetLineIndex(int line)
{
  unsigned int len = 0,len1;

  BrZ.Begin();
  while (BrZ.NextEint())
  {
    len1 = len;
    len += BrZ.AkZ().FZeile.FLength()+2;
    if (line == -1)
    {
      if (BrZ.AkEintNummer() == m_Markiert.GetEnd().Zeile)
        return len1;
    }
    else
    if (line+1 == BrZ.AkEintNummer())
      return len1;
  }
  return 0;
}

int TEFEdit::GetLineFromPos(int pos)
{
  unsigned int len = 0,len1;

  BrZ.Begin();
  if (pos != -1)
  { // if valid index
    while (BrZ.NextEint())
    {
      len1 = len;
      len += BrZ.AkZ().FZeile.FLength()+2;
      if ((int(len1) <= pos) && (int(len) > pos))
        return BrZ.AkEintNummer()-1;
    }
    return 0;
  }
  else
  if (m_Markiert.ZeichenMarkiert() || m_Markiert.MehrereZeilen())
  { // if selection available
    m_Markiert.SortPos();
    return m_Markiert.AkPos[0].Zeile-1; // 1. line with selection
  }
  else
    return m_Markiert.GetEnd().Zeile-1; // line with caret
}

void TEFEdit::Insert(const _TCHAR *L,bool Scroll,
                     bool getundo,bool recordmacro)
{
  MGPStr deltext;

  // Simulate a delete press
  if (MyMacroRecorder::Get()->IsRecording() && getundo && recordmacro &&
      (m_Markiert.ZeichenMarkiert() || m_Markiert.MehrereZeilen()) && 
      (m_Mode != DialogStyle))
    MyMacroRecorder::Get()->AddEvent(wxT("CMD_EDITDELETE"),GetId());

  GetSelText(deltext,true,false);
  TextPos NeuAnf = m_Markiert.GetAnf(),NeuEnd = m_Markiert.GetEnd();

  if (MyMacroRecorder::Get()->IsRecording() && getundo && recordmacro &&
      (m_Mode != DialogStyle))
    MyMacroRecorder::Get()->InsertString(L,GetId(),NeuEnd.Pos,NeuEnd.Zeile);

  TextPos tmpp(NeuEnd);
  BrZ.SetAkEint(NeuEnd.Zeile);
  int AEndZeile = NeuEnd.Zeile;
  if (IstPagebreak() && !_tcscmp(L,wxT(LNBREAK)))
  { // treat LNBREAK special after page break
    if (BrZ.GetMaxEintrag() == BrZ.AkEintNummer())
      BrZ.NeuAkEint();
    else
    {
      BrZ.NextEint();
      BrZ.InsAkEint();
    }
    NeuEnd.Zeile++;
    L += _tcslen(L); // ignore LNBREAK
  }
  else
  {
    if (IstPagebreak() && *L)
      BrZ.AkZ().FZeile.SetStr(wxT("")); // delete page break
    const _TCHAR *brk = _tcsstr(L,wxT(LNBREAK));
    bool lnbreak = (brk != 0);
    MGPStr tmp(&BrZ.AkZ().FZeile(NeuEnd.Pos));
    if (lnbreak)
      BrZ.AkZ().FZeile.FShortStr(NeuEnd.Pos);
    while (brk)
    {
      BrZ.AkZ().FZeile.AddNChar(L,(unsigned int)(brk-L));
      if (BrZ.GetMaxEintrag() == BrZ.AkEintNummer())
        BrZ.NeuAkEint();
      else
      {
        BrZ.NextEint();
        BrZ.InsAkEint();
      }
      NeuEnd.Zeile++;
      NeuEnd.Pos = 0;
      L += (unsigned int)(brk-L);
      L += 2; // ignore LNBREAK
      brk = _tcsstr(L,wxT(LNBREAK)); // determine position of next LNBREAK
    }
    if (lnbreak)
      BrZ.AkZ().FZeile.SetStr(tmp);
  }
  if (*L != 0)
  {
    BrZ.AkZ().FZeile.FInsStr(NeuEnd.Pos,L);
    FormatStrObj tmp;
    tmp.SetStr(L);
    NeuEnd.Pos += tmp.FLength();
  }
  if (getundo)
    Rueck.NewUndo(tmpp,NeuEnd,tmpp,deltext);
  NeuAnf = NeuEnd;

  m_Markiert.Set(NeuAnf,NeuEnd);
  GetDC();
  UpdateZeilen(AEndZeile,NeuEnd.Zeile,true);
  DelDC();
  modified = true;
  SendModified();
  UpdateScrollPos(XAnf);
  if (Scroll && IsShown())
    ScrollToCaret();
  EFInvalidate();
}

unsigned int TEFEdit::GetTextLen()
{
  unsigned int len = 0;

  BrZ.Begin();
  while (BrZ.NextEint())
  {
    len += BrZ.AkZ().FZeile.FLength();
    if (BrZ.AkEintNummer() < BrZ.GetMaxEintrag())
      len += 2;
  }
  return len;
}

void TEFEdit::Cut()
{
  wxDataObjectComposite *cmp;
  if (wxTheClipboard->Open())
  {
    wxTheClipboard->Clear();
    cmp = new wxDataObjectComposite;
    MetafileToClipboard(*cmp);
    if (SelToClipboard(true,*cmp))
    {
      modified = true;
      SendModified();
      if (FirstVisZeile > BrZ.GetMaxEintrag())
        SetFirstVisZeile(BrZ.GetMaxEintrag());
      ScrollToCaret();
      EFInvalidate();
    }
    wxTheClipboard->SetData(cmp);
    wxTheClipboard->Close();
  }
}

const MGPStr &TEFEdit::GetLine(int N)
{
  BrZ.SetAkEint(N+1);
  return BrZ.AkZ().FZeile;
}

int TEFEdit::Scroll(int x,int y)
{
  int FirstVis = FirstVisZeile + y;
  int ret = SetFirstVisZeile(FirstVis,true);
  if (x)
  {
    int tmp = (XAnf+x >= 0 ? x : -XAnf);
    XAnf += tmp;
    ScrollWindowPos(tmp,0);
  }
  return ret;
}

void TEFEdit::InitCaret()
{
  if (GetCaret())
  {
    HideCaret();
    if (InsertMode)
      GetCaret()->SetSize(2,FHeight*COUNTSPERLINE*5/6);
    else
      GetCaret()->SetSize(AveChrW,FHeight*COUNTSPERLINE*5/6);
    if (GetCaret())
      GetCaret()->Move(CaretXPos,CaretYPos);
    ShowCaret();
  }
}

// Copy current selection into clipboard in EMF format
void TEFEdit::MetafileToClipboard(wxDataObjectComposite &cmp)
{
#ifdef __WXMSW__
  int AFHeight = FHeight,AAveW = AveChrW;
  float AZoom = OrgZoom;
  OrgZoom = 1;
  GetDC();
  FontMetricStruct ATm = FontMetric; // save original font properties
  FontMetric.OrgHeight *= 2540/PixPerInchY; // font height in MM_HIMETRIC
  SetFormatFont(TEFTextCalc::Normal,0,true);
  GetFontMetrics(FontMetric,*WFont);
  AveChrW = FontMetric.OrgAveCharW;
  FHeight = int((FontMetric.OrgHeight*FSizeFak+1)/COUNTSPERLINE);
  // Determine unzoomed line heights
  UpdateZeilen(m_Markiert.AkPos[0].Zeile,m_Markiert.AkPos[1].Zeile);
  DelDC();
  MGPStr Zeile;
  wxEnhMetaFileDC dc;
  dc.SetTextForeground(wxColour(0,0,0));
  GetDC(); // get client DC and create fonts
  wxDC *saveddc = DC; // save client DC
  DC = &dc;
  if (WFont)
    DC->SetFont(*WFont);
  m_Markiert.Count = 2; // make sure caret is unchanged
  int AkPos = 0;
  // transparency mode so that lines are not overwritten by chars
  dc.SetBackgroundMode(wxTRANSPARENT);
  int AAkLen = AkLen; // save current max line length
  int AX = XAnf;
  XAnf = X0; // Distance from left border
  for (int i = m_Markiert.AkPos[0].Zeile;i <= m_Markiert.AkPos[1].Zeile;i++)
  { // Paint selected lines
    BrZ.SetAkEint(i);
    PaintZeile(AkPos);
  }
  // Draw dummy point to slightly increase rectangle
  // to avoid cutoff of chars at the outline
  dc.SetPen(*wxWHITE_PEN);
  dc.DrawPoint(0,AkPos);
  dc.SetPen(wxNullPen);
  if (WFont)
    dc.SetFont(wxNullFont);
  wxEnhMetaFile *mf = dc.Close();
  DC = saveddc;
  DelDC();
  if (mf)
  {
    wxEnhMetaFileSimpleDataObject *data;
    data = new wxEnhMetaFileSimpleDataObject(*mf);
    cmp.Add(data,true);
    delete mf;
  }
  AveChrW = AAveW;
  FHeight = AFHeight;
  OrgZoom = AZoom;
  FontMetric = ATm;
  GetDC();
  // Restore original line lengths
  UpdateZeilen(m_Markiert.AkPos[0].Zeile,m_Markiert.AkPos[1].Zeile);
  AkLen = AAkLen; // restore old max line length
  DelDC();
  XAnf = AX;
#endif
}

bool TEFEdit::SelToClipboard(bool erase,wxDataObjectComposite &cmp,bool addbrackets)
{
  bool ret = false;
  FormatStrObj Zeile;
  bool oldmacro = m_Markiert.SetMacroOut(false);
  if (GetSelText(Zeile,erase))
  {
    MGPStr AZeile;
    Zeile.GetTextStr(AZeile);
    wxString tmp;
    if (addbrackets)
    {
      tmp = wxT("(");
      tmp += AZeile.c_str();
      tmp += wxT(")");
    }
    else
      tmp = AZeile.c_str();
    wxTextDataObject *data = new wxTextDataObject(tmp);
    cmp.Add(data,true);
    wxCustomDataObject *cdo = new wxCustomDataObject(wxGetApp().editformat);
    char t = 0;
    cdo->SetData(1,&t);
    cmp.Add(cdo);
    ret = true;
  }
  m_Markiert.SetMacroOut(oldmacro);
  return ret;
}

bool TEFEdit::Paste()
{
  bool ret = false;
  bool oldmacro = m_Markiert.SetMacroOut(false);
  if (wxTheClipboard->Open())
  {
    if (wxTheClipboard->IsSupported(wxDF_TEXT))
    { // Text in the Clipboard
      wxTextDataObject data;
      wxTheClipboard->GetData(data);
      wxString tmp(data.GetText().c_str());
#ifdef __WXGTK__
      // FIXME: LINUX version should work with \n natively
      tmp.Replace(wxT("\n"),wxT(LNBREAK),true);
#endif
      Insert(tmp.c_str(),true,true,false);
      ret = true;
    }
    wxTheClipboard->Close();
  }
  m_Markiert.SetMacroOut(oldmacro);
  return ret;
}

void TEFEdit::Copy(bool addbrackets)
{
  wxDataObjectComposite *cmp;
  if (wxTheClipboard->Open())
  {
    wxTheClipboard->Clear();
    cmp = new wxDataObjectComposite;
    MetafileToClipboard(*cmp);
    SelToClipboard(false,*cmp,addbrackets);
    wxTheClipboard->SetData(cmp);
    wxTheClipboard->Close();
  }
}

const _TCHAR *TEFEdit::LockBuffer(int)
{
  MGPStr Zeile;

  Buffer.SetStr(wxT(""));
  BrZ.Begin();
  while (BrZ.NextEint())
  {
    BrZ.AkZ().FZeile.GetTextStr(Zeile);
    Buffer.AddStr(Zeile);
    if (BrZ.AkEintNummer() < BrZ.GetMaxEintrag())
      Buffer.AddStr(wxT(LNBREAK));
  }
  return Buffer.c_str();
}

//
// Similar to strstr(), but is case sensitive or insensitive, uses Windows
// string functions to work with different language drivers
//
static const _TCHAR* strstrcd(const _TCHAR* str1, const _TCHAR* str2, bool caseSens)
{
  int len2 = _tcslen(str2);
  _TCHAR* p = (_TCHAR*)str1;
  const _TCHAR* endp = str1 + _tcslen(str1) - len2 + 1;

  if (caseSens)
    while (p < endp)
    {
      _TCHAR c = p[len2];            // must term p to match str2 len
      p[len2] = 0;                 // for strcmp to work.
      if (_tcscmp(p, str2) == 0)
      {
        p[len2] = c;
        return p;
      }
      p[len2] = c;
      p++;
    }
  else
    while (p < endp)
    {
      _TCHAR c = p[len2];
      p[len2] = 0;
      if (MGStricmp(p, str2) == 0)
      {
        p[len2] = c;
        return p;
      }
      p[len2] = c;
      p++;
    }
  return 0;
}

//
// Similar to strstrcd(), but searches backwards. Needs the length of str1
// to know where to start.
//
static const _TCHAR* strrstrcd(const _TCHAR* str1, unsigned int len1,
                             const _TCHAR* str2,bool caseSens)
{
  int len2 = _tcslen(str2);
  _TCHAR* p = (_TCHAR*)(str1 + len1 - len2);

  if (caseSens)
    while (p >= str1)
    {
      _TCHAR c = p[len2];            // must term p to match str2 len
      p[len2] = 0;                 // for strcmp to work.
      if (_tcscmp(p, str2) == 0)
      {
        p[len2] = c;
        return p;
      }
      p[len2] = c;
      p--;
    }
  else
    while (p >= str1)
    {
      _TCHAR c = p[len2];
      p[len2] = 0;
      if (MGStricmp(p, str2) == 0)
      {
        p[len2] = c;
        return p;
      }
      p[len2] = c;
      p--;
    }
  return 0;
}

//
// searchs for and selects the given text and returns the offset of the text
// or -1 if the text is not found
//
// if "startPos" = -1 then it is assumed that the start pos is the
// end/beginning (depending on direction) of the current selection
//
int TEFEdit::Search(int startPos, const _TCHAR* text, bool caseSensitive,
              bool wholeWord, bool up)
{
  if (!text || !text[0])
    return -1;

  if (startPos == -1)
  {
    unsigned int sBeg, sEnd;
    GetSelection(sBeg, sEnd);
    startPos = up ? sBeg : sEnd;
  }
  unsigned int textLen = _tcslen(text);

  //
  // Lock the text buffer to get the pointer, and search thru it for the text
  //
  const _TCHAR* buffer = LockBuffer();
  const _TCHAR* pos;
  for (;;)
  {
    if (up)
      pos = strrstrcd(buffer, startPos, text, caseSensitive);
    else
      pos = strstrcd(buffer+startPos, text, caseSensitive);

    //
    // If whole-word matching is enabled and we have a match so far, then make
    // sure the match is on word boundaries.
    //
    if (wholeWord && pos)
    {
      if (pos > buffer && _istalnum(pos[-1]) || // Match is in preceding word
          textLen < _tcslen(pos) && _istalnum(pos[textLen]))
      {
        startPos = (unsigned int)(pos-buffer) + !up;
        continue;  // Skip this match and keep searching
      }
    }
    break;  // Out of for loop
  }

  // If we've got a match, select that text, cleanup & return.
  //
  if (pos)
  {
    unsigned int sBeg = (unsigned int)(pos - buffer);
    UnlockBuffer(buffer);
    SetSelection(sBeg, sBeg + textLen);

    return sBeg;
  }
  UnlockBuffer(buffer);
  return -1;
}


