// Version 6.06  29.11.2009 Markus Greither
// Copyright (c) 1991-2009 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file efedevts.cpp
    \brief Source code for the function editor event handling
*/


#include "efprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#include <wx/xrc/xmlres.h>

#ifndef WX_PRECOMP
#ifdef __DMC__
#include <typeinfo.h>
#endif
#include <wx/wx.h>
#include <wx/resizec.h>

#include <wx/clipbrd.h>

#include <complex>
#include <limits>
#include <climits>
#include <set>
#include <functional>
#if defined(__MINGW32__)
#include <wx/msw/winundef.h>
#endif

using namespace std;

#include "efdefs.h"
#include "mg.h"
#include "cpxpsr.h"
#include "diagramm.h"
#include "efgemein.h"
#include "ef.h"
#include "efresid.h"
#include "efedit.h"

#endif

#include "efsprexp.h"
#include "efeddnd.h" // local include because problem for prec. headers
#include "efmacro.h"

// ----------------------------------------------------------------------------
// TextSelection
// ----------------------------------------------------------------------------

void SortedTextSelection::Set(const TextPos &A,const TextPos &E,bool tst)
{
  TEFEdit *edit = wxDynamicCast(m_Wnd,TEFEdit);
  if (tst && m_Wnd && m_MacroOut && MyMacroRecorder::Get()->IsRecording() &&
      ((Anf != A) || (End != E)) && (edit->GetEditorMode() != TEFEdit::DialogStyle))
  {
    MyMacroRecorder::Get()->AddEvent(wxT("CMD_SETEDITSELECTION"),m_Wnd->GetId());
    MyMacroRecorder::Get()->AddPos(A.Pos,A.Zeile,E.Pos,E.Zeile);
  }
  TextSelection::Set(A,E);
}

// ----------------------------------------------------------------------------
// TEFEdit
// ----------------------------------------------------------------------------

bool TEFEdit::ProcessEvent(wxEvent& event)
{
  bool ret = TEFTextCalc::ProcessEvent(event);

  if (ret && (m_Mode != DialogStyle))
    MyMacroRecorder::Get()->AddEvent(event,GetId());

  return ret;
}

void TEFEdit::OnRButtonUp(wxMouseEvent &event)
{
  if (m_Mode = FindReplaceStyle)
  {
    wxPoint hp;
    hp = event.GetPosition();
    if (FindFocus() == this)
    {
      wxMenu *PMenue = wxGetApp().CreateCalcMenu(true);
      PopupMenu(PMenue,hp);
      delete PMenue;
    }
    else
      event.Skip();
  }
  else
  {
    wxMenu *menu = wxXmlResource::Get()->LoadMenu(wxT("menuEquations"));
    if (m_Mode == EquationStyle)
    {
      wxMenu *c_menu = wxXmlResource::Get()->LoadMenu(wxT("menuCalc"));
      wxMenuItemList &lst = c_menu->GetMenuItems();
      wxMenuItem *item;
      menu->AppendSeparator();
      for (int i = 4;i < 7;i++)
      {
#ifdef __WXMSW__
        wxString label = lst[i]->GetItemLabel();
#else
        wxString label = lst[i]->GetLabel();
#endif
        item = new wxMenuItem(menu,lst[i]->GetId(),
                              label,
                              lst[i]->GetHelp());
        menu->Append(item);
      }
      delete c_menu;
      menu->AppendSeparator();
      item = new wxMenuItem(menu,XRCID("ICM_DELEDIT"),
                            _("&Delete Window"),
                            _("Delete this window"));
      menu->Append(item);
    }
    wxPoint hp = event.GetPosition();
    PopupMenu(menu,hp);
    MenuCursor = false;
    delete menu;
    if (DelWin)
    { // verzoegertes Loeschen des Fensters, damit PopupMenu sicher beendet
      DelWin = false; // Probleme nach Veto vermeiden
      wxCommandEvent cevent;
      OnDelEdit(cevent);
    }
  }
}

void TEFEdit::OnDelEdit(wxCommandEvent &)
{
  if (!MenuCursor)
  {
    wxCommandEvent cevent(wxEVT_COMMAND_MENU_SELECTED,XRCID("ICM_DELWINDOW"));
    // only applies to equation style windows, with TEFEquationControl parent
    cevent.SetEventObject(GetParent());
    if (GetParent())
    {
      GetParent()->SetFocus();
      // Append command to view, crash when sent to immediate parent
      if (GetParent()->GetParent())
        GetParent()->GetParent()->GetEventHandler()->AddPendingEvent(cevent);
    }
  }
  else
    DelWin = true;
}

void TEFEdit::OnTimer()
{
  if (!ValueWin)
  {
    wxPoint tmp = ScreenToClient(wxGetMousePosition());
     wxWindowListNode *child = GetChildren().GetFirst();
     bool inchild = false;
     while (child)
     { // check for mouse position in child window
       wxWindow *wnd = (wxWindow *)child->GetData();
       wxRect rect = wnd->GetRect();
       inchild |= ((tmp.y >= rect.GetTop()) && (tmp.y <= rect.GetBottom()) &&
                   (tmp.x >= rect.GetLeft()) && (tmp.x <= rect.GetRight()));
       child = child->GetNext();
    }
    if ((m_Mode != DialogStyle) && (FindFocus() == this) &&
        (tmp.y >= 0) && (tmp.y < GetClientSize().y) &&
        (tmp.x >= 0) && (tmp.x < GetClientSize().x) &&
        (timerlin == aktimerlin) && !inchild)
    {
      ValueWinPos = tmp;
      wxCommandEvent event(wxEVT_COMMAND_MENU_SELECTED,
                           XRCID("ICM_WERTBERECHNEN"));
      event.SetInt(timerlin);
      event.SetEventObject(this);
      GetParent()->GetEventHandler()->ProcessEvent(event);
      IdleTimer.Start(5000,true);
    }
  }
  else
  {
    ValueWin->Destroy();
    ValueWin = 0;
  }
}

void TEFEdit::OnChar(wxKeyEvent &event)
{ // Tastatur-Eingaben
  if (event.AltDown())
  {
    event.Skip();
    return;
  }
  if ((event.GetKeyCode() >= 32)  && (event.GetKeyCode() < 256))
  {
    TextPos NeuAnf = m_Markiert.GetAnf(),NeuEnd = m_Markiert.GetEnd();

    int Min = -BrZ.AkZ().ObZ[1],Max = BrZ.AkZ().UntZ[1];
    UpdateAll = false;
    if (m_Markiert.ZeichenMarkiert() || m_Markiert.MehrereZeilen())
    {
      // Simulate a delete press
      if (MyMacroRecorder::Get()->IsRecording() && (m_Mode != DialogStyle))
        MyMacroRecorder::Get()->AddEvent(wxT("CMD_EDITDELETE"),GetId());
      DeleteSelection();
      NeuAnf = m_Markiert.GetAnf();
      NeuEnd = m_Markiert.GetEnd();
      UpdateAll = true;
    }
    BrZ.SetAkEint(NeuEnd.Zeile);
    if (!IstPagebreak())
    { // Seitenumbruch-Zeile nichts eingeben
      _TCHAR DelChar = 0;
      if (InsertMode) // Einfuege-Modus
        BrZ.AkZ().FZeile.FInsChar(NeuEnd.Pos,_TCHAR(event.GetKeyCode()));
      else
      { // Ueberschreib-Modus
        if (NeuEnd.Pos < BrZ.AkZ().FZeile.FLength())
        {
          DelChar = BrZ.AkZ().FZeile(NeuEnd.Pos);
          BrZ.AkZ().FZeile.FReplChar(NeuEnd.Pos,_TCHAR(event.GetKeyCode()));
        }
        else
          BrZ.AkZ().FZeile.AddChar(_TCHAR(event.GetKeyCode()));
      }
      Rueck.UpdateInsert(NeuEnd,
                         TextPos(NeuEnd.Pos+1,NeuEnd.Zeile),
                         DelChar);
      NeuEnd.Pos++;
      NeuAnf.Pos = NeuEnd.Pos;
      GetDC();
      UpdateZeilen(NeuEnd.Zeile,NeuEnd.Zeile,true);
      DelDC();
      m_Markiert.Set(NeuAnf,NeuEnd,false);
      if (MyMacroRecorder::Get()->IsRecording() && (m_Mode != DialogStyle))
        MyMacroRecorder::Get()->InsertChar(event.GetKeyCode(),GetId(),
                                           NeuEnd.Pos,NeuEnd.Zeile);
      ScrollToCaret();
      if ((Min == -BrZ.AkZ().ObZ[1]) && (Max == BrZ.AkZ().UntZ[1]) &&
          (UpdateAll == false))
      {
        RePaintAkZeile();
      }
      else
        EFInvalidate();
      modified = true;
      SendModified();
    }
    else
      wxBell();
  }
}

void TEFEdit::CeEditUndo(wxUpdateUIEvent& ce)
{
  ce.Enable((FindFocus() == this) && Rueck.UndoAvailable());
}

void TEFEdit::CeEditRedo(wxUpdateUIEvent& ce)
{
  ce.Enable((FindFocus() == this) && Rueck.RedoAvailable());
}

void TEFEdit::OnFontHeight(wxCommandEvent &)
{
  wxString FontSizes[] =
  {
    wxT("8"), wxT("9"),wxT("10"),wxT("11"),wxT("12"),
   wxT("14"),wxT("16"),wxT("18"),wxT("20"),wxT("22"),
   wxT("24"),wxT("26"),wxT("28"),wxT("36"),wxT("42"),
   wxT("71"),wxT("\0")
  };
  wxString tmp;
  tmp.Printf(wxT("%i"),FontMetric.FPoints);
  int i = 0,ind = 0;
  while (FontSizes[i] != wxT("\0"))
  {
    if (wxStrcmp(FontSizes[i],tmp) == 0)
      ind = i;
    i++;
  }
  wxSingleChoiceDialog dlg(wxGetApp().GetTopWindow(),
                           _("Font height for selected text :"),
                           wxT("Easy Funktion"),i,FontSizes);
  dlg.SetSelection(ind);
  int ret = dlg.ShowModal();
  if (ret == wxID_OK)
  {
    wxString erg = dlg.GetStringSelection();
    long H;
    erg.ToLong(&H);
    SetSelectionFormat(TEFTextCalc::FontHeight,H);
    if (MyMacroRecorder::Get()->IsRecording() && (m_Mode != DialogStyle))
    {
      MyMacroRecorder::Get()->AddEvent(wxT("CMD_FONTHEIGHT"),GetId());
      EFXmlNode *node = MyMacroRecorder::Get()->GetAkNode();
      node->AddProperty(wxT("points"),erg.c_str());
    }  
  }
}

void TEFEdit::OnEditUndo(wxCommandEvent &)
{
  UndoRedo(false);
}

void TEFEdit::OnEditRedo(wxCommandEvent &)
{
  UndoRedo(true);
}

void TEFEdit::OnMousewheel(wxMouseEvent &event)
{ // Maus-Rad gedreht
  static int AkDelta = 0;

  AkDelta += event.GetWheelRotation();
  int delta = event.GetWheelDelta();
  if (!delta)
    delta = 1;
  int lines = AkDelta / delta;
  AkDelta -= lines * delta;
  if (lines != 0)
  {
    lines = FirstVisZeile-lines*event.GetLinesPerAction();
    if (lines > BrZ.GetMaxEintrag())
      lines = BrZ.GetMaxEintrag();
    if (lines < 1)
      lines = 1;
    OnVScroll(wxEVT_SCROLLWIN_THUMBRELEASE,lines-1);
  }
}

void TEFEdit::OnKeyDown(wxKeyEvent &event)
{ // Funktionstasten-Auswertung
  const _TCHAR *tmp;
  bool erase = false;
  int p;

  if (ValueWin)
  {
    ValueWin->Destroy();
    ValueWin = 0;
    IdleTimer.Stop();
  }
  if (event.AltDown())
  {
    event.Skip();
    return;
  }  
  if (m_Mode == DialogStyle)
  { // Wenn Dialog-Child
    if (event.ControlDown())
    { // CTRL ist gedrueckt
      switch (event.GetKeyCode())
      {
        case 'c':
        case 'C': Copy(); break;
        case 'x':
        case 'X': Cut(); break;
        case 'v':
        case 'V': Paste(); break;
      }
      return;
    }
  }
  TextPos NeuAnf = m_Markiert.GetAnf(),NeuEnd = m_Markiert.GetEnd();

  BrZ.SetAkEint(NeuEnd.Zeile);
  m_Markiert.AltPos = NeuEnd;
  UpdateAll = false;
  bool scrolled = false;
  switch (event.GetKeyCode())
  {
    case WXK_F5 :
      Refresh();
      return;
    case WXK_TAB :
      if ((m_Mode == EquationStyle) && GetParent())
        GetParent()->SetFocus();
      else
      if ((m_Mode == FunktionStyle) || (m_Mode = FindReplaceStyle))
      {
        wxWindowListNode *first = GetChildren().GetFirst();
        while (first && !first->GetData()->IsShown())
          first = first->GetNext();
        if (first)
          first->GetData()->SetFocus();
      }
      return;
    case WXK_INSERT :
      InsertMode = !InsertMode;
      InitCaret();
      UpdateAll = true;
      return;
    case WXK_UP :
      if (!event.ControlDown())
      {
        if (NeuEnd.Zeile > 1)
        {
          NeuEnd.Zeile--;
          BrZ.SetAkEint(NeuEnd.Zeile);
          if (NeuEnd.Pos > BrZ.AkZ().FZeile.FLength())
              NeuEnd.Pos = BrZ.AkZ().FZeile.FLength();
        }
        if (!event.ShiftDown())
        {
          UpdateAll = m_Markiert.AltPos.Zeile != NeuAnf.Zeile;
          NeuAnf = NeuEnd;
        }
      }
      else
        ChangeFontSize(true);
      break;
    case WXK_DOWN :
      if (!event.ControlDown())
      {
        if (NeuEnd.Zeile < BrZ.GetMaxEintrag())
        {
          NeuEnd.Zeile++;
          BrZ.SetAkEint(NeuEnd.Zeile);
          if (NeuEnd.Pos > BrZ.AkZ().FZeile.FLength())
            NeuEnd.Pos = BrZ.AkZ().FZeile.FLength();
        }
        if (!event.ShiftDown())
        {
          UpdateAll = m_Markiert.AltPos.Zeile != NeuAnf.Zeile;
          NeuAnf = NeuEnd;
        }
      }
      else
        ChangeFontSize(false);
      break;
    case WXK_RIGHT :
      if (NeuEnd.Pos < BrZ.AkZ().FZeile.FLength())
        NeuEnd.Pos++;
      else
      if (NeuEnd.Zeile < BrZ.GetMaxEintrag())
      {
        NeuEnd.Zeile++;
        NeuEnd.Pos = 0;
      }
      if (!event.ShiftDown())
      {
        UpdateAll = m_Markiert.AltPos.Zeile != NeuAnf.Zeile;
        NeuAnf = NeuEnd;
      }
      break;
    case WXK_LEFT:
      if (NeuEnd.Pos > 0)
        NeuEnd.Pos--;
      else
      if (NeuEnd.Zeile > 1)
      {
        NeuEnd.Zeile--;
        BrZ.SetAkEint(NeuEnd.Zeile);
        NeuEnd.Pos = BrZ.AkZ().FZeile.FLength();
      }
      if (!event.ShiftDown())
      {
        UpdateAll = m_Markiert.AltPos.Zeile != NeuAnf.Zeile;
        NeuAnf = NeuEnd;
      }
      break;
    case WXK_BACK :
      if (m_Markiert.ZeichenMarkiert() || m_Markiert.MehrereZeilen())
      {
        DeleteSelection();
        NeuAnf = m_Markiert.GetAnf();
        NeuEnd = m_Markiert.GetEnd();
        erase = true;
        UpdateAll = true;
      }
      else
      if (NeuEnd.Pos > 0)
      {
        int Min = BrZ.AkZ().ObZ[1],Max = BrZ.AkZ().UntZ[1];
        TextPos tmpp(NeuEnd);
        NeuEnd.Pos--;
        Rueck.UpdateDelete(NeuEnd,tmpp,
                           BrZ.AkZ().FZeile(NeuEnd.Pos));
        BrZ.AkZ().FZeile.FDelNChar(NeuEnd.Pos,1);
        GetDC();
        UpdateZeilen(NeuEnd.Zeile,NeuEnd.Zeile,true);
        DelDC();
        UpdateAll = !((Min == BrZ.AkZ().ObZ[1]) && (Max == BrZ.AkZ().UntZ[1]));
        erase = UpdateAll;
      }
      else
      if (NeuEnd.Zeile > 1)
      {
        tmp = BrZ.AkZ().FZeile.c_str();
        TextPos tmpp(NeuEnd);
        NeuEnd.Zeile--;
        BrZ.SetAkEint(NeuEnd.Zeile);
        if (IstPagebreak())
          BrZ.AkZ().FZeile.SetStr(wxT("")); // Seitenumbruch nicht kopieren
        NeuEnd.Pos = BrZ.AkZ().FZeile.FLength();
        Rueck.UpdateDelete(NeuEnd,tmpp,wxT(LNBREAK));
        BrZ.AkZ().FZeile.AddStr(tmp);
        BrZ.NextEint();
        BrZ.LoeschAkEint();
        GetDC();
        UpdateZeilen(NeuEnd.Zeile,NeuEnd.Zeile,true);
        DelDC();
        erase = true;
        UpdateAll = true;
      }
      modified = true;
      SendModified();
      NeuAnf = NeuEnd;
      if (!UpdateAll)
      {
        m_Markiert.Set(NeuAnf,NeuEnd,false);
        RePaintAkZeile();
      }  
      if (MyMacroRecorder::Get()->IsRecording() && (m_Mode != DialogStyle))
        MyMacroRecorder::Get()->AddEvent(wxT("CMD_EDITBACKSPACE"),GetId());
      break;
    case WXK_DELETE :
      if (m_Markiert.ZeichenMarkiert() || m_Markiert.MehrereZeilen())
      {
        DeleteSelection();
        NeuAnf = m_Markiert.GetAnf();
        NeuEnd = m_Markiert.GetEnd();
        erase = true;
        UpdateAll = true;
      }
      else
      if (NeuEnd.Pos < BrZ.AkZ().FZeile.FLength())
      {
        int Min = BrZ.AkZ().ObZ[1],Max = BrZ.AkZ().UntZ[1];
        Rueck.UpdateDelete(NeuEnd,NeuEnd,
                           BrZ.AkZ().FZeile(NeuEnd.Pos));
        BrZ.AkZ().FZeile.FDelNChar(NeuEnd.Pos,1);
        GetDC();
        UpdateZeilen(NeuEnd.Zeile,NeuEnd.Zeile,true);
        DelDC();
        UpdateAll = !((Min == BrZ.AkZ().ObZ[1]) && (Max == BrZ.AkZ().UntZ[1]));
        erase = UpdateAll;
      }
      else
      if (NeuEnd.Zeile < BrZ.GetMaxEintrag())
      {
        if (IstPagebreak())
          BrZ.AkZ().FZeile.SetStr(wxT("")); // Seitenumbruch loeschen
        FormatStrObj *Ak = &BrZ.AkZ().FZeile;
        Rueck.UpdateDelete(NeuEnd,NeuEnd,wxT(LNBREAK));
        BrZ.NextEint();
        if (!(IstPagebreak()))
          Ak->AddStr(BrZ.AkZ().FZeile); // Seitenumbruch nicht kopieren
        BrZ.LoeschAkEint();
        GetDC();
        UpdateZeilen(NeuEnd.Zeile,NeuEnd.Zeile,true);
        DelDC();
        erase = true;
        UpdateAll = true;
      }
      modified = true;
      SendModified();
      NeuAnf = NeuEnd;
      if (!UpdateAll)
      {
        m_Markiert.Set(NeuAnf,NeuEnd,false);
        RePaintAkZeile();
      }  
      if (MyMacroRecorder::Get()->IsRecording() && (m_Mode != DialogStyle))
        MyMacroRecorder::Get()->AddEvent(wxT("CMD_EDITDELETE"),GetId());
      break;
    case WXK_RETURN :
    {
      if (m_Markiert.ZeichenMarkiert() || m_Markiert.MehrereZeilen())
      {
        DeleteSelection();
        NeuAnf = m_Markiert.GetAnf();
        NeuEnd = m_Markiert.GetEnd();
        UpdateAll = true;
      }
      FormatStrObj *tmp1 = &BrZ.AkZ().FZeile;
      TextPos tmpp(NeuEnd);
      NeuEnd.Zeile++;
      Rueck.UpdateInsert(tmpp,TextPos(0,NeuEnd.Zeile),0);
      if (NeuEnd.Zeile > BrZ.GetMaxEintrag())
        BrZ.NeuAkEint();
      else
      {
        BrZ.NextEint();
        BrZ.InsAkEint();
      }
      if ((*tmp1)(NeuEnd.Pos) != 0)
      {
        BrZ.AkZ().FZeile.SetStr(&(*tmp1)(NeuEnd.Pos));
        tmp1->FShortStr(NeuEnd.Pos);
      }
      erase = true;
      GetDC();
      UpdateZeilen(NeuEnd.Zeile-1,NeuEnd.Zeile,true);
      DelDC();
      UpdateAll = true;
      modified = true;
      SendModified();
      NeuEnd.Pos = 0;
      NeuAnf = NeuEnd;
      if (MyMacroRecorder::Get()->IsRecording() && (m_Mode != DialogStyle))
        MyMacroRecorder::Get()->AddEvent(wxT("CMD_EDITRETURN"),GetId());
      m_Markiert.Set(NeuAnf,NeuEnd,false);
      break;
    }
    case WXK_PAGEUP :
      p = FirstVisZeile;
      SetFirstVisZeile(FirstVisZeile-PgUpZeilen(FirstVisZeile));
      NeuEnd.Zeile += (FirstVisZeile-p);
      if (NeuEnd.Zeile < 1)
        NeuEnd.Zeile = 1;
      BrZ.SetAkEint(NeuEnd.Zeile);
      if (NeuEnd.Pos > BrZ.AkZ().FZeile.FLength())
        NeuEnd.Pos = BrZ.AkZ().FZeile.FLength();
      erase = true;
      scrolled = true;
      if (!event.ShiftDown())
      {
        UpdateAll = m_Markiert.AltPos.Zeile != NeuAnf.Zeile;
        NeuAnf = NeuEnd;
      }
      else
        UpdateAll = (p != FirstVisZeile);
      break;
    case WXK_PAGEDOWN :
      p = FirstVisZeile;
      SetFirstVisZeile(FirstVisZeile+PgDownZeilen(FirstVisZeile));
      NeuEnd.Zeile += (FirstVisZeile-p);
      if (NeuEnd.Zeile > BrZ.GetMaxEintrag())
        NeuEnd.Zeile = BrZ.GetMaxEintrag();
      BrZ.SetAkEint(NeuEnd.Zeile);
      if (NeuEnd.Pos > BrZ.AkZ().FZeile.FLength())
        NeuEnd.Pos = BrZ.AkZ().FZeile.FLength();
      erase = true;
      scrolled = true;
      if (!event.ShiftDown())
      {
        UpdateAll = m_Markiert.AltPos.Zeile != NeuAnf.Zeile;
        NeuAnf = NeuEnd;
      }
      else
        UpdateAll = (p != FirstVisZeile);
      break;
    case WXK_HOME :
      NeuEnd.Pos = 0;
      if (!event.ShiftDown())
      {
        UpdateAll = m_Markiert.AltPos.Zeile != NeuAnf.Zeile;
        NeuAnf = NeuEnd;
      }
      break;
    case WXK_END :
      NeuEnd.Pos = BrZ.AkZ().FZeile.FLength();
      if (!event.ShiftDown())
      {
        UpdateAll = m_Markiert.AltPos.Zeile != NeuAnf.Zeile;
        NeuAnf = NeuEnd;
      }
      break;
    default :
      event.Skip();
      return;
  }
  m_Markiert.Set(NeuAnf,NeuEnd);
  scrolled |= ScrollToCaret();
  if (UpdateAll)
    EFInvalidate(erase);
  else
  {
    if (scrolled) // updaten und Caret-Position aktualisieren
      Update();
    // Akt. Abschnitt neu zeichnen wg. neuer Caret-Position
    // und beim "Runterrollen" mit Cursortasten, damit letzte Zeile
    // aktualisiert wird
    UpdateAll = false;
    HideCaret();
    GetDC();
    wxRect rect(0,0,GetClientSize().x,GetClientSize().y);
    Paint(*DC,false,rect);
    DelDC();
    ShowCaret();
    SetDocSize();
  }
  if (!Printing && (m_Mode == EquationStyle)) // Caret in sichtbaren Breich rollen
  {
    Update();
    TEFEdit *edit = wxDynamicCast(GetGrandParent(),TEFEdit);
    wxPoint pt1 = ClientToScreen(wxPoint(CaretXPos,CaretYPos+NextAkPos()));
    wxPoint pt = edit->ScreenToClient(pt1);
    HideCaret();
    while (edit && (pt.y - edit->GetClientSize().y > 0))
    {
      pt1 = ClientToScreen(wxPoint(CaretXPos,CaretYPos+NextAkPos()));
      pt = edit->ScreenToClient(pt1);
      edit->Scroll(0,1);
    }
    pt1 = ClientToScreen(wxPoint(CaretXPos,CaretYPos-NextAkPos()));
    pt = edit->ScreenToClient(pt1);
    while (edit && (pt.y < 0) && (edit->GetFirstVisibleLine() > 0))
    {
      pt1 = ClientToScreen(wxPoint(CaretXPos,CaretYPos-NextAkPos()));
      pt = edit->ScreenToClient(pt1);
      edit->Scroll(0,-1);
    }
    ShowCaret();
  }
  ShowCaretPos();
}

void TEFEdit::OnSetFocus(wxFocusEvent &event)
{
  if (!Printing)
  {
    HideCaret();
    GetDC();
    DrawCaretPosMarker(CaretXPos,CaretYPos,true);
    DelDC();
    ShowCaret();
    InitCaret();
    wxCommandEvent event(wxEVT_COMMAND_MENU_SELECTED,XRCID("ICM_EDITSETFOCUS"));
    event.SetEventObject(this);
    GetParent()->GetEventHandler()->ProcessEvent(event);
  }
  ShowCaretPos();
  event.Skip();
}

void TEFEdit::OnKillFocus(wxFocusEvent &event)
{
  if (!Printing)
  {
    GetDC();
    DrawCaretPosMarker(CaretXPos,CaretYPos);
    DelDC();
  }
  if (ValueWin)
  {
    ValueWin->Destroy();
    ValueWin = 0;
  }
  IdleTimer.Stop();
  wxGetApp().sb->SetStatusText(wxT(""),1);
  event.Skip();
}

void TEFEdit::OnLButtonDown(wxMouseEvent &event)
{
  if ((FindFocus() == this) && !Printing)
  {
    TextPos NeuAnf = m_Markiert.GetAnf(),NeuEnd = m_Markiert.GetEnd();

    Pt = wxPoint(event.GetX(),event.GetY());
    DownPos = Pt;
    if (!Capt)
      CaptureMouse();
    Capt = true;
    SortedTextSelection OM = m_Markiert;
    bool oldmo = m_Markiert.SetMacroOut(false);
    OM.SortPos();
    m_Markiert.SortPos();
    int Zeile;
    unsigned int Pos;
    bool links = MousePos(false,Zeile,Pos,true);
    DragMode = ((m_Markiert.ZeichenMarkiert() || m_Markiert.MehrereZeilen()) &&
                !links &&
                ((OM.GetEnd().Zeile == OM.GetAnf().Zeile) && (OM.GetEnd().Zeile == Zeile) &&
                 (Pos >= OM.GetAnf().Pos) && (Pos <= OM.GetEnd().Pos) ||
                 (OM.GetEnd().Zeile != OM.GetAnf().Zeile) &&
                 ((Zeile == OM.AkPos[1].Zeile) && (Pos <= OM.AkPos[1].Pos) ||
                  (Zeile == OM.AkPos[0].Zeile) && (Pos >= OM.AkPos[0].Pos) ||
                  (Zeile < OM.AkPos[1].Zeile) && (Zeile > OM.AkPos[0].Zeile))));
    m_Markiert = OM;
    m_Markiert.SetMacroOut(oldmo);
    if (m_Markiert.DblClk)
      m_Markiert.DblClk--;

    if (DragMode)
      return;

    m_Markiert.AltPos.Zeile = NeuEnd.Zeile;
    // wenn SHIFT gedrueckt alte Anfangspos beibehalten
    bool Links = MousePos(!event.ShiftDown(),Zeile,Pos);
    NeuEnd.Zeile = Zeile;
    NeuEnd.Pos = Pos;
    if (!event.ShiftDown())
      UpdateAll = m_Markiert.AltPos.Zeile != NeuAnf.Zeile;
    else
      UpdateAll = false;
    // wenn SHIFT gedrueckt markieren von vorheriger Pos bis aktueller Pos
    if (!event.ShiftDown() && !event.ControlDown())
      NeuAnf.Zeile = NeuEnd.Zeile;
    if (!Links)
    { // kein Zeilen-Markierungsmodus
      if ((NeuAnf.Zeile == NeuEnd.Zeile) &&
          (m_Markiert.AltPos.Zeile == NeuEnd.Zeile))
        m_Markiert.AltPos.Pos = 0;
      if (!event.ShiftDown())
        NeuAnf.Pos = NeuEnd.Pos;
    }
    else
    // wird Links neben den Text geklickt und CNTL gedrueckt
    // -> gesamter Text markiert
    {
      if (event.ControlDown())
      {
        NeuAnf.SetPos(0,1);
        NeuEnd.Zeile = BrZ.GetMaxEintrag();
        BrZ.SetAkEint(NeuEnd.Zeile);
        NeuEnd.Pos = BrZ.AkZ().FZeile.FLength();
      }
      else
      {
        if (NeuEnd.Zeile < NeuAnf.Zeile)
          NeuEnd.Pos = 0;
        else
          NeuAnf.Pos = 0;
      }
    }
    m_Markiert.Set(NeuAnf,NeuEnd,false);
    HideCaret();
    GetDC();
    wxRect rect(0,0,GetClientSize().x,GetClientSize().y);
    Paint(*DC,false,rect);
    DelDC();
    ShowCaret();
  }
  else
    SetFocus();
  ShowCaretPos();
}

void TEFEdit::OnLButtonDblClk(wxMouseEvent &event)
{
  if ((FindFocus() == this) && !Printing)
  {
    TextPos NeuAnf = m_Markiert.GetAnf(),NeuEnd = m_Markiert.GetEnd();

    m_Markiert.AltPos.Zeile = NeuEnd.Zeile;
    Pt = wxPoint(event.GetX(),event.GetY());
    ScreenToClient(Pt);
    bool OrgCapt = Capt;
    Capt = true;
    int Z;
    unsigned int Pos;
    MousePos(false,Z,Pos);
    NeuEnd.Zeile = Z;
    NeuEnd.Pos = Pos;
    if (!event.ShiftDown())
      UpdateAll = m_Markiert.AltPos.Zeile != NeuAnf.Zeile;
    else
      UpdateAll = false;
    Capt = OrgCapt;
    m_Markiert.Set(NeuAnf,NeuEnd);
    ExtendSelection(true);
    if (MyMacroRecorder::Get()->IsRecording() && (m_Mode != DialogStyle))
      MyMacroRecorder::Get()->AddEvent(wxT("CMD_EXTENDSEL"),GetId());
  }
  else
    SetFocus();
}

void TEFEdit::OnExtendSelection(wxCommandEvent &)
{
  ExtendSelection(false);
}

void TEFEdit::ExtendSelection(bool DoubleClick)
{
  static unsigned int Anf,End;
  static int Zeile;
  TextPos NeuAnf = m_Markiert.GetAnf(),NeuEnd = m_Markiert.GetEnd();

  BrZ.SetAkEint(NeuEnd.Zeile);
  if (BrZ.AkZ().Commentline) // Kommentarzeilen immer ganz markieren
  {
    NeuAnf.Pos = 0;
    NeuEnd.Pos = BrZ.AkZ().FZeile.FLength();
  }
  else
  {
    if ((!m_Markiert.DblClk && DoubleClick) || (Zeile != NeuEnd.Zeile) ||
        (Anf > NeuEnd.Pos) || (End < NeuEnd.Pos) ||
        (Anf <= 0) || (End > BrZ.AkZ().FZeile.FLength()))
    { // Alte Markierung loeschen
      Anf = NeuEnd.Pos;
      End = NeuEnd.Pos;
    }
    GetNewSelection(Anf,End);
    NeuAnf.Pos = Anf;
    NeuEnd.Pos = End;
  }
#ifdef __WXMSW__
  m_Markiert.DblClk = 2;
#else
  m_Markiert.DblClk = 3;
#endif
  Anf = NeuAnf.Pos;
  End = NeuEnd.Pos;
  Zeile = NeuEnd.Zeile;
  NeuAnf.Zeile = NeuEnd.Zeile;
  m_Markiert.Set(NeuAnf,NeuEnd,false);
  HideCaret();
  GetDC();
  wxRect rect(0,0,GetClientSize().x,GetClientSize().y);
  Paint(*DC,false,rect);
  DelDC();
  ShowCaret();
}

void TEFEdit::OnLButtonUp(wxMouseEvent &event)
{
  if (DragMode && Capt)
  { // nicht gedragt -> Markierung loeschen
    TextPos NeuAnf = m_Markiert.GetAnf(),NeuEnd = m_Markiert.GetEnd();

    Pt = wxPoint(event.GetX(),event.GetY());
    int Zeile;
    unsigned int Pos;
    MousePos(false,Zeile,Pos);
    NeuEnd.Zeile = Zeile;
    NeuEnd.Pos = Pos;
    NeuAnf = NeuEnd;
    m_Markiert.Set(NeuAnf,NeuEnd);
    HideCaret();
    GetDC();
    wxRect rect(0,0,GetClientSize().x,GetClientSize().y);
    Paint(*DC,false,rect);
    DelDC();
    ShowCaret();
  }
  if (Capt)
  {
    ReleaseMouse();
    if (MyMacroRecorder::Get()->IsRecording() && (m_Mode != DialogStyle))
    {
      TextPos NeuAnf = m_Markiert.GetAnf(),NeuEnd = m_Markiert.GetEnd();
      // force different value, so that next call will write to macro
      m_Markiert.Set(TextPos(NeuAnf.Pos+1,
                             NeuAnf.Zeile),NeuEnd,false);
      m_Markiert.Set(NeuAnf,NeuEnd);
    }  
  }
  if (!ValueWin)
  {
    IdleTimer.Stop();
    int PixelPos;
    MouseLine(PixelPos);
    aktimerlin = BrZ.AkEintNummer();
    IdleTimer.Start(500,true);
    timerlin = aktimerlin;
  }
  if (ValueWin)
  {
    ValueWin->Destroy();
    ValueWin = 0;
  }
  Capt = false;
}

void TEFEdit::OnMouseLeave(wxMouseEvent &event)
{
  if (ValueWin)
  {
    ValueWin->Destroy();
    ValueWin = 0;
  }
  IdleTimer.Stop();
  event.Skip();
}

void TEFEdit::OnMouseMove(wxMouseEvent &event)
{
  if (ValueWin &&
      ((event.GetX() > ValueWinPos.x+5) || (event.GetX() < ValueWinPos.x-5) ||
       (event.GetY() > ValueWinPos.y+5) || (event.GetY() < ValueWinPos.y-5)))
  {
    ValueWin->Destroy();
    ValueWin = 0;
    IdleTimer.Stop();
  }
  Pt = wxPoint(event.GetX(),event.GetY());
  ScreenToClient(Pt);
  if (!Capt && !Printing)
  {
    int PixelPos;
    MouseLine(PixelPos);
    aktimerlin = BrZ.AkEintNummer();
    if (timerlin != aktimerlin)
    {
      IdleTimer.Stop();
      IdleTimer.Start(500,true);
      timerlin = aktimerlin;
    }
  }
  else
  if (Capt && !Printing)
  {
    if (DragMode)
    {
      if ((abs(DownPos.x-Pt.x) > 5) || (abs(DownPos.y-Pt.y) > 5))
      {
        FormatStrObj Zeile;
        if (GetSelText(Zeile,false))
        {
          ReleaseMouse();
          Capt = false;
          MGPStr AZeile;
          Zeile.GetTextStr(AZeile);
          wxString tmp(AZeile.c_str());
          wxTextDataObject data(tmp);
          wxDropSource dragsrc(this);
          dragsrc.SetData(data);
          unsigned int OrgA,OrgE;
          GetSelection(OrgA,OrgE);
          m_Markiert.SortPos();
          TextPos tmpp(m_Markiert.AkPos[0]);
          if (dragsrc.DoDragDrop(true) == wxDragMove)
          {
            unsigned int A,E;
            GetSelection(A,E);
            // dragged after original position ->
            // correct insert marking minus deleted characters
            if ((A > OrgA) && (E > OrgE))
            {
              A -= OrgE-OrgA;
              E -= OrgE-OrgA;
            }
            else
            // dragged before original position ->
            // correct delete marking plus inserted characters
            if ((A < OrgA) && (E < OrgE))
            {
              OrgA += E-A;
              OrgE += E-A;
            }
            SetSelection(OrgA,OrgE);
            MGPStr ttmp;
            if (MyMacroRecorder::Get()->IsRecording() && (m_Mode != DialogStyle))
              MyMacroRecorder::Get()->AddEvent(wxT("CMD_EDITDELETE"),GetId());
            GetSelText(ttmp,true,false);
            
            if ((A != OrgA) || (E != OrgE))
            { // enter only undo info for remove because Insert() already done
              Rueck.GetAkItem().Str.SetStr(ttmp);
              Rueck.GetAkItem().Ins = tmpp;
              // update insert position, corrected for deleted characters
              SetSelection(A,E);
              m_Markiert.SortPos();
              Rueck.GetAkItem().Anf.SetPos(m_Markiert.AkPos[0].Pos,
                                           m_Markiert.AkPos[0].Zeile);
              Rueck.GetAkItem().End.SetPos(m_Markiert.AkPos[1].Pos,
                                           m_Markiert.AkPos[1].Zeile);
            }
            else
              Rueck.NewUndo(m_Markiert.GetAnf(),m_Markiert.GetEnd(),tmpp,ttmp);
          }
          return;
        }
      }
      return;
    }
    TextPos NeuAnf = m_Markiert.GetAnf(),NeuEnd = m_Markiert.GetEnd();

    m_Markiert.AltPos = NeuEnd;
    int XAlt = XAnf,AltZ = FirstVisZeile;
    int Zeile;
    unsigned int Pos;
    bool Links = MousePos(false,Zeile,Pos);
    NeuEnd.Zeile = Zeile;
    NeuEnd.Pos = Pos;
    if (Links)
    {
      if (NeuEnd.Zeile < NeuAnf.Zeile)
        NeuEnd.Pos = 0;
      else
        NeuAnf.Pos = 0;
    }
    m_Markiert.Set(NeuAnf,NeuEnd,false);
    bool TimeOut = ScrollWithCursor(Pt);
    if ((m_Markiert.AltPos.Pos != NeuEnd.Pos) ||
        (m_Markiert.AltPos.Zeile != NeuEnd.Zeile) ||
        (XAlt != XAnf) || (AltZ != FirstVisZeile))
    {
      UpdateAll = false;
      HideCaret();
      GetDC();
      wxRect rect(0,0,GetClientSize().x,GetClientSize().y);
      Paint(*DC,false,rect);
      DelDC();
      ShowCaret();
    }
    if (TimeOut)
      while (m_sw.Time() < 50);
  }
#ifdef __WXGTK__
  wxSetCursorEvent evt(Pt.x,Pt.y);
  OnSetCursor(evt);
#endif
}

unsigned int TEFEdit::OnDragText(wxCoord x,wxCoord y,bool keepselection)
{
  bool OrgCapt = Capt;
  TextPos NeuAnf = m_Markiert.GetAnf(),NeuEnd = m_Markiert.GetEnd();

  Capt = true;
  Pt = wxPoint(x,y);
  ScreenToClient(Pt);
  SortedTextSelection OrgM = m_Markiert;
  bool oldmo = m_Markiert.SetMacroOut(false);
  m_Markiert.AltPos = NeuEnd;
  int Zeile;
  unsigned int Pos;
  MousePos(false,Zeile,Pos,keepselection);
  NeuEnd.Pos = Pos;
  NeuEnd.Zeile = Zeile;
  NeuAnf = NeuEnd;
  m_Markiert.Set(NeuAnf,NeuEnd);
  bool TimeOut = ScrollWithCursor(Pt);
  unsigned int APos;
  GetSelection(APos,APos);
  if (keepselection)
    m_Markiert = OrgM;
  m_Markiert.SetMacroOut(oldmo);
  Capt = OrgCapt;
  if (TimeOut)
    while (m_sw.Time() < 50);
  return APos;
}

void TEFEdit::OnPaint(wxPaintEvent &)
{
  if (!Printing)
  {
    wxPaintDC dc(this);
    dc.SetBackgroundMode(wxSOLID);
    wxRegion rgn = GetUpdateRegion();
    wxRect rect = rgn.GetBox(); // Viereck um die UpdateRegion ermitteln
    UpdateAll = true; // ganze UpdateRegion aktualisieren
    DC = &dc; // Dummy so that side lines are drawn
    SetFormatFont(TEFTextCalc::Normal,0,true);
    Paint(dc,true,rect);
    DC->SetFont(wxNullFont);
    DC = 0;
  }
}

void TEFEdit::OnVScroll(WXTYPE ScrollCode,int line)
{
  wxScrollWinEvent event(ScrollCode,line,wxVERTICAL);
  OnScroll(event);
}

void TEFEdit::OnHScroll(WXTYPE ScrollCode,int line)
{
  wxScrollWinEvent event(ScrollCode,line,wxHORIZONTAL);
  OnScroll(event);
}

void TEFEdit::OnScroll(wxScrollWinEvent &event)
{
  static ToolWindow *hlptxt = 0;

  if (event.GetOrientation() == wxVERTICAL)
  {
    if (event.GetEventType() == wxEVT_SCROLLWIN_BOTTOM)
      SetFirstVisZeile(BrZ.GetMaxEintrag());
    else
    if (event.GetEventType() == wxEVT_SCROLLWIN_TOP)
      SetFirstVisZeile(1);
    else
    if (event.GetEventType() == wxEVT_SCROLLWIN_LINEDOWN)
      SetFirstVisZeile(FirstVisZeile+1);
    else
    if (event.GetEventType() == wxEVT_SCROLLWIN_LINEUP)
       SetFirstVisZeile(FirstVisZeile-1);
    else
    if (event.GetEventType() == wxEVT_SCROLLWIN_PAGEDOWN)
      SetFirstVisZeile(PgDownZeilen(FirstVisZeile)+FirstVisZeile);
    else
    if (event.GetEventType() == wxEVT_SCROLLWIN_PAGEUP)
      SetFirstVisZeile(FirstVisZeile-PgUpZeilen(FirstVisZeile));
    else
    if (event.GetEventType() == wxEVT_SCROLLWIN_THUMBTRACK)
    {
      if (ViewOptions & ViewScrollTrack)
        SetFirstVisZeile(event.GetPosition()+1);
      else
      {
        if (!hlptxt && ((m_Mode == FunktionStyle) || (m_Mode = FindReplaceStyle)))
          hlptxt = new ToolWindow(this);
        _TCHAR AkF[50];
        if (event.GetPosition() <= BrZ.GetMaxEintrag())
        {
          int OldLine = BrZ.AkEintNummer();
          BrZ.SetAkEint(event.GetPosition()+1);
          const _TCHAR *tst = _tcsstr(BrZ.AkZ().FZeile.c_str(),wxT(" = "));
          if (tst && !BrZ.AkZ().FormatError && !BrZ.AkZ().Commentline)
          { // = gefunden und gueltige Funktionszeile
            _tcsncpy(AkF,BrZ.AkZ().FZeile.c_str(),20);
            AkF[tst-BrZ.AkZ().FZeile.c_str() < 20 ?
                int(tst-BrZ.AkZ().FZeile.c_str()) : 19] = 0;
          }
          else
            AkF[0] = 0;
          BrZ.SetAkEint(OldLine);
        }
        else AkF[0] = 0;
        wxString Zeile;
        Zeile.Printf(_("Line %u"),event.GetPosition()+1);
        if (AkF[0])
        {
          Zeile += wxT(" : '");
          Zeile += AkF;
          Zeile += wxT("'");
        }
        if (hlptxt)
        {
          MGPStr tmp(Zeile.c_str());
          hlptxt->SetText(tmp);
          wxPoint pt = wxPoint(GetClientSize().x-hlptxt->GetSize().x-20,
                               GetClientSize().y/2);
          hlptxt->Move(pt);
          hlptxt->Refresh();
        }
        return;
      }
    }
    else
    if (event.GetEventType() == wxEVT_SCROLLWIN_THUMBRELEASE)
    {
      if (hlptxt)
        hlptxt->Show(false);
      SetFirstVisZeile(event.GetPosition()+1);
      if (hlptxt)
      {
        hlptxt->Destroy();
        hlptxt = 0;
      }
    }
    SetDocSize(false,0,event.GetEventType() != wxEVT_SCROLLWIN_THUMBTRACK);
  }
  else
  {
    int i = XAnf;
    if (event.GetEventType() == wxEVT_SCROLLWIN_BOTTOM)
      XAnf = AkLen;
    else
    if (event.GetEventType() == wxEVT_SCROLLWIN_TOP)
      XAnf = 0;
    else
    if (event.GetEventType() == wxEVT_SCROLLWIN_LINEDOWN)
    {
      if ((XAnf <= AkLen-AveChrW) || (XAnf <= VisXMax-AveChrW))
        XAnf += AveChrW;
      else
        XAnf = (AkLen > VisXMax ? AkLen : VisXMax);
    }
    else
    if (event.GetEventType() == wxEVT_SCROLLWIN_LINEUP)
    {
      if (XAnf >= AveChrW)
        XAnf -= AveChrW;
      else XAnf = 0;
    }
    else
    if (event.GetEventType() == wxEVT_SCROLLWIN_PAGEDOWN)
    {
      XAnf += GetClientSize().x;
      if ((XAnf > AkLen) && (XAnf > VisXMax))
        XAnf = AkLen;
    }
    else
    if (event.GetEventType() == wxEVT_SCROLLWIN_PAGEUP)
    {
      XAnf -= GetClientSize().x;
      if (XAnf < 0)
        XAnf = 0;
    }
    else
    if (event.GetEventType() == wxEVT_SCROLLWIN_THUMBTRACK)
    {
      if (ViewOptions & ViewScrollTrack)
        XAnf = event.GetPosition();
      else
        return;
    }
    else
    if (event.GetEventType() == wxEVT_SCROLLWIN_THUMBRELEASE)
      XAnf = event.GetPosition();
    int scx = i-XAnf,scy = 0;
    ScrollWindowPos(scx,scy);
    SetDocSize(false,0,event.GetEventType() != wxEVT_SCROLLWIN_THUMBTRACK);
  }
}

void TEFEdit::OnSize(wxSizeEvent &)
{
  MaxZeilen = (GetClientSize().y)/(FHeight*COUNTSPERLINE);
  SetDocSize(true);
}

void TEFEdit::OnSetCursor(wxSetCursorEvent &event)
{
  wxCursor csr;
  if (!MenuCursor)
  {
    if ((event.GetX() < X0-1-XAnf) && RechtsCursor)
      csr = *RechtsCursor;
    else
      csr = *IBeamCursor;
  }
  else
    csr = *wxSTANDARD_CURSOR;
#ifdef __WXGTK__
  SetCursor(csr);
#else
  event.SetCursor(csr);
#endif
}

void TEFEdit::OnEditPaste(wxCommandEvent &event)
{
  if (!Paste())
    event.Skip();
}

void TEFEdit::OnEditCopy(wxCommandEvent &)
{
  Copy();
}

void TEFEdit::OnEditCopyBracketText(wxCommandEvent &)
{
  Copy(true);
}

void TEFEdit::OnSpreadCopy(wxCommandEvent &)
{
  MGStrListObj fus;
  if (m_Markiert.ZeichenMarkiert() || m_Markiert.MehrereZeilen())
  { // wenn Markierung vorhanden
    m_Markiert.SortPos();
    for (int i = m_Markiert.AkPos[0].Zeile;i <= m_Markiert.AkPos[1].Zeile;i++)
    {
      BrZ.SetAkEint(i);
      fus.NeuAkEint();
      fus.GetAkEint() = BrZ->FZeile;
    }
  }
  EFSpreadsheetExport spexp(Parser.GetLDV(),fus);
  FormatStrObj res;
  spexp.Convert(res);
  if (wxTheClipboard->Open())
  {
    wxTheClipboard->Clear();
    MGPStr AZeile;
    res.GetTextStr(AZeile);
    wxTextDataObject *data = new wxTextDataObject(AZeile.c_str());
    wxTheClipboard->SetData(data);
    wxTheClipboard->Close();
  }
}

void TEFEdit::OnEditCut(wxCommandEvent &)
{
  Cut();
}

void TEFEdit::CeEditPaste(wxUpdateUIEvent& ce)
{
  ce.Enable(false);
#ifdef __WXGTK__
  if (wxTheClipboard->IsOpened())
    return;
#endif
  if (wxTheClipboard->Open())
  {
    ce.Enable(wxTheClipboard->IsSupported(wxDF_TEXT));
    wxTheClipboard->Close();
  }
}

void TEFEdit::OnMarkAll(wxCommandEvent &)
{
  SetSelection(0,GetTextLen());
}

void TEFEdit::OnMacroSetSelection(wxCommandEvent &event)
{
  EFXmlNode *node = (EFXmlNode *)event.GetClientData();
  if (node->GetProperties())
  {
    TextPos NeuAnf,NeuEnd;
    EFXmlProperty *prop = node->GetProperties();
    int x1,y1,x2,y2;
    MyMacroRecorder::Get()->GetPos(prop,x1,y1,x2,y2);
    NeuAnf.Pos = x1;
    NeuAnf.Zeile = y1;
    NeuEnd.Pos = x2;
    NeuEnd.Zeile = y2;
    m_Markiert.Set(NeuAnf,NeuEnd,false);
    if (IsShown())
      ScrollToCaret();
    // true, to erase background (for CaretPosMarker in X-Pos 0 !)
    EFInvalidate(true);
  }
}

void TEFEdit::OnMacroInsert(wxCommandEvent &event)
{
  EFXmlNode *node = (EFXmlNode *)event.GetClientData();
  EFXmlProperty *prop = node->GetProperties();
  if (prop)
  {
    int x1,y1,x2,y2;
    if (MyMacroRecorder::Get()->GetPos(prop,x1,y1,x2,y2))
    {
      TextPos NeuAnf(x1,y1);
      // Add blank lines so that insert line exists
      if (y1 > BrZ.GetMaxEintrag())
        BrZ.CreateAkEint(y1);
      m_Markiert.Set(NeuAnf,NeuAnf);
    }
    if (node->GetChildren())
      Insert(node->GetChildren()->GetContent().c_str());
    else // blank string inserted
    {
      wxString tmp;
      for (int i = 0;i < abs(x2-x1);i++)
        tmp += wxT(" ");
      Insert(tmp.c_str());
    }
  }
}

void TEFEdit::OnMacroDelete(wxCommandEvent &)
{
  wxKeyEvent evt(wxEVT_KEY_DOWN);
  evt.m_keyCode = WXK_DELETE;
  ProcessEvent(evt);
}

void TEFEdit::OnMacroBackspace(wxCommandEvent &)
{
  wxKeyEvent evt(wxEVT_KEY_DOWN);
  evt.m_keyCode = WXK_BACK;
  ProcessEvent(evt);
}

void TEFEdit::OnMacroReturn(wxCommandEvent &)
{
  wxKeyEvent evt(wxEVT_KEY_DOWN);
  evt.m_keyCode = WXK_RETURN;
  ProcessEvent(evt);
}

void TEFEdit::OnChildSizeMove(wxCommandEvent &event)
{
  modified = true;
  SetDocSize();
  if (event.GetEventType() == wxEVT_COMMAND_CHILD_CLOSED)
  {
    wxCommandEvent cevent(wxEVT_COMMAND_MENU_SELECTED,XRCID("ICM_RENUMBERCHILDREN"));
    AddPendingEvent(cevent);
    if (MyMacroRecorder::Get()->IsRecording() && (m_Mode != DialogStyle))
      MyMacroRecorder::Get()->AddEvent(wxT("CMD_DELWINDOW"),event.GetId());
  }
  else
  {
    if (MyMacroRecorder::Get()->IsRecording() && (m_Mode != DialogStyle))
    {
      MyMacroRecorder::Get()->AddEvent(wxT("CMD_RESIZEWINDOW"),event.GetId());
      wxWindow *wnd = FindWindowById(event.GetId(),this);
      if (wnd)
      {
        wxSize sz = wnd->GetSize();
        wxPoint pt = wnd->GetPosition();
        MyMacroRecorder::Get()->AddPos(pt.x,pt.y,pt.x+sz.x,pt.y+sz.y);
      }
    }
  }
}

void TEFEdit::OnMacroResizeChild(wxCommandEvent &event)
{
  EFXmlNode *node = (EFXmlNode *)event.GetClientData();
  wxWindow *wnd = wxDynamicCast(event.GetEventObject(),wxWindow);
  if (wnd)
  {
    int x1,y1,x2,y2;
    MyMacroRecorder::Get()->GetPos(node->GetProperties(),x1,y1,x2,y2);
    wxSize sz(x2-x1,y2-y1);
    wxPoint pt(x1,y1);
    wnd->SetSize(wxRect(pt,sz));
  }
}

void TEFEdit::ShowCaretPos()
{ // Zeigt aktuelle Zeilennummer und Zeilenposition an
  unsigned int Anf,End,LnNmb;
  MGPStr MGZeile;

  GetSelection(Anf,End);
  LnNmb = GetLineFromPos(Anf);
  MGZeile.AddInt(LnNmb+1);
  MGZeile.AddStr(wxGetApp().GetLDV().ListSeparator);
  MGZeile.AddInt(Anf-GetLineIndex(LnNmb));
  wxGetApp().sb->SetStatusText(MGZeile.c_str(),1);
}

//
// Perform a search or replace operation based on information in SearchData
//
void
TEFEdit::DoSearch(ReplaceMode Mode)
{
  unsigned int AnfPos,EndPos;
  do
  {
    if (Mode == Replace)
    {
      GetSelection(AnfPos,EndPos);
      SetSelection(AnfPos,AnfPos);
    }
    if (Search(-1, SearchData.GetFindString().c_str(),
               bool(SearchData.GetFlags() & wxFR_MATCHCASE),
               bool(SearchData.GetFlags() & wxFR_WHOLEWORD),
               !(SearchData.GetFlags() & wxFR_DOWN)) >= 0)
    {
      if ((Mode == Replace) || (Mode == ReplaceAll))
        Insert(SearchData.GetReplaceString());
    }
    else
    {
      if (Mode != ReplaceAll)
      {
        _TCHAR errMsg[200];
        wxString str(_("Cannot find string '%s' !"));
        _stprintf(errMsg, str.EF_C_STR(),
                SearchData.GetFindString().EF_C_STR());
        wxWindow* w = SearchDialog ? (wxWindow*)SearchDialog : (wxWindow*)this;
        wxMessageBox(errMsg, wxT("Easy Funktion"),
                      wxOK | wxICON_EXCLAMATION,w);
      }
      else
      if (Mode == ReplaceAll)
        break;
    }
  } 
  while (Mode == ReplaceAll);
}

//
// Open the modeless Find commdlg
//

void TEFEdit::OnEditFind(wxCommandEvent&)
{
  EditFindReplace(Find);
}

void TEFEdit::OnEditFindNext(wxFindDialogEvent&)
{
  EditFindReplace(Find);
}

void TEFEdit::OnFindClose(wxFindDialogEvent&)
{
  SearchDialog->Destroy();
  SearchDialog = 0;
}

void TEFEdit::OnFindReplace(wxFindDialogEvent&)
{
  EditFindReplace(Replace);
}

void TEFEdit::OnFindReplaceAll(wxFindDialogEvent&)
{
  EditFindReplace(ReplaceAll);
}

void TEFEdit::EditFindReplace(ReplaceMode Mode)
{
  if (SearchDialog)
  {
    DoSearch(Mode);
  }
  else
  {
    wxString title = (Mode == Replace ? _("Replace Text") : _("Search Text"));
    SearchDialog = new wxFindReplaceDialog(this, &SearchData,title,
                                           Mode == Replace ?
                                           wxFR_REPLACEDIALOG : 0);
    SearchDialog->Show(true);
  }
}

//
// Open the modeless Replace commdlg
//
void
TEFEdit::OnEditReplace(wxCommandEvent&)
{
  EditFindReplace(Replace);
}

//
// Enable the find or replace option only if no dialog is up
//
void
TEFEdit::CeEditFindReplace(wxUpdateUIEvent& ce)
{
  ce.Enable((FindFocus() == this) && !SearchDialog);
}

void TEFEdit::OnMacroFontHeight(wxCommandEvent &event)
{
  EFXmlNode *node = (EFXmlNode *)event.GetClientData();
  EFXmlProperty *prop = node->GetProperties();
  int points;
  MyMacroRecorder::Get()->GetPropertyInt(prop,wxT("points"),points);
  SetSelectionFormat(TEFTextCalc::FontHeight,points);
}

BEGIN_MACRO_EVENT_TABLE(TEFEdit,TEFTextCalc, MyMacroRecorder)
  EVT_UPDATE_UI(wxID_FIND,TEFEdit::CeEditFindReplace)
  EVT_UPDATE_UI(XRCID("CMD_EDITREPLACE"),TEFEdit::CeEditFindReplace)
  EVT_MENU(wxID_FIND, TEFEdit::OnEditFind)
  EVT_MENU(XRCID("CMD_EDITREPLACE"), TEFEdit::OnEditReplace)
  EVT_FIND_REPLACE(-1,TEFEdit::OnFindReplace)
  EVT_FIND_REPLACE_ALL(-1,TEFEdit::OnFindReplaceAll)
  EVT_FIND(-1,TEFEdit::OnEditFindNext)
  EVT_FIND_NEXT(-1,TEFEdit::OnEditFindNext)
  EVT_FIND_CLOSE(-1,TEFEdit::OnFindClose)
  // Macro events
  EVT_MENU(MACROID("CMD_FONTHEIGHT"),TEFEdit::OnMacroFontHeight)
  EVT_MENU(MACROID("CMD_RESIZEWINDOW"),TEFEdit::OnMacroResizeChild)
  EVT_MENU(MACROID("CMD_SETEDITSELECTION"),TEFEdit::OnMacroSetSelection)
  EVT_MENU(MACROID("CMD_EDITINSERT"),TEFEdit::OnMacroInsert)
  EVT_MENU(MACROID("CMD_EDITDELETE"),TEFEdit::OnMacroDelete)
  EVT_MENU(MACROID("CMD_EDITBACKSPACE"),TEFEdit::OnMacroBackspace)
  EVT_MENU(MACROID("CMD_EDITRETURN"),TEFEdit::OnMacroReturn)

  // Clipboard events
  EVT_MENU(wxID_COPY,TEFEdit::OnEditCopy)
  EVT_UPDATE_UI(wxID_COPY,TEFEdit::CeEditCopy)
  EVT_MENU(XRCID("CMD_COPYBRACKETS"),TEFEdit::OnEditCopyBracketText)
  EVT_UPDATE_UI(XRCID("CMD_COPYBRACKETS"),TEFEdit::CeEditCopy)
  EVT_MENU(wxID_CUT,TEFEdit::OnEditCut)
  EVT_UPDATE_UI(wxID_CUT,TEFEdit::CeEditCopy)

  EVT_MENU(MACROID("CMD_SPREADCOPY"),TEFEdit::OnSpreadCopy)
  EVT_UPDATE_UI(XRCID("CMD_SPREADCOPY"),TEFEdit::CeSpreadCopy)

  // General window events
  EVT_CHAR(TEFEdit::OnChar)
  EVT_KEY_DOWN(TEFEdit::OnKeyDown)
  EVT_RIGHT_UP(TEFEdit::OnRButtonUp)
  EVT_LEFT_DOWN(TEFEdit::OnLButtonDown)
  EVT_LEFT_DCLICK(TEFEdit::OnLButtonDblClk)
  EVT_LEFT_UP(TEFEdit::OnLButtonUp)
  EVT_MOTION(TEFEdit::OnMouseMove)
  EVT_SET_CURSOR(TEFEdit::OnSetCursor)
  EVT_MOUSEWHEEL(TEFEdit::OnMousewheel)
  EVT_LEAVE_WINDOW(TEFEdit::OnMouseLeave)
  EVT_PAINT(TEFEdit::OnPaint)
  EVT_SET_FOCUS(TEFEdit::OnSetFocus)
  EVT_KILL_FOCUS(TEFEdit::OnKillFocus)
  EVT_SCROLLWIN(TEFEdit::OnScroll)
  EVT_SIZE(TEFEdit::OnSize)

  // Child notification events
  EVT_CHILD_MOVED(-1,TEFEdit::OnChildSizeMove)
  EVT_CHILD_RESIZED(-1,TEFEdit::OnChildSizeMove)
  EVT_CHILD_CLOSED(-1,TEFEdit::OnChildSizeMove)
  EVT_CHILD_CREATED(-1,TEFEdit::OnChildSizeMove)

  // Update events
  EVT_MENU(XRCID("ICM_SETMODIFIED"),TEFEdit::OnSetModified)
  EVT_MENU(XRCID("ICM_DELEDIT"),TEFEdit::OnDelEdit)

  // Comment font events
  EVT_UPDATE_UI(MACROID("CMD_COMMENTFONT"),TEFEdit::CeFontFormat)

  EVT_MENU(MACROID("CMD_FONTBOLD"), TEFEdit::OnFontBold)
  EVT_UPDATE_UI(XRCID("CMD_FONTBOLD"),TEFEdit::CeFontFormat)

  EVT_MENU(MACROID("CMD_FONTUNDERLINE"), TEFEdit::OnFontUnderline)
  EVT_UPDATE_UI(XRCID("CMD_FONTUNDERLINE"),TEFEdit::CeFontFormat)

  EVT_MENU(MACROID("CMD_FONTITALIC"), TEFEdit::OnFontItalic)
  EVT_UPDATE_UI(XRCID("CMD_FONTITALIC"),TEFEdit::CeFontFormat)

  // Not macro recorded events
  EVT_MENU(XRCID("ICM_FONTHEIGHT"), TEFEdit::OnFontHeight)
  EVT_UPDATE_UI(XRCID("ICM_FONTHEIGHT"),TEFEdit::CeFontFormat)

  // Editor options events
  EVT_MENU(MACROID("CMD_FRACTIONVIEW"),TEFEdit::OnBruchDarst)
  EVT_UPDATE_UI(XRCID("CMD_FRACTIONVIEW"),TEFEdit::CeBruchDarst)

  EVT_MENU(MACROID("CMD_SHOWALLBRACKETS"),TEFEdit::OnKlammerDarst)
  EVT_UPDATE_UI(XRCID("CMD_SHOWALLBRACKETS"),TEFEdit::CeKlammerDarst)

  EVT_MENU(MACROID("CMD_HIDEDIAGS"),TEFEdit::OnHideDiags)
  EVT_UPDATE_UI(XRCID("CMD_HIDEDIAGS"),TEFEdit::CeHideDiags)

  EVT_MENU(MACROID("CMD_MARKLABELS"),TEFEdit::OnMarkLabels)
  EVT_UPDATE_UI(XRCID("CMD_MARKLABELS"),TEFEdit::CeMarkLabels)

  // Editor selection events
  EVT_MENU(MACROID("CMD_EXTENDSEL"),TEFEdit::OnExtendSelection)

  EVT_MENU(WXMACROID(wxID_SELECTALL), TEFEdit::OnMarkAll)

  // Undo/Redo events
  EVT_MENU(WXMACROID(wxID_REDO), TEFEdit::OnEditRedo)
  EVT_UPDATE_UI(wxID_REDO, TEFEdit::CeEditRedo)

  EVT_MENU(WXMACROID(wxID_UNDO), TEFEdit::OnEditUndo)
  EVT_UPDATE_UI(wxID_UNDO, TEFEdit::CeEditUndo)
END_EVENT_TABLE()


