// Version 6.05 20.07.2008 Markus Greither
// Copyright (c) 1991-2008 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file efedplvw.cpp
    \brief Source code for the plain text function viewer
*/


#include "efprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#ifdef __DMC__
#include <typeinfo.h>
#endif
#include <wx/wx.h>

#include <complex>
#include <limits>
#include <climits>
#include <set>
#include <functional>
#if defined(__MINGW32__)
#include <wx/msw/winundef.h>
#endif
#include <wx/dnd.h>

using namespace std;

#include "efdefs.h"
#include "mg.h"
#include "cpxpsr.h"
#include "diagramm.h"
#include "efgemein.h"
#include "ef.h"
#include "efresid.h"
#include "efedit.h"
#include "efedplvw.h"

#endif

// ----------------------------------------------------------------------------
// TEFEditPlainView
// ----------------------------------------------------------------------------

IMPLEMENT_DYNAMIC_CLASS(TEFEditPlainView, wxTextCtrl)

TEFEditPlainView::TEFEditPlainView(wxWindow* parent, wxWindowID id,
                                   const wxPoint& pos,const wxSize& size,
                                  long style)
 : wxTextCtrl(parent,id,wxT(""),pos,size,style)
{
#ifdef __WXMSW__
  wxFont font(10,wxSWISS,wxNORMAL,wxNORMAL,FALSE,wxT("Arial"));
  SetFont(font);
#endif
  wxClientDC DC(this);
  int y,extl;
  DC.GetTextExtent(wxT("Mg"),0,&y,0,&extl);
  SetClientSize(GetClientSize().x,y+extl+1);
  AkAnf = 0;
  AkEnd = 0;
}

void TEFEditPlainView::OnIdle(wxIdleEvent &event)
{
  wxWindow *wnd = FindFocus();
  TEFEdit *Editor = wxDynamicCast(wnd,TEFEdit);
  wxWindow *parent = GetParent();
  wxDocMDIChildFrame *frmwin;
  frmwin = wxDynamicCast(parent,wxDocMDIChildFrame);
  wxDocMDIParentFrame *parwin;
  parwin = wxDynamicCast(wxGetApp().GetTopWindow(),wxDocMDIParentFrame);
  if (Editor && frmwin && parwin && (frmwin == parwin->GetActiveChild()))
  {
    int index = Editor->GetLineFromPos(-1);
    FormatStrObj line;
    line.SetStr(Editor->GetLine(index));
    bool textchanged = false;
    if (_tcscmp(line.c_str(),AkZeile.c_str()) != 0)
    {
      MGPStr tmp;
      line.GetTextStr(tmp);
      SetValue(tmp.c_str());
      AkZeile = line;
      textchanged = true;
    }
    unsigned int Anf,End;
    Editor->GetSelection(Anf,End);
    int LinePos = Editor->GetLineIndex(index);
    Anf -= LinePos;
    End -= LinePos;
    if ((Anf == End) && (int(End) < GetLastPosition()))
      End++;
    if ((Anf != AkAnf) || (End != AkEnd) || textchanged)
    {
      SetInsertionPoint(Anf);
      SetSelection(Anf,End);
    }
    AkAnf = Anf;
    AkEnd = End;
    ShowPosition(Anf);
  }
  else
  if (AkZeile.Length() > 0)
  {
    Clear();
    AkZeile.SetStr(wxT(""));
  }
  event.Skip();
}

BEGIN_EVENT_TABLE(TEFEditPlainView,wxTextCtrl)
  EVT_IDLE(TEFEditPlainView::OnIdle)
  EVT_LEFT_DOWN(TEFEditPlainView::OnLButtonDown)
  EVT_MOTION(TEFEditPlainView::OnMouseMove)
END_EVENT_TABLE()



