// Version 6.05 20.07.2008 Markus Greither
// Copyright (c) 1991-2008 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file efedpnt.cpp
    \brief Source code for the function editor painting code
*/


#include "efprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#ifdef __DMC__
#include <typeinfo.h>
#endif
#include <wx/wx.h>
#include <wx/resizec.h>
#include <wx/dnd.h>

#include <complex>
#include <limits>
#include <climits>
#include <set>
#include <functional>
#if defined(__MINGW32__)
#include <wx/msw/winundef.h>
#endif

using namespace std;

#include "efdefs.h"
#include "mg.h"
#include "cpxpsr.h"
#include "diagramm.h"
#include "efgemein.h"
#include "ef.h"
#include "efresid.h"
#include "eftabcon.h"
#include "efedit.h"

#endif

#define WINDOWCOLOUR wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW)
#define HIGHLIGHT wxSystemSettings::GetColour(wxSYS_COLOUR_HIGHLIGHT)
#define HIGHLIGHTTEXT wxSystemSettings::GetColour(wxSYS_COLOUR_HIGHLIGHTTEXT)
#define WINDOWTEXT wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOWTEXT)

bool TEFEdit::DrawAndMarkText(wxPoint p, const _TCHAR* string, int count,
                              wxSize& size,int &LZaehler)
{
  int altpos = 0;
  wxSize size1;

  size.x = 0;
  while (altpos < count)
  {
    int anf = altpos;
    // if not BruchDarst or not MarkLabels -> no marked labels
    if (!(ViewOptions & ViewMarkLabels) || !(ViewOptions & ViewBruchDarst) ||
        BrZ.AkZ().FormatError || BrZ.AkZ().Commentline)
      anf = count; // no marked labels in case of errors
    else
    if (LZaehler)
      anf = 0; // Start of label was before this string section
    else
    while ((anf < count) && (string[anf] != '[') && (string[anf] != '"'))
      anf++; // determine label or comment begin position
    if (anf > altpos)
    { // string before label or comment available
      DrawText(p,&string[altpos],anf-altpos,size1);
      size.x += size1.x;
      size.y = size1.y;
      p.x += size1.x;
      altpos += anf-altpos;
    }
    if ((anf < count) || LZaehler)
    { // Label or comment available
      int end = anf;
      _TCHAR EndChar;
      if (LZaehler)
        EndChar = _TCHAR(LZaehler >> 8);
      else
      {
        EndChar = string[anf];
        end++;
      }
      LZaehler &= 0xFF; // Delete search character from LZaehler
      if (EndChar == '[')
        EndChar = ']';

      if (EndChar == ']')
      { // set label font
        DC->SetFont(*IFont);
        if (CDC)
          CDC->SetFont(*IFont);
      }
      else
      { // set comment font
        DC->SetFont(*CFont);
        if (CDC)
          CDC->SetFont(*CFont);
      }

      wxColour OldCol = DC->GetTextForeground();
      if ((DC->GetTextBackground() == WINDOWCOLOUR) &&
          (!Printing || ColorPrint))
      { // standard background colour -> text not selected
        wxColour NewCol;
        if (EndChar == ']')
          NewCol = wxColour(LabelFontColour & 0xFF,
                            (LabelFontColour >> 8) & 0xFF,
                            (LabelFontColour >> 16) & 0xFF);
        else
          NewCol = wxColour(KommentFontColour & 0xFF,
                            (KommentFontColour >> 8) & 0xFF,
                            (KommentFontColour >> 16) & 0xFF);
        OldCol = DC->GetTextForeground();
        DC->SetTextForeground(NewCol);
        if (CDC)
          CDC->SetTextForeground(NewCol);
      }

      while ((end < count) && (string[end] != EndChar))
        end++;
      int EndPos = (end == count ? end-1 : end);
      MGPStr tmpstr;
      tmpstr.AddNChar(&string[altpos],EndPos-anf+1);

      if (!LZaehler && (!Printing || (EndChar == ']') || ColorPrint))
        tmpstr.ReplChar(0,' ');

      if (!LZaehler)
        LZaehler++;

      if ((end < count) && (!Printing || (EndChar == ']') || ColorPrint))
        tmpstr.ReplChar(EndPos-anf,' ');

      DrawText(p,tmpstr.c_str(),EndPos-anf+1,size1);
      size.x += size1.x;
      size.y = size1.y;
      p.x += size1.x;

      DC->SetFont(*WFont); // set normal font
      if (CDC)
        CDC->SetFont(*WFont);

      DC->SetTextForeground(OldCol);
      if (CDC)
        CDC->SetTextForeground(OldCol);
      altpos += EndPos-anf+1;
      if (end < count)
        LZaehler--;
      if (LZaehler)
        LZaehler |= EndChar << 8; // Search character in upper byte of LZaehler
    }
  }
  return true;
}

void TEFEdit::DrawText(wxPoint p, const _TCHAR* string, int count, wxSize& size)
{
  if (!CDC)
  {
    wxString str(string);
    str.Truncate(count);
    EFTextExtent(*DC,str,&size.x,&size.y);
    int dely = KlSize.y-size.y;
    DC->DrawText(str,p.x,p.y+(dely > 0 ? dely : 0));
  }
  else
  {
    size.x = 0;
    wxCoord len;
    for (int i = 0;i < count;i++)
    { // Replace tabs with 3 spaces
      if (string[i] == '\t')
      {
        const _TCHAR *tab = wxT("   ");
        wxCoord y;
        EFTextExtent(*DC,wxString(tab),&len,&y);
        CDC->DrawText(wxString(tab),p.x,p.y);
      }
      else
      {
        wxCoord y;
        wxString str(&string[i]);
        str.Truncate(1);
        EFTextExtent(*DC,str,&len,&y);
        CDC->DrawText(str,p.x,p.y);
      }
      size.x += len;
      p.x += len;
    }
    size.y = KlSize.y;
  }
}

void TEFEdit::Paint(wxDC& dc, bool /*erase*/, wxRect& rect,bool forcenoborder)
{
  int AkPos,C;

  bool DCVorh = (DC != 0);
  if (!Printing || forcenoborder)
    AkPos = 0;
  else
    AkPos = 10*PrntTL.y*PixPerInchY/254;
  C = Capt;
  Capt = false;
  m_Markiert.SortPos();
  DC = &dc;
  if (!DCVorh)
    SetFormatFont(TEFTextCalc::Normal,0,true);
  if (UpdateAll)
  { // update complete client rectangle
    BrZ.SetAkEint(FirstVisZeile-1);
    bool tst;
    while (((tst = BrZ.NextEint()) == true) &&
           (AkPos+NextAkPos() < rect.GetTop()))
      AkPos += NextAkPos();
    if ((m_Markiert.AkPos[0].Zeile < BrZ.AkEintNummer()) &&
        (m_Markiert.AkPos[1].Zeile >= BrZ.AkEintNummer()))
      m_Markiert.Count = 1;
    else
      m_Markiert.Count = 0;
    if (tst)
    do
      PaintZeile(AkPos);
    while (BrZ.NextEint() && (AkPos <= rect.GetBottom()));
  }
  else
  { // only update line range from AltPos.Zeile and End.Zeile
    int Frst,Lst;
    if (m_Markiert.AltPos.Zeile < m_Markiert.GetEnd().Zeile)
    {
      Frst = m_Markiert.AltPos.Zeile;
      Lst = m_Markiert.GetEnd().Zeile;
    }
    else
    {
      Frst = m_Markiert.GetEnd().Zeile;
      Lst = m_Markiert.AltPos.Zeile;
    }
    if ((m_Markiert.AkPos[0].Zeile < Frst) && (m_Markiert.AkPos[1].Zeile >= Frst))
      m_Markiert.Count = 1;
    else
      m_Markiert.Count = 0;
    BrZ.SetAkEint(FirstVisZeile-1);
    while (BrZ.NextEint() && (BrZ.AkEintNummer() < Frst))
      AkPos += NextAkPos();
    do
      PaintZeile(AkPos);
    while (BrZ.NextEint() &&
           ((AkPos <= rect.GetBottom()) || Printing) &&
           (BrZ.AkEintNummer() <= Lst));
  }
  if (!Printing && DCVorh)
    DrawPageLine(dc);
  if (!DCVorh)
  {
    DC->SetFont(wxNullFont);
    DC = 0;
  }
  UpdateAll = true; // so that at the end of any function UpdateAll=true
  Capt = C;
}

// Draw vertical and horizontal separation line between pages
void TEFEdit::DrawPageLine(wxDC &dc)
{
  if (PrntBreite)
  {
    int XPos = int((PrntBreite-(PrntTL.x+PrntBR.x))*10*PixPerInchX/254*Zoom);
    wxPen pen(*wxLIGHT_GREY,1,wxDOT);
    dc.SetPen(pen);
    for (int AkX = XPos; AkX+X0-XAnf < GetClientSize().x; AkX += XPos)
    {
      dc.DrawLine(AkX+X0-XAnf,0,AkX+X0-XAnf,GetClientSize().y);
    }
    dc.SetPen(wxNullPen);
  }
  if (PrntLaenge)
  {
    wxPen pen(*wxLIGHT_GREY,1,wxDOT);
    dc.SetPen(pen);
    // determine unzoomed font height
    int FH = int((FontMetric.OrgHeight*FSizeFak+1)/COUNTSPERLINE);
    // invisible range on top of first visible line
    int InvBer = AbstVonAnf(FirstVisZeile,FHeight);
    BrZ.Begin();
    int CurH = 0;
    while (BrZ.NextEint())
    { // calculate page breaks
      int ZH = NextAkPos(FH);
      if (IstPagebreak())
      { // Seitenumbruch
        CurH = 0; // don't count page break lines
      }
      else
      if (CurH+ZH <= (PrntLaenge-(PrntTL.y+PrntBR.y))*10*PixPerInchY/254)
        CurH += ZH; // this line fits on current page
      else
      { // start new page
        if (BrZ.AkEintNummer() >= FirstVisZeile)
        {
          int ObBer = AbstVonAnf(BrZ.AkEintNummer(),FHeight);
          if (ObBer-InvBer-1 <= GetClientSize().y)
          { // if visible
            dc.DrawLine(0,ObBer-InvBer-1,GetClientSize().x,ObBer-InvBer-1);
          }
        }
        CurH = ZH;
      }
    }
    dc.SetPen(wxNullPen);
  }
}

// Wrapper for the many text output routines
bool TEFEdit::DrawFormattedText(wxPoint p, const _TCHAR* string, int count,
                               wxSize& size,int &LZaehler,int &KlZaehler)
{
  if (BrZ.AkZ().Commentline)
    return DrawAndMarkText(p,string,count,size,LZaehler);
  else
  if (!(ViewOptions & ViewBruchDarst) || BrZ.AkZ().FormatError)
    return DrawColorBracketText(p,string,count,size,LZaehler,KlZaehler);
  else
    return DrawBigBracketText(p,string,count,size,LZaehler);
}

// Text output with coloured bracked levels
bool TEFEdit::DrawColorBracketText(wxPoint p, const _TCHAR* string, int count,
                              wxSize& size,int &LZaehler,int &KlZaehler)
{
  int tmp = 0,altpos = 0;

  size.x = 0;
  while (tmp < count)
  { // more chars available
    while ((tmp < count) && (string[tmp] != '(') && (string[tmp] != ')'))
      tmp++;
    if (tmp > altpos)
    { // output string with normal colour
      wxSize size1(0,0);
      DrawAndMarkText(p,&string[altpos],tmp-altpos,size1,LZaehler);
      size.x += size1.x;
      size.y = size1.y;
      p.x += size1.x;
      altpos = tmp;
    }
    if ((tmp < count) && ((string[tmp] == '(') || (string[tmp] == ')')))
    { // output brackets in colour
      wxDC *CurDC;
      if (CDC)
        CurDC = CDC;
      else
        CurDC = DC;
      wxColour orgcol = DC->GetTextForeground();
      if (string[tmp] == '(')
        KlZaehler++;
      wxSize size1(0,0);
      CurDC->SetTextForeground(KlZaehler > 0 ?
                               FarbArr[(KlZaehler-1) % AnzFarbe] : 0);
      DrawAndMarkText(p,&string[tmp],1,size1,LZaehler);
      size.x += size1.x;
      size.y = size1.y;
      p.x += size1.x;
      if (string[tmp] == ')')
        KlZaehler--;
      tmp++;
      altpos = tmp;
      CurDC->SetTextForeground(orgcol);
    }
  }
  return true;
}

// Text output with big brackets
bool TEFEdit::DrawBigBracketText(wxPoint p, const _TCHAR* string, int count,
                                 wxSize& size,int &LZaehler)
{
  int tmp = 0,altpos = 0;

  size.x = 0;
  while (tmp < count)
  { // more chars available
    int LZaehler1 = LZaehler;
    if ((LastMidZ != -1) && ((BrZ.AkZ().ObZ[LastMidZ] > 0) ||
        (BrZ.AkZ().UntZ[LastMidZ] > 0)))
      while ((tmp < count) &&
             !(((string[tmp] == '(') || (string[tmp] == ')')) && !LZaehler1))
      {
        switch (string[tmp]) // evaluate comments and labels
        {
          case '[':
            if (!LZaehler) // not yet a comment or label
            {
              LZaehler1++;
              LZaehler1 += (']' << 8); // enter label end char for stop char
            }
            break;
          case ']':
            if ((LZaehler1 >> 8) == ']')
              LZaehler1 = 0;
            break;
          case '\"':
            if ((LZaehler1 >> 8) == '\"')
              LZaehler1 = 0;
            else
            if (!LZaehler1) // not yet a comment or label
            {
              LZaehler1++;
              LZaehler1 += ('\"' << 8); // enter comment end char for stop char
            }
            break;
        }
        tmp++;
      }
    else
      tmp = count;
    if (tmp > altpos)
    { // output of normal colour string
      wxSize size1(0,0);
      DrawAndMarkText(p,&string[altpos],tmp-altpos,size1,LZaehler);
      size.x += size1.x;
      size.y = size1.y;
      p.x += size1.x;
      altpos = tmp;
    }
    if ((tmp < count) && ((string[tmp] == '(') || (string[tmp] == ')')))
    { // output large brackets
      int Ob = 0,Unt = int((FontMetric.OrgHeight)*OrgZoom);
      if (LastMidZ != -1)
      { // valid positon available
        Ob += BrZ.AkZ().ObZ[LastMidZ]*FHeight;
        Unt += BrZ.AkZ().UntZ[LastMidZ]*FHeight;
      }
      wxDC *CurDC;
      if (CDC)
        CurDC = CDC;
      else
        CurDC = DC;
      if (!CDC && (DC->GetBackgroundMode() == wxSOLID))
      { // paint background
        wxBrush WBrush(DC->GetTextBackground(),wxSOLID);
        const wxBrush &OldBr = DC->GetBrush();
        wxPen WPen(DC->GetTextBackground(),1,wxSOLID);
        CurDC->SetPen(WPen);
        CurDC->SetBrush(WBrush);
        if (p.x+KlSize.x >= 0)
        { // optimisation for broken graphic adapters,
          // only paint background in visible range
          int xpos = p.x;
          int xsize = KlSize.x;
          if (xpos < 0)
          {
            xpos = 0;
            xsize += p.x;
          }
          CurDC->DrawRectangle(xpos,p.y-Ob,xsize,Unt+Ob);
        }
        CurDC->SetBrush(OldBr);
        CurDC->SetPen(wxNullPen);
      }
      int pw = FHeight*COUNTSPERLINE/18;
      if (!pw)
        pw = 1;
      Ob -= pw;
      Unt -= int(FontMetric.ExternalLeading*OrgZoom+pw+1);
      if (!Printing)
      { // draw brackets using lines
        wxPen pen(DC->GetTextForeground(),pw,wxSOLID);
        CurDC->SetPen(pen);
        int XMin,XMax,XBer,XDel;
        if (string[tmp] == '(')
        {
          XMin = p.x+pw;
          XMax = p.x+KlSize.x-1-pw/2;
          if (XMax <= XMin)
            XMax = XMin+1;
        }
        else
        {
          XMax = p.x+pw/2;
          XMin = p.x+KlSize.x-1-pw;
          if (XMax >= XMin)
            XMax = XMin-1;
        }
        XBer = XMax-XMin;
        if ((abs(XBer) < 3) && (abs(XBer) > 0))
          XDel = 0;
        else
          XDel = XBer/3;
        int YDel = (Ob+Unt)/5;
        if (YDel < abs(XDel))
          YDel = abs(XDel);
        // center point on top
        CurDC->DrawLine(XMax,p.y-Ob,XMin+XDel,p.y-Ob+YDel);
        CurDC->DrawLine(XMin+XDel,p.y-Ob+YDel,XMin,p.y-Ob+YDel*2);
        // line down
        CurDC->DrawLine(XMin,p.y-Ob+YDel*2,XMin,p.y+Unt-YDel*2);
        // center point below
        CurDC->DrawLine(XMax,p.y+Unt,XMin+XDel,p.y+Unt-YDel);
        CurDC->DrawLine(XMin+XDel,p.y+Unt-YDel,XMin,p.y+Unt-YDel*2);
        // line up
        CurDC->DrawLine(XMin,p.y+Unt-YDel*2,XMin,p.y-Ob+YDel*2);
        // center point on top
        CurDC->DrawLine(XMin,p.y-Ob+YDel*2,XMin+XDel,p.y-Ob+YDel);
        CurDC->SetPen(wxNullPen);
      }
      else // draw brackets with arcs for print and clipboard
      {
        double x,y;
        CurDC->GetUserScale(&x,&y);
        CurDC->SetUserScale(1,1);
        wxPen pen(DC->GetTextForeground(),x >= 1 ? int(x) : 1,wxSOLID);
        CurDC->SetPen(pen);
        wxBrush OrgBr = CurDC->GetBrush();
        CurDC->SetBrush(*wxTRANSPARENT_BRUSH);
        wxPoint p1(p);
        p1.x = int(p1.x*x);
        p1.y = int(p1.y*y);
        pw = int(pw*x);
        Ob = int(Ob*y);
        Unt = int(Unt*y);
        int klsz = int(KlSize.x*x);
        if (string[tmp] == '(')
          CurDC->DrawEllipticArc(p1.x+pw+1,p1.y-Ob,
                                 klsz*2-2-pw,Ob+Unt,90,270);
        else
          CurDC->DrawEllipticArc(p1.x-klsz+1+pw,p1.y-Ob,
                                 klsz*2-2-pw,Ob+Unt,-90,90);
        CurDC->SetUserScale(x,y);
        CurDC->SetPen(wxNullPen);
        CurDC->SetBrush(OrgBr);
      }
      size.x += KlSize.x;
      p.x += KlSize.x;
      tmp++;
      altpos = tmp;
    }
  }
  return true;
}

// Output string parts of formatted functions or comment lines
void TEFEdit::TextSectionOut(const _TCHAR *Zeile,unsigned int &Pos,
                               unsigned int &XPos,unsigned int &YPos,
                               unsigned int &ZPos,int &LZaehler)
{
  unsigned int f;
  int KlZaehler = 0; // bracket level counter
  int OrgPos = Pos;

  do
  {
    f = Pos;
    while ((f < _tcslen(Zeile)) && (!IstBMarker(Zeile[f])))
      f++;
    if (Capt)
    { // no text output if left mouse button down
      TextPos NeuAnf = m_Markiert.GetAnf(),NeuEnd = m_Markiert.GetEnd();

      if (IstPagebreak())
      {
        f += 2;
        Pos = f;
      } // treat page break same as blank line
      if (m_Markiert.GetEnd().Zeile == 0)
      { // mouse position not found yet
        if ((m_Markiert.GetEnd().Zeile == 0) && (Pt.x <= int(XPos-XAnf+X0)) &&
            (Pt.y <= int(YPos+FHeight*COUNTSPERLINE)))
        { // first position below and right of mouse position
          NeuEnd.Zeile = BrZ.AkEintNummer();
          NeuEnd.Pos = ZPos;
          SetCaretPos(XPos-XAnf+X0-1,YPos);
        }
        int tst = XPos+EFTabbedTextExtent(&Zeile[Pos],f-Pos,GetMarkLabels()).x;
        if ((NeuEnd.Zeile == 0) && (Pt.x <= tst-XAnf+X0) &&
            (Pt.y <= int(YPos+FHeight*COUNTSPERLINE)))
        { // end of string below and right of mouse position
          int i;
          int Min = 1,Max = f-Pos;
          tst = XPos+EFTabbedTextExtent(&Zeile[Pos],1,GetMarkLabels()).x;
          // if not position after first char
          // (binary method never sets Max = 1)
          if (tst-XAnf+X0 < Pt.x)
          {
            // binary method for determining the first char right of mouse pos
            do
            {
              i = (Min+Max)/2;
              tst = XPos+EFTabbedTextExtent(&Zeile[Pos],i,GetMarkLabels()).x;
              if (tst-XAnf+X0 < Pt.x)
                Min = i;
              else
                Max = i;
            }
            while (Max > Min+1);
            i = Max;
          }
          else
            i = 1;
          tst = XPos+EFTabbedTextExtent(&Zeile[Pos],i,GetMarkLabels()).x;
          NeuEnd.Zeile = BrZ.AkEintNummer();
          NeuEnd.Pos = ZPos+i;
          SetCaretPos(tst-XAnf+X0-1,YPos);
        }
        XPos += EFTabbedTextExtent(&Zeile[Pos],f-Pos,GetMarkLabels()).x;
        ZPos += (f-Pos);
        Pos = f;
        if ((NeuEnd.Zeile == 0) && (Zeile[Pos] == 0) && Capt)
        { // at the end of line and still left of mouse position
          NeuEnd.Zeile = BrZ.AkEintNummer();
          NeuEnd.Pos = ZPos;
          SetCaretPos(tst-XAnf+X0-1,YPos);
        }
        m_Markiert.Set(NeuAnf,NeuEnd);
      }
      else
        Pos = f;
    }
    else
    { // text output if left mouse button not down
    
      // temporary variables for drawing the strings in reversed order
      // Maximum of 4 strings, separated by mark positions and/or error position 
      // -> maximum of 3 positions to be cached
      int StrCount = 0;
      int KlZaehler1 = KlZaehler, LZaehler1 = LZaehler,Pos1 = Pos;
      int EndStrPos[3], XPos1 = XPos, MarkCount[3];
      wxPoint APos1(XPos-XAnf+X0,YPos);

      ZPos += (f-Pos);
      // Check if this string contains a error position
      bool FehlerPos = BrZ.AkZ().FehlerPos &&
                       (BrZ.AkZ().FehlerPos <= ZPos) &&
                       !BrZ.AkZ().Commentline &&
                       (Zeile[Pos] != _TCHAR(TEFTextCalc::PosMarker));
      // Check if this string contains a begin and/or end of the selection
      bool ValCPos = (m_Markiert.Count < 2) &&
                     (m_Markiert.AkPos[m_Markiert.Count].Zeile ==
                      BrZ.AkEintNummer()) &&
                     (m_Markiert.AkPos[m_Markiert.Count].Pos <= ZPos) &&
                     (Zeile[Pos] != _TCHAR(TEFTextCalc::PosMarker));
      while (ValCPos || FehlerPos)
      { // cursor of error position in current string
        if (ValCPos && (Pos < f-(ZPos-m_Markiert.AkPos[m_Markiert.Count].Pos)) ||
            FehlerPos && (Pos < f-(ZPos-BrZ.AkZ().FehlerPos)))
        { // current position before the last char of current string
          wxSize Size;
          wxPoint APos(XPos-XAnf+X0,YPos);
          SetTextColor(m_Markiert.Count == 1,GetRedText(Pos,f,ZPos));
          int EndPos;
          if (ValCPos && (!FehlerPos || (FehlerPos &&
              (m_Markiert.AkPos[m_Markiert.Count].Pos < BrZ.AkZ().FehlerPos))))
            EndPos = f-(ZPos-m_Markiert.AkPos[m_Markiert.Count].Pos);
          else
            EndPos = f-(ZPos-int(BrZ.AkZ().FehlerPos));

          // Store partial string position and mark information for reversed redraw
          EndStrPos[StrCount] = EndPos;
          MarkCount[StrCount] = m_Markiert.Count;
          StrCount++;

          // Draw partial string.
          DrawFormattedText(APos,&Zeile[Pos],EndPos-Pos,Size,LZaehler,KlZaehler);
          Pos = EndPos;
          XPos += Size.x;
          SetTextColor(false,GetRedText(Pos,f,ZPos));
        }
        if (Zeile[Pos] != _TCHAR(TEFTextCalc::PosMarker))
        { // not evaluate end of partial string
          if (ValCPos && (Pos == f-(ZPos-m_Markiert.AkPos[m_Markiert.Count].Pos)))
          {
            if (m_Markiert.AkPos[m_Markiert.Count] == m_Markiert.GetEnd())
              SetCaretPos(XPos-XAnf+X0-1,YPos); // end position found

            if (!m_Markiert.ZeichenMarkiert() && !m_Markiert.MehrereZeilen())
              m_Markiert.Count++;
            m_Markiert.Count++;
          }
        }
        FehlerPos = BrZ.AkZ().FehlerPos &&
                    (Pos < f-(ZPos-BrZ.AkZ().FehlerPos)) &&
                    !BrZ.AkZ().Commentline &&
                    (Zeile[Pos] != _TCHAR(TEFTextCalc::PosMarker));
        ValCPos = (Pos < f-(ZPos-m_Markiert.AkPos[m_Markiert.Count].Pos)) &&
                  (m_Markiert.Count < 2) &&
                  (m_Markiert.AkPos[m_Markiert.Count].Zeile ==
                   BrZ.AkEintNummer()) &&
                  (m_Markiert.AkPos[m_Markiert.Count].Pos <= ZPos) &&
                  (Zeile[Pos] != _TCHAR(TEFTextCalc::PosMarker));
      }
      SetTextColor(m_Markiert.Count == 1,GetRedText(Pos,f,ZPos));
      wxSize Size(0,0);

      // Put out strings in reversed sequence for best alignment on screen
      // Under Windows, the length of the strings on output is different
      // if printed in more than one part, and it is dependent on the 
      // position where it is cut. It may be related to kerning and such.
      // Therefore, printing all strings alligned on the left is used.
      if ((StrCount > 0) && !Printing)
      {
        wxPoint APos1(XPos1-XAnf+X0,YPos);

        // Print complete string, with colors set for rightmost part
        SetTextColor(m_Markiert.Count == 1,GetRedText(EndStrPos[StrCount-1],f,ZPos));
        KlZaehler = KlZaehler1;
        LZaehler = LZaehler1;
        wxSize Size1;
        DrawFormattedText(APos1,&Zeile[Pos1],f-Pos1,Size1,LZaehler,KlZaehler);

        // Print the shorter strings, with colors set matching the string part
        do
        {
          SetTextColor(MarkCount[StrCount-1] == 1,GetRedText(StrCount > 1 ? EndStrPos[StrCount-2] : Pos1,f,ZPos));
          int KlZaehler2 = KlZaehler1;
          int LZaehler2 = LZaehler1;
          DrawFormattedText(APos1,&Zeile[Pos1],EndStrPos[StrCount-1]-Pos1,Size1,LZaehler2,KlZaehler2);
          StrCount--;
        }
        while (StrCount > 0);

        Pos = f;
        SetTextColor(m_Markiert.Count == 1,GetRedText(Pos,f,ZPos));
      }
      if (Pos < f)
      {
        wxPoint APos(XPos-XAnf+X0,YPos);
        DrawFormattedText(APos,&Zeile[Pos],f-Pos,Size,LZaehler,KlZaehler);
        XPos += Size.x;
      }
      SetTextColor(false,GetRedText(Pos,f,ZPos));
      Pos = f;
    }
    if (Zeile[Pos] == _TCHAR(TEFTextCalc::FontMarker))
    {
      Pos++;
      if (Zeile[Pos] != _TCHAR(TEFTextCalc::PageMarker))
      { // ignore page break
        SetFormatFont(Zeile[Pos],
                      Zeile[Pos] & TEFTextCalc::FontHeight ? Zeile[Pos+1] : 0);
        if (Zeile[Pos] & TEFTextCalc::FontHeight)
          Pos++;
      }
      Pos++;
    }
  }
  while ((Zeile[Pos] != 0) && (Zeile[Pos] != _TCHAR(TEFTextCalc::PosMarker)));
  SetFormatFont(TEFTextCalc::Normal);
  if (Zeile[OrgPos] == 0)
  {
    if (!Capt)
    { // output a blank char for blank lines
      SetTextColor(m_Markiert.Count == 1);
      DC->DrawText(wxString(wxT(" ")),XPos-XAnf+X0,YPos);
    }
  }
}

// Repaint the current line
void TEFEdit::RePaintAkZeile()
{
  wxRect arect;
  if (GetAkZeileRect(arect))
  { // if line visible
    GetDC();
    HideCaret();
    wxPen WPen(WINDOWCOLOUR,1,wxSOLID);
    wxBrush WBrush(WINDOWCOLOUR,wxSOLID);
    const wxBrush &OldBr = DC->GetBrush();
    DC->SetBrush(WBrush);
    DC->SetPen(WPen);
    // clear line background
    DC->DrawRectangle(arect.x,arect.y,arect.width,arect.height);
    DC->SetBrush(OldBr);
    DC->SetPen(wxNullPen);
    m_Markiert.SortPos();
    if ((m_Markiert.AkPos[0].Zeile < BrZ.AkEintNummer()) &&
        (m_Markiert.AkPos[1].Zeile >= BrZ.AkEintNummer()))
      m_Markiert.Count = 1;
    else
      m_Markiert.Count = 0;
    int AkPos = arect.GetTop();
    PaintZeile(AkPos); // paint current line
    DrawPageLine(*DC);
    ShowCaret();
    DelDC();
  }
}

int TEFEdit::PaintZeile(int &AkPos)
{ // output current line
  if (BrZ.AkZ().FormatError || BrZ.AkZ().Commentline ||
      !(ViewOptions & ViewBruchDarst))
  { // if error or no formatted output
    unsigned int XPos = 0,ZPos = 0,Pos = 0;

    if (IstPagebreak() && !Printing)
    {
      wxPen Pen(*wxLIGHT_GREY,1,wxDOT);
      DC->SetPen(Pen);
      DC->DrawLine(X0-XAnf,AkPos+FHeight*COUNTSPERLINE/2-1,
                   GetClientSize().GetWidth(),AkPos+FHeight*COUNTSPERLINE/2-1);
      DC->SetPen(wxNullPen);
      SetFormatFont(TEFTextCalc::Normal);
      wxColour OldCol = DC->GetTextForeground();
      DC->SetTextForeground(*wxLIGHT_GREY);
      DC->DrawText(wxString(_("Page break")),int(X0-XAnf+100*Zoom),AkPos);
      DC->SetTextForeground(OldCol);
    }
    unsigned int YPos;
    if (BrZ.AkZ().Commentline)
      YPos = AkPos;
    else
      YPos = BrZ.AkZ().ObZ[1]*FHeight+AkPos;
    TextSectionOut(BrZ.AkZ().FZeile.c_str(),Pos,XPos,YPos,ZPos);
    AkPos += NextAkPos();
    return XPos;
  }
  else
  { // draw the current (formatted) line
    unsigned int i,XPos,YPos,ZPos;
    int LZaehler = 0; // label or comment counter

    SetTextColor(m_Markiert.Count == 1);
    LastMidZ = -1;
    i = 0;
    ZPos = 0;
    XPos = 0;
    YPos = BrZ.AkZ().ObZ[1]*FHeight+AkPos;
    do
    {
      if (BrZ.AkZ().BZeile[i] == _TCHAR(TEFTextCalc::PosMarker))
      { // new position found
        i++;
        XPos = BrZ.AkZ().X[BruchCalcObj::GetBIndex(BrZ.AkZ().BZeile,i)];
        YPos = FHeight*(BrZ.AkZ().ObZ[1]+
               BrZ.AkZ().MidZ[BruchCalcObj::GetBIndex(BrZ.AkZ().BZeile,i)])+
               AkPos;
        LastMidZ = BrZ.AkZ().RefMidZ[BruchCalcObj::GetBIndex(BrZ.AkZ().BZeile,i)];
        i += 2; // ignore TEFTextCalc::PosMarker number
        if ((BrZ.AkZ().BZeile[i] == _TCHAR(TEFTextCalc::BruchMarker)) &&
            (BrZ.AkZ().BZeile[i+1] == _TCHAR(TEFTextCalc::PosMarker)))
        { // found already processed division line
          EvalCaretPos(XPos,YPos,ZPos);
          ZPos++;
          int Pwith = FHeight*COUNTSPERLINE/12;
          if (Pwith == 0)
            Pwith = 1;
          int XPos1 = XPos;
          int YPos1 = YPos;
          i += 2; // ignore \ and TEFTextCalc::PosMarker
          XPos = BrZ.AkZ().X[BruchCalcObj::GetBIndex(BrZ.AkZ().BZeile,i)];
          // determine end of division line
          YPos = FHeight*(BrZ.AkZ().ObZ[1]+
                 BrZ.AkZ().MidZ[BruchCalcObj::GetBIndex(BrZ.AkZ().BZeile,i)])+
                 AkPos;
          wxDC *tmpDC = (CDC ? CDC : DC);
          wxPen Pen(*wxBLACK,Pwith,wxSOLID);
          tmpDC->SetPen(Pen);
          tmpDC->DrawLine(XPos1+Pwith-XAnf+X0,
                          YPos1+FHeight*COUNTSPERLINE/2-Pwith/2-1,
                          XPos-Pwith-XAnf+X0,
                          YPos+FHeight*COUNTSPERLINE/2-Pwith/2-1);
          tmpDC->SetPen(wxNullPen);
          i += 2; // ignore TEFTextCalc::PosMarker number
        }
      }
      else
        TextSectionOut(BrZ.AkZ().BZeile.c_str(),i,XPos,YPos,ZPos,LZaehler);
    }
    while (i < BrZ.AkZ().BZeile.Length());
    SetFormatFont(TEFTextCalc::Normal);

    // Output a blank for empty lines
    i = 0;
    if (BrZ.AkZ().BZeile.Length() == 0)
      TextSectionOut(wxT(""),i,XPos,YPos,ZPos);

    SetTextColor(false);
    EvalCaretPos(XPos,YPos,ZPos);
    if (CDC)  // output a blank char for the clipboard
      TextSectionOut(wxT(" "),i,XPos,YPos,ZPos);
    AkPos += NextAkPos();
    return XPos;
  }
}

void TEFEdit::DrawCaretPosMarker(int x,int y,bool erase)
{
  if (DC && !m_Markiert.ZeichenMarkiert() && !m_Markiert.MehrereZeilen())
  { // only if DC available and no selection
    // background transparent so that carent stays visible
    wxPen Pen;
    if (erase)
      Pen = wxPen(WINDOWCOLOUR,1,wxSOLID);
    else
    {
      DC->SetBackgroundMode(wxTRANSPARENT);
      Pen = wxPen(wxColour(0x7F,0x7F,0x7F),1,wxSOLID);
    }
    DC->SetPen(Pen);
    DC->DrawLine(x,y,x+1,y);
    DC->DrawLine(x+1,y,x+1,y+FHeight*COUNTSPERLINE*5/6-1);
    DC->DrawLine(x+1,y+FHeight*COUNTSPERLINE*5/6-1,
                 x,y+FHeight*COUNTSPERLINE*5/6-1);
    DC->DrawLine(x,y+FHeight*COUNTSPERLINE*5/6-1,x,y);
    DC->SetPen(wxNullPen);
  }
}

// Set text colour (selected - not selected, red for syntax errors)
void TEFEdit::SetTextColor(bool highlight,bool redtext)
{
  if (highlight)
  {
    DC->SetTextBackground(HIGHLIGHT);
    if (redtext)
      DC->SetTextForeground(*wxRED);
    else
      DC->SetTextForeground(HIGHLIGHTTEXT);
  }
  else
  {
    DC->SetTextBackground(WINDOWCOLOUR);
    if (redtext)
      DC->SetTextForeground(*wxRED);
    else
      DC->SetTextForeground(WINDOWTEXT);
  }
}


