// Version 6.05 20.07.2008 Markus Greither
// Copyright (c) 1991-2008 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file efedprnt.cpp
    \brief Source code for the function editor printing code
*/


#include "efprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#ifdef __DMC__
#include <typeinfo.h>
#endif
#include <wx/wx.h>
#include <wx/resizec.h>
#include <wx/dnd.h>

#include <complex>
#include <limits>
#include <climits>
#include <set>
#include <functional>
#if defined(__MINGW32__)
#include <wx/msw/winundef.h>
#endif

using namespace std;

#include "efdefs.h"
#include "mg.h"
#include "cpxpsr.h"
#include "diagramm.h"
#include "efgemein.h"
#include "ef.h"
#include "efresid.h"
#include "eftabcon.h"
#include "efedit.h"

#endif

void TEFEdit::PaintChildren(wxDC &dc,int AAnfX,
                            int DrckFVisZeile,int DrckLVisZeile,
                            wxRect &ClipRect)
{
  wxWindowListNode *child = GetChildren().GetFirst();
  while (child)
  { // print all child windows
    wxRect rect;
    if (IsChildVisible((wxWindow *)child->GetData(),
                       DrckFVisZeile,DrckLVisZeile,AAnfX,FHeight,rect))
    { // if visible
      double x,y;
      dc.GetUserScale(&x,&y);
      dc.SetClippingRegion(ClipRect);
      dc.SetDeviceOrigin(int(x*rect.GetLeft()),int(y*rect.GetTop()));
      wxSize size(rect.GetSize());
      dc.SetClippingRegion(wxPoint(0,0),wxSize(size.x+1,size.y+1));
      wxResizeableControl *tmp = wxDynamicCast(child->GetData(),
                                               wxResizeableControl);
      tmp->Paint(dc,true,size);
      dc.DestroyClippingRegion();
      dc.SetDeviceOrigin(0,0);
      dc.SetUserScale(x,y);
    }
    child = child->GetNext();
  }
}

bool TEFEdit::VisibleChildren(int Zeile,int DrckFVisZeile,
                              int DrckLVisZeile,int FH)
{
  int Spalte = (Zeile-2)/VisYMax;
  int OrgXAnf = XAnf;
  XAnf = -10*PrntTL.x*PixPerInchX/254+X0+
         Spalte*(PrntBreite-(PrntTL.x+PrntBR.x))*10*PixPerInchX/254;
  if (DrckLVisZeile > 0)
    DrckLVisZeile--;
  else
    DrckLVisZeile = VisYMax;
  if (DrckLVisZeile == 0)
    DrckLVisZeile = VisYMax; // first line of next column found
  bool ret = false;
  wxWindowListNode *child = GetChildren().GetFirst();
  while (child)
  {
    wxRect rect;
    ret |= IsChildVisible((wxWindow *)child->GetData(),DrckFVisZeile,
                          DrckLVisZeile,OrgXAnf,FH,rect);
    child = child->GetNext();
  }
  XAnf = OrgXAnf;
  return ret;
}

bool TEFEdit::IsChildVisible(wxWindow *child,int DrckFVisZeile,
                             int DrckLVisZeile,int AAnfX,int FH,wxRect &rect)
{
  // Scrolled range for printing
  int AbstD = AbstVonAnf(DrckFVisZeile,FH);
  // Last visible line (in case of PAGEBREAK)
  int AbstU = AbstVonAnf(DrckLVisZeile+1,FH);
  rect = child->GetRect();
  rect.x += (AAnfX-X0);
  rect.y += (FirstVisPos);
  rect.x = int(rect.x/Zoom); // Zooming
  rect.width = int(rect.width/Zoom);
  rect.y = int(rect.y/Zoom);
  rect.height = int(rect.height/Zoom);
  int RHeightT = 10*PrntTL.y*PixPerInchY/254;
  int RHeightB = 10*PrntBR.y*PixPerInchY/254;
  int RWidthL = 10*PrntTL.x*PixPerInchX/254;
  int RWidthR = 10*PrntBR.x*PixPerInchX/254;
  rect.x += (-XAnf+X0);
  rect.y += (RHeightT-AbstD);
  wxResizeableControl *tmp = wxDynamicCast(child,wxResizeableControl);
  if ((PrntLaenge*10*PixPerInchY/254-RHeightB >= rect.GetTop()) &&
      (RHeightT <= rect.GetBottom()) && tmp &&
      (AbstU-AbstD+RHeightT > rect.GetTop()) &&
      (PrntBreite*10*PixPerInchY/254-RWidthR >= rect.GetLeft()) &&
      (RWidthL <= rect.GetRight()))
    return true;
  else
    return false;
}

// Printing routine
void TEFEdit::PaintPage(wxDC &dc,int Page)
{
  int Z = FirstVisZeile;
  int AXAnf = XAnf;
  int ACapt = Capt;
  Capt = false;
  float AOrgZoom = OrgZoom;
  float AZoom = Zoom;
  int AFHeight = FHeight;
  int AAveW = AveChrW;

  OrgZoom = 1;
  Zoom = 1;
  DC = &dc;
  SetFormatFont(TEFTextCalc::Normal,0,true);
  FontMetricStruct AFM = FontMetric;
  GetFontMetrics(FontMetric,*WFont);
  DC->SetFont(wxNullFont);
  DC = 0;
  FHeight = int((FontMetric.OrgHeight*FSizeFak+1)/COUNTSPERLINE);
  AveChrW = FontMetric.OrgAveCharW;
  dc.SetBackgroundMode(wxTRANSPARENT);
  int Zeile = 1;
  FirstVisZeile = 1;
  for (int i = 1;i < Page;i++)
    FirstVisZeile = NextPg(FHeight,Zeile); // first line
  int f1 = NextPg(FHeight,Zeile);

  if (f1 > 0)
    f1--; // determine last line
  else
    f1 = VisYMax;
  if (f1 == 0)
    f1 = VisYMax; // if first line of next column found

  int Spalte = (Zeile-2)/VisYMax;
  XAnf = -10*PrntTL.x*PixPerInchX/254+X0+
         Spalte*(PrntBreite-(PrntTL.x+PrntBR.x))*10*PixPerInchX/254;
  UpdateAll = false;
  // Make sure that cursor is not updated
  SortedTextSelection mtemp = m_Markiert;
  bool oldmo = m_Markiert.SetMacroOut(false);
  m_Markiert.AltPos.Zeile = FirstVisZeile;
  TextPos NeuAnf = m_Markiert.GetAnf(),NeuEnd = m_Markiert.GetEnd();

  NeuEnd.Zeile = f1;
  NeuAnf.Zeile = NeuEnd.Zeile+1;
  BrZ.SetAkEint(NeuEnd.Zeile);
  NeuEnd.Pos = BrZ.AkZ().FZeile.FLength()+1;
  NeuAnf.Pos = NeuEnd.Pos;
  m_Markiert.Set(NeuAnf,NeuEnd,false);
  wxRect cliprect(10*PrntTL.x*PixPerInchX/254,10*PrntTL.y*PixPerInchY/254,
                  (PrntBreite-(PrntTL.x+PrntBR.x))*10*PixPerInchX/254,
                  (PrntLaenge-(PrntTL.y+PrntBR.y))*10*PixPerInchY/254);
  dc.SetClippingRegion(cliprect);

  double x,y; // Change scale temporarily to fix a bug in wxWindows 2.4.X
  dc.GetUserScale(&x,&y); // that only shows when printing but not in preview
  dc.SetUserScale(x*2,y*2); // Symptom: much too small fonts
  dc.SetUserScale(x,y);

  wxRect rect(0,0,0,0); // Dummy values, because UpdateAll = false

  Paint(dc,false,rect);
  dc.DestroyClippingRegion();
  OrgZoom = AOrgZoom; // restore zoom for zooming of child windows
  Zoom = AZoom;
  if (!(ViewOptions & ViewHideDiags))
    PaintChildren(dc,AXAnf,FirstVisZeile,f1,cliprect);
  m_Markiert = mtemp;
  m_Markiert.SetMacroOut(oldmo);
  Capt = ACapt;
  XAnf = AXAnf;
  FHeight = AFHeight;
  AveChrW = AAveW;
  FirstVisZeile = Z;
  FontMetric = AFM;
  dc.SetFont(wxNullFont);
}

// Calculate document size for printer (no zoom), returns printer font height
int TEFEdit::SetPrintDocSize(wxDC &dc)
{
  float AZoom = Zoom;
  float AOrgZoom = OrgZoom;
  int AFHeight = FHeight;
  int AAveW = AveChrW;

  Zoom = 1;
  OrgZoom = 1;
  DC = &dc;
  SetFormatFont(TEFTextCalc::Normal,0,true);
  FontMetricStruct AFM = FontMetric;
  GetFontMetrics(FontMetric,*WFont);
  FHeight = int((FontMetric.OrgHeight*FSizeFak+1)/COUNTSPERLINE);
  AveChrW = FontMetric.OrgAveCharW;
  UpdateZeilen(1,BrZ.GetMaxEintrag()); // calculate lines for printer
  GetDocSize(AFHeight,AZoom); // Determine VisXMax and VisYMax for Zoom = 100%
  DC->SetFont(wxNullFont);
  CDC = &dc; // Clipboard-DC, for string calculation char by char
  DC = 0;
  CDC = 0;
  Zoom = AZoom;
  OrgZoom = AOrgZoom;
  int PHeight = FHeight;
  FHeight = AFHeight;
  AveChrW = AAveW;
  FontMetric = AFM;
  Printing = true;
  return PHeight;
}

void TEFEdit::SetScreenDocSize()
{
  Printing = false;
  GetDC();
  UpdateZeilen(1,BrZ.GetMaxEintrag()); // calculate lines for screen
  GetDocSize(); // determine VisXMax and VisYMax for screen zoom
  DelDC();
  Refresh(); // eventually restore caret position
}

// Get number of lines that fit on printer page, starting with line Zeile
int TEFEdit::NextPg(int fh,int &Zeile,bool *HasVisibleLine)
{
  int AkPos = 0;
  if (HasVisibleLine)
    *HasVisibleLine = false;
  int PrintWidth = (PrntBreite-(PrntTL.x+PrntBR.x))*10*PixPerInchX/254;
  int PrintLength = (PrntLaenge-(PrntTL.y+PrntBR.y))*10*PixPerInchY/254;

  int Spalte = 0;
  while (Zeile > VisYMax)
  { // get current column
    Zeile -= VisYMax;
    Spalte++;
  }
  int i1 = Zeile;
  do
  { // all lines
    if (Zeile <= BrZ.GetMaxEintrag())
    { // real, non-virtual line
      BrZ.SetAkEint(Zeile);
      AkPos += NextAkPos(fh);
      if (IstPagebreak())
      { // in case of page break return next line
        int erg = ++Zeile;
        Zeile += Spalte*VisYMax;
        return erg;
      }
      if (HasVisibleLine && (AkPos <= PrintLength) &&
          (BrZ.AkZ().LineLength > PrintWidth*Spalte))
        *HasVisibleLine = true;
    }
    else
      AkPos += fh*COUNTSPERLINE;
    if (AkPos > PrintLength)
    { // Printer page end reached
      int erg = ((Zeile > i1) ? Zeile : 0);
      Zeile += Spalte*VisYMax;
      return erg;
    }
    Zeile++;
  }
  while (Zeile <= VisYMax);
  if (Spalte < VisXMax/PrintWidth)
  { // not yet last column
    int erg = Zeile-VisYMax;
    Zeile += Spalte*VisYMax;
    return erg;
  }
  else
  { // last column
    Zeile += Spalte*VisYMax;
    return 0;
  }
}

void TEFEdit::GetPrintSetup(int &br,int &h,wxPoint &tplft,wxPoint &btmrgt,
                            bool &Or,bool &col)
{
  br = PrntBreite;
  h = PrntLaenge;
  tplft = PrntTL;
  btmrgt = PrntBR;
  Or = SeitenAusrichtung;
  col = ColorPrint;
}

// set page size and page direction of current printer
void TEFEdit::SetPrintSetup(int br,int h,wxPoint tplft,wxPoint btmrgt,
                            bool Or,bool col)
{
  modified |= (PrntBreite != br) || (PrntLaenge != h) ||
              (SeitenAusrichtung != Or) || (ColorPrint != col) ||
              (PrntTL != tplft) || (PrntBR != btmrgt);
  PrntBreite = br;
  PrntLaenge = h;
  PrntTL = tplft;
  PrntBR = btmrgt;
  SeitenAusrichtung = Or;
  ColorPrint = col;
  if (modified)
  {
    SendModified();
    EFInvalidate();
  }
}

void TEFEdit::Paint(wxDC& dc,bool Prntng,wxSize& size)
{
  if (Printing || !Prntng)
    return;
  SetPrintDocSize(dc);
  wxRect rect(0,0,size.x,size.y);
  UpdateAll = false;

  // make sure that cursor is not updated
  SortedTextSelection mtemp = m_Markiert;
  bool oldmo = m_Markiert.SetMacroOut(false);

  float AOrgZoom = OrgZoom;
  float AZoom = Zoom;
  int AFHeight = FHeight;
  int AAveW = AveChrW;

  OrgZoom = 1;
  Zoom = 1;
  DC = &dc;
  SetFormatFont(TEFTextCalc::Normal,0,true);
  FontMetricStruct AFM = FontMetric;
  GetFontMetrics(FontMetric,*WFont);
  DC->SetFont(wxNullFont);
  DC = 0;
  FHeight = int((FontMetric.OrgHeight*FSizeFak+1)/COUNTSPERLINE);
  AveChrW = FontMetric.OrgAveCharW;
  dc.SetBackgroundMode(wxTRANSPARENT);

  TextPos NeuAnf = m_Markiert.GetAnf(),NeuEnd = m_Markiert.GetEnd();

  m_Markiert.AltPos.Zeile = FirstVisZeile;
  NeuEnd.Zeile = LastVisZeile(FirstVisZeile);
  NeuAnf.Zeile = NeuEnd.Zeile+1;
  BrZ.SetAkEint(NeuEnd.Zeile);
  NeuEnd.Pos = BrZ.AkZ().FZeile.FLength()+1;
  NeuAnf.Pos = NeuEnd.Pos;
  m_Markiert.Set(NeuAnf,NeuEnd,false);

  CDC = &dc; // Clipboard-DC, for string output char by char
  Paint(dc,false,rect,Prntng);
  CDC = 0;

  m_Markiert = mtemp;
  m_Markiert.SetMacroOut(oldmo);
  Zoom = AZoom;
  OrgZoom = AOrgZoom;
  FHeight = AFHeight;
  AveChrW = AAveW;
  FontMetric = AFM;

  SetScreenDocSize();
}


