// Version 6.07  21.01.2011 Markus Greither
// Copyright (c) 1991-2011 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file efeqatns.cpp
    \brief Source code for the equations window class
*/


#include "efprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#include <wx/xrc/xmlres.h> // must be before any templates for DMC (?!)

#ifndef WX_PRECOMP
#ifdef __DMC__
#include <typeinfo.h>
#endif
#include <wx/wx.h>
#include <wx/resizec.h>

#include <wx/settings.h>
#include <wx/clipbrd.h>

#include <complex>
#include <limits>
#include <set>
#include <functional>
#if defined(__MINGW32__)
#include <wx/msw/winundef.h>
#endif

using namespace std;

#include "efdefs.h"
#include "mg.h"
#include "cpxpsr.h"
#include "eftabcon.h"
#include "efedit.h"
#include "efeqatns.h"
#include "diagramm.h"
#include "efgemein.h"
#include "ef.h"

#endif

// ----------------------------------------------------------------------------
// TEFChildEdit
// ----------------------------------------------------------------------------

void TEFChildEdit::OnKillFocus(wxFocusEvent &event)
{
  wxCommandEvent notification(wxEVT_COMMAND_HIDE_SIZERS);
  if (GetParent())
    GetParent()->GetEventHandler()->ProcessEvent(notification);
  event.Skip();
}

void TEFChildEdit::OnSetFocus(wxFocusEvent &event)
{
  wxCommandEvent notification(wxEVT_COMMAND_SHOW_SIZERS);
  if (GetParent())
    GetParent()->GetEventHandler()->ProcessEvent(notification);
  event.Skip();
}

void TEFChildEdit::OnContextMenu(wxMouseEvent &event)
{ // gibt Context-Menue aus
  wxPoint hp;
  hp = event.GetPosition();
  wxMenu *PMenue = wxGetApp().CreateCalcMenu(true);
  PMenue->AppendSeparator();
  wxMenuItem *item = new wxMenuItem(PMenue,XRCID("ICM_DELEDIT"),
                                    _("&Delete Window"),
                                    _("Delete this window"));
  PMenue->Append(item);
  PopupMenu(PMenue,hp);
  delete PMenue;
}

BEGIN_EVENT_TABLE(TEFChildEdit,TEFEdit)
  EVT_SET_FOCUS(TEFChildEdit::OnSetFocus)
  EVT_RIGHT_UP(TEFChildEdit::OnContextMenu)
  EVT_KILL_FOCUS(TEFChildEdit::OnKillFocus)
END_EVENT_TABLE()

IMPLEMENT_DYNAMIC_CLASS(TEFChildEdit, TEFEdit)

// ----------------------------------------------------------------------------
// TEFEquationsControl
// ----------------------------------------------------------------------------

IMPLEMENT_DYNAMIC_CLASS(TEFEquationsControl,TEFTabParentControl)

TEFEquationsControl::TEFEquationsControl(wxWindow *AParent, int AnId,
                                         const wxPoint &pos,
                                         const wxSize &size,long style,
                                         const NumListFormat &LDV,
                                         bool ScrollTrack)
 : TEFTabParentControl(AParent,AnId,pos,size,style)
{
  Init(LDV,ScrollTrack,size);
}

void TEFEquationsControl::Create(wxWindow *AParent, int AnId,
                                 const wxPoint &pos,
                                 const wxSize &size,long style,
                                 const NumListFormat &LDV,
                                 bool ScrollTrack)
{
  TEFTabParentControl::Create(AParent,AnId,pos,size,style);
  Init(LDV,ScrollTrack,size);
}

void TEFEquationsControl::Init(const NumListFormat &LDV,bool ScrollTrack,const wxSize &size)
{
  SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW));
#if wxCHECK_VERSION(2,5,0)
  ClearBackground();
#else
  Clear();
#endif
  TEFEdit *Editor;
  Editor = new TEFChildEdit((wxWindow *)this,1,
                             wxPoint(wxResizeableControl::SizeXRad*2,
                                     wxResizeableControl::SizeYRad*2),
                             wxSize(size.x-2*wxResizeableControl::SizeXRad*2,
                                    size.y-2*wxResizeableControl::SizeYRad*2),
                             wxCLIP_CHILDREN |
                             wxNO_FULL_REPAINT_ON_RESIZE | wxWANTS_CHARS |
                             wxVSCROLL | wxHSCROLL,
                             wxT("Equations"),LDV,TEFEdit::EquationStyle);

  Editor->SetViewOption(ScrollTrack,TEFEdit::ViewScrollTrack,false);
  AddManagedChild(Editor);
}

void TEFEquationsControl::OnKeyDown(wxKeyEvent &event)
{
  if (event.AltDown())
  {
    event.Skip();
    return;
  }
  switch (event.GetKeyCode())
  {
    case WXK_RIGHT :
    case WXK_LEFT :
    case WXK_UP :
    case WXK_DOWN :
    case WXK_RETURN :
      if (Editor() && !event.AltDown() && !event.ControlDown())
        Editor()->SetFocus();
      else
        event.Skip();
      break;
    default:
      event.Skip();
  }
}

void TEFEquationsControl::GetData(MGPStr &data)
{
  data.SetStr(wxT(""));
  for (size_t i = 0;i < Editor()->GetNumLines();i++)
  {
    data.AddStr(Editor()->GetLine(i));
    if (Editor()->GetNumLines() > i+1)
      data.AddStr(wxT("\r\n"));
  }
}

void TEFEquationsControl::Paint(wxDC &dc,bool Printing,wxSize &size)
{
  double x,y;
  dc.GetUserScale(&x,&y);
  wxCoord xo,yo;
  dc.GetDeviceOrigin(&xo,&yo);
  wxPoint pt(int(xo+wxResizeableControl::SizeXRad*2*x),
             int(yo+wxResizeableControl::SizeYRad*2*y));
// for some strange reason, changing the device origin under GTK breaks printing
#ifdef __WXMSW__
  dc.SetDeviceOrigin(pt.x,pt.y);
#endif
  Editor()->Paint(dc,Printing,size);
#ifdef __WXMSW__
  dc.SetDeviceOrigin(xo,yo);
#endif
  wxBrush OrgBr = dc.GetBrush();
  dc.SetBrush(*wxTRANSPARENT_BRUSH);
  dc.SetPen(*wxBLACK_PEN);
  dc.SetUserScale(1,1); // draw the rectangle in device resolution
  dc.DrawRectangle(0,0,wxCoord(size.x*x),wxCoord(size.y*y));
  dc.SetPen(wxNullPen);
  dc.SetBrush(OrgBr);
  dc.SetUserScale(x,y);
}

void TEFEquationsControl::OnPaint(wxPaintEvent &)
{
  wxPaintDC dc(this);
  dc.SetPen(*wxBLACK_PEN);
  wxBrush OrgBr = dc.GetBrush();
  dc.SetBrush(*wxTRANSPARENT_BRUSH);
  wxSize size = GetSize();
  dc.DrawRectangle(0,0,size.x,size.y);
  dc.SetPen(wxNullPen);
  dc.SetBrush(OrgBr);
  if (m_hasfocus)
    DrawSizeRect(dc);
}

BEGIN_EVENT_TABLE(TEFEquationsControl,TEFTabParentControl)
  EVT_KEY_DOWN(TEFEquationsControl::OnKeyDown)
  EVT_PAINT(TEFEquationsControl::OnPaint)
END_EVENT_TABLE()

