// Version 6.06  02.12.2009 Markus Greither
// Copyright (c) 1991-2009 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file efgemein.cpp
    \brief Source code for the data management classes
*/

#ifdef __BORLANDC__

#include "efprec.h"

#pragma hdrstop
#else

#ifdef __DMC__
#include <typeinfo.h>
#endif
#include <cstring>
#include <set>
#include <functional>
#include <cstdlib>
#include <cstdio>
#include <iostream>
#include <fstream>
#include <ctime>
#include <complex>
#include <limits>
#include <cfloat>
#include <string>

using namespace std;

#include "efdefs.h"
#include "mg.h"
#include "cpxpsr.h"
#include "diagtype.h"
#include "efgemein.h"
#include "efresid.h"

#endif

#if defined(__BORLANDC__) && defined(__WIN32__)
#include <excpt.h>
#endif

// ----------------------------------------------------------------------------
// File Constants
// ----------------------------------------------------------------------------

const char *GraphManagerObj::AltDateiVersion = "\6EFV1.2";

const char *GraphManagerObj::DatenDateiVersion = "\6EFV1.3";

const unsigned long GraphManagerObj::FarbArr[GraphManagerObj::MaxFarbe+1] =
{
  0x000000FFL,
  0x0000BF00L,
  0x00FF0000L,
  0x007F007FL,
  0x007F7F00L,
  0x00007F7FL
};

#if defined(UNICODE) && defined(__MINGW32__)
// _tfopen funktioniert nicht unter MINGW und UNICOWS
FILE *Mingw_tfopen(const _TCHAR *Name,const _TCHAR *Mode)
{
  char *buff = new char[_tcslen(Name)+1];
  char *buff2 = new char[_tcslen(Mode)+1];
  wcstombs(buff,Name,_tcslen(Name)+1);
  wcstombs(buff2,Mode,_tcslen(Mode)+1);
  FILE *Datei = fopen(buff,buff2);
  delete[] buff;
  delete[] buff2;
  return Datei;
}
#endif

// ----------------------------------------------------------------------------
// GraphDataObj
// ----------------------------------------------------------------------------

GraphDataObj::GraphDataObj()
{
  XAxisIndex[0] = 0;
  YAxisIndex[0] = 0;
  XAxisIndex[1] = 0;
  YAxisIndex[1] = 0;
  Punkte = 0;
  AdjustArraySize(0);
  VerString[0] = 0;
  FuString[0] = 0;
  FileString[0] = 0;
  Farbe = 0;
  GraphSymbol = GraphMitSymbol;
  GraphOptionen = GraphFett+GraphDia1+GraphDia2;
  ZeichenArt = 1;
}

bool GraphDataObj::AdjustArraySize(unsigned int pkt)
{
  X.resize(pkt > 1 ? pkt : 2);
  Y.resize(pkt > 1 ? pkt : 2);
  return true;
}

void GraphDataObj::SetPunkte(unsigned int pkt)
{
  AdjustArraySize(pkt);
  Punkte = pkt;
}

void GraphDataObj::IncPunkte()
{
  AdjustArraySize(Punkte+1);
  Punkte++;
}

void GraphDataObj::DecPunkte()
{
  Punkte--;
}

GraphDataObj::~GraphDataObj()
{
}

void GraphDataObj::CopyData(GraphDataObj *Ziel,bool CopyDaten) const
{
  _tcscpy(Ziel->FuString,FuString);
  _tcscpy(Ziel->FileString,FileString);
  strcpy(Ziel->VerString,VerString);
  Ziel->SetPunkte(Punkte);
  if (CopyDaten && Punkte)
  {
    Ziel->X = X;
    Ziel->Y = Y;
  }
  Ziel->GraphOptionen = GraphOptionen;
  Ziel->GraphSymbol = GraphSymbol;
  Ziel->ZeichenArt = ZeichenArt;
  Ziel->Farbe = Farbe;
  Ziel->XAxisIndex[0] = XAxisIndex[0];
  Ziel->YAxisIndex[0] = YAxisIndex[0];
  Ziel->XAxisIndex[1] = XAxisIndex[1];
  Ziel->YAxisIndex[1] = YAxisIndex[1];
}

void *GraphDataObj::ConvertToMemBlock(void *append,int *size)
{
  void *tmp,*aktmp;
  int aksize = sizeof(GraphDataObj)+GetXSize()+GetYSize();
  if (append)
  {
    tmp = realloc(append,*size+aksize);
    aktmp = &((char *)tmp)[*size];
  }
  else
  {
    tmp = malloc(aksize);
    aktmp = tmp;
  }
  if (size)
    *size += aksize;
  memcpy(aktmp,this,sizeof(GraphDataObj));
  double *xarr = (double *)&((char *)aktmp)[sizeof(GraphDataObj)];
  Cpx *yarr = (Cpx *)&((char *)aktmp)[sizeof(GraphDataObj)+GetXSize()];
  for (size_t i = 0;i < Punkte;i++)
  {
    xarr[i] = X[i];
    yarr[i] = Y[i];
  }
  return tmp;
}

size_t GraphDataObj::ConvertFromMemBlock(void *mblock)
{
  GraphDataObj *tmp = (GraphDataObj *)mblock;

  double *xarr = (double *)&((char *)mblock)[sizeof(GraphDataObj)];
  Cpx *yarr = (Cpx *)&((char *)mblock)[sizeof(GraphDataObj)+tmp->GetXSize()];
  tmp->CopyData(this,false);
  X.resize(Punkte);
  Y.resize(Punkte);
  for (size_t i = 0;i < Punkte;i++)
  {
    X[i] = xarr[i];
    Y[i] = yarr[i];
  }
  return sizeof(GraphDataObj)+GetXSize()+GetYSize();
}

// ----------------------------------------------------------------------------
// MaTyp
// ----------------------------------------------------------------------------

double MaTyp::GetFirst()
{
  if (Mode == ModeParList)
  {
    double tmp;
    AkPos = 0;
    GetNext(tmp);
    return tmp;
  }
  else
    return Anf;
}

bool MaTyp::NextStepAvail(const double &Ak)
{
  if ((Mode == ModeVar) || (Mode == ModeNo))
    return false;
  else
  if (Mode == ModeParList)
    return (AkPos < ParListe.Length());
  else
    return (Ak+Step <= Ende);
}

bool MaTyp::GetNext(double &Ak)
{ // Holt naechsten Parameter
  if ((Mode == ModeVar) || (Mode == ModeNo))
    return false;
  if (Mode != ModeParList)
  {
    if (Ak+Step <= Ende)
    {
      Ak += Step;
      return true;
    }
    else
      return false;
  }
  if (AkPos >= ParListe.Length())
    return false;
  ZahlFormatObj ZF;
  int Err = 0;
  Ak = ZF.Zahl(ParListe.c_str(),AkPos,Err);
  while ((ParListe[AkPos] == ' ') || (ParListe[AkPos] == ','))
    AkPos++;
  if (Err)
    return false;
  return true;
}

void MaTyp::ParBezeichner(MGPStr &HiString,double HiVar,ZahlFormatObj &ZF)
{
  MGPStr AkStr;

  if ((Mode == ModePar) || (Mode == ModeVar) || (Mode == ModeParList))
  {
    HiString.AddStr(_TEXT(" "));
    if (Bezeichner[0] != 0)
      HiString.AddStr(Bezeichner.c_str());
    else
      HiString.AddStr(_TEXT("Hi"));
    HiString.AddStr(_TEXT("="));
    ZF.Ausgabe(AkStr,HiVar,ZahlFormatObj::ScalForm);
    HiString.AddStr(AkStr);
    HiString.AddStr(Einheit.c_str());
  }
}

void MaTyp::FunktionsBezeichner(MGPStr &HiString,ZahlFormatObj &ZF)
{
  MGPStr AnfStr,EndeStr,Schritt;

  ZF.Ausgabe(AnfStr,Anf,ZahlFormatObj::ScalForm);
  ZF.Ausgabe(EndeStr,Ende,ZahlFormatObj::ScalForm);
  ZF.Ausgabe(Schritt,Step,ZahlFormatObj::ScalForm);
  switch (Mode)
  {
   case ModeFourier :
   case ModeSum :
     if (Bezeichner[0] != 0)
     {
       HiString.AddStr(_TEXT(" "));
       HiString.AddStr(Bezeichner.c_str());
     }
     else
     {
       if (Mode == ModeSum)
         HiString.AddStr(_TEXT(" S "));
       else
         HiString.AddStr(_TEXT(" F Hi="));
       HiString.AddStr(AnfStr);
       HiString.AddStr(_TEXT("-"));
       HiString.AddStr(EndeStr);
       HiString.AddStr(_TEXT(" d"));
       HiString.AddStr(Schritt);
       if (Einheit[0])
       {
         HiString.AddStr(_TEXT(" "));
         HiString.AddStr(Einheit.c_str());
       }
     }
     break;
   case ModeVar :
   case ModePar :
   case ModeParList :
     break;
   default :;
  }
}

// ----------------------------------------------------------------------------
// GraphManagerObj
// ----------------------------------------------------------------------------

void GraphManagerObj::MoveData(GraphManagerObj &Other)
{
  Other.DatFeld->Begin();
  DatFeld->Begin();
  if (Other.DatFeld->GetMaxEintrag() >= 0)
    do
    {
      DatFeld->CreateAkEint(Other.DatFeld->AkEintNummer());
      DatFeld->GetAkEint().MoveData(Other.DatFeld->GetAkEint());
    }
    while (Other.DatFeld->NextEint());
  PstPrc = Other.PstPrc;
  SetLDV(Other.GetLDV());
}

void GraphManagerObj::UniLength(MGPStr &x,unsigned int y)
{
  while (x.Length() < y)
    x.AddStr(_TEXT(" "));
}

void *GraphManagerObj::CopyAllGraphs(ParserObj &FuObj,int &size)
{
  int AkPos = DatFeld->AkEintNummer();
  GraphDataObj OrgData;
  AkFeld.CopyData(&OrgData,true);
  DatFeld->Begin();
  void *data = 0;
  size = 0;
  if (DatFeld->GetMaxEintrag() >= 0)
    do
    {
      DatFeld->GetAkEint().CopyData(&AkFeld,true);
      AkFeldUmrechnen(FuObj);
      data = AkFeld.ConvertToMemBlock(data,&size);
    }
    while (DatFeld->NextEint());
  DatFeld->SetAkEint(AkPos);
  OrgData.CopyData(&AkFeld,true);
  return data;
}

void GraphManagerObj::WriteFeld(ostream &Datei)
{
  const unsigned short Num = 0;
  int Len;

  Datei.write((char *)&AkFeld.VerString,sizeof(AkFeld.VerString));
  if ((AkFeld.GraphOptionen & GraphDataObj::TextDatDatei) > 0)
  {
    char filestring[256];
#ifdef UNICODE
    wcstombs(filestring,AkFeld.FileString,_tcslen(AkFeld.FileString)+1);
#else
    strcpy(filestring,AkFeld.FileString);
#endif
    Datei.write((char *)&Num,sizeof(unsigned short));
    Len = strlen(filestring);
    Datei.write((char *)&Len,1);
    Datei.write((char *)filestring,sizeof(filestring)-1);
  }
  else
  {
    unsigned short tmp = (unsigned short)AkFeld.GetPunkte();
    Datei.write((char *)&tmp,sizeof(unsigned short));
    Datei.write((char *)&AkFeld.GetX(0),
                AkFeld.GetPunkte()*sizeof(AkFeld.GetX(1)));
    Datei.write((char *)&AkFeld.GetY(0),
                AkFeld.GetPunkte()*sizeof(AkFeld.GetY(1)));
  }
  char fustring[256];
#ifdef UNICODE
    wcstombs(fustring,AkFeld.FuString,_tcslen(AkFeld.FuString)+1);
#else
    strcpy(fustring,AkFeld.FuString);
#endif
  Len = strlen(fustring);
  Datei.write((char *)&Len,1);
  Datei.write((char *)&fustring,sizeof(fustring)-1);
  Datei.write((char *)&AkFeld.GraphSymbol,sizeof(AkFeld.GraphSymbol));
  Datei.write((char *)&AkFeld.GraphOptionen,sizeof(AkFeld.GraphOptionen));
  Datei.write((char *)&AkFeld.ZeichenArt,sizeof(AkFeld.ZeichenArt));
  Datei.write((char *)&AkFeld.Farbe,sizeof(AkFeld.Farbe));
}

bool GraphManagerObj::ValidGraphDaten()
{
  return ((strlen((*DatFeld)->VerString) > 0) &&
          ((strncmp(DatenDateiVersion,(*DatFeld)->VerString,
                    strlen(DatenDateiVersion)) == 0) ||
           (strncmp(AltDateiVersion,(*DatFeld)->VerString,
                    strlen(AltDateiVersion)) == 0)));
}

bool GraphManagerObj::LoadFeld(istream &Datei)
{
  unsigned char l;
  int AkFarbe;

  long AkPos = Datei.tellg(); // aktuelle Stream-Position speichern
  (*DatFeld)->VerString[0] = 0;
  Datei.read((char *)&(*DatFeld)->VerString,sizeof((*DatFeld)->VerString));
  if (!ValidGraphDaten())
  { // wenn keine Graph-Daten, abbrechen
    if (Datei.gcount() == (int)sizeof((*DatFeld)->VerString))
      Datei.seekg(AkPos);
    return false;
  }
  unsigned short tmp;
  Datei.read((char *)&tmp,sizeof(unsigned short));
  (*DatFeld)->SetPunkte(tmp);
  if ((*DatFeld)->GetPunkte() <= GraphDataObj::MaxPunkte)
  {
    if ((*DatFeld)->GetPunkte() == 0) // Graph in Text-Datei, Dateinamen Laden
    {
      char filestring[256];
      Datei.read((char *)&l,sizeof(l));
      Datei.read((char *)&filestring,sizeof(filestring)-1);
      filestring[l] = 0;
#ifdef UNICODE
      mbstowcs((*DatFeld)->FileString,filestring,strlen(filestring)+1);
#else
      _tcscpy((*DatFeld)->FileString,filestring);
#endif
      if (!LoadGraph((*DatFeld)->FileString,0,false,false,true))
      {
        Datei.read((char *)&l,sizeof(l));
        char fustring[256];
        Datei.read((char *)&fustring,sizeof(fustring)-1);
        (*DatFeld)->SetX(0,0);
        (*DatFeld)->SetY(0,Cpx(0,0));
        (*DatFeld)->SetX(1,0);
        (*DatFeld)->SetY(1,Cpx(0,0));
        (*DatFeld)->SetPunkte(2);
        _tcscpy((*DatFeld)->FuString,_TEXT("WERTE-DATEI NICHT GEFUNDEN"));
      }
      else
      {
        Datei.read((char *)&l,sizeof(l));
        char fustring[256];
        Datei.read((char *)&fustring,sizeof(fustring)-1);
        fustring[l] = 0;
#ifdef UNICODE
        mbstowcs((*DatFeld)->FuString,fustring,strlen(fustring)+1);
#else
        _tcscpy((*DatFeld)->FuString,fustring);
#endif
      }
    }
    else
    {
      (*DatFeld)->FileString[0] = 0;
      Datei.read((char *)&(*DatFeld)->GetX(0),
                 (*DatFeld)->GetPunkte()*sizeof((*DatFeld)->GetX(0)));
      Datei.read((char *)&(*DatFeld)->GetY(0),
                 (*DatFeld)->GetPunkte()*sizeof((*DatFeld)->GetY(0)));
      Datei.read((char *)&l,sizeof(l));
      char fustring[256];
      Datei.read((char *)&fustring,sizeof(fustring)-1);
      fustring[l] = 0;
#ifdef UNICODE
      mbstowcs((*DatFeld)->FuString,fustring,strlen(fustring)+1);
#else
      _tcscpy((*DatFeld)->FuString,fustring);
#endif
    }
    Datei.read((char *)&(*DatFeld)->GraphSymbol,sizeof((*DatFeld)->GraphSymbol));
    Datei.read((char *)&(*DatFeld)->GraphOptionen,sizeof((*DatFeld)->GraphOptionen));
    if (!((*DatFeld)->GraphOptionen &
          (GraphDataObj::GraphDia1 | GraphDataObj::GraphDia2)))
      (*DatFeld)->GraphOptionen |= GraphDataObj::GraphDia1+GraphDataObj::GraphDia2;
    Datei.read((char *)&(*DatFeld)->ZeichenArt,sizeof((*DatFeld)->ZeichenArt));
    if (strcmp(DatenDateiVersion,(*DatFeld)->VerString) == 0)
      Datei.read((char *)&(*DatFeld)->Farbe,sizeof((*DatFeld)->Farbe));
    else
    {
      AkFarbe = DatFeld->AkEintNummer();
      while (AkFarbe > MaxFarbe)
        AkFarbe -= (MaxFarbe+1);
      (*DatFeld)->Farbe = FarbArr[AkFarbe];
    }
  }
  return true;
}

void GraphManagerObj::Init(int PostProc,
                           const NumListFormat &iLDV)
{
  SetLDV(iLDV);
  PstPrc = PostProc;
  ResetYDaten();
}

void GraphManagerObj::ResetYDaten()
{
  DatFeld->Begin();
  if (DatFeld->GetMaxEintrag() >= 0)
    CopyFeldData(&DatFeld->GetAkEint(),&AkFeld,true);
}

bool GraphManagerObj::ReloadGraph()
{ // laedt den aktuellen Graphen ins Feld AkFeld
  if (DatFeld->GetMaxEintrag() >= 0)
    CopyFeldData(&DatFeld->GetAkEint(),&AkFeld,true);
  return true;
}

void GraphManagerObj::ModifyGraph(const GraphDataObj *YT,
                                  bool AlleGr,bool CopyDaten)
{
  int AkPos;

  AkPos = DatFeld->AkEintNummer();
  if (AlleGr)
  {
    DatFeld->Begin();
    if (DatFeld->GetMaxEintrag() >= 0)
      do
      {
        DatFeld->GetAkEint().GraphSymbol = YT->GraphSymbol;
        DatFeld->GetAkEint().GraphOptionen =
                           (unsigned short)((YT->GraphOptionen &
                           ~(unsigned short)(GraphDataObj::TextDatDatei |
                                             GraphDataObj::PstPrcOff)) |
                           (DatFeld->GetAkEint().GraphOptionen &
                           (unsigned short)(GraphDataObj::TextDatDatei |
                                            GraphDataObj::PstPrcOff)));
        DatFeld->GetAkEint().ZeichenArt = YT->ZeichenArt;
      }
      while (DatFeld->NextEint());
  }
  DatFeld->SetAkEint(AkPos);
  CopyFeldData(YT,&DatFeld->GetAkEint(),CopyDaten);
}

bool GraphManagerObj::YDatenVorhanden()
{
  return DatFeld->GetMaxEintrag() >= 0;
}

bool GraphManagerObj::LoadYDatei(istream &Datei)
{
  bool geladen = true;

  DatFeld->ResetKette();
  DatFeld->Begin();
  if (Datei)
  {
    while (geladen && !Datei.eof())
    {
      DatFeld->NeuAkEint();
      geladen = LoadFeld(Datei);
      if (!geladen)
        DatFeld->LoeschAkEint();
    }
    return true;
  }
  else
    return false;
}

bool GraphManagerObj::WriteYDatei(ostream &Datei)
{
  if (DatFeld->GetMaxEintrag() >= 0)
  {
     DatFeld->Begin();
     strcpy(AkFeld.VerString,DatenDateiVersion);
     if (Datei.good())
     {
       GraphDataObj *AltFeld = new GraphDataObj;
       CopyFeldData(&AkFeld,AltFeld,true);// bereits umgerechnetes Feld retten
       do
       {
         CopyFeldData(&DatFeld->GetAkEint(),&AkFeld,true);
         WriteFeld(Datei);
       }
       while (DatFeld->NextEint());
       CopyFeldData(AltFeld,&AkFeld,true);
       delete AltFeld;
       return true;
     }
     else
       return false;
  }
  else
    return false;
}

void GraphManagerObj::LoeschAkGraph()
{
  DatFeld->LoeschAkEint();
  DatFeld->Begin();
}

void GraphManagerObj::GetAkFeldData(GraphDataObj *Ziel)
{ // Kopiert den Graphen in AkFeld in Variable Ziel
  AkFeld.CopyData(Ziel,true);
}

// Kopiert den Graphen in Quelle nach AkFeld,
// CopyDaten = true wenn auch Daten kopiert werden sollen
void GraphManagerObj::SetAkFeldData(const GraphDataObj *Quelle,bool CopyDaten)
{
  Quelle->CopyData(&AkFeld,CopyDaten);
}

void GraphManagerObj::CopyFeldData(const GraphDataObj *Quelle,
                                   GraphDataObj *Ziel,bool CopyDaten)
{
  Quelle->CopyData(Ziel,CopyDaten);
}

void GraphManagerObj::AppendYFeld(const GraphDataObj *Feld,bool KeepColor)
{
  DatFeld->Begin();
  ShortArrObj FarbFeld;
  int i;
  if (DatFeld->GetMaxEintrag() >= 0)
    do
    { // Alle verwendeten Farben in FarbFeld eintragen
      i = 0;
      while (i <= MaxFarbe)
      { // Farbe des aktuellen Feldes mit allen moeglichen Farben vergleichen
        if (DatFeld->GetAkEint().Farbe == FarbArr[i])
          FarbFeld.IncValue(i); // wenn vorhanden, eintragen
        i++;
      }
    }
    while (DatFeld->NextEint());
  DatFeld->NeuAkEint();
  CopyFeldData(Feld,&DatFeld->GetAkEint(),true);
  int AkFarbe = FarbFeld.MinNr(); // Farbe die am seltensten vorkommt eintragen
  if ((FarbFeld.MaxInt() < MaxFarbe) && (FarbFeld[AkFarbe] != 0))
    AkFarbe = FarbFeld.MaxInt()+1;
  if (!KeepColor)
    DatFeld->GetAkEint().Farbe = FarbArr[AkFarbe];
}

bool GraphManagerObj::LoadYFeld(int nr)
{
  if (nr == -1)
  { // naechste Funktion laden
    if (DatFeld->NextEint())
    {
      CopyFeldData(&DatFeld->GetAkEint(),&AkFeld,true);
      return true;
    }
    else
      return false;
  }
  else
  { // Funktion nr laden
    if ((nr > 0) && (nr-1 <= DatFeld->GetMaxEintrag()))
    {
      DatFeld->SetAkEint(nr-1);
      CopyFeldData(&DatFeld->GetAkEint(),&AkFeld,true);
      return true;
    }
    else
      return false;
  }
}

void GraphManagerObj::LoeschYDaten()
{
  DatFeld->ResetKette();
}

GraphManagerObj::~GraphManagerObj()
{
  delete DatFeld;
}

void GraphManagerObj::AkFeldUmrechnen(ParserObj &FuObj)
{
  if ((PstPrc == 1) &&
      ((AkFeld.GraphOptionen & GraphDataObj::PstPrcOff) == 0) &&
      FuObj.AkFunktionVorhanden())
    for (unsigned int i = 0;i < AkFeld.GetPunkte();i++)
    {
      Cpx z;
      if (!FuObj.Ergebnis(AkFeld.GetX(i),AkFeld.GetY(i),z))
        AkFeld.SetY(i,Cpx(0,0));
      else
        AkFeld.SetY(i,z);
    }
}

bool GraphManagerObj::SkipSeparators(MGPStr &MGZeile,unsigned int &Position,
                                     NumListFormat &AkLDV, 
                                     bool TabSeparator, bool SpaceSeparator)
{
  _TCHAR sep = AkLDV.ListSeparator.c_str()[0];
  bool sepavailable = false;
  while ((Position <= MGZeile.Length()) &&
         ((MGZeile[Position] == ' ') && SpaceSeparator ||
          (MGZeile[Position] == 9) && TabSeparator ||
          (MGZeile[Position] == sep)))
  {
    if (MGZeile[Position] == sep)
    {
      if (!_tcsncmp(&MGZeile[Position],AkLDV.ListSeparator.c_str(),
                    AkLDV.ListSeparator.Length()))
        Position += AkLDV.ListSeparator.Length();
      else
        break;
    }
    else
      Position++;
    sepavailable = true;
  }
  return sepavailable;
}

void GraphManagerObj::GetLabelEnd(MGPStr &MGZeile,unsigned int &Position,
                                  NumListFormat &AkLDV,
                                  bool TabSeparator, bool SpaceSeparator)
{
  _TCHAR sep = AkLDV.ListSeparator.c_str()[0];
  while ((Position < MGZeile.Length()) &&
         (((MGZeile[Position] != ' ') || !SpaceSeparator) &&
          ((MGZeile[Position] != 9) || !TabSeparator) &&
          (MGZeile[Position] != sep)))
    Position++;
}

bool GraphManagerObj::LoadGraph(const _TCHAR *Name,const _TCHAR *Editor,
                                bool Sortieren,bool InDatDatei,
                                bool ExtDatDatei,bool NeuGraph,
                                bool ImagDataIncluded, bool HeaderIncluded,
                                NumListFormat *LDV,
                                bool XValuesIncluded, int XColumn,
                                bool TabSep, bool SpaceSep)
{
  ifstream datei;
  string zeile;
  MGPStr MGZeile;
  unsigned int Position;
  unsigned long Punkte = 0;
  const unsigned int PunkteStep = 20000;
  bool ErsteZeile;
  int Fehler;
  Cpx HiY;
  double HiX;
  bool Gewechselt;
  vector<GraphDataObj> Feld;
  NumListFormat OrgLDV,AkLDV;
  const _TCHAR *Ed = Editor;
  bool ret = false;

  if (Name)
  {
    char *nstr;
    nstr = new char[_tcslen(Name)+1];
    ConvertStr(Name,nstr);
    datei.open(nstr,ios::in);
    delete[] nstr;
    if (!datei)
      return false;
  }
  Feld.resize(1);
  if ((Editor || Name) && YDatenVorhanden() && !NeuGraph)
    DatFeld->GetAkEint().CopyData(&Feld[0],true);
  else
    Feld[0].FuString[0] = 0;
  if (datei || Editor)
  {
    ErsteZeile = true;
    OrgLDV = GetLDV();
    if (LDV)
      AkLDV = *LDV;
    else
      AkLDV = GetLDV();

    SetLDV(AkLDV);
    Feld[0].SetPunkte(PunkteStep);
    Feld[0].GraphOptionen = 0;
    while ((Ed || (datei && !datei.eof())) &&
           (Punkte < GraphDataObj::MaxPunkte))
    {
      Fehler = ZahlFormatObj::NoError;
      if (Ed)
      {
        const _TCHAR *Tst = _tcsstr(Ed,_TEXT("\r\n"));
        if (Tst)
        { // Found line break
          unsigned long i = 0;
          MGZeile.SetStr(_TEXT(""));
          while (i < (unsigned long)(Tst-Ed))
          { // Copy characters (faster than strncpy)
            MGZeile.AddChar(Ed[i]);
            i++;
          }
          Ed = Tst+2;
        }
        else
        { // last line
          MGZeile.SetStr(Ed);
          Tst = Ed;
          Ed = 0;
        }
        if (!Tst)
          break;
      }
      else
      if (getline(datei,zeile))
        MGZeile.SetStr(ConvertStr(zeile.c_str()));
      else
        break;

      if (ErsteZeile)
      {
        if (HeaderIncluded)
        {
          unsigned headcount = 0;
          int column = 1;
          Position = 0;
          do
          {
            SkipSeparators(MGZeile,Position,AkLDV,TabSep,SpaceSep);
            if (XValuesIncluded && (column == XColumn))
              GetLabelEnd(MGZeile,Position,AkLDV,TabSep,SpaceSep);
            else
            {
              headcount++;
              if (Position < MGZeile.Length())
              {
                if (headcount > Feld.size())
                {
                  Feld.resize(headcount);
                  Feld[headcount-1].SetPunkte(Feld[0].GetPunkte());
                  Feld[headcount-1].GraphOptionen = 0;
                }
                unsigned int endpos = Position;
                GetLabelEnd(MGZeile,endpos,AkLDV,TabSep,SpaceSep);
                _tcscpy(Feld[headcount-1].FuString,&MGZeile.c_str()[Position]);
                Feld[headcount-1].FuString[endpos-Position] = 0;
                Position = endpos;
              }
            }
            column++;
          }
          while (Position < MGZeile.Length());
          if (!datei.eof())
            getline(datei,zeile);
          MGZeile.SetStr(ConvertStr(zeile.c_str()));
        }
        else
        if (strncmp(PROGRAMSTRING,zeile.c_str(),strlen(PROGRAMSTRING)) == 0)
        { // GRD-Format
          // Ignore blank line
          if (!datei.eof())
            getline(datei,zeile);
          // Ignore date
          if (!datei.eof())
            getline(datei,zeile);
          // Read separators
          if (strncmp("Trennzeichen",zeile.c_str(),12) == 0)
          {
            unsigned int i = 0;
            while ((i < zeile.length()) && (zeile.c_str()[i] != ' '))
              i++;
            if (i < zeile.length())
            {
              AkLDV.ListSeparator.SetStr(_TEXT(""));
              AkLDV.ListSeparator.AddChar(zeile.c_str()[i+1]);
              AkLDV.DecSeparator.SetStr(_TEXT(""));
              AkLDV.DecSeparator.AddChar(zeile.c_str()[i+2]);
              AkLDV.LeadZero = zeile.c_str()[i+3];
            }
          }
          // Get label
          if (!datei.eof())
            getline(datei,zeile);
          _tcscpy(Feld[0].FuString,ConvertStr(zeile.c_str()).c_str());
          // Get next line
          if (!datei.eof())
            getline(datei,zeile);
          MGZeile.SetStr(ConvertStr(zeile.c_str()));
          SetLDV(AkLDV);
        }
        else
        if (InDatDatei && !Editor)
          _tcscpy(Feld[0].FuString,_TEXT(""));
        ErsteZeile = false;
      }

      Punkte++;
      if (Feld[0].GetPunkte() < Punkte)
        for (unsigned int akgr = 0;akgr < Feld.size();akgr++)
          Feld[akgr].SetPunkte(Punkte+PunkteStep);
      ret = true;
      Feld[0].SetX(Punkte-1,0);
      Feld[0].SetY(Punkte-1,Cpx(0,0));
      Position = 0;

      unsigned akgr = 0;
      int column = 1;
      SkipSeparators(MGZeile,Position,AkLDV,TabSep,SpaceSep);
      // read all graphs
      do
      {
        // Read X value if this is the X column
        if (XValuesIncluded && (XColumn == column))
        {
          double res = ZahlOhneBruch(MGZeile.c_str(),Position,Fehler);
          // Go back to last separator, as a blank might have been skipped by ZahlOhneBruch
          if ((Fehler == NoError) && (Position > 0) && (MGZeile.c_str()[Position-1] == ' '))
            Position--;
          if ((Position < MGZeile.Length()) &&
              !SkipSeparators(MGZeile,Position,AkLDV,TabSep,SpaceSep) || (Fehler != NoError))
            Fehler = WrongValue;
          else
            Feld[0].SetX(Punkte-1,res);
        }
        else // Read y value
        {
          if (Position < MGZeile.Length())
          {
            double res = ZahlOhneBruch(MGZeile.c_str(),Position,Fehler);
            if (akgr >= Feld.size())
            {
              Feld.resize(akgr+1);
              Feld[akgr].SetPunkte(Feld[0].GetPunkte());
              Feld[akgr].GraphOptionen = 0;
            }
            // Go back to last separator, as a blank might have been skipped by ZahlOhneBruch
            if ((Fehler == NoError) && (Position > 0) && (MGZeile.c_str()[Position-1] == ' '))
              Position--;
            if ((Position < MGZeile.Length()) &&
                !SkipSeparators(MGZeile,Position,AkLDV,TabSep,SpaceSep) || (Fehler != NoError))
            {
              GetLabelEnd(MGZeile,Position,AkLDV,TabSep, SpaceSep);
              Fehler = NoError;
              Feld[akgr].GraphOptionen = 1;
            }
            else
              Feld[akgr].SetY(Punkte-1,Cpx(res,0));
          }
          else
          if (akgr == 0)
            Fehler = WrongValue;

          // read imaginary part if included
          if (ImagDataIncluded && (Fehler == NoError))
          {
            if (Position < MGZeile.Length())
            {
              double res = ZahlOhneBruch(MGZeile.c_str(),Position,Fehler);
              // Go back to last separator, as a blank might have been skipped by ZahlOhneBruch
              if ((Fehler == NoError) && (Position > 0) && (MGZeile.c_str()[Position-1] == ' '))
                Position--;
              if ((Position < MGZeile.Length()) &&
                  !SkipSeparators(MGZeile,Position,AkLDV,TabSep,SpaceSep) || (Fehler != NoError))
              {
                GetLabelEnd(MGZeile,Position,AkLDV,TabSep, SpaceSep);
                Fehler = NoError;
                Feld[akgr].GraphOptionen = 1;
              }
              else
                Feld[akgr].SetY(Punkte-1,Cpx(real(Feld[0].GetY(Punkte-1)),res));
            }
          }
          akgr++;
        }
        column++;
      }
      while ((Position < MGZeile.Length()) && (Fehler == NoError));

      if (!XValuesIncluded)
        Feld[0].SetX(Punkte-1,Punkte);

      if (Fehler != NoError)
        Punkte--;
    }
    SetLDV(OrgLDV);

    // Copy X values to all graphs
    for (unsigned akgr = 0;akgr < Feld.size();akgr++)
    {
      Feld[akgr].SetPunkte(Punkte);
      if (akgr > 0)
        for (unsigned int f = 0;f < Feld[0].GetPunkte();f++)
          Feld[akgr].SetX(f,Feld[0].GetX(f));
    }

    // Remove graphs that had reading errors
    vector<GraphDataObj>::iterator it = Feld.begin();
    while (it != Feld.end())
      if (it->GraphOptionen == 1)
        it = Feld.erase(it);
      else
        it++;

    if (datei.is_open())
      datei.close();

    // A graph needs to have a minimum of 2 points
    if ((Feld[0].GetPunkte() >= 2) &&
        (Feld[0].GetPunkte() <= GraphDataObj::MaxPunkte))
    {
      if (Sortieren)
      {
        do
        {
          Gewechselt = false;
          for (unsigned int i = 1;i < Feld[0].GetPunkte();i++)
          {
            if (Feld[0].GetX(i) < Feld[0].GetX(i-1))
            {
              for (unsigned akgr = 0;akgr < Feld.size();akgr++)
              {
                HiX = Feld[akgr].GetX(i);
                HiY = Feld[akgr].GetY(i);
                Feld[akgr].SetX(i,Feld[akgr].GetX(i-1));
                Feld[akgr].SetY(i,Feld[akgr].GetY(i-1));
                Feld[akgr].SetX(i-1,HiX);
                Feld[akgr].SetY(i-1,HiY);
              }
              Gewechselt = true;
            }
          }
        }
        while (Gewechselt);
      }

      // Setup graph properties and store it
      for (unsigned akgr = 0;akgr < Feld.size();akgr++)
      {
        if (NeuGraph)
        {
           Feld[akgr].GraphSymbol = GraphDataObj::GraphMitSymbol;
           Feld[akgr].GraphOptionen = GraphDataObj::GraphFett+
                                      GraphDataObj::GraphDia1+
                                      GraphDataObj::GraphDia2;
           Feld[akgr].ZeichenArt = GraphDataObj::gPunkteVerbinden;
        }

        // For now, only old format graphs (no multi graph files) can be auto reloaded from external files
        if (ExtDatDatei && (Feld.size() == 1) && ImagDataIncluded)
        {
          Feld[akgr].GraphOptionen |= GraphDataObj::TextDatDatei;
          _tcscpy(Feld[akgr].FileString,Name);
        }
        else
          Feld[akgr].FileString[0] = 0;
        if (InDatDatei)
        {
          AppendYFeld(&Feld[akgr]);
          // Downsize the field to save memory for loading a large number of huge graphs
          Feld[akgr].SetPunkte(2);
          ReloadGraph();
        }
        else
          Feld[akgr].CopyData(&DatFeld->GetAkEint(),true);
      }
    }
    else
      ret = false;
  }
  return ret;
}

bool GraphManagerObj::WriteGraph(const _TCHAR *Name,MGPStr *Editor,
                                 bool GRD_Format,bool TAB,
                                 bool Komma,bool IngNotation)
{
    FILE *Datei;
    char Zeile[255];
    MGPStr SWert;
    MGPStr MGZeile;
    unsigned int Position;
    MGPStr TrennZ;
    int AusgForm;
    time_t timer;
    struct tm *TS;

    if (Name)
#if defined(UNICODE) && defined(__MINGW32__)
      Datei = Mingw_tfopen(Name,_TEXT("w"));
#else
      Datei = _tfopen(Name,_TEXT("w"));
#endif
    else
      Datei = 0;
    if (Datei || Editor)
    {
      if (TAB)
        TrennZ = _TEXT("\t");
      if (Komma)
        TrennZ = GetLDV().ListSeparator;
      if (TrennZ[0] == 0)
        TrennZ = _TEXT(" ");
      if (Editor)
      {
        MGZeile.SetStr(_TEXT(""));
        SWert.SetStr(_TEXT("X-Werte"));
        UniLength(SWert,11);
        MGZeile.AddStr(SWert);
        MGZeile.AddStr(TrennZ);
        SWert.SetStr(_TEXT("Y-Real"));
        UniLength(SWert,11);
        MGZeile.AddStr(SWert);
        MGZeile.AddStr(TrennZ);
        SWert.SetStr(_TEXT("Y-Imag"));
        UniLength(SWert,11);
        MGZeile.AddStr(SWert);
        MGZeile.AddStr(_TEXT("\r\n"));
        Editor->AddStr(MGZeile);
      }
      if (GRD_Format)
      {
         fputs(PROGRAMSTRING,Datei);
         fputs(" V ",Datei);
         fputs(VERSIONSTRING,Datei);
         fputs(" - Graph-Datei\n",Datei);
         timer = time(NULL);
         TS = localtime(&timer);
         fputs("Erstellt am ",Datei);
         fprintf(Datei,"%02hi.%02hi.%4i  %02hi:%02hi:%02hi\n",
                 TS->tm_mday,TS->tm_mon+1,TS->tm_year+1900,TS->tm_hour,
                 TS->tm_min,TS->tm_sec);
         char *listsep = new char[GetLDV().ListSeparator.Length()+1];
         char *decsep = new char[GetLDV().DecSeparator.Length()+1];
         ConvertStr(GetLDV().ListSeparator.c_str(),listsep);
         ConvertStr(GetLDV().DecSeparator.c_str(),decsep);
         fprintf(Datei,"%s %s%s%c\n","Trennzeichen",
                 listsep,decsep,
                 GetLDV().LeadZero);
         delete[] listsep;
         delete[] decsep;
         char *fustring = new char[_tcslen(AkFeld.FuString)+1];
         ConvertStr(AkFeld.FuString,fustring);
         fprintf(Datei,"%s\n",fustring);
         delete[] fustring;
      }
      if (IngNotation)
        AusgForm = IngForm;
      else
        AusgForm = ExpForm;
      Position = 0;
      while (Position < AkFeld.GetPunkte())
      {
        if (Editor)
        {
          MGZeile.SetStr(_TEXT(""));
          Ausgabe(SWert,AkFeld.GetX(Position),AusgForm);
          UniLength(SWert,11);
          MGZeile.AddStr(SWert);
          MGZeile.AddStr(TrennZ);
          Ausgabe(SWert,real(AkFeld.GetY(Position)),AusgForm);
          UniLength(SWert,11);
          MGZeile.AddStr(SWert);
          if (imag(AkFeld.GetY(Position)) != 0)
          { // Imag nur ausgeben wenn nicht null
            MGZeile.AddStr(TrennZ);
            Ausgabe(SWert,imag(AkFeld.GetY(Position)),AusgForm);
            UniLength(SWert,11);
            MGZeile.AddStr(SWert);
          }
          MGZeile.AddStr(_TEXT("\r\n"));
          Editor->AddStr(MGZeile);
        }
        else
        {
          Ausgabe(SWert,AkFeld.GetX(Position),AusgForm);
          _TCHAR AZeile[255];
          _tcscpy(AZeile,SWert.c_str());
          _tcscat(AZeile,TrennZ.c_str());
          Ausgabe(SWert,real(AkFeld.GetY(Position)),AusgForm);
          _tcscat(AZeile,SWert.c_str());
          if (imag(AkFeld.GetY(Position)) != 0)
          { // Imag nur ausgeben wenn nicht null
            _tcscat(AZeile,TrennZ.c_str());
            Ausgabe(SWert,imag(AkFeld.GetY(Position)),AusgForm);
            _tcscat(AZeile,SWert.c_str());
          }
#ifdef UNICODE
          wcstombs(Zeile,AZeile,_tcslen(AZeile)+1);
#else
          strcpy(Zeile,AZeile);
#endif
          fputs(Zeile,Datei);
          fputs("\n",Datei);
        }
        if (Datei && (ferror(Datei) != 0))
          return false;
        Position++;
      }
      if (Datei)
        fclose(Datei);
      return true;
    }
    return false;
}

GraphDataObj *GraphManagerObj::FindGraph(const _TCHAR *Name)
{
  int AltFeld = DatFeld->AkEintNummer();
  GraphDataObj *ret = 0;
  DatFeld->Begin();
  if (DatFeld->GetMaxEintrag() >= 0)
    do
    {
      if (_tcscmp(DatFeld->GetAkEint().FuString,Name) == 0)
        ret = &DatFeld->GetAkEint();
    }
    while (DatFeld->NextEint());
  DatFeld->SetAkEint(AltFeld);
  return ret;
}

// ----------------------------------------------------------------------------
// DarstDataObj
// ----------------------------------------------------------------------------

void DarstDataObj::InitWerber()
{
  XAxis.ResetKette();
  XAxis.Begin();
  do
  {
    XAxis->Label.SetStr(_TEXT(""));
    XAxis->Min = 1;
    XAxis->Max = 10;
    XAxis->SetVisible(XAxis.AkEintNummer() == 0);
  }
  while (XAxis.NextEint());
  YAxis.ResetKette();
  YAxis.Begin();
  do
  {
    YAxis->Label.SetStr(_TEXT(""));
    YAxis->Min = 1;
    YAxis->Max = 10;
    YAxis->SetVisible(YAxis.AkEintNummer() == 0);
  }
  while (YAxis.NextEint());
}

void DarstDataObj::Init(int conversion,
                        unsigned char DN,int MD,
                        const _TCHAR *TT,int ND)
{
  Conversion = (unsigned char)conversion;
  DiagNr = DN;
  Mode = MD;
  Titel.SetStr(TT);
  NumDigits = ND;
  m_SameXAndYScaling = false;
};

bool DarstDataObj::ErmittleWerBer(int Mode,ParserObj &Psr,
                                  GraphManagerObj *YDaten)
{
  AxisDataArr HiBerX,HiBerY;
  bool ModifiedYAxis[AxisDataArr::MaxValue+1];
  ZahlFormatObj ZFO;
  bool OK = true;

  fptr oldhndl = SetFPESignal();
  try
  {
    HiBerX = XAxis;
    HiBerY = YAxis;
    for (size_t i = 0;i <= YAxis.GetMaxEintrag();i++)
      ModifiedYAxis[i] = false;
    bool ModifiedX = false;
    if (YDaten->YDatenVorhanden())
    {
      YDaten->ResetYDaten();
      if (YDaten->GetMaxPos() > 0)
        do
        {
          XAxis.SetAkEint(YDaten->AkGraph().XAxisIndex[DiagNr]);
          YAxis.SetAkEint(YDaten->AkGraph().YAxisIndex[DiagNr]);
          HiBerX.SetAkEint(YDaten->AkGraph().XAxisIndex[DiagNr]);
          HiBerY.SetAkEint(YDaten->AkGraph().YAxisIndex[DiagNr]);
          if ((XAxis->GetLog()) && (Conversion == ortskurY))
            XAxis->SetLog(false);
          if (((YDaten->AkGraph().GraphOptionen & GraphDataObj::GraphDia1) &&
               (DiagNr == 0)) ||
              ((YDaten->AkGraph().GraphOptionen & GraphDataObj::GraphDia2) &&
               (DiagNr == 1)))
          {
            YDaten->AkFeldUmrechnen(Psr);
            for (unsigned int i = 0;i < YDaten->AkGraph().GetPunkte();i++)
            {
              double HiX = YDaten->AkGraph().GetX(i);
              if ((Mode & AutoX) || (HiX >= XAxis->Min) && (HiX <= XAxis->Max) ||
                  (Conversion == ortskurY))
              {
                double AkY = YUmrechnen(YDaten->AkGraph().GetY(i),HiX);
                if (!ModifiedYAxis[YDaten->AkGraph().YAxisIndex[DiagNr]])
                {
                  HiBerY->Min = AkY;
                  HiBerY->Max = AkY;
                  ModifiedYAxis[YDaten->AkGraph().YAxisIndex[DiagNr]] = true;
                }
                else
                {
                  if (HiBerY->Min > AkY)
                    HiBerY->Min = AkY;
                  if (HiBerY->Max < AkY)
                    HiBerY->Max = AkY;
                }
                if ((Mode & AutoX) > 0)
                {
                  if (!ModifiedX)
                  {
                    HiBerX->Min = HiX;
                    HiBerX->Max = HiX;
                    ModifiedX = true;
                  }
                  else
                  {
                    if (HiBerX->Min > HiX)
                      HiBerX->Min = HiX;
                    if (HiBerX->Max < HiX)
                      HiBerX->Max = HiX;
                  }
                }
              }
            }
          }
        }
        while (YDaten->LoadYFeld());
      YAxis.Begin();
      HiBerY.Begin();
      do
      {
        bool scaled = true;
        if (ModifiedYAxis[YAxis.AkEintNummer()])
          scaled = ScaleBer(HiBerY.GetAkEint(),0.98);
        if (Mode & AutoY)
        {
          if (scaled)
          {
            YAxis->Min = HiBerY->Min;
            YAxis->Max = HiBerY->Max;
          }
          else
            OK = false;
        }
        // Wertebereich auf 6 Stellen runden
        int Fehler = ZahlFormatObj::NoError;
        unsigned int p;
        MGPStr Zeile;
        ZFO.Ausgabe(Zeile,YAxis->Min,ZahlFormatObj::IngForm);
        p = 0;
        YAxis->Min = ZFO.Zahl(Zeile.c_str(),p,Fehler);
        ZFO.Ausgabe(Zeile,YAxis->Max,ZahlFormatObj::IngForm);
        p = 0;
        YAxis->Max = ZFO.Zahl(Zeile.c_str(),p,Fehler);
      }
      while (YAxis.NextEint() && HiBerY.NextEint());
      XAxis.Begin();
      HiBerX.Begin();
      do
      {
        if (Mode & AutoX)
        {
          if (GetBerOK(HiBerX.GetAkEint()))
          {
            XAxis->Min = HiBerX->Min;
            XAxis->Max = HiBerX->Max;
          }
          else
            OK = false;
        }
        int Fehler = ZahlFormatObj::NoError;
        unsigned int p;
        MGPStr Zeile;
        ZFO.Ausgabe(Zeile,XAxis->Min,ZahlFormatObj::IngForm);
        p = 0;
        XAxis->Min = ZFO.Zahl(Zeile.c_str(),p,Fehler);
        ZFO.Ausgabe(Zeile,XAxis->Max,ZahlFormatObj::IngForm);
        p = 0;
        XAxis->Max = ZFO.Zahl(Zeile.c_str(),p,Fehler);
      }
      while (XAxis.NextEint() && HiBerX.NextEint());
    }
  }
  catch (...)
  {
    OK = false;
  }  
  ClearFPESignal(oldhndl);
  return OK;
}

bool DarstDataObj::ScaleBer(AxisData &org,double scalefactor)
{
  AxisData ad(org);
  bool ret = true;
  fptr oldhndl = SetFPESignal();
  try
  {
    if ((fabs(ad.Max) < 1e100) && (fabs(ad.Min) < 1e100))
    {
      double del = ad.Max-ad.Min;
      if (del < 1e-12)
        del = 1;
      if (!ad.GetLog())
      {
        ad.Min = ad.Min-del*(1.0-scalefactor);
        ad.Max = ad.Max+del*(1.0-scalefactor);
      }
      else
      if ((fabs(ad.Min) > 1e-100) && (fabs(ad.Max) > 1e-100))
      { // logarithmische Skalierung
        ad.Min = ad.Min*(scalefactor > 1 ? 2.0 : 0.5);
        ad.Max = ad.Max/(scalefactor > 1 ? 2.0 : 0.5);
      }
      else
        ret = false;
    }
    else
      ret = false;
  }
  catch (...)
  {
    ret = false;
  }
  if (ret && GetBerOK(ad))
    org = ad;
  else
    ret = false;
  ClearFPESignal(oldhndl);
  return ret;
}

void DarstDataObj::SortAxis(AxisData &WB)
{
  double Hi;

  if (WB.Max < WB.Min)
  {
    Hi = WB.Max;
    WB.Max = WB.Min;
    WB.Min = Hi;
  }
}

bool DarstDataObj::GetBerOK(const AxisData &WB)
{
  bool Hi;

  int divider = 1;
  for (int i = 1;i < NumDigits;i++)
    divider *= 10;
  Hi  =   (!WB.GetLog() || (WB.Min > 1e-50) && (WB.Max >= WB.Min*2)) &&
          (WB.Max > WB.Min) &&
          (fabs(WB.Max) < 1e50) &&
          ((WB.Max-WB.Min) > fabs(WB.Max+WB.Min)/divider);
  return Hi;
}

double DarstDataObj::YUmrechnen(Cpx y,double &f)
{
  double Winkel,Hi;

  double ret;
  fptr oldhndl = SetFPESignal();
  try
  {
    switch (Conversion)
    {
      case imagY   :
        ret = imag(y);
        break;
      case dbY     :
        Hi = abs(y);
        ret = log10(Hi)*20;
        break;
      case ortskurY:
        f = real(y);
        ret = imag(y);
        break;
      case logY    :
      case cabY    :
        ret = abs(y);
        break;
      case canY    :
        ret = arg(y)*180/M_PI;
        break;
      case ctiY    :
        ret = -arg(y)/f/2/M_PI;
        break;
      default :
        ret = real(y);
    }
  }
  catch (...)
  {
    if ((Conversion == dbY) || (Conversion == logY) || (YAxis->GetLog()))
      ret = 1e-30;
    else
      ret = 0;
  }
  ClearFPESignal(oldhndl);
  return ret;
}

size_t DarstDataObj::GetVisYAxisCount()
{
  size_t count = 1;
  for (size_t i = 1;i <= YAxis.GetMaxEintrag();i++)
  {
    YAxis.SetAkEint(i);
    if (YAxis->GetVisible())
      count++;
  }
  return count;
}  
