// Version 6.06  29.11.2009 Markus Greither
// Copyright (c) 1991-2009 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file efgraphc.cpp
    \brief Source code for the diagram window class
*/


#include "efprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#ifdef __DMC__
#include <typeinfo.h>
#endif
#include <wx/wx.h>
#include <wx/spinctrl.h>
#include <wx/resizec.h>

#include <wx/clipbrd.h>

#include <cfloat>
#include <cstring>
#include <set>
#include <functional>
#if defined(__MINGW32__)
#include <wx/msw/winundef.h>
#endif
#include <complex>
#include <limits>
#include <cstdio>

using namespace std;

#include "efdefs.h"
#include "mg.h"
#include "cpxpsr.h"
#include "diagramm.h"
#include "efgemein.h"
#include "ef.h"
#include "efmcxio.h"
#include "eftabcon.h"
#include "efgraphc.h"
#include "efedit.h"
#include "efresid.h"
#include "efgrdlgs.h"

#endif

#ifdef __WXGTK__
#include "xpm/hand.xbm"
#include "xpm/cross.xbm"
#endif

#include "efgrdnd.h" // local include because of problem with prec. headers
#include "efmacro.h"

// ----------------------------------------------------------------------------
// TEFGraphControlData
// ----------------------------------------------------------------------------

void TEFGraphControlData::CopyData(int UndoElems,TEFGraphControlData *GCD, 
                                   bool UpdateGCD)
{
    TEFGraphControlData Org;
    if (UndoElems & UndoPostProcFunction)
    {
      Org.PostProcFunktion = GCD->PostProcFunktion;
      if (UpdateGCD)
        GCD->PostProcFunktion = PostProcFunktion;
    }
    if (UndoElems & UndoDiagDataBase)
    {
      *(DiagDataBaseObj *)&Org.DDiagData = GCD->DDiagData;
      if (UpdateGCD)
        *(DiagDataBaseObj *)&(GCD->DDiagData) = DDiagData;
    }
    if (UndoElems & UndoDDiagData)
    {
      Org.DDiagData = GCD->DDiagData;
      if (UpdateGCD)
        GCD->DDiagData = DDiagData;
    }
    (*this) = Org;
}

// ----------------------------------------------------------------------------
// TEFGraphControl
// ----------------------------------------------------------------------------

const wxChar *TEFGraphControl::DiagClipFormat = wxT("EasyFunktion Diagramm 1.2");

const wxChar *TEFGraphControl::GraphClipFormat = wxT("EasyFunktion Graph 1.1");

IMPLEMENT_DYNAMIC_CLASS(TEFGraphControl,TEFTabControl)

void TEFGraphControl::UndoRedo(bool redo)
{
  if ((!redo && (m_UndoCount > 0)) ||
      (redo && (m_UndoCount <= m_UndoItems.GetMaxEintrag())))
  {
    if (redo)
      m_UndoCount++;
    TEFGraphControlData Org;  
    m_UndoItems.SetAkEint(m_UndoCount-1);
    m_UndoItems->Data.CopyData(m_UndoItems->UndoElems,this,true);
    if (!redo)
      m_UndoCount--;
    AkLabel = 0;
    SetAkDiag(0);
    UpdateWerBer();
  }  
}

void TEFGraphControl::AddUndo(int UndoElems)
{
  // no undos if count = 0
  if (!wxGetApp().m_UndoCount)
    return;
  // remove all further redos
  while (m_UndoItems.GetMaxEintrag()+1 > m_UndoCount)
  {
    m_UndoItems.SetAkEint(m_UndoItems.GetMaxEintrag());
    m_UndoItems.LoeschAkEint();
  }
  // remove undos that exceed max count
  while (m_UndoItems.GetMaxEintrag()+1 >= wxGetApp().m_UndoCount)
  {
    m_UndoItems.Begin();
    m_UndoItems.LoeschAkEint();
  }
  m_UndoItems.NeuAkEint();
  m_UndoItems->UndoElems = UndoElems;
  m_UndoItems->Data.CopyData(UndoElems,this,false);
  m_UndoCount = m_UndoItems.GetMaxEintrag()+1;
}

const _TCHAR *TEFGraphControl::GetAkFuString()
{
  if (DDiagData.YDaten.YDatenVorhanden())
    return DDiagData.YDaten.AkGraph().FuString;
  else
    return 0;
}

void TEFGraphControl::GraphenAusgabe(wxDC &DC)
{
  if (DDiagData.YDaten.YDatenVorhanden())
  {
    DDiagData.YDaten.ResetYDaten();
    if (DDiagData.YDaten.GetMaxPos() > 0)
      do
      {
        DDiagData.YDaten.AkFeldUmrechnen(Parser);
        DDiagData.DarstData[0].XAxis.SetAkEint(DDiagData.YDaten.AkGraph().XAxisIndex[0]);
        DDiagData.DarstData[0].YAxis.SetAkEint(DDiagData.YDaten.AkGraph().YAxisIndex[0]);
        if (DDiagData.YDaten.AkGraph().GraphOptionen & GraphDataObj::GraphDia1)
          Diag1.ZeichneGraph(DDiagData.YDaten.AkGraph().Farbe,DC);
        if (ZweiDiags())
          if (DDiagData.YDaten.AkGraph().GraphOptionen & GraphDataObj::GraphDia2)
          {
            DDiagData.DarstData[1].XAxis.SetAkEint(DDiagData.YDaten.AkGraph().XAxisIndex[1]);
            DDiagData.DarstData[1].YAxis.SetAkEint(DDiagData.YDaten.AkGraph().YAxisIndex[1]);
            Diag2.ZeichneGraph(DDiagData.YDaten.AkGraph().Farbe,DC);
          }
      }
      while ((DDiagData.YDaten.LoadYFeld()));
  }
}

void TEFGraphControl::SetActive(bool act)
{
  if (act != Aktiv)
  {
    Aktiv = act;
    wxClientDC dc(this);
    if (m_hasfocus)
      DrawSizeRect(dc);
    SetScreenParams(dc);
    DrawActiveFrame(dc,false);
    dc.SetUserScale(1.0,1.0);
    if (m_hasfocus)
      DrawSizeRect(dc);
  }
}

void TEFGraphControl::DrawActiveFrame(wxDC &DC,bool erase,bool Printing)
{
  wxBrush brs(wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW),wxSOLID);
  const wxBrush &OldBr = DC.GetBrush();
  if (erase)
    DC.SetBrush(brs);
  else
    DC.SetBrush(*wxTRANSPARENT_BRUSH);
  if (Aktiv)
    DC.SetPen(*wxRED_PEN);
  else
    DC.SetPen(*wxBLACK_PEN);
  wxSize NewSize = GetWindowDimensions(Printing);
  wxRect Abm(0,0,NewSize.x,NewSize.y);
  DC.DrawRectangle(Abm.x,Abm.y,Abm.width,Abm.height);
  DC.SetPen(wxNullPen);
  DC.SetBrush(OldBr);
}

void TEFGraphControl::DiagTitel(DarstDataObj &DarstData,wxString &DiagText)
{
  if (!(DarstData.Mode & GraphBaseObj::DiaTitel))
  {
    DarstData.XAxis.Begin();
    DarstData.YAxis.Begin();
    if (DarstData.Conversion == DarstDataObj::ortskurY)
      DiagText = _("X: Real ");
    else
    {
      if (DarstData.XAxis->GetLog())
        DiagText = _("X: Logarithmic ");
      else
        DiagText = _("X: Linear ");
    }
    switch (DarstData.Conversion)
    {
      case DarstDataObj::realY :
        DiagText += _("Y: Real");
        break;
      case DarstDataObj::imagY :
        DiagText += _("Y: Imaginary");
        break;
      case DarstDataObj::cabY :
        DiagText += _("Y: Absolute");
        break;
      case DarstDataObj::logY :
        DiagText += _("Y: Absolute");
        break;
      case DarstDataObj::canY :
        DiagText += _("Y: Phase");
        break;
      case DarstDataObj::ctiY :
        DiagText += _("Y: Delay");
        break;
      case DarstDataObj::dbY :
        DiagText += _("Y: Absolute, dB");
        break;
      case DarstDataObj::ortskurY :
        DiagText += _("Y: Imaginary");
        break;
    }
    if (DarstData.YAxis->GetLog())
      DiagText += _(", log.");
  }
  else
    DiagText = DarstData.Titel.c_str();
}

void TEFGraphControl::BildschirmAusgabe(wxDC &DC,bool Update,bool Printing,bool DefaultWindowXYRatio)
{
  int AltFeld;
  GraphBaseObj::BildType BildBer,BildBer1,BildBer2;
  wxSize Abm;
  int ReX;
  int MinY;

  fptr oldhndl = SetFPESignal();
  try
  {
    // Do window update, two rounds if size will change
    int rounds = 1;
    if (Update && !Printing && (DDiagData.DarstData[0].m_SameXAndYScaling ||
                                ZweiDiags() && DDiagData.DarstData[1].m_SameXAndYScaling))
      rounds++;

    for (int i = 0;i < rounds;i++)
    {
      if (Update)
      {
        delete LegendeFont;
        LegendeFont = Diag1.GetFont(DC,DDiagData.Legende);
      }

      DC.SetBackgroundMode(wxTRANSPARENT);
      DC.SetFont(*LegendeFont);

      Abm = GetWindowDimensions(Printing,DefaultWindowXYRatio);
      if (Update)
      {
        size_t count1 = DDiagData.DarstData[0].GetVisYAxisCount();
        size_t count2 = DDiagData.DarstData[1].GetVisYAxisCount();
        FunkMenue.Init(Abm.x-40,DDiagData.Legende.Frame,&Diag1,DC);
        AltFeld = DDiagData.YDaten.GetAkPos();
        DDiagData.YDaten.ResetYDaten();
        // Load graphs and compute legend strings
        if (DDiagData.YDaten.GetMaxPos() > 0)
          do
          {
            MGPStr tmp;
            if ((count1 > 1) || (DDiagData.YDaten.AkGraph().YAxisIndex[0] != 0) ||
                ZweiDiags() &&
                ((count2 > 1) || (DDiagData.YDaten.AkGraph().YAxisIndex[1] != 0)))
            {
              tmp += _TEXT("[");
              tmp << DDiagData.YDaten.AkGraph().YAxisIndex[0];
              if (ZweiDiags())
              {
                tmp += wxGetApp().GetLDV().ListSeparator;
                tmp << DDiagData.YDaten.AkGraph().YAxisIndex[1];
              }
              tmp += _TEXT("] ");
            }
            tmp += DDiagData.YDaten.AkGraph().FuString;
            FunkMenue.NeuEintrag(DDiagData.YDaten.GetAkPos(),
                                 tmp.c_str(),
                                 DDiagData.YDaten.AkGraph().Farbe,
                                 DDiagData.YDaten.AkGraph().GraphOptionen,
                                 DDiagData.YDaten.AkGraph().GraphSymbol,
                                 LegendeOben(),DC);
          }
          while (DDiagData.YDaten.LoadYFeld());
        DDiagData.YDaten.SetAkPos(AltFeld);
      }
      // Compute legend size and position
      if (!LegendeOben())
      {
        MinY = 0;
        ReX = FunkMenue.BildBer.XMax-FunkMenue.BildBer.XMin;
        int PosX = ReX;
        if (-DDiagData.LegX*Abm.x/10000 > ReX)
          PosX -= ReX+DDiagData.LegX*Abm.x/10000;
        PosX += int(0.02*Abm.x);
        ReX += DDiagData.LegX*Abm.x/10000;
        if (ReX <= 0)
          ReX = 0;
        else
          ReX += int(0.02*Abm.x);
        int YPos = Abm.y/2-(FunkMenue.BildBer.YMax-FunkMenue.BildBer.YMin)/2;
        YPos += DDiagData.LegY*Abm.y/(ZweiDiags() ? 14140 : 7070);
        if (Update)
          FunkMenue.SetPos(Abm.x-PosX,YPos);
      }
      else
      {
        MinY = FunkMenue.BildBer.YMax+int(0.02*Abm.y);
        ReX = int(0.02*Abm.x);
        if (Update)
          FunkMenue.SetPos(20,int(0.02*Abm.y));
      }
      BildBer.XMin = 0;
      BildBer.XMax = Abm.x-ReX;
      BildBer.YMin = MinY;
      BildBer1.XMin = BildBer.XMin;
      BildBer1.XMax = BildBer.XMax;
      BildBer1.YMin = BildBer.YMin;
      if (ZweiDiags())
        BildBer1.YMax = BildBer.YMin+(Abm.y-BildBer.YMin)/2;
      else
        BildBer1.YMax = Abm.y;
  
      if (Update)
      {
        wxString Titel;
        DiagTitel(DDiagData.DarstData[0],Titel);
        // Initialize diagram and compute text sizes
        Diag1.Init(BildBer1,&DDiagData.YDaten,&FunkMenue,
                   &(DDiagData.DarstData[0]),DDiagData.Achse,DDiagData.Titel,
                   DDiagData.AchsLabels,DDiagData.m_Colour,Parser.GetLDV(),DC,
                   Titel.c_str(),0);
      }
      else
        Diag1.Linien(DC,true);
      if (ZweiDiags())
      {
        BildBer2.XMin = BildBer.XMin;
        BildBer2.XMax = BildBer.XMax;
        BildBer2.YMin = BildBer.YMin+(Abm.y-BildBer.YMin)/2;
        BildBer2.YMax = Abm.y;
        if (Update)
        {
          wxString Titel;
          DiagTitel(DDiagData.DarstData[1],Titel);
          Diag2.Init(BildBer2,&DDiagData.YDaten,&FunkMenue,
                     &(DDiagData.DarstData[1]),DDiagData.Achse,DDiagData.Titel,
                     DDiagData.AchsLabels,DDiagData.m_Colour,Parser.GetLDV(),DC,
                     Titel.c_str(),1);
          int XMin,XMax;
          // Make same x range for both diagrams
          if (BildBer1.XMin > BildBer2.XMin)
            XMin = BildBer1.XMin;
          else
            XMin = BildBer2.XMin;
          if (BildBer1.XMax < BildBer2.XMax)
            XMax = BildBer1.XMax;
          else
            XMax = BildBer2.XMax;
          Diag1.SetXRange(XMin,XMax);
          Diag2.SetXRange(XMin,XMax);
        }
        else
          Diag2.Linien(DC,true);

      }
      if (Update && !Printing && (i == 0))
      {
        wxSize size = GetSize();
        int deltax = 0;

        if (DDiagData.DarstData[0].m_SameXAndYScaling)
          deltax += Diag1.GetXDeltaForSameXYScaling();

        if ((deltax != 0) && ((deltax > 0) || (size.x-deltax > 0)))
        {
          // Don't make window wider than the parent window
          if (size.x+deltax > GetParent()->GetSize().x-GetPosition().x)
          {
            double scaler = (GetParent()->GetSize().x-GetPosition().x)/double(size.x+deltax);
            SetSize(int(scaler*(size.x+deltax)), int(scaler*size.y));
          }
          else
            SetSize(size.x+deltax, size.y);
        }
      }
    }
    if (!Update)
    {
      GraphenAusgabe(DC);
      DC.SetFont(*LegendeFont);
      unsigned long BColor;
      if (m_movemode == Legende)
        BColor = wxLIGHT_GREY->Red() |
                 wxLIGHT_GREY->Green() << 8 |
                 wxLIGHT_GREY->Blue() << 16;
      else
        BColor = GetBackgroundColour().Red() |
                 GetBackgroundColour().Green() << 8 |
                 GetBackgroundColour().Blue() << 16;
      FunkMenue.DrawMenue(DDiagData.Legende.Color,BColor,Printing,DC);
      DrawMarkers(DC);
      DrawLabels(DC);
      DC.SetFont(wxNullFont);
    }
    else
      CalcLabelSizes(DC);
  }
  catch (...)
  {
    wxMessageBox(_("Error during diagram calculation !"),
                 wxT("Easy Funktion"),wxOK,wxGetApp().GetTopWindow());
  }
  ClearFPESignal(oldhndl);
}

void TEFGraphControl::DrawMarkers(wxDC &DC)
{
  Diag1.Mark1.Draw(DC);
  Diag1.Mark2.Draw(DC);
  if (ZweiDiags())
  {
    Diag2.Mark1.Draw(DC);
    Diag2.Mark2.Draw(DC);
  }
}

void TEFGraphControl::SetMarkersVisible(bool visible,wxDC &DC)
{
  Diag1.Mark1.SetVisible(visible,DC);
  Diag1.Mark2.SetVisible(visible,DC);
  if (ZweiDiags())
  {
    Diag2.Mark1.SetVisible(visible,DC);
    Diag2.Mark2.SetVisible(visible,DC);
  }
}

void TEFGraphControl::InitGraphs(int AkGraph,wxDC &DC)
{
  DDiagData.YDaten.ResetYDaten();
  SetMarkersVisible(false,DC);
  if (AkGraph > 0)
    SetActiveGraph(char(AkGraph),DC);
  else
    SetActiveGraph(EFDiagObj::NotMoveCursor,DC);
  SetMarkersVisible(true,DC);
}

void TEFGraphControl::FocusRect(wxDC &DC)
{
  wxPoint Top,Bottom;

  double x,y;
  DC.GetUserScale(&x,&y);
  const GraphBaseObj::BildType &AB = GetAkDiag().GetBildBer();
  Top.x = AB.XMin;
  Top.y = AB.YMin;
  Bottom.x = AB.XMax;
  Bottom.y = AB.YMax;
  wxRect rct(Top,Bottom);
  DC.SetUserScale(1.0,1.0);
  FocusRectCoord(DC,rct.x-4,rct.y-4,rct.width+7,rct.height+7);
  DC.SetUserScale(x,y);
}

TEFGraphControl::~TEFGraphControl()
{
  delete LegendeFont;
  delete DiagrammCursor;
  delete LegendeCursor;
}

TEFGraphControl::TEFGraphControl(wxWindow *AParent, int AnId,DiagDataObj *DiDa,
                                 const NumListFormat &LDV,
                                 const wxPoint &pos,const wxSize &size,
                                 long style,const wxString &name)
 : TEFTabControl(AParent,AnId,pos,size,style,name),
   Parser(LDV),
   Diag1(&Parser,wxGetApp().ZahlForm),
   Diag2(&Parser,wxGetApp().ZahlForm),
   AkLabel(0)
{
  Init(DiDa);
}

void TEFGraphControl::Create(wxWindow *AParent, int AnId,DiagDataObj *DiDa,
                                 const NumListFormat &LDV,
                                 const wxPoint &pos,const wxSize &size,
                                 long style,const wxString &name)
{
  TEFTabControl::Create(AParent,AnId,pos,size,style,name),
  Parser.SetLDV(LDV);
  Diag1.SetFormats(&Parser,wxGetApp().ZahlForm);
  Diag2.SetFormats(&Parser,wxGetApp().ZahlForm);
  Init(DiDa);
}

void TEFGraphControl::Init(DiagDataObj *DiDa)
{
  DDiagData.Copy(*DiDa,true,true);
  m_SameXAndYScaling = false;
  m_UndoCount = 0;
  wxColour clr = wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW);
  SetBackgroundColour(clr);
#if wxCHECK_VERSION(2,5,0)
  ClearBackground();
#else
  Clear();
#endif
  PostProcFunktion->Term.SetStr(DDiagData.PostProcTerm.c_str());
  Parser.InitFunktions(&PostProcFunktion);
  wxCustomDataObject *cdo = new wxCustomDataObject(wxGetApp().graphformat);
  SetDropTarget(new EFGraphDropTarget(cdo,this));
  GraphBaseObj::BildType BildBer;
  LegendeFont = 0;
  m_hasfocus = false;
  AkDiag = 0;
  AkLabel = 0;
  DDiagData.VerifyData();
  DDiagData.YDaten.Init(DDiagData.PostProc,Parser.GetLDV());
  m_capt = 0;
  Painted = false;
  MenuCursor = false;
  DelDiag = false;
  Aktiv = false;
  m_movemode = MoveWin;
#ifdef __WXMSW__
  DiagrammCursor = new wxCursor(wxT("IDC_FADENKREUZ"));
  LegendeCursor = new wxCursor(wxT("IDC_EFHAND"));
#else
  DiagrammCursor = new wxCursor(cross,32,32,16,16);
  LegendeCursor = new wxCursor(hand,32,32,5,0);
#endif
  wxClientDC DC(this);
  LegendeFont = Diag1.GetFont(DC,DDiagData.Legende);
  DC.SetFont(*LegendeFont);
  BildBer.XMin = 0;
  BildBer.YMin = 0;
  BildBer.XMax = 100;
  BildBer.YMax = 100;
  Diag1.Init(BildBer,&DDiagData.YDaten,&FunkMenue,
             &(DDiagData.DarstData[0]),DDiagData.Achse,DDiagData.Titel,
             DDiagData.AchsLabels,DDiagData.m_Colour,Parser.GetLDV(),DC,0,0);
  Diag2.Init(BildBer,&DDiagData.YDaten,&FunkMenue,
             &(DDiagData.DarstData[1]),DDiagData.Achse,DDiagData.Titel,
             DDiagData.AchsLabels,DDiagData.m_Colour,Parser.GetLDV(),DC,0,1);
  DC.SetFont(wxNullFont);
  wxClientDC cdc(this);
  SetScreenParams(cdc);
  UpdateDiag(cdc);
}


const GraphDataObj *TEFGraphControl::FuStringVorh(const _TCHAR *FuStr)
{
  return DDiagData.YDaten.FindGraph(FuStr);
}

void TEFGraphControl::DiagCurPos(wxDC &/*DC*/)
{
  if (_tcslen(GetAkDiag().GetAkPosText()) > 0)
  {
    MGPStr tmp;
    if ((GetAkDiag().GetAkDelText()[0] != 0) && GetAkDiag().Mark1.Markiert())
      tmp.SetStr(GetAkDiag().GetAkDelText());
    else
      tmp.SetStr(GetAkDiag().GetAkPosText());
    wxGetApp().sb->SetStatusText(wxString(tmp.c_str()),2);
  }
  else
    wxGetApp().sb->SetStatusText(wxT(""),2);
}

void TEFGraphControl::SetAkDiag(unsigned char newdiag,bool recordmacro)
{
  if ((AkDiag != newdiag) && recordmacro && MyMacroRecorder::Get()->IsRecording())
  {
    MyMacroRecorder::Get()->AddEvent(wxT("CMD_SELECTDIAGRAM"),GetId());
    EFXmlNode *node = MyMacroRecorder::Get()->GetAkNode();
    node->AddProperty(wxT("index"),newdiag ? wxT("1") : wxT("0"));
  }
  AkDiag = newdiag;
}

int TEFGraphControl::MarkDiag(const wxPoint &hp,wxDC &DC,
                              bool GetSelDiag,unsigned char *NewDiag)
{
  bool einzel = !ZweiDiags();
  for (int i = 0;i < (einzel ? 1 : 2);i++)
  {
    if ((!GetSelDiag && (AkDiag == i)) || (NewDiag && (*NewDiag == i)) ||
        !NewDiag && GetDiag(i)->PunktInDiagramm(hp.x,hp.y))
    { // if point in current diagram or GetSelDiag and point in diagram i
      if (GetSelDiag)
      {
        if (m_hasfocus) // eventually hide current focus rectangle
          FocusRect(DC);
        if (AkDiag != i)
          DiagCurPos(DC);
        SetAkDiag(i);
        FocusRect(DC); // draw new focus rectangle
      }
      else
      if (DDiagData.YDaten.YDatenVorhanden() && !wxGetApp().GetZoomIn())
      { // graphs available and not zoom-in mode
        GetAkDiag().Move(EFDiagObj::AkXPos,hp.x,DC);
        DiagCurPos(DC);
      }
      return i+1;
    }
  }
  return 0;
}


wxSize TEFGraphControl::GetWindowDimensions(bool Printing,bool DefaultWindowXYRatio)
{
  if (Printing)
  {
    wxSize size;
    size.x = 5000;
    if (ZweiDiags())
      size.y = 7070;
    else
      size.y = 3535;

    wxSize clsize = GetClientSize();

    if (!DefaultWindowXYRatio)
      size.x = int(float(clsize.x)*size.y/clsize.y);

    return size;
  }
  else
    return GetClientSize();
}

void TEFGraphControl::SetScreenParams(wxDC &dc,int w,int h,bool DefaultWindowXYRatio)
{
  wxSize NewSize = GetWindowDimensions(true,DefaultWindowXYRatio);
  wxSize VSize;
  if (!w || !h)
  { // screen output
    VSize = GetClientSize();
    // base scaling only on y axis, as otherwise there is a conflict with
    // SameXAndYScaling, as the font dimension cannot change when x changes 
    // Otherwise the program will crash !!
    float sizefak = float(VSize.y)/NewSize.y;
    Diag1.SetZoom(sizefak/2);
    if (ZweiDiags())
      Diag2.SetZoom(sizefak/2);
  }
  else
  { // printing
    Diag1.SetZoom(0.5);
    if (ZweiDiags())
      Diag2.SetZoom(0.5);
    VSize = wxSize(w,h);
    double x,y;
    dc.GetUserScale(&x,&y);
    dc.SetUserScale(x*double(VSize.x)/double(NewSize.x),
                    y*double(VSize.y)/double(NewSize.y));
  }
}

void TEFGraphControl::UpdateDiag(wxDC &dc,bool Printing,bool DefaultWindowXYRatio)
{
  BildschirmAusgabe(dc,true,Printing,DefaultWindowXYRatio);
  Painted = true;
}

void TEFGraphControl::PrintDiag(wxDC &dc,int width,int height)
{
  double x,y;

  dc.GetUserScale(&x,&y);
  SetScreenParams(dc,width,height,true);
  double newx,newy;
  dc.GetUserScale(&newx,&newy);
  dc.SetUserScale(0.85*newx,0.85*newy);
  UpdateDiag(dc,true,true);
  int OrgDiagMode1 = Diag1.GetDiagMode();
  int OrgDiagMode2 = Diag2.GetDiagMode();
  Diag1.SetDiagMode(OrgDiagMode1 | GraphBaseObj::DiaAlleLinienX |
                    GraphBaseObj::DiaAlleLinienY); // alle Linien ausgeben
  Diag2.SetDiagMode(OrgDiagMode2 | GraphBaseObj::DiaAlleLinienX |
                    GraphBaseObj::DiaAlleLinienY);
  dc.GetUserScale(&newx,&newy);
  dc.SetDeviceOrigin(int(x*width/10),int(x*height/10));
  BildschirmAusgabe(dc,false,true);
  wxFont font(ZweiDiags() ?
              -250 : -125,
              wxDEFAULT,wxNORMAL,wxNORMAL,false,wxT("Arial"));
  dc.SetFont(font);
  MGPStr tmp;
  wxCoord w,h;
  dc.GetTextExtent(wxString(wxT("Mg")),&w,&h);
  if (Diag1.Mark1.Markiert() && (Diag1.GetAkPosText()[0] != 0))
  {
    tmp.SetStr(wxT("Diag 1 Pos :"));
    tmp.AddStr(Diag1.GetAkPosText());
    dc.DrawText(wxString(tmp.c_str()),width/20,height-height/50);
  }
  if (Diag1.Mark2.Markiert() && (Diag1.GetAkDelText()[0] != 0))
  {
    tmp.SetStr(wxT("Diag 1 Del :"));
    tmp.AddStr(Diag1.GetAkDelText());
    dc.DrawText(wxString(tmp.c_str()),width/20,height-height/50+h);
  }
  if (Diag2.Mark1.Markiert() && (Diag2.GetAkPosText()[0] != 0))
  {
    tmp.SetStr(wxT("Diag 2 Pos :"));
    tmp.AddStr(Diag2.GetAkPosText());
    dc.DrawText(wxString(tmp.c_str()),width/2,height-height/50);
  }
  if (Diag2.Mark2.Markiert() && (Diag2.GetAkDelText()[0] != 0))
  {
    tmp.SetStr(wxT("Diag 2 Del :"));
    tmp.AddStr(Diag2.GetAkDelText());
    dc.DrawText(wxString(tmp.c_str()),width/2,height-height/50+h);
  }
  dc.SetDeviceOrigin(0,0);
  dc.SetUserScale(x,y);
  dc.SetFont(wxNullFont);
  Diag1.SetDiagMode(OrgDiagMode1);
  Diag2.SetDiagMode(OrgDiagMode2);
  wxClientDC cdc(this);
  SetScreenParams(cdc);
  UpdateDiag(cdc);
}

void TEFGraphControl::Paint(wxDC& dc, bool Printing, wxSize& size)
{
  if (!IsShown())
    return;
  double x,y;
  dc.GetUserScale(&x,&y);
  if (Printing)
    SetScreenParams(dc,size.x,size.y);
  else
    SetScreenParams(dc);
  if (!Painted)
  {
    Diag1.Mark1.UnMark(dc);
    Diag1.Mark2.UnMark(dc);
    Diag2.Mark1.UnMark(dc);
    Diag2.Mark2.UnMark(dc);
  }
  bool tmp = Aktiv;
  if (Printing)
    Aktiv = false;
  DrawActiveFrame(dc,true,Printing);
  if (Printing)
    Aktiv = tmp;
  if (!Painted || Printing)
    UpdateDiag(dc,Printing);
  BildschirmAusgabe(dc,false,Printing);
  if (m_hasfocus)
  {
    dc.SetFont(*LegendeFont);
    InitGraphs(DDiagData.AkGraph,dc);
    dc.SetFont(wxNullFont);
    FocusRect(dc);
    dc.SetUserScale(1.0,1.0);
    DrawSizeRect(dc);
  }
  if (Printing)
  {
    wxClientDC cdc(this);
    SetScreenParams(cdc);
    UpdateDiag(cdc);
  }
}

void TEFGraphControl::CopyGraph(wxDataObjectComposite &cmp)
{
  MGPStr AkStr;
  wxBeginBusyCursor();
  DDiagData.YDaten.WriteGraph(0,&AkStr,false,true,false,false);
  wxEndBusyCursor();
  cmp.Add( new wxTextDataObject(AkStr.c_str()));
  wxCustomDataObject *doo = GetGraphData();
  cmp.Add(doo);
}

wxCustomDataObject *TEFGraphControl::GetGraphData()
{
  wxCustomDataObject *doo = new wxCustomDataObject(wxGetApp().graphformat);
  GraphDataObj Ydata;
  DDiagData.YDaten.GetAkFeldData(&Ydata);
  void *tmp = Ydata.ConvertToMemBlock();
  doo->SetData(sizeof(GraphDataObj)+Ydata.GetXSize()+Ydata.GetYSize(),tmp);
  free(tmp);
  return doo;
}

void TEFGraphControl::CopyDiag(wxDataObjectComposite &cmp)
{
  wxCustomDataObject *doo = new wxCustomDataObject(wxGetApp().diagformat);
  DiagDataObj Pt(wxGetApp().GetLDV());
  Pt.Copy(DDiagData);
  wxRect rect = GetRect();
  Pt.WinXPos = rect.x;
  Pt.WinYPos = rect.y;
  doo->SetData(sizeof(DiagDataObj),&Pt);
  cmp.Add(doo);
}

int TEFGraphControl::ShowValueTable(const wxChar *label,bool Neu)
{
  int WRetStatus = wxID_CANCEL;
  if (DDiagData.YDaten.YDatenVorhanden() || Neu)
  {
    int AkGraphPos = DDiagData.YDaten.GetAkPos();
    DDiagData.YDaten.ReloadGraph();
    MGPStr AkStr;
    wxBeginBusyCursor();
    if (!Neu)
      DDiagData.YDaten.WriteGraph(0,&AkStr,false,false,false,
                                  wxGetApp().ZahlForm ==
                                  ZahlFormatObj::IngForm);
    wxEndBusyCursor();
    TEFWerteDlg *Edit = new TEFWerteDlg(AkStr);
    if (Edit)
    {
      wxString Zeile;
      wxString str(_("Modify data points"));
      Zeile = str;
      if (label[0] != 0)
      {
        Zeile += wxT(" - ");
        Zeile += label;
      }
      wxBeginBusyCursor();
      Edit->Create(wxGetApp().GetTopWindow(),dlgWerte,Zeile);
      wxEndBusyCursor();
      WRetStatus = Edit->ShowModal();
      Edit->Destroy();
      // Nach moegl. Bildschirm-Update aktuellen Graphen wieder laden
      DDiagData.YDaten.SetAkPos(AkGraphPos);
      wxBeginBusyCursor();
      bool ret = DDiagData.YDaten.LoadGraph(0,AkStr.c_str(),true,Neu,false);
      wxEndBusyCursor();
      if ((WRetStatus == wxID_OK) && !ret)
      { // Wenn OK gedrueckt, aber Daten konnten nicht geladen werden
        wxMessageBox(_("Invalid entry !"),wxT("Easy Funktion"),
                     wxOK+wxICON_EXCLAMATION,wxGetApp().GetTopWindow());
        WRetStatus = wxID_CANCEL;
      }
    }
  }
  return WRetStatus;
}

void TEFGraphControl::GraphOptionen(bool Neu, bool onlyvaluetable)
{
  GraphData GD;
  int RetStatus,WRetStatus;

  if (DDiagData.YDaten.YDatenVorhanden() || Neu)
  {
    int AkGraphPos = DDiagData.YDaten.GetAkPos();
    GD.Legende = DDiagData.Legende;
    if (Neu)
    {
      GD.Symb = wxGetApp().GraphSymb;
      GD.Opt = wxGetApp().GraphOpt |
               GraphDataObj::GraphDia1 |
               GraphDataObj::GraphDia2;
      GD.Darst = wxGetApp().GraphDarst;
      GD.Farbe = 0;
      GD.XAxisI[0] = 0;
      GD.YAxisI[0] = 0;
      GD.XAxisI[1] = 0;
      GD.YAxisI[1] = 0;
    }
    else
    {
      GD.Symb = DDiagData.YDaten.AkGraph().GraphSymbol;
      GD.Opt = DDiagData.YDaten.AkGraph().GraphOptionen;
      GD.Darst = DDiagData.YDaten.AkGraph().ZeichenArt;
      GD.Farbe = DDiagData.YDaten.AkGraph().Farbe;
      GD.Txt.SetStr(DDiagData.YDaten.AkGraph().FuString);
      GD.XAxisI[0] = DDiagData.YDaten.AkGraph().XAxisIndex[0];
      GD.YAxisI[0] = DDiagData.YDaten.AkGraph().YAxisIndex[0];
      GD.XAxisI[1] = DDiagData.YDaten.AkGraph().XAxisIndex[1];
      GD.YAxisI[1] = DDiagData.YDaten.AkGraph().YAxisIndex[1];
      if (_tcslen(DDiagData.YDaten.AkGraph().FileString) > 0)
      {
        wxString str(_("In File :"));
        GD.FileS.SetStr(str.c_str());
        GD.FileS.AddStr(DDiagData.YDaten.AkGraph().FileString);
      }
      else
      {
        wxString str(_("Number points"));
        str += wxT(":%u");
        wxString tmp;
        tmp.Printf(str,DDiagData.YDaten.AkGraph().GetPunkte());
        GD.FileS.SetStr(tmp.c_str());
      }
    }
    WRetStatus = wxID_CANCEL;
    if (onlyvaluetable)
      RetStatus = wxID_NO;
    else
    {
      TEFGraphDlg *Dlg = new TEFGraphDlg(GD);
      RetStatus = wxID_CANCEL;
      if (Dlg)
      {
        wxString Zeile;
        if (Neu)
        {
          wxString str(_("New graph from values"));
          Zeile = str;
        }
        else
        {
          wxString str(_("Graph properties"));
          Zeile = str;
        }
        Dlg->Create(wxGetApp().GetTopWindow(),dlgGraph,Zeile);
        Dlg->FindWindow(wxID_OK)->Enable(!Neu);
        Dlg->FindWindow(wxID_YES)->Enable(!Neu);
        RetStatus = Dlg->ShowModal();
        Dlg->Destroy();
        // After evtl. screen update reload current graph
        DDiagData.YDaten.SetAkPos(AkGraphPos);
      }
    }
    if ((RetStatus == wxID_NO) && ((GD.Opt & GraphDataObj::TextDatDatei) == 0))
    { // Value button pressed
      AddUndo(UndoDDiagData);
      WRetStatus = ShowValueTable(GD.Txt.c_str(),Neu);
    }
    else
    if (RetStatus == wxID_NO)
    {
      wxMessageBox(_("Data points of graph in external text file cannot be modified here !"),
                   wxT("Easy Funktion"),wxOK+wxICON_EXCLAMATION,
                   wxGetApp().GetTopWindow());
      return;
    }
    else
    if (Neu)
    {
       wxMessageBox(_("No or not enough values for graph !"),
                    wxT("Easy Funktion"),wxOK+wxICON_EXCLAMATION,
                    wxGetApp().GetTopWindow());
       return;
    }
    if ((RetStatus != wxID_CANCEL) && (!Neu || (WRetStatus == wxID_OK)))
    {
      if (RetStatus != wxID_NO)
        AddUndo(UndoDDiagData);
      if (!Neu && (WRetStatus != wxID_OK))
        DDiagData.YDaten.ReloadGraph();
      GraphDataObj *YT = new GraphDataObj;
      if (YT)
      {
        DDiagData.YDaten.GetAkEint().CopyData(YT,true);
        DiagDataBaseObj tmpbase(DDiagData);
        DDiagData.Legende = GD.Legende;
        YT->GraphSymbol = GD.Symb;
        YT->ZeichenArt = GD.Darst;
        YT->GraphOptionen = GD.Opt;
        YT->XAxisIndex[0] = GD.XAxisI[0];
        YT->YAxisIndex[0] = GD.YAxisI[0];
        YT->XAxisIndex[1] = GD.XAxisI[1];
        YT->YAxisIndex[1] = GD.YAxisI[1];
        if (GD.Farbe)
          YT->Farbe = GD.Farbe;
        if (!onlyvaluetable)
        {
          if (Neu || (_tcscmp(YT->FuString,GD.Txt.c_str()) != 0))
            if (CheckGraphName(GD.Txt) == wxID_CANCEL)
              RetStatus = wxID_CANCEL;
        }
        if ((RetStatus == wxID_OK) || (RetStatus == wxID_YES) ||
            (RetStatus == wxID_NO))
        { // OK, values or all button pressed
          if (!onlyvaluetable)
            _tcscpy(YT->FuString,GD.Txt.c_str());

          GraphDataObj *tmp = 0;
          if (MyMacroRecorder::Get()->IsRecording())
            tmp = new GraphDataObj(DDiagData.YDaten.GetAkEint());
          DDiagData.YDaten.ModifyGraph(YT,RetStatus == wxID_YES,true);
          if (MyMacroRecorder::Get()->IsRecording())
          {
            if (Neu)
              MyMacroRecorder::Get()->AddEvent(wxT("CMD_ADDGRAPH"),GetId());
            else
              MyMacroRecorder::Get()->AddEvent(wxT("CMD_MODIFYGRAPH"),GetId());
            if ((RetStatus == wxID_NO) && (WRetStatus == wxID_OK))
              tmp->SetPunkte(0); // Make sure new values from value table are saved
            EFXmlNode *node = MyMacroRecorder::Get()->GetAkNode();
            node->AddProperty(wxT("allgraphs"),RetStatus == wxID_YES ? wxT("1") : wxT("0"));
            DDiagData.YDaten.GetAkEint().XmlWriteGraph(node,tmp);
            DDiagData.XmlWriteLegend(node,&tmpbase);
            delete tmp;
          }
        }
        delete YT;
      }
    }
    if (RetStatus != wxID_CANCEL)
    {
      UpdateWerBer();
      SendModified();
    }
  }
}

int TEFGraphControl::CheckGraphName(MGPStr &Name,wxWindow *parent,bool dragndrop)
{
  int RetStatus = wxID_OK;
  while ((RetStatus == wxID_OK) && ((Name[0] == 0) ||
         (FuStringVorh(Name.c_str()))))
  {
    // For drag and drop we cannot ask for better graph name 
    // (causes endless loop as long the mouse pointer is within window), 
    // so we cancel
    if (dragndrop)
      return wxID_CANCEL;

    wxString tmp(Name.c_str());
    wxTextEntryDialog *ID;
    ID = new wxTextEntryDialog(parent ? parent : wxGetApp().GetTopWindow(),
                               _("Function name already used !"),
                               wxT("Easy Funktion"),tmp);
    RetStatus = wxID_CANCEL;
    if (ID)
    {
      RetStatus = ID->ShowModal();
      Name.SetStr(ID->GetValue().c_str());
      ID->Destroy();
    }
  }
  return (RetStatus == wxID_OK ? wxID_OK : wxID_CANCEL);
}

void TEFGraphControl::GraphenLoeschen(bool Alle)
{
  if (DDiagData.YDaten.YDatenVorhanden())
  {
    int ret = wxYES;
    if (Alle)
    {
      if ((wxGetApp().Options & MyApp::keyWarnDelete) &&
          (wxGetApp().m_UndoCount == 0))
        ret = wxMessageBox(_("Delete all graphs in diagram ? \n\nThis command cannot be undone !"),
                           wxT("Easy Funktion"),wxYES_NO+wxICON_QUESTION,
                           wxGetApp().GetTopWindow());
      if (ret == wxYES)
        DDiagData.YDaten.LoeschYDaten();
    }
    else
    {
      if ((wxGetApp().Options & MyApp::keyWarnDelete) &&
          (wxGetApp().m_UndoCount == 0))
        ret = wxMessageBox(_("Delete current graph ? \n\nThis command cannot be undone !"),
                           wxT("Easy Funktion"),wxYES_NO+wxICON_QUESTION,
                           wxGetApp().GetTopWindow());
      if (ret == wxYES)
        GetAkDiag().DelGraph();
    }
    if (DDiagData.AutoMode > 0)
      UpdateWerBer();
    else
      Invalidate();
    DDiagData.AkGraph = 0;
    SendModified();
  }
}

void TEFGraphControl::Invalidate()
{
  Painted = false;
  Refresh();
}

bool TEFGraphControl::UpdateWerBer()
{
  DarstDataObj *DD;
  bool OK = true;

  SendGetFunktions();
  DD = GetDiag(0)->GetDarstData();
  OK &= DD->ErmittleWerBer(DDiagData.AutoMode,
                          Parser,&DDiagData.YDaten);
  if (ZweiDiags())
  {
    DD = GetDiag(1)->GetDarstData();
    OK &= DD->ErmittleWerBer(DDiagData.AutoMode,
                             Parser,&DDiagData.YDaten);
  }
  if (!OK && (DDiagData.YDaten.YDatenVorhanden()))
  {
    wxMessageBox(_("Automatic adjustment of value range failed !"),
                 wxT("Easy Funktion"),
                 wxOK+wxICON_EXCLAMATION,
                 wxGetApp().GetTopWindow());
  }
  Invalidate();
  return (OK);
}

wxPoint TEFGraphControl::GetLabelPos(LabelObj &Label,bool offset)
{
  wxPoint pnt(0,0);
  fptr oldhndl = SetFPESignal();
  try
  {
    EFDiagObj *diag = GetLabelDiag(Label);
    pnt.x = diag->XPos(Label.XPos);
    pnt.y = diag->YPos(Label.YPos);
    if (offset)
    {
      int xsize = diag->BildBer.XMax-diag->BildBer.XMin;
      int ysize = diag->BildBer.YMax-diag->BildBer.YMin;
      pnt.x += int(double(Label.XOffs)*xsize/1000);
      pnt.y += int(double(Label.YOffs)*ysize/1000);
    }
  }
  catch (...)
  {
    wxMessageBox(_("Value range invalid !"),wxT("Easy Funktion"),
                 wxOK+wxICON_EXCLAMATION,wxGetApp().GetTopWindow());
  }
  ClearFPESignal(oldhndl);
  return pnt;
}

void TEFGraphControl::CalcLabelSizes(wxDC &dc)
{
  DDiagData.Labels.Begin();
  if (DDiagData.Labels.GetMaxEintrag() >= 0)
    do
    {
      LabelSizeNDraw(dc,DDiagData.Labels.GetAkEint(),0,0,false);
    }
    while (DDiagData.Labels.NextEint());
}

void TEFGraphControl::LabelSizeNDraw(wxDC &DC,LabelObj &Label,
                                     int xp,int yp,bool draw)
{
  MGPStr str;
  ZahlFormatObj ZF(wxGetApp().GetLDV());
  wxFont *font = GetAkDiag().GetFont(DC,Label.Font);
  DC.SetFont(*font);
  wxCoord x,y,ysize;
  DC.GetTextExtent(wxT("Mg"),&x,&ysize);
  wxSize size(0,0);
  if (Label.Flags & LabelObj::ShowLabel)
  {
    int line = 0,numlines = GetNumLines(Label.Label.c_str());
    do
    {
      MGPStr str;
      GetLine(Label.Label.c_str(),str,line);
      DC.GetTextExtent(str.Length() ? str.c_str() : wxT(" "),&x,&y);
      if (draw)
        DC.DrawText(str.c_str(),xp+ysize/4,yp+size.y+ysize/16);
      if (x > size.x)
        size.x = x;
      size.y += y;
      line++;
    }
    while (line < numlines);
  }
  if (Label.Flags & LabelObj::ShowX)
  {
    MGPStr Zeile;
    ZF.Ausgabe(Zeile,Label.XPos,wxGetApp().ZahlForm == ZahlFormatObj::IngForm ?
                                ZahlFormatObj::FitForm : ZahlFormatObj::ExpForm);
    str.SetStr(wxT("X = "));
    str.AddStr(Zeile);
    DC.GetTextExtent(str.c_str(),&x,&y);
    if (draw)
      DC.DrawText(str.c_str(),xp+ysize/4,yp+size.y+ysize/16);
    if (x > size.x)
      size.x = x;
    size.y += y;
  }
  if (Label.Flags & LabelObj::ShowY)
  {
    MGPStr Zeile;
    ZF.Ausgabe(Zeile,Label.YPos,wxGetApp().ZahlForm == ZahlFormatObj::IngForm ?
                                ZahlFormatObj::FitForm : ZahlFormatObj::ExpForm);
    str.SetStr(wxT("Y = "));
    str.AddStr(Zeile);
    DC.GetTextExtent(str.c_str(),&x,&y);
    if (draw)
      DC.DrawText(str.c_str(),xp+ysize/4,yp+size.y+ysize/16);
    if (x > size.x)
      size.x = x;
    size.y += y;
  }
  size.x += ysize/2;
  size.y += ysize/8;
  Label.XSize = size.x;
  Label.YSize = size.y;
  DC.SetFont(wxNullFont);
  delete font;
}

size_t TEFGraphControl::FindLabel(int x,int y)
{
  DDiagData.Labels.Begin();
  if (DDiagData.Labels.GetMaxEintrag() >= 0)
    do
    {
      LabelObj &AkLabel = DDiagData.Labels.GetAkEint();
      if (!(AkLabel.Flags & LabelObj::Diag2) || ZweiDiags())
      {
        wxPoint pnt = GetLabelPos(DDiagData.Labels.GetAkEint());
        wxSize sz = GetLabelSize(DDiagData.Labels.GetAkEint());
        if ((x >= pnt.x) && (x <= pnt.x+sz.x) &&
            (y >= pnt.y) && (y <= pnt.y+sz.y))
          return DDiagData.Labels.AkEintNummer()+1;
      }
    }
    while (DDiagData.Labels.NextEint());
  return 0;
}

void TEFGraphControl::DrawLabels(wxDC &DC)
{
  DDiagData.Labels.Begin();
  if (DDiagData.Labels.GetMaxEintrag() >= 0)
    do
    {
      LabelObj &ThisLabel = DDiagData.Labels.GetAkEint();
      if (!(ThisLabel.Flags & LabelObj::Diag2) ||
          ZweiDiags())
      {
        wxSize size = GetLabelSize(ThisLabel);
        wxPoint pos = GetLabelPos(ThisLabel);
        wxFont *font = GetAkDiag().GetFont(DC,ThisLabel.Font);
        DC.SetFont(*font);
        DC.SetTextForeground(wxColour(ThisLabel.Font.Color & 0xFF,
                                      (ThisLabel.Font.Color >> 8) & 0xFF,
                                      (ThisLabel.Font.Color >> 16) & 0xFF));
        bool border = (ThisLabel.Flags & LabelObj::Frame);
        bool fill = !(ThisLabel.Flags & LabelObj::Transparent);
        const wxBrush &OldBr = DC.GetBrush();
        if (fill || border)
        {
          wxPen pen(GetBackgroundColour(),1,wxSOLID);
          const wxBrush &OldBr = DC.GetBrush();
          wxColour col = GetBackgroundColour();
          if ((AkLabel == &ThisLabel) && (m_movemode == Label))
            col = *wxLIGHT_GREY;
          wxBrush brush(col,wxSOLID);
          if (border)
            DC.SetPen(*wxBLACK_PEN);
          else
            DC.SetPen(pen);
          if (fill)
            DC.SetBrush(brush);
          else
            DC.SetBrush(*wxTRANSPARENT_BRUSH);
          DC.DrawRectangle(pos.x,pos.y,size.x,size.y);
          DC.SetPen(wxNullPen);
          DC.SetBrush(OldBr);
        }
        DrawLabel(DC,ThisLabel,pos.x,pos.y);
        if (ThisLabel.Flags & LabelObj::Arrow)
        {
          wxPoint pnt = GetLabelPos(ThisLabel,false);
          int xsize = GetAkDiag().BildBer.XMax-GetAkDiag().BildBer.XMin;
          GraphBaseObj::BildType rect;
          rect.XMin = pos.x;
          rect.YMin = pos.y;
          rect.XMax = pos.x+size.x-1;
          rect.YMax = pos.y+size.y-1;
          pos.x += size.x/2;
          pos.y += size.y/2;
          double AnfX,AnfY;
          GraphBaseObj::ClipPoint(AnfX,AnfY,pos.x,pos.y,pnt.x,pnt.y,
                                  rect,false);
          pos.x = wxCoord(AnfX);
          pos.y = wxCoord(AnfY);
          DC.SetPen(*wxBLACK_PEN);
  
          DC.DrawLine(pnt.x,pnt.y,pos.x,pos.y);
          double delx = pos.x-pnt.x;
          double dely = pos.y-pnt.y;
          double len = sqrt(pow(delx,2)+pow(dely,2));
          if (len < 1)
            len = 1;
  
          wxPoint pts[] = {pnt,
                           wxPoint(int(pnt.x+delx/len*xsize/60-
                                       (dely/len*xsize/150)),
                                   int(pnt.y+dely/len*xsize/60+
                                       (delx/len*xsize/150))),
                           wxPoint(int(pnt.x+delx/len*xsize/60+
                                       (dely/len*xsize/150)),
                                   int(pnt.y+dely/len*xsize/60-
                                       (delx/len*xsize/150))),
                           pnt};
          DC.SetBrush(*wxBLACK_BRUSH);
          DC.DrawPolygon(4,pts);
          DC.SetPen(wxNullPen);
          DC.SetBrush(OldBr);
        }
        DC.SetFont(wxNullFont);
        delete font;
      }
    }
    while (DDiagData.Labels.NextEint());
}

