// Version 6.06  29.11.2009 Markus Greither
// Copyright (c) 1991-2009 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file efgrdlgs.cpp
    \brief Source code for dialogs related to the diagram window class
*/


#include "efprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#include <wx/xrc/xmlres.h>

#ifndef WX_PRECOMP
#ifdef __DMC__
#include <typeinfo.h>
#endif
#include <wx/wx.h>
#include <wx/resizec.h>
#include <wx/dnd.h>

#include <wx/notebook.h>
#include <wx/spinctrl.h>
#include <wx/spinctrl.h>

#include <complex>
#include <limits>
#include <cstdio>
#include <set>
#include <functional>
#if defined(__MINGW32__)
#include <wx/msw/winundef.h>
#endif

using namespace std;

#include "efdefs.h"
#include "mg.h"
#include "cpxpsr.h"
#include "diagramm.h"
#include "efgemein.h"
#include "efmcxio.h"
#include "efedit.h"
#include "ef.h"
#include "eftabcon.h"
#include "efgraphc.h"
#include "efresid.h"
#include "efgrdlgs.h"

#endif

// ----------------------------------------------------------------------------
// TEFFourierDlg
// ----------------------------------------------------------------------------

TEFFourierDlg::TEFFourierDlg(FourierData &D)
  : TEFHilfeDlg(), Data(D)
{
  FourMode = 0;
  Data.State = 2;
  ResMode = 0;
}

double TEFFourierDlg::GetAkWert(int Mode,int FourMode,double Wert)
{
  if (Mode == FourMode)
  {
    if (Wert > 1e-18)
      return Wert;
    else
      return 1;
  }
  else
  {
    if (Wert > 1e-18)
      return 0.5/Wert;
    else
      return 1;
  }
}

void TEFFourierDlg::Create(wxWindow *parent,wxWindowID Id,const wxString &title)
{
  ZahlFormatObj ZF(wxGetApp().GetLDV());
//  wxDialog::Create(parent,Id,title);
//  LoadNativeDialog(parent,Id);
  wxXmlResource::Get()->LoadDialog(this,parent,wxT("dlgFourier"));
  defbutton = GetDefaultItem();
  SetId(Id);
  SetTitle(title);
  Centre();
  wxString tmp = wxT("ID_SW1");
  tmp += wxChar(FourMode+48);
  FindRB(wxXmlResource::Get()->GetXRCID(tmp))->SetValue(true);
  tmp = wxT("ID_SW2");
  tmp += wxChar(ResMode+48);
  FindRB(wxXmlResource::Get()->GetXRCID(tmp))->SetValue(true);
  tmp = wxT("ID_SW3");
  tmp += wxChar(Data.State+48);
  FindRB(wxXmlResource::Get()->GetXRCID(tmp))->SetValue(true);
  if ((fabs(Data.Ma.Ende-Data.Ma.Anf) > (MaTyp::FourIt+1)*fabs(Data.Ma.Step)) ||
      (Data.Ma.Step == 0) || (Data.Ma.Ende <= Data.Ma.Anf))
    Obw = 1;
  else
    Obw = int(floor((Data.Ma.Ende-Data.Ma.Anf)/Data.Ma.Step));
  wxString Zeile;
  Zeile.Printf(wxT("%u"),Obw);
  FindTC(XRCID("ID_FOUOBW"))->SetValue(Zeile);
  GFreq = GetAkWert(0,0,Data.Ma.Anf);
  MGPStr MZeile;
  ZF.Ausgabe(MZeile,GFreq,wxGetApp().ZahlForm);
  FindTC(XRCID("ID_FOUTEXT"))->SetValue(MZeile.c_str());
  FindTC(XRCID("ID_FOUDESC"))->SetValue(Data.Ma.Bezeichner.c_str());
  FindTC(XRCID("ID_FOUDESC"))->SetFocus();
}

void TEFFourierDlg::OnBnClicked(wxCommandEvent &)
{
  BnClicked();
}

void TEFFourierDlg::BnClicked()
{
  int i;
  double tmp;
  int Mode = 0;
  ZahlFormatObj ZF(wxGetApp().GetLDV());

  wxString Zeile = FindTC(XRCID("ID_FOUTEXT"))->GetValue();
  tmp = ZF.ZahlWert(Zeile.c_str());
  wxString ast = wxT("ID_SW1");
  for (i = 0;i <= 1;i++)
  {
    ast += wxChar(i+48);
    if (FindRB(wxXmlResource::Get()->GetXRCID(ast))->GetValue())
      Mode = i;
    ast.RemoveLast();
  }
  if ((Mode != FourMode) && (Mode == 1))
    GFreq = tmp;
  tmp = GetAkWert(Mode,FourMode,tmp);
  if ((Mode != FourMode) && (Mode == 0))
    GFreq = tmp;
  FourMode = Mode;
  MGPStr Z;
  ZF.Ausgabe(Z,tmp,wxGetApp().ZahlForm);
  FindTC(XRCID("ID_FOUTEXT"))->SetValue(Z.c_str());
}

void TEFFourierDlg::OnBnResMode(wxCommandEvent &)
{
  BnResMode();
}

void TEFFourierDlg::BnResMode()
{
  int i;
  double tmp;
  int Mode = 0;
  ZahlFormatObj ZF(wxGetApp().GetLDV());

  wxString Zeile = FindTC(XRCID("ID_FOUOBW"))->GetValue();
  tmp = ZF.ZahlWert(Zeile.c_str());
  wxString ast = wxT("ID_SW2");
  for (i = 0;i <= 1;i++)
  {
    ast += wxChar(i+48);
    if (FindRB(wxXmlResource::Get()->GetXRCID(ast))->GetValue())
      Mode = i;
    ast.RemoveLast();
  }
  if ((Mode != ResMode) && (Mode == 0) || (Mode == ResMode) && (Mode == 1))
  {
    if ((tmp > 1e-18) && (GFreq > 1e-18))
      tmp = 1/GFreq/tmp;
    else
      tmp = 0;
  }
  if ((tmp >= MaTyp::FourIt) || (tmp <= 0))
  {
    wxMessageBox(_("Too many harmonics, set to maximum"),
                 wxT("Easy Funktion"),wxOK+wxICON_EXCLAMATION,this);
    Obw = MaTyp::FourIt-1;
  }
  else
    Obw = int(floor(tmp));
  Zeile = FindTC(XRCID("ID_FOUTEXT"))->GetValue();
  GFreq = ZF.ZahlWert(Zeile.c_str());
  if (FourMode == 1)
    GFreq = GetAkWert(0,FourMode,GFreq);
  if (Mode == 1) // Wenn tsettle
  {
    if ((Obw > 1e-18) && (GFreq > 1e-18))
      tmp = 1/GFreq/Obw;
    else
      tmp = 1;
  }
  ResMode = Mode;
  MGPStr Z;
  if (ResMode)
  {
    ZF.Ausgabe(Z,tmp,wxGetApp().ZahlForm);
    FindTC(XRCID("ID_FOUOBW"))->SetValue(Z.c_str());
  }
  else
  {
    _TCHAR Zeile[200];
    _stprintf(Zeile,wxT("%u"),Obw);
    FindTC(XRCID("ID_FOUOBW"))->SetValue(Zeile);
  }
}

void TEFFourierDlg::OnOk(wxCommandEvent &)
{
  int i;

  BnResMode();
  wxString tmp = wxT("ID_SW3");
  for (i = 0;i <= 2;i++)
  {
    tmp += wxChar(i+48);
    if (FindRB(wxXmlResource::Get()->GetXRCID(tmp))->GetValue())
      Data.State = i;
    tmp.RemoveLast();
  }
  Data.State |= FourOK;
  Data.Ma.Anf = GFreq;
  if ((Obw == 0) || (Obw >= MaTyp::FourIt))
    Data.State = 0;
  else
  {
    Data.Ma.Step = 2*Data.Ma.Anf;
    Data.Ma.Ende = Obw*Data.Ma.Step+Data.Ma.Anf;
  }
  tmp = FindTC(XRCID("ID_FOUDESC"))->GetValue();
  Data.Ma.Bezeichner.SetStr(tmp.c_str());
  EndModal(wxID_OK);
}

BEGIN_EVENT_TABLE(TEFFourierDlg,TEFHilfeDlg)
  EVT_RADIOBUTTON(XRCID("ID_SW10"),TEFFourierDlg::OnBnClicked)
  EVT_RADIOBUTTON(XRCID("ID_SW11"),TEFFourierDlg::OnBnClicked)
  EVT_RADIOBUTTON(XRCID("ID_SW20"),TEFFourierDlg::OnBnResMode)
  EVT_RADIOBUTTON(XRCID("ID_SW21"),TEFFourierDlg::OnBnResMode)
  EVT_TEXT(XRCID("ID_FOUTEXT"),TEFFourierDlg::OnBnResMode)
  EVT_BUTTON(wxID_OK, TEFFourierDlg::OnOk)
  EVT_BUTTON(wxID_CANCEL, TEFFourierDlg::OnCancel)
END_EVENT_TABLE()

// ----------------------------------------------------------------------------
// TEFLabelDlg
// ----------------------------------------------------------------------------

TEFLabelDlg::TEFLabelDlg(LabelObj &D) : TEFHilfeDlg(),Data(D)
{
}

void TEFLabelDlg::Create(wxWindow *parent,wxWindowID Id,
                            const wxString &title)
{
//  wxDialog::Create(parent,Id,title);
//  LoadNativeDialog(parent,Id);
  wxXmlResource::Get()->LoadDialog(this,parent,wxT("dlgLabel"));
  defbutton = GetDefaultItem();
  SetId(Id);
  SetTitle(title);
  Centre();
  FindCBX(XRCID("ID_SHOWLABEL"))->SetValue(Data.Flags & LabelObj::ShowLabel);
  FindCBX(XRCID("ID_SHOWX"))->SetValue(Data.Flags & LabelObj::ShowX);
  FindCBX(XRCID("ID_SHOWY"))->SetValue(Data.Flags & LabelObj::ShowY);
  FindCBX(XRCID("ID_FRAME"))->SetValue(Data.Flags & LabelObj::Frame);
  FindCBX(XRCID("ID_TRANSPARENT"))->SetValue(Data.Flags &
                                             LabelObj::Transparent);
  FindCBX(XRCID("ID_ARROW"))->SetValue(Data.Flags & LabelObj::Arrow);

  ZahlFormatObj ZF(wxGetApp().GetLDV());

  MGPStr Zeile;
  FindTC(XRCID("ID_LABEL"))->SetValue(Data.Label.c_str());
  ZF.Ausgabe(Zeile,Data.XPos,wxGetApp().ZahlForm);
  FindTC(XRCID("ID_XPOS"))->SetValue(Zeile.c_str());
  ZF.Ausgabe(Zeile,Data.YPos,wxGetApp().ZahlForm);
  FindTC(XRCID("ID_YPOS"))->SetValue(Zeile.c_str());
  BnClicked();
  FindTC(XRCID("ID_LABEL"))->SetFocus();
}

void TEFLabelDlg::OnBnClicked(wxCommandEvent &)
{
  BnClicked();
}

void TEFLabelDlg::BnClicked()
{
  FindTC(XRCID("ID_LABEL"))->Enable(FindCBX(XRCID("ID_SHOWLABEL"))->GetValue());
  FindTC(XRCID("ID_XPOS"))->Enable(FindCBX(XRCID("ID_SHOWX"))->GetValue());
  FindTC(XRCID("ID_YPOS"))->Enable(FindCBX(XRCID("ID_SHOWY"))->GetValue());
}

void TEFLabelDlg::OnFont(wxCommandEvent &)
{
  GetFont(this,Data.Font,_("Label font"));
}

void TEFLabelDlg::OnOk(wxCommandEvent &)
{
  ZahlFormatObj ZF(wxGetApp().GetLDV());

  Data.Flags = (Data.Flags & ~LabelObj::ShowLabel);
  Data.Flags |= (FindCBX(XRCID("ID_SHOWLABEL"))->GetValue() ?
                 LabelObj::ShowLabel : 0);
  Data.Flags = (Data.Flags & ~LabelObj::ShowX);
  Data.Flags |= (FindCBX(XRCID("ID_SHOWX"))->GetValue() ?
                 LabelObj::ShowX : 0);
  Data.Flags = (Data.Flags & ~LabelObj::ShowY);
  Data.Flags |= (FindCBX(XRCID("ID_SHOWY"))->GetValue() ?
                 LabelObj::ShowY : 0);
  Data.Flags = (Data.Flags & ~LabelObj::Frame);
  Data.Flags |= (FindCBX(XRCID("ID_FRAME"))->GetValue() ?
                 LabelObj::Frame : 0);
  Data.Flags = (Data.Flags & ~LabelObj::Transparent);
  Data.Flags |= (FindCBX(XRCID("ID_TRANSPARENT"))->GetValue() ?
                 LabelObj::Transparent : 0);
  Data.Flags = (Data.Flags & ~LabelObj::Arrow);
  Data.Flags |= (FindCBX(XRCID("ID_ARROW"))->GetValue() ?
                 LabelObj::Arrow : 0);
  Data.Label.SetStr(FindTC(XRCID("ID_LABEL"))->GetValue().c_str());
  wxString Zeile;
  Zeile = FindTC(XRCID("ID_XPOS"))->GetValue();
  Data.XPos = ZF.ZahlWert(Zeile.c_str());
  Zeile = FindTC(XRCID("ID_YPOS"))->GetValue();
  Data.YPos = ZF.ZahlWert(Zeile.c_str());
  EndModal(wxID_OK);
}

BEGIN_EVENT_TABLE(TEFLabelDlg,TEFHilfeDlg)
  EVT_BUTTON(wxID_OK,TEFLabelDlg::OnOk)
  EVT_BUTTON(wxID_CANCEL, TEFLabelDlg::OnCancel)
  EVT_BUTTON(XRCID("ID_FONT"), TEFLabelDlg::OnFont)
  EVT_CHECKBOX(XRCID("ID_SHOWLABEL"),TEFLabelDlg::OnBnClicked)
  EVT_CHECKBOX(XRCID("ID_SHOWX"),TEFLabelDlg::OnBnClicked)
  EVT_CHECKBOX(XRCID("ID_SHOWY"),TEFLabelDlg::OnBnClicked)
END_EVENT_TABLE()

// ----------------------------------------------------------------------------
// TEFWerteDlg
// ----------------------------------------------------------------------------

TEFWerteDlg::TEFWerteDlg(MGPStr &D)
  : TEFEditDlg(), Data(D)
{
}

void TEFWerteDlg::Create(wxWindow *parent,wxWindowID Id,const wxString &title)
{
  TEFEditDlg::Create(parent,Id,title);
  Editor->Insert(Data.c_str());
  Editor->SetSelection(0,0);
}

void TEFWerteDlg::OnOk(wxCommandEvent &)
{
  const _TCHAR *AkStr = Editor->LockBuffer();
  Data.SetStr(AkStr);
  Editor->UnlockBuffer(AkStr);
  EndModal(wxID_OK);
}

void TEFWerteDlg::OnHilfe(wxCommandEvent &)
{
  wxGetApp().HelpCtrl.DisplaySection(GetId());
}

BEGIN_EVENT_TABLE(TEFWerteDlg,TEFEditDlg)
  EVT_BUTTON( wxID_OK, TEFWerteDlg::OnOk )
  EVT_BUTTON(wxID_CANCEL, TEFWerteDlg::OnCancel)
  EVT_BUTTON(wxID_HELP,TEFWerteDlg::OnHilfe)
END_EVENT_TABLE()

// ----------------------------------------------------------------------------
// TEFDarstellungDlg
// ----------------------------------------------------------------------------

TEFDarstellungDlg::TEFDarstellungDlg(DarstellungsData &D)
  : TEFHilfeDlg(), Data(D)
{
}

void TEFDarstellungDlg::Create(wxWindow *parent,wxWindowID Id,
                               const wxString &title)
{
  MGPStr Zeile;
  ZahlFormatObj ZF(wxGetApp().GetLDV());

  wxXmlResource::Get()->LoadDialog(this,parent,wxT("dlgDarstellung"));
  defbutton = GetDefaultItem();
  SetId(Id);
  SetTitle(title);
  Centre();
  wxSpinCtrl *numdigit = (wxSpinCtrl *)FindWindow(XRCID("ID_NUMDIGITS"));
  numdigit->SetValue(Data.NumDigits);
  numdigit->SetRange(1,10);
  FindCBX(XRCID("ID_POSTPROC"))->SetValue(Data.PostProc);
  FindTC(XRCID("ID_POSTTERM"))->SetValue(Data.PostTerm.c_str());
  FindTC(XRCID("ID_DIAGTEXT"))->SetValue(Data.DiagText.c_str());
  ZF.Ausgabe(Zeile,Data.XAxis->Min,wxGetApp().ZahlForm);
  FindTC(XRCID("ID_X1"))->SetValue(Zeile.c_str());
  ZF.Ausgabe(Zeile,Data.XAxis->Max,wxGetApp().ZahlForm);
  FindTC(XRCID("ID_X2"))->SetValue(Zeile.c_str());
  FindTC(XRCID("ID_X3"))->SetValue(Data.XAxis->Label.c_str());
  FindCBX(XRCID("ID_XLOG"))->SetValue(Data.XAxis->GetLog());
  FindCBX(XRCID("ID_SAMEXANDYSCALING"))->SetValue(Data.SameXAndYScaling);
  FindCBX(XRCID("ID_DIAGTITEL"))->SetValue((Data.Mode &
                                            GraphBaseObj::DiaTitel) > 0);
  FindCBX(XRCID("ID_DIARAHMEN"))->SetValue((Data.Mode &
                                            GraphBaseObj::DiaRahmen) > 0);
  FindCBX(XRCID("ID_XALLLINES"))->SetValue((Data.Mode &
                                           GraphBaseObj::DiaAlleLinienX) > 0);
  FindCBX(XRCID("ID_YALLLINES"))->SetValue((Data.Mode &
                                           GraphBaseObj::DiaAlleLinienY) > 0);
  FindCBX(XRCID("ID_SW22"))->SetValue((Data.Mode &
                                       GraphBaseObj::DiaXVertikal) > 0);
  FindCBX(XRCID("ID_SW23"))->SetValue((Data.Mode &
                                       GraphBaseObj::DiaEndLabels) > 0);
  ZF.Ausgabe(Zeile,Data.EData.CXMin,wxGetApp().ZahlForm);
  FindTC(XRCID("ID_CALCX1"))->SetValue(Zeile.c_str());
  ZF.Ausgabe(Zeile,Data.EData.CXMax,wxGetApp().ZahlForm);
  FindTC(XRCID("ID_CALCX2"))->SetValue(Zeile.c_str());
  _TCHAR Z[200];
  _stprintf(Z,wxT("%u"),Data.EData.Punkte);
  FindTC(XRCID("ID_PUNKTE"))->SetValue(Z);
  FindTC(XRCID("ID_HIBEZEICHNER"))->SetValue(
    Data.EData.Ma.Bezeichner.c_str());
  FindTC(XRCID("ID_HIEINHEIT"))->SetValue(
    Data.EData.Ma.Einheit.c_str());
  ZF.Ausgabe(Zeile,Data.EData.Ma.Anf,wxGetApp().ZahlForm);
  FindTC(XRCID("ID_HISTARTWERT"))->SetValue(Zeile.c_str());
  ZF.Ausgabe(Zeile,Data.EData.Ma.Ende,wxGetApp().ZahlForm);
  FindTC(XRCID("ID_HIENDWERT"))->SetValue(Zeile.c_str());
  ZF.Ausgabe(Zeile,Data.EData.Ma.Step,wxGetApp().ZahlForm);
  FindTC(XRCID("ID_HISCHRITTWEITE"))->SetValue(Zeile.c_str());
  MGPStr MGZeile;
  MGZeile.SetStr(Data.EData.Ma.GetParListe());
  ZF.StdToWin(MGZeile);
  FindTC(XRCID("ID_PARAMETER"))->SetValue(wxString(MGZeile.c_str()));
  wxString tmp = wxT("ID_HIMODE2");
  tmp += wxChar(Data.EData.Ma.Mode+48);
  FindRB(wxXmlResource::Get()->GetXRCID(tmp))->SetValue(true);
  tmp = wxT("ID_VALUERANGE");
  tmp += wxChar(Data.EData.XMode+48);
  FindRB(wxXmlResource::Get()->GetXRCID(tmp))->SetValue(true);
  FindCBX(XRCID("ID_FEHLERPOS"))->SetValue(Data.EData.Debug);
  FindCBX(XRCID("ID_AUTOUPDATE"))->SetValue(Data.EData.AutoUpdate);
  FindCBX(XRCID("ID_CALCXLOG"))->SetValue(Data.EData.XLog);
  for (size_t i = 0;i < 5;i++)
  {
    wxString tmp;
    tmp << i;
    FindCH(XRCID("ID_AXISNUMBER"))->Append(tmp);
  }
  FindCH(XRCID("ID_AXISNUMBER"))->SetSelection(Data.YAxisNr);
  wxString convs[7] = {_("Real part"),
                       _("Imaginary part"),
                       _("Absolute"),
                       _("Absolute, dB"),
                       _("Phase"),
                       _("Delay"),
                       _("Ortskurve")};
  for (size_t i = 0;i < 7;i++)
    FindCH(XRCID("ID_CONVERSION"))->Append(convs[i]);
  int conv = Data.Conversion;
  if (conv >= DarstDataObj::logY)
    conv--;  
  FindCH(XRCID("ID_CONVERSION"))->SetSelection(conv);
  ((wxNotebook *)FindWindow(XRCID("ID_NOTEBOOK")))->SetSelection(Data.DiagPage);
  wxCommandEvent event;
  OnAxisNumber(event);
  BnClicked();
}

void TEFDarstellungDlg::OnAchseFont(wxCommandEvent &)
{
  GetFont(this,Data.Achse,_("Axis numbers"));
}

void TEFDarstellungDlg::OnTitelFont(wxCommandEvent &)
{
  GetFont(this,Data.Titel,_("Diagram titel"));
}

void TEFDarstellungDlg::OnAchsLabelsFont(wxCommandEvent &)
{
  GetFont(this,Data.AchsLabels,_("Axis labels"));
}

void TEFDarstellungDlg::OnColour(wxCommandEvent &)
{
  GetColour(this,Data.Colour,_("Diagram color"));
}

void TEFDarstellungDlg::OnBnClicked(wxCommandEvent &)
{
  BnClicked();
}

void TEFDarstellungDlg::BnClicked()
{
  bool enable = (FindCBX(XRCID("ID_DIAGTITEL"))->GetValue());
  FindTC(XRCID("ID_DIAGTEXT"))->Enable(enable);

  enable = (FindCBX(XRCID("ID_POSTPROC"))->GetValue());
  FindTC(XRCID("ID_POSTTERM"))->Enable(enable);

  enable = !(FindRB(XRCID("ID_HIMODE24"))->GetValue());
  bool parlist = (FindRB(XRCID("ID_HIMODE25"))->GetValue());
  FindWindow(XRCID("ID_PARAMETER"))->Enable(parlist);
  FindWindow(XRCID("ID_HISTARTWERT"))->Enable(enable && !parlist);
  FindWindow(XRCID("ID_HIENDWERT"))->Enable(enable && !parlist);
  FindWindow(XRCID("ID_HISCHRITTWEITE"))->Enable(enable && !parlist);

  enable = (FindRB(XRCID("ID_VALUERANGE1"))->GetValue());
  FindWindow(XRCID("ID_CALCX1"))->Enable(enable);
  FindWindow(XRCID("ID_CALCX2"))->Enable(enable);
  FindWindow(XRCID("ID_CALCXLOG"))->Enable(enable);
}

void TEFDarstellungDlg::GetYAxisSettings(int axisno)
{
  ZahlFormatObj ZF(wxGetApp().GetLDV());
  ParserDoubleObj Psr(wxGetApp().GetLDV());
  Data.YAxis.SetAkEint(axisno);
  wxString Zeile;
  Zeile = FindTC(XRCID("ID_Y1"))->GetValue();
  Data.YAxis->Min = Psr.Ergebnis(Zeile.c_str(),0);
  Zeile = FindTC(XRCID("ID_Y2"))->GetValue();
  Data.YAxis->Max = Psr.Ergebnis(Zeile.c_str(),0);
  Zeile = FindTC(XRCID("ID_Y3"))->GetValue();
  Data.YAxis->Label.SetStr(Zeile.c_str());
  Data.YAxis->SetLog(FindCBX(XRCID("ID_YLOG"))->GetValue());
  Data.YAxis->SetVisible(FindCBX(XRCID("ID_VISIBLEAXIS"))->GetValue());
  Data.YAxis->SetOppositeSide(FindCBX(XRCID("ID_RIGHT"))->GetValue());
}

void TEFDarstellungDlg::OnAxisNumber(wxCommandEvent &event)
{
  ZahlFormatObj ZF(wxGetApp().GetLDV());
  int nr = FindCH(XRCID("ID_AXISNUMBER"))->GetSelection();

  if (event.GetId())
    GetYAxisSettings(OldAxisNumber);
  OldAxisNumber = nr;

  Data.YAxis.SetAkEint(nr);
  MGPStr Zeile;
  ZF.Ausgabe(Zeile,Data.YAxis->Min,wxGetApp().ZahlForm);
  FindTC(XRCID("ID_Y1"))->SetValue(Zeile.c_str());
  ZF.Ausgabe(Zeile,Data.YAxis->Max,wxGetApp().ZahlForm);
  FindTC(XRCID("ID_Y2"))->SetValue(Zeile.c_str());
  FindTC(XRCID("ID_Y3"))->SetValue(Data.YAxis->Label.c_str());
  FindCBX(XRCID("ID_YLOG"))->SetValue(Data.YAxis->GetLog());
  FindCBX(XRCID("ID_RIGHT"))->SetValue(Data.YAxis->GetOppositeSide());

  if (!nr)
  {
    FindCBX(XRCID("ID_VISIBLEAXIS"))->SetValue(true);
    FindCBX(XRCID("ID_VISIBLEAXIS"))->Enable(false);
    FindCH(XRCID("ID_CONVERSION"))->Enable(true);
    FindCBX(XRCID("ID_RIGHT"))->Enable(false);
  }
  else
  {
    FindCBX(XRCID("ID_VISIBLEAXIS"))->SetValue(Data.YAxis->GetVisible());
    FindCBX(XRCID("ID_VISIBLEAXIS"))->Enable(true);
    FindCBX(XRCID("ID_RIGHT"))->Enable(true);
    FindCH(XRCID("ID_CONVERSION"))->Enable(false);
  }
}

void TEFDarstellungDlg::OnOk(wxCommandEvent &)
{
  wxString Zeile;
  ZahlFormatObj ZF(wxGetApp().GetLDV());
  ParserDoubleObj Psr(wxGetApp().GetLDV());
  int i;

  Zeile = FindTC(XRCID("ID_DIAGTEXT"))->GetValue();
  Data.DiagText.SetStr(Zeile.c_str());
  Data.PostProc = FindCBX(XRCID("ID_POSTPROC"))->GetValue();
  Data.PostTerm.SetStr(FindTC(XRCID("ID_POSTTERM"))->GetValue().c_str());
  Data.NumDigits =
    ((wxSpinCtrl *)FindWindow(XRCID("ID_NUMDIGITS")))->GetValue();
  Zeile = FindTC(XRCID("ID_X1"))->GetValue();
  Data.XAxis->Min = Psr.Ergebnis(Zeile.c_str(),0);
  Zeile = FindTC(XRCID("ID_X2"))->GetValue();
  Data.XAxis->Max = Psr.Ergebnis(Zeile.c_str(),0);
  Zeile = FindTC(XRCID("ID_X3"))->GetValue();
  Data.XAxis->Label.SetStr(Zeile.c_str());
  wxString tmp = wxT("ID_SW1");
  Data.Conversion = FindCH(XRCID("ID_CONVERSION"))->GetSelection();
  if (Data.Conversion >= DarstDataObj::logY)
    Data.Conversion++;
  Data.XAxis->SetLog(FindCBX(XRCID("ID_XLOG"))->GetValue());
  Data.SameXAndYScaling = FindCBX(XRCID("ID_SAMEXANDYSCALING"))->GetValue();
  Data.Mode = (int)(GraphBaseObj::DiaTitel*
                    FindCBX(XRCID("ID_DIAGTITEL"))->GetValue() +
                    GraphBaseObj::DiaRahmen*
                    FindCBX(XRCID("ID_DIARAHMEN"))->GetValue() +
                    GraphBaseObj::DiaAlleLinienX*
                    FindCBX(XRCID("ID_XALLLINES"))->GetValue() +
                    GraphBaseObj::DiaAlleLinienY*
                    FindCBX(XRCID("ID_YALLLINES"))->GetValue() +
                    GraphBaseObj::DiaXVertikal*
                    FindCBX(XRCID("ID_SW22"))->GetValue() +
                    GraphBaseObj::DiaEndLabels*
                    FindCBX(XRCID("ID_SW23"))->GetValue());
  Zeile = FindTC(XRCID("ID_CALCX1"))->GetValue();
  Data.EData.CXMin = Psr.Ergebnis(Zeile.c_str(),0);
  Zeile = FindTC(XRCID("ID_CALCX2"))->GetValue();
  Data.EData.CXMax = Psr.Ergebnis(Zeile.c_str(),0);
  Zeile = FindTC(XRCID("ID_PUNKTE"))->GetValue();
  double Pkts = ZF.ZahlWert(Zeile.c_str());
  if (Pkts > GraphDataObj::MaxPunkte)
    Data.EData.Punkte = GraphDataObj::MaxPunkte+1;
  else  
  if (Pkts < 2)
    Data.EData.Punkte = 1;
  else  
    Data.EData.Punkte = int(Pkts);
  Zeile = FindTC(XRCID("ID_HISTARTWERT"))->GetValue();
  Data.EData.Ma.Anf = Psr.Ergebnis(Zeile.c_str(),0);
  Zeile = FindTC(XRCID("ID_HIENDWERT"))->GetValue();
  Data.EData.Ma.Ende = Psr.Ergebnis(Zeile.c_str(),0);
  Zeile = FindTC(XRCID("ID_HISCHRITTWEITE"))->GetValue();
  Data.EData.Ma.Step = Psr.Ergebnis(Zeile.c_str(),0);
  Zeile = FindTC(XRCID("ID_HIBEZEICHNER"))->GetValue();
  Data.EData.Ma.Bezeichner.SetStr(Zeile.c_str());
  Zeile = FindTC(XRCID("ID_HIEINHEIT"))->GetValue();
  Data.EData.Ma.Einheit.SetStr(Zeile.c_str());
  Zeile = FindTC(XRCID("ID_PARAMETER"))->GetValue();
  MGPStr MGZeile;
  MGZeile.SetStr(Zeile.c_str());
  ZF.WinToStd(MGZeile);
  Data.EData.Ma.SetParListe(MGZeile.c_str());
  tmp = wxT("ID_HIMODE2");
  for (i = 0;i <= 5;i++)
  {
    tmp += wxChar(i+48);
    if (FindRB(wxXmlResource::Get()->GetXRCID(tmp))->GetValue())
      Data.EData.Ma.Mode = i;
    tmp.RemoveLast();
  }
  tmp = wxT("ID_VALUERANGE");
  for (i = 0;i <= 1;i++)
  {
    tmp += wxChar(i+48);
    if (FindRB(wxXmlResource::Get()->GetXRCID(tmp))->GetValue())
      Data.EData.XMode = i;
    tmp.RemoveLast();
  }
  Data.EData.Debug = FindCBX(XRCID("ID_FEHLERPOS"))->GetValue();
  Data.EData.AutoUpdate = FindCBX(XRCID("ID_AUTOUPDATE"))->GetValue();
  Data.EData.XLog = FindCBX(XRCID("ID_CALCXLOG"))->GetValue();

  int nr = FindCH(XRCID("ID_AXISNUMBER"))->GetSelection();
  GetYAxisSettings(nr);

  EndModal(wxID_OK);
}

BEGIN_EVENT_TABLE( TEFDarstellungDlg, TEFHilfeDlg )
  EVT_CHOICE(XRCID("ID_AXISNUMBER"),TEFDarstellungDlg::OnAxisNumber)
  EVT_CHECKBOX(XRCID("ID_DIAGTITEL"),TEFDarstellungDlg::OnBnClicked)
  EVT_CHECKBOX(XRCID("ID_POSTPROC"),TEFDarstellungDlg::OnBnClicked)
  EVT_BUTTON(wxID_CANCEL, TEFDarstellungDlg::OnCancel)
  EVT_BUTTON(wxID_OK, TEFDarstellungDlg::OnOk)
  EVT_BUTTON(wxID_YES, TEFDarstellungDlg::OnAchseFont)
  EVT_BUTTON(wxID_NO, TEFDarstellungDlg::OnTitelFont)
  EVT_BUTTON(XRCID("IDRETRY"), TEFDarstellungDlg::OnAchsLabelsFont)
  EVT_BUTTON(XRCID("ID_COLOUR"), TEFDarstellungDlg::OnColour)
  EVT_RADIOBUTTON(XRCID("ID_HIMODE20"),TEFDarstellungDlg::OnBnClicked)
  EVT_RADIOBUTTON(XRCID("ID_HIMODE21"),TEFDarstellungDlg::OnBnClicked)
  EVT_RADIOBUTTON(XRCID("ID_HIMODE22"),TEFDarstellungDlg::OnBnClicked)
  EVT_RADIOBUTTON(XRCID("ID_HIMODE23"),TEFDarstellungDlg::OnBnClicked)
  EVT_RADIOBUTTON(XRCID("ID_HIMODE24"),TEFDarstellungDlg::OnBnClicked)
  EVT_RADIOBUTTON(XRCID("ID_HIMODE25"),TEFDarstellungDlg::OnBnClicked)
  EVT_RADIOBUTTON(XRCID("ID_VALUERANGE0"),TEFDarstellungDlg::OnBnClicked)
  EVT_RADIOBUTTON(XRCID("ID_VALUERANGE1"),TEFDarstellungDlg::OnBnClicked)
END_EVENT_TABLE()

// ----------------------------------------------------------------------------
// TEFGraphDlg
// ----------------------------------------------------------------------------

TEFGraphDlg::TEFGraphDlg(GraphData &D)
  : TEFHilfeDlg(), Data(D)
{
}

void TEFGraphDlg::Create(wxWindow *parent,wxWindowID Id,const wxString &title)
{
//  wxDialog::Create(parent,Id,title);
//  LoadNativeDialog(parent,Id);
  wxXmlResource::Get()->LoadDialog(this,parent,wxT("dlgGraph"));
  defbutton = GetDefaultItem();
  SetId(Id);
  SetTitle(title);
  Centre();
  FindTC(XRCID("ID_GRAPHTEXT"))->SetValue(Data.Txt.c_str());
  // Get maximum field width for ID_GRAPHFILE by combining GRAPHTEXT and LABEL
  int fieldsize = FindTC(XRCID("ID_GRAPHTEXT"))->GetClientSize().x;
  fieldsize += FindST(XRCID("ID_LABEL"))->GetClientSize().x;
  // Resize Data.FileS so that it fits the field length
  wxFont font = FindST(XRCID("ID_GRAPHFILE"))->GetFont();
  wxClientDC dc(this);
  dc.SetFont(font);
  int x;
  wxString tmp(Data.FileS.c_str());
  dc.GetTextExtent(tmp,&x,0);
  if (x > fieldsize)
  {
    tmp.Prepend(wxT("..."));
    while ((x > fieldsize) && (tmp.Length() > 3))
    {
      tmp.Remove(3,1);
      dc.GetTextExtent(tmp,&x,0);
    }
    FindST(XRCID("ID_GRAPHFILE"))->SetToolTip(Data.Txt.c_str());
  }
  FindST(XRCID("ID_GRAPHFILE"))->SetLabel(tmp);
  FindCBX(XRCID("ID_MARKPOINTS"))->SetValue(
    ((Data.Darst & ~GraphDataObj::gPunkteSplines) == GraphDataObj::gPunkteVerbMark) ||
    ((Data.Darst & ~GraphDataObj::gPunkteSplines) == GraphDataObj::gPunkteMarkieren));
  FindCBX(XRCID("ID_GRAPHINTERPOLATE"))->SetValue(
    ((Data.Darst & ~GraphDataObj::gPunkteSplines) == GraphDataObj::gPunkteVerbMark) ||
    ((Data.Darst & ~GraphDataObj::gPunkteSplines) == GraphDataObj::gPunkteVerbinden));
  FindCBX(XRCID("ID_GRAPHSPLINES"))->SetValue(
    Data.Darst & GraphDataObj::gPunkteSplines);
  FindCBX(XRCID("ID_GRAPHSYMBOL"))->SetValue(~Data.Symb &
                                             GraphDataObj::GraphOhneSymbol);
  FindCBX(XRCID("ID_GRAPHFETT"))->SetValue(Data.Opt & GraphDataObj::GraphFett);
  FindCBX(XRCID("ID_GRAPHDIAG1"))->SetValue(Data.Opt & GraphDataObj::GraphDia1);
  FindCBX(XRCID("ID_GRAPHDIAG2"))->SetValue(Data.Opt & GraphDataObj::GraphDia2);
  FindCBX(XRCID("ID_GRAPHPOSTPROC"))->SetValue(Data.Opt & GraphDataObj::PstPrcOff);
  FindCBX(XRCID("ID_RAHMEN"))->SetValue(Data.Legende.Frame);
  for (size_t i = 0;i < 5;i++)
  {
    wxString tmp;
    tmp << i;
    FindCH(XRCID("ID_YAXIS1"))->Append(tmp);
    FindCH(XRCID("ID_YAXIS2"))->Append(tmp);
  }
  FindCH(XRCID("ID_YAXIS1"))->SetSelection(Data.YAxisI[0]);
  FindCH(XRCID("ID_YAXIS2"))->SetSelection(Data.YAxisI[1]);
  FindTC(XRCID("ID_GRAPHTEXT"))->SetFocus();
  BnClicked();
}

void TEFGraphDlg::OnLegendeFont(wxCommandEvent &)
{
  GetFont(this,Data.Legende,_("Legend"));
}

void TEFGraphDlg::OnFarbe(wxCommandEvent &)
{
  GetColour(this,Data.Farbe,_("Graph color"));
}

void TEFGraphDlg::OnYes(wxCommandEvent &)
{
  EndModal(wxID_YES);
}

void TEFGraphDlg::OnNo(wxCommandEvent &)
{
  EndModal(wxID_NO);
}

void TEFGraphDlg::EndModal(int Id)
{
  wxString Zeile;

  Zeile = FindTC(XRCID("ID_GRAPHTEXT"))->GetValue();
  Zeile.Truncate(255);
  Data.Txt.SetStr(Zeile.c_str());
  Data.Darst = 0;
  if (FindCBX(XRCID("ID_MARKPOINTS"))->GetValue() &&
      FindCBX(XRCID("ID_GRAPHINTERPOLATE"))->GetValue())
    Data.Darst = GraphDataObj::gPunkteVerbMark;
  else
  if (FindCBX(XRCID("ID_GRAPHINTERPOLATE"))->GetValue())
    Data.Darst = GraphDataObj::gPunkteVerbinden;
  if (FindCBX(XRCID("ID_GRAPHSPLINES"))->GetValue())
    Data.Darst += GraphDataObj::gPunkteSplines;
  Data.Symb = (unsigned short)((Data.Symb & ~GraphDataObj::GraphOhneSymbol) |
              (GraphDataObj::GraphOhneSymbol &
               ~(unsigned short)FindCBX(XRCID("ID_GRAPHSYMBOL"))->GetValue()));
  Data.Opt = (unsigned short)((Data.Opt & ~GraphDataObj::GraphFett) |
             (unsigned short)FindCBX(XRCID("ID_GRAPHFETT"))->GetValue());
  Data.Opt = (unsigned short)((Data.Opt & ~GraphDataObj::PstPrcOff) |
             (FindCBX(XRCID("ID_GRAPHPOSTPROC"))->GetValue()*GraphDataObj::PstPrcOff));
  Data.Opt = (unsigned short)((Data.Opt & ~GraphDataObj::GraphDia1) |
             (FindCBX(XRCID("ID_GRAPHDIAG1"))->GetValue()*GraphDataObj::GraphDia1));
  Data.Opt = (unsigned short)((Data.Opt & ~GraphDataObj::GraphDia2) |
             (FindCBX(XRCID("ID_GRAPHDIAG2"))->GetValue()*GraphDataObj::GraphDia2));
  Data.Legende.Frame = (int)FindCBX(XRCID("ID_RAHMEN"))->GetValue();
  Data.YAxisI[0] = FindCH(XRCID("ID_YAXIS1"))->GetSelection();
  Data.YAxisI[1] = FindCH(XRCID("ID_YAXIS2"))->GetSelection();
  wxDialog::EndModal(Id);
}

void TEFGraphDlg::OnBnClicked(wxCommandEvent &)
{
  BnClicked();
}

void TEFGraphDlg::BnClicked()
{
  FindCBX(XRCID("ID_GRAPHSPLINES"))->Enable(
    FindCBX(XRCID("ID_GRAPHINTERPOLATE"))->GetValue());
}

BEGIN_EVENT_TABLE(TEFGraphDlg,TEFHilfeDlg)
  EVT_CHECKBOX(XRCID("ID_GRAPHINTERPOLATE"),TEFGraphDlg::OnBnClicked)
  EVT_BUTTON(wxID_YES,TEFGraphDlg::OnYes)
  EVT_BUTTON(wxID_NO,TEFGraphDlg::OnNo)
  EVT_BUTTON(XRCID("IDRETRY"), TEFGraphDlg::OnFarbe)
  EVT_BUTTON(XRCID("IDIGNORE"), TEFGraphDlg::OnLegendeFont)
  EVT_BUTTON(wxID_CANCEL, TEFGraphDlg::OnCancel)
  EVT_BUTTON(wxID_OK, TEFGraphDlg::OnOK)
END_EVENT_TABLE()

// ----------------------------------------------------------------------------
// TEFGraphLadenDlg
// ----------------------------------------------------------------------------

TEFGraphladenDlg::TEFGraphladenDlg(GraphladenData &D)
  : TEFHilfeDlg(), Data(D)
{
}

void TEFGraphladenDlg::Create(wxWindow *parent,wxWindowID Id,
                              const wxString &title)
{
//  wxDialog::Create(parent,Id,title);
//  LoadNativeDialog(parent,Id);
  wxXmlResource::Get()->LoadDialog(this,parent,wxT("dlgGraphladen"));
  defbutton = GetDefaultItem();
  SetId(Id);
  SetTitle(title);
  Centre();
  FindTC(XRCID("ID_GRAPHFILE"))->SetValue(Data.FileName.c_str());
  FindCBX(XRCID("ID_GRAPHSORT"))->SetValue(Data.Mode &
                                           GraphladenData::GraphSortieren);
  FindCBX(XRCID("ID_GRAPHTEXT"))->SetValue(Data.Mode &
                                           GraphladenData::GraphInDatei);

  FindCBX(XRCID("ID_TAB"))->SetValue(Data.TabSeparator);
  FindCBX(XRCID("ID_SPACE"))->SetValue(Data.SpaceSeparator);
  FindCBX(XRCID("ID_IMG_DATA"))->SetValue(Data.ImaginaryDataIncluded);
  FindCBX(XRCID("ID_HEADER"))->SetValue(Data.HeaderIncluded);
  FindCBX(XRCID("ID_XVALUES"))->SetValue(Data.XValuesIncluded);
  FindTC(XRCID("ID_DEC_SEPARATOR"))->SetValue(Data.DecSeparator.c_str());
  FindTC(XRCID("ID_LIST_SEPARATOR"))->SetValue(Data.ListSeparator.c_str());
  FindSC(XRCID("ID_XCOLUMN"))->SetValue(Data.XValuesColumn);

  FindTC(XRCID("ID_GRAPHFILE"))->SetFocus();
  
  wxCommandEvent evt;
  OnBnClicked(evt);
}

void TEFGraphladenDlg::OnYes(wxCommandEvent &)
{
  int RetStatus;
  wxFileDialog *TFD;
  long Flags = wxFD_FILE_MUST_EXIST | wxFD_OPEN;

  TFD = new wxFileDialog(this,_("Load graph"),wxT(""),wxT(""),wxT("*.grd"),Flags);
  if (TFD)
  {
     TFD->SetWildcard(_("GRD Files (*.grd)|*.grd|All Files (*.*)|*.*"));
     RetStatus = TFD->ShowModal();
     if (RetStatus == wxID_OK)
     {
       Data.FileName.SetStr(TFD->GetPath().c_str());
       FindTC(XRCID("ID_GRAPHFILE"))->SetValue(Data.FileName.c_str());
     }
     TFD->Destroy();
  }
}

void TEFGraphladenDlg::OnBnClicked(wxCommandEvent &)
{
  FindSC(XRCID("ID_XCOLUMN"))->Enable(FindCBX(XRCID("ID_XVALUES"))->GetValue());
}

void TEFGraphladenDlg::OnOk(wxCommandEvent &)
{
  wxString tmp = FindTC(XRCID("ID_GRAPHFILE"))->GetValue();
  Data.FileName.SetStr(tmp.c_str());
  Data.Mode = 0;
  Data.Mode = (FindCBX(XRCID("ID_GRAPHSORT"))->GetValue() ?
               GraphladenData::GraphSortieren : 0) |
              (FindCBX(XRCID("ID_GRAPHTEXT"))->GetValue() ?
               GraphladenData::GraphInDatei : 0);

  Data.TabSeparator = FindCBX(XRCID("ID_TAB"))->GetValue();
  Data.SpaceSeparator = FindCBX(XRCID("ID_SPACE"))->GetValue();
  Data.ImaginaryDataIncluded = FindCBX(XRCID("ID_IMG_DATA"))->GetValue();
  Data.HeaderIncluded = FindCBX(XRCID("ID_HEADER"))->GetValue();
  Data.XValuesIncluded = FindCBX(XRCID("ID_XVALUES"))->GetValue();
  Data.DecSeparator.SetStr(FindTC(XRCID("ID_DEC_SEPARATOR"))->GetValue().c_str());
  Data.ListSeparator.SetStr(FindTC(XRCID("ID_LIST_SEPARATOR"))->GetValue().c_str());
  Data.XValuesColumn = FindSC(XRCID("ID_XCOLUMN"))->GetValue();

  EndModal(wxID_OK);
}

BEGIN_EVENT_TABLE(TEFGraphladenDlg,TEFHilfeDlg)
  EVT_CHECKBOX(XRCID("ID_XVALUES"),TEFGraphladenDlg::OnBnClicked)
  EVT_BUTTON(wxID_YES,TEFGraphladenDlg::OnYes)
  EVT_BUTTON(wxID_OK,TEFGraphladenDlg::OnOk)
  EVT_BUTTON(wxID_CANCEL, TEFGraphladenDlg::OnCancel)
END_EVENT_TABLE()

// ----------------------------------------------------------------------------
// TEFGraphspeichernDlg
// ----------------------------------------------------------------------------

TEFGraphspeichernDlg::TEFGraphspeichernDlg(GraphspeichernData &D)
  : TEFHilfeDlg(), Data(D)
{
}

void TEFGraphspeichernDlg::Create(wxWindow *parent,wxWindowID Id,
                                  const wxString &title)
{
//  wxDialog::Create(parent,Id,title);
//  LoadNativeDialog(parent,Id);
  wxXmlResource::Get()->LoadDialog(this,parent,wxT("dlgGraphspeichern"));
  defbutton = GetDefaultItem();
  SetId(Id);
  SetTitle(title);
  Centre();
  FindTC(XRCID("ID_GRAPHFILE"))->SetValue(Data.FileName.c_str());
  FindCBX(XRCID("ID_GRAPHING"))->SetValue(Data.Mode &
                                          GraphspeichernData::GraphIngForm);
  FindCBX(XRCID("ID_GRAPHGRD"))->SetValue(Data.Mode &
                                          GraphspeichernData::GraphGRD);
  FindCBX(XRCID("ID_GRAPHTAB"))->SetValue(Data.Mode &
                                          GraphspeichernData::GraphTab);
  FindCBX(XRCID("ID_GRAPHKOMMA"))->SetValue(Data.Mode &
                                            GraphspeichernData::GraphKomma);
  FindTC(XRCID("ID_GRAPHFILE"))->SetFocus();
}

void TEFGraphspeichernDlg::OnYes(wxCommandEvent &)
{
  int RetStatus;
  wxFileDialog *TFD;
  long Flags = wxFD_OVERWRITE_PROMPT;

  TFD = new wxFileDialog(this,_("Save graph"),wxT(""),wxT(""),wxT("*.grd"),Flags);
  if (TFD)
  {
     TFD->SetWildcard(_("GRD Files (*.grd)|*.grd|All Files (*.*)|*.*"));
     RetStatus = TFD->ShowModal();
     if (RetStatus == wxID_OK)
     {
       Data.FileName.SetStr(TFD->GetPath().c_str());
       FindTC(XRCID("ID_GRAPHFILE"))->SetValue(Data.FileName.c_str());
     }
     TFD->Destroy();
  }
}

void TEFGraphspeichernDlg::OnOk(wxCommandEvent &)
{
  wxString tmp = FindTC(XRCID("ID_GRAPHFILE"))->GetValue();
  Data.FileName.SetStr(tmp.c_str());
  Data.Mode = 0;
  Data.Mode |= (FindCBX(XRCID("ID_GRAPHING"))->GetValue() ?
                GraphspeichernData::GraphIngForm : 0);
  Data.Mode |= (FindCBX(XRCID("ID_GRAPHGRD"))->GetValue() ?
                GraphspeichernData::GraphGRD : 0);
  Data.Mode |= (FindCBX(XRCID("ID_GRAPHTAB"))->GetValue() ?
                GraphspeichernData::GraphTab : 0);
  Data.Mode |= (FindCBX(XRCID("ID_GRAPHKOMMA"))->GetValue() ?
                GraphspeichernData::GraphKomma : 0);
  EndModal(wxID_OK);
}

BEGIN_EVENT_TABLE(TEFGraphspeichernDlg,TEFHilfeDlg)
  EVT_BUTTON(wxID_YES,TEFGraphspeichernDlg::OnYes)
  EVT_BUTTON(wxID_OK,TEFGraphspeichernDlg::OnOk)
  EVT_BUTTON(wxID_CANCEL, TEFGraphspeichernDlg::OnCancel)
END_EVENT_TABLE()

