// Version 6.05 20.07.2008 Markus Greither
// Copyright (c) 1991-2008 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file efgrelem.cpp
    \brief Source code for support classes for the diagram window class
*/


#include "efprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#ifdef __DMC__
#include <typeinfo.h>
#endif
#include <wx/wx.h>

#include <cfloat>
#include <cstring>
#include <set>
#include <functional>
#if defined(__MINGW32__)
#include <wx/msw/winundef.h>
#endif
#include <wx/dnd.h>
#include <complex>
#include <limits>
#include <cstdio>

using namespace std;

#include "efdefs.h"
#include "mg.h"
#include "cpxpsr.h"
#include "diagramm.h"
#include "efgemein.h"
#include "ef.h"
#include "efmcxio.h"
#include "efgrelem.h"
#include "efresid.h"

#endif

// ----------------------------------------------------------------------------
// GraphSymbolObj
// ----------------------------------------------------------------------------

//! Class for drawing symbols on graphs
class GraphSymbolObj
{
  //! Constants for number of defined symbols and array size
  enum SymbolConsts
  {
    AnzMark = 11,
    AnzMarkWerte = 12
  };
  //! Array of vertices that define the symbols
  static short int MarkArr[AnzMark][AnzMarkWerte];

 public:
  //! Draws the symbol Typ around coordinate X and Y
  static void ZeichneSymbol(int X,int Y,int Typ,wxDC &DC,int SymbolRad);
};

void GraphSymbolObj::ZeichneSymbol(int X,int Y,int Typ,wxDC &DC,int SymbolRad)
{
  while (Typ >= AnzMark)
    Typ -= AnzMark;
  wxPen newpen(wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW),
               SymbolRad,wxSOLID);
  for (int f = 0;f <= 2;f++)
  {
    wxPen oldpen;
    if (f < 2)
    { // Hintergrund loeschen mit dicker weisser Linie
      oldpen = DC.GetPen();
      DC.SetPen(newpen);
    }
    int x = X+MarkArr[Typ][0]*(f+1)*SymbolRad/3;
    int y = Y+MarkArr[Typ][1]*(f+1)*SymbolRad/3;
    for (int i = 0;i < AnzMarkWerte/2;i++)
    {
      int x1 = X+MarkArr[Typ][i*2]*(f+1)*SymbolRad/3;
      int y1 = Y+MarkArr[Typ][i*2+1]*(f+1)*SymbolRad/3;
      DC.DrawLine(x,y,x1,y1);
      x = x1;
      y = y1;
    }
    if (f < 2)
      DC.SetPen(oldpen);
  }
}

short int GraphSymbolObj::MarkArr[GraphSymbolObj::AnzMark]
                                 [GraphSymbolObj::AnzMarkWerte] =
{
  { 1, 1,-1,-1, 0, 0,-1, 1, 1,-1, 0, 0}, // Kreuz
  {-1, 1, 1, 1, 1,-1,-1,-1,-1, 1, 1, 1}, // Rechteck
  {-1,-1, 1,-1, 0, 1,-1,-1,-1,-1,-1,-1}, // Dreieck, Spitze unten
  { 0,-1, 1, 1,-1, 1, 0,-1, 0,-1, 0,-1}, // Dreieck, Spitze oben
  { 0,-1, 1, 0, 0, 1,-1, 0, 0,-1, 0,-1}, // Trapez
  { 0,-1, 0, 0, 1, 1, 0, 0,-1, 1, 0, 0}, // Dreizack Spitze oben
  {-1,-1, 0, 0, 1,-1, 0, 0, 0, 1, 0, 0}, // Dreizack Spitze unten
  {-1,-1, 1, 0,-1, 1,-1,-1,-1,-1,-1,-1}, // Dreieck, Spitze rechts
  {-1,-1, 0, 0,-1, 1, 0, 0, 1, 0, 0, 0}, // Dreizack Spitze rechts
  {-1, 0, 0, 0, 1, 1, 0, 0, 1,-1, 0, 0}, // Dreizack Spitze links
  {-1, 0, 1,-1, 1, 1,-1, 0,-1, 0,-1, 0}  // Dreieck, Spitze links
};

// ----------------------------------------------------------------------------
// MenueItemObj
// ----------------------------------------------------------------------------

void MenueItemObj::Draw(int Mode,int x,int y,unsigned long Colour,wxDC &DC)
{
  wxColour OldColor = DC.GetTextForeground();
  if ((Mode & ItemEnabled) == 0)
    DC.SetTextForeground(wxColour(128,128,128));
  else
    DC.SetTextForeground(Colour);
  int OldMode = DC.GetBackgroundMode();
  DC.SetBackgroundMode(wxTRANSPARENT);
  bool Underscore = false;
  int H = BildBer.YMax-BildBer.YMin;
  for (unsigned int i = 0;i < Eintrag.Length();i++)
  {
    wxString str(&Eintrag[i]);
    str.Truncate(1);
    if (str == ' ')
      Underscore = false; // Nach Leerzeichen kein Tiefstellen mehr !
    if (str == '_')
      Underscore = true; // Nach _ tiefstellen, _ nicht ausgeben !
    else
      DC.DrawText(str,Diag->SymbolRad*8+BildBer.XMin+x+CharPos[i],
                  BildBer.YMin+y+(Underscore ? int(H*0.3) : 0));
  }
  DC.SetTextForeground(OldColor);
  DC.SetBackgroundMode(OldMode);
  ZeichneSymbol(x,y,DC);
}

MenueItemObj::~MenueItemObj()
{
}

void MenueItemObj::Init(const _TCHAR *Zeile,int &PosX,
                        int &PosY,unsigned long FarbVar,
                        unsigned short Opt,unsigned short Symb,
                        int Nr,EFDiagObj *DO,wxDC &DC)
{
  wxSize TE(0,0);

  Eintrag = Zeile;
  BildBer.XMin = PosX;
  BildBer.YMin = PosY;
  Diag = DO;
  Farbe = FarbVar;
  Optionen = Opt;
  Symbol = Symb;
  int w,h;
  bool Underscore = false;
  for (unsigned int i = 0;i < _tcslen(Zeile);i++)
  {
    CharPos.SetValue(i,short(TE.x));
    wxString str(&Eintrag[i]);
    str.Truncate(1);
    if (str == '_')
      Underscore = true;
    else
    {
      DC.GetTextExtent(str,&w,&h);
      TE.x += w;
    }
  }
  BildBer.XMax = BildBer.XMin+TE.x+Diag->SymbolRad*8;
  wxString str(wxT("Mg"));
  DC.GetTextExtent(str,&w,&h);
  BildBer.YMax = BildBer.YMin+(Underscore ? int(1.5*h) : int(1.1*h));
  PosX = BildBer.XMax;
  PosY = BildBer.YMax;
  SymbX = BildBer.XMin+Diag->SymbolRad*4;
  SymbY = (BildBer.YMin+BildBer.YMax)/2;
  Nummer = Nr;
}

void MenueItemObj::ZeichneSymbol(int x,int y,wxDC &DC)
{
  wxPen Pen(Farbe,Optionen & GraphDataObj::GraphFett ?
                  Diag->FettLinie : Diag->DuennGraphLinie,wxSOLID);
  DC.SetPen(Pen);
  DC.DrawLine(SymbX+x-Diag->SymbolRad*5/2,SymbY+y,
              SymbX+x+Diag->SymbolRad*5/2,SymbY+y);
  DC.DrawLine(SymbX+x+Diag->SymbolRad*5/2,SymbY+y,SymbX+x,SymbY+y);
  if (!(Symbol & GraphDataObj::GraphOhneSymbol))
    GraphSymbolObj::ZeichneSymbol(SymbX+x,SymbY+y,Nummer,DC,Diag->SymbolRad);
  DC.SetPen(wxNullPen);
}


// ----------------------------------------------------------------------------
// FunkMenueObj
// ----------------------------------------------------------------------------

int FunkMenueObj::HoleBefehl(int X,int Y)
{ // ermittelt ob Position X,Y in FunkMenue und/oder welchem MenueItem
  int Xrel = X-BildBer.XMin;
  int Yrel = Y-BildBer.YMin;
  for (size_t i = 0;i < LegendItems.size();i++)
  {
    if (LegendItems[i].PunktInObject(Xrel,Yrel))
      return LegendItems[i].Nummer;
  }
  if (PunktInObject(X,Y))
    return -2; // Indikator fuer Punkt im FunkMenue, ausserhalb von MenueItems
  else
    return -1;
}


FunkMenueObj::~FunkMenueObj()
{
  LoescheMenue();
}

void FunkMenueObj::LoescheMenue()
{
  LegendItems.clear();
}

void FunkMenueObj::Init(int XE,bool F,EFDiagObj *DO,wxDC &DC)
{
  Diag = DO;
  LoescheMenue();
  BildBer.XMin = 0;
  BildBer.YMin = 0;
  BildBer.XMax = 0;
  BildBer.YMax = 0;
  AkX = 0;
  AkY = 0;
  XExt = XE;
  Frame = F;
  int w,h;
  DC.GetTextExtent(wxString(wxT(" ")),&w,&h);
//  MarkerSize = (w/2 > 4 ? w/2 : 4);
  MarkerSize = w;
}

void FunkMenueObj::NeuEintrag(int Nr,const _TCHAR *Zeile,unsigned long FarbVar,
                              unsigned short Opt,unsigned short Symb,
                              bool LegendeOben,wxDC &DC)
{
  wxSize Tmp;

  DC.GetTextExtent(wxString(Zeile),&Tmp.x,&Tmp.y);
  Tmp.x += Diag->SymbolRad*8; // Make room for symbol output
  int x,y;
  DC.GetTextExtent(wxString(wxT(" ")),&x,&y);
  if ((LegendeOben && (Tmp.x+x+AkX+2*BildBer.XMin > XExt)) ||
      (!LegendeOben && (LegendItems.size() > 0)))
  {
    AkX = BildBer.XMin;
    DC.GetTextExtent(wxString(wxT("Mg")),&x,&y);
    AkY += int(1.1*y);
  }
  MenueItemObj HiPoint;
  int X = AkX-BildBer.XMin; // relative Coordinates for MenueItem
  int Y = AkY-BildBer.YMin;
  // initialize MenueItem -> returns X and Y end coordinates
  HiPoint.Init(Zeile,X,Y,FarbVar,Opt,Symb,Nr,Diag,DC);
  DC.GetTextExtent(wxString(wxT(" ")),&x,&y);
  X += x; // Add room for legend move marker
  if (!LegendeOben && (Y-int(1.1*y) > AkY)) // bei Underscore AkY anpassen
    AkY = Y-int(1.1*y);
  if (X+BildBer.XMin > BildBer.XMax)
    BildBer.XMax = X+BildBer.XMin; // Menue-Groesse updaten
  if (Y+BildBer.YMin > BildBer.YMax)
    BildBer.YMax = Y+BildBer.YMin;
  AkX = X+BildBer.XMin; // AkX aktualisieren
  LegendItems.push_back(HiPoint);
}

void FunkMenueObj::DrawFrame(wxDC &DC)
{ // Rahmen um Legende zeichnen
  wxPen NeuPen(*wxBLACK,Diag->DuennLinie,wxSOLID);
  const wxBrush &OldBr = DC.GetBrush();
  DC.SetPen(NeuPen);
  DC.SetBrush(*wxTRANSPARENT_BRUSH);
  DC.DrawRectangle(BildBer.XMin,BildBer.YMin,BildBer.XMax-BildBer.XMin,
                   BildBer.YMax-BildBer.YMin);
  DC.SetPen(wxNullPen);
  DC.SetBrush(OldBr);
}

void FunkMenueObj::DrawMenue(unsigned long Colour,unsigned long BColour,
                             bool Printing,wxDC &DC)
{
  if (LegendItems.size() > 0)
  { // MenueItems available
    wxBrush brs(BColour,wxSOLID);
    const wxBrush &OldBr = DC.GetBrush();
    DC.SetBrush(brs);
    wxPen pen(BColour,1,wxSOLID);
    DC.SetPen(pen);
    DC.DrawRectangle(BildBer.XMin,BildBer.YMin,BildBer.XMax-BildBer.XMin,
                     BildBer.YMax-BildBer.YMin);
    DC.SetBrush(OldBr);
    DC.SetPen(wxNullPen);
    for (size_t i = 0;i < LegendItems.size();i++)
      LegendItems[i].Draw(MenueItemObj::ItemEnabled,BildBer.XMin,BildBer.YMin,Colour,DC);

    if (Frame)
      DrawFrame(DC);
    if (!Printing)
    {
      wxBrush OrgBr = DC.GetBrush();
      DC.SetBrush(*wxBLACK_BRUSH);
      DC.SetPen(*wxBLACK_PEN);
      DC.DrawRectangle(BildBer.XMax-MarkerSize,BildBer.YMin,
                       MarkerSize,MarkerSize);
      DC.SetBrush(OrgBr);
      DC.SetPen(wxNullPen);
    }
  }
}

void FunkMenueObj::Selekt(int Nr,unsigned long Colour,wxDC &DC)
{
  for (size_t i = 0;i < LegendItems.size();i++)
  {
    if (i+1 != size_t(Nr))
      LegendItems[i].Draw(0,BildBer.XMin,BildBer.YMin,Colour,DC);
    else
      LegendItems[i].Draw(MenueItemObj::ItemEnabled,
                    BildBer.XMin,BildBer.YMin,Colour,DC);
  }
  if (Frame)
    DrawFrame(DC);
}

// ----------------------------------------------------------------------------
// EFDiagObj
// ----------------------------------------------------------------------------

void EFDiagObj::Init(BildType &BildBerVar,GraphManagerObj *YD,
                     FunkMenueObj *FM,DarstDataObj *DD,const FontType &A,
                     const FontType &T,const FontType &AT,
                     const unsigned long &col,
                     const NumListFormat &LDV,
                     wxDC &DC,const _TCHAR *DT,int DNr)
{
  DiagNr = DNr;
  YDaten = YD;
  FunkMenue = FM;
  DarstData = DD;
  Markiert = false;
  Nummer = 0;
  if (DT)
    DiagText.SetStr(DT);
  GrInit(BildBerVar,DarstData->XAxis,DarstData->YAxis,A,T,AT,col,LDV,DarstData->Mode,
         DarstData->NumDigits,DC);
  Mark1.Init(BildBerVar,DarstData->XAxis,DarstData->YAxis,0);
  Mark2.Init(BildBerVar,DarstData->XAxis,DarstData->YAxis,0);
//  SetDashedStyle(0,0); // Durchgezogene Linien, keine Strichpunkte
}

void EFDiagObj::Markieren(wxDC &DC,bool mark)
{
  double HiX;

  HiX = YDaten->AkGraph().GetX(Nummer);
  Mark1.UnMark(DC);
  if (Mark2.Markiert())
    Mark2.UnMark(DC);
  if (mark)
  {
    MarkX = YDaten->AkGraph().GetX(Nummer);
    MarkY = DarstData->YUmrechnen(YDaten->AkGraph().GetY(Nummer),HiX);
    Mark2.Mark((int)XPos(MarkX),(int)YPos(MarkY),DC);
  }
  Mark1.Mark((int)XPos(YDaten->AkGraph().GetX(Nummer)),
             (int)YPos(DarstData->YUmrechnen(YDaten->AkGraph().GetY(Nummer),
                                             HiX)),DC);
  Text(false);
}

bool EFDiagObj::DelGraph()
{
  YDaten->LoeschAkGraph();
  return true;
}

void EFDiagObj::Move(int Ort,int DeltaX,wxDC &DC)
{
  unsigned int AltNummer,i;
  double Ak;

  if (YDaten->YDatenVorhanden())
  { // Graphen vorhanden
    DarstData->YAxis.SetAkEint(YDaten->AkGraph().YAxisIndex[DiagNr]);
    AltNummer = Nummer;
    switch (Ort)
    {
      case Peak: // Peak ab Nummer bestimmen
        {
          if (YDaten->AkGraph().GetPunkte() < 3)
            break;
          int f = 0;
          do
          {
            if (Nummer < YDaten->AkGraph().GetPunkte()-2)
              i = Nummer;
            else
              i = 0;
            Nummer = 0;
            do
            {
              // Wert mit Vorgaenger und Nachfolger vergleichen
              // zur Spitzen-Ermittlung
              i++;
              double HiX2 = YDaten->AkGraph().GetX(i-1);
              double YUnt = DarstData->YUmrechnen(YDaten->AkGraph().GetY(i-1),
                                                  HiX2);
              HiX2 = YDaten->AkGraph().GetX(i);
              Ak = DarstData->YUmrechnen(YDaten->AkGraph().GetY(i),HiX2);
              HiX2 = YDaten->AkGraph().GetX(i+1);
              double YOber = DarstData->YUmrechnen(YDaten->AkGraph().GetY(i+1),
                                                   HiX2);
              if (((YUnt < Ak) && (YOber < Ak)) ||
                  ((YUnt > Ak) && (YOber > Ak)))
                Nummer = i;
            }
            while ((Nummer == 0) && (i != YDaten->AkGraph().GetPunkte()-2));
            f++;
          }
          while ((Nummer == 0) && (f != 2));
        }
        break;
      case AkXPos :  // Punkt mit X-Bildschirm-Koordinate DeltaX markieren
        if (DeltaX > BildBer.XMax)
          Ak = (*XAxis)->Max;
        else
        if (DeltaX < BildBer.XMin)
          Ak = (*XAxis)->Min;
        else
          Ak = XWert(DeltaX);
        i = 0;
        if (DarstData->Conversion == DarstDataObj::ortskurY)
          while ((Ak < real(YDaten->AkGraph().GetY(i))) &&
                 (i < YDaten->AkGraph().GetPunkte()-1))
            i++;
        else
          while ((Ak > YDaten->AkGraph().GetX(i)) &&
                 (i < YDaten->AkGraph().GetPunkte()-1))
            i++;
        Nummer = i;
        break;
      case SchnellVorWaerts : // im 10-Schritt vorwaerts
        if (Nummer+10 < YDaten->AkGraph().GetPunkte())
          Nummer += 10;
        else
          Nummer = YDaten->AkGraph().GetPunkte()-1;
        break;
      case SchnellRueckWaerts : // im 10-Schritt rueckwaerts
        if (Nummer > 10)
          Nummer -= 10;
        else
          Nummer = 0;
        break;
      case VorWaerts : // im 1-Schritt vorwaerts
        if (Nummer < YDaten->AkGraph().GetPunkte())
          Nummer++;
        break;
      case RueckWaerts : // im 1-Schritt rueckwaerts
        if (Nummer > 0) Nummer--;
        break;
      case Min : // absolutes Minimum oder Maximum finden
      case Max :
        Nummer = 0; // 1. Punkt als Vergleich
        for (i = 0;i < YDaten->AkGraph().GetPunkte();i++)
        { // alle Punkte pruefen
          // X des aktuellen Extremwertes
          double HiX1 = YDaten->AkGraph().GetX(Nummer);
          double HiX2 = YDaten->AkGraph().GetX(i); // X des aktuellen Wertes
          if (Ort == Min)
          {
            if (DarstData->YUmrechnen(YDaten->AkGraph().GetY(Nummer),HiX1) >
                DarstData->YUmrechnen(YDaten->AkGraph().GetY(i),HiX2))
              Nummer = i;
          }
          else
          if (DarstData->YUmrechnen(YDaten->AkGraph().GetY(Nummer),HiX1) <
              DarstData->YUmrechnen(YDaten->AkGraph().GetY(i),HiX2))
            Nummer = i;
        }
        break;
      case Links : // linkesten Punkt markieren
        Nummer = 0;
        break;
      case Rechts : // rechtesten Punkt markieren
        Nummer = YDaten->AkGraph().GetPunkte()-1;
        break;
    }
    if ((Nummer != AltNummer) || ((Ort == NeuY) ||
        ((Ort == AkXPos) && !Mark1.Markiert())))
    { // wenn sich die Nummer geaendert hat
      if (Nummer >= YDaten->AkGraph().GetPunkte())
        Nummer = YDaten->AkGraph().GetPunkte()-1;
      double HiX1 = YDaten->AkGraph().GetX(Nummer);
      Ak = DarstData->YUmrechnen(YDaten->AkGraph().GetY(Nummer),HiX1);
      Mark1.UnMark(DC);
      Mark1.SetFarbe(YDaten->AkGraph().Farbe);
      Mark1.Mark((int)XPos(HiX1),(int)YPos(Ak),DC);
      Text(false);
    }
  }
}

bool EFDiagObj::NaechsteFunktion(int Mode,unsigned long Colour,wxDC &DC)
{
  int DeltaX = 0;
  bool NF;
  NF = true;

  if (YDaten->YDatenVorhanden())
  { // wenn Graphen vorhanden
     if (Mode == NotMoveCursor)
     { // Funktion nicht wechseln
       if (!YDaten->LoadYFeld())
       {
         YDaten->ResetYDaten();
         YDaten->LoadYFeld();
         NF = false;
       }
       FunkMenue->Selekt(YDaten->GetAkPos(),Colour,DC);
     }
     else
     { // Funktion Nummer Mode laden
       if (Mode <= 0)
         Mode = 1;
       if (YDaten->LoadYFeld(Mode))
         FunkMenue->Selekt(Mode,Colour,DC);
     }
     YDaten->AkFeldUmrechnen(*Parser);
     if (Mark1.Markiert())
       Move(NeuY,DeltaX,DC);
     return NF;
  }
  return false;
}

void EFDiagObj::Text(bool Clear)
{
  MGPStr AString,DString;
  MGPStr HiString;
  double HiX;

  if (!Mark2.Markiert())
  {
    HiX = YDaten->AkGraph().GetX(Nummer);
    if (!Clear)
    {
      AString.SetStr(wxT(" X:"));
      Parser->Ausgabe(HiString,YDaten->AkGraph().GetX(Nummer),ZahlForm);
      AString.AddStr(HiString);
      AString.AddStr(Parser->GetLDV().ListSeparator);
      AString.AddStr(wxT(" Y:"));
      Parser->Ausgabe(HiString,
                      DarstData->YUmrechnen(YDaten->AkGraph().GetY(Nummer),HiX),
                      ZahlForm);
      AString.AddStr(HiString);
    }
    AkPosText.SetStr(AString);
  }
  else
  {
    HiX = YDaten->AkGraph().GetX(Nummer);
    if (!Clear)
    {
      DString.SetStr(wxT("dX:"));
      Parser->Ausgabe(HiString,YDaten->AkGraph().GetX(Nummer)-MarkX,ZahlForm);
      DString.AddStr(HiString);
      DString.AddStr(Parser->GetLDV().ListSeparator);
      DString.AddStr(wxT(" dY:"));
      double y = DarstData->YUmrechnen(YDaten->AkGraph().GetY(Nummer),HiX);
      Parser->Ausgabe(HiString,y-MarkY,ZahlForm);
      DString.AddStr(HiString);
      try
      {
        if (YDaten->AkGraph().GetX(Nummer) != MarkX)
        {
          double Erg = (y-MarkY)/(YDaten->AkGraph().GetX(Nummer)-MarkX);
          DString.AddStr(Parser->GetLDV().ListSeparator);
          DString.AddStr(wxT(" dY/dX:"));
          Parser->Ausgabe(HiString,Erg,ZahlForm);
          DString.AddStr(HiString);
        }
      }
      catch (...)
      {
      }
    }
  }
  AkDelText.SetStr(DString);
}

void EFDiagObj::ZeichneGraph(unsigned int AkFarbe,wxDC &DC)
{
  double f;
  Cpx v;
  long x,y;
  wxPen NeuPen;

  if ((YDaten->AkGraph().GraphOptionen & GraphDataObj::GraphFett) == 1)
    NeuPen = wxPen(AkFarbe,FettLinie,wxSOLID);
  else
    NeuPen = wxPen(AkFarbe,DuennGraphLinie,wxSOLID);
  DC.SetPen(NeuPen);
  bool ErsterPunkt = true;
  double minlim = (*XAxis)->Min-3*((*XAxis)->Max-(*XAxis)->Min);
  double maxlim = (*XAxis)->Max+3*((*XAxis)->Max-(*XAxis)->Min);
  int oldstyle = YDaten->AkGraph().ZeichenArt & ~GraphDataObj::gPunkteSplines;
  bool mark = (oldstyle == GraphDataObj::gPunkteMarkieren) ||
              (oldstyle == GraphDataObj::gPunkteVerbMark);
  bool line = (oldstyle == GraphDataObj::gPunkteVerbinden) ||
              (oldstyle == GraphDataObj::gPunkteVerbMark);
  bool splines = (YDaten->AkGraph().ZeichenArt & GraphDataObj::gPunkteSplines);
  for (size_t i = 0;i < YDaten->AkGraph().GetPunkte();i++)
  {
    f = YDaten->AkGraph().GetX(i);
    v = Cpx(DarstData->YUmrechnen(YDaten->AkGraph().GetY(i),f),0);
    if ((f > minlim) && (f < maxlim))
    {
      x = XPos(f);
      y = YPos(real(v));
      if (ErsterPunkt)
      {
        ResetCLineTo(f,real(v),splines);
        ErsterPunkt = false;
      }
      if (mark)
      {
        if ((x >= BildBer.XMin+2) && (x <= BildBer.XMax-2) &&
            (y >= BildBer.YMin+2) && (y <= BildBer.YMax-2))
          DC.DrawRectangle(int(x-40*Zoom),int(y-40*Zoom),
                           int(1+80*Zoom),int(1+80*Zoom)); //Punkte Markieren
      }
      if (line)
        CLineTo(DC,f,real(v));
    }
  }
  FlushCLineTo(DC);
  if (!(YDaten->AkGraph().GraphSymbol == GraphDataObj::GraphOhneSymbol))
  {
    double Step = 0;
    if ((DarstData->Conversion == DarstDataObj::ortskurY) ||
        (DarstData->XAxis->GetLog() != DarstDataObj::linX))
    {
      if ((YDaten->AkGraph().GetX(YDaten->AkGraph().GetPunkte()-1) > 0) &&
          (YDaten->AkGraph().GetX(0) > 0))
        Step = exp(log(YDaten->AkGraph().GetX(YDaten->AkGraph().GetPunkte()-1)/
               YDaten->AkGraph().GetX(0))/
               (YDaten->AkGraph().GetPunkte()-1));
    }
    else
      Step = YDaten->AkGraph().GetX(YDaten->AkGraph().GetPunkte()-1)-
                                    YDaten->AkGraph().GetX(0);
    double AltX = YDaten->AkGraph().GetX(0);
    double minlim = (*XAxis)->Min-((*XAxis)->Max-(*XAxis)->Min);
    double maxlim = (*XAxis)->Max+((*XAxis)->Max-(*XAxis)->Min);
    int bildxmin = BildBer.XMin+SymbolRad;
    int bildxmax = BildBer.XMax-SymbolRad;
    int bildymin = BildBer.YMin+SymbolRad;
    int bildymax = BildBer.YMax-SymbolRad;
    bool lin = (DarstData->XAxis->GetLog() == DarstDataObj::linX) &&
               (DarstData->Conversion != DarstDataObj::ortskurY);
    for (size_t i = 0;i < YDaten->AkGraph().GetPunkte();i++)
    {
      f = YDaten->AkGraph().GetX(i);
      v = Cpx(DarstData->YUmrechnen(YDaten->AkGraph().GetY(i),f),0);
      if ((f > minlim) && (f < maxlim))
      {
        x = XPos(f);
        y = YPos(real(v));
        if (lin)
        {
          if (GraphDataObj::MaxFMark*(YDaten->AkGraph().GetX(i)-AltX) > Step)
          {
            if ((x >= bildxmin) && (x <= bildxmax) &&
                (y >= bildymin) && (y <= bildymax))
              GraphSymbolObj::ZeichneSymbol((int)x,(int)y,YDaten->GetAkPos(),
                                            DC,SymbolRad);
            AltX = YDaten->AkGraph().GetX(i);
          }
        }
        else
        if ((Step > 0) && (AltX > 0) && (YDaten->AkGraph().GetX(i) > 0))
        {
          if (log(YDaten->AkGraph().GetX(i)/AltX)/log(Step) >
              (YDaten->AkGraph().GetPunkte())/GraphDataObj::MaxFMark)
          {
            if ((x >= bildxmin) && (x <= bildxmax) &&
                (y >= bildymin) && (y <= bildymax))
              GraphSymbolObj::ZeichneSymbol((int)x,(int)y,YDaten->GetAkPos(),
                                            DC,SymbolRad);
            AltX = YDaten->AkGraph().GetX(i);
          }
        }
      }
    }
  }
  DC.SetPen(wxNullPen);
}
