// Version 6.05 20.07.2008 Markus Greither
// Copyright (c) 1991-2008 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file efkrzn.cpp
    \brief Source code for the elimination part of the symbolic function solver
*/

#ifdef __BORLANDC__

#include "efprec.h"

#pragma hdrstop
#else

#ifdef __DMC__
#include <typeinfo.h>
#endif
#include <cstdlib>
#include <complex>
#include <limits>
#include <cstring>
#include <set>
#include <functional>
#include <cstdio>
#include <ctime>

using namespace std;

#include "efdefs.h"
#include "mg.h"
#include "cpxpsr.h"
#include "efsolver.h"
#include "efkrzn.h"

#endif

// ----------------------------------------------------------------------------
// BitArrObj
// ----------------------------------------------------------------------------

bool BitArrObj::BitsSet(unsigned int Bit1,unsigned int Bit2) const
{
  for (unsigned int i = Bit1;i <= Bit2;i++)
    if (!BitSet(i))
      return false;
  return true;
}

int BitArrObj::FirstCommonBit(BitArrObj &B2,unsigned int Anf)
{
  while (Anf <= MxBit)
  {
    if (BitSet(Anf) && B2.BitSet(Anf))
      return Anf;
    else
      Anf++;
  }
  return -1;
}

void BitArrObj::CopyBits(BitArrObj &B2,unsigned int Anf,unsigned int End)
{
  B2.Clear();
  for (unsigned int i = Anf;i <= End;i++)
  {
    if (BitSet(i))
      B2.SetBit(i-Anf);
    else
      B2.DelBit(i-Anf);
  }
}

void BitArrObj::BitString(MGPStr &res)
{
  for (unsigned int i = 0;i <= MxBit;i++)
  {
    if (BitSet(i))
      res.AddStr(_TEXT("1"));
    else
      res.AddStr(_TEXT("0"));
  }
}

#if VECTOR_BITS
void BitArrObj::Inc()
{
  bool carry = true;
  vector<bool>::iterator iter = begin();
  while (carry)
  {
    if (iter == end())
    {
      resize(size()+1,false);
      iter = --end();
    }
    carry = *iter;
    *iter = !carry;
    iter++;
  }
  if (BitSet(MxBit+1))
    MxBit++;
  AnzBit = 0;
}
#else
void BitArrObj::Inc()
{
  unsigned int i = 0;

  do
  {
    Data[i]++;
    i++;
    if (!Data[i-1] && (i == NBytes))
      IncMem(i+1);
  }
  while (Data[i-1] == 0);
  if (BitSet(MxBit+1))
    MxBit++;
  AnzBit = 0;
}
#endif

#if VECTOR_BITS
void BitArrObj::SetBits(const BitArrObj &other)
{
  if (other.MxBit > MxBit)
    MxBit = other.MxBit;
  if (other.size() > size())
    resize(other.size(),false);
  unsigned int i = 0;
  while (i < other.size())
  {
    at(i) = at(i) | other.at(i);
    i++;
  }
  AnzBit = 0;
}
#else
void BitArrObj::SetBits(const BitArrObj &other,bool exactcopy)
{
  unsigned int i = 0;
  if ((other.MxBit > MxBit) || exactcopy)
    MxBit = other.MxBit;
  if (other.NBytes > NBytes)
    IncMem(other.NBytes+1);
  do
  {
    if (i < other.NBytes)
      Data[i] |= other.Data[i];
    else
      Data[i] = 0;
    i++;
  }
  while (i < NBytes);
  AnzBit = 0;
}
#endif

#if VECTOR_BITS
void BitArrObj::Shl()
{
  if ((MxBit > 0) || BitSet(0))
    MxBit++;
  insert(begin(),1,false);
}
#else
void BitArrObj::Shl()
{
  unsigned int i = 0;
  bool C = false,C1;

  if ((MxBit > 0) || BitSet(0))
    MxBit++;
  do
  {
    C1 = C;
    C = (Data[i] > 127);
    Data[i] <<= 1;
    if (C1)
      Data[i] |= 1;
    i++;
    if (C && (i == NBytes))
      IncMem(i+1);
  }
  while (i < NBytes);
}
#endif

// ermittelt die Nummer des hoechsten gesetzten Bits
void BitArrObj::GetMaxBit(unsigned int StartWert)
{
  MxBit = StartWert;
  while ((MxBit > 0) && !BitSet(MxBit))
    MxBit--;
}

void BitArrObj::GetAnzBits(unsigned int Anf,int End)
{
  unsigned int i = Anf;
  unsigned int E = (End == -1 ? MxBit : End);

  AnzBit = 0;
  while (i <= E)
  {
    if (BitSet(i))
      AnzBit++;
    i++;
  }
}

unsigned int BitArrObj::CommonBits(const BitArrObj &B2,unsigned int Anf) const
{
  unsigned int CmnB = 0;
  for (;Anf <= MxBit;Anf++)
    CmnB += (BitSet(Anf) && B2.BitSet(Anf));
  return CmnB;
}

unsigned int BitArrObj::OnlyCommonBits(const BitArrObj &B2,unsigned int Anf) const
{
  unsigned int CmnB = 0;
  for (;Anf <= MxBit;Anf++)
  {
    if (BitSet(Anf))
      if (B2.BitSet(Anf))
        CmnB++;
      else
        return 0;
  }
  return CmnB;
}

bool BitArrObj::Identical(const BitArrObj &B2) const
{
  bool Id = true;
  unsigned int i;
  for (i = 0;((i <= MxBit) || (i <= B2.MaxBit())) && (Id == true);i++)
    Id = (BitSet(i) == B2.BitSet(i));
  return Id;
}

#if VECTOR_BITS
void BitArrObj::DelBit(unsigned int Bit)
{
  if (Bit >= size())
    return;
  bool value = at(Bit);
  erase(begin() + Bit);
  if (MxBit > Bit)
    MxBit--;
  else
    GetMaxBit(Bit);
  if (value)
    AnzBit--;
}
#else
void BitArrObj::DelBit(unsigned int Bit)
{
  bool C = false,C1;
  unsigned int i;

  // Bytes oberhalb um 1 nach rechts schieben
  for (i = NBytes-1;i > (Bit>>3);i--)
  {
    C1 = C;
    C = (Data[i] & 1);
    Data[i] >>= 1;
    if (C1)
      Data[i] |= 128; // bertrag bercksichtigen
  }
  for (i = Bit;(i & 7) != 7;i++) // Bits oberhalb um 1 nach rechts
  {
    if (BitSet(i+1))
      SetBit(i);
    else
      ClearBit(i);
  }
  if (C)
    SetBit((Bit>>3)*8+7); // oberstes Bit des aktuellen Bytes korrigieren
  else
    ClearBit((Bit>>3)*8+7);
}
#endif

// ----------------------------------------------------------------------------
// PrimZahlObj
// ----------------------------------------------------------------------------

PrimZahlObj::PrimZahlObj(long count) : BitArrObj()
{
  for (int i = 0;i <= count;i++)
    SetBit(i);
  // Sieb des Erathostenes
  for (int i = 2;i <= count;i++)
  {
    int j = i+i;
    while (j <= count)
    { // alle n-fachen von i sind keine Primzahlen
      ClearBit(j);
      j += i;
    }
  }
}

// ----------------------------------------------------------------------------
// SumFakMatrixObj
// ----------------------------------------------------------------------------

// Debug-Funktion zum Anzeigen der Matrix, opt. Ausgabe an Datei Filename
void SumFakMatrixObj::MatrixOut(const _TCHAR *Filename)
{
  FILE *file;
  if (Filename)
    file = _tfopen(Filename,_TEXT("at"));
  else
    file = stdout;
  _ftprintf(file,_TEXT(" "));
  unsigned int i;
  for (i = 0;i < Vorz.Length();i++)
    _ftprintf(file,_TEXT(" %c"),(_TCHAR)Vorz[i]);
  _ftprintf(file,_TEXT("\r\n"));
  for (i = 1;i <= (unsigned int)AnzProSum.GetMaxEintrag();i++)
  {
    Faktor.SetAkEint(i);
    AnzProSum.SetAkEint(i);
    _ftprintf(file,_TEXT("%i"),Faktor.GetAkStr()[2]);
    int f;
    for (f = 1;f <= AnzLSum;f++)
      _ftprintf(file,_TEXT(" %i"),AnzProSum.GetAkEint()[f]);
    _ftprintf(file,_TEXT(" |"));
    for (f = AnzLSum+1;f <= AnzLSum+AnzRSum;f++)
      _ftprintf(file,_TEXT(" %i"),AnzProSum.GetAkEint()[f]);
    _ftprintf(file,_TEXT("\r\n"));
  }
  _ftprintf(file,_TEXT("\r\n"));
  if (Filename)
    fclose(file);
}

void SumFakMatrixObj::Create(MGPStr &LAbsStr,MGPStr &RAbsStr)
{ // Summanten-Faktoren-Matrix erzeugen
  _TCHAR RechZei1;
  unsigned int p = 0;
  int f = 0;
  MGPStr *AkStr = &LAbsStr;

  AnzLSum = 0;
  AnzRSum = 0;
  while ((p < AkStr->Length()) && !Fehler && (f < 255))
  { // Faktoren die Summanten zuordnen, in denen Sie vorkommen
    f++;
    MGPStr SumStr;
    Summant(AkStr->c_str(),SumStr,p,RechZei1);
    if (RechZei1 == '-') // Vorzeichen des Summanten eintragen
      Vorz.AddChar('-');
    else
      Vorz.AddChar('+');
    unsigned int p1 = 0;
    while ((p1 < SumStr.Length()) && !Fehler)
    { // alle Faktoren des aktuellen Summanten
      MGPStr FakStr;
      NextFakt(SumStr,FakStr,p1);
      Faktor.Begin();
      if (FakStr[2] != 1)
      { // keine 1 eintragen !
        unsigned int p2 = 0;
        while (Faktor.NextEint() && (p2 == 0))
        { // aktuellen Faktor suchen
          if ((Faktor.GetAkStr()[0] == FakStr[0]) &&
              (Faktor.GetAkStr()[2] == FakStr[2]))
            p2 = Faktor.AkEintNummer();
        }
        if (!p2)
        { // wenn Faktor noch nicht vorhanden, erzeugen
          Faktor.NeuAkEint();
          Faktor.SetAkStr(FakStr.c_str());
          AnzProSum.NeuAkEint();
        }
        else
        {
          Faktor.SetAkEint(p2);
          AnzProSum.SetAkEint(p2);
        }
        // Anzahl des Faktors in diesem Summanten eintragen
        AnzProSum->IncValue(f);
      }
    }
    if (p >= AkStr->Length())
    {
      if (AkStr == &LAbsStr)
      { // RAbsStr nach LAbsStr bearbeiten
        AkStr = &RAbsStr;
        p = 0;
        AnzLSum = f;
      }
      else
        AnzRSum = f-AnzLSum;
    }
  }
//  MatrixOut("c:\\tst.txt");
}

// ----------------------------------------------------------------------------
// FaktorenVar
// ----------------------------------------------------------------------------

// Debug-Funktion zur Anzeige einer Matrix
void FaktorenVar::MatrixOut(BitArrKettObj &AkArr)
{
  for (int i = 1;i <= AkArr.GetMaxEintrag();i++)
  {
    FakStrings.SetAkEint(i);
    AkArr.SetAkEint(i);
    printf("%i",FakStrings.GetAkStr()[2]);
    int f;
    for (f = 1;f <= Matrix.AnzLSum;f++)
    {
      if (AkArr->BitSet(f))
        printf(" %i",1);
      else
        printf(" %i",0);
    }
    printf(" |");
    for (f = 1;f <= Matrix.AnzRSum;f++)
    {
      if (AkArr->BitSet(f+Matrix.AnzLSum))
        printf(" %i",1);
      else
        printf(" %i",0);
    }
    printf("\r\n");
  }
  printf("\r\n");
}

// Debug-Funktion zur Anzeige der FaktorenGruppen
void FaktorenVar::FGOut(ShortArrKettObj &FI)
{
  for (int i = 1;i <= FI.GetMaxEintrag();i++)
  {
    FI.SetAkEint(i);
    for (unsigned int f = 0;f <= FI->MaxInt();f++)
      printf("  %i",FI.GetAkEint()[f]);
    printf("\r\n");
  }
  printf("\r\n");
}

FaktorenVar::FaktorenVar(SumFakMatrixObj &Mtrx)
 : Matrix(Mtrx)
{ // erzeugt die FaktorenVariation unter Verwendung der Mtrx
  Matrix.AnzProSum.Begin();
  while (Matrix.AnzProSum.NextEint())
  { // Faktoren-Array aufbauen
    Matrix.Faktor.SetAkEint(Matrix.AnzProSum.AkEintNummer());
    FaktorVars.NeuAkEint();
    FaktorVars->Init
    (
      Matrix.AnzProSum.GetAkEint(),
      Matrix.Faktor.GetAkStr(),
      FakProSums,
      FakStrings,
      FaksProLine,
      Matrix.AnzRSum+Matrix.AnzLSum);
  }
//  MatrixOut(FakProSums);
}

double FaktorenVar::GetAnzKombs()
{ // Abschtzung der Anzahl der Kombinationen (grob !)
  double Kombs = 1;
  FaktorVars.Begin();
  while (FaktorVars.NextEint())
  {
    Kombs *= FaktorVars->GetAnzLines() *
             (Matrix.AnzRSum+Matrix.AnzLSum);
  }
  return Kombs;
}

bool FaktorenVar::NextVar()
{ // naechste Variante suchen
  bool Valid = false;
  FaktorVars.Begin();
  while (!Valid && FaktorVars.NextEint())
  {
    do
    {
      Valid = FaktorVars->NextVar(FakProSums,FaksProLine);
    }
    while (Valid && !CheckVar());
  }
//  MatrixOut(FakProSums);
  return Valid;
}

bool FaktorenVar::CheckVar()
{ // Prueft ob diese Variation ueberhaupt kuerzbar ist
  int Anz1L = 0,Anz2L = 0;
  int Anz1R = 0,Anz2R = 0;
  FakProSums.Begin();
  bool Valid = true;
  while (Valid && FakProSums.NextEint())
  { // alle Zeilen pruefen
    // Anzahl LAbsStr-Bits ermitteln
    int tmp = FakProSums->AnzBits(1,Matrix.AnzLSum);
    if (tmp)
    { // dieser Faktor kommt auf der linken Seite vor
      if (!Anz1L && (tmp != Matrix.AnzLSum))
      {
        if (Matrix.AnzLSum % tmp)
          Valid = false; // Zahl der Summanten muss vielfaches sein
        else
          Anz1L = tmp; // 1. moegliche Anzahl festlegen
      }
      else
      if ((tmp != Anz1L) && !Anz2L && (tmp != Matrix.AnzLSum))
      {
        if (Matrix.AnzLSum % tmp)
          Valid = false; // Zahl der Summanten muss vielfaches sein
        else
          Anz2L = tmp; // 2. moegliche Anzahl festlegen
      }
      else
      if ((tmp != Anz1L) && (tmp != Anz2L) && (tmp != Matrix.AnzLSum))
        Valid = false;
    }
    tmp = FakProSums->AnzBits(Matrix.AnzLSum+1,
                                         Matrix.AnzLSum+Matrix.AnzRSum);
    if (tmp)
    { // dieser Faktor kommt auf der rechten Seite vor
      if (!Anz1R && (tmp != Matrix.AnzRSum))
      {
        if (Matrix.AnzRSum % tmp)
          Valid = false; // Zahl der Summanten muss vielfaches sein
        else
          Anz1R = tmp; // 1. moegliche Anzahl festlegen
      }
      else
      if ((tmp != Anz1R) && !Anz2R && (tmp != Matrix.AnzRSum))
      {
        if (Matrix.AnzRSum % tmp)
          Valid = false; // Zahl der Summanten muss vielfaches sein
        else
          Anz2R = tmp; // 2. moegliche Anzahl festlegen
      }
      else
      if ((tmp != Anz1R) && (tmp != Anz2R) && (tmp != Matrix.AnzRSum))
        Valid = false;
    }
  }
  return Valid;
}

// ermittelt die Faktorengruppen
// (Faktoren, die in den gleichen Summanten vorkommen)
void FaktorenVar::FaktorenGruppen(BitArrKettObj &FaktorGr,
                                  ShortArrKettObj &FaktorIndex)
{
  FakProSums.Begin();
  while (FakProSums.NextEint())
  { // alle Faktor-Varianten durchgehen und Gruppen ermitteln
    FaktorGr.Begin();
    bool Ident = false;
    // feststellen, ob diese FaktorenGruppe (selbe Summanten) schon eingetragen
    while (!Ident && FaktorGr.NextEint())
      Ident = FaktorGr->Identical(FakProSums.GetAkEint());
    if (!Ident)
    { // wenn neu, eintragen
      if (FakProSums->AnzBits() > 0)
      { // nur vorhandene Faktoren eintragen
        FaktorGr.NeuAkEint();
        FaktorIndex.NeuAkEint();
        FaktorGr->SetBits(FakProSums.GetAkEint());
        FaktorIndex->SetValue(0,(short)FakProSums.AkEintNummer());
      }
    }
    else
    { // wenn bereits vorhanden, Faktoren-Index in bestehendes Array eintragen
      FaktorIndex.SetAkEint(FaktorGr.AkEintNummer());
      FaktorIndex->SetValue(FaktorIndex->MaxInt()+1,
                                       (short)FakProSums.AkEintNummer());
    }
  }
}

// fasst Faktoren, die in den gleichen Summanten vorkommen,
// zu Faktorengruppen (FG) zusammmen
// und prueft, welche Kombinationen der FGs gekuerzt werden koennen
bool FaktorenVar::Kuerzen(MGPStr &LAbsStr,MGPStr &RAbsStr,MGPStr &ElimStr)
{

  BitArrKettObj FaktorGr;
  ShortArrKettObj FaktorIndex;

  FaktorenGruppen(FaktorGr,FaktorIndex);
//  MatrixOut(FaktorGr);
  FaktorGr.NeuAkEint(); // Eintrag fuer 1 vorsehen
  bool Kuerzen = false;
  BitArrObj FgKomb;
  BitArrObj Komb;
  do
  {
    Komb.Clear();
    do
    {
      FgKomb.Inc(); // Kombination auswaehlen
    }
    while ((FgKomb.BitSet(FaktorGr.GetMaxEintrag()-1) &&
           (FgKomb.AnzBits() <= 1) || // Kombs mit 1 nur mit mind. 2 FGs
           // FG-Anzahl muss Teiler von Summantenzahl sein
           (Matrix.AnzLSum % FgKomb.AnzBits() != 0) ||
           (Matrix.AnzRSum % FgKomb.AnzBits() != 0) ||
           // Nicht mehr FGs als Summanten auf einer Seite
           ((int)FgKomb.AnzBits() > Matrix.AnzLSum) ||
           ((int)FgKomb.AnzBits() > Matrix.AnzRSum)) &&
           // noch nicht letzte Komb erreicht
           ((int)FgKomb.MaxBit() <= FaktorIndex.GetMaxEintrag()));
    if ((int)FgKomb.MaxBit() <= FaktorIndex.GetMaxEintrag())
    { // weitere Kombination gefunden
//      MGPStr tmp;
//      FgKomb.BitString(tmp);
//      printf("%s\r\n\r\n",(const char *)tmp);
      for (unsigned int f = 0;f <= FgKomb.MaxBit();f++)
      { // alle Faktorengruppen
        if (FgKomb.BitSet(f))
        { // wenn diese Faktorengruppe ausgewaehlt
          FaktorGr.SetAkEint(f+1);
          if (f == (unsigned int)FaktorIndex.GetMaxEintrag())
          { // 1 er in den fehlenden Summanten einsetzen
            FaktorGr->Clear();
            for (int i = 1;i <= Matrix.AnzLSum+Matrix.AnzRSum;i++)
              if (!Komb.BitSet(i))
                FaktorGr->SetBit(i);
            Komb.SetBits(FaktorGr.GetAkEint());
          }
          else
          { // Feststellen in welchen Summanten die Kombination vorkommt
            if (Komb.CommonBits(FaktorGr.GetAkEint()))
              break; // wenn Ueberlappung, weiter
            else
              Komb.SetBits(FaktorGr.GetAkEint());
          }
        }
      }
//      MatrixOut(FaktorGr);
      // Kombination gefunden, die in allen Summanten vorkommt
      if ((int)Komb.AnzBits() == Matrix.AnzLSum+Matrix.AnzRSum)
      { // wenn gueltige Kombination gefunden, andere Faktorengruppen pruefen
        Kuerzen = true;
        int ComBits,ComBitsR = 0;
        bool First;
        for (unsigned int f = 0;(f <= FgKomb.MaxBit()) && Kuerzen;f++)
        { // alle FGs nicht in der Kombination durchgehen
          ComBits = 0;
          if (!FgKomb.BitSet(f))
          { // wenn nicht in der gesuchten Kombination
            FaktorGr.SetAkEint(f+1);
            BitArrObj &tmp = FaktorGr.GetAkEint();
            First = true;
            for (int i = 0;i < FaktorGr.GetMaxEintrag();i++)
            { // alle anderen FGs durchgehen
              if (FgKomb.BitSet(i))
              { // FG der Kombination
                FaktorGr.SetAkEint(i+1);
                if (First)
                {
                  // bei 1. FG Zahl der Ueberschneidungen ermitteln
                  ComBits = tmp.CommonBits(FaktorGr.GetAkEint());
                  // bei 1. FG Zahl der Ueberschneidungen ermitteln
                  ComBitsR = tmp.CommonBits(FaktorGr.GetAkEint(),
                                            Matrix.AnzLSum+1);
                }
                else
                {
                  int NeuCom = tmp.CommonBits(FaktorGr.GetAkEint());
                  int NeuComR = tmp.CommonBits(FaktorGr.GetAkEint(),
                                               Matrix.AnzLSum+1);
                  // bei weiteren FGs sicherstellen da
                  // gleiche Zahl Ueberschneidungen
                  if (!ComBits && !ComBitsR ||
                      (ComBits != NeuCom) || (ComBitsR != NeuComR))
                    Kuerzen = false;
                }
                First = false;
              }
            }
          }
        }
      }
    }
  }
  while (!Kuerzen && ((int)FgKomb.MaxBit() <= FaktorIndex.GetMaxEintrag()));
  ElimStr.SetStr(_TEXT(""));
  Kuerzen &= CheckVorz(FgKomb,FaktorGr,FaktorIndex,ElimStr);
  if (Kuerzen)
  { // wenn Vorzeichen erfolgreich geprueft, FG-Kombination kuerzen
    LAbsStr.SetStr(_TEXT(""));
    RAbsStr.SetStr(_TEXT(""));
    int f = 0;
    while (!FgKomb.BitSet(f))
      f++; // erste FG der Kombination finden
    FaktorGr.SetAkEint(f+1);
    BitArrObj &FG1 = FaktorGr.GetAkEint();
    for (f = 1;f <= Matrix.AnzRSum+Matrix.AnzLSum;f++)
    { // alle Summanten
      if (FG1.BitSet(f))
      { // nur Summanten, in denen FG1 vorkommt, uebernehmen, Rest kuerzen
        MGPStr tmp;
        tmp.AddChar(Matrix.Vorz[f-1]);
        for (int i = 0;i < FaktorIndex.GetMaxEintrag();i++)
        { // alle FGs
          FaktorGr.SetAkEint(i+1);
          if (!FgKomb.BitSet(i))
          { // nur FGs, die nicht in der Kombination sind
            if (FaktorGr->BitSet(f))
            { // wenn diese FG in diesem Summanten, eintragen
              FaktorIndex.SetAkEint(i+1);
              for (unsigned int t = 0;t <= FaktorIndex->MaxInt();t++)
              { // alle Faktoren dieser FG
                 FakStrings.SetAkEint(FaktorIndex.GetAkEint()[t]);
                 for (int l = 0;l < FaksProLine[FaktorIndex.GetAkEint()[t]];l++)
                   // alle Anzahlen des Faktors eintragen
                   tmp.AddStr(FakStrings.GetAkStr());
              }
            }
          }
        }
        if (tmp.Length() == 1)
        { // wenn keine FG vorhanden, 1 eintragen
          tmp.AddChar('*');
          MarkerObj::AddMarker(tmp,MarkerObj::ZahlMarker,1);
        }
        MGPStr tmp2;
        SolverTools::FormatFakStr(tmp2,tmp.c_str());
        if (f <= Matrix.AnzLSum)
          LAbsStr.AddStr(tmp2);
        else
          RAbsStr.AddStr(tmp2);
      }
    }
  }
  return Kuerzen;
}

// wenn kuerzbare Kombination gefunden, Vorzeichen ermitteln und ueberpruefen
bool FaktorenVar::CheckVorz(BitArrObj &FgKomb,BitArrKettObj &FaktorGr,
                            ShortArrKettObj &FaktorIndex,MGPStr &ElimStr)
{
  BitArrObj Minus; // BitArray fuer negative Vorzeichen
  bool VorzOK = true,First = true,Invertiert = false;
  BitArrObj Summanten;
  int AkSum = 1;
  do
  {
    int f = -1;
    do
    { // FG aus der Kombination suchen, die in diesem Summanten vorkommt
      f++;
      FaktorGr.SetAkEint(f+1);
    }
    while ((f < FaktorGr.GetMaxEintrag()) &&
           (!FgKomb.BitSet(f) || !FaktorGr->BitSet(AkSum)));
    BitArrObj &AkKombFG = FaktorGr.GetAkEint();
    int t = -1;
    // FG nicht in der Kombination suchen, die in diesem Summanten vorkommt
    // und 1 gem. Summanten mit vorh. FG hat
    do
    {
      t++;
      FaktorGr.SetAkEint(t+1);
    }
    while ((t < FaktorGr.GetMaxEintrag()) && (FgKomb.BitSet(t) ||
           !FaktorGr->BitSet(AkSum) ||
           (AkKombFG.CommonBits(FaktorGr.GetAkEint()) != 1)));
    BitArrObj &AkNKombFG = FaktorGr.GetAkEint();
    bool OtherFG = (t < FaktorGr.GetMaxEintrag());
    if (First) // bei 1. FG Vorzeichen eintragen
    {
      if (Matrix.Vorz[AkSum-1] == '-')
        Minus.SetBit(f);
      ElimStr.AddChar(Matrix.Vorz[AkSum-1]);
      FaktorIndex.SetAkEint(f+1);
      for (unsigned int t = 0;t <= FaktorIndex->MaxInt();t++)
      { // alle Faktoren dieser FG
         FakStrings.SetAkEint(FaktorIndex.GetAkEint()[t]);
         for (int l = 0;l < FaksProLine[FaktorIndex.GetAkEint()[t]];l++)
           // alle Anzahlen des Faktors eintragen
           ElimStr.AddStr(FakStrings.GetAkStr());
      }
    }
    else // bei weiteren FGs Unterschied zu 1. FG ermitteln
      if (Minus.BitSet(f) && (Matrix.Vorz[AkSum-1] == '-') ||
          !Minus.BitSet(f) && (Matrix.Vorz[AkSum-1] != '-'))
        Invertiert = false;
      else
        Invertiert = true;
    // restliche FGs der Kombination durchgehen und
    // Vorzeichen ermitteln/vergleichen
    for (unsigned int i = 0;i <= FgKomb.MaxBit();i++)
    {
      if (((int)i != f) && FgKomb.BitSet(i))
      { // weitere FG der Kombination
        FaktorGr.SetAkEint(i+1);
        int AndSum;
        if (OtherFG)
          AndSum = AkNKombFG.FirstCommonBit(FaktorGr.GetAkEint());
        else
        {
          // wenn keine andere FG, auf der gleichen Seite wie AkSum
          // den 1 gem. Summanten nehmen
          if (AkSum > Matrix.AnzLSum)
            AndSum = FaktorGr->FirstCommonBit(FaktorGr.GetAkEint(),
                                                         Matrix.AnzLSum+1);
          else
            AndSum = FaktorGr->FirstCommonBit(FaktorGr.GetAkEint());
        }
        if (First) // bei 1. FG Vorzeichen eintragen
        {
          if (Matrix.Vorz[AndSum-1] == '-')
            Minus.SetBit(i);
          ElimStr.AddChar(Matrix.Vorz[AndSum-1]);
          FaktorIndex.SetAkEint(i+1);
          for (unsigned int t = 0;t <= FaktorIndex->MaxInt();t++)
          { // alle Faktoren dieser FG
             FakStrings.SetAkEint(FaktorIndex.GetAkEint()[t]);
             for (int l = 0;l < FaksProLine[FaktorIndex.GetAkEint()[t]];l++)
               // alle Anzahlen des Faktors eintragen
               ElimStr.AddStr(FakStrings.GetAkStr());
          }
        }
        else
        // bei weiteren FGs pruefen,
        // ob alle Vorzeichen zum ermittelten Unterschied bei 1. FG passen
        if ((Minus.BitSet(i) && (Matrix.Vorz[AndSum-1] == '-') ||
            !Minus.BitSet(i) && (Matrix.Vorz[AndSum-1] != '-')) == Invertiert)
          VorzOK = false;
        if (Summanten.BitSet(AndSum))
          VorzOK = false;
        else
          Summanten.SetBit(AndSum);
      }
    }
    if (Summanten.BitSet(AkSum))
      VorzOK = false;
    else
      Summanten.SetBit(AkSum);
    First = false;
    AkSum++;
    while (Summanten.BitSet(AkSum))
      AkSum++;
  }
  while (VorzOK && (AkSum < Matrix.AnzRSum+Matrix.AnzLSum));
  return VorzOK;
}

// ----------------------------------------------------------------------------
// FaktorVar
// ----------------------------------------------------------------------------

void FaktorVar::Init(ShortArrObj &FaktorV,MGPStr &FakStr,
                     BitArrKettObj &FakProSums,MGStrObj &FakStrings,
                     ShortArrObj &FaksProLine,int ASum)
{ // initialisiert diese FaktorVariation
  FaktorVert = FaktorV;
  AnzSum = ASum;
  int i;
  AnzLines = 0;
  for (i = 1;i <= AnzSum;i++)
  { // Faktor-Varianten fuer jeden Summant erzeugen und AnzLines ermitteln
    FaktorVars.NeuAkEint();
    if (AnzLines < FaktorVert[i])
      AnzLines = FaktorVert[i];
  }
  for (i = 0; i < AnzLines;i++)
  { // Zeilen fuer die Varianten einfuegen
    FakProSums.NeuAkEint();
    FakStrings.NeuAkEint();
    FakStrings.GetAkStr().SetStr(FakStr);
    ZeilenIndex.SetValue(i,short(FakProSums.AkEintNummer()));
    FaksProLine.SetValue(FakProSums.AkEintNummer(),1); // 1 Faktor pro Zeile
    for (int f = 1;f <= AnzSum;f++)
    { // FakProSums und FaktorVars initialisieren
      if (i < FaktorVert[f])
      { // wenn dieser Faktor oft genug vorhanden, dann die Bits setzen
        // setzt Bit in dieser Zeile fuer Summant f
        FakProSums->SetBit(f);
        FaktorVars.SetAkEint(f);
        // setzt Bit in der Variation fuer diesen Faktor fuer die i-te Zeile
        FaktorVars->SetBit(i);
      }
    }
  }
}

// naechste Variante fuer diesen Faktor bestimmen
bool FaktorVar::NextVar(BitArrKettObj &FakProSums,ShortArrObj &FaksProLine)
{
  int AkSum = 1;
  bool NeuVar = false;
  while (!NeuVar && (AkSum <= AnzSum))
  {
    FaktorVars.SetAkEint(AkSum);
    int AnzFak;
    do
    { // naechste Kombination suchen
      FaktorVars->Inc();
      AnzFak = 0;
      for (unsigned int t = 0;t <= FaktorVars->MaxBit();t++)
        if (FaktorVars->BitSet(t))
          AnzFak += FaksProLine[ZeilenIndex[t]];
    }
    while (((int)FaktorVars->MaxBit() < AnzLines) &&
           (AnzFak != FaktorVert[AkSum]));
    if ((int)FaktorVars->MaxBit() < AnzLines)
    { // wenn neue Kombination gefunden, eintragen
      for (int f = 0; f < AnzLines;f++)
      { // alle Zeilen durchgehen
        // in die aktuelle Zeile verzweigen
        FakProSums.SetAkEint(ZeilenIndex[f]);
        if (FaktorVars->BitSet(f))
          FakProSums->SetBit(AkSum);
        else
          FakProSums->ClearBit(AkSum);
      }
      NeuVar = true;
    }
    else
    { // keine weitere Kombination fuer diesen Summanten gefunden
      // wenn letzter Summant, 1. Kombination wiederherstellen,
      // naechste Faktorenanzahl-Variante suchen
      if (AkSum == AnzSum)
      {
        int t = 0;
        // Neue Anzahl ist die Anzahl der naechsten Zeile+1,
        // solange Max. noch nicht erreicht
        while ((t < AnzLines) && (FaksProLine[ZeilenIndex[t]] == (AnzLines-t)))
        {
          FaksProLine.SetValue(ZeilenIndex[t],
                               short(FaksProLine[ZeilenIndex[t+1]]+1));
          t++;
        }
        if ((t < AnzLines) && (FaksProLine[ZeilenIndex[t]] < AnzLines-t))
        { // weitere Varianten vorhanden
          // Faktoren-Anzahl fuer diese Zeile hochzaehlen
          FaksProLine.IncValue(ZeilenIndex[t]);
          AkSum = 1; // wieder beim 1. Summanten anfangen
          NeuVar = true; // Erste Kombination ist immer gueltig
//          KombOut(FaksProLine);
        }
        else
        { // keine weiteren Varianten vorhanden
          for (t = 0;t < AnzLines;t++) // FaksProLine neu initialisieren
            FaksProLine.SetValue(ZeilenIndex[t],1); // 1 Faktor pro Zeile
          AkSum++; // Ende-Bedingung
        }
        for (t = 1;t <= AnzSum;t++) // alle Faktoren neu initialisieren
          InitFaktor(t,FakProSums,FaksProLine);
      }
      else
      {
        InitFaktor(AkSum,FakProSums,FaksProLine);
        AkSum++; // naechsten Summanten bearbeiten
      }
    }
  }
  return NeuVar;
}

void FaktorVar::KombOut(ShortArrObj &FaksProLine)
{
  for (int i = 0;i < AnzLines;i++)
    printf("%hi ",FaksProLine[ZeilenIndex[i]]);
  printf("\r\n");
}

void FaktorVar::InitFaktor(int Summant,BitArrKettObj &FakProSums,
                           ShortArrObj &FaksProLine)
{ // initialisiert FaktorVars fuer den Summant
  FaktorVars.SetAkEint(Summant);
  FaktorVars->Clear(); // Kombination loeschen
  int AnzFak = 0;
  for (int f = 0;f < AnzLines;f++)
  {
    FakProSums.SetAkEint(ZeilenIndex[f]); // in die aktuelle Zeile verzweigen
    if (FaktorVert[Summant]-AnzFak >= FaksProLine[ZeilenIndex[f]])
    { // wenn dieser Faktor oft genug vorhanden, dann die Bits setzen
      // setzt Bit in dieser Zeile fuer Summant i
      FakProSums->SetBit(Summant);
      // setzt Bit in der Variation fuer diesen Faktor fuer die i-te Zeile
      FaktorVars->SetBit(f);
      AnzFak += FaksProLine[ZeilenIndex[f]];
    }
    else // loescht Bit in dieser Zeile fuer Summant i
      FakProSums->ClearBit(Summant);
  }
}

