// Version 6.05 20.07.2008 Markus Greither
// Copyright (c) 1991-2008 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file efmcxio.cpp
    \brief Source code for the old format data storage classes
*/

#ifdef __BORLANDC__

#include "efprec.h"

#pragma hdrstop
#else

#ifdef __DMC__
#include <typeinfo.h>
#endif
#include <cstring>
#include <set>
#include <functional>
#include <cstdlib>
#include <cstdio>
#include <iostream>
#include <ctime>
#include <complex>
#include <limits>
#include <cfloat>
#include <string>

using namespace std;

#include "efdefs.h"
#include "mg.h"
#include "cpxpsr.h"
#include "diagtype.h"
#include "efgemein.h"
#include "efmcxio.h"
#include "efresid.h"

#endif

#if defined(__BORLANDC__) && defined(__WIN32__)
#include <excpt.h>
#endif

#define _NOCONVERT(x) x

// ----------------------------------------------------------------------------
// EFDataIOObj
// ----------------------------------------------------------------------------

double EFDataIOObj::Eval(const _TCHAR *Wert,bool Kompat)
{
  ZahlFormatObj ZFO;

  ZFO.SetKompatMode(Kompat); // Kompatibles Datenformat mit alten Versionen
  int Fehler = ParserObj::NoError;
  unsigned int P = 0;
  double Hi = ZFO.Zahl(Wert,P,Fehler);
  if (Fehler != ZahlFormatObj::Overflow)
    return Hi;
  else
    return 0;
}

void EFDataIOObj::IngFormOut(ostream &Datei,const char *Label,double Wert)
{
  ZahlFormatObj ZFO;
  MGPStr tmp;
  ZFO.Ausgabe(tmp,Wert,ZahlFormatObj::IngForm);
  Datei << Label << "  :" << tmp << "\r\n";
}

void EFDataIOObj::IngFormIndOut(ostream &Datei,const char *Label,int index,
                                double Wert)
{
  ZahlFormatObj ZFO;
  MGPStr tmp;
  ZFO.Ausgabe(tmp,Wert,ZahlFormatObj::IngForm);
  Datei << Label << index << "  :" << tmp << "\r\n";
}

#ifdef __DMC__
void EFDataIOObj::FontOut(ostream &Datei,
                          const char *Label,
                          FontType Wert)
#else
//! Output FontType data into
template <> void EFDataIOObj::ValueOut<FontType>(ostream &Datei,
                                                 const char *Label,
                                                 FontType Wert)
#endif
{
  Datei << Label << "  :" << Wert.Font << "\r\n";
  Datei << Label << "size  :" << Wert.Size << "\r\n";
  Datei << Label << "style  :" << Wert.Style << "\r\n";
  Datei << Label << "color  :" << Wert.Color << "\r\n";
}

template <class T> void EFDataIOObj::ValueOut(ostream &Datei,const char *Label,
                                              T Wert)
{
  Datei << Label << "  :" << Wert << "\r\n";
}

#ifdef __DMC__
void EFDataIOObj::FontIndOut(ostream &Datei,
                             const char *Label,
                             int index,FontType Wert)
#else
//! Output font data with index
template <> void EFDataIOObj::ValueIndOut<FontType>(ostream &Datei,
                                                    const char *Label,
                                                    int index,FontType Wert)
#endif
{
  string tmp;
  tmp = Label;
  tmp.resize(strlen(Label)-1); // abschliessendes # entfernen
  Datei << Label << index << "  :" << Wert.Font << "\r\n";
  Datei << tmp.c_str() << "size#" << index << "  :" << Wert.Size << "\r\n";
  Datei << tmp.c_str() << "style#" << index << "  :" << Wert.Style << "\r\n";
  Datei << tmp.c_str() << "color#" << index << "  :" << Wert.Color << "\r\n";
}

template <class T> void EFDataIOObj::ValueIndOut(ostream &Datei,
                                                 const char *Label,
                                                 int index,T Wert)
{
  Datei << Label << index << "  :" << Wert << "\r\n";
}

void EFDataIOObj::KlammernUmParamStr(MGPStr &str)
{
  ParamsObj Par;
  ParserObj Psr;
  if (!Psr.GetParamsNamen(str.c_str(),&Par))
    return;
  str.SetStr(_TEXT("("));
  for (size_t i = 0;i < Par.Name.size();i++)
  {
    str.AddStr(Par.Name[i]);
    if (i < Par.Name.size()-1)
      str.AddChar(',');
  }
  str.AddStr(_TEXT(")"));
}

void EFDataIOObj::LoadFunktionen(FunctionArrObj *Funktion,int i,ParserObj &Psr,
                                 istream &Datei,bool Equations,
                                 MGPStr *PostTerm)
{
  //! Post processor function name
  const _TCHAR *PostProcOldName = _TEXT("PostProc");
  //! Post processor function label
  const _TCHAR *PostProcName = _TEXT("[PostProc]");
  int AkPos;
  unsigned int TmpPos;
  MGPStr Zeile;

  Funktion->ResetKette(); // bisherige Funktionen loeschen
  for (int f = 1;f <= i;f++)
  { // alle Funktionen einlesen
    Zeile.SetStr(_TEXT(""));
    string tmp1;
    getline(Datei,tmp1);
    Zeile.AddStr(ConvertStr(tmp1.c_str()));
    if (Zeile[0] && (Zeile[Zeile.Length()-1] == '\r'))
      Zeile.ShortStr(Zeile.Length()-1);
    Funktion->NeuAkEint();
    if ((Zeile[0] <= ' ') && (Zeile[0] != 0))
    { // Kommentarzeile
      (*Funktion)->Namen.SetStr(Zeile);
      (*Funktion)->Term.SetStr(_TEXT(""));
    }
    else
    { // Zeile in Term und Bezeichnung aufteilen
      AkPos = Psr.IstGleichPos(Zeile.c_str());
      if (AkPos >= 0)
      { // '=' gefunden
        TmpPos = AkPos+1;
        while ((TmpPos < Zeile.Length()) && (Zeile[TmpPos] == ' '))
          TmpPos++;
        (*Funktion)->Term.SetStr(&Zeile[TmpPos]);
        TmpPos = AkPos;
        while ((TmpPos > 0) && (Zeile[TmpPos-1] == ' '))
          TmpPos--;
        Zeile.ShortStr(TmpPos);
        if (!Equations)
        {
          int APar = Psr.GetParamsAnfPos(Zeile.c_str());
          if (APar > 0)
          {
            MGPStr tmp(&Zeile[APar]);
            KlammernUmParamStr(tmp);
            (*Funktion)->ParamStr.SetStr(tmp);
            Zeile.ShortStr(APar);
          }
          if (Zeile[0] && (Zeile[0] != '['))
          {
            Zeile.InsChar(0,'[');
            Zeile.AddChar(']');
          }
        }
        (*Funktion)->Namen.SetStr(Zeile);
      }
      else
      { // keine Bezeichnung
        (*Funktion)->Term.SetStr(Zeile);
        (*Funktion)->Namen.SetStr(_TEXT(""));
      }
      // PostProc-Funktion nach PostTerm kopieren
      if (((_tcscmp((*Funktion)->Namen.c_str(),PostProcName) == 0) ||
           (_tcscmp((*Funktion)->Namen.c_str(),PostProcOldName) == 0)) &&
           PostTerm)
        PostTerm->SetStr((*Funktion)->Term);
    }
  }
}

// ----------------------------------------------------------------------------
// EFDaten
// ----------------------------------------------------------------------------

const char *EFDaten::GemNamen[EFDaten::NumGemNamen] =
{
  _NOCONVERT("Ak._Funktion_NR"), // aktuelle Funktion - obsolet (<4.21)
  _NOCONVERT("Funktion_#"),      // Funktion Nummer n - obsolet (<4.0)
  _NOCONVERT("Fu.-Name_#"),      // Funktions-Bezeichnung n - obsolet (<4.0)
  _NOCONVERT("Post_Term"),       // Postprozessor-Funktion - obsolet (<3.x)
  _NOCONVERT("Fkt_Datei_Name"),  // Funktions-Dateiname - obsolet (<3.x)
  _NOCONVERT("SuchString"),      // obsolet (<3.x)
  _NOCONVERT("ErhaltString"),    // obsolet (<3.x)
  _NOCONVERT("AuflsMode"),      // obsolet (<3.x)
  _NOCONVERT("AuflsBereich"),   // obsolet (<3.x)
  _NOCONVERT("Anhngen"),        // obsolet (<3.x)
  _NOCONVERT("Funktions_Anz"),   // Funktions-Anzahl - Allgemein
  _NOCONVERT("Solver_Ergebnis"), // Solver-Parameter - Allgemein
  _NOCONVERT("Solver_Parameter"),
  _NOCONVERT("Solver_Mode"),
  _NOCONVERT("Solver_Bereich"),
  _NOCONVERT("Solver_Optionen"),
  _NOCONVERT("Bildschirm_Mode"), // Bildschirm-Modus - Allgemein
  _NOCONVERT("Editor_Font"),     // Editor-Schrift - Allgemein
  _NOCONVERT("Editor_Fontsize"),
  _NOCONVERT("Editor_Fontstyle"),
  _NOCONVERT("Editor_Fontcolor"),
  _NOCONVERT("Bildschirm_Teilung"), // Bildschirm-Teilung - Allgemein
  _NOCONVERT("Einzelwert_History#"), //ListBox-Histories - Allgemein
  _NOCONVERT("Ergebnis_History#"),
  _NOCONVERT("Parameter_History#"),
  _NOCONVERT("Bruchdarstellung"), // Bruchdarstellung - Allgemein
  _NOCONVERT("Klammern_anzeigen"), // Klammern anzeigen - Allgemein
  _NOCONVERT("Editor_Scrollpos"), // Scrollposition (Zeile) - Allgemein
  _NOCONVERT("Editor_SelAnf"),   // Editor-Selektion - Allgemein
  _NOCONVERT("Editor_SelEnde"),
  _NOCONVERT("Naeherung_X#"),    // Naeherungsfkt-Parameter - Allgemein
  _NOCONVERT("Naeherung_Y#"),
  _NOCONVERT("Naeherung_Bezeichnung"),
  _NOCONVERT("Naeherung_Werte"),
  _NOCONVERT("Naeherung_Optionen"),
  _NOCONVERT("Naeherung_Modus"),
  _NOCONVERT("[Diagramm #]"), // Spezifiziert die Diagramm-Nummer fuer folgenden Datenblock
  _NOCONVERT("Zoom"), // Editor-Zoomfaktor
  _NOCONVERT("SeiteQuer"), // Editor-Seitenausrichtung
  _NOCONVERT("Diagramme_verstecken"), // Editor-Diagramme verstecken
  _NOCONVERT("Editor_ScrollposX"), // Scrollposition X - Allgemein
  _NOCONVERT("Labels_markieren"), // Editor-Labels markieren
  _NOCONVERT("[Picture #]"), // Bild-Nummer fuer folgende Daten
  _NOCONVERT("Solver_Zusammenhalten"), // Solver-Parameter ZUSAMMENHALTEN
  _NOCONVERT("Zusammenhalten_History#"), // History fuer ZUSAMMENHALTEN
  _NOCONVERT("Editor_ScrollposY"), // Scrollposition Y - Allgemein
  _NOCONVERT("[SolverVorlage #]"), // Spezifiziert Solver-Vorlage-Nummer f. folgenden Block
  _NOCONVERT("SolverParams_Anz"), // Solver-Parameter (XXX = 0)
  _NOCONVERT("SolverVorlage_Name"), // Name dieser Vorlage
  _NOCONVERT("[Equations #]") // Nummer fuer Gleichungs-Fenster
};

EFDaten::EFDaten(const NumListFormat &LDV)
{
  Funktion = new FunctionArrObj;
  DiagDataArr = new DiagDataArrObj(LDV);
  PictureArr = new PictureArrObj;
  EquationsArr = new EquationsArrObj;
  Anfangswerte();
}

EFDaten::~EFDaten()
{
  delete DiagDataArr;
  delete PictureArr;
  delete EquationsArr;
  delete Funktion;
}

void EFDaten::Anfangswerte()
{
  DiagDataArr->ResetKette();
  Editor.Font.SetStr(_TEXT("Times New Roman"));
  Editor.Size = 16*3/4; // Anpassung an Punkt-Groesse
  Editor.Style = 0;
  Editor.Color = 0;
  BildschOpt.SetMarkLabels(true);
  BildschOpt.SetBildschBruch(true);
  Funktion->ResetKette();
  AuflVorlage.ResetKette();
  AuflVorlage.NeuAkEint();
  AuflVorlage.Begin();
  AuflVorlage->ErgStr.SetStr(_TEXT(""));
  AuflVorlage->BeibehaltenStr.SetStr(_TEXT(""));
  AuflVorlage->ZusammenStr.SetStr(_TEXT(""));
  AuflVorlage->Mode = 0;
  AuflVorlage->Bereich = 0;
  // Aufloes-Ergebnis anhaengen
  AuflVorlage->Anh = UAnh+UComment+URules;
  EinzelwertHistory.SetCount(0);
  ErgebnisHistory.SetCount(0);
  BeibehaltenHistory.SetCount(0);
  ZusammenHistory.SetCount(0);
  ScrollLine = 0;
  ScrollPosX = 0;
  ScrollPosY = 0;
  // Anf groesser als End -> Indikator fuer Selektion nicht gespeichert
  EditSelAnf = 1;
  EditSelEnd = 0;
  NaehData.Clear();
  Zoom = 100;
  SeiteQuer = 0;
  m_OpenPanels.ResetKette(false);
}

bool EFDaten::EinstDateiLaden(istream &Datei)
{
  int f;
  unsigned int i;
  string Zeile,Name;
  MGPStr PostTerm;
  const char *SWert;
  int Nr;
  int AltFunk;
  bool Kompat = true; // Zahlen-Kompatibilitaet mit Programmversionen vor 4.11
  bool MemFehler;
  int Nummer;
  ParserObj Psr;

  AuflVorlage.SetAkEint(0);
  if (Datei)
  {
    Anfangswerte();
    getline(Datei,Zeile);
    if (Zeile.length() > 0)
    { // Datei kann gelesen werden
      const char *tmp = strstr(Zeile.c_str()," V ");
      int MajorVersion = atoi(&tmp[3]);
      if (MajorVersion < 5)
        DiagDataArr->NeuAkEint(); // Version 4.XX hat immer mind. 1 Diagramm
      do
      {
        getline(Datei,Zeile);
        if (Zeile[0] && (Zeile[Zeile.length()-1] == '\r'))
          Zeile.resize(Zeile.length()-1);
        if (Zeile.length() > 0)
        { // keine Leerzeile
          i = 0; // Ende des Bezeichners ermitteln
          while ((i < Zeile.length()) && (Zeile[i] != ' ') &&
                 (!IST_ZAHL(Zeile[i])) && (Zeile[i] != ':'))
            i++;
          f = 0; // Nummer des Bezeichners im GemNamen ermitteln
          while ((f < NumGemNamen) && (strncmp(GemNamen[f],Zeile.c_str(),i) != 0))
            f++;
          Name = "";
          Name.append(Zeile.c_str(),i);
          i = 0; // Anfang des Indexes des Bezeichners ermitteln
          while ((i < Zeile.length()) &&
                 (!IST_ZAHL(Zeile[i])) && (Zeile[i] != ':'))
            i++;
          Nummer = 0; // Index lesen
          if (Zeile[i] == ':')
            Nummer = 255;
          else
          while ((i < Zeile.length()) && IST_ZAHL(Zeile[i]))
          {
            Nummer = Nummer*10+Zeile[i]-48;
            i++;
          }
          i = 0; // Anfang des Wertes fr den Bezeichner ermitteln
          while ((i < Zeile.length()) && (Zeile[i] != ':'))
            i++;
          SWert = Zeile.c_str();
          if (i < Zeile.length())
            SWert = &SWert[i+1]; // SWert auf 1. Zeichen des Wertes setzen
          switch (f)
          {
            case 0 : // Ak_Funktion_NR
              Nr = atoi(SWert)+1;
              if (Nr>Funktion->GetMaxEintrag())
                while (Nr>Funktion->GetMaxEintrag())
                  Funktion->NeuAkEint();
              else
                Funktion->SetAkEint(Nr);
              break;
            case 1 :  // Funktion_#
              Nummer++;
              AltFunk = Funktion->AkEintNummer();
              MemFehler = false;
              if (Nummer > Funktion->GetMaxEintrag())
              {
                while ((Nummer > Funktion->GetMaxEintrag()) && (!MemFehler))
                  MemFehler = Funktion->NeuAkEint();
              }
              else
                Funktion->SetAkEint(Nummer);
              if (!MemFehler)
                AssignStr((*Funktion)->Term,SWert);
              Funktion->SetAkEint(AltFunk);
              break;
            case 2 : // Fu.-Name_#
              Nummer++;
              AltFunk = Funktion->AkEintNummer();
              MemFehler = false;
              if (Nummer>Funktion->GetMaxEintrag())
              {
                while ((Nummer>Funktion->GetMaxEintrag()) && (!MemFehler))
                  MemFehler = Funktion->NeuAkEint();
              }
              else
                Funktion->SetAkEint(Nummer);
              if (!MemFehler)
                AssignStr((*Funktion)->Namen,SWert);
              Funktion->SetAkEint(AltFunk);
              break;
            case 3 : // Post_Term
              if (strlen(SWert) == 0)
                break;
              AssignStr(PostTerm,SWert);
              break;
            case 4 : // Fkt_Datei_Name
              LoadTextDatei(ConvertStr(SWert).c_str());
              break;
            case 5 :
              AssignStr(AuflVorlage->ErgStr,SWert);
              break; // SuchString
            case 6 :
              AssignStr(AuflVorlage->BeibehaltenStr,SWert);
              break; // ErhaltString
            case 7 :
              AuflVorlage->Mode = (unsigned short)strtoul(SWert,0,10);
              break; // AufloesMode
            case 8 :
              AuflVorlage->Bereich = (unsigned short)strtoul(SWert,0,10);
              break; // AufloesBereich
            case 9 :
              AuflVorlage->Anh = (unsigned short)strtoul(SWert,0,10);
              break; // Anhaengen
            case 10 : // Funktions_Anz
              // Dateien mit diesem Funktionsformat brauchen
              // keinen Zahlen-Kompatibilitaetsmodus mehr
              Kompat = false;
              i = atoi(SWert); // Anzahl der Funktionen
              EFDataIOObj::LoadFunktionen(Funktion,i,Psr,Datei,false,&PostTerm);
              break;
            case 11 :
              AssignStr(AuflVorlage->ErgStr,SWert);
              break; // Solver_Ergebnis
            case 12 :
              AssignStr(AuflVorlage->BeibehaltenStr,SWert);
              break; // Solver_Parameter
            case 13 :
              AuflVorlage->Mode = (unsigned short)strtoul(SWert,0,10);
              break; // Solver_Mode
            case 14 :
              AuflVorlage->Bereich = (unsigned short)strtoul(SWert,0,10);
              break; // Solver_Bereich
            case 15 :
              AuflVorlage->Anh = (unsigned short)strtoul(SWert,0,10);
              break; // Solver_Optionen
            case 16 :
              break; // Bildschirm_Mode (obsolet)
            case 17 :
              AssignStr(Editor.Font,SWert);
              break; // Editor_Font
            case 18 :
              Editor.Size = atoi(SWert)*3/(MajorVersion < 5 ? 4 : 3);
              break; // Editor_Fontsize, angepasst an Punktgroesse
            case 19 :
              Editor.Style = atoi(SWert);
              break; // Editor_Fontstyle
            case 20 :
              Editor.Color = strtoul(SWert,0,10);
              break; // Editor_Fontcolor
            case 22 :
              EinzelwertHistory.SetString(Nummer,ConvertStr(SWert).c_str());
              break; // Einzelwert_History
            case 23 :
              ErgebnisHistory.SetString(Nummer,ConvertStr(SWert).c_str());
              break; // Ergebnis_History
            case 24 :
              BeibehaltenHistory.SetString(Nummer,ConvertStr(SWert).c_str());
              break; // Parameter_History
            case 25 :
              BildschOpt.SetBildschBruch(atoi(SWert) != 0);
              break; // Bruchdarstellung
            case 26 :
              BildschOpt.SetBildschKlammer(atoi(SWert) != 0);
              break; // Klammern_anzeigen
            case 27 :
              ScrollLine = strtoul(SWert,0,10);
              break; // Editor_Scrollpos
            case 28 :
              EditSelAnf = atoi(SWert);
              break; // Editor_SelAnf
            case 29 :
              EditSelEnd = atoi(SWert);
              break; // Editor_SelEnde
            case 30 :
              AssignStr(NaehData.XStr[Nummer],SWert);
              break; // Naeherung_X#
            case 31 :
              AssignStr(NaehData.YStr[Nummer],SWert);
              break; // Naeherung_Y#
            case 32 :
              AssignStr(NaehData.ErgStr,SWert);
              break; // Naeherung_Bezeichnung
            case 33 :
              AssignStr(NaehData.ParamStr,SWert);
              break; // Naeherung_Werte
            case 34 :
              NaehData.m_CurGraph = atoi(SWert) > 0;
              break; // Naeherung_Optionen
            case 35 :
              NaehData.Mode = (unsigned short)atoi(SWert);
              break; // Naeherung_Modus
            case 36 : // [Diagramm #]
            {
              int AkD = atoi(SWert);
              DiagDataArr->CreateAkEint(AkD);
              break;
            }
            case 37 :
              Zoom = atoi(SWert);
              break; // Zoom
            case 38 :
              SeiteQuer = atoi(SWert);
              break; // Seitenausrichtung
            case 39 :
              BildschOpt.SetHideDiags(atoi(SWert) != 0);
              break; // Diagramme_verstecken
            case 40 :
              ScrollPosX = strtoul(SWert,0,10);
              break; // Editor_ScrollposX
            case 41 :
              BildschOpt.SetMarkLabels(atoi(SWert) != 0);
              break; // Labels_markieren
            case 42 : // [Picture #]
            {
              int AkP = atoi(SWert);
              PictureArr->CreateAkEint(AkP);
              break;
            }
            case 43 :
              AssignStr(AuflVorlage->ZusammenStr,SWert);
              break; // ZusammenString
            case 44 :
              ZusammenHistory.SetString(Nummer,ConvertStr(SWert).c_str());
              break; // Zusammen_History
            case 45 :
              ScrollPosY = strtoul(SWert,0,10);
              break; // Editor_ScrollposY
            case 46 : // [SolverVorlage #]
              while (AuflVorlage.GetMaxEintrag() < atoi(SWert))
                AuflVorlage.NeuAkEint();
              AuflVorlage.SetAkEint(atoi(SWert));
              break;
            case 47 : // SolverParams_Anz
            {
              AuflVorlage->ParamsList.ResetKette();
              int i = atoi(SWert); // Anzahl der Parameter
              for (f = 0;f < i;f++)
              { // alle Parameter einlesen
                Zeile = "";
                getline(Datei,Zeile);
                if (Zeile[0] && (Zeile[Zeile.length()-1] == '\r'))
                  Zeile.resize(Zeile.length()-1);
                AuflVorlage->ParamsList.NeuAkEint();
                AssignStr(AuflVorlage->ParamsList.GetAkEint(),Zeile.c_str());
              }
              break;
            }
            case 48 : // SolverVorlage_Name
              AssignStr(AuflVorlage->Name,SWert);
              break;
            case 49 : // [Equations #]
            {
              int AkP = atoi(SWert)-1;
              EquationsArr->CreateAkEint(AkP);
              break;
            }
            default :
              if (((PictureArr->GetMaxEintrag() < 0) ||
                   !PictureArr->AkPicture()->SetData(Name.c_str(),SWert,Nummer,Datei)) &&
                  ((EquationsArr->GetMaxEintrag() < 0) ||
                   !EquationsArr->AkEquations()->SetData(Name.c_str(),SWert,
                                                         Nummer,Datei)))
                if (DiagDataArr->GetMaxEintrag() >= 0)
                  DiagDataArr->AkDiagData()->SetData(Name.c_str(),SWert,Nummer,Datei,
                                                     Kompat,MajorVersion);
          }
        }
      }
      while (!Datei.eof());
      // for compatibility with older files, copy PostProc into diagrams
      if (PostTerm.Length() > 0)
      {
        DiagDataArr->Begin();
        if (DiagDataArr->GetMaxEintrag() >= 0)
          do
            if (DiagDataArr->AkDiagData()->PostProcTerm.Length() == 0)
              DiagDataArr->AkDiagData()->PostProcTerm.SetStr(PostTerm.c_str());
          while (DiagDataArr->NextEint());
      }
      return true;
    }
  }
  return false;
}

// Declared in efgemein.cpp
#if defined(UNICODE) && defined(__MINGW32__)
extern FILE *Mingw_tfopen(const _TCHAR *Name,const _TCHAR *Mode);
#endif

bool EFDaten::LoadTextDatei(const _TCHAR *TextDateiName)
{
  int i;
  FILE *Datei;
  char Zeile[2000],Zeile2[2000];
  int AltFunk;
  int IstPos;
  unsigned int AnfPos,EndPos;
  ParserObj Psr;

  AltFunk = Funktion->AkEintNummer();
  Funktion->ResetKette();
#if defined(UNICODE) && defined(__MINGW32__)
  Datei = Mingw_tfopen(TextDateiName,_TEXT("r"));
#else
  Datei = _tfopen(TextDateiName,_TEXT("r"));
#endif
  if (Datei != 0)
  {
    fgets(Zeile,2000,Datei);
    if ((strlen(Zeile) > 0) && (Zeile[strlen(Zeile)-1] == '\n'))
      Zeile[strlen(Zeile)-1] = 0; // \n berschreiben
    if (ferror(Datei) != 0)
      return false;
    if (strncmp(PROGRAMSTRING,Zeile,strlen(PROGRAMSTRING)) == 0)
    {
      fgets(Zeile,2000,Datei);
      if (ferror(Datei) != 0)
        return false;
      fgets(Zeile,2000,Datei);
      if ((strlen(Zeile) > 0) && (Zeile[strlen(Zeile)-1] == '\n'))
        Zeile[strlen(Zeile)-1] = 0; // \n berschreiben
      if (ferror(Datei) != 0)
        return false;
    }
    i = 1;
    do
    {
      if (Funktion->GetMaxEintrag() < i)
        Funktion->NeuAkEint();
      (*Funktion)->Term.SetStr(_TEXT(""));
      (*Funktion)->Namen.SetStr(_TEXT(""));
      if ((Zeile[0] <= ' ') && (Zeile[0] != 0))
      {
        (*Funktion)->Namen.SetStr(ConvertStr(Zeile));
        (*Funktion)->Term.SetStr(ConvertStr(""));
      }
      else
      {
        IstPos = Psr.IstGleichPos(ConvertStr(Zeile).c_str());
        if (IstPos > 0)
          EndPos = IstPos-1;
        else
          EndPos = 0;
        while ((EndPos>0) && (Zeile[EndPos] == ' '))
          EndPos--;
        AnfPos = IstPos+1;
        while ((EndPos < strlen(Zeile)) && (Zeile[AnfPos] == ' '))
          AnfPos++;
        if (AnfPos > strlen(Zeile))
          AnfPos = strlen(Zeile);
        (*Funktion)->Term.SetStr(ConvertStr(&Zeile[AnfPos]));
        strncpy(Zeile2,Zeile,EndPos+1);
        Zeile2[EndPos+1] = 0;
        (*Funktion)->Namen.SetStr(ConvertStr(Zeile2));
      }
      i++;
      strcpy(Zeile,"");
      if (!feof(Datei))
        fgets(Zeile,2000,Datei);
      if ((strlen(Zeile) > 0) && (Zeile[strlen(Zeile)-1] == '\n'))
        Zeile[strlen(Zeile)-1] = 0; // \n berschreiben
      if (ferror(Datei) != 0)
        return false;
    }
    while ((strlen(Zeile) > 0) || !feof(Datei));
    Funktion->SetAkEint(AltFunk);
    fclose(Datei);
    return true;
  }
  return false;
}

bool EFDaten::WriteTextDatei(const _TCHAR *TextDateiName,bool Header)
{
  int i;
  FILE *Datei;
  int AltFunk;

  {
#if defined(UNICODE) && defined(__MINGW32__)
    Datei = Mingw_tfopen(TextDateiName,_TEXT("w"));
#else
    Datei = _tfopen(TextDateiName,_TEXT("w"));
#endif
    if (Datei != 0)
    {
      if (Header)
      {
        fprintf(Datei,"%s%s%s - Funktions-Datei\n",PROGRAMSTRING," V ",
                VERSIONSTRING);
      }
      if (ferror(Datei) != 0)
        return false;
      if (Header)
        fprintf(Datei,"\n");
      if (ferror(Datei) != 0)
        return false;
      AltFunk = Funktion->AkEintNummer();
      for (i = 1;i<=Funktion->GetMaxEintrag();i++)
      {
        Funktion->SetAkEint(i);
        if (((*Funktion)->Term.Length() > 0) ||
            ((*Funktion)->Namen.Length() > 0))
        {
          char *name = new char[(*Funktion)->Namen.Length()+1];
          char *term = new char[(*Funktion)->Term.Length()+1];
          char *param = new char[(*Funktion)->ParamStr.Length()+1];
          ConvertStr((*Funktion)->Term.c_str(),term);
          ConvertStr((*Funktion)->Namen.c_str(),name);
          ConvertStr((*Funktion)->ParamStr.c_str(),param);
          if ((*Funktion)->Namen.Length() > 0)
          {
            if (((*Funktion)->Namen[0] <= ' ') &&
                ((*Funktion)->Namen[0] != 0) ||
               ((*Funktion)->Term.Length() == 0))
              fprintf(Datei,"%s\n",name);
            else
            {
              fprintf(Datei,"%s%s = ",name,param);
              fprintf(Datei,"%s\n",term);
            }
          }
          else
            fprintf(Datei,"%s\n",term);
          delete[] param;
          delete[] name;
          delete[] term;
          if (ferror(Datei) != 0)
            return false;
        }
        else
         fprintf(Datei,"\n");
      }
      Funktion->SetAkEint(AltFunk);
      fclose(Datei);
      return true;
    }
  }
  return false;
}

// ----------------------------------------------------------------------------
// PictureObj
// ----------------------------------------------------------------------------

const char *PictureObj::PictNamen[PictureObj::NumPictNamen] =
{
  _NOCONVERT("Picture_Type"),
  _NOCONVERT("Picture_Size"),
  _NOCONVERT("Picture_Data"),
  _NOCONVERT("Picture_XPos"),  // Picture-Fenster-Koordinaten
  _NOCONVERT("Picture_YPos"),
  _NOCONVERT("Picture_Width"),
  _NOCONVERT("Picture_Height")
};

void PictureObj::Speichern(ostream &Datei)
{
  ValueOut(Datei,PictNamen[3],WinXPos);
  ValueOut(Datei,PictNamen[4],WinYPos);
  ValueOut(Datei,PictNamen[5],WinWidth);
  ValueOut(Datei,PictNamen[6],WinHeight);
  Datei << "\r\n";
  ValueOut(Datei,PictNamen[0],TypeStr);
  ValueOut(Datei,PictNamen[1],Size);
  Datei << PictNamen[2] << "  :\r\n"; // Markierung fuer Picture-Daten
  Datei.write(Data,Size);
  delete[] Data;
  Data = 0;
  Size = 0;
  Datei << "\r\n";
}

bool PictureObj::SetData(const char *Name,const char *SWert,int /*Nummer*/,
                         istream &Datei)
{
  int i = 0;

  while ((i < NumPictNamen) && (strcmp(PictNamen[i],Name) != 0))
    i++;
  if (i < NumPictNamen)
  {
    switch (i)
    {
      case 0 :
        TypeStr.SetStr(ConvertStr(SWert));
        break; // Picture_Type
      case 1 :
        Size = atoi(SWert);
        break; // Picture_Size
      case 2 :
      {
        Data = new char[Size];
        Datei.read(Data,Size); // Picture_Data
        break;
      }
      case 3 :
        WinXPos = atoi(SWert);
        break; // Picture_XPos
      case 4 :
        WinYPos = atoi(SWert);
        break; // Picture_YPos
      case 5 :
        WinWidth = atoi(SWert);
        break; // Picture_Width
      case 6 :
        WinHeight = atoi(SWert);
        break; // Picture_Height
    }
    return true;
  }
  else
    return false;
}

// ----------------------------------------------------------------------------
// DiagDataObj
// ----------------------------------------------------------------------------

const char *DiagDataObj::DiagNamen[DiagDataObj::NumDiagNamen] =
{
  _NOCONVERT("Anzahl_Punkte"),   // Anzahl der zu berechnenden Punkte - Diagramm
  _NOCONVERT("Diagramm#"),       // Darstellungs-Art - Diagramm
  _NOCONVERT("XMin_#"),          // Diagramm-Grenzen - Diagramm
  _NOCONVERT("XMax_#"),
  _NOCONVERT("YMin_#"),
  _NOCONVERT("YMax_#"),
  _NOCONVERT("hi-Anfang"),       // Hilfsvariable - Diagramm
  _NOCONVERT("hi-Ende"),         //
  _NOCONVERT("hi-Delta"),
  _NOCONVERT("hi-Mode"),
  _NOCONVERT("X-Modus"),         // X linear oder log - obsolet (<2.x)
  _NOCONVERT("hi-Bezeichner"),   // Hilfsvariable - Diagramm
  _NOCONVERT("hi-Einheit"),
  _NOCONVERT("XEin_#"),          // Diagramm-Beschriftung - Diagramm
  _NOCONVERT("YEin_#"),
  _NOCONVERT("Dia-Mode#"),       // Beschriftungs-Modus - Diagramm
  _NOCONVERT("Dia-Titel#"),      // Beschriftung - Diagramm
  _NOCONVERT("Ak_Graph"),        // Aktueller Graph - Diagramm
  _NOCONVERT("Post_Proc"),       // PostProzessor ein/aus - Diagramm
  _NOCONVERT("X-Mode_#"),        // X-Modus - Diagramm
  _NOCONVERT("Berech_XMin"),     // Berechnungs-X-Bereich - Diagramm
  _NOCONVERT("Berech_XMax"),     //
  _NOCONVERT("Berech_XMode"),    // Berechnungs-X-Modus - Diagramm
  _NOCONVERT("Berech_XLog"),     // Berechnungs-X-lin/log - Diagramm
  _NOCONVERT("Auto_Mode"),       // Automatische Berechnungs-Modus - Diagramm
  _NOCONVERT("Legend_Font"),     // Legende-Format - Diagramm
  _NOCONVERT("Legend_Fontsize"),
  _NOCONVERT("Legend_Fontstyle"),
  _NOCONVERT("Legend_Fontcolor"),
  _NOCONVERT("Legend_frame"),
  _NOCONVERT("Axis_Font"),       // Achsen-Format - Diagramm
  _NOCONVERT("Axis_Fontsize"),
  _NOCONVERT("Axis_Fontstyle"),
  _NOCONVERT("Axis_Fontcolor"),
  _NOCONVERT("Title_Font"),           // Titel-Format - Diagramm
  _NOCONVERT("Title_Fontsize"),
  _NOCONVERT("Title_Fontstyle"),
  _NOCONVERT("Title_Fontcolor"),
  _NOCONVERT("Graph-Daten"),     // Graph-Daten - Diagramm
  _NOCONVERT("hi-Parameterliste"), // hi-Parameterliste - Diagramm
  _NOCONVERT("Debug"),           // Debug-Modus - Diagramm
  _NOCONVERT("WinXPos"),  // Diagramm-Fenster-Koordinaten
  _NOCONVERT("WinYPos"),
  _NOCONVERT("WinWidth"),
  _NOCONVERT("WinHeight"),
  _NOCONVERT("LegendeX"), // Legende-Relativpositionen
  _NOCONVERT("LegendeY"),
  _NOCONVERT("AxisLabel_Font"),       // Achsen-Beschriftungs-Format - Diagramm
  _NOCONVERT("AxisLabel_Fontsize"),
  _NOCONVERT("AxisLabel_Fontstyle"),
  _NOCONVERT("AxisLabel_Fontcolor"),
  _NOCONVERT("AutoUpdate"),
  _NOCONVERT("Label_Title#"),      // Label-Werte
  _NOCONVERT("Label_Font#"),
  _NOCONVERT("Label_Fontsize#"),
  _NOCONVERT("Label_Fontstyle#"),
  _NOCONVERT("Label_Fontcolor#"),
  _NOCONVERT("Label_Flags#"),
  _NOCONVERT("Label_XPos#"),
  _NOCONVERT("Label_YPos#"),
  _NOCONVERT("Label_XOffs#"),
  _NOCONVERT("Label_YOffs#"),
  _NOCONVERT("Num_Digits_#"),
  _NOCONVERT("PostProc_Term")
};

// Kopiert Diagramm-Einstellungen, die von Instanz zu Instanz verschiebbar sind
void DiagDataObj::Copy(DiagDataObj &DDO,bool CopyPos,bool MoveData)
{
  PostProc = DDO.PostProc;
  PostProcTerm = DDO.PostProcTerm;
  if (MoveData) // can't copy Labels into clipboard
    Labels = DDO.Labels;
  Legende = DDO.Legende;
  Achse = DDO.Achse;
  Titel = DDO.Titel;
  AchsLabels = DDO.AchsLabels;
  if (MoveData) // for clipboard data is copied differently
    YDaten.MoveData(DDO.YDaten);
  DarstData[0] = DDO.DarstData[0];
  DarstData[1] = DDO.DarstData[1];
  AutoMode = DDO.AutoMode;
  if (CopyPos)
  {
    WinXPos = DDO.WinXPos;
    WinYPos = DDO.WinYPos;
    WinHeight = DDO.WinHeight;
    WinWidth = DDO.WinWidth;
  }
  LegX = DDO.LegX;
  LegY = DDO.LegY;

  // EinstData items
  XLog = DDO.XLog;
  XMode = DDO.XMode;
  CXMin = DDO.CXMin;
  CXMax = DDO.CXMax;
  Debug = DDO.Debug;
  Punkte = DDO.Punkte;
  AutoUpdate = DDO.AutoUpdate;
  Ma = DDO.Ma;
  m_Colour = DDO.m_Colour;
}

void DiagDataObj::Clear()
{ // initialisiert das DiagDataObj mit Defaultwerten
  WinXPos = 250;
  WinYPos = 20;
  WinHeight = 267;
  WinWidth = 378;

  LegX = 0;
  LegY = 0;
  Legende.Font.SetStr(_TEXT("Arial"));
  Legende.Size = 21*3/4; // Angepasst an Punktgroesse
  Legende.Frame = 1;

  Achse.Font.SetStr(_TEXT("Arial"));
  Achse.Size = 21*3/4; // Angepasst an Punktgroesse

  AchsLabels.Font.SetStr(_TEXT("Arial"));
  AchsLabels.Size = 21*3/4; // Angepasst an Punktgroesse

  Titel.Font.SetStr(_TEXT("Arial"));
  Titel.Size = 32*3/4; // Angepasst an Punktgroesse
  Titel.Style = FontType::Bold;

  AutoMode = DarstDataObj::AutoY;
  PostProc = 0;
  XLog = 0;
  XMode = 0;
  CXMin = 1;
  CXMax = 10;
  Debug = 1;
  AutoUpdate = 1;
  AkGraph = 0;
  Punkte = 200;
  m_Colour = 0xFFFFFF;
  DarstData[0].Init(DarstDataObj::realY,0,0,_TEXT(""),3);
  DarstData[1].Init(DarstDataObj::einzel,1,0,_TEXT(""),3);
  DarstData[0].InitWerber();
  DarstData[1].InitWerber();
}

void DiagDataObj::SetData(const char *Name,const char *SWert,
                          int Nummer,istream &Datei,bool Kompat,
                          int MajorVersion)
{
  int i = 0;

  while ((i < NumDiagNamen) && (strcmp(DiagNamen[i],Name) != 0))
    i++;
  switch (i)
  {
    case 0 :
      Punkte = strtoul(SWert,0,10);
      break; // Anzahl_Punkte
    case 1 :
      DarstData[Nummer-1].YAxis.Begin();
      DarstData[Nummer-1].Conversion = (char)atoi(SWert);
      DarstData[Nummer-1].YAxis->SetLog(DarstData[Nummer-1].Conversion == DarstDataObj::logY);
      break; // Diagramm#
    case 2 : // XMin_#
    case 3 : // XMax_#
    case 4 : // YMin_#
    case 5 : // YMax_#
      if (Nummer == 255)
        Nummer = 0;
      if (Nummer < 2)
      {
        DarstData[Nummer].XAxis.Begin();
        DarstData[Nummer].YAxis.Begin();
        switch (i)
        {
          case 2 :
            DarstData[Nummer].XAxis->Min = Eval(ConvertStr(SWert).c_str(),Kompat);
            break;
          case 3 :
            DarstData[Nummer].XAxis->Max = Eval(ConvertStr(SWert).c_str(),Kompat);
            break;
          case 4 :
            DarstData[Nummer].YAxis->Min = Eval(ConvertStr(SWert).c_str(),Kompat);
            break;
          case 5 :
            DarstData[Nummer].YAxis->Max = Eval(ConvertStr(SWert).c_str(),Kompat);
            break;
        }
      }
      break;
    case 6 :
      Ma.Anf = Eval(ConvertStr(SWert).c_str(),Kompat);
      break; // hi_Anfang
    case 7 :
      Ma.Ende = Eval(ConvertStr(SWert).c_str(),Kompat);
      break; // hi_Ende
    case 8 :
      Ma.Step = Eval(ConvertStr(SWert).c_str(),Kompat);
      break; // hi_Delta
    case 9 :
      Ma.Mode = atoi(SWert);
      break; // hi_Modus
    case 10 : // X-Modus
      DarstData[0].XAxis.Begin();
      DarstData[0].XAxis->SetLog(atoi(SWert) != 0);
      DarstData[1].XAxis.Begin();
      DarstData[1].XAxis->SetLog(atoi(SWert) != 0);
      break;
    case 11 :
      Ma.Bezeichner.SetStr(ConvertStr(SWert).c_str());
      break; // hi-Bezeichner
    case 12 :
      Ma.Einheit.SetStr(ConvertStr(SWert).c_str());
      break; // hi-Einheit
    case 13 : // XEin_#
    case 14 : // YEin_#
      if (Nummer == 255)
        Nummer = 0;
      if (Nummer < 2)
      {
        DarstData[Nummer].XAxis.Begin();
        DarstData[Nummer].YAxis.Begin();
        switch (i)
        {
          case 13 :
            DarstData[Nummer].XAxis->Label.SetStr(ConvertStr(SWert).c_str());
            break;
          case 14 :
            DarstData[Nummer].YAxis->Label.SetStr(ConvertStr(SWert).c_str());
            break;
        }
      }
      break;
    case 15 :
      DarstData[Nummer-1].Mode = atoi(SWert);
      break; // Dia-Mode#
    case 16 :
      DarstData[Nummer-1].Titel.SetStr(ConvertStr(SWert).c_str());
      break; // Dia-Titel#
    case 17 :
      AkGraph = atoi(SWert);
      break; // Ak_Graph
    case 18 :
      PostProc = atoi(SWert);
      break; // Post_Proc
    case 19 :
      DarstData[Nummer-1].XAxis.Begin();
      DarstData[Nummer-1].XAxis->SetLog(atoi(SWert) != 0);
      break; // X-Mode_#
    case 20 :
      CXMin = Eval(ConvertStr(SWert).c_str(),Kompat);
      break; // Berech_XMin
    case 21 :
      CXMax = Eval(ConvertStr(SWert).c_str(),Kompat);
      break; // Berech_XMax
    case 22 :
      XMode = atoi(SWert);
      break; // Berech_XMode
    case 23 :
      XLog = atoi(SWert);
      break; // Berech_XLog
    case 24 :
      AutoMode = atoi(SWert);
      break; // Auto_Mode
    case 25 :
      Legende.Font.SetStr(ConvertStr(SWert).c_str());
      break; // Legend_Font
    case 26 :
      Legende.Size = atoi(SWert)*3/(MajorVersion < 5 ? 4 : 3);
      break; // Legend_Fontsize, angepasst an Punktgroesse
    case 27 :
      Legende.Style = atoi(SWert);
      break; // Legend_Fontstyle
    case 28 :
      Legende.Color = strtoul(SWert,0,10);
      break; // Legend_Fontcolor
    case 29 :
      Legende.Frame = atoi(SWert);
      break; // Legend_Frame
    case 30 :
      Achse.Font.SetStr(ConvertStr(SWert).c_str());
      break; // Axis_Font
    case 31 :
      Achse.Size = atoi(SWert)*3/(MajorVersion < 5 ? 4 : 3);
      break; // Axis_Fontsize, angepasst an Punktgroesse
    case 32 :
      Achse.Style = atoi(SWert);
      break; // Axis_Fontstyle
    case 33 :
      Achse.Color = strtoul(SWert,0,10);
      break; // Axis_Fontcolor
    case 34 :
      Titel.Font.SetStr(ConvertStr(SWert).c_str());
      break; // Title_Font
    case 35 :
      Titel.Size = atoi(SWert)*3/(MajorVersion < 5 ? 4 : 3);
      break; // Title_Fontsize, angepasst an Punktgroesse
    case 36 :
      Titel.Style = atoi(SWert);
      break; // Title_Fontstyle
    case 37 :
      Titel.Color = strtoul(SWert,0,10);
      break; // Title_Fontcolor
    case 38 : // Graph-Daten
      YDaten.LoadYDatei(Datei);
      break;
    case 39 :
      Ma.SetParListe(ConvertStr(SWert).c_str());
      break; // hi-Parameterliste
    case 40 :
      Debug = atoi(SWert);
      break; // Debug
    case 41 :
      WinXPos = atoi(SWert);
      break; // WinXPos
    case 42 :
      WinYPos = atoi(SWert);
      break; // WinYPos
    case 43 :
      WinWidth = atoi(SWert);
      break; // WinWidth
    case 44 :
      WinHeight = atoi(SWert);
      break; // WinHeight
    case 45 :
      LegX = atoi(SWert);
      break; // Legende-Positionen
    case 46 :
      LegY = atoi(SWert);
      break;
    case 47 :
      AchsLabels.Font.SetStr(ConvertStr(SWert).c_str());
      break; // AxisLabel_Font
    case 48 :
      AchsLabels.Size = atoi(SWert)*3/(MajorVersion < 5 ? 4 : 3);
      break; // AxisLabel_Fontsize, angepasst an Punktgroesse
    case 49 :
      AchsLabels.Style = atoi(SWert);
      break; // AxisLabel_Fontstyle
    case 50 :
      AchsLabels.Color = strtoul(SWert,0,10);
      break; // AxisLabel_Fontcolor
    case 51 :
      AutoUpdate = atoi(SWert);
      break; // AutoUpdate
    case 52 : // Label_Title#
      Labels.CreateAkEint(Nummer-1);
      Labels->Label.SetStr(ConvertStr(SWert).c_str());
      break;
    case 53 : // Label_Font#
      Labels->Font.Font.SetStr(ConvertStr(SWert).c_str());
      break;
    case 54 : // Label_Fontsize#
      Labels->Font.Size = atoi(SWert);
      break;
    case 55 : // Label_Fontstyle#
      Labels->Font.Style = atoi(SWert);
      break;
    case 56 : // Label_Fontcolor#
      Labels->Font.Color = strtoul(SWert,0,10);
      break;
    case 57 : // Label_Flags#
      Labels->Flags = atoi(SWert);
      break;
    case 58 : // Label_XPos#
      Labels->XPos = Eval(ConvertStr(SWert).c_str(),Kompat);
      break;
    case 59 : // Label_YPos#
      Labels->YPos = Eval(ConvertStr(SWert).c_str(),Kompat);
      break;
    case 60 : // Label_XOffs#
      Labels->XOffs = atoi(SWert);
      break;
    case 61 : //Label_YOffs#
      Labels->YOffs = atoi(SWert);
      break;
    case 62 : //Num_Digits_#
      DarstData[Nummer].NumDigits = atoi(SWert);
      break;
    case 63 : //PostProcTerm
      PostProcTerm.SetStr(ConvertStr(SWert).c_str());
      break;
    default :;
  }
}

void DiagDataObj::VerifyData()
{
  DarstData[0].XAxis.Begin();
  DarstData[0].YAxis.Begin();
  if (!DarstData[0].GetBerOK(DarstData[0].XAxis.GetAkEint()) ||
      !DarstData[0].GetBerOK(DarstData[0].YAxis.GetAkEint()))
    DarstData[0].InitWerber();
  DarstData[1].XAxis.Begin();
  DarstData[1].YAxis.Begin();
  if (!DarstData[1].GetBerOK(DarstData[1].XAxis.GetAkEint()) ||
      !DarstData[1].GetBerOK(DarstData[1].YAxis.GetAkEint()))
    DarstData[1].InitWerber();
}

// ----------------------------------------------------------------------------
// EquationsObj
// ----------------------------------------------------------------------------

const char *EquationsObj::EqNamen[EquationsObj::NumEqNamen] =
{
  _NOCONVERT("Equations_Lines"),
  _NOCONVERT("Equations_XPos"),  // Equations-Fenster-Koordinaten
  _NOCONVERT("Equations_YPos"),
  _NOCONVERT("Equations_Width"),
  _NOCONVERT("Equations_Height"),
  _NOCONVERT("Equations_Font"),
  _NOCONVERT("Equations_Fontsize"),
  _NOCONVERT("Equations_Fontstyle"),
  _NOCONVERT("Equations_Fontcolor"),
  _NOCONVERT("Equations_Viewoptions"),
  _NOCONVERT("Equations_ScrollPosX"),
  _NOCONVERT("Equations_ScrollPosY"),
  _NOCONVERT("Equations_SelAnf"),
  _NOCONVERT("Equations_SelEnd")
};

bool EquationsObj::SetData(const char *Name,const char *SWert,int /*Nummer*/,
                         istream &Datei)
{
  int i = 0;

  while ((i < NumEqNamen) && (strcmp(EqNamen[i],Name) != 0))
    i++;
  if (i < NumEqNamen)
  {
    switch (i)
    {
      case 0: // Equations_Lines
      {
        int i = atoi(SWert);
        ParserObj Psr;
        FunctionArrObj Funktionen;
        EFDataIOObj::LoadFunktionen(&Funktionen,i,Psr,Datei,true);
        Funktionen.Begin();
        Equations.SetStr(_TEXT(""));
        while (Funktionen.NextEint())
        {
          if (Funktionen->Namen[0] != 0)
          {
            Equations.AddStr(Funktionen->Namen);
            if (Funktionen->Term[0] != 0)
              Equations.AddStr(_TEXT(" = "));
          }
          Equations.AddStr(Funktionen->Term);
          if (Funktionen.GetMaxEintrag() > Funktionen.AkEintNummer())
            Equations.AddStr(_TEXT("\r\n"));
        }
        break;
      }
      case 1 : // Equations_XPos
        WinXPos = atoi(SWert);
        break;
      case 2 : // Equations_YPos
        WinYPos = atoi(SWert);
        break;
      case 3 : // Equations_Width
        WinWidth = atoi(SWert);
        break;
      case 4 : // Equations_Height
        WinHeight = atoi(SWert);
        break;
      case 5 : // Equations_Font
        Font.Font.SetStr(ConvertStr(SWert).c_str());
        break;
      case 6 : // Equations_Fontsize
        Font.Size = atoi(SWert);
        break;
      case 7 : // Equations_Fontstyle
        Font.Style = atoi(SWert);
        break;
      case 8 : // Equations_Fontcolor
        Font.Color = strtoul(SWert,0,10);
        break;
      case 9 : // Darstellungsoptionen
        BildschOpt.Set(atoi(SWert));
        break;
      case 10 : // ScrollPosX
        ScrollPosX = atoi(SWert);
        break;
      case 11 : // ScrollPosY
        ScrollPosY = atoi(SWert);
        break;
      case 12 : // SelAnf
        SelAnf = atoi(SWert);
        break;
      case 13 : // SelEnd
        SelEnd = atoi(SWert);
        break;
    }
    return true;
  }
  else
    return false;
}

void EquationsObj::Speichern(ostream &Datei)
{
  ValueOut(Datei,EqNamen[1],WinXPos);
  ValueOut(Datei,EqNamen[2],WinYPos);
  ValueOut(Datei,EqNamen[3],WinWidth);
  ValueOut(Datei,EqNamen[4],WinHeight);
#ifdef __DMC__
  FontOut(Datei,EqNamen[5],Font);
#else
  ValueOut(Datei,EqNamen[5],Font);
#endif
  ValueOut(Datei,EqNamen[9],BildschOpt.Get());
  ValueOut(Datei,EqNamen[10],ScrollPosX);
  ValueOut(Datei,EqNamen[11],ScrollPosY);
  ValueOut(Datei,EqNamen[12],SelAnf);
  ValueOut(Datei,EqNamen[13],SelEnd);
  ValueOut(Datei,EqNamen[0],GetNumLines(Equations.c_str()));
  Datei << Equations << "\r\n"; // Equations
  Datei << "\r\n";
}

