// Version 6.07  21.01.2011 Markus Greither
// Copyright (c) 1991-2011 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file efnmdlgs.cpp
    \brief Source code for the non modal dialogs
*/


#include "efprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#include <wx/xrc/xmlres.h>

#ifndef WX_PRECOMP
#ifdef __DMC__
#include <typeinfo.h>
#endif
#include <wx/wx.h>
#include <wx/resizec.h>

#include <wx/docview.h>
#include <wx/clipbrd.h>
#include <wx/spinctrl.h>

#include <complex>
#include <limits>
#include <cstdio>
#include <set>
#include <functional>
#if defined(__MINGW32__)
#include <wx/msw/winundef.h>
#endif

using namespace std;

#include "efdefs.h"
#include "mg.h"
#include "cpxpsr.h"
#include "diagramm.h"
#include "efgemein.h"
#include "efsolver.h"
#include "efedit.h"
#include "efdocum.h"
#include "ef.h"
#include "eftabcon.h"
#include "efgraphc.h"
#include "efresid.h"
#include "effnkclc.h"
#include "efview.h"
#include "efnmdlgs.h"
#include "efgrdlgs.h"

#endif

// ----------------------------------------------------------------------------
// TEFLabelComboBox
// ----------------------------------------------------------------------------

void TEFLabelComboBox::OnSetFocus(wxFocusEvent &event)
{
  wxString org = GetValue();
  MGPStrSet Labels;
  FunctionArrObj *Funktion = new FunctionArrObj();
  m_view->GetAkEditFunktionen(Funktion);
  Clear();
  Funktion->Begin();
  while (Funktion->NextEint())
  {
    TEFFunktionsPanel::GetLabels((*Funktion)->Namen,Labels);
    TEFFunktionsPanel::GetLabels((*Funktion)->Term,Labels);
  }
  delete Funktion;
  for (MGPStrSet::iterator curr = Labels.begin();curr != Labels.end();curr++)
    Append(curr->c_str());
  SetValue(org);
  event.Skip();
}

BEGIN_EVENT_TABLE(TEFLabelComboBox,wxComboBox)
  EVT_SET_FOCUS(TEFLabelComboBox::OnSetFocus)
END_EVENT_TABLE()

// ----------------------------------------------------------------------------
// TEFLabelEntryDlg
// ----------------------------------------------------------------------------

TEFLabelEntryDlg::TEFLabelEntryDlg(TEFView *V)
  : TEFHilfeDlg(),View(V)
{
}

void TEFLabelEntryDlg::Create(wxWindow *parent,wxWindowID Id,
                                     const wxString &title)
{
  wxXmlResource::Get()->LoadDialog(this,parent,wxT("dlgLabelEntry"));
  defbutton = GetDefaultItem();
  SetId(Id);
  SetTitle(title);
  Centre();
}

void TEFLabelEntryDlg::OnInsert(wxCommandEvent &)
{
  wxComboBox *cb = FindCoB(XRCID("ID_LABEL"));
  wxString org = cb->GetValue();
  org.Prepend(wxT("["));
  org.Append(wxT("]"));
  View->InsertString(org.c_str(),org.Length(),org.Length());
}

void TEFLabelEntryDlg::OnActivate(wxActivateEvent &event)
{
  wxComboBox *cb = FindCoB(XRCID("ID_LABEL"));
  wxString org = cb->GetValue();
  MGPStrSet Labels;
  FunctionArrObj Funktion;
  View->GetAkEditFunktionen(&Funktion);
  cb->Clear();
  Funktion.Begin();
  while (Funktion.NextEint())
  {
    TEFFunktionsPanel::GetLabels(Funktion->Namen,Labels,false);
    TEFFunktionsPanel::GetLabels(Funktion->Term,Labels,false);
  }
  for (MGPStrSet::iterator curr = Labels.begin();curr != Labels.end();curr++)
    cb->Append(curr->c_str());
  cb->SetValue(org);
  event.Skip();
}

BEGIN_EVENT_TABLE(TEFLabelEntryDlg,TEFHilfeDlg)
  EVT_ACTIVATE(TEFLabelEntryDlg::OnActivate)
  EVT_BUTTON(XRCID("ID_INSERT"), TEFLabelEntryDlg::OnInsert)
  EVT_COMMAND_ENTER(XRCID("ID_LABEL"),TEFLabelEntryDlg::OnInsert)
END_EVENT_TABLE()

// ----------------------------------------------------------------------------
// TEFInteractiveSolverPanel
// ----------------------------------------------------------------------------

IMPLEMENT_CLASS(TEFInteractiveSolverPanel,wxPanel)

#ifdef EFMULTITHREAD
TEFInteractiveSolverPanel::TEFInteractiveSolverPanel(InteractiveSolverData &F)
  : wxPanel(),Data(F),threadid(0)
{
}

void TEFInteractiveSolverPanel::RunThread(bool getoperand)
{
  if (getoperand)
  {
    if ((Data.Operator != '=') && (Data.Operator != '+'))
      Data.Operand = wxT("(");
    else
      Data.Operand = wxT("");

    Data.Operand += ((wxComboBox *)FindWindow(XRCID("ID_EXPRESSION")))->GetValue();

    if ((Data.Operator != '=') && (Data.Operator != '+'))
      Data.Operand += wxT(")");
  }

  Data.Fu = &Fu;
  Data.Solver = new SolverObj(&Fu,wxT(""),wxT(""),wxT(""),0,Data.Parser);
  threadid = new InteractiveSolverThread(Data);
  if (threadid)
  {
    threadid->Create();
    threadid->SetPriority(75);
    threadid->Run();
  }
}

void TEFInteractiveSolverPanel::OnAdd(wxCommandEvent &)
{
  Data.Operator = '+';
  RunThread();
}

void TEFInteractiveSolverPanel::OnSubtract(wxCommandEvent &)
{
  Data.Operator = '-';
  RunThread();
}

void TEFInteractiveSolverPanel::OnMultiply(wxCommandEvent &)
{
  Data.Operator = '*';
  RunThread();
}

void TEFInteractiveSolverPanel::OnDivide(wxCommandEvent &)
{
  Data.Operator = '/';
  RunThread();
}

void TEFInteractiveSolverPanel::OnOptimize(wxCommandEvent &)
{
  Data.Operand = wxT("0");
  Data.Operator = '+';
  RunThread(false);
}

void TEFInteractiveSolverPanel::OnSolve(wxCommandEvent &)
{
  Data.Operator = '=';
  RunThread();
}

void TEFInteractiveSolverPanel::OnSetEquation(wxCommandEvent &)
{
  Data.View->GetAkEditFunktionen(&Fu,false);
  Fu.SetAkEint(1);
  MGPStr tmp;
  if (Fu.GetAkEint().Namen[0] && Fu.GetAkEint().Term[0])
  {
    tmp = Fu.GetAkEint().Namen;
    Name = Fu.GetAkEint().Namen;
    tmp += _TEXT(" = ");
    tmp += Fu.GetAkEint().Term;
    Term = _TEXT("(");
    Term += Fu.GetAkEint().Term;
    Term += _TEXT(")");
    ((wxTextCtrl *)FindWindow(XRCID("ID_CUREQUATION")))->SetValue(tmp.c_str());
  }
}

void TEFInteractiveSolverPanel::OnInsertEquation(wxCommandEvent &)
{
  Data.View->GetAkEditFunktionen(&Fu,false);
  Data.Fu = &Fu;
  Fu.SetAkEint(0);
  while (Fu.NextEint() && Term[0] && Name[0])
  {
    if (Fu.GetAkEint().Namen[0] && Fu.GetAkEint().Term[0])
    {
      wxString tmp;
      tmp = Fu.GetAkEint().Namen.c_str();
      tmp.Replace(Name.c_str(),Term.c_str(),true);
      Fu.GetAkEint().Namen = tmp.EF_C_STR();
      tmp = Fu.GetAkEint().Term.c_str();
      tmp.Replace(Name.c_str(),Term.c_str(),true);
      Fu.GetAkEint().Term = tmp.EF_C_STR();
    }
  }
  Data.View->ShowInteractiveSolveResult(Fu,0);
}

void TEFInteractiveSolverPanel::OnClose(wxCloseEvent &event)
{ // close any open thread
  if (threadid)
  {
    if (threadid->IsRunning())
      threadid->Kill();
    delete Data.Solver;
    Data.Solver = 0;
    delete threadid;
    threadid = 0;
  }
  event.Skip();
}

void TEFInteractiveSolverPanel::OnCancel(wxCommandEvent &evt)
{
  if (threadid)
  {
    if (threadid->IsRunning())
      threadid->Kill();
    delete Data.Solver;
    Data.Solver = 0;
    delete threadid;
    threadid = 0;
  }
  if (evt.GetId() == wxID_CANCEL)
    Destroy();
}

void TEFInteractiveSolverPanel::OnIdle(wxIdleEvent &event)
{
  if (threadid)
  {
    if (!threadid->IsRunning())
    {
      delete threadid;
      threadid = 0;
      Data.View->ShowInteractiveSolveResult(Fu,Data.Solver);
      delete Data.Solver;
      Data.Solver = 0;
    }
  }
  wxWindowListNode *Child = GetChildren().GetFirst();
  while (Child)
  {
    if (Child->GetData()->GetId() == XRCID("ID_BREAK"))
      Child->GetData()->Enable(threadid);
    else
    if (Child->GetData()->GetId() != wxID_CANCEL)
      Child->GetData()->Enable(!threadid);
    Child = Child->GetNext();
  }
  if (threadid)
#if wxCHECK_VERSION(2,6,0)
    wxMilliSleep(500);
#else
    wxUsleep(500);
#endif
  event.RequestMore(threadid);
#ifdef __WXGTK__
  if (threadid)
    wxWakeUpIdle(); // FIXME: wakeup GTK, still necessary (?!)
#endif
  event.Skip();
}

void TEFInteractiveSolverPanel::OnHilfe(wxCommandEvent &)
{
  wxGetApp().HelpCtrl.DisplaySection(dlgInteractSolver);
}

BEGIN_EVENT_TABLE(TEFInteractiveSolverPanel,wxPanel)
  EVT_IDLE(TEFInteractiveSolverPanel::OnIdle)
  EVT_CLOSE(TEFInteractiveSolverPanel::OnClose)
  EVT_BUTTON(wxID_HELP,TEFInteractiveSolverPanel::OnHilfe)
  EVT_BUTTON(wxID_CANCEL,TEFInteractiveSolverPanel::OnCancel)
  EVT_BUTTON(XRCID("ID_BREAK") ,TEFInteractiveSolverPanel::OnCancel)
  EVT_BUTTON(XRCID("ID_MULTIPLY"), TEFInteractiveSolverPanel::OnMultiply)
  EVT_BUTTON(XRCID("ID_DIVIDE"), TEFInteractiveSolverPanel::OnDivide)
  EVT_BUTTON(XRCID("ID_ADD"), TEFInteractiveSolverPanel::OnAdd)
  EVT_BUTTON(XRCID("ID_SUBTRACT"), TEFInteractiveSolverPanel::OnSubtract)
  EVT_BUTTON(XRCID("ID_OPTIMIZE"), TEFInteractiveSolverPanel::OnOptimize)
  EVT_BUTTON(XRCID("ID_SETCURRENT"), TEFInteractiveSolverPanel::OnSetEquation)
  EVT_BUTTON(XRCID("ID_INSERT"), TEFInteractiveSolverPanel::OnInsertEquation)
  EVT_BUTTON(XRCID("ID_SOLVE"), TEFInteractiveSolverPanel::OnSolve)
END_EVENT_TABLE()
#endif

// ----------------------------------------------------------------------------
// TEFSolverDlg
// ----------------------------------------------------------------------------

TEFSolverDlg::TEFSolverDlg(FunctionArrObj *F,const _TCHAR *SStr,const _TCHAR *EStr,
                           const _TCHAR *ZStr,int M,ParserObj &psr)
 : TEFEditDlg(),Parser(psr)
{
  Mode = M;
  Solver = new SolverObj(F,SStr,EStr,ZStr,M,&Parser,
                         wxGetApp().SolverDbgOut ? &DbgOut : 0,
                         &RulesOut);
  threadid = 0;
}

TEFSolverDlg::~TEFSolverDlg()
{
  delete Solver;
}

void TEFSolverDlg::Create(wxWindow *parent,wxWindowID ID,const wxString &title)
{
  TEFEditDlg::Create(parent,ID,title);
  Solver->StartWerte();
  if (!(Mode & SolverObj::SolverDebug))
  {
    EnableButtons(false);
    StartLabelThread();
  }
  else
  {
    Editor->Insert(wxT(" "));
    unsigned int Anf,End,tmp;
    Editor->GetSelection(Anf,tmp);
    Editor->Insert(_("Equations to solve:"));
    Editor->Insert(wxT("\r\n"));
    Editor->Insert(Solver->GetErgString());
    Editor->GetSelection(tmp,End);
    Editor->SetSelection(Anf,End);
    Editor->SetSelectionFormat(TEFTextCalc::Bold | TEFTextCalc::FontHeight,12);
    Editor->SetSelection(End,End);
    if (Mode & SolverObj::SolverRegelnOut)
      Editor->Insert(Solver->GetEinschString());
  }
}

bool TEFSolverDlg::StartLabelThread()
{
  if (!threadid)
  {
    Editor->SetSelection(Editor->GetTextLen(),Editor->GetTextLen());
    if (Mode & SolverObj::SolverDebug)
    {
      Editor->Insert(wxT(" "));
      unsigned int Anf,End,tmp;
      Editor->GetSelection(Anf,tmp);
      Editor->Insert(_("Elimination in progress..."));
      Editor->Insert(wxT("\r\n"));
      Editor->GetSelection(tmp,End);
      Editor->SetSelection(Anf,End);
      Editor->SetSelectionFormat(TEFTextCalc::Bold | TEFTextCalc::FontHeight,
                                 12);
      Editor->SetSelection(End,End);
    }
#if defined(EFMULTITHREAD)
    threadid = new EFSolverThread(*Solver);
    if (threadid)
    {
      threadid->Create();
      threadid->SetPriority(75);
      threadid->Run();
    }
    return ((long)threadid != -1);
#else
    threadid = (EFSolverThread *)1;
    Solver->ElimLabel();
    return true;
#endif
  }
  else
    return false;
}

void TEFSolverDlg::OnOk(wxCommandEvent &)
{
  if (!threadid)
  { // only do when no thread is currently running
    EnableButtons(false);
    if ((Mode & SolverObj::SolverDebug) &&  Solver->GetElimLabelStatus() &&
        !Solver->GetAufgeloest() && !Solver->GetFehler())
      StartLabelThread();
    else
    {
      if (Solver->GetAufgeloest())
        EndModal(wxID_OK);
      else
        EndModal(wxID_CANCEL);
    }
  }
  else
    wxBell();
}

void TEFSolverDlg::OnClose(wxCloseEvent &event)
{ // Make sure that open threads are closed
#if defined(EFMULTITHREAD)
  if (threadid)
  {
    if (threadid->IsRunning())
      threadid->Kill();
    delete threadid;
    threadid = 0;
  }
#endif
  event.Skip();
}

void TEFSolverDlg::OnCancel(wxCommandEvent &event)
{
#if defined(EFMULTITHREAD)
  if (threadid)
  {
    if (threadid->IsRunning())
      threadid->Kill();
    delete threadid;
    threadid = 0;
  }
#endif
  CopyDbgOut();
  event.Skip();
}

void TEFSolverDlg::OnIdle(wxIdleEvent &event)
{
  TEFEditDlg::OnIdle(event);
  if (threadid)
  {
#if defined(EFMULTITHREAD)
    if (!threadid->IsRunning())
    { // Thread is already finished
      delete threadid;
#endif
      threadid = 0;
      CopyDbgOut();
      if (Solver->GetFehler()) // in case of error
      {
        Editor->SetSelection(Editor->GetTextLen(),Editor->GetTextLen());
        if (Solver->GetFehler())
        {
          Editor->Insert(wxT(" "));
          unsigned int Anf,End,tmp;
          Editor->GetSelection(Anf,tmp);
          Editor->Insert(FehlerMeldung(Solver->GetFehler()).c_str());
          Editor->GetSelection(tmp,End);
          Editor->SetSelection(Anf,End);
          Editor->SetSelectionFormat(TEFTextCalc::Bold | TEFTextCalc::FontHeight,12);
          Editor->SetSelection(End,End);
          wxBell();
        }
        Editor->Insert(wxT("\r\n "));
        if (Solver->GetErgString()[0] != '\r')
        {
          unsigned int Anf,End,tmp;
          Editor->GetSelection(Anf,tmp);
          Editor->Insert(_("Error during isolation of "));
          Editor->Insert(Solver->GetErgString());
          Editor->GetSelection(tmp,End);
          Editor->SetSelection(Anf,End);
          Editor->SetSelectionFormat(TEFTextCalc::Bold |
                                     TEFTextCalc::FontHeight,
                                     12);
          Editor->SetSelection(End,End);
        }
        if (Mode & SolverObj::SolverRegelnOut)
          Editor->Insert(Solver->GetEinschString());
      }
      else
      if (Mode & SolverObj::SolverDebug)
      { // Debug mode
        EnableButtons(true);
        Editor->SetSelection(Editor->GetTextLen(),Editor->GetTextLen());
        Editor->Insert(wxT(" "));
        unsigned int Anf,End,tmp;
        Editor->GetSelection(Anf,tmp);
        if (!_tcslen(Solver->GetErgString())) // All labels already eliminated
        {
          Editor->Insert(_("No label to eliminate in this equation !"));
          Editor->Insert(wxT("\r\n"));
        }
        else
        if (Solver->GetAufgeloest())
          Editor->Insert(_("Solved for "));
        else
        if (Solver->GetElimLabelStatus()) // Intermediate result
          Editor->Insert(_("Eliminated label "));
        Editor->Insert(Solver->GetErgString());
        Editor->GetSelection(tmp,End);
        Editor->SetSelection(Anf,End);
        Editor->SetSelectionFormat(TEFTextCalc::Bold | TEFTextCalc::FontHeight,
                                   12);
        Editor->SetSelection(End,End);
        int i = 0;
        while (i < RulesOut.GetNumLines()-1)
        {
          MGPStr Zeile;
          RulesOut.GetLine(Zeile,i);
          if (Zeile.Length() > 0)
          {
            wxString z = Zeile.c_str();
            wxString tmp1 = wxGetTranslation(z);
            Editor->Insert(wxT(" "));
            Editor->Insert(tmp1.c_str());
            Editor->Insert(wxT("\r\n"));
          }
          i++;
        }
        RulesOut.SetStr(wxT(""));
        if (Mode & SolverObj::SolverRegelnOut)
          Editor->Insert(Solver->GetEinschString());
      }
      else
      if (Solver->GetElimLabelStatus())
      {
        Editor->Insert(Solver->GetErgString());
        int i = 0;
        while (i < RulesOut.GetNumLines()-1)
        {
          MGPStr Zeile;
          RulesOut.GetLine(Zeile,i);
          if (Zeile.Length() > 0)
          {
            wxString z = Zeile.c_str();
            wxString tmp1 = wxGetTranslation(z);
            Editor->Insert(wxT(" "));
            Editor->Insert(tmp1.c_str());
            Editor->Insert(wxT("\r\n"));
          }
          i++;
        }
        RulesOut.SetStr(wxT(""));
        StartLabelThread();
      }
      else
      {
        if (Mode & SolverObj::SolverRegelnOut)
          Editor->Insert(Solver->GetEinschString());
        EnableButtons(true);
        wxBell();
      }
#if defined(EFMULTITHREAD)
    }
    event.RequestMore(true);
#ifdef __WXGTK__
    wxWakeUpIdle(); // FIXME: wakeup GTK, still necessary (?!)
#endif
#endif
  }
  else
    event.RequestMore(false);
  event.Skip();
}

MGPStr TEFSolverDlg::FehlerMeldung(const int &Fehler)
{
  MGPStr tmp;
  if (Fehler)
  {
    switch (Fehler)
    {
      case SolverObj::OutOfMemory:
        tmp = wxString(_("Memory overflow !")).EF_C_STR();
        break;
      case SolverObj::QuadIntr:
        tmp = wxString(_("No square intermediate result !")).EF_C_STR();
        break;
      case SolverObj::Syntax:
        tmp = wxString(_("Syntax error !")).EF_C_STR();
        break;
      case SolverObj::Parenthesis:
        tmp = wxString(_("Parenthesis error !")).EF_C_STR();
        break;
      case SolverObj::LineToLong:
        tmp = wxString(_("Line too long !")).EF_C_STR();
        break;
      case SolverObj::ConstOutOfRange:
        tmp = wxString(_("Overflow during elimination of constants !")).EF_C_STR();
        break;
      case SolverObj::DivBZero:
        tmp = wxString(_("Division by zero during elimination of constants !")).EF_C_STR();
        break;
      case SolverObj::NotSolved:
        tmp = wxString(_("Couldn't solve for searched label !")).EF_C_STR();
        break;
      case SolverObj::NoEquationEdit:
        tmp = wxString(_("Cannot solve equations in function editor!\nSolve equations in equations window!")).EF_C_STR();
        break;
      default:
        tmp = wxString(_("Unknown error !")).EF_C_STR();
        tmp += wxString(_(" Index:")).EF_C_STR();
        tmp.AddInt(Fehler);
    }
  }
  return tmp;
}

void TEFSolverDlg::CopyDbgOut()
{ // copy current content from DbgOut to clipboard, if available
  if (wxGetApp().SolverDbgOut)
  {
    if (wxTheClipboard->Open())
    {
      wxTheClipboard->SetData( new wxTextDataObject(DbgOut.c_str()) );
      wxTheClipboard->Close();
    }
  }
}

BEGIN_EVENT_TABLE(TEFSolverDlg,TEFEditDlg)
  EVT_IDLE(TEFSolverDlg::OnIdle)
  EVT_BUTTON(wxID_OK, TEFSolverDlg::OnOk)
  EVT_BUTTON(wxID_CANCEL, TEFSolverDlg::OnCancel)
  EVT_CLOSE(TEFSolverDlg::OnClose)
END_EVENT_TABLE()

// ----------------------------------------------------------------------------
// TEFFunktionsPanel
// ----------------------------------------------------------------------------

IMPLEMENT_CLASS(TEFFunktionsPanel,wxPanel)

void TEFFunktionsPanel::Initialize()
{
  for(int i = 0;i < ParserObj::FuTypAnz;i++)
  {
    wxString str = ParserObj::FuTypArr[i].FuTyp;
    ((wxListBox *)FindWindow(XRCID("ID_FUKAT")))->Append(wxGetTranslation(str));
  }
  SetFuStrs(0);
  FindWindow(wxID_YES)->Enable(false);
  FindWindow(XRCID("ID_FUKAT"))->SetFocus();
}

// insert current function
void TEFFunktionsPanel::OnYes(wxCommandEvent &)
{
  long f;

  wxString str = ((wxTextCtrl *)FindWindow(XRCID("ID_SELECTION")))->GetValue();
  unsigned int Anf = str.Length(),End = str.Length();
  size_t i = 0;
  while ((i < str.Length()) && (str[i] != '('))
    i++;
  if ((i < str.Length()) && (str[i] == '('))
    Anf = i+1;
  if (str.Length())
    i = str.Length()-1;
  else
    i = 0;
  while (i && (str[i] != ')'))
    i--;
  if (str[i] == ')')
    End = i;
  if (End < Anf)
    End = Anf;
  f = ((wxListBox *)FindWindow(XRCID("ID_FUKAT")))->GetSelection();
  if (f == ParserObj::FuBezeichner)
  {
    str.Prepend(wxT("["));
    str.Append(wxT("]"));
    Anf += 2;
    End += 2;
  }
  Data.EFView->InsertString(str.c_str(),Anf,End);
}

void TEFFunktionsPanel::OnLbnSelChanged(wxCommandEvent &)
{
  long f;

  f = ((wxListBox *)FindWindow(XRCID("ID_FUKAT")))->GetSelection();
  SetFuStrs(f);
}

// Get all label identifiers
void TEFFunktionsPanel::GetLabels(MGPStr &AkStr,MGPStrSet &Labels,bool withbrackets)
{
  const _TCHAR *tst = AkStr.c_str();
  bool comment = false;
  while (*tst)
  {
    switch (*tst)
    {
      case '"':
        comment = !comment;
        break;
      case '[':
        if (!comment)
        {
          const _TCHAR *end = tst;
          end++;
          while (*end && (*end != ']'))
            end++;
          if ((*end == ']') && (end-tst > 1))
          {
            MGPStr tmp;
            if (!withbrackets)
              tmp.AddNChar(tst+1,end-tst-1);
            else
              tmp.AddNChar(tst,end-tst+1);
            Labels.insert(tmp);
            tst = end;
          }
        }
    }
    tst++;
  }
}

#ifdef __WXGTK__
void TEFFunktionsPanel::AddLineBreaks(wxStaticText *tmp,wxString &trans)
{
  wxSize sz = tmp->GetClientSize();
  wxCoord xsize;
  size_t anfpos = 0,p = 0;
  wxClientDC dc(tmp);
  // Insert line breaks if missing
  do
  {
    do
    {
      p++;
      dc.GetTextExtent(wxString(&trans.c_str()[anfpos],p-anfpos),&xsize,0);
    }
    while ((p < trans.Length()) && (xsize < sz.x));
    // If string too long, find last word and insert break
    if (p < trans.Length())
    {
      p--;
      while ((p > 1) && (p > anfpos+1) &&
             (trans[p] != _TCHAR(' ')) && (trans[p] != _TCHAR('\n')))
        p--;
      trans[p] = _TCHAR('\n');
      anfpos = ++p;
    }
  }
  while (p < trans.Length());
}
#endif

void TEFFunktionsPanel::SetFuStrs(long f)
{
  long i;

  wxListBox *lb = ((wxListBox *)FindWindow(XRCID("ID_FUNKTIONEN")));
  lb->Clear();
  for (i = 1; i <= ParserObj::MaxFuStr+ParserObj::MaxOpStr;i++)
  {
    if ((ParserObj::FuDefArr[int(i)].Mode == f) || (f == ParserObj::FuAlle))
    {
      _TCHAR Zeile[200];
      _tcscpy(Zeile,ParserObj::FuDefArr[int(i)].FuName);
      if (Data.Mode & MyApp::keyLowerCase)
        MGTolower(Zeile);
      lb->Append(Zeile);
    }
  }
  if ((f == ParserObj::FuBezeichner) || (f == ParserObj::FuAlle))
  { // insert label names
    MGPStrSet Labels;
    FunctionArrObj *Funktion = new FunctionArrObj();
    Data.EFView->GetAkEditFunktionen(Funktion);
    Funktion->Begin();
    while (Funktion->NextEint())
    {
      GetLabels((*Funktion)->Namen,Labels,false);
      GetLabels((*Funktion)->Term,Labels,false);
    }
    delete Funktion;
    for (MGPStrSet::iterator curr = Labels.begin();curr != Labels.end();curr++)
      lb->Append(curr->c_str());
  }
  ((wxTextCtrl *)FindWindow(XRCID("ID_SELECTION")))->SetValue(wxT(""));
  wxStaticText *tmp = (wxStaticText *)FindWindow(XRCID("ID_FUDESC"));
  tmp->SetWindowStyle(tmp->GetWindowStyle() | wxST_NO_AUTORESIZE);
  wxString str(ParserObj::FuTypArr[int(f)].Desc);
  wxString trans = wxGetTranslation(str);
#ifdef __WXGTK__
  AddLineBreaks(tmp,trans);
#endif
  tmp->SetLabel(trans);
}

void TEFFunktionsPanel::OnLbnFunkChanged(wxCommandEvent &)
{
  long i,f;
  _TCHAR Zeile[200];
  _TCHAR FBez[200];

  wxListBox *lb = (wxListBox *)FindWindow(XRCID("ID_FUNKTIONEN"));
  wxTextCtrl *tct = (wxTextCtrl *)FindWindow(XRCID("ID_SELECTION"));
  f = lb->GetSelection();
  wxString tmp1 = lb->GetString(f);
  _tcscpy(FBez,tmp1.c_str());
  f = 0;
  _tcscpy(Zeile,FBez);
  if (Data.Mode & MyApp::keyLowerCase)
    MGToupper(FBez);

  // look for the function name but only if it is not a label
  if (((wxListBox *)FindWindow(XRCID("ID_FUKAT")))->GetSelection() != ParserObj::FuBezeichner)
    do
      f++;
    while ((f <= ParserObj::MaxFuStr+ParserObj::MaxOpStr) &&
           (_tcscmp(FBez,ParserObj::FuDefArr[int(f)].FuName) != 0));

  wxString trans;
  if ((f > 0) && (f <= ParserObj::MaxFuStr+ParserObj::MaxOpStr))
  {
    if (ParserObj::FuDefArr[int(f)].AnzParam > 0)
    {
      _tcscat(Zeile,wxT("("));
      i = 1;
      while (i <= ParserObj::FuDefArr[int(f)].AnzParam)
      {
        _tcscat(Zeile,wxT("Arg"));
        Zeile[_tcslen(Zeile)+1] = 0;
        Zeile[_tcslen(Zeile)] = _TCHAR(i+48);
        if (i != ParserObj::FuDefArr[int(f)].AnzParam)
          _tcscat(Zeile,wxGetApp().GetLDV().ListSeparator.c_str());
        i++;
      }
      _tcscat(Zeile,wxT(")"));
    }
    tct->SetValue(Zeile);
    wxString str(ParserObj::FuDefArr[int(f)].Desc);
    trans = wxGetTranslation(str);
  }
  else
  { // For labels
    tct->SetValue(Zeile);
    wxString str(ParserObj::BezString);
    trans = wxGetTranslation(str);
  }
  wxStaticText *tmp = (wxStaticText *)FindWindow(XRCID("ID_FUDESC"));
#ifdef __WXGTK__
  AddLineBreaks(tmp,trans);
#endif
  tmp->SetWindowStyle(tmp->GetWindowStyle() | wxST_NO_AUTORESIZE);
  tmp->SetLabel(trans);
}

void TEFFunktionsPanel::OnTextChanged(wxCommandEvent &)
{
  wxTextCtrl *tct = (wxTextCtrl *)FindWindow(XRCID("ID_SELECTION"));
  FindWindow(wxID_YES)->Enable(tct->GetLastPosition());
}

void TEFFunktionsPanel::OnCancel(wxCommandEvent &/*event*/)
{
  Destroy();
}

void TEFFunktionsPanel::OnHilfe(wxCommandEvent &)
{
  wxGetApp().HelpCtrl.DisplaySection(dlgFunktionen);
}

BEGIN_EVENT_TABLE(TEFFunktionsPanel,wxPanel)
  EVT_LISTBOX(XRCID("ID_FUKAT"),TEFFunktionsPanel::OnLbnSelChanged)
  EVT_LISTBOX(XRCID("ID_FUNKTIONEN"),TEFFunktionsPanel::OnLbnFunkChanged)
  EVT_LISTBOX_DCLICK(XRCID("ID_FUNKTIONEN"),TEFFunktionsPanel::OnYes)
  EVT_BUTTON(wxID_YES, TEFFunktionsPanel::OnYes)
  EVT_BUTTON(wxID_CANCEL, TEFFunktionsPanel::OnCancel)
  EVT_BUTTON(wxID_HELP, TEFFunktionsPanel::OnHilfe)
  EVT_TEXT(XRCID("ID_SELECTION"),TEFFunktionsPanel::OnTextChanged)
END_EVENT_TABLE()

// ----------------------------------------------------------------------------
// TEFSubmodulesPanel
// ----------------------------------------------------------------------------

IMPLEMENT_CLASS(TEFSubmodulesPanel,wxPanel)

TEFSubmodulesPanel::TEFSubmodulesPanel(SubmodulesData &D)
  : wxPanel()
{
  Data = D; // Local copy for non modal dialog
}

void TEFSubmodulesPanel::Initialize()
{
  wxListBox *lb = (wxListBox *)FindWindow(XRCID("ID_SUBMODULES"));

  Data.m_submodules.Begin();
  if (Data.m_submodules.GetMaxEintrag() >= 0)
    do
      lb->Append(Data.m_submodules.GetAkStr().c_str());
    while (Data.m_submodules.NextEint());
}

void TEFSubmodulesPanel::OnDelete(wxCommandEvent &)
{
  wxListBox *lb = (wxListBox *)FindWindow(XRCID("ID_SUBMODULES"));

  int pos = lb->GetSelection();
  if (pos != wxNOT_FOUND)
  {
    lb->Delete(pos);

    ((TEFDocument *)Data.EFView->GetDocument())->m_submodules.SetAkEint(pos);
    ((TEFDocument *)Data.EFView->GetDocument())->m_submodules.LoeschAkEint();
    lb->SetToolTip(wxT(""));
    Data.EFView->GetDocument()->Modify(true);
  }
}

void TEFSubmodulesPanel::OnAdd(wxCommandEvent &)
{
  wxListBox *lb = (wxListBox *)FindWindow(XRCID("ID_SUBMODULES"));

  wxFileDialog *fd = new wxFileDialog(this,_("Pick submodule file"),wxT(""),wxT(""),wxT("*.mcx"),
                                      wxFD_DEFAULT_STYLE | wxFD_OPEN | wxFD_FILE_MUST_EXIST);
  if (fd)
  {
    fd->ShowModal();

    if (fd->GetFilename() != wxT(""))
    {
      wxString tmp;
      tmp = fd->GetDirectory()+wxFileName::GetPathSeparator()+fd->GetFilename();
      lb->Append(tmp);

      ((TEFDocument *)Data.EFView->GetDocument())->m_submodules.NeuAkEint();
      ((TEFDocument *)Data.EFView->GetDocument())->m_submodules.SetAkStr(tmp.c_str());
      Data.EFView->GetDocument()->Modify(true);
    }

    delete fd;
  }
}

void TEFSubmodulesPanel::OnSelect(wxCommandEvent &)
{
  wxListBox *lb = (wxListBox *)FindWindow(XRCID("ID_SUBMODULES"));
  int pos = lb->GetSelection();
  if (pos != wxNOT_FOUND)
  {
    lb->SetToolTip(lb->GetStringSelection());
  }
}
void TEFSubmodulesPanel::OnOpen(wxCommandEvent &)
{
  wxListBox *lb = (wxListBox *)FindWindow(XRCID("ID_SUBMODULES"));
  int pos = lb->GetSelection();
  if (pos != wxNOT_FOUND)
  {
    wxGetApp().m_docManager->CreateDocument(lb->GetStringSelection(),wxDOC_SILENT);
  }
}

void TEFSubmodulesPanel::OnDblClk(wxCommandEvent &)
{
  wxListBox *lb = (wxListBox *)FindWindow(XRCID("ID_SUBMODULES"));
  int pos = lb->GetSelection();
  if (pos != wxNOT_FOUND)
  {
    wxTextEntryDialog txt(this,_("Edit submodule path"),_("Edit path"),lb->GetStringSelection());
    if (txt.ShowModal() == wxID_OK)
    {
      lb->SetString(lb->GetSelection(),txt.GetValue());
      ((TEFDocument *)Data.EFView->GetDocument())->m_submodules.SetAkEint(lb->GetSelection());
      ((TEFDocument *)Data.EFView->GetDocument())->m_submodules.SetAkStr(txt.GetValue().c_str());
      Data.EFView->GetDocument()->Modify(true);
    }
  }
}

void TEFSubmodulesPanel::OnHilfe(wxCommandEvent &)
{
  wxGetApp().HelpCtrl.DisplaySection(dlgSubmodules);
}

BEGIN_EVENT_TABLE(TEFSubmodulesPanel,wxPanel)
  EVT_BUTTON(wxID_CANCEL, TEFSubmodulesPanel::OnCancel)
  EVT_BUTTON(wxID_HELP, TEFSubmodulesPanel::OnHilfe)
  EVT_BUTTON(XRCID("ID_OPEN"),TEFSubmodulesPanel::OnOpen)
  EVT_BUTTON(XRCID("ID_ADD"),TEFSubmodulesPanel::OnAdd)
  EVT_BUTTON(XRCID("ID_REMOVE"),TEFSubmodulesPanel::OnDelete)
  EVT_LISTBOX(XRCID("ID_SUBMODULES"),TEFSubmodulesPanel::OnSelect)
  EVT_LISTBOX_DCLICK(XRCID("ID_SUBMODULES"),TEFSubmodulesPanel::OnDblClk)
END_EVENT_TABLE()

// ----------------------------------------------------------------------------
// TEFEinzelwertPanel
// ----------------------------------------------------------------------------

IMPLEMENT_CLASS(TEFEinzelwertPanel,wxPanel)

TEFEinzelwertPanel::TEFEinzelwertPanel(EinzelwertData &D)
  : wxPanel()
{
  Data = D; // Local copy for non modal dialog
  if (wxGetApp().Options & MyApp::keyOnlyRealResults)
    Parser = new ParserDoubleObj(D.LDV);
  else
    Parser = new ParserCpxObj(D.LDV);
  Parser->InitFunktions(&Data.Fu);
  InsertionPoint = 0;
}

void TEFEinzelwertPanel::InsertString(const _TCHAR *zeile,int Anf,int End)
{ // inserts zeile in TE text control, selects from Anf to End
  _TCHAR tmp[255];
  wxString tmp1 = TE->GetValue(); // Get editor content
  _tcscpy(tmp,tmp1.c_str());
  int len = _tcslen(tmp); // Get string length
  if (len+_tcslen(zeile) < 255)
  { // line length ok
    _tcscat(tmp,zeile); // append line
    TE->SetValue(tmp); // Insert new line
    TE->SetFocus(); // Set focus to editor
    TE->SetSelection(Anf+len,End+len); // Select new entry
    InsertionPoint = Anf+len;
  }
}

void TEFEinzelwertPanel::Initialize()
{
  MGPStr Zeile;

  TE = (TEFComboBox *)FindWindow(XRCID("ID_DIESEFUNKTXT"));
  ((wxRadioButton *)FindWindow(XRCID("ID_DIESEFUNK")))->SetValue(true);
  Parser->Ausgabe(Zeile,0,
                 wxGetApp().ZahlForm == ZahlFormatObj::IngForm ?
                 ZahlFormatObj::LongForm : ZahlFormatObj::LongExpForm);
  ((wxTextCtrl *)FindWindow(XRCID("ID_X1")))->SetValue(Zeile.c_str());
  FindWindow(wxID_NO)->Enable(false);
  TE->SetComboData(*(Data.CD));
  wxStaticText *tmp = (wxStaticText *)FindWindow(XRCID("ID_ERGEBNIS"));
  tmp->SetWindowStyle(tmp->GetWindowStyle() | wxST_NO_AUTORESIZE);
  tmp = (wxStaticText *)FindWindow(XRCID("ID_ERGEBNIS1"));
  tmp->SetWindowStyle(tmp->GetWindowStyle() | wxST_NO_AUTORESIZE);
  TE->SetFocus();
}

TEFEinzelwertPanel::~TEFEinzelwertPanel()
{
  delete Parser;
  delete TE;
}

void TEFEinzelwertPanel::OnNo(wxCommandEvent &)
{
  if (AkErg[0] == '\"')
  { // Append results to function editor at the end
    unsigned int Anf,End,Pos;
    Data.Editor->GetSelection(Anf,End);
    int len = Data.Editor->GetLineLength(Data.Editor->GetLineFromPos(Anf));
    Pos = Data.Editor->GetLineIndex(Data.Editor->GetLineFromPos(Anf))+len;
    Data.Editor->SetSelection(Pos,Pos);
  }
  Data.Editor->Insert(AkErg.c_str());
}

void TEFEinzelwertPanel::OnTextButton(wxCommandEvent &event)
{
  wxButton *bt = (wxButton *)FindWindow(event.GetId());
  ((wxRadioButton *)FindWindow(XRCID("ID_DIESEFUNK")))->SetValue(true);
  TE->SetFocus();
  if (event.GetId() == XRCID("ID_CE"))
  {
    TE->SetValue(wxT(""));
    InsertionPoint = 0;
  }
  else
  {
    wxString res = TE->GetValue();
    if (InsertionPoint > (long)res.Length())
      InsertionPoint = res.Length();
    res.Truncate(InsertionPoint);
    res += bt->GetLabel();
    res += &TE->GetValue().EF_C_STR()[InsertionPoint];
    TE->SetValue(res);
    InsertionPoint += bt->GetLabel().Length();
    TE->SetInsertionPoint(InsertionPoint);
  }
}

void TEFEinzelwertPanel::OnBerechnen(wxCommandEvent &)
{
  Berechnen(true);
}

void TEFEinzelwertPanel::OnAlleBerechnen(wxCommandEvent &)
{
  Cpx y;
  MGPStr Zeile;
  FunctionArrObj *Funktion;
  ParserObj::FehlerRec Fehler;

  wxString tmp1 = ((wxTextCtrl *)FindWindow(XRCID("ID_X1")))->GetValue();
  Zeile.SetStr(tmp1.c_str());
  // Separate parser for x-Values necessary, otherwise function overwrite
  ParserDoubleObj xparser(wxGetApp().GetLDV());
  double x = xparser.Ergebnis(Zeile.c_str(),0);

  Data.EFView->GetFunktionen(&Data.Fu,true,Data.Editor);
  Funktion = &Data.Fu;
  
  MGPStr ergebnis;

  for (int i = 1;i <= Funktion->GetMaxEintrag();i++)
  {
    Funktion->SetAkEint(i);
    if ((Funktion->GetAkEint().ParamStr.Length() == 0) &&
        ((*Funktion)->Namen[0] > ' ') && ((*Funktion)->Namen[0] != 0))
    { // No comment line
      MGPStr Namen;
      Namen = (*Funktion)->Namen;
      Namen += (*Funktion)->ParamStr;

      Fehler.ClearError();
      Parser->ResetFunktions();
      Parser->FunktionsBerechnung(y,x,0,Fehler,0,true);

      if (Fehler.Art == ParserObj::NoError)
      {
        MGPStr RealStr,ImagStr,AkErg;
        int Conversion = ((wxChoice *)FindWindow(XRCID("ID_CONV")))->GetSelection();
        FormatResult(y,x,true,Namen,(*Funktion)->Term,RealStr,ImagStr,AkErg,Conversion);
        ergebnis += AkErg;
        ergebnis += _TEXT("\r\n");
      }
      else
      {
        ergebnis += Namen;
        ergebnis += _TEXT(" = 0 \"");
        ergebnis += _("Error");
        ergebnis += _TEXT(" !");
        ergebnis += _TEXT("\r\n");
      }
    }
  }
  TEFWerteDlg *Edit = new TEFWerteDlg(ergebnis);
  Edit->Create(wxGetApp().GetTopWindow(),dlgAllResults,_("Condensation table for functions"));
  Edit->ShowModal();
  Edit->Destroy();
}

void TEFEinzelwertPanel::Berechnen(bool forceerroroutput)
{
  MGPStr Zeile,Zeile1;
  int DieseFunk;
  FunctionArrObj *Funktion;
  ParserObj::FehlerRec Fehler;
  Cpx y;
  double x;
  bool FehlerImEditor;

  wxString tmp = TE->GetValue();
  Zeile.SetStr(tmp.c_str());
  TE->GetComboData(*(Data.CD)); // Save current listbox entries
  wxCommandEvent evt(wxEVT_COMMAND_MENU_SELECTED,XRCID("ICM_SETMODIFIED"));
  evt.SetEventObject(this);
  GetParent()->GetEventHandler()->ProcessEvent(evt);
  if ((Zeile[0] != 0) && (TE->FindString(Zeile.c_str()) < 0))
    TE->Append(Zeile.c_str()); // Current line in list box not available
  if (((wxRadioButton *)FindWindow(XRCID("ID_AKFUNK")))->GetValue())
    DieseFunk = 0;
  else
    DieseFunk = 1;
  Funktion = &Data.Fu;
  int Conversion = ((wxChoice *)FindWindow(XRCID("ID_CONV")))->GetSelection();
  Data.EFView->GetFunktionen(&Data.Fu,true,Data.Editor);

  if (DieseFunk)
  {
    Funktion->NeuAkEint();
    (*Funktion)->Namen.SetStr(wxT(""));
    (*Funktion)->Term.SetStr(Zeile);
  }

  wxString tmp1 = ((wxTextCtrl *)FindWindow(XRCID("ID_X1")))->GetValue();
  Zeile.SetStr(tmp1.c_str());
  // Separate parser for x-Values necessary, otherwise function overwrite
  ParserDoubleObj xparser(wxGetApp().GetLDV());
  x = xparser.Ergebnis(Zeile.c_str(),0);
  Fehler.ClearError();
  Parser->ResetFunktions();
  Parser->FunktionsBerechnung(y,x,0,Fehler,0,true);
  AkErg.SetStr(wxT(""));
  ((wxRadioButton *)FindWindow(XRCID("ID_AKFUNK")))->SetLabel(_("Current &function"));
  if (Fehler.Art != ParserObj::NoError)
  {
    FehlerImEditor = !((Fehler.FuNummer == Funktion->GetMaxEintrag()) &&
                       DieseFunk);
    if (forceerroroutput)
    {
      Data.EFView->Fehlermeldung(Fehler,FehlerImEditor,Data.Editor);
      if (!FehlerImEditor)
      {
        TE->SetFocus();
        TE->SetSelection(Fehler.Pos,Fehler.Pos);
        InsertionPoint = Fehler.Pos;
      }
    }
    ((wxStaticText *)FindWindow(XRCID("ID_ERGEBNIS")))->SetLabel(wxT(""));
    ((wxStaticText *)FindWindow(XRCID("ID_ERGEBNIS1")))->SetLabel(wxT(""));
    FindWindow(wxID_NO)->Enable(false);
  }
  else
  {
    MGPStr RealStr, ImagStr, Namen;
    Namen = (*Funktion)->Namen;
    Namen += (*Funktion)->ParamStr;
    FormatResult(y,x,false,Namen,(*Funktion)->Term,RealStr,ImagStr,AkErg,Conversion);
    AkErg.InsChar(0,'"');

    Zeile.SetStr(RealStr);
    ((wxStaticText *)FindWindow(XRCID("ID_ERGEBNIS")))->SetLabel(Zeile.c_str());
    if (imag(y))
    {
      Zeile.SetStr(ImagStr);
      ((wxStaticText *)FindWindow(XRCID("ID_ERGEBNIS1")))->SetLabel(Zeile.c_str());
    }
    else
      ((wxStaticText *)FindWindow(XRCID("ID_ERGEBNIS1")))->SetLabel(wxT(""));
    if ((*Funktion)->Namen.Length() == 0)
      AkErg.AddStr(wxT("\r\n"));
    if (DieseFunk)
    {
      TE->SetFocus();
      TE->SetInsertionPoint(InsertionPoint);
    }
    else
    if ((*Funktion)->Namen[0])
    {
      wxString tmp = _("Current &function");
      if ((*Funktion)->Namen[0] != '[')
        tmp += wxT(" [");
      else
        tmp += wxT(" ");
      tmp += (*Funktion)->Namen.c_str();
      if ((*Funktion)->Namen[0] != '[')
        tmp += wxT("]");
      ((wxRadioButton *)FindWindow(XRCID("ID_AKFUNK")))->SetLabel(tmp);
      ((wxRadioButton *)FindWindow(XRCID("ID_AKFUNK")))->Fit();
    }
    FindWindow(wxID_NO)->Enable(true);
  }
  if (DieseFunk)
    Funktion->LoeschAkEint();
}

Cpx TEFEinzelwertPanel::ConvertResult(Cpx y,int Conversion)
{
  Cpx ret;
  fptr oldhndl = SetFPESignal();
  try
  {
    switch (Conversion)
    {
      case 1     :
        ret = Cpx(abs(y),arg(y)*180/M_PI);
        break;
      case 2     :
        ret = Cpx(log10(abs(y))*20,arg(y)*180/M_PI);
        break;
      default :
        ret = y;
    }
  }
  catch (...)
  {
    ret = Cpx(0,0);
  }
  ClearFPESignal(oldhndl);
  return ret;
}

void TEFEinzelwertPanel::FormatResult(Cpx y,double x, bool StandartFormat, 
                                      const MGPStr &Namen,const MGPStr &Term,
                                      MGPStr &RealStr, MGPStr &ImagStr, MGPStr &AkErg,
                                      int Conversion)
{
  MGPStr Zeile1;

  y = ConvertResult(y,Conversion);
  if (imag(y))
  {
    switch (Conversion)
    {
      case 1: 
        RealStr.SetStr(_("Absolute"));
        RealStr.AddStr(wxT(": "));
        break;
      case 2: 
        RealStr.SetStr(_("Absolute, dB"));
        RealStr.AddStr(wxT(": "));
        break;
      default: RealStr.SetStr(wxT("Real: "));
    }
  }
  else
    RealStr.SetStr(wxT(""));

  if (Namen.Length() > 0)
    AkErg.AddStr(Namen);
  else
    AkErg.AddStr(Term);
  AkErg.AddStr(wxT("(x = "));

  int ZFormat;
  if (StandartFormat)
    ZFormat = wxGetApp().ZahlForm;
  else
    ZFormat = (wxGetApp().ZahlForm == ZahlFormatObj::IngForm ?
               ZahlFormatObj::LongForm : ZahlFormatObj::LongExpForm);

  Parser->Ausgabe(Zeile1,x,ZFormat);
  AkErg.AddStr(Zeile1);
  AkErg.AddStr(wxT(") = "));
  Parser->Ausgabe(Zeile1,real(y),ZFormat);
  if (Conversion != 0)
  {
    AkErg.AddStr(wxT("\""));
    if (Conversion == 1)
      AkErg.AddStr(_("Absolute"));
    else
      AkErg.AddStr(_("Absolute, dB"));
    AkErg.AddStr(wxT(":\""));
  }
  AkErg.AddStr(Zeile1);
  Parser->Ausgabe(Zeile1,real(y),ZFormat);
  RealStr.AddStr(Zeile1);
  if (imag(y))
  {
    if (Conversion == 0)
      ImagStr.SetStr(wxT("Imag: "));
    else
    {
      ImagStr.SetStr(_("Angle"));
      ImagStr.AddStr(wxT(": "));
    }
    Parser->Ausgabe(Zeile1,imag(y),ZFormat);
    if (Conversion != 0)
    {
      AkErg.AddStr(wxT(",\""));
      AkErg.AddStr(_("Angle"));
      AkErg.AddStr(wxT(":\""));
      AkErg.AddStr(Zeile1);
    }
    else
    {
      if (imag(y) > 0)
        AkErg.AddStr(wxT("+"));
      AkErg.AddStr(Zeile1);
      AkErg.AddStr(wxT("*j"));
    }
    Parser->Ausgabe(Zeile1,imag(y),ZFormat);
    ImagStr.AddStr(Zeile1);
  }
  else
    ImagStr = _TEXT("");
}

void TEFEinzelwertPanel::OnEditorBerechnen(wxCommandEvent &)
{
  ((wxRadioButton *)FindWindow(XRCID("ID_AKFUNK")))->SetValue(true);
  ((wxRadioButton *)FindWindow(XRCID("ID_DIESEFUNK")))->SetValue(false);
  Berechnen(false);
}

void TEFEinzelwertPanel::EnSetFocus(wxCommandEvent &)
{
  ((wxRadioButton *)FindWindow(XRCID("ID_AKFUNK")))->SetValue(false);
  ((wxRadioButton *)FindWindow(XRCID("ID_DIESEFUNK")))->SetValue(true);
}

bool TEFEinzelwertPanel::GetLocalEditorActive()
{
  return ((wxRadioButton *)FindWindow(XRCID("ID_DIESEFUNK")))->GetValue();
}

void TEFEinzelwertPanel::OnIdle(wxIdleEvent &event)
{
  if (TE && (FindFocus() == TE))
    InsertionPoint = TE->GetInsertionPoint();
  event.Skip();
}

void TEFEinzelwertPanel::OnHilfe(wxCommandEvent &)
{
  wxGetApp().HelpCtrl.DisplaySection(dlgEinzelwert);
}

BEGIN_EVENT_TABLE(TEFEinzelwertPanel,wxPanel)
  EVT_IDLE(TEFEinzelwertPanel::OnIdle)
  EVT_MENU(XRCID("ICM_WERTBERECHNEN"),TEFEinzelwertPanel::OnEditorBerechnen)
  EVT_BUTTON(XRCID("ID_0"),TEFEinzelwertPanel::OnTextButton)
  EVT_BUTTON(XRCID("ID_1"),TEFEinzelwertPanel::OnTextButton)
  EVT_BUTTON(XRCID("ID_2"),TEFEinzelwertPanel::OnTextButton)
  EVT_BUTTON(XRCID("ID_3"),TEFEinzelwertPanel::OnTextButton)
  EVT_BUTTON(XRCID("ID_4"),TEFEinzelwertPanel::OnTextButton)
  EVT_BUTTON(XRCID("ID_5"),TEFEinzelwertPanel::OnTextButton)
  EVT_BUTTON(XRCID("ID_6"),TEFEinzelwertPanel::OnTextButton)
  EVT_BUTTON(XRCID("ID_7"),TEFEinzelwertPanel::OnTextButton)
  EVT_BUTTON(XRCID("ID_8"),TEFEinzelwertPanel::OnTextButton)
  EVT_BUTTON(XRCID("ID_9"),TEFEinzelwertPanel::OnTextButton)
  EVT_BUTTON(XRCID("ID_SEPARATOR"),TEFEinzelwertPanel::OnTextButton)
  EVT_BUTTON(XRCID("ID_KOMMENT"),TEFEinzelwertPanel::OnTextButton)
  EVT_BUTTON(XRCID("ID_PLUS"),TEFEinzelwertPanel::OnTextButton)
  EVT_BUTTON(XRCID("ID_MINUS"),TEFEinzelwertPanel::OnTextButton)
  EVT_BUTTON(XRCID("ID_MAL"),TEFEinzelwertPanel::OnTextButton)
  EVT_BUTTON(XRCID("ID_DIFF"),TEFEinzelwertPanel::OnTextButton)
  EVT_BUTTON(XRCID("ID_BRUCH"),TEFEinzelwertPanel::OnTextButton)
  EVT_BUTTON(XRCID("ID_PARALLEL"),TEFEinzelwertPanel::OnTextButton)
  EVT_BUTTON(XRCID("ID_POT"),TEFEinzelwertPanel::OnTextButton)
  EVT_BUTTON(XRCID("ID_KLAUF"),TEFEinzelwertPanel::OnTextButton)
  EVT_BUTTON(XRCID("ID_KLZU"),TEFEinzelwertPanel::OnTextButton)
  EVT_BUTTON(XRCID("ID_LABELANF"),TEFEinzelwertPanel::OnTextButton)
  EVT_BUTTON(XRCID("ID_LABELEND"),TEFEinzelwertPanel::OnTextButton)
  EVT_BUTTON(XRCID("ID_GLEICH"),TEFEinzelwertPanel::OnTextButton)
  EVT_BUTTON(XRCID("ID_GROESSER"),TEFEinzelwertPanel::OnTextButton)
  EVT_BUTTON(XRCID("ID_KLEINER"),TEFEinzelwertPanel::OnTextButton)
  EVT_BUTTON(XRCID("ID_PI"),TEFEinzelwertPanel::OnTextButton)
  EVT_BUTTON(XRCID("ID_E"),TEFEinzelwertPanel::OnTextButton)
  EVT_BUTTON(XRCID("ID_SIN"),TEFEinzelwertPanel::OnTextButton)
  EVT_BUTTON(XRCID("ID_COS"),TEFEinzelwertPanel::OnTextButton)
  EVT_BUTTON(XRCID("ID_TAN"),TEFEinzelwertPanel::OnTextButton)
  EVT_BUTTON(XRCID("ID_CE"),TEFEinzelwertPanel::OnTextButton)
  EVT_BUTTON(XRCID("ID_W"),TEFEinzelwertPanel::OnTextButton)
  EVT_BUTTON(XRCID("ID_X"),TEFEinzelwertPanel::OnTextButton)
  EVT_BUTTON(XRCID("ID_J"),TEFEinzelwertPanel::OnTextButton)
  EVT_BUTTON(XRCID("ID_LN"),TEFEinzelwertPanel::OnTextButton)
  EVT_BUTTON(wxID_YES,TEFEinzelwertPanel::OnBerechnen)
  EVT_BUTTON(wxID_NO,TEFEinzelwertPanel::OnNo)
  EVT_BUTTON(wxID_RETRY,TEFEinzelwertPanel::OnAlleBerechnen)
  EVT_BUTTON(wxID_CANCEL, TEFEinzelwertPanel::OnCancel)
  EVT_BUTTON(wxID_HELP, TEFEinzelwertPanel::OnHilfe)
  EVT_CHOICE(XRCID("ID_CONV"), TEFEinzelwertPanel::OnBerechnen)
  EVT_TEXT(XRCID("ID_DIESEFUNKTXT"),TEFEinzelwertPanel::EnSetFocus)
  EVT_COMMAND_ENTER(XRCID("ID_DIESEFUNKTXT"),TEFEinzelwertPanel::OnBerechnen)
END_EVENT_TABLE()

