// Version 6.05  18.07.2008 Markus Greither
// Copyright (c) 1991-2008 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file efreg.cpp
    \brief Source code for the program registration values and class
*/


#include "efprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#ifdef __DMC__
#include <typeinfo.h>
#endif
#include <wx/wx.h>
#endif

#if defined(UNICODE) && !defined(_UNICODE)
#define _UNICODE
#endif

#ifdef __WXMSW__

#include <windows.h>

#include <tchar.h>

#ifdef __MINGW32__
#define _TCHAR TCHAR
#endif

#define WXHKEY HKEY

#include "efreg.h"           

// ----------------------------------------------------------------------------
// EFRegister
// ----------------------------------------------------------------------------
          
EFKeyData EFRegister::Keys[EFRegister::AnzKeys] =
{
  {_TEXT("EasyFunktion.Document.6"),0,_TEXT("EasyFunktion Arbeitsblatt")},
  {_TEXT("EasyFunktion.Document.6\\protocol"),0,0},
  {_TEXT("EasyFunktion.Document.6\\protocol\\StdFileEditing"),0,0},
  {_TEXT("EasyFunktion.Document.6\\protocol\\StdFileEditing\\verb"),0,0},
  {_TEXT("EasyFunktion.Document.6\\protocol\\StdFileEditing\\verb\\0"),
   0,_TEXT("&Bearbeiten")},
  {_TEXT("EasyFunktion.Document.6\\protocol\\StdFileEditing\\verb\\1"),0,_TEXT("&Open")},
  {_TEXT("EasyFunktion.Document.6\\Shell"),0,0},
  {_TEXT("EasyFunktion.Document.6\\Shell\\Bearbeiten"),0,0},
  {_TEXT("EasyFunktion.Document.6\\Shell\\Bearbeiten\\Command"),
   0,_TEXT("$wnotepad.exe \"%1\"")},
  {_TEXT("EasyFunktion.Document.6\\Shell\\Open"),0,0},
  {_TEXT("EasyFunktion.Document.6\\Shell\\Open\\Command"),0,_TEXT("$p \"%1\"")},
  {_TEXT("EasyFunktion.Document.6\\DefaultIcon"),0,_TEXT("$P,1")},
  {_TEXT(".mcx"),0,_TEXT("EasyFunktion.Document.6")},
  {_TEXT(".mcx"),_TEXT("Content Type"),_TEXT("application/EasyFunktion")},
  {_TEXT(".mcx\\ShellNew"),_TEXT("NullFile"),_TEXT("")}
};

bool EFRegister::CreateKey(const _TCHAR *name,WXHKEY &key)
{
  unsigned long disp;
  DWORD error;
  error = ::RegCreateKeyEx(HKEY_CLASSES_ROOT,name,0,0,
                           REG_OPTION_NON_VOLATILE,KEY_CREATE_SUB_KEY+KEY_SET_VALUE,
                           0,&key,&disp);
  if (error != ERROR_SUCCESS)
  {
    _TCHAR str[200];
    _stprintf(str,_TEXT("Error %i creating key"),error);
    ::MessageBox(0,str,name,MB_OK+MB_ICONEXCLAMATION);
    return false;
  }
  else
    return true;
}

bool EFRegister::SetValueKey(WXHKEY &key,const _TCHAR *value,const _TCHAR *data)
{
  size_t size = (_tcslen(data)+1)*sizeof(_TCHAR);
  if (::RegSetValueEx(key,value,0,REG_SZ,(const unsigned char *)data,
                      size) != ERROR_SUCCESS)
  {
    ::MessageBox(0,_TEXT("Error setting key"),value,MB_OK+MB_ICONEXCLAMATION);
    return false;
  }
  else
    return true;
}

void EFRegister::SetRegValues(const _TCHAR *Path, bool debug)
{
  FILE *file;
  if (debug)
    file = _tfopen(_TEXT("debugreg.txt"),_TEXT("w"));
  for (int i = 0;i < AnzKeys;i++)
  { // alle Keys eintragen
    HKEY tmpkey;
    if (!CreateKey(Keys[i].key,tmpkey) && debug)
      _ftprintf(file,_TEXT("Failed to create key %s.\n"),Keys[i].key);
    if (tmpkey)
    { // wenn Key offen
      if (Keys[i].data)
      { // wenn Wert vorhanden
        _TCHAR Zeile[260];
        const _TCHAR *tmp;
        if (Keys[i].data[0] == '$')
        { // Pfad eintragen
          if (Keys[i].data[1] == 'w')
          { // Windows-Verzeichnis eintragen
            ::GetWindowsDirectory(Zeile,260);
            if (Zeile[_tcslen(Zeile)-1] != '\\')
              _tcscat(Zeile,_TEXT("\\"));
          }
          else
          if (Keys[i].data[1] == 'p')
          { // Programm-Pfad in Hochkommas eintragen
            _tcscpy(Zeile,_TEXT("\""));
            _tcscat(Zeile,Path); // Programm-Pfad eintragen
            _tcscat(Zeile,_TEXT("\""));
          }
          else
          if (Keys[i].data[1] == 'P')
            _tcscpy(Zeile,Path); // Programm-Pfad eintragen
          _tcscat(Zeile,&Keys[i].data[2]);
          tmp = Zeile;
        }
        else
          tmp = Keys[i].data;
        if (!SetValueKey(tmpkey,Keys[i].value,tmp) && debug)
          _ftprintf(file,_TEXT("Failed to set value %s to key %s.\n"),Keys[i].value,Keys[i].key);
      }
      ::RegCloseKey(tmpkey);
    }
    else
    if (!debug)
      return;
  }
  if (debug)
    fclose(file);
}

void EFRegister::DelRegValues()
{ // Keys von hinten loeschen
  // (jeder Key in der Hierarie muss separat geloescht werden !)
  const _TCHAR *lastkey = 0;
  for (int i = AnzKeys-1;i >= 0;i--)
  {
    if (!lastkey || (_tcscmp(lastkey,Keys[i].key) != 0))
    { // jeden key nur 1 mal loeschen !
      if (::RegDeleteKey(HKEY_CLASSES_ROOT,Keys[i].key) != ERROR_SUCCESS)
        ::MessageBox(0,_TEXT("Error deleting key"),
                     Keys[i].key,MB_OK+MB_ICONEXCLAMATION);
      lastkey = Keys[i].key;
    }
  }
}

#endif
