// Version 6.05 20.07.2008 Markus Greither
// Copyright (c) 1991-2008 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file efsolver.cpp
    \brief Source code for the symbolic function solver
*/

#ifdef __BORLANDC__

#include "efprec.h"

#pragma hdrstop
#else

#ifdef __DMC__
#include <typeinfo.h>
#endif
#include <cstdio>
#include <cfloat>
#include <complex>
#include <limits>
#include <cstdlib>
#include <cstring>
#include <set>
#include <functional>
#include <ctime>

using namespace std;

#include "efdefs.h"
#include "mg.h"
#include "cpxpsr.h"
#include "efsolver.h"
#include "efkrzn.h"

#endif

#if defined(__BORLANDC__) && defined(__WIN32__)
#include <excpt.h>
#endif

// ----------------------------------------------------------------------------
// SolverTools
// ----------------------------------------------------------------------------

void SolverTools::DebugString(const _TCHAR *tst)
{ // gibt Debug-Informationen in DbgOut aus
  if (!DbgOut)
    return;
  DbgOut->AddStr(tst);
  DbgOut->AddStr(_TEXT(" "));
  int tmp = 0;
  if (firsttime)
    tmp = (int)difftime(time(0),firsttime);
  time(&firsttime);
  DbgOut->AddInt(tmp);
}

void SolverTools::RulesString(const _TCHAR *tst)
{ // gibt Regel-Informationen in DbgOut aus
  if (!RulesOut)
    return;
  RulesOut->AddStr(tst);
  RulesOut->AddStr(_TEXT("\r\n"));
}

// Verarbeitet eine Kette von Faktoren
// so da kein * oder / vorne steht
void SolverTools::FaksToSummant(MGPStr &AkStr)
{
  unsigned int p = 0;
  // evtl. Vorzeichen beruecksichtigen
  if ((AkStr[p] == '+') || (AkStr[p] == '-'))
    p++;

  if (!AkStr[p])
    return;

  int diffpos = p;
  bool gef = false;
  if (AkStr[p] == '/')
  {
    do
    {
      MGPStr FakStr;
      NextFakt(AkStr,FakStr,p);
      if (AkStr[p] == '*')
      {
        MGPStr tmp(&AkStr[p]); // Faktoren ab * in tmp
        AkStr.ShortStr(p); // String ab * loeschen
        AkStr.InsStr(diffpos,tmp); // Faktoren ab * vorne einfuegen
        gef = true;
      }
    }
    while (!Fehler && !gef && (p < AkStr.Length()));
    if (!gef) // immer noch / vorne
    {
      MGPStr NeuStr;
      AddMarker(NeuStr,ZahlMarker,1);
      AkStr.InsStr(diffpos,NeuStr);
    }
  }

  if (AkStr[diffpos] == '*') // * Zeichen loeschen
    AkStr.DelNChar(diffpos,1);

  // sollte hinter dem * ein Vorzeichen aufgetaucht sein,
  // dieses Vorzeichen mit evtl. Gesamt-Vorzeichen verrechnen
  if (diffpos && ((AkStr[diffpos] == '+') || (AkStr[diffpos] == '-')))
  {
    AkStr.ReplChar(0,MultVorZei(AkStr[diffpos],AkStr[0]));
    AkStr.DelNChar(diffpos,1);
  }
}

// fuegt einen String in die AbstrFeld-Liste
// und gibt in f die Nummer in der Liste zurueck
bool SolverTools::AddAbstrString(const _TCHAR *AkStr,int &f)
{
  bool MemFehler;

  f = 0;
  AbstrFeld.Begin();
  while ((f < AbstrFeld.GetMaxEintrag()) &&
         (_tcscmp(AbstrFeld.GetAkStr().c_str(),AkStr) != 0))
  {
    f++;
    AbstrFeld.SetAkEint(f);
  }
  if (_tcscmp(AbstrFeld.GetAkStr().c_str(),AkStr) != 0)
    f++;
  if ((f > AbstrFeld.GetMaxEintrag()))
  {
    MemFehler = AbstrFeld.NeuAkEint();
    if (!MemFehler)
      AbstrFeld.SetAkStr(AkStr);
    else
      return false;
  }
  return true;
}

// liest naechsten Faktor ab Position p in AkStr, aktualisiert p,
// gibt Faktor in FakStr zurueck, bearbeitet Strings mit Klammern
void SolverTools::NextFakt(MGPStr &AkStr,MGPStr &FakStr,unsigned int &p)
{
  int KlZaehler = 0;

//  DebugString(" NEF");
  FakStr.SetStr(_TEXT(""));
  if ((p < AkStr.Length()) && !Fehler)
  { // wenn noch nicht am Ende des Strings
    const _TCHAR *pt = &AkStr[p];
    if ((*pt  == '*') || (*pt == '/'))
    {
      pt++; // FakZeichen ueberlesen
      if ((*pt == '+') || (*pt == '-'))
        pt++; // Vorzeichen ueberlesen
    }
    else
      FakStr.AddChar('*');
    while ((*pt != 0) && (!((KlZaehler == 0) && ((*pt == '*') ||
           (*pt == '/') || (*pt == '+') || (*pt == '-')))))
    { // solange nicht am Ende des Strings, kein Rechenzeichen, Klammern
      if ((*pt == _TCHAR(FuMarker)) || (*pt == _TCHAR(ZahlMarker)) ||
          (*pt == _TCHAR(LabelMarker)) || (*pt == _TCHAR(PotMarker)))
        pt += 2; // Label gefunden Position hochzaehlen
      else
      { // alle anderen Zeichen eintragen
        if (*pt == '(')
          KlZaehler++; // Klammern beruecksichtigen
        else
        if (*pt == ')')
          KlZaehler--;
        pt++;
      }
    }
    int del = int(pt-&AkStr[p]);
    FakStr.AddNChar(&AkStr[p],del);
    p += del;
  }
  // Fehler wenn Klammerebenen nicht Null am Ende des Strings
  if ((p >= AkStr.Length()) && (KlZaehler != 0))
    Fehler = Parenthesis;
}

// liest den naechsten Summanten und gibt in in SumStr zurueck, aktualisiert p,
// vorhandenes Vorzeichen in RechZei, gibt zurueck, ob Summant Klammern enthaelt
bool SolverTools::Summant(const _TCHAR *AkStr,MGPStr &SumStr,
                          unsigned int &p,_TCHAR &RechZei)
{
  bool klammern = false;
//  DebugString(" SUM");
  SumStr.SetStr(_TEXT("")); // temporaerer String fuer Summanten
  RechZei = 0;
  if ((p < _tcslen(AkStr)) && !Fehler) // noch nicht am Ende des Strings
  {
    if ((AkStr[p] == '+') || (AkStr[p] == '-'))
    { // wenn Vorzeichen vorhanden
      RechZei = AkStr[p]; // Vorzeichen eintragen in RechZei
      p++;
    }
    const _TCHAR *pt = &AkStr[p];
    int KlZaehler = 0; // Klammerzaehler zuruecksetzen
    while ((*pt != 0) && !((KlZaehler == 0) && ((*pt == '+') || (*pt == '-'))))
    { // solange nicht Ende der Zeile, Klammern vorhanden und kein + oder -
      switch (*pt)
      {
        case PotMarker :
        case FuMarker :
        case ZahlMarker :
        case LabelMarker : // Label beruecksichtigen
            pt += 2;
            break;
        case '(' :
            KlZaehler++; // Klammern beruecksichtigen
            klammern = true;
            pt++;
            break;
        case ')' :
            KlZaehler--;
            klammern = true;
            pt++;
            break;
        case '/' :
        case '*' : // wenn Multiplikation oder Division
            pt++;
            if ((*pt == '+') || (*pt == '-'))
              pt++; // wenn Vorzeichen
            break;
        default :
            pt++;
      }
    }
    SumStr.SetStr(&AkStr[p]); // Summanten eintragen
    SumStr.ShortStr(int(pt-&AkStr[p]));
    p += int(pt-&AkStr[p]); // Zaehler korrigieren
    if (KlZaehler != 0)
      Fehler = Parenthesis;
  }
  return klammern;
}

bool SolverTools::InAbstrFeld(const _TCHAR *tst)
{ // pruefen ob tst in AbstrFeld vorhanden
  AbstrFeld.Begin();
  while (AbstrFeld.NextEint())
    if (_tcscmp(AbstrFeld.GetAkStr().c_str(),tst) == 0)
      return true;
  return false;
}

// formatiert FakStr fuer Ausgabe nach AStr,
// *-Zeichen vor Summanten loeschen, /-Zeichen 1 voranstellen
void SolverTools::FormatFakStr(MGPStr &AStr,const _TCHAR *FakStr)
{
  // der 1. Summant MUSS ein Vorzeichen haben !
  unsigned int p2 = 0;
  size_t faklen = _tcslen(FakStr);
  while (p2 < faklen)
  { // alle Summanten in FakStr
    AStr.AddChar(FakStr[p2]);
    p2++;
    unsigned int p3 = p2;
    while ((p3 < faklen) && (FakStr[p3] != '+') && (FakStr[p3] != '-'))
      p3 += 3; // solange weitere Faktoren dieses Summanten
    if (FakStr[p2] == '*')
      p2++; // wenn 1. Faktor in diesem Summanten mit *
    else
    if (FakStr[p2] == '/') // wenn 1. Faktor mit /, 1 voranstellen
      MarkerObj::AddMarker(AStr,MarkerObj::ZahlMarker,1);
    AStr.AddNChar(&FakStr[p2],p3-p2); // diesen Summant kopieren
    p2 = p3;
  }
}

// ----------------------------------------------------------------------------
// SolverObj
// ----------------------------------------------------------------------------

SolverObj::SolverObj(FunctionArrObj *Fu,const _TCHAR *SStr,const _TCHAR *EStr,
                     const _TCHAR *ZStr,int Mode,ParserObj *P,
                     MGPStr *DO,MGPStr *RL)
 : SolverTools(P,DO,RL)
{
  Aufgeloest = false;
  ElimLabelStatus = true;
  NoMoreLabels = false;
  Funktion = Fu;
  SuchFaktor.SetStr(SStr);
  ErhFaktoren.SetStr(EStr);
  ZusFaktoren.SetStr(ZStr);
  Teilen = (Mode & SolverTeilen) > 0;
  Debug = (Mode & SolverDebug) > 0;
  KlKrz = (Mode & SolverKlKuerzen) > 0;
  if ((Mode & SolverRegelnOut) == 0)
    RulesOut = 0;
  ErgebnisOpt = false;
}

SolverObj::~SolverObj()
{
}

// kuerzt Klammerausdruecke
void SolverObj::FakGruppenKuerzen(MGPStr &LAbsStr,MGPStr &RAbsStr)
{
  SumFakMatrixObj Matrix(Parser,DbgOut);

  if (DbgOut) DebugString(_TEXT(" FGK"));
  if ((LAbsStr[0] != 0) && (RAbsStr[0] != 0))
  { // nur wenn ueberhaupt Terme vorhanden
    // Summanten-Faktoren-Matrix ermitteln
    Matrix.Create(LAbsStr,RAbsStr);
    bool GueltigVar = (Matrix.AnzLSum && Matrix.AnzRSum);
    if (GueltigVar) // gueltige Anzahl von Summanten
    {
      FaktorenVar FakVar(Matrix);
      // Pruefen auf maximale Kombinations-Anzahl
      GueltigVar &= (FakVar.GetAnzKombs() < 100000L);
      if (GueltigVar && !FakVar.CheckVar())
        GueltigVar = FakVar.NextVar(); // 1. gueltige Variation finden
      MGPStr ElimStr;
      bool gef = false;
      if (GueltigVar)
      do
        gef = FakVar.Kuerzen(LAbsStr,RAbsStr,ElimStr);
      while (!gef && FakVar.NextVar());
      if (gef)
      {
        MGPStr tmp;
        SolverTools::FormatFakStr(tmp,ElimStr.c_str());
        EinschString.AddStr(_TEXT(" Elim: "));
        EinschString.AddStr(ReAbstrahieren(tmp));
        EinschString.AddStr(_TEXT(" <> 0\r\n"));
      }
    }
  }
}

// loescht doppelte Zeilen in AStr, nach jeder Zeile muss '\r\n' stehen
void SolverObj::DelDuplicateLines(MGPStr &AStr)
{
  MGPStrSet Zeilen;

  int i = 0;
  while (AStr[i])
  { // Zeilen ermitteln
    int AkPos = i;
    while (AStr[i] && (AStr[i] != '\n'))
      i++;
    if (AStr[i] == '\n')
      i++;
    MGPStr tmp;  
    tmp.AddNChar(&AStr[AkPos],i-AkPos);
    Zeilen.insert(tmp);
  }
  AStr.SetStr(_TEXT(""));
  for (MGPStrSet::iterator curr = Zeilen.begin();curr != Zeilen.end();curr++)
    AStr.AddStr(*curr);
}

// wandelt eine Funktion in das SolvObj-Format
void SolverObj::Abstrahieren(MGPStr &LAbsStr, MGPStr &RAbsStr,bool NeuFeld)
{
  Abstrahieren(LAbsStr,NeuFeld);
  Abstrahieren(RAbsStr);
}

// Basis und Exponent von Potenz-Ausdruck optimieren
bool SolverObj::OptimizePotenz(MGPStr &AkStr,unsigned int &p,
                               unsigned int &AnfPos,int &KlZaehler)
{
  bool ret = true;
  unsigned int potpos = AnfPos;
  Parser->AkWert(AkStr,potpos,false,false); // Basis ermitteln
  if ((AkStr[potpos] == '^') && (potpos < p))
  { // wenn Potenz gefunden
    MGPStr ExpStr;
    ExpStr.AddNChar(&AkStr[potpos+1],p-potpos-1); // Exponent kopieren
    if (!_tcsstr(ExpStr.c_str(),_TEXT("^")))
    { // wenn nur 1 Potenz
      if ((ExpStr[0] == '(') && (ExpStr[ExpStr.Length()-1] == ')'))
      { // evtl. Klammern entfernen
        ExpStr.DelNChar(0,1);
        ExpStr.ShortStr(ExpStr.Length()-1);
      }
      MGStrObj OrgAbstrFeld(AbstrFeld); // aktuelles AbstrFeld retten
      Abstrahieren(ExpStr);
      OptimizeSubExp(ExpStr,false); // Exponent optimieren
      MGPStr BasStr;
      BasStr.SetStr(_TEXT(""));
      BasStr.AddNChar(&AkStr[AnfPos],potpos-AnfPos);
      Abstrahieren(BasStr);
      OptimizeSubExp(BasStr,false); // Basis optimieren
      MGPStr SumStr;
      unsigned int p1 = 0;
      _TCHAR RechZei;
      Summant(BasStr.c_str(),SumStr,p1,RechZei);
      double Fak = 1;
      if ((SumStr[0] == ZahlMarker) && (SumStr.Length() == 2))
      { // wenn Basis eine numerische Konstante
        unsigned int p2 = 0;
        int PrsFehler = ZahlFormatObj::NoError;
        AbstrFeld.SetAkEint((unsigned char)SumStr[1]);
        double Basis = Parser->Zahl(AbstrFeld.GetAkStr().c_str(),p2,PrsFehler);
        if ((PrsFehler == ZahlFormatObj::NoError) &&
            (p2 == AbstrFeld.GetAkStr().Length()))
        { // Basis ist gueltige Zahl
          if (RechZei == '-')
            Basis *= -1;
          MGPStr SumStr1;
          p2 = 0;
          _TCHAR RechZei1;
          MGPStr Exp;
          while ((p2 < ExpStr.Length()) && !Fehler)
          { // alle Summanten des Exponenten
            Summant(ExpStr.c_str(),SumStr1,p2,RechZei1);
            if ((SumStr1[0] == ZahlMarker) && (SumStr1.Length() == 2))
            { // Summant ist num. Konstante
              unsigned int p3 = 0;
              int PrsFehler = ZahlFormatObj::NoError;
              AbstrFeld.SetAkEint((unsigned char)SumStr1[1]);
              double Wert = Parser->Zahl(AbstrFeld.GetAkStr().c_str(),p3,PrsFehler);
              if ((PrsFehler == ZahlFormatObj::NoError) &&
                  (p3 == AbstrFeld.GetAkStr().Length()))
              {
                if (RechZei1 == '-')
                  Fak /= pow(Basis,Wert);
                else
                  Fak *= pow(Basis,Wert);
              }
            }
            else
            {
              if ((Exp.Length() > 0) && !RechZei1)
                Exp.AddChar('+');
              else
                Exp.AddChar(RechZei1);
              Exp.AddStr(SumStr1);
            }
          }
          ExpStr.SetStr(Exp);
        }
      }
      Einklammern(ExpStr);
      ExpStr = ReAbstrahieren(ExpStr);
      int exppos = potpos+1;
      AkStr.DelNChar(exppos,p-exppos);
      p -= (p-exppos);
      if (ExpStr.Length() > 0)
      { // wenn noch Exponent vorhanden
        if (!_tcscmp(ExpStr.c_str(),_TEXT("1"))) // wenn Exponent 1
        {
          AkStr.DelNChar(potpos,1); // ^ entfernen
          p--;
        }
        else
        { // gueltiger Exponent gefunden
          AkStr.InsStr(exppos,ExpStr);
          p += ExpStr.Length();
          Einklammern(BasStr);
          BasStr = ReAbstrahieren(BasStr);
          // AbstrFeld wiederherstellen, weil evtl. SuchStr eingetragen wurde
          AkStr.DelNChar(AnfPos,potpos-AnfPos);
          p -= (potpos-AnfPos);
          AkStr.InsStr(AnfPos,BasStr);
          p += BasStr.Length();
        }
      }
      else
      { // Exponent vollstaendig aufgeloest
        AkStr.DelNChar(AnfPos,potpos-AnfPos+1); // Basis und ^ entfernen
        p -= (potpos-AnfPos+1);
      }
      AbstrFeld = OrgAbstrFeld;
      if (Fak != 1)
      { // nummerische Konstante einfuegen
        RulesString(_EFT("Constant extracted from exponent (Base^(Exp+Const) -> Base^Const*Base^Exp)"));
        MGPStr buff;
        Parser->Ausgabe(buff,Fak,ZahlFormatObj::FitForm);
        int f1;
        AddAbstrString(buff.c_str(),f1);
        if (ExpStr.Length() > 0)
          buff.SetStr(_TEXT("(")); // Faktor*Bruch einklammern wg. Operatorreihenfolge
        else
          buff.SetStr(_TEXT(""));
        AddMarker(buff,ZahlMarker,f1);
        if (ExpStr.Length() > 0)
          buff.AddChar('*');
        AkStr.InsStr(AnfPos,buff);
        AnfPos += buff.Length();
        p += buff.Length();
        if (ExpStr.Length() > 0)
        { // wenn Potenz noch vorhanden
          AkStr.InsChar(p,')');
          KlZaehler++;
        }
        else // wenn Potenz vollstaendig aufgeloest
          ret = false;
      }
    }
    else
      Fehler = Syntax;
  }
  return ret;
}

// Abstrahieren-Version mit nur 1 String
void SolverObj::Abstrahieren(MGPStr &AkStr,bool NeuFeld)
{
  unsigned int p,AnfPos;
  int f;
  MGPStr AkFu,tmp;
  int KlAnz;
  _TCHAR Marker;
  bool SumZeichen,FakZeichen;

  DebugString(_TEXT(" ABS"));
  if (NeuFeld)
  { // wenn AbstrFeld neu initialisiert werden soll
    AbstrFeld.ResetKette();
    if (!AddAbstrString(_TEXT("1"),f))
      Fehler = SolverObj::OutOfMemory;// im AbstrFeld[1] steht immer eine 1
  }
  f = 0;
  SumZeichen = false;
  FakZeichen = false;
  KlAnz = 0;
  p = 0;
  while ((p < AkStr.Length()) && !Fehler)
  { // String lesen
    if (IST_BUCHS(AkStr[p]) || IST_ZAHL(AkStr[p]) ||
        (AkStr[p] == '[') || (AkStr[p] == '('))
    { // Wenn Buchstabe, Zahl oder Label oder Klammer
     AnfPos = p;
     Marker = Parser->AkWert(AkStr,p,true);
     if ((Marker != 'P') && (Marker != 'L'))
     { // Wenn kein |
       if ((Marker == ZahlMarker) || (Marker == LabelMarker) ||
           (Marker == PotMarker) || (Marker == FuMarker))
       { // Funktion oder Operator ^
         if (!Fehler)
         { // Kein Fehler
           if ((Marker != PotMarker) || OptimizePotenz(AkStr,p,AnfPos,KlAnz))
           {
             AkFu.SetStr(&AkStr[AnfPos]); // Funktion kopieren
             AkFu.ShortStr(p-AnfPos);
             if (!AddAbstrString(AkFu.c_str(),f))
               Fehler = SolverObj::OutOfMemory;
             tmp.SetStr(AkStr); // Anfang von AkStr vor AkFu in tmp
             tmp.ShortStr(AnfPos);
             AddMarker(tmp,Marker,f);
             tmp.AddStr(&AkStr[p]); // Rest von AkStr nach AkFu anhaengen
             AkStr.SetStr(tmp); // AkStr aktualisieren
             p = AnfPos+2; // aktuelle Position in der Zeile korrigieren
           }
         }
         FakZeichen = false;
       }
       else
       if (Marker == 'K')
       { // Klammerausdruck
         KlAnz++; // Klammer mitzaehlen
         p = AnfPos+1; // Klammer ueberlesen
         FakZeichen = true; // Keine Multiplikationen/Divisionen zulassen
       }
       else
       {
         Fehler = SolverObj::Syntax;
         DebugString(AkStr.c_str());
       }
       SumZeichen = false;
     }
     else
     {
       if (Marker == 'L')
         RulesString(_EFT("Resolved Parallel expression (A || B -> (A+B)/(A*B))"));
       else
       if (Marker == 'P')
         RulesString(_EFT("Exponents multiplied (Base^Exp1^Exp2 -> Base^(Exp1*Exp2))"));
       p = AnfPos;
     }
    }
    else
    switch (AkStr[p])
    {
      case ')' :
        if (KlAnz > 0)
          {p++; KlAnz--;} // wenn Klammern offen, Klammerzaehler reduzieren
        else
          Fehler = SolverObj::Parenthesis; // ansonsten Klammerfehler
        SumZeichen = false; // alle Rechenzeichen zulassen
        FakZeichen = false;
        break;
      case ' ' : // Leerzeichen loeschen
        AkStr.DelNChar(p,1);
        break;
      case '"' : // Kommentare loeschen
      {
        int anfp = p;
        Parser->LeerZNKomment(AkStr.c_str(),p);
        AkStr.DelNChar(anfp,p-anfp);
        p = anfp;
        break;
      }
      default:
        if (AkStr[p] == '\\')
          AkStr.ReplChar(p,'/'); // Bruchstriche in Divisionen aendern
        Fehler  |= SumZeichen && ((AkStr[p] == '+') || (AkStr[p] == '-'))
                || FakZeichen && ((AkStr[p] == '*') || (AkStr[p] == '/'))
                || (!((AkStr[p] == '+') || (AkStr[p] == '-')
                || (AkStr[p] == '*') || (AkStr[p] == '/')));
        SumZeichen = ((AkStr[p] == '+') || (AkStr[p] == '-'));
        FakZeichen = ((AkStr[p] == '*') || (AkStr[p] == '/'));
        if (Fehler)
        {
          Fehler = SolverObj::Syntax;
          DebugString(AkStr.c_str());
        }
        p++;
    }
  }
}

// AkStr wieder zurueckwandeln in lesbaren String
const _TCHAR *SolverObj::ReAbstrahieren(MGPStr &AkStr)
{
  unsigned int p;
  static MGPStr ErgStr;

  MGPStr tmp;
  tmp.AddStr(_TEXT(" RAB "));
  tmp.AddInt(AbstrFeld.GetMaxEintrag());
  DebugString(tmp.c_str());
//   FILE * file = fopen("c:\\abstrfeld.txt","a"); // Test-Code fuer AbstrFeld
//   fprintf(file,"---- Neue Tabelle \n");
//   AbstrFeld.Begin();
//   while (AbstrFeld.NextEint())
//     fprintf(file,"%s\n",AbstrFeld->c_str());
//   fclose(file);
  ErgStr.SetStr(_TEXT(""));
  p = 0;
  while ((p < AkStr.Length()) && !Fehler)
  {
    switch (AkStr[p])
    { // String zurueckwandeln
      case '+':
      case '-':
      case '*':
      case '/':
      case '\\':
      case '(':
      case ')': // Rechenzeichen und Klammern ueberlesen
        if (AkStr[p] == '/')
          ErgStr.AddChar('\\'); // Bruchstriche eintragen
        else
          ErgStr.AddChar(AkStr[p]);
        p++;
        break;
      case ZahlMarker :
      {
        AbstrFeld.SetAkEint((unsigned char)AkStr[p+1]);
        BruchZahl Wert;
        unsigned int p2 = 0;
        int PsrFehler = ZahlFormatObj::NoError;
        Parser->ParseZahl(AbstrFeld.GetAkStr().c_str(),p2,PsrFehler,Wert);
        if ((PsrFehler == ZahlFormatObj::NoError) &&
            (Wert.GemischterBruch))
        {
          MGPStr tmp;
          Parser->Ausgabe(tmp,Wert.Zaehler,ZahlFormatObj::FitForm);
          ErgStr.AddChar('(');
          ErgStr.AddStr(tmp);
          ErgStr.AddStr(_TEXT("\\"));
          Parser->Ausgabe(tmp,Wert.Nenner,ZahlFormatObj::FitForm);
          ErgStr.AddStr(tmp);
          ErgStr.AddChar(')');
        }
        else
          ErgStr.AddStr(AbstrFeld.GetAkStr());
        p += 2;
        break;
      }
      case PotMarker :
      case FuMarker :
      case LabelMarker : // alle Labels durch AbstrStr-Listeneintrag ersetzen
        AbstrFeld.SetAkEint((unsigned char)AkStr[p+1]);
        p++;
        ErgStr.AddStr(AbstrFeld.GetAkStr());
        p++;
        break;
      default : // alles andere ueberlesen
        p++;
    }
  }
  return ErgStr.c_str();
}

// holt den ersten Faktor eines Multiplikations/Divisionsausdruckes mit Klammern
void SolverObj::Faktor1(MGPStr &SumStr,MGPStr &FStr,MGPStr &KlStr,
                        unsigned int &p1)
{
  int KlZaehler;
  bool KeineKl;

  FStr.SetStr(_TEXT(""));      // Produktterm VOR der Klammer
  KlStr.SetStr(_TEXT(""));     // gefundene Klammer
  KlZaehler = 0;
  KeineKl = true;
  do
  {
    switch (SumStr[p1])
    {
      case ')' : // Ende eines Summanten erreicht
          KlZaehler--;
          KlStr.AddChar(SumStr[p1]);
          p1++;
          break;
      case '(' : // Beginn eines Klammerausdrucks
          if (KeineKl)
          { // wenn erste Klammer
            KeineKl = false;
            FStr.SetStr(KlStr); // Faktoren vor Klammer in FStr
            KlStr.SetStr(_TEXT("")); // Klammer neu beginnen
          }
          KlZaehler++;
          KlStr.AddChar(SumStr[p1]);
          p1++;
          break;
      case PotMarker :
      case FuMarker :
      case ZahlMarker :
      case LabelMarker : // alle Eintrge an AkStr
          KlStr.AddChar(SumStr[p1]);
          KlStr.AddChar(SumStr[p1+1]);
          p1 += 2;
          break;
      case '*' :
      case '/' : // wenn * /
          KlStr.AddChar(SumStr[p1]);
          p1++;
          if ((SumStr[p1] == '+') || (SumStr[p1] == '-'))
          { // wenn Faktor mit Vorzeichen
            KlStr.AddChar(SumStr[p1]);
            p1++;
          }
          break;
      default : // alle anderen Zeichen
          KlStr.AddChar(SumStr[p1]);
          p1++;
    }
  }
  while (!((SumStr[p1] == ')') && (KlZaehler == 1)) &&
         (p1 < SumStr.Length()) && !Fehler);
  if (KeineKl)
  { // noch keine Klammer gefunden

    FStr.SetStr(_TEXT(""));
    KlStr.SetStr(_TEXT(""));
  }
  else
  { // bereits Klammer gefunden
    KlStr.AddChar(SumStr[p1]); // letzte Klammer hinzufuegen
    p1++;
  }
}

void SolverObj::Faktor2(MGPStr &SumStr,MGPStr &FStr,unsigned int &p1)
{ // holt den Faktor NACH der Klammer
  int KlZaehler;
  MGPStr HiStr;
  bool DivMul;

 KlZaehler = 0;
 HiStr.SetStr(_TEXT(""));
 FStr.SetStr(_TEXT(""));
 if (!((SumStr[p1] == ')') || (SumStr[p1] == '+') || (SumStr[p1] == '-')))
 do
 {
   if (SumStr[p1] == '(')
     KlZaehler++;
   else
   if (SumStr[p1] == ')')
   {
     if (KlZaehler > 0) KlZaehler--;
     else Fehler = SolverObj::Parenthesis;
   }
   else
   if ((SumStr[p1] == _TCHAR(FuMarker)) || (SumStr[p1] == _TCHAR(ZahlMarker)) ||
       (SumStr[p1] == _TCHAR(LabelMarker)) || (SumStr[p1] == _TCHAR(PotMarker)))
   {
     FStr.AddChar(SumStr[p1]);
     p1++;
   }
   FStr.AddChar(SumStr[p1]);
   DivMul = ((SumStr[p1] == '*') || (SumStr[p1] == '/'));
   p1++;
   if (DivMul && ((SumStr[p1] == '+') || (SumStr[p1] == '-')))
   { // Vorzeichen nach * /
     FStr.AddChar(SumStr[p1]);
     p1++;
   }
 }
 while ((DivMul || !((SumStr[p1] == '+') || (SumStr[p1] == '-') ||
        (SumStr[p1] == ')')) ||
        (KlZaehler != 0)) && !Fehler && (p1 < SumStr.Length()));
}

void SolverObj::AusMult(MGPStr &LAbsStr,MGPStr &RAbsStr)
{ // entfernen aller Klammern durch Ausmultiplizieren
  MGPStr AkStr,SumStr,KlStr1,KlStr2,NeuStr,HiStr,HiStr1,HiStr2;
  int f1;
  unsigned int p1 = 0;
  _TCHAR RechZei1 = '+',RechZei2,RechZei3,RechZei4;
  bool DivEntf;

  DebugString(_TEXT(" AML"));
  do
  { // solange Divisionen entfernt wurden
    DivEntf = false;
    for (int i = 1;i <= 2;i++)
    { // zuerst LAbsStr, dann RAbsStr
      if (i == 1)
        AkStr.SetStr(LAbsStr);
      else
        AkStr.SetStr(RAbsStr);
      unsigned int p5 = 0;
      bool Kl1;
      do
      { // solange Klammerausdruecke gefunden wurden
        unsigned int p = p5; // auf zuletzt bearbeiteten Summanten zuruecksetzen
        Kl1 = false;
        while (!Kl1 && (p < AkStr.Length()) && !Fehler)
        { // Klammerausdruck suchen
          p5 = p; // p5 = Anfangsposition des aktuellen Summanten
          if (Summant(AkStr.c_str(),SumStr,p,RechZei1)) // holt ersten Summanten
          { // wenn Summant mit Klammern
            p1 = 0;
            HiStr1.SetStr(_TEXT(""));
            Faktor1(SumStr,HiStr1,KlStr1,p1); // holt ersten Faktor
            if (KlStr1[0] != 0)
            { // Klammer 1 vorhanden
              Kl1 = true;
              KlStr1.DelNChar(0,1);
              KlStr1.ShortStr(KlStr1.Length()-1);; // entfernt Klammern
            }
            if (Kl1 && (p1 < SumStr.Length())) // SUMSTR nicht ausgeschpft
              Faktor2(SumStr,KlStr2,p1); // holt zweiten Faktor
            else
              KlStr2.SetStr(_TEXT(""));
            if (RechZei1 == 0)
              RechZei1 = '+'; // Default-Wert
            else
              p1++; // fr P1 = ALLE Zeichen, die bearbeitet wurden
          }
        }
        // wenn entweder 2. Klammer oder Faktor vor Klammer
        if (Kl1 && ((HiStr1[0] != 0) || (KlStr2[0] != 0)) &&
           ((HiStr1[0] == 0) || (HiStr1[HiStr1.Length()-1] == '*')) &&
        // wenn keine KlStr2, oder Multiplikation, oder KlStr2 ist keine Klammer
           ((KlStr2[0] == 0) || (KlStr2[0] == '*') || (KlStr2[1] != '(')))
        { // wenn Multiplikation mit HiStr1
          // (Ausdruck VOR Klammern) und/oder KlStr2 (2. Klammer)
          unsigned int p3 = 0;
          NeuStr.SetStr(_TEXT(""));
          while ((p3 < KlStr1.Length()) && !Fehler)
          { // jeder Summant aus KlStr1 (1. Klammer)
            Summant(KlStr1.c_str(),HiStr,p3,RechZei2);
            if (RechZei2 == 0)
              RechZei2 = '+';
            unsigned int p4 = 0;
            do
            { // jeder Summant aus HiStr1 (Ausdruck vor Klammer 1)
              Summant(HiStr1.c_str(),HiStr2,p4,RechZei3);
              RechZei4 = MultVorZei(RechZei1 != RechZei2 ? '-' : '+',RechZei3);
              NeuStr.AddChar(RechZei4); // Korrigiertes Rechenzeichen
              NeuStr.AddStr(HiStr2); // Aktueller Summant aus HiStr1
              NeuStr.AddStr(HiStr); // Aktueller Summant aus KlStr1
              NeuStr.AddStr(KlStr2); // und der ganze KlStr2
            }
            while ((p4 < HiStr1.Length()) && !Fehler);
          }
          MGPStr tmp(AkStr); // String vor dem Anfang des untersuchten Ausdrucks
          tmp.ShortStr(p5);
          tmp.AddStr(NeuStr); // aufgeloester Klammer-Summant
          tmp.AddStr(&AkStr[p1+p5]);
          AkStr.SetStr(tmp);
        }
        else
        if (Kl1 && (HiStr1[0] != 0) && (HiStr1[HiStr1.Length()-1] == '/') ||
           (KlStr2[0] != 0) && (KlStr2[0] == '/'))
        { // Division
          if ((KlStr2[0] != 0) && (KlStr2[0] == '/'))
            KlStr1.SetStr(&KlStr2[1]); // es wird durch KlStr2 geteilt
          else
          {
            KlStr1.InsStr(0,_TEXT("("));
            KlStr1.AddStr(_TEXT(")")); // wenn durch KlStr1 geteilt wird, einklammern
          }
          unsigned int p4 = 0;
          NextFakt(KlStr1,KlStr2,p4); // 1. Faktor aus KlStr1
          if (i == 1)
            EntfDiv(AkStr,RAbsStr,&KlStr2[1]);
          else
            EntfDiv(LAbsStr,AkStr,&KlStr2[1]);
          p5 = 0; // nach EntfDiv wieder beim 1. Summanten anfangen
          DivEntf = true;
        }
        else
        // String OHNE Klammern um KlStr1 vorzeichenrichtig zusammensetzen
        // (Klammer ohne HiStr1 und KlStr2)
        if (Kl1 && (KlStr1[0] != 0) && (HiStr1[0] == 0) && (KlStr2[0] == 0))
        {
          HiStr2.SetStr(_TEXT(""));
          unsigned int p4 = 0;
          while ((p4 < KlStr1.Length()) && !Fehler)
          { // alle Summanten in KlStr1
            // ermittelt aktuellen Summanten + Vorzeichen vor KlStr1
            Summant(KlStr1.c_str(),HiStr1,p4,RechZei2);
            RechZei2 = MultVorZei(RechZei2,RechZei1);
            HiStr2.AddChar(RechZei2); // korrigiertes Rechenzeichen
            HiStr2.AddStr(HiStr1); // und aktuellen Summanten aus KlStr1
          }
          MGPStr tmp(AkStr); // String vor dem Anfang des untersuchten Ausdrucks
          tmp.ShortStr(p5);
          tmp.AddStr(HiStr2); // Vorzeichen-korrigierter KlStr1
          tmp.AddStr(&AkStr[p1+p5]); // Restlicher String
          AkStr.SetStr(tmp);
        }
      }
      while (Kl1 && !Fehler);
      if (i == 1)
        LAbsStr.SetStr(AkStr);
      else
        RAbsStr.SetStr(AkStr);
    }
  }
  while ((DivEntf == true) && !Fehler);
  Vereinfachen(LAbsStr);
  Vereinfachen(RAbsStr);
  Aufadd(LAbsStr);
  if (LAbsStr[0] == 0)
  {
    if (!AddAbstrString(_TEXT("0"),f1))
      Fehler = SolverObj::OutOfMemory;
    LAbsStr.SetStr(_TEXT(""));
    AddMarker(LAbsStr,ZahlMarker,f1);
  }
  Aufadd(RAbsStr);
  if (RAbsStr[0] == 0)
  {
    if (!AddAbstrString(_TEXT("0"),f1))
      Fehler = SolverObj::OutOfMemory;
    RAbsStr.SetStr(_TEXT(""));
    AddMarker(RAbsStr,ZahlMarker,f1);
  }
  SumKuerzen(LAbsStr);
  SumKuerzen(RAbsStr);
  Kuerzen(LAbsStr,RAbsStr);
}

// prueft ob alle Labels in AkStr in GefLabels
// -> String kann nicht weiter eliminiert werden
bool SolverObj::Konstanten(MGPStr &AkStr)
{
  MGPStr SumStr;
  unsigned int p,p1;
  _TCHAR RechZei;
  bool Kons;

  DebugString(_TEXT(" KON"));
  Kons = true;
  p = 0;
  if (AkStr.Length() < 4)
    Kons = false; // einzelne Labels nicht auslagern
  else
  while ((p < AkStr.Length()) && Kons && !Fehler)
  { // ganzen AkStr lesen
    Summant(AkStr.c_str(),SumStr,p,RechZei); // aktuellen Summanten lesen
    p1 = 0;
    while ((p1 < SumStr.Length()) && Kons && !Fehler)
    { // alle Faktoren des Summanten
      MGPStr FStr;
      NextFakt(SumStr,FStr,p1);
      const _TCHAR *FakStr = FStr.c_str();
      FakStr++; // * oder / -Zeichen von FakStr entfernen
      if (FakStr[0] == '(')
      { // vorhandene Klammern entfernen
        MGPStr tmp(FakStr);
        tmp.SetStr(&tmp[1]); // Klammern entfernen
        tmp.ShortStr(tmp.Length()-1);
        Kons = Konstanten(tmp); // rekursiver Aufruf fr Klammerausdruck
      }
      else
      if (FakStr[0] == LabelMarker)
      { // Label gefunden
        AbstrFeld.SetAkEint((unsigned char)FakStr[1]);
        // Wenn 1. Zeichen von AbstrStr '@' -> Keine Konstanten 2mal ausl.
        // frueher: Konstante Ausdruecke konnten mehrfach ausgelagert werden
        // -> viel mehr Konstanten-Ausdruecke
        // -> viel kuerzere Terme
        // -> sehr undurchsichtige Ergebnisse
        if (AbstrFeld.GetAkStr()[1] == '@')
          Kons = false;
        else
        // Wenn Label noch nicht eliminiert oder nicht gesperrt
        //-> kein konstanter Ausdruck
        if (!InGefLabels(AbstrFeld.GetAkStr().c_str()))
          Kons = false;
        else // aktueller AbstrStr gesuchtes Label -> kein konstanter Ausdruck
        if (_tcscmp(AbstrFeld.GetAkStr().c_str(),SuchFaktor.c_str()) == 0)
          Kons = false;
      }
    }
  }
  return Kons;
}

void SolverObj::EntfVorzeichen(MGPStr &AkStr)
{ // entfernt Vorzeichen vor Faktoren
  MGPStr SumStr,FakStr,NeuStr,TempStr;
  _TCHAR RechZei;
  unsigned int p,p1;

  DebugString(_TEXT(" ENV"));
  p = 0;
  NeuStr.SetStr(_TEXT(""));
  while ((p < AkStr.Length()) && !Fehler)
  { // alle Zeichen von AkStr bearbeiten
    Summant(AkStr.c_str(),SumStr,p,RechZei);
    p1 = 0;
    TempStr.SetStr(_TEXT(""));
    while ((p1 < SumStr.Length()) && !Fehler)
    { // alle Faktoren des aktuellen Summanten
      int anfpos = (p1 == 0 ? 1 : 0); // Anfang des Original-Faktors (ohne *)
      NextFakt(SumStr,FakStr,p1);
      if ((FakStr.Length() > 3))
      { // wenn Faktor lange genug fuer Vorzeichen
        if ((FakStr[1] == '+') || (FakStr[1] == '-'))
        { // * oder / und dann + oder -
          // wenn negativ, das Vorzeichen des aktuellen Summanten 'umdrehen'
          RechZei = MultVorZei(RechZei,FakStr[1]);
          FakStr.DelNChar(1,1); // Vorzeichen entfernen
        }
      }
      TempStr.AddStr(&FakStr[anfpos]); // bereinigten Faktor in TempStr
    }
    NeuStr.AddChar(RechZei); // das korrigierte Vorzeichen in den NeuStr
    NeuStr.AddStr(TempStr); // bereinigte Faktoren anhaengen
  }
  AkStr.SetStr(NeuStr); // Vorzeichen-bereinigten NeuStr in AkStr kopieren
}

MGStrObj *SolverObj::SucheFak(MGPStr &LAbsStr,MGPStr &RAbsStr)
{ // sucht Faktoren, die in LAbsStr und RAbsStr in jedem Summanten vorkommen
  unsigned int p,p1;
  _TCHAR RechZei;
  MGPStr AkStr,SumStr,FakStr;
  MGStrObj *FakFeld,*FakFeld1;

  FakFeld = new MGStrObj;
  FakFeld1 = new MGStrObj;
  p = 0;
  Summant(RAbsStr.c_str(),SumStr,p,RechZei);
  p1 = 0;
  while ((p1 < SumStr.Length()) && !Fehler)
  { // holt alle Faktoren aus 1. Summanten in RAbsStr in FakFeld
    NextFakt(SumStr,FakStr,p1);
    if (FakFeld->NeuAkEint()) Fehler = SolverObj::OutOfMemory;
    FakFeld->SetAkStr(FakStr.c_str()); // in neuen Eintrag in FakFeld eintragen
  }
  for (int i = 1;i <= 2;i++)
  { // LAbsStr und RAbsStr absuchen
    if (i == 1)
      AkStr.SetStr(LAbsStr);
    else
      AkStr.SetStr(RAbsStr);
    p = 0;
    if (AkStr[0] == 0)
      FakFeld->ResetKette(); // wenn aktueller String leer, keine gem. Faktoren
    else
    while ((FakFeld->GetMaxEintrag() > 0) &&
           (p < AkStr.Length()) && !Fehler)
    { // ermittelt alle Faktoren, die in allen Summanten vorkommen
      Summant(AkStr.c_str(),SumStr,p,RechZei);
      p1 = 0;
      while ((p1 < SumStr.Length()) && !Fehler)
      { // alle Faktoren
        NextFakt(SumStr,FakStr,p1);
        FakFeld->Begin();
        while (FakFeld->NextEint() && !Fehler)
        { // alle Eintraege in FakFeld
          if (_tcscmp(FakFeld->GetAkStr().c_str(),FakStr.c_str()) == 0)
          { // Wenn aktueller Faktor in FakFeld
            FakFeld->LoeschAkEint(); // aus FakFeld loeschen
            if (FakFeld1->NeuAkEint())
              Fehler = SolverObj::OutOfMemory;
            FakFeld1->SetAkStr(FakStr.c_str()); // in FakFeld1 kopieren
          }
        }
      }
      FakFeld->ResetKette(); // FakFeld mit nicht gefundenen Eintraegen loeschen
      MGStrObj *HiFakFeld = FakFeld; // FakFeld1 und FakFeld tauschen,
      FakFeld = FakFeld1;  // so dass die gefundenen Eintraege nun in FakFeld
      FakFeld1 = HiFakFeld;
    }
  }
  delete FakFeld1;
  return FakFeld;
}

bool SolverObj::GetBruchWert(MGPStr &AkStr,BruchZahl &NeuWert)
{
  int PrsFehler = ParserObj::NoError;
  unsigned int pos = 0;
  if ((AkStr[0] == '-') || (AkStr[0] == '+'))
    pos++;
  if ((AkStr[pos] == ZahlMarker) &&
      ((AkStr.Length() == pos+2) || (AkStr.Length() == pos+5) &&
       (AkStr[pos+2] == '/') && (AkStr[pos+3] == ZahlMarker)))
  { // wenn Summant einzelne numerische Konstante oder Bruch
    unsigned int p2 = 0;
    AbstrFeld.SetAkEint((unsigned char)AkStr[pos+1]);
    Parser->ParseZahl(AbstrFeld.GetAkStr().c_str(),p2,PrsFehler,NeuWert);
    if (AkStr[pos+2] == '/') // wenn Bruch vorhanden
    {
      BruchZahl NennWert;
      p2 = 0;
      AbstrFeld.SetAkEint((unsigned char)AkStr[pos+4]);
      Parser->ParseZahl(AbstrFeld.GetAkStr().c_str(),p2,PrsFehler,NennWert);
      NeuWert.Nenner *= NennWert; // Nenner einfuegen
    }
    if (AkStr[0] == '-')
      NeuWert.Zaehler *= -1;
    return true;
  }
  return false;
}

void SolverObj::BruchOut(BruchZahl Wert,MGPStr &AkStr)
{
  MGPStr ausbuff;
  int f;
  if (Wert.IsBruch())
    Parser->Ausgabe(ausbuff,Wert.Zaehler,ZahlFormatObj::FitForm);
  else
    Parser->Ausgabe(ausbuff,Wert,ZahlFormatObj::FitForm);
  const _TCHAR *tmp = ausbuff.c_str();
  if (tmp[0] == '-')
  { // wenn 1. Zeichen -
    AkStr.AddChar('-');
    tmp++; // Vorzeichen ueberlesen
  }
  AddAbstrString(tmp,f);
  AddMarker(AkStr,ZahlMarker,f); // + Label fuer num. Konstante anhaengen
  if (Wert.IsBruch()) // wenn Bruch, /Nenner ausgeben
  {
    Parser->Ausgabe(ausbuff,Wert.Nenner,ZahlFormatObj::FitForm);
    AkStr.AddChar('/');
    AddAbstrString(ausbuff.c_str(),f);
    AddMarker(AkStr,ZahlMarker,f);
  }
}

void SolverObj::Aufadd(MGPStr &AkStr)
{ // summiert numerische Konstanten auf
  unsigned int p,p1;
  MGPStr SumStr;
  _TCHAR RechZei;
  BruchZahl AkWert, NeuWert;

  DebugString(_TEXT(" AUF"));
  p = 0;
  while ((p < AkStr.Length()) && !Fehler)
  { // berechnet Werte fr Zahlen und entfernt sie aus AkStr
    p1 = p;
    Summant(AkStr.c_str(),SumStr,p,RechZei);
    if (GetBruchWert(SumStr,NeuWert))
    { // wenn Summant einzelne numerische Konstante oder Bruch
      if (RechZei == '-')
        AkWert = AkWert - NeuWert; // wenn Vorzeichen -, abziehen
      else
        // sonst zu bisherigem Teilergebnis dazuzaehlen
        AkWert = AkWert + NeuWert;
      AkStr.DelNChar(p1,p-p1); // numerische Konstante loeschen in AkStr
      p = p1; // p auf Anfangsposition des letzten Summanten zuruecksetzen
    }
  }
  // hngt Gesamtergebnis an AkStr
  if (AkWert != 0)
  { // kein Rechenfehler, Teilergebnis ungleich 0
    MGPStr ausbuff;
    BruchOut(AkWert,ausbuff);
    if ((ausbuff[0] != '+') && (ausbuff[0] != '-'))
      AkStr.AddChar('+');
    AkStr.AddStr(ausbuff);
  }
}

void SolverObj::Vereinfachen(MGPStr &AkStr)
{ // kuerzt Labels und multipliziert/dividiert numerische Konstanten
  unsigned int p,p1,p2;
  int f,f1,v,PsrFehler;
  MGPStr SumStr,NeuStr,FakStr,HiStr;
  MGStrObj *FakFeld;
  MGPStr HStr1,HStr2;
  _TCHAR RechZei,RechZei1,RechZei2;

  DebugString(_TEXT(" EIN"));
  FakFeld = new MGStrObj;
  PsrFehler = ParserObj::NoError;
  NeuStr.SetStr(_TEXT(""));
  p = 0;
  while ((p < AkStr.Length()) && !Fehler)
  { // alle Zeichen von AkStr, alle Summanten
    FakFeld->ResetKette();
    Summant(AkStr.c_str(),SumStr,p,RechZei);
    p1 = 0;
    while ((p1 < SumStr.Length()) && !Fehler)
    { // alle Faktoren
      NextFakt(SumStr,FakStr,p1); // alle Faktoren von SumStr in FakFeld
      if (FakStr[0] != 0)
      { // alle Faktoren in FakFeld eintragen
        if (FakFeld->NeuAkEint())
          Fehler = SolverObj::OutOfMemory;
        else
          FakFeld->SetAkStr(FakStr.c_str());
      }
    }
    f = 0;
    while ((f < FakFeld->GetMaxEintrag()) && !Fehler)
    { // alle Faktoren bis auf letzten
      f++;
      v = f;
      FakFeld->SetAkEint(f);
      // Sonderfall letzter Faktor ist 1 und weit. Fakt. vorh.
      if ((f > 1) && (f == FakFeld->GetMaxEintrag()) &&
          (FakFeld->GetAkStr()[1] == ZahlMarker) &&
          (FakFeld->GetAkStr()[2] == 1))
        FakFeld->LoeschAkEint();
      else
      while ((v < FakFeld->GetMaxEintrag()) && !Fehler)
      { // alle Faktoren mit Listennummer > f
        v++;
        FakFeld->SetAkEint(f);
        RechZei1 = FakFeld->GetAkStr()[0];
        HStr1.SetStr(&FakFeld->GetAkStr()[1]); // Faktor f in HStr1
        FakFeld->SetAkEint(v);
        RechZei2 = FakFeld->GetAkStr()[0];
        HStr2.SetStr(&FakFeld->GetAkStr()[1]); // Faktor v in HStr2
        if ((_tcscmp(HStr1.c_str(),HStr2.c_str()) == 0) && (RechZei1 != RechZei2))
        { // Division durch gleichen Faktor
          if (HStr1[0] != ZahlMarker)
          { // Einschraenkung des Definitionsbereichs fuer HStr1 ausgeben
            MGPStr tmp;
            tmp.SetStr(HStr1);
            EinschString.AddStr(_TEXT(" ElimSame: "));
            EinschString.AddStr(ReAbstrahieren(tmp));
            EinschString.AddStr(_TEXT(" <> 0\r\n"));
          }
          else
          {
            AbstrFeld.SetAkEint((unsigned char)HStr1[1]);
            if (_tcscmp(AbstrFeld.GetAkStr().c_str(),_TEXT("0")) == 0) // Faktor 0
            {
              Fehler = SolverObj::DivBZero;
              FakFeld->ResetKette();
            }
          }
          FakFeld->SetAkEint(v);
          FakFeld->LoeschAkEint();
          if ((f > 1) || (v <= FakFeld->GetMaxEintrag()) || (v-f > 1))
          { // wenn nicht erster oder letzter Faktor oder Faktoren dazwischen
            //-> andere Faktoren noch vorhanden, krze v und f
            FakFeld->SetAkEint(f);
            FakFeld->LoeschAkEint();
          }
          else
          { // setze auf "1"
            FakFeld->SetAkEint(f);
            // Listennummer fuer '1' eintragen
            FakFeld->GetAkStr().SetStr(_TEXT("*"));
            AddMarker(FakFeld->GetAkStr(),ZahlMarker,1);
          }
          v = f; // neuen f-Faktor wieder mit allen anderen vergleichen
        }
        else
        if ((HStr1[0] == ZahlMarker) && (HStr2[0] == ZahlMarker))
        { // 2 numerische Konstanten gefunden
          BruchZahl Wert1,Wert2;
          AbstrFeld.SetAkEint((unsigned char)HStr1[1]);
          p2 = 0;
          Parser->ParseZahl(AbstrFeld.GetAkStr().c_str(),p2,PsrFehler,Wert1);
          if ((Wert1 == 0) && (PsrFehler == ParserObj::NoError))
          { // wenn ein Faktor 0, lsche alle Faktoren
            if (RechZei1 == '/')
              Fehler = SolverObj::DivBZero; // Fehler wenn /0
            FakFeld->ResetKette();
          }
          else
          if (PsrFehler)
            Fehler = SolverObj::ConstOutOfRange;
          AbstrFeld.SetAkEint((unsigned char)HStr2[1]);
          p2 = 0;
          Parser->ParseZahl(AbstrFeld.GetAkStr().c_str(),p2,PsrFehler,Wert2);
          if ((Wert2 == 0) && (PsrFehler == ParserObj::NoError))
          { // wenn ein Faktor 0, lsche alle Faktoren
            if (RechZei2 == '/')
              Fehler = SolverObj::DivBZero; // Fehler wenn /0
            FakFeld->ResetKette();
          }
          else
          if (PsrFehler)
            Fehler = SolverObj::ConstOutOfRange;
          if (Wert1 && Wert2)
          { // 2 gueltige Werte gefunden, Ergebnis berechnen
            BruchZahl Wert3;
#if defined(__BORLANDC__) && defined(__WIN32__)
            try
            {
#endif
              Parser->BruchMulDiv(Wert1,Wert2,Wert3,
                                  RechZei1 == RechZei2 ? '*' : '/');
#if defined(__BORLANDC__) && defined(__WIN32__)
            }
            __except(EXCEPTION_EXECUTE_HANDLER)
            {
              _fpreset();
              Fehler = SolverObj::ConstOutOfRange;
              FakFeld->ResetKette();
            }
#endif
            MGPStr ausbuff;
            if (Wert3.IsBruch())
            {
              FakFeld->SetAkEint(f);
              Parser->Ausgabe(ausbuff,Wert3.Zaehler,ZahlFormatObj::FitForm);
              FakFeld->GetAkStr().ShortStr(1);
              int f;
              AddAbstrString(ausbuff.c_str(),f);
              AddMarker(FakFeld->GetAkStr(),ZahlMarker,f);
              FakFeld->SetAkEint(v);
              Parser->Ausgabe(ausbuff,Wert3.Nenner,ZahlFormatObj::FitForm);
              if (RechZei1 == '/')
                FakFeld->GetAkStr().SetStr(_TEXT("*"));
              else
                FakFeld->GetAkStr().SetStr(_TEXT("/"));
              AddAbstrString(ausbuff.c_str(),f);
              AddMarker(FakFeld->GetAkStr(),ZahlMarker,f);
            }
            else
            {
              Parser->Ausgabe(ausbuff,Wert3,ZahlFormatObj::FitForm);
              FakFeld->SetAkEint(v);
              FakFeld->LoeschAkEint();
              v--;
              AddAbstrString(ausbuff.c_str(),f1);
              FakFeld->SetAkEint(f);
              HiStr.SetStr(_TEXT(""));
              AddMarker(HiStr,ZahlMarker,f1);
              HiStr.InsChar(0,FakFeld->GetAkStr()[0]);
              FakFeld->SetAkStr(HiStr.c_str());
            }
          }
        }
        else
        if ((HStr1[0] == ZahlMarker) && (HStr1[1] == 1))
        { // HStr1 ist keine Klammer und hat Wert 1 (2. Faktor v vorhanden !)
          FakFeld->SetAkEint(f);
          FakFeld->LoeschAkEint();
          v = f; // neuen f-Faktor wieder mit allen anderen vergleichen
        }
        else
        if ((HStr1[0] == _TCHAR(PotMarker)) || (HStr2[0] == _TCHAR(PotMarker)))
        { // wenn 1 und / od. 2 Potenzausdruecke, pruefen auf gleiche Basis
          MGPStr tmp1,tmp2;
          tmp1 = ReAbstrahieren(HStr1);
          tmp2 = ReAbstrahieren(HStr2);
          unsigned int j1 = 0;
          Parser->AkWert(tmp1,j1,false,false);
          unsigned int j2 = 0;
          Parser->AkWert(tmp2,j2,false,false);
          if ((j1 == j2) &&
              ((tmp1[j1] == '^') || (HStr1[0] != _TCHAR(PotMarker))) &&
              ((tmp2[j2] == '^') || (HStr2[0] != _TCHAR(PotMarker))) &&
              !_tcsncmp(tmp1.c_str(),tmp2.c_str(),j1))
          { // wenn in beiden Ausdruecken gleiche Basis gefunden
            RulesString(_EFT("Combined two exponential expressions with same base (Base^Exp1*Base^Exp2 -> Base^(Exp1+Exp2))"));
            MGPStr HiStr1;
            if (tmp1[j2] == '^')
              HiStr1.SetStr(&tmp1[j1+1]);
            else
              HiStr1.AddStr(_TEXT("1"));
            _TCHAR RechZei3;
            if (RechZei1 == RechZei2)
              RechZei3 = '+';
            else
              RechZei3 = '-';
            if (tmp2[j2] == '^') // wenn Potenz vorhanden
              Add(HiStr1,RechZei3,&tmp2[j2+1]);
            else
            {
              HiStr1.AddChar(RechZei3);
              HiStr1.AddStr(_TEXT("1"));
            }
            Abstrahieren(HiStr1);
            if (RechZei1 == '/') // Ergebnis-Potenz immer mit Multiplikation
              Mult(HiStr1,'*',_TEXT("-")); // wenn /, Vorzeichen "umdrehen"
            OptimizeSubExp(HiStr1);
            HiStr1 = ReAbstrahieren(HiStr1);
            tmp1.ShortStr(j1);
            tmp1.AddChar('^');
            tmp1.AddStr(HiStr1);
            int f1;
            AddAbstrString(tmp1.c_str(),f1);
            FakFeld->SetAkEint(f);
            FakFeld->GetAkStr().SetStr(_TEXT(""));
            FakFeld->GetAkStr().AddChar('*');
            AddMarker(FakFeld->GetAkStr(),PotMarker,f1);
            FakFeld->SetAkEint(v);
            FakFeld->LoeschAkEint();
            v--;
          }
          else
          if ((tmp1[j1] == '^') && (tmp2[j2] == '^') &&
              !_tcscmp(&tmp1[j1+1],&tmp2[j2+1]))
          { // wenn in beiden Ausdruecken gleicher Exponent gefunden
            RulesString(_EFT("Combined two exponential expressions with same exponent (Base1^Exp*Base2^Exp -> (Base1*Base2)^Exp)"));
            MGPStr HiStr1;
            HiStr1.AddNChar(tmp1,j1);
            if (RechZei1 == RechZei2)
              HiStr1.AddChar('*');
            else
              HiStr1.AddChar('/');
            HiStr1.AddNChar(tmp2,j2);
            Abstrahieren(HiStr1);
            OptimizeSubExp(HiStr1);
            HiStr1 = ReAbstrahieren(HiStr1);
            HiStr1.AddChar('^');
            HiStr1.AddStr(&tmp1[j1+1]);
            int f1;
            AddAbstrString(HiStr1.c_str(),f1);
            FakFeld->SetAkEint(f);
            FakFeld->GetAkStr().SetStr(_TEXT(""));
            FakFeld->GetAkStr().AddChar(RechZei1);
            AddMarker(FakFeld->GetAkStr(),PotMarker,f1);
            FakFeld->SetAkEint(v);
            FakFeld->LoeschAkEint();
            v--;
          }
        }
      }
    }
    if (FakFeld->GetMaxEintrag() > 0)
    { // wenn Faktoren vorhanden
      MGPStr tmp;
      tmp.AddChar(RechZei); // Vorzeichen des Summanten eintragen
      FakFeld->Begin();
      while (FakFeld->NextEint()) // alle uebrigen Faktoren eintragen
        tmp.AddStr(FakFeld->GetAkStr());
      FaksToSummant(tmp); // * oder / verarbeiten
      NeuStr.AddStr(tmp);
    }
  }
  AkStr.SetStr(NeuStr); // Ergebnisstring in AkStr kopieren
  delete FakFeld; // Kette loeschen
}

// symbolische Addition von AkStr und AddStr
void SolverObj::Add(MGPStr &AkStr,_TCHAR AddZei,const _TCHAR *AddStr)
{
  if ((AddStr[0] == '-') || (AddStr[0] == '+'))
  { // wenn Potenz mit Vorzeichen, mit RechZei3 verrechnen
    AddZei = MultVorZei(AddZei,AddStr[0]);
    AddStr++;
  }
  AkStr.AddChar(AddZei);
  AkStr.AddStr(AddStr);
}

// symbolische Multiplikation von AkStr und MuStr
void SolverObj::Mult(MGPStr &AkStr,_TCHAR MultZei,const _TCHAR *MuStr)
{
  MGPStr NeuStr,SumStr;
  _TCHAR RechZei1,RechZei2;

  DebugString(_TEXT(" MUL"));
  NeuStr.SetStr(_TEXT(""));
  unsigned int p = 0;
  MGPStr SumStr1;
  size_t mulen = _tcslen(MuStr);
  while ((p < mulen) && !Fehler)
  {
    if (MultZei == '*')
      Summant(MuStr,SumStr1,p,RechZei2);
    else
    { // Bei Division muss immer durch ganzen MuStr geteilt werden
      SumStr1.SetStr(MuStr);
      if (!IstEinSummant(SumStr1.c_str(),RechZei2))
      {
        SumStr1.InsChar(0,'(');
        SumStr1.AddChar(')');
        RechZei2 = 0;
      }
      else
      if (RechZei2)
        SumStr1.DelNChar(0,1);
      p = mulen;
    }
    unsigned int p1 = 0;
    while ((p1 < AkStr.Length()) && !Fehler)
    { // alle Zeichen von Akstr
      Summant(AkStr.c_str(),SumStr,p1,RechZei1);
      RechZei1 = MultVorZei(RechZei1,RechZei2);
      NeuStr.AddChar(RechZei1); // Rechenzeichen
      NeuStr.AddStr(SumStr); // und Summanten in NeuStr
      if (_tcscmp(MuStr,_TEXT("-")) != 0)
      { // wenn nicht mit - multipliziert wird (nicht nur Vorzeichen ausm.)
        unsigned int p2 = 0;
        while (p2 < SumStr1.Length())
        {
          MGPStr FakStr;
          NextFakt(SumStr1,FakStr,p2);
          if (MultZei != FakStr[0]) // Multiplikationszeichen anhaengen
            NeuStr.AddChar('/');
          else
            NeuStr.AddChar('*');
          NeuStr.AddStr(&FakStr[1]); // Multiplikationsstring anhaengen
        }
      }
    }
  }
  Vereinfachen(NeuStr); // Ausdruck vereinfachen
  AkStr.SetStr(NeuStr); // Ergebnis in AkStr
}

bool SolverObj::EntfDiv(MGPStr &LAbsStr,MGPStr &RAbsStr,const _TCHAR *DivStr)
{ // beide Seiten mit DivStr multiplizieren, um Division zu entfernen
  bool ED;

  DebugString(_TEXT(" DIV"));
  ED = false;
  MGPStr TstStr;
  TstStr.SetStr(_TEXT("/"));
  TstStr.AddStr(DivStr); // /DivStr, zum kuerzen
  if (DivStr[0] != 0)
  { // wenn DivStr vorhanden
    for (int i = 1;i <= 2;i++)
    {
      const _TCHAR *AkStr;
      if (i == 1)
        AkStr = LAbsStr.c_str();
      else
        AkStr = RAbsStr.c_str();
      MGPStr SumStr,NeuAkStr;
      unsigned int ps = 0;
      _TCHAR RechZei;
      size_t aklen = _tcslen(AkStr);
      while ((ps < aklen) && !Fehler)
      { // alle Summanten
        Summant(AkStr,SumStr,ps,RechZei);
        bool gef = EntfSuchStr(SumStr,TstStr);
        ED |= gef;
        NeuAkStr.AddChar(RechZei);
        NeuAkStr.AddStr(SumStr);
        if (!gef)
        { // nicht gefunden, mit DivStr multiplizieren
          NeuAkStr.AddStr(_TEXT("*"));
          NeuAkStr.AddStr(DivStr);
        }
      }
      if (NeuAkStr[0] == 0)
        AddMarker(NeuAkStr,ZahlMarker,1); // wenn Ergebnis leer, auf 1 setzen
      if (i == 1)
        LAbsStr.SetStr(NeuAkStr);
      else
        RAbsStr.SetStr(NeuAkStr);
    }
  }
  if (ED)
  {
    MGPStr tmp2;
    tmp2.SetStr(DivStr);
    EinschString.AddStr(_TEXT(" ElimDiv: "));
    EinschString.AddStr(ReAbstrahieren(tmp2));
    EinschString.AddStr(_TEXT(" <> 0\r\n"));
  }
  return ED;
}

void SolverObj::SucheDiv(MGPStr &AkStr,MGPStr &DivStr)
{ // Division suchen, 1.Faktor mit / zurueckgeben
  unsigned int p,p1;
  _TCHAR RechZei;
  MGPStr SumStr;

  DebugString(_TEXT(" SDV"));
  DivStr.SetStr(_TEXT(""));
  p = 0;
  while ((p < AkStr.Length()) && (DivStr[0] != '/') && !Fehler)
  { // alle Zeichen von AkStr
    Summant(AkStr.c_str(),SumStr,p,RechZei);
    p1 = 0;
    while ((p1 < SumStr.Length()) && (DivStr[0] != '/') && !Fehler)
    { // alle Faktoren des aktuellen Summanten auf Faktor mit / untersuchen
      NextFakt(SumStr,DivStr,p1);
    }
  }
  if (DivStr[0] != '/')
    DivStr.SetStr(_TEXT(""));
}

void SolverObj::Kuerzen(MGPStr &LAbsStr,MGPStr &RAbsStr)
{ // kuerzt Faktoren, die in allen Summanten vorkommen
  const _TCHAR *SumStr;
  _TCHAR RechZei;

  DebugString(_TEXT(" KZN"));
  MGStrObj *FakFeld = SucheFak(LAbsStr,RAbsStr); // alle gem. Faktoren holen
  FakFeld->Begin();
  while (FakFeld->NextEint() && !Fehler)
  { // alle gefundenen gemeinsamen Faktoren bearbeiten
    SumStr = FakFeld->GetAkStr().c_str(); // holt gemeinsamen Faktor
    RechZei = SumStr[0]; // Rechenzeichen merken
    if (RechZei == '*')
      RechZei = '/';
    else
      RechZei = '*'; // Rechenzeichen 'umdrehen'
    if (SumStr[2] != 1) // wenn nicht 1
    { // L und R mit Faktor multiplizieren und kuerzen lassen
      Mult(RAbsStr,RechZei,&SumStr[1]);
      Mult(LAbsStr,RechZei,&SumStr[1]);
    }
  }
  delete FakFeld;
}

bool SolverObj::EntfSuchStr(MGPStr &AStr,MGPStr &SStr)
{ // kuerzt Faktor SStr aus AStr (SuchStr: (* od /)Faktor)
  unsigned int p = 0;
  MGPStr FakStr;
  bool gef = false;
  MGPStr NeuSumStr;
  while ((p < AStr.Length()) && !Fehler)
  { // alle Faktoren durchsuchen
    NextFakt(AStr,FakStr,p);
    if (!gef && (_tcscmp(FakStr.c_str(),SStr.c_str()) == 0))
      gef = true; // gesuchten Faktor nicht eintragen
    else
      NeuSumStr.AddStr(FakStr);
  }
  FaksToSummant(NeuSumStr); // / oder * bearbeiten
  if (NeuSumStr[0] == 0)
    AddMarker(NeuSumStr,ZahlMarker,1); // wenn leer, auf 1
  if (gef)
    AStr.SetStr(NeuSumStr);
  return gef;
}

// Fr jeden Summanten alle zug. Faktoren in neuen Eintrag von FakFeld
// (aufsteigend sortiert)
void SolverObj::SumNFaktor(MGPStr &AkStr,MGStrObj *FakFeld,bool addone)
{
  _TCHAR RechZei1;
  MGPStr SumStr,FakStr;
  MGStrObj KlStrs;
  int i,f;
  unsigned int p,p1;

  DebugString(_TEXT(" SUF"));
  if (addone)
    i = 0;
  else
    i = 1; // alle Faktoren in AbstrFeld auer '1'
  bool Erster = true;

  // do-Schleife, damit sicher fuer jeden Summanten FakFeld-Eintrag
  // sonst werden "+-1" geloescht wenn AbstrFeld.GetMaxEintrag() == 1 !
  do
  {
    i++;
    f = 0; // Summanten-Zhler
    p = 0;
    // Summanten die Faktoren zuordnen, die in ihnen vorkommen
    while ((p < AkStr.Length()) && !Fehler)
    {
      f++;
      Summant(AkStr.c_str(),SumStr,p,RechZei1);
      if (FakFeld->GetMaxEintrag() < f) // 1. Durchlauf
      {
        while (FakFeld->GetMaxEintrag() < f)
          FakFeld->NeuAkEint();
        if (RechZei1 == 0)
          FakFeld->GetAkStr().AddChar('+');
        else
          FakFeld->GetAkStr().AddChar(RechZei1);
      }
      else
        FakFeld->SetAkEint(f);
      p1 = 0;
      while ((p1 < SumStr.Length()) && !Fehler)
      {
        NextFakt(SumStr,FakStr,p1);
        if (FakStr[1] == '(') // Klammerausdruecke zwischenspeichern
        {
          if (Erster)
          {
            while (KlStrs.GetMaxEintrag() < f)
              KlStrs.NeuAkEint();
            KlStrs.SetAkEint(f);
            KlStrs.GetAkStr().AddStr(FakStr);
          }
        }
        else
        if (FakStr[2] == i) // Wenn aktueller Faktor gefunden
          FakFeld->GetAkStr().AddStr(FakStr); // Rechenzeichen + Faktor anhngen
      }
    }
    Erster = false;
  }
  while ((i < AbstrFeld.GetMaxEintrag()) && !Fehler);
  if (KlStrs.GetMaxEintrag() > 0)
  {
    KlStrs.Begin();
    while (KlStrs.NextEint())
    {
      FakFeld->SetAkEint(KlStrs.AkEintNummer());
      FakFeld->GetAkStr().AddStr(KlStrs.GetAkStr());
    }
  }
}

// Sucht nach Potenzen und macht aus a*a*a -> a^3
void SolverObj::MakePotenzen(MGPStr &AkStr)
{
  DebugString(_TEXT(" MKP"));
  MGPStr SumStr,FakStr1,FakStr2,ErgStr;
  _TCHAR RechZei;
  unsigned int p = 0;
  while (!Fehler && (p < AkStr.Length()))
  {
    Summant(AkStr.c_str(),SumStr,p,RechZei);
    ErgStr.AddChar(RechZei);
    MGPStr FaksStr;
    unsigned int p1 = 0;
    while (!Fehler && (p1 < SumStr.Length()))
    {
      NextFakt(SumStr,FakStr1,p1);
      if (FakStr1[1] == '(')
      { // fuer Klammerausdruecke rekursiver Aufruf fuer Klammerausdruck
        MGPStr tmp;
        tmp.AddStr(&FakStr1[2]);
        tmp.ShortStr(FakStr1.Length()-3);
        MakePotenzen(tmp);
        FakStr1.ShortStr(2);
        FakStr1.AddStr(tmp);
        FakStr1.AddStr(_TEXT(")"));
      }
      int count = 1;
      unsigned int p2 = p1;
      while (!Fehler && (p2 < SumStr.Length()))
      {
        unsigned int p3 = p2;
        NextFakt(SumStr,FakStr2,p2);
        if (!_tcscmp(FakStr1.c_str(),FakStr2.c_str()))
        {
          SumStr.DelNChar(p3,FakStr2.Length());
          count++;
          p2 = p3;
        }
      }
      if (count > 1)
      { // wenn mehrere gleiche Faktoren, Potenz-Ausdruck eintragen
        MGPStr tmp;
        tmp.SetStr(&FakStr1[1]);
        tmp = ReAbstrahieren(tmp);
        tmp.AddStr(_TEXT("^"));
        tmp.AddInt(count);
        int f;
        AddAbstrString(tmp.c_str(),f);
        FaksStr.AddChar(FakStr1[0]);
        AddMarker(FaksStr,PotMarker,f);
      }
      else
        FaksStr.AddStr(FakStr1);
    }
    FaksToSummant(FaksStr);
    ErgStr.AddStr(FaksStr);
  }
  AkStr.SetStr(ErgStr);
}

// Krzt gleiche Summanten mit untersch. Vorzeichen,
// fasst gleiche Summanten mit gl. Vorz. zusammen
void SolverObj::SumKuerzen(MGPStr &AkStr)
{
  int i,f,t,c;
  const _TCHAR *Pos,*Pos1;
  MGStrObj *FakFeld;
  _TCHAR Zeile[10];

  DebugString(_TEXT(" SKZ"));
  // evtl. Vorzeichen vor Faktoren entfernen (5*-4), sonst Fehler in SumNFaktor
  EntfVorzeichen(AkStr);
  FakFeld = new MGStrObj;
  SumNFaktor(AkStr,FakFeld); // Summanten mit zug. Faktoren ermitteln (sortiert)
  i = 0;
  while ((i < FakFeld->GetMaxEintrag()) && !Fehler)
  { // zuerst gleiche Summanten mit unt. Vorzeichen kuerzen
    i++;
    f = i;
    FakFeld->SetAkEint(i);
    Pos = FakFeld->GetAkStr().c_str();
    while ((f < FakFeld->GetMaxEintrag()) && !Fehler)
    { // alle nachfolgenden Summanten
      f++;
      FakFeld->SetAkEint(f);
      Pos1 = FakFeld->GetAkStr().c_str();
      // wenn gleiche Faktoren, aber unt. Vorzeichen
      if ((Pos[0] != Pos1[0]) && (_tcscmp(Pos+1,Pos1+1) == 0))
      { // beide Summanten loeschen
        FakFeld->LoeschAkEint();
        FakFeld->SetAkEint(i);
        FakFeld->LoeschAkEint();
        i--; // beim n. Durchlauf nochmal die gleiche Nummer (wg. loeschen)
        f = FakFeld->GetMaxEintrag(); // Ende der f-Schleife
      }
    }
  }
  i = 0;
  while ((i < FakFeld->GetMaxEintrag()) && !Fehler)
  { // dann gleiche Summanten mit gleichem Vorzeichen zusammenfassen
    i++;
    f = i;
    FakFeld->SetAkEint(i);
    Pos = FakFeld->GetAkStr().c_str();
    c = 1;
    while ((f < FakFeld->GetMaxEintrag()) && !Fehler)
    {
      f++;
      FakFeld->SetAkEint(f);
      Pos1 = FakFeld->GetAkStr().c_str();
      // wenn gleiche Faktoren mit gleichen Vorz.
      if ((Pos[0] == Pos1[0]) && (_tcscmp(Pos+1,Pos1+1) == 0))
      {
        c++; // Anzahl gleicher Summanten ist 2
        f--; // Naechster Durchlauf nochmal mit gleichem Index (wg. loeschen)
        FakFeld->LoeschAkEint();
      }
    }
    if (c > 1)
    { // mehrfache Summanten gefunden
      FakFeld->SetAkEint(i); // auf 1. Eintrag mit mehrfachen Summanten setzen
      _stprintf(Zeile,_TEXT("%i"),c);
      AddAbstrString(Zeile,t); // neuer Eintrag in AbstrFeld
      MGPStr tmp;
      tmp.SetStr(_TEXT("*"));
      AddMarker(tmp,ZahlMarker,t); // neuer Marker in tmp
      FakFeld->GetAkStr().AddStr(tmp); // Zaehler eintragen
    }
  }
  FakFeld->Begin();
  AkStr.SetStr(_TEXT(""));
  while (FakFeld->NextEint() && !Fehler) // AkStr wieder zusammensetzen
  {
    Pos = FakFeld->GetAkStr().c_str();
    if (*(Pos+1) == 0)
    { // String enthaelt nur Vorzeichen -> Eintrag ist '1'
      AkStr.AddChar(Pos[0]); // Vorzeichen eintragen
      AddMarker(AkStr,ZahlMarker,1); // '1' eintragen
    }
    else
    {
      FaksToSummant(FakFeld->GetAkStr()); // * oder / bearbeiten
      AkStr.AddStr(FakFeld->GetAkStr());
    }
  }
  delete FakFeld;
}

bool SolverObj::IstEinSummant(const _TCHAR *AkStr,_TCHAR &RechZei)
{
  unsigned int p = 0;
  MGPStr SumStr;
  Summant(AkStr,SumStr,p,RechZei);
  return (_tcscmp(&AkStr[RechZei ? 1 : 0],SumStr.c_str()) == 0);
}

void SolverObj::MakeFaktor(MGPStr &AkStr)
{
  _TCHAR RechZei;
  bool tst = IstEinSummant(AkStr.c_str(),RechZei);
  if (AkStr[0] == '+')
    AkStr.DelNChar(0,1);
  if (!tst || (RechZei == '-'))
  {
    AkStr.InsStr(0,_TEXT("*("));
    AkStr.AddChar(')');
  }
  else
    AkStr.InsChar(0,'*');
}

void SolverObj::Ausklammern(MGPStr &AkStr)
{ // klammert mehrfach vorkommende Faktoren(gruppen) aus
  MGPStr FakStr,FakStr1,NeuStr,SumStr,SumStr1,KlStr,MaxFakStr,MaxZahlStr;
  _TCHAR RechZei1,RechZei2,RechZei3;
  MGStrObj FakFeld1;
  unsigned int p2,p4;
  int f1,f;
  int AkMaxFak,MaxFak;
  int MaxZahl;

  DebugString(_TEXT(" AUS"));
  NeuStr.SetStr(_TEXT(""));
  FakStr.SetStr(_TEXT(""));
//  EntfVorzeichen(AkStr);
  SumNFaktor(AkStr,&FakFeld1,true);
  bool Ausgeklammert = true;
  while (Ausgeklammert && !Fehler)
  { // solange Ausgeklammert
    Ausgeklammert = false;
    MaxFakStr.SetStr(_TEXT(""));
    MaxZahlStr.SetStr(_TEXT(""));
    MGPStrSet Faks; // Speichert bereits gefundene Faktoren
    ShortArrObj AkMaxFakSums,MaxFakSums,MaxZahlSums;
    f = 1;
    MaxFak = 0;
    MaxZahl = 0;
    while ((f <= FakFeld1.GetMaxEintrag()) && !Fehler)
    { // alle Summanten auswerten
      FakFeld1.SetAkEint(f);
      SumStr.SetStr(&FakFeld1.GetAkStr()[1]);
      p2 = 0;
      while ((p2 < SumStr.Length()) && !Fehler)
      { // ermittelt den am hufigsten vorkommenden Faktor im Summanten SumStr
        NextFakt(SumStr,FakStr,p2);
        if (Faks.find(FakStr) == Faks.end())
        { // diesen Faktor noch nicht gefunden
          AkMaxFak = 0;
          AkMaxFakSums.Clear();
          AkMaxFakSums.SetValue(0,(short)f);
          bool inzuslabels = false;
          if (FakStr.Length() == 3)
          { // wenn kein Klammerausdruck
            // AbstrFeld auf String dieses Faktors setzen
            AbstrFeld.SetAkEint((unsigned char)FakStr[2]);
            inzuslabels = InZusLabels(AbstrFeld.GetAkStr().c_str());
          }
          if (!inzuslabels) // String nicht in ZusLabels
          {
            Faks.insert(FakStr);
            f1 = f+1;
            while ((f1 <= FakFeld1.GetMaxEintrag()) && !Fehler)
            { // mit allen Faktoren der restlichen Summanten vergleichen
              FakFeld1.SetAkEint(f1);
              SumStr1.SetStr(&FakFeld1.GetAkStr()[1]);
              p4 = 0;
              bool gef = false;
              while (!gef && (p4 < SumStr1.Length()) && !Fehler)
              { // alle Faktoren
                NextFakt(SumStr1,FakStr1,p4);
                if ((_tcscmp(FakStr.c_str(),FakStr1.c_str()) == 0))
                { // wenn Faktor identisch mit aktuellem Faktor in SumStr
                  gef = true;
                  AkMaxFak++; //erhhe Anzahl-Zhler
                  AkMaxFakSums.SetValue(AkMaxFak,(short)f1);
                }
              }
              f1++;
            }
          }
          if (FakStr[1] == ZahlMarker)
          { // Zahlen separat zaehlen
            if ((FakStr[2] != 1) && (AkMaxFak > MaxZahl))
            { // am haeuf. vork. Zahl speichern, aber keine 1 ausklammern !
              MaxZahlStr.SetStr(FakStr); // aktuellen Faktor speichern
              MaxZahl = AkMaxFak;
              MaxZahlSums = AkMaxFakSums;
            }
          }
          else
          if (AkMaxFak > MaxFak)
          { // Faktor ausklammern, der am haeufigsten vorkommt
            MaxFakStr.SetStr(FakStr); // aktuellen Faktor speichern
            MaxFak = AkMaxFak;
            MaxFakSums = AkMaxFakSums;
          }
        }
      }
      f++;
    }
    KlStr.SetStr(_TEXT(""));
    if ((MaxFak == 0) && (MaxZahl > 0))
    { // wenn keine Faktoren gefunden, evtl. Zahl ausklammern
      MaxFak = MaxZahl;
      MaxFakStr.SetStr(MaxZahlStr);
      MaxFakSums = MaxZahlSums;
    }
    if (MaxFak > 0)
    { // mehrere Summanten enthalten FakStr
      FakStr.SetStr(MaxFakStr);
      int i;
      for (i = 0;i <= MaxFak;i++)
      { // FakStr aus allen Summanten, in denen er vorkommt, loeschen
        FakFeld1.SetAkEint(MaxFakSums[i]);
        MGPStr tmp1;
        RechZei1 = FakFeld1.GetAkStr()[0];
        tmp1.SetStr(&FakFeld1.GetAkStr()[1]);
        if (EntfSuchStr(tmp1,FakStr))
        { // wenn FakStr im akuellen SumStr und aus diesem erfolgreich geloescht
          KlStr.AddChar(RechZei1);
          KlStr.AddStr(tmp1);
        }
        else
          Fehler = SolverObj::Syntax;
      }
      for (i = MaxFak;i >= 0;i--)
      { // Summanten mit FakStr loeschen
        FakFeld1.SetAkEint(MaxFakSums[i]);
        FakFeld1.LoeschAkEint();
      }
    }
    if ((KlStr[0] != 0) && !Fehler)
    {
      if (KlStr[0] == '+')
        KlStr.DelNChar(0,1);
      Vereinfachen(KlStr);
      Aufadd(KlStr);
      if (KlStr.Length() > 3) // wenn mehrere Faktoren
      {
        int ret = TermDivision(KlStr,FakFeld1);
        if (ret)
        {
          MGPStr tmp; // *( FakStr +- 1)
          FaksToSummant(FakStr);
          Vereinfachen(FakStr);
          tmp.AddStr(FakStr);
          if (ret == -1)
            tmp.AddChar('-');
          else
            tmp.AddChar('+');
          AddMarker(tmp,ZahlMarker,1);
          Ausklammern(tmp);
          MakeFaktor(tmp);
          FakStr.SetStr(tmp);
        }
        Ausklammern(KlStr); // rek. Aufruf fr Restausdruck
        MGPStr RestStr;
        do
        {
          ret = false;
          if (KlStr.Length() > 0)
          {
            MGPStr ZaehlRestStr;
            ret = TeilNennStrDivision(KlStr,FakFeld1,RestStr,ZaehlRestStr);
            if (ret)
            {
              MGPStr tmp; // *( RestStr */ FakStr +- ZaehlRestStr)
              if (RestStr.Length() == 0)
                FaksToSummant(FakStr);
              else
                tmp.AddStr(RestStr);
              tmp.AddStr(FakStr);
              tmp.AddStr(ZaehlRestStr);
              Vereinfachen(tmp);
              Ausklammern(tmp);
              MakeFaktor(tmp);
              FakStr.SetStr(tmp);
            }
          }
          if (KlStr.Length() > 0)
          {
            ret = TeilZaehlSumsDivision(KlStr,FakFeld1,RestStr);
            if (ret)
            {
              MGPStr tmp; // *( FakStr +- RestStr)
              FaksToSummant(FakStr);
              Vereinfachen(FakStr);
              tmp.AddStr(FakStr);
              tmp.AddStr(RestStr);
              Ausklammern(tmp);
              MakeFaktor(tmp);
              FakStr.SetStr(tmp);
            }
          }
        }
        while (ret && !Fehler);
      }
      if (KlStr[0] != 0)
      { // wenn was uebrig
        Vereinfachen(KlStr);
        const _TCHAR *kltmp = KlStr.c_str();
        if (IstEinSummant(kltmp,RechZei3))
        { // nur 1 Summant
          if (RechZei3)
            kltmp++;
          if (!RechZei3)
            RechZei3 = '+';
          RechZei2 = 0;
        }
        else // mehrere Summanten
        {
          if (*kltmp == '+')
            kltmp++;
          RechZei3 = '+';
          RechZei2 = '(';
        }
        NeuStr.SetStr(_TEXT(""));
        NeuStr.AddChar(RechZei3);
        NeuStr.AddChar('*');
        if (RechZei2 == '(')
          NeuStr.AddStr(_TEXT("("));
        NeuStr.AddStr(kltmp);
        if (RechZei2 == '(')
          NeuStr.AddStr(_TEXT(")"));
        NeuStr.AddStr(FakStr);
        if (FakFeld1.NeuAkEint())
          Fehler = SolverObj::OutOfMemory;
        FakFeld1.SetAkStr(NeuStr.c_str());
      }
      Ausgeklammert = true;
    }
  }
  AkStr.SetStr(_TEXT(""));
  FakFeld1.Begin();
  while (FakFeld1.NextEint() && !Fehler)
  {
    FaksToSummant(FakFeld1.GetAkStr());
    AkStr.AddStr(FakFeld1.GetAkStr());
  }
}

// Zerlegt NennStr in Faktoren, prueft ob Klammerausdruck und
// ruft dann TeilZaehlSumsDivision auf
int SolverObj::TeilNennStrDivision(MGPStr &NennStr,MGStrObj &ZaehlSums,
                                   MGPStr &RestStr,MGPStr &ZaehlRestStr)
{
  unsigned int p = 0;
  _TCHAR RechZei;
  if (!IstEinSummant(NennStr.c_str(),RechZei))
    return 0; // nur 1 Summant erlaubt !

  if (RechZei)
    p++;
  RestStr.SetStr(_TEXT(""));
  int ret = 0;
  MGPStr NeuNennStr;
  while (!Fehler && (p < NennStr.Length()))
  {
    MGPStr FakStr;
    NextFakt(NennStr,FakStr,p);
    if (FakStr[1] == '(' && !ret)
    {
      MGPStr tmp = &FakStr[2]; // Klammern entfernen
      tmp.ShortStr(tmp.Length()-1);
      ret = TermDivision(tmp,ZaehlSums);
      ZaehlRestStr.SetStr(_TEXT(""));
      if (ret) // 1 ganzer ZaehlSums ausgeklammert
      {
        ZaehlRestStr.AddChar(ret == -1 ? '-' : '+');
        AddMarker(ZaehlRestStr,ZahlMarker,1);
      }
      else
        ret = TeilZaehlSumsDivision(tmp,ZaehlSums,ZaehlRestStr);
      if (ret)
        NeuNennStr.SetStr(FakStr);
      else
        RestStr.AddStr(FakStr);
    }
    else
      RestStr.AddStr(FakStr);
  }
  if (ret)
  {
    if (RestStr.Length() > 0)
      FaksToSummant(RestStr);
    FaksToSummant(NeuNennStr);
    NennStr.SetStr(NeuNennStr);
    return ret;
  }
  return 0;
}

// Zerlegt alle Summanten in ZaehlSums in die Faktoren,
// prueft ob Klammerausdruck und ruft dann TermDivison fuer
// den Klammerausdruck auf
int SolverObj::TeilZaehlSumsDivision(MGPStr &NennStr,MGStrObj &ZaehlSums,
                                  MGPStr &RestStr)
{
  ZaehlSums.Begin();
  while (!Fehler && ZaehlSums.NextEint())
  {
    unsigned int p = 1;
    MGPStr &AkStr = ZaehlSums.GetAkStr();
    RestStr.SetStr(_TEXT(""));
    RestStr.AddChar(AkStr[0]); // Vorzeichen von ZaehlSums eintragen
    int ret = 0;
    while (!Fehler && (p < AkStr.Length()))
    {
      MGPStr FakStr;
      NextFakt(AkStr,FakStr,p);
      if ((FakStr[1] == '(') && !ret) // Faktor mit Klammern
      {
        MGPStr tmp;
        tmp.SetStr(&FakStr[2]); // Klammern entfernen
        tmp.ShortStr(tmp.Length()-1);
        MGStrObj UntZaehlSums;
        SumNFaktor(tmp,&UntZaehlSums,true);
        ret = TermDivision(NennStr,UntZaehlSums,true);
        if (!ret)
          RestStr.AddStr(FakStr);
      }
      else
        RestStr.AddStr(FakStr);
    }
    if (ret)
    {
      FaksToSummant(RestStr);
      if (ret == -1) // Vorzeichen vor ZaehlSums verrechnen
        RestStr.ReplChar(0,MultVorZei('-',RestStr[0]));
      if (RestStr.Length() == 1) // falls kein weiterer Faktor, 1 eintragen
        AddMarker(RestStr,ZahlMarker,1);
      ZaehlSums.LoeschAkEint(); // Diesen Summant loeschen
      return ret;
    }
  }
  return 0;
}

// Dividiert ZaehlSums durch NennStr
// Ergebnis ist != 0 wenn ein ZaehlSums-Eintrag
// kompletten NennStr (und evtl. mehr) enthaelt
int SolverObj::TermDivision(MGPStr &NennStr,MGStrObj &ZaehlSums,bool norest)
{
  DebugString(_TEXT(" PDV"));
  MGStrObj NennSums;
  ShortArrObj GemSums,ZaehlGemSums;
  SumNFaktor(NennStr,&NennSums,true); // sortierte Summanten ermitteln
  ZaehlSums.Begin();
  // gleiche Summanten in Zaehler und Nenner finden
  while (!Fehler && ZaehlSums.NextEint())
  {
    NennSums.Begin();
    while (!Fehler && NennSums.NextEint()) // alle Summanten des Nenners
    {
      if ((_tcscmp(&ZaehlSums.GetAkStr()[1],&NennSums.GetAkStr()[1]) == 0) &&
          (GemSums[NennSums.AkEintNummer()-1] == 0))
      { // wenn gleich mit Zaehler und noch nicht verwendet
        GemSums.SetValue(NennSums.AkEintNummer()-1,
                         (ZaehlSums.GetAkStr()[0] != NennSums.GetAkStr()[0] ?
                         -1 : 1));
        ZaehlGemSums.SetValue(ZaehlSums.AkEintNummer()-1,1);
        break;
      }
    }
  }
  int Erstwert = GemSums[0];
  bool gef = (Erstwert != 0) &&
             ((int)GemSums.AnzInts() == NennSums.GetMaxEintrag());

  if (gef) // testen ob alle Summanten im Nenner gefunden wurden
    for (int i = 0;i < NennSums.GetMaxEintrag();i++)
      if (GemSums[i] != Erstwert)
        gef = false;

  if (gef && norest) // testen ob alle Summanten im Zaehler gefunden wurden
    for (int i = 0;i < ZaehlSums.GetMaxEintrag();i++)
      if (ZaehlGemSums[i] != 1)
        gef = false;

  if (gef)
  {
    // dividierte Eintraege aus ZaehlSums loeschen
    for (int i = ZaehlSums.GetMaxEintrag()-1;i >= 0;i--)
    {
      if (ZaehlGemSums[i] != 0)
      {
        ZaehlSums.SetAkEint(i+1);
        ZaehlSums.LoeschAkEint();
      }
    }
    return Erstwert;
  }
  else
    return 0;
}

// Gleichung umstellen nach dem gesuchten Ausdruck
//
// Voraussetzung: KEINE Vorzeichen vor Faktoren (*-5)
//
void SolverObj::Isolieren(MGPStr &LAbsStr,MGPStr &RAbsStr,
                          MGPStr &SuchStr,bool Teilen)
{
  MGPStr NennStr,ZaehlStr,AkStr,SumStr,ErgLStr,HiStr,PotStr;
  _TCHAR RechZei,RechZei1;
  int f1,PotAnz = 0;
  unsigned int p,p1;
  int i,Anz;
  bool DivSuch = false,ErstSuch = false;

  DebugString(_TEXT(" ISO"));
  for (i = 1;i <= 2;i++)
  { // LAbsStr und RAbsStr nacheinander bearbeiten
    if (i == 1)
      AkStr.SetStr(LAbsStr);
    else
      AkStr.SetStr(RAbsStr);
    p = 0;
    while ((p < AkStr.Length()) && !Fehler)
    { // alle Summanten des aktuellen AkStr
      Summant(AkStr.c_str(),SumStr,p,RechZei);
      p1 = 0;
      ErgLStr.SetStr(SuchStr);
      // fr RAbsStr Vorzeichen umdrehen
      RechZei1 = MultVorZei(RechZei,i == 2 ? '-' : '+');
      p1 = 0;
      HiStr.SetStr(_TEXT(""));
      HiStr.AddChar(RechZei1);
      Anz = 0;
      while ((p1 < SumStr.Length()) && !Fehler)
      { // alle Faktoren bearbeiten und SuchStr suchen
        MGPStr FStr;
        NextFakt(SumStr,FStr,p1);
        unsigned int PotCount = 0;
        if (FStr[1] == _TCHAR(PotMarker))
        { // wenn Potenzausdruck
          MGPStr tmp;
          AbstrFeld.SetAkEint((unsigned char)FStr[2]);
          tmp.SetStr(AbstrFeld.GetAkStr());
          unsigned int potpos = 0;
          Parser->AkWert(tmp,potpos,false,false);
          AbstrFeld.SetAkEint((unsigned char)SuchStr[1]);
          if ((tmp[potpos] == '^') &&
              (potpos == AbstrFeld.GetAkStr().Length()) &&
              !_tcsncmp(AbstrFeld.GetAkStr().c_str(),tmp.c_str(),potpos))
          { // SuchStr ist Basis
            int PrsFehler = ZahlFormatObj::NoError;
            unsigned int p2 = 0;
            double Wert = Parser->Zahl(&tmp[potpos+1],p2,PrsFehler);
            if ((p2+potpos+1 == tmp.Length()) &&
                (PrsFehler == ZahlFormatObj::NoError) &&
                (Wert > 0) && (Wert <= 3) && (fmod(Wert,2) == 0))
            { // Exponent ist Zahl 1,2,3
              PotCount = int(Wert);
            }
          }
        }
        if (!PotCount)
        {
          if (FStr[2] != SuchStr[1]) // Nicht SuchStr
            HiStr.AddStr(FStr); // Faktor an HiStr anhngen
          else
            PotCount = 1; // Wenn SuchStr
        }
        if (PotCount)
        {
          if (!Anz)
            ErgLStr.SetStr(SuchStr); // Gesuchter Ausdruck gefunden
          Anz += PotCount;
          if ((FStr[0] == '/') && (ErstSuch == false))
            DivSuch = true; // wenn durch SuchStr geteilt wird
          else
          if ((FStr[0] == '/') && (DivSuch == false) ||
              (FStr[0] == '*') && (DivSuch == true))
          {
            Fehler = SolverObj::QuadIntr;
            return;
          }  
          ErstSuch = true;
        }
      }
      if (HiStr.Length() == 1) // nur SuchStr oder Potenz davon in Summant
        AddMarker(HiStr,ZahlMarker,1);
      FaksToSummant(HiStr); // * oder / bearbeiten
      if (Anz == 0)
        ZaehlStr.AddStr(HiStr);
      else
      if (Anz == 1)
      {
        HiStr.ReplChar(0,MultVorZei(HiStr[0],'-'));
        NennStr.AddStr(HiStr);
      }
      else // Mehrfaches Vorkommen
      if ((PotAnz == 0) || (PotAnz == Anz)) // nur 1 Potenztyp erlaubt
      {
        PotStr.AddStr(HiStr);
        PotAnz = Anz;
        if (HiStr.Length() == 1) // wenn nur Vorzeichen
          AddMarker(PotStr,ZahlMarker,1); // 1 anhngen
      }
      else
        return;
    }
  }
  // wenn durch SuchStr geteilt wird ZaehlStr und NennStr tauschen
  if (DivSuch)
  {
    MGPStr tmp;
    tmp.SetStr(ZaehlStr);
    ZaehlStr.SetStr(NennStr);
    NennStr.SetStr(tmp);
  }
  // wenn ZaehlStr '1', versuchen,
  // ein div. Label aus NennStr in ZaehlStr zu bringen
  if ((ZaehlStr.Length() <= 3) && (ZaehlStr.Length() >= 2) &&
      (ZaehlStr[ZaehlStr.Length()-1] == 1))
  {
    MGPStr tmp;
    SucheDiv(NennStr,tmp); // Suche Division
    if (tmp.Length() > 0) // Wenn Division gefunden, entfernen
      EntfDiv(NennStr,ZaehlStr,&tmp[1]);
  }

  if (ZaehlStr.Length() && NennStr.Length())
  { // Pruefen ob NennStr 1/... mit 1 Summant
    p = 0;
    Summant(NennStr.c_str(),SumStr,p,RechZei);
    MGPStr tmp;
    tmp.AddChar(RechZei);
    tmp.AddStr(SumStr);
    if (SumStr[0] && (SumStr[1] == 1) && (SumStr[2] == '/') &&
        (_tcscmp(tmp.c_str(),NennStr.c_str()) == 0))
    {
      Mult(ZaehlStr,'/',&SumStr[2]); // ZaehlStr mit NennStr ohne 1 mult.
      NennStr.SetStr(_TEXT(""));
      NennStr.AddChar(RechZei);
      AddMarker(NennStr,ZahlMarker,1);
    }
  }

  Vereinfachen(NennStr);
  Vereinfachen(ZaehlStr);
  Aufadd(ZaehlStr);
  if (ZaehlStr[0] == 0)
  { // Ergebnis ist Null
    if (!AddAbstrString(_TEXT("0"),f1))
      Fehler = SolverObj::OutOfMemory;
    AddMarker(ZaehlStr,ZahlMarker,f1);
  }
  Aufadd(NennStr);
  SumKuerzen(NennStr);
  SumKuerzen(ZaehlStr);

  // Only eliminate when no PotStr, otherwise PotStr may not match Zaehl/Nenn any more
  if (PotAnz == 0)
    Kuerzen(ZaehlStr,NennStr);

  if (KlKrz && (PotAnz == 0))
    FakGruppenKuerzen(ZaehlStr,NennStr);

  _TCHAR RechZei5;
  MGPStr SumStr5;
  int SignCount = 0;
  unsigned int p5 = 0;
  while ((p5 < NennStr.Length()) && !Fehler)
  {  // test auf mehrheitlich negative Vorzeichen im Nenner
    Summant(NennStr.c_str(),SumStr5,p5,RechZei5);
    if (RechZei5 == '-')
      SignCount--;
    else
      SignCount++;
  }
  if (SignCount < 0)
  { // evtl. Vorzeichen umdrehen vor Ausklammern
    Mult(ZaehlStr,'*',_TEXT("-"));
    Mult(NennStr,'*',_TEXT("-"));
    Mult(PotStr,'*',_TEXT("-"));
  }
  Ausklammern(ZaehlStr);
  Ausklammern(NennStr);
  HiStr.SetStr(SuchFaktor);
  if (HiStr[0] != '[')
  { // Wenn SuchFaktor nicht eingeklammert, einklammern
    HiStr.InsStr(0,_TEXT("["));
    HiStr.AddStr(_TEXT("]"));
  }
  if (PotAnz > 0)
  {
    if (DivSuch || (_tcscmp(HiStr.c_str(),ReAbstrahieren(SuchStr)) != 0))
    { // wenn SuchStr != HiStr (keine quadratischen Zwischenlsungen)
      SuchStr.SetStr(_TEXT(""));
      return;
    }
    else
    if ((NennStr.Length() == 0) && PotErg(ZaehlStr,ErgLStr,PotStr,PotAnz))
      AddMarker(NennStr,ZahlMarker,1);
    else
    if ((PotStr.Length() > 0) && (PotAnz == 2))
      QuadErg(NennStr,ZaehlStr,ErgLStr,PotStr);
    else
    {
      SuchStr.SetStr(_TEXT(""));
      return;
    }
  }
  if (NennStr.Length() == 0)
  { // wenn SuchStr nicht isoliert
    SuchStr.SetStr(_TEXT(""));
    return;
  }
  LAbsStr.SetStr(ErgLStr);
  BruchVereinfachen(NennStr,ZaehlStr,Teilen);
  RAbsStr.SetStr(ZaehlStr);
  Vereinfachen(RAbsStr);
}

// Brueche vereinfachen (Vorzeichen, einklammern u.s.w.)
void SolverObj::BruchVereinfachen(MGPStr &NennStr,MGPStr &ZaehlStr,bool Teilen)
{
  int p;
  BruchZahl NennWert,ZaehlWert;
  if (NennStr[0] == '+')
    NennStr.DelNChar(0,1); // + Vorzeichen entfernen
  if (ZaehlStr[0] == '+')
    ZaehlStr.DelNChar(0,1); // + Vorzeichen entfernen

  if (GetBruchWert(ZaehlStr,ZaehlWert) && GetBruchWert(NennStr,NennWert))
  {
    ZaehlWert = ZaehlWert / NennWert;
    ZaehlStr.ShortStr(0);
    BruchOut(ZaehlWert,ZaehlStr);
    NennStr.ShortStr(0);
  }
  else
  if (NennStr.Length() <= 3)
  { // nur ein Faktor in NennStr (unter dem Bruchstrich)
    if (NennStr[0] == '-') // wenn /-
    {
      Mult(ZaehlStr,'*',_TEXT("-")); // ZaehlStr Vorzeichen 'umdrehen'
      NennStr.DelNChar(0,1);
    }
    if (NennStr[NennStr.Length()-1] == 1)  // wenn NennStr = "1"
      NennStr.SetStr(_TEXT(""));
  }
  else // NennStr lnger als 3 in jedem Fall einklammern
  {
    NennStr.InsStr(0,_TEXT("(")); // NennStr einklammern
    NennStr.AddStr(_TEXT(")"));
  }

  if ((ZaehlStr.Length() > 2) && (NennStr.Length() != 0))
  { // gleiches gilt fuer ZaehlStr
    ZaehlStr.InsStr(0,_TEXT("("));
    ZaehlStr.AddStr(_TEXT(")"));
  }

  if (ZaehlStr.Length() == 0)
  { // wenn ZaehlStr ist Null, Ergebnis Null
    int f1;
    if (!AddAbstrString(_TEXT("0"),f1))
      Fehler = SolverObj::OutOfMemory;
    AddMarker(ZaehlStr,ZahlMarker,f1);
  }
  else
  if (NennStr.Length() > 0)
  {
    if (Teilen && Konstanten(NennStr))
    { // pruefen, ob NennStr nur nicht bearbeitbare Faktoren enthaelt
      // AkTeil ist die laufende Nummer fuer ausgelagerte Ausdruecke
      MGPStr prbuff = SuchFaktor;
      prbuff.InsChar(1,'@');
      prbuff.ShortStr(prbuff.Length()-1);
      prbuff.AddChar('-');
      prbuff.AddInt(AkTeil);
      prbuff.AddChar(']');
      p = Funktion->AkEintNummer();
      if (Funktion->NeuAkEint())
        Fehler = SolverObj::OutOfMemory;
      if (!Fehler)
      {
        MGPStr tmp;
        (*Funktion)->Status |= CpxObj::Berechnet;
        tmp.AddStr(prbuff); // Label-String fuer aktuellen Ausdruck erstellen
        (*Funktion)->Namen.SetStr(tmp); // als Name eintragen
        NennStr.SetStr(ReAbstrahieren(NennStr));
        ZaehlStr.SetStr(ReAbstrahieren(ZaehlStr));
        // zurueckgewandelten String als Term
        (*Funktion)->Term.SetStr(NennStr);
        ZaehlStr.AddStr(_TEXT("/"));
        // Ergebnis ist ZaehlStr/(Label fuer aktuellen Ausdruck)
        ZaehlStr.AddStr(prbuff);
        Abstrahieren(NennStr,ZaehlStr,false);
      }
      Funktion->SetAkEint(p);
      AkTeil++;
    }
    else
    { // Normales Ergebnis, ZaehlStr/NennStr
      ZaehlStr.AddStr(_TEXT("/"));
      ZaehlStr.AddStr(NennStr);
    }
  }
}

// Unterausdruecke optimieren
void SolverObj::OptimizeSubExp(MGPStr &AkStr,bool einklammern)
{
  MGPStr NStr;
  AddMarker(NStr,ZahlMarker,1);
  AusMult(NStr,AkStr);
  Ausklammern(AkStr);
  if (NStr[0] == '+')
    NStr.DelNChar(0,1);
  if (AkStr[0] == '+')
    AkStr.DelNChar(0,1);
  Mult(AkStr,'/',NStr.c_str());
  if (AkStr[0] == '+')
    AkStr.DelNChar(0,1);
  if (AkStr.Length() == 0)
  {
    int f;
    AddAbstrString(_TEXT("0"),f);
    AddMarker(AkStr,ZahlMarker,f);
  }
  if (einklammern)
    Einklammern(AkStr);
}

// Einklammern zur Weiterverrechnung bei Bedarf
void SolverObj::Einklammern(MGPStr &AkStr)
{
  if ((AkStr.Length() > 2) || (AkStr[0] == '-'))
  {
    AkStr.InsChar(0,'(');
    AkStr.AddChar(')');
  }
}

// Einfache Potenzen aufloesen
bool SolverObj::PotErg(MGPStr &ZaehlStr,MGPStr &ErgLStr,
                       MGPStr &PotStr,int PotAnz)
{
  int f1;

  RulesString(_EFT("Removed exponent by applying its inversion (Base^Exp^(1/Exp) -> Base)"));
  DebugString(_TEXT(" PER"));
  MGPStr HiStr;
  OptimizeSubExp(ZaehlStr);
  Mult(PotStr,'*',_TEXT("-")); // Vorzeichen in PotStr richtigstellen
  OptimizeSubExp(PotStr);
  HiStr.SetStr(ZaehlStr);
  HiStr.AddStr(_TEXT("/"));
  HiStr.AddStr(PotStr);
  OptimizeSubExp(HiStr);
  HiStr = ReAbstrahieren(HiStr);
  MGPStr HiStr1;
  HiStr1.AddStr(_TEXT("(1\\"));
  HiStr1.AddInt(PotAnz);
  HiStr.AddStr(_TEXT("^"));
  HiStr.AddStr(HiStr1);
  HiStr.AddStr(_TEXT(")"));
  if (!AddAbstrString(HiStr.c_str(),f1))
    Fehler = SolverObj::OutOfMemory; // Ergebnis eintragen
  ZaehlStr.SetStr(_TEXT(""));
  AddMarker(ZaehlStr,PotMarker,f1);

  if (PotAnz % 2 == 0) // 2. Ergebnis bei geraden Potenzen
  {
    int p = Funktion->AkEintNummer();
    if (Funktion->NeuAkEint())
      Fehler = SolverObj::OutOfMemory;
    if (!Fehler)
    {
      (*Funktion)->Status |= CpxObj::Berechnet; // 2. Ergebnis anhaengen
      (*Funktion)->Namen.SetStr(ReAbstrahieren(ErgLStr));
      Abstrahieren(HiStr);
      Mult(HiStr,'*',_TEXT("-"));
      HiStr = ReAbstrahieren(HiStr);
      (*Funktion)->Term.SetStr(HiStr);
    }
    Funktion->SetAkEint(p);
  }
  return true;
}

// Quadratische Loesungsformel eintragen
// A*x^2+B*x+C = 0 -> x1/2 = (-B+-sqrt(B^2-4*A*C)/(2*A)
void SolverObj::QuadErg(MGPStr &BStr,MGPStr &ZaehlStr,
                        MGPStr &ErgLStr,MGPStr &QStr)
{
  int f1,p;
  MGPStr HiStr;

  RulesString(_EFT("Solved square expression (A*x^2+B*x+C = 0 -> x1/2 = (-B+-sqrt(B^2-4*A*C)/(2*A))"));
  DebugString(_TEXT(" QER"));
  OptimizeSubExp(BStr);
  OptimizeSubExp(QStr);
  OptimizeSubExp(ZaehlStr);

  MGPStr TermStr1; // 4*A*C
  TermStr1.AddStr(_TEXT("4*"));
  TermStr1.AddStr(ReAbstrahieren(ZaehlStr)); // ZaehlStr ist C
  TermStr1.AddStr(_TEXT("*"));
  TermStr1.AddStr(ReAbstrahieren(QStr)); // QStr ist A
  Abstrahieren(TermStr1);
  OptimizeSubExp(TermStr1);
  TermStr1 = ReAbstrahieren(TermStr1);

  MGPStr TermStr2; // A*2
  TermStr2.AddStr(ReAbstrahieren(QStr)); // QStr ist A
  TermStr2.AddStr(_TEXT("*2"));
  Abstrahieren(TermStr2);
  OptimizeSubExp(TermStr2);
  TermStr2 = ReAbstrahieren(TermStr2);

  MGPStr TermStr3; // sqrt(B^2-4*A*C)
  TermStr3.AddStr(_TEXT("sqrt("));
  TermStr3.AddStr(ReAbstrahieren(BStr)); // B^2
  TermStr3.AddStr(_TEXT("^2"));
  TermStr3.AddStr(_TEXT("-"));
  TermStr3.AddStr(TermStr1);
  TermStr3.AddStr(_TEXT(")"));

  HiStr.AddStr(_TEXT("("));
  HiStr.AddStr(ReAbstrahieren(BStr)); // NennStr ist -B
  HiStr.AddStr(_TEXT("+"));
  HiStr.AddStr(TermStr3);
  HiStr.AddStr(_TEXT(")"));
  Abstrahieren(HiStr);
  OptimizeSubExp(HiStr);
  HiStr = ReAbstrahieren(HiStr);
  if (!AddAbstrString(HiStr.c_str(),f1))
    Fehler = SolverObj::OutOfMemory; // Ergebnis eintragen

  HiStr.SetStr(_TEXT("(")); // und nun die -Version
  HiStr.AddStr(ReAbstrahieren(BStr)); // NennStr ist -B
  HiStr.AddStr(_TEXT("-"));
  HiStr.AddStr(TermStr3);
  HiStr.AddStr(_TEXT(")\\"));
  HiStr.AddStr(TermStr2);
  p = Funktion->AkEintNummer();
  if (Funktion->NeuAkEint())
    Fehler = SolverObj::OutOfMemory;
  if (!Fehler)
  {
    (*Funktion)->Status |= CpxObj::Berechnet; // 2. Ergebnis anhaengen
    (*Funktion)->Namen.SetStr(ReAbstrahieren(ErgLStr));
    (*Funktion)->Term.SetStr(HiStr);
  }
  Funktion->SetAkEint(p);
  ZaehlStr.SetStr(_TEXT(""));
  AddMarker(ZaehlStr,PotMarker,f1); // 1. Ergebnis in ZaehlStr
  if (!AddAbstrString(TermStr2.c_str(),f1))
    Fehler = SolverObj::OutOfMemory;
  BStr.SetStr(_TEXT(""));
  AddMarker(BStr,FuMarker,f1); // Division durch 2*A in NennStr
}

// loest die Funktion nach einem der nicht gesperrten Labels od. dem SuchStr auf
void SolverObj::FunktionAufloesen(MGPStr &LStr,MGPStr &RStr,MGPStr &SuchStr,
                                  bool AndereLabels,bool Teilen,bool NoExp)
{
  MGPStr HiStr,LAbsStr,RAbsStr;
  bool SuchStrVorh;

  LAbsStr.SetStr(LStr);
  RAbsStr.SetStr(RStr);
  if (DbgOut)
  {
    DbgOut->AddStr(_TEXT("\r\n"));
    DbgOut->AddStr(SuchStr);
    DebugString(_TEXT(" FAL"));
  }
  MGPStr GesStr;
  GesStr.SetStr(SuchStr);
  SuchStrVorh = true;
  Abstrahieren(LAbsStr,RAbsStr,!NoExp);
  int i = -1;
  AbstrFeld.Begin();
  if (SuchStr[0] != 0) // wenn SuchStr vorgegeben
  do
  { // Feststellen ob SuchStr in AbstrFeld
    if (_tcscmp(AbstrFeld.GetAkStr().c_str(),SuchStr.c_str()) == 0)
      i = AbstrFeld.AkEintNummer();
  }
  while ((i == -1) && AbstrFeld.NextEint() && !Fehler);
  if (i != -1)
  { // wenn SuchStr gefunden
    HiStr.SetStr(_TEXT(""));
    AddMarker(HiStr,LabelMarker,i); // Label fuer SuchStr eintragen
    AbstrFeld.SetAkEint(i); // setze AbstrFeld auf Eintrag mit SuchStr
    if (InGefLabels(AbstrFeld.GetAkStr().c_str()))
    { // wenn Label in GefLabels -> es darf nicht danach aufgeloest werden
      HiStr.SetStr(_TEXT("")); // Label fuer SuchStr loeschen
      SuchStrVorh = false;
    }
  }
  else
    HiStr.SetStr(_TEXT(""));
  if ((HiStr[0] != 0) || (AndereLabels == true))
  { // wenn SuchStr-Label vorhanden ODER suchen nach anderen Labels zulaessig
    Vereinfachen(LAbsStr); // evtl. Klammerausdruecke zusammenfassen
    Vereinfachen(RAbsStr);
    AusMult(LAbsStr,RAbsStr);
    SuchStrVorh = false;
    int f = 0;
    do
    { // alle moeglichen Labels durchsuchen wenn AndereLabels
      if (AndereLabels)
      { // wenn beliebiges Label eliminiert werden darf
        bool tst = true;
        do // suche noch nicht bearbeitetes Label
        {
          AbstrFeld.SetAkEint(f);
          if ((AbstrFeld.GetAkStr()[0] == '[') &&
             (AbstrFeld.GetAkStr()[1] != '@'))
          { // keine Bezeichnung von ausgelagerten konstanten Ausdruecken
            tst = InGefLabels(AbstrFeld.GetAkStr().c_str());
          }
          f++;
        }
        while ((f <= AbstrFeld.GetMaxEintrag()) && tst && !Fehler);
        if (!tst) // wenn noch nicht bearbeitetes Label gefunden
        {
          HiStr.SetStr(_TEXT(""));
          AddMarker(HiStr,LabelMarker,f-1);
        }
      }
      if (HiStr[0] != 0)
      { // Label zum eliminieren gefunden
        MGPStr tmp,Ltmp,Rtmp;
        tmp = HiStr;
        Ltmp = LAbsStr;
        Rtmp = RAbsStr;

        Isolieren(LAbsStr,RAbsStr,HiStr,Teilen);
        // for QuadIntr-Errors try to multiply with HiStr, to create correct square expression
        if (Fehler == SolverObj::QuadIntr)
        {
          Fehler = 0;
          HiStr = tmp;
          LAbsStr = Ltmp;
          RAbsStr = Rtmp;

          Mult(LAbsStr,'*',HiStr.c_str());
          Mult(RAbsStr,'*',HiStr.c_str());

          Isolieren(LAbsStr,RAbsStr,HiStr,Teilen);
        }
        if (HiStr[0] != 0)
        { // Ergebnis gefunden
          AbstrFeld.SetAkEint((unsigned char)HiStr[1]);
          SuchStr.SetStr(AbstrFeld.GetAkStr()); // holt SuchStr aus AbstrFeld
        }
        else
          SuchStr.SetStr(_TEXT("")); // SuchStr loeschen
      }
      else
        SuchStr.SetStr(_TEXT(""));
    }
    while (AndereLabels && (f <= AbstrFeld.GetMaxEintrag()) &&
           (SuchStr[0] == 0));
  }
  else
    SuchStr.SetStr(_TEXT(""));
  if ((SuchStr[0] != 0) || !SuchStrVorh)
  { // Label isoliert
    MakePotenzen(LAbsStr);
    MakePotenzen(RAbsStr);
    Ausklammern(LAbsStr);
    Ausklammern(RAbsStr);
    LStr.SetStr(ReAbstrahieren(LAbsStr));
    RStr.SetStr(ReAbstrahieren(RAbsStr));
    if (RStr.Length() == 0)
      RStr.SetStr(_TEXT("0"));
  }
  if ((SuchStr[0] == 0) && (GesStr[0] != 0) && SuchStrVorh && !NoExp)
  { // wenn Ergebnis nicht gefunden, aber SuchStr war vorhanden
    i = 0;
    bool AufGel = false;
    do
    {
      i++;
      AbstrFeld.SetAkEint(i);
      if (_tcsstr(AbstrFeld.GetAkStr().c_str(),GesStr.c_str()))
      { // wenn SuchStr in aktuellem AbstrFeld
        MGPStr AkLStr,AkRStr;
        AkLStr.SetStr(LStr);
        AkRStr.SetStr(RStr);
        // Funktion, die SuchStr enthaelt, in SuchStr eintragen
        SuchStr.SetStr(AbstrFeld.GetAkStr());
        // Funktion, die SuchStr enthaelt, in AkLStr bringen
        FunktionAufloesen(AkLStr,AkRStr,SuchStr,false,Teilen,true);
        if (_tcsstr(AkLStr.c_str(),GesStr.c_str()))
        { // Wenn Gesuchtes Label in AkLStr
          unsigned char p,p1;
          AufGel = true;
          unsigned int j = 0;
          p = ParserObj::InvalidFkt;
          Parser->AkWertLesen(AkLStr.c_str(),j,p);
          p1 = p;
          while ((p1 < ParserObj::InvalidFkt) &&
                 (ParserObj::FuDefArr[p1].AnzParam > 0) && !Fehler)
          { // solange gueltiger Ausdruck und Parameter vorhanden
            Parser->AkWertLesen(AkLStr.c_str(),j,p1); // Parameter lesen
          }
          if (AkLStr.Length() > j)
          { // wenn noch nicht am Ende des Strings
            if (AkLStr[j] == '^')
            { // Potenz gefunden
              const _TCHAR *pos = _tcsstr(AkLStr.c_str(),GesStr.c_str());
              unsigned int hp = 0;
              if (pos)
                hp = pos-AkLStr.c_str();
              if (pos && (hp < j))
              { // Wenn ges. Label links von ^
                MGPStr HiStr2;
                MGPStr tmp;
                tmp.AddStr(&AkLStr[j+1]);
                if ((tmp[0] == '(') && (tmp[tmp.Length()-1] == ')'))
                {
                  tmp.DelNChar(0,1);
                  tmp.ShortStr(tmp.Length()-1);
                }
                Abstrahieren(tmp);
                Vereinfachen(tmp);
                tmp = ReAbstrahieren(tmp);
                RulesString(_EFT("Removed exponent by applying its inversion (Base^Exp^(1/Exp) -> Base)"));
                BruchZahl Wert;
                unsigned int p2 = 0;
                int PsrFehler = ZahlFormatObj::NoError;
                Parser->ParseZahl(tmp.c_str(),p2,PsrFehler,Wert);
                if ((PsrFehler == ZahlFormatObj::NoError) &&
                    (Wert != 0) && (p2 == tmp.Length()))
                { // wenn
                  HiStr2.SetStr(_TEXT("(1\\"));
                  HiStr2.AddStr(tmp);
                  HiStr2.AddStr(_TEXT(")"));
                }
                else
                {
                  HiStr2.SetStr(_TEXT("1\\("));
                  HiStr2.AddStr(&AkLStr[j+1]);
                  HiStr2.AddStr(_TEXT(")")); // (1/vorhandene Potenz) in HiStr2
                  Abstrahieren(HiStr2); // 'subern' von 1/()
                  OptimizeSubExp(HiStr2);
                  HiStr2 = ReAbstrahieren(HiStr2);
                }
                Abstrahieren(AkRStr);
                OptimizeSubExp(AkRStr);
                AkRStr = ReAbstrahieren(AkRStr);
                AkRStr.AddStr(_TEXT("^"));
                AkRStr.AddStr(HiStr2);
                AkLStr.ShortStr(j);// AkLStr auf Ges. Ausdruck
                if ((Wert > 0) && (fmod(Wert.GetDoubleValue(),2.0) == 0))
                { // 2. Ergebnis bei geraden Potenzen
                  MGPStr HiStr;
                  HiStr.SetStr(AkRStr);
                  int p = Funktion->AkEintNummer();
                  Funktion->NeuAkEint();
                  (*Funktion)->Status |= CpxObj::Berechnet;
                  (*Funktion)->Namen.SetStr(AkLStr);
                  Abstrahieren(HiStr);
                  Mult(HiStr,'*',_TEXT("-"));
                  HiStr = ReAbstrahieren(HiStr);
                  (*Funktion)->Term.SetStr(HiStr);
                  Funktion->SetAkEint(p);
                }
                SuchStr.SetStr(AkLStr);
              }
              else
              { // Ergebnis ber Logarithmus
                RulesString(_EFT("Solve for exponent using ln (ln(Base^Exp) -> Exp*ln(Base))"));
                Abstrahieren(AkRStr);
                OptimizeSubExp(AkRStr,false);
                AkRStr = ReAbstrahieren(AkRStr);
                MGPStr HiStr2;
                HiStr2.SetStr(AkLStr); // Wert unter der Potenz eintragen
                HiStr2.ShortStr(j);
                Abstrahieren(HiStr2);
                OptimizeSubExp(HiStr2,false);
                HiStr2 = ReAbstrahieren(HiStr2);
                AkRStr.InsStr(0,_TEXT("ln("));
                AkRStr.AddStr(_TEXT(")\\ln("));
                AkRStr.AddStr(HiStr2); // Optimierter Wert unter der Potenz
                AkRStr.AddChar(')');
                AkLStr.DelNChar(0,j+1); // Potenz-Ausdruck ist das Ergebnis
                SuchStr.SetStr(AkLStr); // in SuchStr eintragen
              }
            }
            else
              SuchStr.SetStr(_TEXT(""));
          }
          else // keine Potenz, so auf Funktion pruefen
          {
            if (p < ParserObj::InvalidFkt) // wenn gueltige Funktion
            { // gueltige Ausdruecke
              if (ParserObj::FuDefArr[p].ArcusName[0] != 0)
              { // Arcusfunktion definiert
                RulesString(_EFT("Solve by using arcus function (B = func(A) -> A = func^-1(B))"));
                Abstrahieren(AkRStr);
                OptimizeSubExp(AkRStr,false);
                AkRStr = ReAbstrahieren(AkRStr);
                AkRStr.InsStr(0,_TEXT("("));
                AkRStr.InsStr(0,ParserObj::FuDefArr[p].ArcusName);
                // Arcus-Funktion um bisheriges Ergebnis zeichnen
                AkRStr.AddStr(_TEXT(")"));
                AkLStr.DelNChar(0,_tcslen(ParserObj::FuDefArr[p].FuName));
                // Funktions-Parameter der alten Funktion in AkLStr
                // (mit evtl. Klammern)
                SuchStr.SetStr(AkLStr);
              }
              else
                SuchStr.SetStr(_TEXT(""));
            }
            else
              SuchStr.SetStr(_TEXT(""));
          }
          if (SuchStr[0] != 0) // Wenn Label gefunden
          {
            SuchStr.SetStr(GesStr);
            // Nun endlich nach dem "entbloessten" Bezeichner aufloesen
            FunktionAufloesen(AkLStr,AkRStr,SuchStr,false,Teilen);
            if (_tcscmp(SuchStr.c_str(),GesStr.c_str()) == 0)
            { // wenn erfolgreich, als Ergebnis eintragen
              LStr.SetStr(AkLStr);
              RStr.SetStr(AkRStr);
            }
          }
        }
        else
          SuchStr.SetStr(_TEXT(""));
      }
    }
    while ((i < AbstrFeld.GetMaxEintrag()) &&
           (SuchStr[0] == 0) && !AufGel && !Fehler);
  }
  if (LStr[0] == '+')
    LStr.DelNChar(0,1);
  if (RStr[0] == '+')
    RStr.DelNChar(0,1);
  if ((RStr[0] == '-') && !_tcscmp(&RStr[1],_TEXT("0")))
    RStr.DelNChar(0,1);
}

// Funktionen nach denselben Labels aufloesen und gleichsetzen
bool SolverObj::FunktionGleichSetzen(MGPStr &LStr,MGPStr &RStr,MGPStr &LZielStr,
                                     MGPStr &RZielStr,bool &Teilen)
{
  bool FktGleich;
  MGPStr LAbsStr,RAbsStr;

  DebugString(_TEXT(" FGS"));
  LAbsStr.SetStr(LStr); // berschreiben v. LStr u. RStr vermeiden
  RAbsStr.SetStr(RStr);
  FktGleich = false;
  FunktionAufloesen(LZielStr,RZielStr,LAbsStr,false,Teilen);
  if (LAbsStr[0] != 0)
  { // wenn LAbsStr in LZielStr und RZielStr aufgeloest
    LZielStr.SetStr(RAbsStr); // LZiel und RAbs gleichsetzen
    FktGleich = true;
  }
  return FktGleich;
}

void SolverObj::LoeschKlammern()
{ // NAMEN-Eintraege von eckigen Klammern befreien
  int f;

  for (f = 0;f <= Funktion->GetMaxEintrag();f++)
  {
    Funktion->SetAkEint(f);
    if (((*Funktion)->Namen.Length() > 0) &&
        ((*Funktion)->Namen[0] == '[') &&
       ((*Funktion)->Namen[(*Funktion)->Namen.Length()-1] == ']'))
    {
      MGPStr tmp;
      tmp.SetStr(&(*Funktion)->Namen[1]);
      tmp.ShortStr(tmp.Length()-1);
      (*Funktion)->Namen.SetStr(tmp);
    }
  }
}

void SolverObj::NichtBerechnet()
{ // BERECHNET-Flag fuer alle Funktionen loeschen
  int f;

  for (f = 0;f <= Funktion->GetMaxEintrag();f++)
  {
    Funktion->SetAkEint(f);
    (*Funktion)->Status &= ~CpxObj::Berechnet;
  }
}

bool SolverObj::ElimLabel()
{ // eliminiert das naechste Label
  MGPStr LAbsStr,RAbsStr,LZielStr,RZielStr,SuchStr,LStr;
  int f,z;
  bool Gleichgesetzt;

  ElimLabelStatus = false;
  ErgString.SetStr(_TEXT(""));
  // alle Label eliminiert oder Fehler oder letzte Funktion ueberschritten
  if ((AkF > Funktion->GetMaxEintrag()) || NoMoreLabels || Fehler)
  {
    if (!NoMoreLabels)
    { // Noch Labels vorhanden
      if (!Fehler) ErgebnisUmformen(); // wenn kein Fehler, fertig
      Aufgeloest = (Fehler == 0);
      NoMoreLabels = true; // fuer naechsten Aufruf Ende eintragen
      ElimLabelStatus = true;
      return ElimLabelStatus;
    }
    ElimLabelStatus = false;
    return ElimLabelStatus;
  }
  if (DbgOut)
    DbgOut->SetStr(_TEXT(""));
  LAbsStr.SetStr(_TEXT(""));
  RAbsStr.SetStr(_TEXT(""));
  do // Finden eines Labels
  {
    SuchStr.SetStr(SuchFaktor);
    Funktion->SetAkEint(AkF);
    if (!Parser->PruefeKlammern(Funktion))
      Fehler = SolverObj::Parenthesis;
    if (!Fehler)
    {
      if ((*Funktion)->Term[0] != 0)
      { // aktuellen Term aufloesen
        LAbsStr.SetStr((*Funktion)->Namen);
        RAbsStr.SetStr((*Funktion)->Term);
        if ((LAbsStr.Length() == 0) || (LAbsStr[0] == ' '))
          SuchStr.SetStr(_TEXT("")); // Kommentarzeile nicht aufloesen
        else
          FunktionAufloesen(LAbsStr,RAbsStr,SuchStr,true,Teilen);
      }
      else
        SuchStr.SetStr(_TEXT(""));
      if (SuchStr[0] != 0)
        (*Funktion)->Status |= CpxObj::Berechnet;
    }
    AkF++;
  }
  while ((AkF <= Funktion->GetMaxEintrag()) && (SuchStr[0] == 0) &&
         (_tcscmp(LAbsStr.c_str(),SuchFaktor.c_str()) != 0) && !Fehler);
  Gleichgesetzt = false;
  // SuchStr gueltig, Endergebnis nicht eliminieren
  if ((SuchStr[0] != 0) && (_tcscmp(LAbsStr.c_str(),SuchFaktor.c_str()) != 0))
  { // Gleichsetzen der aufgeloesten Gleichung
    f = 1;
    while ((f <= Funktion->GetMaxEintrag()) && !Fehler)
    { // alle Funktionen
      if (f != AkF-1)
      { // nicht mit sich selbst gleichsetzen
        Funktion->SetAkEint(f);
        LZielStr.SetStr((*Funktion)->Namen);
        RZielStr.SetStr((*Funktion)->Term);
        if (!((LZielStr.Length() == 0) || (LZielStr[0] == ' ')))
        { // keine Kommentarzeilen oder leere Funktionen gleichsetzen
          Gleichgesetzt |= FunktionGleichSetzen(LAbsStr,RAbsStr,LZielStr,
                                                RZielStr,Teilen);
          (*Funktion)->Namen.SetStr(LZielStr);
          (*Funktion)->Term.SetStr(RZielStr);
        }
      }
      f++;
    }
    GefLabels.insert(LAbsStr);
  }
  else
  if (_tcscmp(LAbsStr.c_str(),SuchFaktor.c_str()) == 0)
  { // wenn nach Endergebnis aufgeloest
    (*Funktion)->Namen.SetStr(LAbsStr);
    (*Funktion)->Term.SetStr(RAbsStr);
    AkF = Funktion->GetMaxEintrag()+1; // Ende-Markierung setzen
  }
  Funktion->SetAkEint(AkF-1);
  if (Gleichgesetzt && (_tcscmp(LAbsStr.c_str(),SuchFaktor.c_str()) != 0))
  {  // Lschen der gleichgesetzten Gleichung
    Funktion->LoeschAkEint();
    if (AkF > 0)
      AkF--;
//    AkF = 1; // beim naechster Suche wieder in 1. Zeile anfangen
// (verhindert evtl. vorzeitigen Abbruch,
// wenn mal 1 Zeile nicht aufgeloest werden kann)
  }
  else
  if (SuchStr[0] != 0)
  {
    (*Funktion)->Term.SetStr(RAbsStr);
    (*Funktion)->Namen.SetStr(LAbsStr);
  }
  DelDuplicateLines(EinschString);
  if (RulesOut)
    DelDuplicateLines(*RulesOut);
  if (Debug)
  {
    if (Gleichgesetzt && (SuchStr[0] != 0) || Fehler)
    {
      ErgString.AddStr(SuchStr);
      if (!Fehler)
        ErgString.AddStr(_TEXT(" :\r\n"));
      else
        ErgString.AddStr(_TEXT(" !\r\n"));
      for (z = 1;z <= Funktion->GetMaxEintrag();z++)
      {
        Funktion->SetAkEint(z);
        LStr.SetStr((*Funktion)->Namen);
        if (!((LStr.Length() == 0) || (LStr[0] == ' ')))
        {
          ErgString.AddStr(LStr);
          ErgString.AddStr(_TEXT("="));
          ErgString.AddStr((*Funktion)->Term);
          ErgString.AddStr(_TEXT("\r\n"));
        }
      }
    }
  }
  else
  {
    if (SuchStr[0] != 0)
    {
      ErgString.AddStr(LAbsStr);
      ErgString.AddStr(_TEXT("\r\n"));
    }
  }
  ElimLabelStatus = true;
  return ElimLabelStatus;
}

void SolverObj::GetFaktoren(MGPStrSet &Faktoren,const _TCHAR *AStr)
{ // ermittelt die Faktoren in AStr und erzeugt Eintraege in Faktoren
  MGPStr AkStr;
  unsigned int f = 0;
  size_t alen = _tcslen(AStr);
  while ((f < alen) && !Fehler)
  { // alle Faktoren
    AkStr.SetStr(_TEXT(""));
    int AkPos = f;
    while (AStr[f] && (AStr[f] != ']') &&
           _tcsncmp(&AStr[f],Parser->GetLDV().ListSeparator.c_str(),
                   Parser->GetLDV().ListSeparator.Length()))
      f++; // Ende des Faktors ermitteln
    if (AStr[f] == ']')
      f++; // hintere Klammer mitnehmen
    AkStr.AddNChar(&AStr[AkPos],f-AkPos);
    if (AkStr[0] != '[')
    { // evtl. fehlende Klammern hinzufuegen
      AkStr.InsStr(0,_TEXT("["));
      AkStr.AddStr(_TEXT("]"));
    }
    Faktoren.insert(AkStr);
    if (!_tcsncmp(&AStr[f],Parser->GetLDV().ListSeparator.c_str(),
                 Parser->GetLDV().ListSeparator.Length()))
      f += Parser->GetLDV().ListSeparator.Length();
  }
}

void SolverObj::SortFunktionen()
{ // Bubble-Sort sortiert die Funktionen nach Term-Laenge (Optimierung)
  DebugString(_TEXT(" SFK"));
  bool gef;
  do
  {
    gef = false;
    Funktion->SetAkEint(1);
    while (Funktion->AkEintNummer() < Funktion->GetMaxEintrag())
    { // alle Funktionen durchgehen
      CpxObj &tmpc = Funktion->GetAkEint();
      MGPStr tmp(tmpc.Namen);
      MGPStr Str1(tmpc.Term);
      Abstrahieren(tmp,Str1,true);
      if (Funktion->NextEint()) // Funktion zum Vergleichen vorhanden
      {
        tmp.SetStr((*Funktion)->Namen);
        MGPStr Str2((*Funktion)->Term);
        Abstrahieren(tmp,Str2,false);
        if (Str2.Length() < Str1.Length())
        { // naechster Term kuerzer als der vorige
          CpxObj tmp(Funktion->GetAkEint());
          Funktion->GetAkEint() = tmpc;
          tmpc = tmp;
          gef = true;
        }
      }
    }
  }
  while (gef);
}

void SolverObj::StartWerte()
{ // Initialisierung der Funktionen
  int z;
  MGPStr AkStr;
  EinschString.SetStr(_TEXT(""));

  firsttime = 0;
  Fehler = 0;
  AkF = 1;
  Funktion->SetAkEint(AkF);
  while (AkF <= Funktion->GetMaxEintrag() && !Fehler)
  { // Kommentarzeile loeschen
    if (((*Funktion)->Namen.Length() == 0) ||
        ((*Funktion)->Namen[0] == ' ') ||
        ((*Funktion)->Namen[0] < 32))
      Funktion->LoeschAkEint();
    else
    {
      if ((*Funktion)->ParamStr[0] != 0)
      { // Evtl. Parameterlisten an Namen anhaengen
        (*Funktion)->Namen.AddStr((*Funktion)->ParamStr);
        (*Funktion)->ParamStr.ShortStr(0);
      }
      AkF++;
    }
    Funktion->SetAkEint(AkF);
  }
  // nicht benutzt, weil es eher schadet,
  // ausserdem kann der Anwender nicht mehr selber sortieren
//  SortFunktionen();
  NichtBerechnet();
  AkTeil = 0;
  if (Debug)
  {
    for (z = 1;z <= Funktion->GetMaxEintrag();z++)
    {
      Funktion->SetAkEint(z);
      ErgString.AddStr((*Funktion)->Namen);
      ErgString.AddStr(_TEXT("="));
      ErgString.AddStr((*Funktion)->Term);
      ErgString.AddStr(_TEXT("\r\n"));
    }
  }
  if ((SuchFaktor.Length() > 0) && (SuchFaktor[0] != '['))
  {
    SuchFaktor.InsStr(0,_TEXT("["));
    SuchFaktor.AddStr(_TEXT("]"));
  }
  if (SuchFaktor[0] != 0)
    GefLabels.insert(SuchFaktor);

  GetFaktoren(GefLabels,ErhFaktoren.c_str());
  GetFaktoren(ZusLabels,ZusFaktoren.c_str());
  AkF = 1;
}

void SolverObj::ErgebnisUmformen()
{ // Endergebnis in Form bringen
  MGPStr LAbsStr,RAbsStr,LZielStr,RZielStr,SuchStr,LStr;
  int f,z;

  DebugString(_TEXT(" EUF"));
  f = 1;
  bool Gef = false;
  while (f <= Funktion->GetMaxEintrag() && !Fehler)
  { // alle Funktionen nochmal Aufloesen zur Ergebnisoptimierung
    GefLabels.clear();
    Funktion->SetAkEint(f);
    if ((*Funktion)->Term[0] != 0)
    {
      LAbsStr.SetStr((*Funktion)->Namen);
      RAbsStr.SetStr((*Funktion)->Term);
      SuchStr.SetStr(SuchFaktor);
      ErgebnisOpt = true;
      FunktionAufloesen(LAbsStr,RAbsStr,SuchStr,false,Teilen);
      if ((_tcscmp(SuchStr.c_str(),SuchFaktor.c_str()) == 0) &&
          (f <= Funktion->GetMaxEintrag()))
        (*Funktion)->Status |= CpxObj::Berechnet;
      // pruefen, ob gesuchter String vorhanden
      Gef |= !_tcscmp(SuchFaktor.c_str(),LAbsStr.c_str());
      if ((LAbsStr.Length() > 0) && (LAbsStr[0] == '['))
      { // neu aufgeloeste Funktion eintragen
        (*Funktion)->Namen.SetStr(LAbsStr);
        (*Funktion)->Term.SetStr(RAbsStr);
      }
    }
    f++;
  }
  DelDuplicateLines(EinschString);
  if (RulesOut)
    DelDuplicateLines(*RulesOut);
  if (!Fehler && !Gef)
    Fehler = SolverObj::NotSolved;
  if (!Fehler)
  {
    ErgString.AddStr(_TEXT("-> "));
    ErgString.AddStr(SuchFaktor);
    ErgString.AddStr(_TEXT(" :\r\n"));
  }
  else
  {
    ErgString.AddStr(_TEXT("-> "));
    ErgString.AddStr(SuchFaktor);
    ErgString.AddStr(_TEXT(" !\r\n"));
  }
  for (z = 1;z <= Funktion->GetMaxEintrag();z++)
  {
    Funktion->SetAkEint(z);
    LStr.SetStr((*Funktion)->Namen);
    if (!((LStr.Length() == 0) || (LStr[0] == ' ') || (LStr[0] < 32)))
    {
      ErgString.AddStr(LStr);
      ErgString.AddStr(_TEXT("="));
      ErgString.AddStr((*Funktion)->Term);
      ErgString.AddStr(_TEXT("\r\n"));
    }
  }
}

bool SolverObj::InteractiveSolve(const _TCHAR *FakString,const _TCHAR Operator)
{
  StartWerte();
  if (!Parser->PruefeKlammern(Funktion))
    Fehler = SolverObj::Parenthesis;
  for (int z = 1;z <= Funktion->GetMaxEintrag();z++)
  {
    Funktion->SetAkEint(z);
    MGPStr LAbsStr,RAbsStr;
    LAbsStr.SetStr((*Funktion)->Namen);
    RAbsStr.SetStr((*Funktion)->Term);
    if ((LAbsStr.Length() == 0) || (LAbsStr[0] == ' '))
      continue; // don't solve comment line

    if (Operator == '=')
    {
      MGPStr SuchStr;
      SuchStr = FakString;
      FunktionAufloesen(LAbsStr,RAbsStr,SuchStr,false,false);
    }
    else
    if (!Fehler && ((*Funktion)->Term[0] != 0))
    {
      Abstrahieren(LAbsStr,RAbsStr,true);
      MGPStr FakStr;
      FakStr = FakString;
      Abstrahieren(FakStr,false);
      Vereinfachen(LAbsStr);
      Vereinfachen(RAbsStr);
      AusMult(LAbsStr,RAbsStr);
      if (Fehler)
        return false;
      if ((Operator == '*') || (Operator == '/'))
        Mult(LAbsStr,Operator,FakStr.c_str());
      else
        Add(LAbsStr,Operator,FakStr.c_str());
      OptimizeSubExp(LAbsStr);
      LAbsStr = ReAbstrahieren(LAbsStr);
      if ((Operator == '*') || (Operator == '/'))
        Mult(RAbsStr,Operator,FakStr.c_str());
      else
        Add(RAbsStr,Operator,FakStr.c_str());
      OptimizeSubExp(RAbsStr);
      RAbsStr = ReAbstrahieren(RAbsStr);
    }
    if (!Fehler)
    {
      (*Funktion)->Namen = LAbsStr;
      (*Funktion)->Term = RAbsStr;
    }
  }
  return !Fehler;
}
