// Version 6.05 20.07.2008 Markus Greither
// Copyright (c) 1991-2008 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file efsprexp.cpp
    \brief Source code for the spreadsheet export class
*/

#ifdef __BORLANDC__

#include "efprec.h"

#pragma hdrstop
#else

#ifdef __DMC__
#include <typeinfo.h>
#endif
#include <cstdlib>
#include <cmath>
#include <complex>
#include <limits>
#include <cstring>
#include <set>
#include <functional>
#include <cctype>
#include <cfloat>

using namespace std;

#include "efdefs.h"
#include "mg.h"
#include "cpxpsr.h"

#endif

#include "efsprexp.h"

EFSpreadsheetExport::ExportErrors EFSpreadsheetExport::Convert(MGPStr &Result)
{
  ExportErrors Res = OK;
  MGStrListObj Headers,Fus;

  // Insert headers and functions
  Functions.Begin();
  if (Functions.GetMaxEintrag() >= 0)
    do
    {
      // Delete blank and comment lines
      int akpos = Functions.AkEintNummer();
      while ((Functions.GetAkEint()[0] == 0) ||
             (Functions.GetAkEint()[0] <= ' '))
      {
        akpos = Functions.AkEintNummer();
        Functions.LoeschAkEint();
        Functions.SetAkEint(akpos);
        if (akpos >= Functions.GetMaxEintrag())
          break;
      }
      if (akpos > Functions.GetMaxEintrag())
        break;
      int pos = ParserObj::IstGleichPos(Functions->c_str());
      MGPStr tmp = Functions->c_str();
      int pos1 = pos;
      while ((pos1 > 0) && (tmp[pos1-1] == ' '))
        pos1--;
      if (pos1 >= 0)
        tmp.ShortStr(pos1);
      // Insert brackets if neccessary
      // FIXME: Only correct for functions not equations
      if (tmp[0] != '[')
      {
        tmp.InsChar(0,'[');
        tmp.AddChar(']');
      }
      Headers.NeuAkEint();
      Headers.GetAkEint() = tmp;
      Fus.NeuAkEint();
      pos++;
      while ((pos < (int)Functions->Length()) && (Functions->c_str()[pos] == ' '))
        pos++;
      if (pos >= 0)
        Fus.GetAkEint() = &Functions->c_str()[pos];
    }
    while (Functions.NextEint());

  // Convert functions
  Fus.Begin();
  if (Fus.GetMaxEintrag() >= 0)
    do
    {
      MGPStr akstr = Fus.GetAkEint();
      Fus.GetAkEint() = _TEXT("=");

      unsigned int pos = 0;
      if (pos >= akstr.Length())
        continue;
      while (pos < akstr.Length())
      {
        ParserObj::LeerZNKomment(akstr.c_str(),pos);
        if (pos >= akstr.Length())
          break;
        unsigned int orgpos = pos;
        unsigned char fn = ParserObj::FuNummer(pos,akstr.c_str());
        if ((fn != ParserObj::InvalidFkt) || (akstr[orgpos] == '['))
        {
          if ((akstr[orgpos] == '[') || // if label or variable
              !_tcscmp(ParserObj::FuDefArr[fn].SpreadName,_TEXT("$")))
          {
            MGPStr val;
            if (akstr[orgpos] == '[')
            {
              pos = orgpos;
              while ((pos < akstr.Length()) && (akstr[pos] != ']'))
                pos++;
              if (akstr[pos] == ']')
                pos++;
              val = &akstr[orgpos];
              val.ShortStr(pos-orgpos);
            }
            else
              val = ParserObj::FuDefArr[fn].FuName;
            Headers.Begin();
            int col = -1;
            if (Headers.GetMaxEintrag() >= 0)
              do
              {
                if (!MGStricmp(Headers.GetAkEint().c_str(),val.c_str()))
                  col = Headers.AkEintNummer();
              }
              while (Headers.NextEint());
            // Insert column for not available variable
            if (col == -1)
            {
              int akpos = Fus.AkEintNummer();
              Headers.NeuAkEint();
              Headers.GetAkEint() = val;
              Fus.NeuAkEint();
              Fus.GetAkEint() = _TEXT("1"); //dummy value
              col = Fus.AkEintNummer();
              Fus.SetAkEint(akpos);
            }
            MGPStr tmp;
            if (col >= 26)
            {
              tmp.AddChar(_TCHAR(64+col/26));
              col = col % 26;
            }
            tmp.AddChar(_TCHAR(65+col));
            tmp += _TEXT("2");
            Fus.GetAkEint() += tmp;
          }
          else
          // In case of error (no spreadsheet function name), use ?FuName?
          if (!_tcscmp(ParserObj::FuDefArr[fn].SpreadName,_TEXT("ERROR")))
          {
            Fus.GetAkEint() += _TEXT("?");
            Fus.GetAkEint() += ParserObj::FuDefArr[fn].FuName;
            Fus.GetAkEint() += _TEXT("?");
          }
          else
            Fus.GetAkEint() += ParserObj::FuDefArr[fn].SpreadName;
        }
        else
        if ((IST_ZAHL(akstr.c_str()[pos])) ||
            (akstr.c_str()[pos] == GetLDV().DecSeparator[0]))
        { // Start of a number found
          int Fehl = ZahlFormatObj::NoError;
          double z = Zahl(akstr.c_str(),pos,Fehl);
          MGPStr tmp;
          Ausgabe(tmp,z,ExpForm);
          Fus.GetAkEint() += tmp;
        }
        else
        {
          if (orgpos == pos)
            pos++;
          if ((pos-orgpos == 1) && (akstr[orgpos] == '\\'))
            Fus.GetAkEint() += _TEXT("/");
          else
            Fus.GetAkEint().AddNChar(&akstr[orgpos],pos-orgpos);
        }
      }
    }
    while (Fus.NextEint());
  Result = _TEXT("");
  Headers.Begin();
  if (Headers.GetMaxEintrag() >= 0)
    do
    {
      Result += Headers->c_str();
      if (Headers.AkEintNummer() < Headers.GetMaxEintrag())
        Result += _TEXT("\t");
      else
        Result += _TEXT("\r\n");
    }
    while (Headers.NextEint());
  Fus.Begin();
  if (Fus.GetMaxEintrag() >= 0)
    do
    {
      Result += Fus->c_str();
      if (Fus.AkEintNummer() < Fus.GetMaxEintrag())
        Result += _TEXT("\t");
      else
        Result += _TEXT("\r\n");
    }
    while (Fus.NextEint());
  return Res;
}

