// Version 6.06  29.11.2009 Markus Greither
// Copyright (c) 1991-2009 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file eftabcon.cpp
    \brief Source code for the tab-key support class
*/


#include "efprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#include <wx/xrc/xmlres.h> // must be before any templates for DMC (?!)

#ifndef WX_PRECOMP
#ifdef __DMC__
#include <typeinfo.h>
#endif
#include <wx/wx.h>

#include <wx/docview.h>
#include <wx/clipbrd.h>
#include <wx/resizec.h>
#include <complex>
#include <list>
#include <functional>
#include <set>

using namespace std;

#include "efdefs.h"
#include "mg.h"
#include "eftabcon.h"

#endif

#include "efmacro.h"

// ----------------------------------------------------------------------------
// TEFTabControl
// ----------------------------------------------------------------------------

template<class T> bool TEFTabControlT<T>::ProcessEvent(wxEvent& event)
{
  bool ret = wxResizeableControl::ProcessEvent(event);

  if (ret)
    MyMacroRecorder::Get()->AddEvent(event,wxWindow::GetId());

  return ret;
}

template<class T> void TEFTabControlT<T>::EvaluateKeys(wxWindow *wnd,wxKeyEvent &event)
{
  if (event.AltDown())
  {
    event.Skip();
    return;
  }
  switch (event.GetKeyCode())
  {
    case WXK_TAB :
      if (wnd && wnd->GetParent())
      {
        wxWindow *tmp = wxWindow::FindFocus();
        wxWindowListNode *item = wnd->GetParent()->GetChildren().Find(tmp);
        if (item) // Child has focus
        {
          do
            item = item->GetNext();
          while (item && !item->GetData()->IsShown());
          if (item)
          {
            wxResizeableParentControl *par = wxDynamicCast(item->GetData(),TEFTabParentControl);
            if (par && par->GetManagedChild())
              par->GetManagedChild()->SetFocus();
            else
              item->GetData()->SetFocus();
          }
          else
            wnd->GetParent()->SetFocus();
        }
      }
      else
        event.Skip();
      break;
    case WXK_DELETE :
    {
      wxCommandEvent cevent(wxEVT_COMMAND_MENU_SELECTED,XRCID("ICM_DELWINDOW"));
      cevent.SetEventObject(wnd);
      if (wnd && wnd->GetParent())
        wnd->GetParent()->GetEventHandler()->AddPendingEvent(cevent);
      break;
    }
    default:
      event.Skip();
  }
}

IMPLEMENT_DYNAMIC_CLASS(TEFTabControl, wxResizeableControl)

BEGIN_EVENT_TABLE(TEFTabControl,wxResizeableControl)
  EVT_KEY_DOWN(TEFTabControl::OnKeyDown)
  EVT_SIZE(TEFTabControl::OnSize)
  EVT_MOVE(TEFTabControl::OnMove)
END_EVENT_TABLE()


IMPLEMENT_DYNAMIC_CLASS(TEFTabParentControl, wxResizeableParentControl)

BEGIN_EVENT_TABLE(TEFTabParentControl,wxResizeableParentControl)
  EVT_KEY_DOWN(TEFTabParentControl::OnKeyDown)
  EVT_SIZE(TEFTabParentControl::OnSize)
  EVT_MOVE(TEFTabParentControl::OnMove)
END_EVENT_TABLE()

// ----------------------------------------------------------------------------
// TEFBitmapControl
// ----------------------------------------------------------------------------

bool TEFBitmapControl::ProcessEvent(wxEvent& event)
{
  bool ret = wxBitmapControl::ProcessEvent(event);

  if (ret)
    MyMacroRecorder::Get()->AddEvent(event,GetId());

  return ret;
}

IMPLEMENT_DYNAMIC_CLASS(TEFBitmapControl, wxBitmapControl)

BEGIN_EVENT_TABLE(TEFBitmapControl,wxBitmapControl)
  EVT_KEY_DOWN(TEFBitmapControl::OnKeyDown)
  EVT_SIZE(TEFBitmapControl::OnSize)
  EVT_MOVE(TEFBitmapControl::OnMove)
END_EVENT_TABLE()

#ifdef __WXMSW__
// ----------------------------------------------------------------------------
// TEFMetafileControl
// ----------------------------------------------------------------------------

bool TEFMetafileControl::ProcessEvent(wxEvent& event)
{
  bool ret = wxMetafileControl::ProcessEvent(event);

  if (ret)
    MyMacroRecorder::Get()->AddEvent(event,GetId());

  return ret;
}

IMPLEMENT_DYNAMIC_CLASS(TEFMetafileControl, wxMetafileControl)

BEGIN_EVENT_TABLE(TEFMetafileControl,wxMetafileControl)
  EVT_KEY_DOWN(TEFMetafileControl::OnKeyDown)
  EVT_SIZE(TEFMetafileControl::OnSize)
  EVT_MOVE(TEFMetafileControl::OnMove)
END_EVENT_TABLE()

#endif
