// Version 5.16  23.11.02 Markus Greither
// Copyright (c) 1991-2003 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file efundo.cpp
    \brief Source code for the undo/redo class used in the function editor
*/


#include "efprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifdef __DMC__
#include <typeinfo.h>
#endif
#include <cstdio>
#include <cstring>
#include <set>
#include <functional>
#include <cmath>
#include <cstdlib>
#include <cctype>
#include <iostream>

using namespace std;

#include "efdefs.h"
#include "mg.h"
#include "efundo.h"

// ----------------------------------------------------------------------------
// UndoData
// ----------------------------------------------------------------------------

void UndoData::UpdateInsert(TextPos A,TextPos E,const _TCHAR C)
{
  // if marking was deleted during this insert, and no insert mode (C == 0)
  if ((UndoInfo->Anf == UndoInfo->End) &&
      (UndoInfo->Anf == A) && !C)
    UndoInfo->End = E;
  else
  // if last undo was delete, or new pos, create new undo item
  if ((UndoInfo->Anf == UndoInfo->End) ||
      !(A == UndoInfo->End))
  {
    MGPStr tmp;
    tmp.AddChar(C);
    NewUndo(A,E,A,tmp);
  }
  else
  // another character input -> add to current insert undo
  {
    UndoInfo->End = E;
    UndoInfo->Str.AddChar(C);
  }
}

void UndoData::NewUndo(TextPos A,TextPos E,TextPos I,
                       const MGPStr &str)
{
  int p = UndoInfo.AkEintNummer();
  // Delete all higher undos (invalid now)
  for (int i = UndoInfo.GetMaxEintrag();i > p;i--)
  {
    UndoInfo.SetAkEint(i);
    UndoInfo.LoeschAkEint();
  }
  UndoInfo.NeuAkEint();
  UndoInfo->Set(A,E,I,str);
}

void UndoData::UpdateDelete(TextPos A,TextPos E,const _TCHAR C)
{
  MGPStr tmp;
  tmp.AddChar(C);
  UpdateDelete(A,E,tmp);
}

void UndoData::UpdateDelete(TextPos A,TextPos E,const MGPStr &str)
{
  // if marking was deleted during this delete, extend current delete undo
  if ((UndoInfo->Anf == A) && (UndoInfo->Anf == E))
  {
    UndoInfo->End = A;
    UndoInfo->Anf = A;
    UndoInfo->Ins = A;
    UndoInfo->Str.AddStr(str);
  }
  else
  // if marking was deleted during this backspace, extend current delete undo
  if ((UndoInfo->Anf == UndoInfo->End) &&
      (UndoInfo->Anf == E))
  {
    UndoInfo->End = A;
    UndoInfo->Anf = A;
    UndoInfo->Ins = A;
    UndoInfo->Str.InsStr(0,str);
  }
  else
  // otherwise, create new undo item
    NewUndo(A,A,A,str);
}


