// Version 6.05 20.07.2008 Markus Greither
// Copyright (c) 1991-2008 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file gauss.cpp
    \brief Source code for the gaussian polynom solver and other approximation functions
*/

#ifdef __BORLANDC__

#include "efprec.h"

#pragma hdrstop
#else

#ifdef __DMC__
#include <typeinfo.h>
#endif
#include <cstdio>
#include <cfloat>
#include <cctype>
#include <complex>
#include <limits>
#include <cstdlib>
#include <cstring>
#include <set>
#include <functional>

using namespace std;

#include "efdefs.h"
#include "mg.h"
#include "cpxpsr.h"
#include "gauss.h"

#endif

// ----------------------------------------------------------------------------
// Variables
// ----------------------------------------------------------------------------

//! Global error flag for matherr, defined in cpxpsr.cpp
extern int AkErr;

// ----------------------------------------------------------------------------
// FuGenObj
// ----------------------------------------------------------------------------

//void FuGenObj::WriteMatrix(double **&Matrix,int MaxUnb)
void FuGenObj::WriteMatrix(double **&Matrix,unsigned char &MaxUnb)
{
  unsigned char i,f;

  for (i=0;i < MaxUnb;i++)
  {
    for (f=0;f < MaxUnb;f++)
    {
      printf("%f ",Matrix[f][i]);
    }
    printf("\n");
  }
}

bool FuGenObj::Elim(double **&Matrix,int AnzUnb)
{
  unsigned char AkZeile;
  int i,f;
  double Faktor;

  bool ret = true;
  fptr oldhndl = SetFPESignal();
  try
  {
    /*  WriteMatrix(Matrix,AnzUnb);*/ // Debug-Ausgabe
    for (AkZeile = 0;AkZeile < AnzUnb-1;AkZeile++)
    {
      for (i = AkZeile+1;i < AnzUnb;i++)
      {
        if (Matrix[AkZeile][AkZeile] != 0)
        {
          Faktor = -Matrix[AkZeile][i]/Matrix[AkZeile][AkZeile];
          for (f = AkZeile;f <= AnzUnb;f++)
            Matrix[f][i] = Matrix[f][i]+Faktor*Matrix[f][AkZeile];
        }
        else
          ret = false;
      }
    }
    if (ret)
      for (f = AnzUnb-1;f >= 0;f--)
      {
        if (f < AnzUnb-1)
        {
          for (i = f;i >= 0;i--)
            Matrix[AnzUnb][i] = Matrix[AnzUnb][i]-
                                Matrix[f+1][AnzUnb]*Matrix[f+1][i];
        }
        Matrix[f][AnzUnb] = Matrix[AnzUnb][f]/Matrix[f][f];
      }
  }
  catch (...)
  {
    ret = false;
  }
  ClearFPESignal(oldhndl);
  return ret;
}

/* Liest die Sttzpunkte der Nherungsfunktion*/
bool FuGenObj::ReadPkts(const _TCHAR *ParamStr)
{
  unsigned int p;
  int Fehler;
  double Erg;

  PktFeld.SetAnzahl(0);
  p = 0;
  Fehler = ZahlFormatObj::NoError;
  while ((p < _tcslen(ParamStr)) && (Fehler == NoError))
  {
    while (ParamStr[p] == ' ')
      p++;
    Erg = Zahl(ParamStr,p,Fehler);
    PktFeld.X[PktFeld.GetAnzahl()] = Erg;
    while ((p < _tcslen(ParamStr)) && ((ParamStr[p] == ' ') ||
           !_tcsncmp(&ParamStr[p],GetLDV().ListSeparator.c_str(),
                    GetLDV().ListSeparator.Length())))
      p++;
    Erg = Zahl(ParamStr,p,Fehler);
    PktFeld.Y[PktFeld.GetAnzahl()] = Erg;
    if (Fehler == NoError)
      PktFeld.SetAnzahl(PktFeld.GetAnzahl()+1);
  }
  return (PktFeld.GetAnzahl() > 1);
}

// ermittelt die Nherungsfunktion
const _TCHAR *FuGenObj::Interpol()
{
  int i,f;
  double AkX;
  MGPStr ErgStr;
  static MGPStr EStr;
  double **Matrix;

  Matrix = new double *[PktFeld.GetAnzahl()+1];
  for (int p = 0;p < PktFeld.GetAnzahl()+1;p++)
    Matrix[p] = new double[PktFeld.GetAnzahl()+1];

  for (i = 0;i < PktFeld.GetAnzahl();i++)
  {
    AkX = 1;
    for (f = 0;f < PktFeld.GetAnzahl();f++)
    {
      Matrix[f][i] = AkX;
      AkX *= PktFeld.X[i];
    }
    Matrix[PktFeld.GetAnzahl()][i] = PktFeld.Y[i];
  }
  if (Elim(Matrix,PktFeld.GetAnzahl()))
  {
    EStr.SetStr(_TEXT(""));
    if (Matrix[0][PktFeld.GetAnzahl()] != 0)
    {
      Ausgabe(ErgStr,Matrix[0][PktFeld.GetAnzahl()],FitForm);
      EStr.AddStr(ErgStr);
    }
    if (Matrix[1][PktFeld.GetAnzahl()] != 0)
    {
      if (Matrix[1][PktFeld.GetAnzahl()] > 0)
        EStr.AddStr(_TEXT("+"));
      else
        EStr.AddStr(_TEXT("-"));
      Ausgabe(ErgStr,fabs(Matrix[1][PktFeld.GetAnzahl()]),FitForm);
      if (fabs(Matrix[1][PktFeld.GetAnzahl()]) != 1)
      {
        EStr.AddStr(ErgStr);
        EStr.AddStr(_TEXT("*"));
      }
      EStr.AddStr(_TEXT("x"));
    }
    for (i = 2;i < PktFeld.GetAnzahl();i++)
    {
       if (Matrix[i][PktFeld.GetAnzahl()] != 0)
       {
         if (Matrix[i][PktFeld.GetAnzahl()] > 0)
           EStr.AddStr(_TEXT("+"));
         else
           EStr.AddStr(_TEXT("-"));
         Ausgabe(ErgStr,fabs(Matrix[i][PktFeld.GetAnzahl()]),ScalNForm,9);
         if (fabs(Matrix[i][PktFeld.GetAnzahl()]) != 1)
         {
           EStr.AddStr(ErgStr);
           EStr.AddStr(_TEXT("*"));
         }
         EStr.AddStr(_TEXT("x^"));
         _TCHAR tmp[10];
         _stprintf(tmp,_TEXT("%i"),i);
         EStr.AddStr(tmp);
       }
    }
  }
  else
    EStr.SetStr(_TEXT(""));
  for (int p = 0;p < PktFeld.GetAnzahl()+1;p++)
    delete Matrix[p];
  delete Matrix;
  return EStr.c_str();
}

bool FuGenObj::Koeffs(double &A,double &B,double &C,bool quadrat)
{ // ermittelt die Koeffizienten der Ausgleichsfunktion
  double YX = 0,X = 0,Y = 0,X2 = 0,X3 = 0,X4 = 0,YX2 = 0;

  bool ret = true;
  fptr oldhndl = SetFPESignal();
  try
  {
    for (int i = 0;i < PktFeld.GetAnzahl();i++)
    {
      X += PktFeld.X[i];
      Y += PktFeld.Y[i];
      double tmp = PktFeld.X[i]*PktFeld.X[i];
      X2 += tmp;
      double tmp1 = PktFeld.X[i]*PktFeld.Y[i];
      YX += tmp1;
      if (quadrat)
      {
        X3 += tmp*PktFeld.X[i];
        X4 += tmp*tmp;
        YX2 += tmp1*PktFeld.X[i];
      }
    }
    double Delta,n = PktFeld.GetAnzahl();
    if (quadrat)
    {
      Delta = (X2*X2/X4+X*X/X2-n)/X3+(n*X3/X2-2*X)/X4;
      A = ((YX*n-Y*X+X*X/X3*YX2)/(X2*X4)+(X2*Y-YX*X-YX2*n)/(X3*X4))/Delta;
      B = ((YX*X2/X3+YX2*n/X2-Y)/X4+(-YX*n/X2+(Y/X2-YX2/X4)*X)/X3)/Delta;
      C = ((YX*X/X2-Y+YX2*X2/X4)/X3+(-YX+(-X*YX2+X3*Y)/X2)/X4)/Delta;
    }
    else
    {
      Delta = n*X2-X*X;
      A = 0;
      B = (n*YX-X*Y)/Delta;
      C = (X2*Y-X*YX)/Delta;
    }
  }
  catch (...)
  {
    ret = false;
  }  
  ClearFPESignal(oldhndl);
  return ret;
}

// Logarithmiert Feld, ermittelt vorher notw. Offset bei neg. Werten
double FuGenObj::LogArr(double *Arr,int Anzahl)
{
  double Offset = -1;
  int i;

  for (i = 0;i < Anzahl;i++) // negative Werte suchen
    if (-Arr[i] > Offset) Offset = -Arr[i];
  if (Offset >= -1e-12) // negative Werte gefunden
    if (Offset < 1e-12)
      Offset = 1e-12; // Mindestwert fuer Offset
    else
      Offset *= 1.01; // Ansonsten Sicherheitsabstand aufschlagen
  else
    Offset = 0; // Kein Offset
  for (i = 0;i < Anzahl;i++)
    Arr[i] = log(Arr[i]+Offset);
  return Offset;
}

// ermittelt die Ausgleichsfunktion
const _TCHAR *FuGenObj::Ausgleich(int Mode)
{
  static MGPStr EStr;
  double A,B,C,XOffset = 0,YOffset = 0;
  MGPStr ErgStr;

  EStr.SetStr(_TEXT(""));
  AkErr = 0;
  
  const _TCHAR *ret = 0;
  fptr oldhndl = SetFPESignal();
  try
  {
    if (Mode == ExpSmallError) // Ausgleichs-Exponentialfunktion
    {
      YOffset = LogArr(PktFeld.Y,PktFeld.GetAnzahl());
    }
    else
    if (Mode == Potenz) // Ausgleichs-Potenzfunktion
    {
      YOffset = LogArr(PktFeld.Y,PktFeld.GetAnzahl());
      XOffset = LogArr(PktFeld.X,PktFeld.GetAnzahl());
    }
  }
  catch (...)
  {
    ret = EStr.c_str();
  }  
  ClearFPESignal(oldhndl);
  if (ret)
    return ret;

  if (AkErr)
    return EStr.c_str();
  if (Koeffs(A,B,C,Mode == Square))
  {
    if (A != 0)
    {
      Ausgabe(ErgStr,A,FitForm);
      EStr.AddStr(ErgStr);
      EStr.AddStr(_TEXT("*x^2"));
    }
    if (B != 0)
    {
      Ausgabe(ErgStr,B,FitForm);
      if (B > 0)
        EStr.AddStr(_TEXT("+"));
      EStr.AddStr(ErgStr);
      EStr.AddStr(_TEXT("*"));
      if (Mode == Potenz)
        EStr.AddStr(_TEXT("ln"));
      if (XOffset)
        EStr.AddStr(_TEXT("("));
      EStr.AddStr(_TEXT("x"));
      if (XOffset)
      {
        Ausgabe(ErgStr,XOffset,FitForm);
        EStr.AddStr(_TEXT("+"));
        EStr.AddStr(ErgStr);
        EStr.AddStr(_TEXT(")"));
      }
    }
    if (C != 0)
    {
      Ausgabe(ErgStr,C,FitForm);
      if (C > 0)
        EStr.AddStr(_TEXT("+"));
      EStr.AddStr(ErgStr);
    }
    if ((Mode == ExpSmallError) || (Mode == Potenz)) // Exp od. Pot.
    {
      EStr.InsStr(0,_TEXT("e^("));
      EStr.AddStr(_TEXT(")"));
    }
    if (YOffset)
    {
      Ausgabe(ErgStr,YOffset,FitForm);
      EStr.AddStr(_TEXT("-"));
      EStr.AddStr(ErgStr);
    }
  }
  return EStr.c_str();
}

