// Version 2.31  12.08.2007 Markus Greither
// Copyright (c) 1991-2007 Markus Greither

/* This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

/*! \file mg.cpp
    \brief Source code for general self defined data types
*/

#ifdef __BORLANDC__

#include "efprec.h"

#pragma hdrstop
#else

#ifdef __DMC__
#include <typeinfo.h>
#endif

#include <cstdio>
#include <cstring>
#include <set>
#include <functional>
#include <cmath>
#include <cstdlib>
#include <cctype>
#include <iostream>

using namespace std;

#include "efdefs.h"
#include "mg.h"

#endif

// ----------------------------------------------------------------------------
// MGPStr
// ----------------------------------------------------------------------------

// ----------------------------------------------------------------------------
// ZahlFormatObj
// ----------------------------------------------------------------------------

const _TCHAR ZahlFormatObj::StdToWinChar = '\x1';

const NumListFormat ZahlFormatObj::DefaultFormat(_TEXT(","),_TEXT("."),2);

MGPStr ZahlFormatObj::LocDecSeparator(_TEXT("."));

void ZahlFormatObj::Ausgabe(MGPStr &AusStr,double w,int Form,int NumDigits)
{
  _TCHAR Bezeichner[13] =
  {
    _TCHAR('p'),
    _TCHAR('n'),
#ifdef UNICODE
    _TCHAR(0xB5/*''*/),
#else
    _TCHAR(''),
#endif
    _TCHAR('m'),
    _TCHAR(' '),
    _TCHAR('K'),
    _TCHAR('M'),
    _TCHAR('G'),
    _TCHAR('T')
  };
  int Potenz;
  _TCHAR Ergebnis[50] = _TEXT("");
  unsigned int i,f;
  if (Form == IngForm)
    NumDigits = 6;
  else
  if (Form == ScalForm)
    NumDigits = 3;
  else
  if (Form == LongForm)
    NumDigits = 15;

  if (w == 0)
  {
    switch ( Form )
    {
      case FitForm :
        AusStr.SetStr(_TEXT("0"));
        break;
      case FitForm2 :
        AusStr.SetStr(_TEXT("0"));
        AusStr.AddStr(LocDecSeparator);
        AusStr.AddStr(_TEXT("00"));
        break;
      case LongForm :
      case ExpForm :
      case LongExpForm :
        AusStr.SetStr(_TEXT(" 0"));
        AusStr.AddStr(LocDecSeparator);
        AusStr.AddStr(_TEXT("0"));
        break;
      case ScalNForm :
      case IngForm :
      case ScalForm :
        AusStr.SetStr(_TEXT(" 0"));
        if (NumDigits > 1)
          AusStr.AddStr(LocDecSeparator);
        for (int i = 0;i < NumDigits-1;i++)
          AusStr.AddStr(_TEXT("0"));
        AusStr.AddStr(_TEXT(" "));
        break;
    }
  }
  else
  // sehr grosse oder sehr kleine Werte
  if (( fabs(w) >= 1e15) || (fabs(w) < 1e-12) )
  {
    if ((Form == LongForm) || (Form == LongExpForm))
      _stprintf(Ergebnis,_TEXT("%.15G"),w);
    else
      _stprintf(Ergebnis,_TEXT("%5.3E"),w); // Ergebnis im E-Format
    AusStr.SetStr(Ergebnis);
  }
  else
  {
    unsigned int MaxWert = 3; // Potenzen in Schritten 0,3,6,9...
    Potenz = 0; // Exponenten bestimmen
    if (fabs(w) < 1)
      while (fabs(w) < 0.99*pow(10.0,Potenz))
        Potenz -= MaxWert;
    else
      while (fabs(w) >= pow(10.0,double(MaxWert))*pow(10.0,double(Potenz)))
        Potenz += MaxWert;
    w /= pow(10.0,double(Potenz)); // Zahl auf Bereich zw. 1 und 1000 normieren
    switch (Form)
    {
      case IngForm :
      case LongForm :
      case ScalNForm :
      case ScalForm :
      {
         if (Form == LongForm)
           _stprintf(Ergebnis,_TEXT("%.15G"),w);
         else
           _stprintf(Ergebnis,_TEXT("%f"),w);
         if ( Ergebnis[0] != '-' ) // " " vorne anhaengen
         {
           for (int f1 = _tcslen(Ergebnis);f1 >= 0;f1--)
             Ergebnis[f1+1] = Ergebnis[f1];
           Ergebnis[0] = ' ';
         }
         i = 0;
         f = 0;
         bool pointfound = false;
         if ((Form == ScalForm) || (Form == IngForm) || (Form == ScalNForm) || (Form == LongForm))
         {
            // nicht mehr als NumDigits Stellen
            while ((i <= _tcslen(Ergebnis)) &&
                   ((f < (unsigned int)(NumDigits)) || (!pointfound)))
            {
              if ( IST_ZAHL(Ergebnis[i]) )
                f++;
              else
              if (!_tcsncmp(&Ergebnis[i],LocDecSeparator.c_str(),
                           LocDecSeparator.Length()))
                pointfound = true;
              i++;
            }
            Ergebnis[i] = 0;
            if (!_tcsncmp(&Ergebnis[_tcslen(Ergebnis)-1],LocDecSeparator.c_str(),
                         LocDecSeparator.Length()))
              Ergebnis[_tcslen(Ergebnis)-1] = 0;
         }
         break;
      }
      case ExpForm :
         _stprintf(Ergebnis,_TEXT("%.4G"),w);
         break;
      case LongExpForm :
         _stprintf(Ergebnis,_TEXT("%.15G"),w);
         break;
      case FitForm :
      case FitForm2 :
         _stprintf(Ergebnis,_TEXT("%5.5f"),w);
         i = 0;
         f = 0;
         while (IST_ZAHL(Ergebnis[i]) ||
                (Ergebnis[i] == '+') ||
                (Ergebnis[i] == '-') ||
                (Ergebnis[i] == ' '))
         {
           if ( IST_ZAHL(Ergebnis[i]) )
             f++;
           i++;
         }
         f = f + _tcslen(&Ergebnis[i]);
         _TCHAR *tmp = &Ergebnis[_tcslen(Ergebnis)-1];
         while ((tmp-Ergebnis >= int(i)) &&
                ((Form != FitForm2) || (f > 4) ||
                 !_tcsncmp(tmp,LocDecSeparator.c_str(),
                          LocDecSeparator.Length())) &&
                ((f > 7) || (*tmp == '0') ||
                 !_tcsncmp(tmp,LocDecSeparator.c_str(),
                          LocDecSeparator.Length())))
         {
           *tmp = 0;
           tmp--;
           f--;
         }
         break;
    }
    switch ( Form )
    {
      case LongForm :
      case IngForm :
      case ScalForm :
      case ScalNForm :
         AusStr.SetStr(Ergebnis);
         AusStr.AddChar(Bezeichner[Potenz/3+4]);
         break;
      case FitForm :
      case FitForm2 :
         AusStr.SetStr(Ergebnis);
         if ( Bezeichner[Potenz/3+4] != ' ' )
           AusStr.AddChar(Bezeichner[Potenz/3+4]);
         break;
      case LongExpForm :
      case ExpForm :
         if (Potenz != 0)
         {
           _TCHAR Exponent[12];
           _stprintf(Exponent,_TEXT("%i"),Potenz);
           _tcscat(Ergebnis,_TEXT("E"));
           _tcscat(Ergebnis,Exponent);
         }
         AusStr.SetStr(Ergebnis);
         break;
    }
  }
  for (i = 0;AusStr[i] != 0;i++)
    if (!_tcsncmp(&AusStr[i],LocDecSeparator.c_str(),
                 LocDecSeparator.Length()))
    {
      AusStr.DelNChar(i,LocDecSeparator.Length());
      AusStr.InsStr(i,Format.DecSeparator);
    }
}

unsigned long ZahlFormatObj::ZiffernFolge(const _TCHAR *Wert,
                                          unsigned int &p,int &Fehler)
{ // liest eine Folge von Ziffern
  unsigned long StWert = 0;
  int AltPos = p;
  while ((p-AltPos < 9) && (Wert[p] != 0) && (IST_ZAHL(Wert[p])))
  { // solange Ziffer und noch nicht am Ende des Strings
    StWert = ((Wert[p])-48)+StWert*10; // Ziffern auswerten
    p++;
  }
  if ((p-AltPos == 9) && IST_ZAHL(Wert[p]))
    Fehler = ZahlFormatObj::WrongValue;
  return StWert;
}

double ZahlFormatObj::ZahlWert(const _TCHAR *Wert)
{
  unsigned int P;
  double Hi;
  int Fehler;

  Fehler = ZahlFormatObj::NoError;
  P = 0;
  Hi = Zahl(Wert,P,Fehler);
  if (Fehler != Overflow)
    return(Hi);
  else
    return(0);
}

bool ZahlFormatObj::BruchMulDiv(BruchZahl Wert1,BruchZahl Wert2,
                                BruchZahl &Wert3,const _TCHAR RechZei)
{
  if (RechZei == '*') // Bruch-Multiplikation
    Wert3 = Wert1*Wert2;
  else // Bruch-Division
    Wert3 = Wert1/Wert2;
  Wert3.Kuerzen();
  return true;
}

bool ZahlFormatObj::ParseZahl(const _TCHAR *Wert,unsigned int &p,
                              int &Fehler,BruchZahl &Z)
{
  Z.Zaehler = 0;
  Z.Nenner = 1;
  double Ganze = ZahlOhneBruch(Wert,p,Fehler);
  if (Fehler != NoError)
    return false;

  int EndPos = p;

  while (Wert[p] == ' ') // evt. Leerzeichen ueberlesen
    p++;

  Z.GemischterBruch = (Wert[p] != '/') && (Wert[p] != '\\');
  // wenn weitere Zahl, auf Bruch testen
  if (IST_ZAHL(Wert[p]) || !_tcsncmp(&Wert[p],Format.DecSeparator.c_str(),
                                    Format.DecSeparator.Length()))
  {
    Z.Zaehler = ZahlOhneBruch(Wert,p,Fehler); // Zaehler lesen
    while (Wert[p] == ' ') // evtl. Leerzeichen ueberlesen
      p++;
    // wenn kein Bruchstrich oder kein guelt. Wert -> Kein gemischter Bruch
    if ((Fehler != NoError) || (Wert[p] != '/') && (Wert[p] != '\\'))
    {
      Z.GemischterBruch = false;
      Z.Zaehler = Ganze;
      p = EndPos;
      return true;
    }
    else // Bruchstrich ueberlesen
      p++;

    while (Wert[p] == ' ') // evtl. Leerzeichen ueberlesen
      p++;

    Z.Nenner = ZahlOhneBruch(Wert,p,Fehler);
    if ((Z.Nenner == 0) || (Fehler != NoError))
    {
      Fehler = WrongValue;
      Z.Nenner = 1;
      p = EndPos;
      return false;
    }
  }
  else // wenn einfacher Bruch, Bruchstrich ueberlesen
  {
    Z.Zaehler = Ganze;
    p = EndPos;
    Z.GemischterBruch = false;
    return true;
  }

  if (Ganze < 0)
    Z.Zaehler = Ganze*Z.Nenner-Z.Zaehler;
  else
  if (Ganze > 0)
    Z.Zaehler += Ganze*Z.Nenner;
  return true;
}


double ZahlFormatObj::Zahl(const _TCHAR *Wert,unsigned int &p,
                           int &Fehler)
{
  BruchZahl Z;
  ParseZahl(Wert,p,Fehler,Z);
  return Z;
}

// liest eine Zahl aus dem String Wert ab Position p und
// wandelt ihn in einen double-Wert
double ZahlFormatObj::ZahlOhneBruch(const _TCHAR *Wert,unsigned int &p,
                                    int &Fehler)
{
  Fehler = ZahlFormatObj::NoError;
  const int MaxStellenZahl = 9;
  long i;
  int StZ;
  unsigned int AltPos;
  bool Negativ;
  double ZWert;

  while ((Wert[p] == ' ') || (Wert[p] == '+') && (p < _tcslen(Wert)))
    p++; // Leerzeichen und + Zeichen ignorieren
  if (Wert[p] == '-')
  { // Vorzeichen auswerten
    Negativ = true;
    p++;
  }
  else
    Negativ = false;
  AltPos = p;
  ZWert = ZiffernFolge(Wert,p,Fehler); // Vorkomma-Wert
  if ((AltPos == p) && _tcsncmp(&Wert[p],Format.DecSeparator.c_str(),
                               Format.DecSeparator.Length()))
  { // Fehler wenn keine Vorkomma-Stellen und kein Dezimal-Trennzeichen
    Fehler = WrongValue;
    return 0;
  }
  StZ = p-AltPos; // Vorkomma-Stellenzahl
  i = AltPos;
  while ((i < int(p)) && (Wert[int(i)] == '0'))
  { // Vornullen beruecksichtigen
    StZ--;
    i++;
  }
  if ((p < _tcslen(Wert)) &&
      !_tcsncmp(&Wert[p],Format.DecSeparator.c_str(),Format.DecSeparator.Length()))
  { // wenn Dezimal-Trennzeichen gefunden
    p++;
    AltPos = p;
    unsigned long tmp = ZiffernFolge(Wert,p,Fehler);
    // Vorkomma- und korrigierten Nachkomma-Wert addieren
    ZWert += tmp/pow(10.0,double(p-AltPos));
    if (StZ == 0)
    { // Wenn Vorkomma-Stellenzahl Null, gueltige Nachkomma-Stelle finden
      i = AltPos;
      while ((i < int(p)) && (Wert[int(i)] == '0'))
      {
        StZ--;
        i++;
      }
    }
  }
  if (Negativ)
    ZWert = -ZWert;
  if ((KompatFormat) && (Wert[p] == ' '))
    p++; // Leerzeichen fuer alte Programmversionen ueberlesen
  if (p < _tcslen(Wert))
  { // wenn noch Zeichen vorhanden
    if ((Wert[p] == 'e') || (Wert[p] == 'E'))
    { // Wert in E Darstellung
      p++; // E ueberlesen
      if ((Wert[p] == ' ') || (Wert[p] == '+') && (p < _tcslen(Wert)))
        p++; // Leerzeichen und + ueberlesen
      if (Wert[p] == '-')
      { // Vorzeichen auswerten
        Negativ = true;
        p++;
      }
      else Negativ = false;
      AltPos = p;
      i = ZiffernFolge(Wert,p,Fehler); // ermittelt Exponenten
      if (AltPos == p)
      { // Fehler wenn kein Exponent
        Fehler = WrongValue;
        return 0;
      }
      if (Negativ)
        i = -i; // Vorzeichen des Exponenten beruecksichtigen
      // Wenn Zahlenbereich nicht ueberschritten und
      // nicht mehr als MaxStellenZahl Stellen
      if ((labs(i+StZ) < 300) && (StZ <= MaxStellenZahl))
        ZWert *= pow(10.0,int(i));
      else
        Fehler = Overflow;
    }
    else
    { // Wenn keine E-Darstellung, auf Multiplikatorzeichen testen
      switch(Wert[p])
      { // Multiplikatorzeichen, nur unmittelbar nach Zahl erlaubt
         case 'p' : ZWert *= 1e-12; break; // Piko
         case 'n' : ZWert *= 1e-9; break; // Nano
#ifdef UNICODE
         case _TCHAR(0xB5/*''*/) :
#else
         case '' :
#endif
         case 'u' : ZWert *= 1e-6; break; // Mikro
         case 'm' : ZWert *= 1e-3; break; // Milli
         case 'c' : ZWert *= 1e-2; break; // Zenti
         case 'd' : ZWert *= 1e-1; break; // Dezi
         case 'k' :
         case 'K' : ZWert *= 1e3; break; // Kilo
         case 'M' : ZWert *= 1e6; break; // Mega
         case 'G' : ZWert *= 1e9; break; // Giga
         case 'T' : ZWert *= 1e12; break; // Tera
         case '%' : ZWert *= 1e-2; break; // Prozent
#ifdef UNICODE
         case _TCHAR(0xB0/*''*/) :
#else
         case _TCHAR('') :
#endif
           ZWert *= M_PI/180; break; // Grad
         case ' ' : break; // Leerzeichen
         default : p--; // Kein Zeichen gefunden -> keines ueberlesen !
      }
      p++; // Multiplikatorzeichen ueberlesen
    }
  }
  if (Wert[p])
  { // Einheiten ueberlesen wenn noch Zeichen vorhanden
    if (!MGStrnicmp(&Wert[p],_TEXT("A/m"),3))
      p += 3; // A/m
    else
    while (IST_BUCHS(Wert[p]) || (Wert[p] == ' '))
      p++; // beliebige Buchstabenfolge ueberlesen
  }
  return(ZWert);
}

bool ZahlFormatObj::BruchOut(BruchZahl Z,MGPStr &Erg)
{
  MGPStr tmp;
  bool Klammern = false;

  double i = 0,Zaehler = Z.Zaehler;
  while (fabs(Zaehler) >= Z.Nenner) // Ganze ermitteln
  {
    if (Zaehler < 0)
    {
      i--;
      Zaehler += Z.Nenner;
    }
    else
    {
      i++;
      Zaehler -= Z.Nenner;
    }
  }
  if ((i == 0) && (Z.Zaehler < 0))
    Erg.AddChar('-');
  Ausgabe(tmp,i,FitForm);
  Erg.AddStr(tmp);
  if (Zaehler != 0)
  {
    Erg.AddStr(_TEXT(" "));
    Ausgabe(tmp,fabs(Zaehler),FitForm);
    Erg.AddStr(tmp);
    if (Z.Nenner != 1)
    {
      Erg.AddStr(_TEXT("/"));
      Ausgabe(tmp,Z.Nenner,FitForm);
      Erg.AddStr(tmp);
    }
  }
  return Klammern;
}

// Wandelt Strings vom englischen Format ins Windows-Format
// (Windows-Trennzeichen)
void ZahlFormatObj::StdToWin(MGPStr &Erg)
{
  unsigned int AkPos;
  unsigned int p,p1;
  const _TCHAR *A;

  p = 0;
  if (Erg[0] > ' ') // Keine Kommentarzeile
  do
  {
     p1 = p;
     while ((p < Erg.Length()) && (Erg[p] != '[') && (Erg[p] != '"'))
       p++;
     // String bis zur 1. Klammer auf oder Anfuehrungszeichen
     MGPStr tmp;
     tmp.AddNChar(&Erg[p1],p-p1);
     if (Format.LeadZero == 0) // Vornullen l�chen
     {
       AkPos = 0;
       while (AkPos < tmp.Length())
       {
          const _TCHAR *pos = _tcsstr(&tmp[AkPos],_TEXT("0."));
          if (pos)
            AkPos = pos-tmp.c_str();
          else
            AkPos = tmp.Length();
          if (tmp[AkPos] && (IST_ZAHL(tmp[AkPos+2])) &&
              ((AkPos == 0) || (!IST_ZAHL(tmp[AkPos-1]))))
            tmp.DelNChar(AkPos,1);
          AkPos++;
       }
     }
     // Dezimaltrennzeichen und Listentrennzeichen verschieden
     if (_tcscmp(Format.DecSeparator.c_str(),Format.ListSeparator.c_str()))
     {
       // um Probleme bei List = "." und Dez = "," o. � zu vermeiden
       tmp.ReplAllChars(',',StdToWinChar);
       tmp.ReplAllChars('.',Format.DecSeparator.c_str());
       tmp.ReplAllChars(StdToWinChar,Format.ListSeparator.c_str());
     }
     A = Erg.c_str();
     MGPStr t;
     t.AddNChar(Erg,p1);
     tmp.InsStr(0,t);
     p1 = tmp.Length();
     tmp.AddStr(&A[p]);
     Erg.SetStr(tmp);
     p = p1+1;
     while ((p < Erg.Length()) && (Erg[p] != ']') && (Erg[p] != '"'))
       p++;
     if (p < Erg.Length())
       p++; // String zw. Klammern oder Anfuehrungszeichen ignorieren
  }
  while (p < Erg.Length());
}

void ZahlFormatObj::WinToStd(MGPStr &Erg)
{
  unsigned int AkPos;
  unsigned int p,p1;
  const _TCHAR *A;

  p = 0;
  if (Erg[0] > ' ') // Keine Kommentarzeile
  do
  {
     p1 = p;
     while ((p < Erg.Length()) && (Erg[p] != '[') && (Erg[p] != '"'))
       p++;
     // String bis zur 1. Klammer auf oder Anfuehrungszeichen
     MGPStr tmp;
     tmp.AddNChar(&Erg[p1],p-p1);
     // Dezimaltrennzeichen und Listentrennzeichen verschieden
     if (_tcscmp(Format.DecSeparator.c_str(),Format.ListSeparator.c_str()))
     { // um Probleme bei List = "." und Dez = "," o. � zu vermeiden
       tmp.ReplAllChars(Format.ListSeparator.c_str(),StdToWinChar);
       tmp.ReplAllChars(Format.DecSeparator.c_str(),'.');
       tmp.ReplAllChars(StdToWinChar,',');
     }
     if (Format.LeadZero == 1) // Vornullen hinzufgen
     {
       AkPos = 0;
       while (AkPos < tmp.Length())
       {
          const _TCHAR *pos = _tcsstr(&tmp[AkPos],_TEXT("."));
          if (pos)
            AkPos = pos-tmp.c_str();
          else
            AkPos = tmp.Length();
          if (tmp[AkPos] && (IST_ZAHL(tmp[AkPos+1])) &&
              ((AkPos == 0) || (!IST_ZAHL(tmp[AkPos-1]))))
          {
            tmp.InsStr(AkPos,_TEXT("0"));
            AkPos++;
          }
          AkPos++;
       }
     }
     A = Erg.c_str();
     MGPStr t;
     t.AddNChar(Erg,p1);
     tmp.InsStr(0,t);
     p1 = tmp.Length();
     tmp.AddStr(&A[p]);
     Erg.SetStr(tmp);
     p = p1+1;
     while ((p < Erg.Length()) && (Erg[p] != ']') && (Erg[p] != '"'))
       p++;
     if (p < Erg.Length())
       p++; // String zw. Klammern oder Anfuehrungszeichen ignorieren
  }
  while (p < Erg.Length());
}

// ----------------------------------------------------------------------------
// BruchZahl
// ----------------------------------------------------------------------------

void BruchZahl::Kuerzen()
{
  double Div = 1;
  while (Div*2 <= fabs(Zaehler)) // Bruch kuerzen
  {
    double AkFak = fabs(Zaehler)/Div;
    if ((fmod(Zaehler,Div) == 0) && (fmod(Nenner,AkFak) == 0))
    {
      if (Zaehler < 0)
        Zaehler = -Div;
      else
        Zaehler = Div;
      Nenner /= AkFak;
    }
    Div++;
  }
  if ((Zaehler == 0) || (Nenner == 0))
    Nenner = 1;
}

// ----------------------------------------------------------------------------
// Functions
// ----------------------------------------------------------------------------

int GetNumLines(const _TCHAR *AkStr)
{
  int i = 1;
  const _TCHAR *tst = AkStr;

  while (*tst)
  { // Until end of string
    while (*tst && (*tst != '\r') && (*tst != '\n'))
      tst++; // find line break
    if ((*tst == '\r') && (*(tst+1) == '\n'))
      tst++; // DOS linebreak
    if (*tst)
    { // if line break, skip it and increase line count
      i++;
      tst += 1;
    }
  }
  return i;
}

// String comparison without case
int MGStricmp(const _TCHAR *str1,const _TCHAR *str2)
{
  int i = 0;
  while (str1[i] && str2[i])
  {
    if ((_totupper(str1[i]) != _totupper(str2[i])))
      return 1;
    i++;
  }
  if ((_totupper(str1[i]) || _totupper(str2[i])))
    return 1;
  return 0;
}

// String comparison without case for n characters
int MGStrnicmp(const _TCHAR *str1,const _TCHAR *str2,int n)
{
  int i;
  for (i = 0;(i < n) && str1[i] && str2[i];i++)
  {
    if ((_totupper(str1[i]) != _totupper(str2[i])))
      return 1;
  }
  if ((i < n) && (_totupper(str1[i]) || _totupper(str2[i])))
    return 1;
  return 0;
}

// Convert string to lower case
void MGTolower(_TCHAR *str1)
{
  while (*str1)
  {
    *str1 = _totlower(*str1);
    str1++;
  }
}

// Convert string to upper case
void MGToupper(_TCHAR *str1)
{
  while (*str1)
  {
    *str1 = _totupper(*str1);
    str1++;
  }
}

const _TCHAR *MGStrnchr(const _TCHAR *str,int len,_TCHAR c)
{
  int i = 0;
  while (str[i] && (i < len))
  {
    if (str[i] == c)
      return &str[i];
    i++;
  }
  return 0;
}

MGPStr ConvertStr(const char *astr)
{
  MGPStr str;
#ifdef UNICODE
  char *cpy = new char[strlen(astr)+1];
  strcpy(cpy,astr);
  char *pos = strchr(cpy,'');
  if (pos)
    *pos = 'u';
  _TCHAR *tmp = new _TCHAR[strlen(astr)+1];
  mbstowcs(tmp,cpy,strlen(cpy)+1);
  str.SetStr(tmp);
  delete [] cpy;
  delete [] tmp;
#else
  str.SetStr(astr);
#endif
  return str;
}

void ConvertStr(const _TCHAR *AStr,char *res)
{
#ifdef UNICODE
  MGPStr cpy(AStr);
  const _TCHAR *pos = _tcschr(cpy.c_str(),_TCHAR(0xB5/*''*/));
  if (pos)
    cpy.ReplChar(pos-cpy.c_str(),_TCHAR('u'));
  wcstombs(res,cpy.c_str(),cpy.Length()+1);
#else
  strcpy(res,AStr);
#endif
}

CharString UnicodeToChar(const _TCHAR *AStr)
{
#ifdef UNICODE
  MGPStr cpy(AStr);
  const _TCHAR *pos = _tcschr(cpy.c_str(),_TCHAR(0xB5/*''*/));
  if (pos)
    cpy.ReplChar(pos-cpy.c_str(),_TCHAR('u'));
  char *res = new char[cpy.Length()+1];
  wcstombs(res,cpy.c_str(),cpy.Length()+1);
  CharString tmp;
  tmp.SetStr(res);
  delete [] res;
  return tmp;
#else
  CharString tmp;
  tmp.SetStr(AStr);
  return tmp;
#endif
}

//! Stream output
ostream &operator <<(ostream &stream,const MGPStr &AStr)
{
  char *tmp = new char[AStr.Length()+1];
  ConvertStr(AStr.c_str(),tmp);
  stream << tmp;
  delete [] tmp;
  return stream;
}

//! Stream output
ostream &operator <<(ostream &stream,const SetString &AStr)
{
  char *tmp = new char[AStr.Length()+1];
  ConvertStr(AStr.c_str(),tmp);
  stream << tmp;
  delete [] tmp;
  return stream;
}

#ifdef UNICODE
//! Stream output
ostream &operator <<(ostream &stream,const _TCHAR *AStr)
{
  char *tmp = new char[_tcslen(AStr)+1];
  ConvertStr(AStr,tmp);
  stream << tmp;
  delete [] tmp;
  return stream;
}
#endif

// ----------------------------------------------------------------------------
// FastString
// ----------------------------------------------------------------------------

template<> size_t FastString<char>::GetLen(const char *str)
{
  return strlen(str);
}

template<> int FastString<char>::Compare(const char *str,
                                          const char *oth,
                                          size_t len)
{
  return strncmp(str,oth,len);
}

template<> bool FastString<char>::AddULong(unsigned long tmp)
{ // haengt den integer tmp an
  char Zeile[20];

  sprintf(Zeile,"%lu",tmp);
  return AddStr(Zeile);
}

template<> bool FastString<char>::AddInt(int tmp)
{ // haengt den integer tmp an
  char Zeile[20];

  sprintf(Zeile,"%i",tmp);
  return AddStr(Zeile);
}

#ifdef UNICODE
template<> size_t FastString<wchar_t>::GetLen(const wchar_t *str)
{
  return wcslen(str);
}

template<> int FastString<wchar_t>::Compare(const wchar_t *str,
                                             const wchar_t *oth,
                                             size_t len)
{
  return wcsncmp(str,oth,len);
}

template<> bool FastString<wchar_t>::AddULong(unsigned long tmp)
{ // haengt den integer tmp an
  wchar_t Zeile[20];

  MGswprintf(Zeile,_TEXT("%lu"),tmp);
  return AddStr(Zeile);
}

template<> bool FastString<wchar_t>::AddInt(int tmp)
{ // haengt den integer tmp an
  wchar_t Zeile[20];

  MGswprintf(Zeile,_TEXT("%i"),tmp);
  return AddStr(Zeile);
}

#if !defined(__WXMSW__) && defined(UNICODE)
FILE *MGwfopen(const wchar_t *n,const wchar_t *m)
{
  char name[1000],mode[1000];
  wcstombs(name,n,1000);
  wcstombs(mode,m,1000);
  return fopen(name,mode);
};

#include <stdarg.h>

int MGswprintf(wchar_t *s,const wchar_t *t,...)
{
  va_list ap;
  va_start(ap,t);
  vswprintf(s,1000,t,ap);
  va_end(ap);
};
#endif

#endif
